@@ -75,15 +75,10 @@ audit_csv_runtime_fields = [
 ]
 
 def hash_cutlass_string(input_string):
-  # Regex pattern to match instruction shape
-  instruction_shape_pattern = r"[a-zA-Z]\d+x\d+x\d+"  # Matches '_s128x128x64', '_h64x128x16', etc.
   mma_cluster_shape_pattern = r"_\d+x\d+x\d+"         # Matches MMA and Cluster shapes (e.g., '_128x128x256', '_0x0x1')
 
-  # Remove instruction shape (e.g., '_s128x128x64', '_h64x128x16')
-  output = re.sub(instruction_shape_pattern, "", input_string)
-
   # Remove MMA and Cluster shapes (e.g., '_128x128x256', '_0x0x1')
-  output = re.sub(mma_cluster_shape_pattern, "", output)
+  output = re.sub(mma_cluster_shape_pattern, "", input_string)
 
   return output
 
@@ -258,7 +253,8 @@ def _getInstType(input_precision, accumulate_precision, math_instruction):
 
   return inst
 # TODO: Computes FLOps/Bytes for GEMM - revisit for conv
-def _computeFlopsPerByte(operation, m, n, k, batch_count=1, beta=0.0):
+def _computeFlopsPerByte(operation, m, n, k, batch_count=1, beta=0.0, num_groups=1):
+  assert not (batch_count > 1 and num_groups > 1)
 
   # TODO: adjust for sparsity
   gmem_bytes = (
@@ -274,21 +270,24 @@ def _computeFlopsPerByte(operation, m, n, k, batch_count=1, beta=0.0):
     gmem_bytes += (DataTypeSize[operation.C.element] * m // 8) * n
     flops += 2 * m * n
 
-  gmem_bytes *= batch_count
-  flops *= batch_count
+  multiplier = max(batch_count, num_groups)
+  gmem_bytes *= multiplier
+  flops *= multiplier
 
   return flops / gmem_bytes
 
 def emit_gemm_kernel_testlist(manifest, curr_build_dir, arch, mode
                               ):
-  profiler_reference_computing = "--verification-providers=device --providers=cutlass"
-  
+  # For functional testing, we prefer to run reference computing on device if any
+  reference_device_archs = ["100a", "103a"]
+  run_reference_on_device = True if arch in reference_device_archs and mode in ["functional_L0", "functional_L1"] else False
+  profiler_flags_for_verification = "device" if run_reference_on_device else "host"
 
   # beta values for L0 and L1
   # TODO: randomize beta values for wider coverage
   beta_values = [0.5]
 
-  is_supported_arch = (arch in ["100a", "101a", "120a"])
+  is_supported_arch = (arch in ["100a", "100f", "101a", "101f", "103a", "110a", "110f", "120a", "120f", "121a", "121f"])
 
   is_runtime_datatype_enabled = mode == "functional_L0" and is_supported_arch
 
@@ -300,23 +299,21 @@ def emit_gemm_kernel_testlist(manifest, curr_build_dir, arch, mode
     #
 
     sm100_mma_data_type_general = [
-      'x16gemm_f16_f16_f16_f16_f16',
-      'x16gemm_f16_f16_f16_void_f16',
-      'x16gemm_f16_f16_f32_f16_f16',
-      'x8tf32gemm_f32_f32_f32_f32_f32',
-      'x16bf16gemm_f32_f32_f32_f32_f32',
+      'gemm_f16_f16_f16_f16_f16',
+      'gemm_f16_f16_f16_void_f16',
+      #'gemm_f16_f16_f32_f16_f16',
+      'tf32gemm_f32_f32_f32_f32_f32',
+      'bf16gemm_f32_f32_f32_f32_f32',
     ]
 
+    exclude_archs = arch not in ("103a")
+    if exclude_archs:
+      sm100_mma_data_type_general.append('gemm_s8_s8_s32_s8_s8')
+
     sm100_mma_data_type_runtime_dtype = [
-      'x32gemm_f4_f4_f32_f32_f32',
-      'x32gemm_f6_f6_f32_f32_f32',
-      'x32gemm_f8_f8_f32_f32_f32',
-    ]
-
-    sm100_mma_data_type_mergeable = [
-      'x32gemm_e4m3_e4m3_f32_f32_f32',# mask out one instance for verification
-      'x32gemm_e2m1_e2m1_f32_f32_f32',
-      'x32gemm_e3m2_e3m2_f32_f32_f32',
+      'gemm.*f4_f4_f32_f32_f32',
+      'gemm.*f6_f6_f32_f32_f32',
+      'gemm.*f8_f8_f32_f32_f32',
     ]
 
     sm100_mma_cluster_size = [
@@ -331,128 +328,93 @@ def emit_gemm_kernel_testlist(manifest, curr_build_dir, arch, mode
       'ntn' 
     ]
 
-    sm100_mma_instruction_shape = [
-      # [0] .1CTA, General
-      ['64x128', '128x128', '128x256'],
-      # [1] .2CTA, General
-      ['128x128', '256x128', '256x256'],
-    ]
-
     # regex list must be in kernel procedural name order
-    mergeable_sm100_mma_filter_regex_1sm = "cutlass3x_sm100_tensorop.*(" + ").*(".join([ "|".join(x) for x in [sm100_mma_instruction_shape[0], sm100_mma_data_type_mergeable, sm100_mma_cluster_size, sm100_mma_layouts]]) + ").*1sm.*"
-    mergeable_sm100_mma_filter_regex_2sm = "cutlass3x_sm100_tensorop.*(" + ").*(".join([ "|".join(x) for x in [sm100_mma_instruction_shape[1], sm100_mma_data_type_mergeable, sm100_mma_cluster_size, sm100_mma_layouts]]) + ").*2sm.*"
+    sm100_mma_filter_regex_1sm = "cutlass3x_sm100_tensorop.*(" + ").*(".join([ "|".join(x) for x in [sm100_mma_data_type_general, sm100_mma_cluster_size, sm100_mma_layouts]]) + ").*1sm.*"
+    sm100_mma_filter_regex_2sm = "cutlass3x_sm100_tensorop.*(" + ").*(".join([ "|".join(x) for x in [sm100_mma_data_type_general, sm100_mma_cluster_size, sm100_mma_layouts]]) + ").*2sm.*"
 
-    sm100_mma_filter_regex_1sm = "cutlass3x_sm100_tensorop.*(" + ").*(".join([ "|".join(x) for x in [sm100_mma_instruction_shape[0], sm100_mma_data_type_general, sm100_mma_cluster_size, sm100_mma_layouts]]) + ").*1sm.*"
-    sm100_mma_filter_regex_2sm = "cutlass3x_sm100_tensorop.*(" + ").*(".join([ "|".join(x) for x in [sm100_mma_instruction_shape[1], sm100_mma_data_type_general, sm100_mma_cluster_size, sm100_mma_layouts]]) + ").*2sm.*"
-
-    sm100_mma_filter_regex_1sm_runtime = "cutlass3x_sm100_tensorop.*(" + ").*(".join([ "|".join(x) for x in [sm100_mma_instruction_shape[0], sm100_mma_data_type_runtime_dtype, sm100_mma_cluster_size, sm100_mma_layouts]]) + ").*1sm.*"
-    sm100_mma_filter_regex_2sm_runtime = "cutlass3x_sm100_tensorop.*(" + ").*(".join([ "|".join(x) for x in [sm100_mma_instruction_shape[1], sm100_mma_data_type_runtime_dtype, sm100_mma_cluster_size, sm100_mma_layouts]]) + ").*2sm.*"
+    sm100_mma_filter_regex_1sm_runtime = "cutlass3x_sm100_tensorop.*(" + ").*(".join([ "|".join(x) for x in [sm100_mma_data_type_runtime_dtype, sm100_mma_cluster_size, sm100_mma_layouts]]) + ").*1sm.*"
+    sm100_mma_filter_regex_2sm_runtime = "cutlass3x_sm100_tensorop.*(" + ").*(".join([ "|".join(x) for x in [sm100_mma_data_type_runtime_dtype, sm100_mma_cluster_size, sm100_mma_layouts]]) + ").*2sm.*"
 
     #
     # Block Scale Gemm
     #
 
-    block_scaled_data_type_base = [
+    block_scaled_data_type = [
       # runtime datatypes
-      'x32gemm.*ue8m0xf4_ue8m0xf4_f32_f16_e5m2',
-      'x64gemm.*ue8m0xf4_ue8m0xf4_f32_f16_e5m2',
-      'x32gemm.*ue8m0xf4_ue8m0xf6_f32_f16_e5m2',
-      'x64gemm.*ue8m0xf4_ue8m0xf4_f32_f16_ue8m0xe2m1',
-      'x32gemm.*ue8m0xf6_ue8m0xf6_f32_f16_ue8m0xe3m2',
+      'gemm.*ue8m0xf4_ue8m0xf4_f32_f16_e5m2',
+      'gemm.*ue4m3xf4_ue4m3xf4_f32_f16_e5m2',
+      'gemm.*ue8m0xf4_ue8m0xf6_f32_f16_e5m2',
+      #'gemm.*ue8m0xf4_ue8m0xf4_f32_f16_ue8m0xe2m1',
+      'gemm.*ue8m0xf6_ue8m0xf6_f32_f16_ue8m0xe3m2',
     ]
 
-    block_scaled_data_type_mergeable = [
-      'x32gemm.*ue8m0xe2m1_ue8m0xe2m1_f32_f16_e5m2',
-      'x64gemm.*ue8m0xe2m1_ue8m0xe2m1_f32_f16_e5m2',
-      'x32gemm.*ue8m0xe2m1_ue8m0xe2m3_f32_f16_e5m2',
-      'x64gemm.*ue8m0xe2m1_ue8m0xe2m1_f32_f16_ue8m0xe2m1',
-      'x32gemm.*ue8m0xe2m3_ue8m0xe2m3_f32_f16_ue8m0xe3m2',
+    sm103_block_scaled_data_type = [
+      'gemm.*ue8m0xf4_ue8m0xf4_f32_f16_e5m2',
+      'gemm.*ue8m0xf4_ue8m0xf4_f32_f16_ue8m0xe2m1',
     ]
 
-    block_scaled_data_type = block_scaled_data_type_base + block_scaled_data_type_mergeable
-
     block_scaled_cluster_size = [
       '4x4x1', '2x1x1',
       '0x0x1' # dynamic cluster
     ]
 
     block_scaled_layouts = ['tnt']
-    block_scaled_instruction_shape = [
-      # .1CTA
-      ['128x128', '128x192', '128x256'],
-      # .2CTA
-      ['256x128', '256x192', '256x256'],
-    ]
     # regex list must be in kernel procedural name order
-    mergeable_block_scaled_filter_regex_1sm = "cutlass3x_sm100_bstensorop.*(" + ").*(".join([ "|".join(x) for x in [block_scaled_instruction_shape[0], block_scaled_data_type_mergeable, block_scaled_cluster_size, block_scaled_layouts]]) + ").*1sm.*"
-    mergeable_block_scaled_filter_regex_2sm = "cutlass3x_sm100_bstensorop.*(" + ").*(".join([ "|".join(x) for x in [block_scaled_instruction_shape[1], block_scaled_data_type_mergeable, block_scaled_cluster_size, block_scaled_layouts]]) + ").*2sm.*"
-
-    block_scaled_filter_regex_1sm = "cutlass3x_sm100_bstensorop.*(" + ").*(".join([ "|".join(x) for x in [block_scaled_instruction_shape[0], block_scaled_data_type, block_scaled_cluster_size, block_scaled_layouts]]) + ").*1sm.*"
-    block_scaled_filter_regex_2sm = "cutlass3x_sm100_bstensorop.*(" + ").*(".join([ "|".join(x) for x in [block_scaled_instruction_shape[1], block_scaled_data_type, block_scaled_cluster_size, block_scaled_layouts]]) + ").*2sm.*"
+    block_scaled_filter_regex_1sm = "cutlass3x_sm100_bstensorop.*(" + ").*(".join([ "|".join(x) for x in [block_scaled_data_type, block_scaled_cluster_size, block_scaled_layouts]]) + ").*1sm.*"
+    block_scaled_filter_regex_2sm = "cutlass3x_sm100_bstensorop.*(" + ").*(".join([ "|".join(x) for x in [block_scaled_data_type, block_scaled_cluster_size, block_scaled_layouts]]) + ").*2sm.*"
     
-    if arch == "100a":
+    sm103_block_scaled_filter_regex_1sm = "cutlass3x_sm103_bstensorop.*(" + ").*(".join([ "|".join(x) for x in [sm103_block_scaled_data_type, block_scaled_cluster_size, block_scaled_layouts]]) + ").*1sm.*"
+    sm103_block_scaled_filter_regex_2sm = "cutlass3x_sm103_bstensorop.*(" + ").*(".join([ "|".join(x) for x in [sm103_block_scaled_data_type, block_scaled_cluster_size, block_scaled_layouts]]) + ").*2sm.*"
+
+    if arch in ["100a", "100f"]:
       kernel_filter = f"({sm100_mma_filter_regex_1sm})|" \
                       f"({sm100_mma_filter_regex_2sm})|" \
                       f"({sm100_mma_filter_regex_1sm_runtime})|" \
                       f"({sm100_mma_filter_regex_2sm_runtime})|" \
                       f"({block_scaled_filter_regex_1sm})|" \
                       f"({block_scaled_filter_regex_2sm})"
-    elif arch == "101a":
+    elif arch in ["101a", "101f", "110a", "110f"]:
       kernel_filter = f"({sm100_mma_filter_regex_1sm})|" \
                       f"({sm100_mma_filter_regex_2sm})|" \
                       f"({sm100_mma_filter_regex_1sm_runtime})|" \
                       f"({sm100_mma_filter_regex_2sm_runtime})|" \
                       f"({block_scaled_filter_regex_1sm})|" \
                       f"({block_scaled_filter_regex_2sm})"
-    elif arch == "120a":
+    elif arch in ["103a"]:
+      kernel_filter = f"({sm100_mma_filter_regex_1sm})|" \
+                      f"({sm100_mma_filter_regex_2sm})|" \
+                      f"({sm100_mma_filter_regex_1sm_runtime})|" \
+                      f"({sm100_mma_filter_regex_2sm_runtime})|" \
+                      f"({block_scaled_filter_regex_1sm})|" \
+                      f"({block_scaled_filter_regex_2sm})|" \
+                      f"({sm103_block_scaled_filter_regex_1sm})|" \
+                      f"({sm103_block_scaled_filter_regex_2sm})"
+    elif arch in ["120a", "120f", "121a", "121f"]:
 
       # blockscaled sm120_mma kernels
       blockscaled_sm120_mma_kernel_cta_tiles = [
         [ '128x128' ]
       ]
 
-      # sm120 MMA instruction shapes
-      blockscaled_sm120_mma_instruction_shapes = [
-        [ 's16x8x64gemm', 
-          's16x8x32gemm'
-        ]
-      ]
-      
       # Restrict to two layouts to reduce L0 build and test time.
       blockscaled_sm120_mma_layouts = [ 'tn' ]
-      filter_regex_blockscaled_sm120_mma = "cutlass3x_sm120_bstensorop.*(" + ").*(".join([ "|".join(x) for x in [blockscaled_sm120_mma_instruction_shapes[0], blockscaled_sm120_mma_kernel_cta_tiles[0], blockscaled_sm120_mma_layouts]]) + ").*"
+      filter_regex_blockscaled_sm120_mma = "cutlass3x_sm120_bstensorop.*(" + ").*(".join([ "|".join(x) for x in [blockscaled_sm120_mma_kernel_cta_tiles[0], blockscaled_sm120_mma_layouts]]) + ").*"
       
       problem_waves = [0.5, 1.25, 2.5]
 
       kernel_filter = f"({filter_regex_blockscaled_sm120_mma})"
     else:
-      error_message = "unsupported arch, only support sm100a, sm101a, sm120a"
+      error_message = "unsupported arch, only support sm100a, sm100f, sm101a, sm101f, sm110a, sm110f, sm103a, sm120a, sm120f, sm121a, sm121f"
       raise Exception(error_message)
-    
-    # Statically encoded kernels are still added to generated_kernels 
-    # but are filtered out from the testing commands to reduce test duration.
-    # The mergeable_kernel_filter specifies the kernels that are already covered 
-    # by the runtime datatype tests so that we safely mark them off
-    # without changing the test coverage.
-    mergeable_kernel_filter = f"({mergeable_sm100_mma_filter_regex_1sm})|" \
-                              f"({mergeable_sm100_mma_filter_regex_2sm})|" \
-                              f"({mergeable_block_scaled_filter_regex_1sm})|" \
-                              f"({mergeable_block_scaled_filter_regex_2sm})"
-  elif mode == "functional_L1":
 
+  elif mode == "functional_L1":
     sm100_mma_cluster_size = [
                     '0x0x1' # dynamic cluster
                      ]
     # Restrict to two layouts to reduce L1 build and test time.
     sm100_mma_layouts = ['tnt', 'ntn']
-    sm100_mma_instruction_shape = [
-      # .1CTA
-      ['64x128', '128x128', '128x256'],
-      # .2CTA
-      ['128x128', '256x128', '256x256']
-    ]
-    sm100_mma_filter_regex_1sm = "cutlass3x_sm100_tensorop.*(" + ").*(".join([ "|".join(x) for x in [sm100_mma_instruction_shape[0], sm100_mma_cluster_size, sm100_mma_layouts]]) + ").*1sm.*"
-    sm100_mma_filter_regex_2sm = "cutlass3x_sm100_tensorop.*(" + ").*(".join([ "|".join(x) for x in [sm100_mma_instruction_shape[1], sm100_mma_cluster_size, sm100_mma_layouts]]) + ").*2sm.*"
+    sm100_mma_filter_regex_1sm = "cutlass3x_sm100_tensorop.*(" + ").*(".join([ "|".join(x) for x in [sm100_mma_cluster_size, sm100_mma_layouts]]) + ").*1sm.*"
+    sm100_mma_filter_regex_2sm = "cutlass3x_sm100_tensorop.*(" + ").*(".join([ "|".join(x) for x in [sm100_mma_cluster_size, sm100_mma_layouts]]) + ").*2sm.*"
     block_scaled_data_type = [
       'ue8m0xe2m1_ue8m0xe2m1_f32_f16_e5m2',
       'ue8m0xe2m1_ue8m0xe2m3_f32_f16_e5m2',
@@ -461,21 +423,27 @@ def emit_gemm_kernel_testlist(manifest, curr_build_dir, arch, mode
       'ue8m0xe2m3_ue8m0xe2m3_f32_f16_ue8m0xe3m2',
     ]
 
-    block_scaled_cluster_size = ['4x4x1', '2x1x1', '0x0x1']
-    block_scaled_layouts = ['tnt']
-    block_scaled_instruction_shape = [
-      # .1CTA
-      ['128x128', '128x192', '128x256'],
-      # .2CTA
-      ['256x128', '256x192', '256x256'],
+    sm103_block_scaled_data_type = [
+      'ue8m0xe2m1_ue8m0xe2m1_f32_f16_e5m2',
+      'ue8m0xe2m1_ue8m0xe2m1_f32_f16_ue8m0xe2m1',
     ]
+
+    block_scaled_cluster_size = ['0x0x1']
+    block_scaled_layouts = ['tnt']
+
     # regex list must be in kernel procedural name order
-    block_scaled_filter_regex_1sm = "cutlass3x_sm100_bstensorop.*(" + ").*(".join([ "|".join(x) for x in [block_scaled_instruction_shape[0], block_scaled_data_type, block_scaled_cluster_size, block_scaled_layouts]]) + ").*1sm.*"
-    block_scaled_filter_regex_2sm = "cutlass3x_sm100_bstensorop.*(" + ").*(".join([ "|".join(x) for x in [block_scaled_instruction_shape[1], block_scaled_data_type, block_scaled_cluster_size, block_scaled_layouts]]) + ").*2sm.*"
+    block_scaled_filter_regex_1sm = "cutlass3x_sm100_bstensorop.*(" + ").*(".join([ "|".join(x) for x in [block_scaled_data_type, block_scaled_cluster_size, block_scaled_layouts]]) + ").*1sm.*"
+    block_scaled_filter_regex_2sm = "cutlass3x_sm100_bstensorop.*(" + ").*(".join([ "|".join(x) for x in [block_scaled_data_type, block_scaled_cluster_size, block_scaled_layouts]]) + ").*2sm.*"
+
+    sm103_block_scaled_filter_regex_1sm = "cutlass3x_sm103_bstensorop.*(" + ").*(".join([ "|".join(x) for x in [sm103_block_scaled_data_type, block_scaled_cluster_size, block_scaled_layouts]]) + ").*1sm.*"
+    sm103_block_scaled_filter_regex_2sm = "cutlass3x_sm103_bstensorop.*(" + ").*(".join([ "|".join(x) for x in [sm103_block_scaled_data_type, block_scaled_cluster_size, block_scaled_layouts]]) + ").*2sm.*"
+
     filter_regex_sm100_mma = f"({sm100_mma_filter_regex_1sm})|" \
                           f"({sm100_mma_filter_regex_2sm})|" \
                           f"({block_scaled_filter_regex_1sm})|" \
-                          f"({block_scaled_filter_regex_2sm})|" 
+                          f"({block_scaled_filter_regex_2sm})" \
+                          f"({sm103_block_scaled_filter_regex_1sm})|" \
+                          f"({sm103_block_scaled_filter_regex_2sm})"
     # CTA tiles for sm120 MMA - only run one tile size to reduce build/test times
     sm120_mma_kernel_cta_tiles = [
       # h1688, s1688, i16832, i8816
@@ -512,7 +480,10 @@ def emit_gemm_kernel_testlist(manifest, curr_build_dir, arch, mode
 
     problem_waves = [0.5, 1.25, 2.5]
 
-    kernel_filter = f"({filter_regex_sm100_mma})|({filter_regex_sm120_mma})"
+    if arch in ["120a", "120f", "121a", "121f"]:
+      kernel_filter = f"({filter_regex_sm120_mma})"
+    else:
+      kernel_filter = f"({filter_regex_sm100_mma})"
   else:
     raise ValueError()
 
@@ -522,10 +493,6 @@ def emit_gemm_kernel_testlist(manifest, curr_build_dir, arch, mode
 
   audit_file_params_name = os.path.join(curr_build_dir, f"FK_{mode}_audit_params_SM{arch}_cutlass3x_gemm.csv")
 
-  if is_runtime_datatype_enabled: 
-    mergeable_kernel_filter_re = re.compile(mergeable_kernel_filter)
-
-
   kernel_filter_re = re.compile(kernel_filter)
   testcase_counter = 0
   kernels_emitted = 0
@@ -553,12 +520,6 @@ def emit_gemm_kernel_testlist(manifest, curr_build_dir, arch, mode
         if 'f16_f16_f16_void_f16' not in kernel_name :
           continue
 
-      # Filter out the statically encoded tests which are 
-      # covered by runtime datatype tests to avoid repetition.
-      if is_runtime_datatype_enabled and len(mergeable_kernel_filter_re.findall(kernel_name)) != 0:
-        continue
-
-
       kernels_emitted += 1
       kernel_name_set.add(kernel_name)
       hashed_kernel_name = hash_cutlass_string(kernel_name)
@@ -622,11 +583,22 @@ def emit_gemm_kernel_testlist(manifest, curr_build_dir, arch, mode
         elif "ue8m0xf8_ue8m0xf8" in kernel_name:
           runtime_input_datatypes = [['e4m3','e4m3']]
 
+      if "bstensorop" in kernel_name or is_blockwise(manifest.operations_by_name[kernel_name].gemm_kind):
+        profiler_flags_for_verification = "host"
+
+      # reduce L1 test runtime if reference kernel is not running on device.
+      if mode == "functional_L1" and profiler_flags_for_verification == "host" :
+        problem_waves = [0.5, 2.5]
+      
+
       if dynamic_cluster:
         if mode == "functional_L0":
-          runtime_cluster_shapes = [[1,1,1],          [2,1,1], [2,2,1],          [4,1,1],                   [4,4,1]]
+          runtime_cluster_shapes = [[1,1,1],                   [2,2,1]]
         else:
           runtime_cluster_shapes = [[1,1,1], [1,2,1], [2,1,1], [2,2,1], [1,4,1], [4,1,1], [2,4,1], [4,2,1], [4,4,1]]
+          # reduce L1 test runtime if reference kernel is not running on device.
+          if profiler_flags_for_verification == "host":
+            runtime_cluster_shapes = [[1,1,1], [1,2,1], [2,1,1], [2,2,1], [1,4,1], [4,1,1]]
         cta_tile_shape_m, cta_tile_shape_n, cta_tile_shape_k = operation.tile_description.threadblock_shape
       else:
         runtime_cluster_shapes = [operation.tile_description.cluster_shape]
@@ -720,10 +692,16 @@ def emit_gemm_kernel_testlist(manifest, curr_build_dir, arch, mode
                       batch_count = 3 if mode == "functional_L0" else 5
                   gemm_op = "gemm"
 
-                  profiler_reference_computing_override = profiler_reference_computing
-                  if "bstensorop" in kernel_name:
-                    profiler_reference_computing_override = "--mode=trace"
+                  grouped = is_grouped(manifest.operations_by_name[kernel_name].gemm_kind)
+                  num_groups = 1
+                  if grouped:
+                    gemm_op = "grouped_gemm"
+                    num_groups = 3 # small to limit test time in host block-scaled reference kernels
+                    batch_count = 1
+                  elif "bstensorop" in kernel_name:
                     gemm_op = "block_scaled_gemm"
+                  elif is_blockwise(manifest.operations_by_name[kernel_name].gemm_kind):
+                    gemm_op = "blockwise_gemm"
 
                   problem_size_category = ['smallK','largeK'][index_k] + '_' + ['beta==0','beta!=0'][bool(beta)]
 
@@ -740,7 +718,7 @@ def emit_gemm_kernel_testlist(manifest, curr_build_dir, arch, mode
                       'n' : n,
                       'k' : k,
                       'beta' : beta,
-                      'flops_per_byte' : _computeFlopsPerByte(operation, m, n, k, batch_count, beta)
+                      'flops_per_byte' : _computeFlopsPerByte(operation, m, n, k, batch_count, beta, num_groups)
                     },
                     "runtime_params": {
                       'ctas_per_mma_instruction' : ctas_per_mma_instruction,
@@ -763,11 +741,14 @@ def emit_gemm_kernel_testlist(manifest, curr_build_dir, arch, mode
                     metadata_dict["runtime_params"]["runtime_datatype_b"] = runtime_datatype_b
 
                   testcase_metadata = [
-                    f"cutlass_profiler --operation={gemm_op} {profiler_reference_computing_override} --error-on-no-match --error-if-nothing-is-profiled" +
+                    f"cutlass_profiler --operation={gemm_op}" +
+                    (f" --verification-providers=device --providers=cutlass" if profiler_flags_for_verification == "device" else " --mode=trace") +
+                    f" --error-on-no-match --error-if-nothing-is-profiled" +
                     f" --kernels={kernel_name}" +
                     f" --m={str(m)}" +
                     f" --n={str(n)}" +
                     f" --k={str(k)}" +
+                    (f" --num_groups={str(num_groups)}" if grouped else "") +
                     f" --cluster_m={str(cluster_shape_m)}" +
                     f" --cluster_n={str(cluster_shape_n)}" +
                     f" --cluster_k={str(cluster_shape_k)}" +
@@ -775,7 +756,7 @@ def emit_gemm_kernel_testlist(manifest, curr_build_dir, arch, mode
                     f" --cluster_n_fallback={str(cluster_n_fallback)}" +
                     f" --cluster_k_fallback={str(cluster_k_fallback)}" +
                     f" --beta={str(beta)}" +
-                    f" --batch_count={str(batch_count)}" +
+                    ("" if grouped else f" --batch_count={str(batch_count)}") +
                     f" --swizzle_size={str(swizzle_size)}" +
                     f" --verification-required={str(verification_required).lower()}"
                   ] \
@@ -788,7 +769,7 @@ def emit_gemm_kernel_testlist(manifest, curr_build_dir, arch, mode
                   testcase_metadata.append(json.dumps(metadata_dict))
                   testlist_csv_rows.append(testcase_metadata)
                   testcase_counter += 1
-    
+
                   alpha = 1.0
 
                   if dynamic_datatype:
