@@ -99,7 +99,7 @@ template <typename OperatorClass> struct ArchMap<arch::Sm86, OperatorClass> {
 
 template <typename OperatorClass> struct ArchMap<arch::Sm89, OperatorClass> {
   static int const kMin = 89;
-  static int const kMax = 89;
+  static int const kMax = 100;
 };
 
 template <typename OperatorClass> struct ArchMap<arch::Sm90, OperatorClass> {
@@ -127,12 +127,25 @@ template <typename OperatorClass> struct ArchMap<arch::Sm100, OperatorClass> {
 
 template <> struct ArchMap<arch::Sm100, arch::OpClassTensorOp> {
   static int const kMin = 100;
-  static int const kMax = 101;
+  #if (__CUDACC_VER_MAJOR__ >= 13)
+    static int const kMax = 110;
+  #else
+      static int const kMax = 103;
+  #endif // __CUDACC_VER_MAJOR__ >= 13
+};
+
+template <typename OperatorClass> struct ArchMap<arch::Sm103, OperatorClass> {
+  static int const kMin = 103;
+  static int const kMax = 1024;
+};
+template <> struct ArchMap<arch::Sm103, arch::OpClassTensorOp> {
+  static int const kMin = 103;
+  static int const kMax = 103;
 };
 
 template <typename OperatorClass> struct ArchMap<arch::Sm120, OperatorClass> {
   static int const kMin = 120;
-  static int const kMax = 120;
+  static int const kMax = 121;
 };
 
 /////////////////////////////////////////////////////////////////////////////////////////////////
