@@ -1,5 +1,5 @@
 /***************************************************************************************************
- * Copyright (c) 2017 - 2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2017 - 2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: BSD-3-Clause
  *
  * Redistribution and use in source and binary forms, with or without
@@ -67,7 +67,8 @@ template <
   typename LayoutC,
   typename ScalarType,
   typename ComputeType,
-  typename ConvertOp = NumericConverter<ElementC, ScalarType>,
+  typename ElementD = ElementC,
+  typename ConvertOp = NumericConverter<ElementD, ScalarType>,
   typename InnerProductOp = multiply_add<ComputeType>,
   int kMblock = 4,
   int kNblock = 4
@@ -81,7 +82,7 @@ __global__ void GemmComplex(
   ComplexTransform transform_b,
   ScalarType beta,
   TensorRef<ElementC, LayoutC> tensor_c,
-  TensorRef<ElementC, LayoutC> tensor_d,
+  TensorRef<ElementD, LayoutC> tensor_d,
   ComputeType initial_accum,
   int batch_count = 1,
   int64_t batch_stride_A = 0,
@@ -198,7 +199,8 @@ template <
   typename LayoutC,
   typename ScalarType,
   typename ComputeType,
-  typename ConvertOp = NumericConverter<ElementC, ScalarType>,
+  typename ElementD = ElementC,
+  typename ConvertOp = NumericConverter<ElementD, ScalarType>,
   typename InnerProductOp = multiply_add<ComputeType>
 >
 void GemmComplex(
@@ -210,7 +212,7 @@ void GemmComplex(
   ComplexTransform transform_b,
   ScalarType beta,
   TensorRef<ElementC, LayoutC> tensor_c,
-  TensorRef<ElementC, LayoutC> tensor_d,
+  TensorRef<ElementD, LayoutC> tensor_d,
   ComputeType initial_accum,
   int batch_count = 1,
   int64_t batch_stride_A = 0,
@@ -243,6 +245,7 @@ void GemmComplex(
       LayoutC,
       ScalarType,
       ComputeType,
+      ElementD,
       ConvertOp,
       InnerProductOp,
       kMblock,
@@ -285,6 +288,7 @@ void GemmComplex(
       LayoutC,
       ScalarType,
       ComputeType,
+      ElementD,
       ConvertOp,
       InnerProductOp,
       kBigMblock,
@@ -322,7 +326,8 @@ template <
   typename LayoutB,
   typename ElementC,
   typename LayoutC,
-  typename ScalarType
+  typename ScalarType,
+  typename ElementD = ElementC
 >
 void GemmComplex(
   gemm::GemmCoord problem_size,
@@ -333,7 +338,7 @@ void GemmComplex(
   ComplexTransform transform_b,
   ScalarType beta,
   TensorRef<ElementC, LayoutC> tensor_c,
-  TensorRef<ElementC, LayoutC> tensor_d) {
+  TensorRef<ElementD, LayoutC> tensor_d) {
 
   GemmComplex(problem_size, alpha, tensor_a, transform_a, tensor_b, transform_b, beta, tensor_c, tensor_d, ScalarType(0));
 }
