@@ -48,19 +48,22 @@
 #include "cutlass_extensions/gemm/warp/mma_tensorop_dequantizer.h"
 #include "cutlass_extensions/interleaved_numeric_conversion.h"
 
-#include "cutlass_extensions/ft_gemm_configs.h"
 #include "cutlass_extensions/gemm/kernel/mixed_gemm_B_layout.h"
+#include "cutlass_extensions/gemm_configs.h"
 
 /////////////////////////////////////////////////////////////////////////////////////////////////
 
-namespace cutlass {
-namespace gemm {
-namespace threadblock {
+namespace cutlass
+{
+namespace gemm
+{
+namespace threadblock
+{
 
 /////////////////////////////////////////////////////////////////////////////////////////////////
 
 /// Structure to compute the matrix product targeting CUDA cores and SIMT math instructions.
-template<
+template <
     /// Size of the Gemm problem - concept: gemm::GemmShape<>
     typename Shape_,
     /// Iterates over tiles of A operand in global memory
@@ -89,31 +92,36 @@ template<
     typename TransformBAfterLDG_,
     /// Converter for B matrix applited immediately after the LDS
     typename TransformBAfterLDS_,
+    /// The quantization operator being used
+    WeightOnlyQuantOp QuantOp_,
     /// Used for partial specialization
     typename Enable = bool>
-class DqMmaPipelined: public DqMmaBase<Shape_, Policy_, typename SmemIteratorScale_::Element, 2> {
+class DqMmaPipelined : public DqMmaBase<Shape_, Policy_, typename SmemIteratorScale_::Element, 2, QuantOp_>
+{
 public:
     ///< Base class
-    using Base = DqMmaBase<Shape_, Policy_, typename SmemIteratorScale_::Element, 2>;
+    using Base = DqMmaBase<Shape_, Policy_, typename SmemIteratorScale_::Element, 2, QuantOp_>;
 
-    using Shape     = Shape_;      ///< Size of the Gemm problem - concept: gemm::GemmShape<>
-    using IteratorA = IteratorA_;  ///< Iterates over tiles of A operand in global memory
-    using IteratorB = IteratorB_;  ///< Iterates over tiles of B operand in global memory
-    using ElementC  = ElementC_;   ///< Data type of accumulator matrix
-    using LayoutC   = LayoutC_;    ///< Layout of accumulator matrix
-    using Policy    = Policy_;     ///< Policy describing tuning details
+    using Shape = Shape_;         ///< Size of the Gemm problem - concept: gemm::GemmShape<>
+    using IteratorA = IteratorA_; ///< Iterates over tiles of A operand in global memory
+    using IteratorB = IteratorB_; ///< Iterates over tiles of B operand in global memory
+    using ElementC = ElementC_;   ///< Data type of accumulator matrix
+    using LayoutC = LayoutC_;     ///< Layout of accumulator matrix
+    using Policy = Policy_;       ///< Policy describing tuning details
 
     using IteratorScale = IteratorScale_;
-    using ElementScale  = typename IteratorScale::Element;
-    using LayoutScale   = typename IteratorScale::Layout;
+    using ElementScale = typename IteratorScale::Element;
+    using LayoutScale = typename IteratorScale::Layout;
 
-    using SmemIteratorA     = SmemIteratorA_;
-    using SmemIteratorB     = SmemIteratorB_;
+    using SmemIteratorA = SmemIteratorA_;
+    using SmemIteratorB = SmemIteratorB_;
     using SmemIteratorScale = SmemIteratorScale_;
 
     using TransformBAfterLDG = TransformBAfterLDG_;
     using TransformBAfterLDS = TransformBAfterLDS_;
 
+    static constexpr WeightOnlyQuantOp QuantOp = QuantOp_;
+
     //
     // Dependent types
     //
@@ -136,12 +144,8 @@ public:
     /// Obtain the arch tag from the warp-level operator
     using ArchTag = typename Policy::Operator::ArchTag;
 
-    using Dequantizer = warp::MmaTensorOpDequantizer<Operator,
-                                                     typename Base::WarpGemm,
-                                                     Operand::kB,
-                                                     typename SmemIteratorScale::Fragment::Element,
-                                                     LayoutScale,
-                                                     32>;
+    using Dequantizer = warp::MmaTensorOpDequantizer<Operator, typename Base::WarpGemm, Operand::kB,
+        typename SmemIteratorScale::Fragment::Element, LayoutScale, 32, QuantOp>;
 
     /// Complex transform on A operand
     static ComplexTransform const kTransformA = Operator::kTransformA;
@@ -157,13 +161,13 @@ private:
     using WarpFragmentB = typename Operator::FragmentB;
     Dequantizer warp_dequantizer_;
 
-    using ElementB          = typename IteratorB::Element;
+    using ElementB = typename IteratorB::Element;
     using LayoutDetailsForB = kernel::LayoutDetailsB<ElementB, ArchTag>;
 
-    static constexpr bool RequiresTileInterleave =
-        layout::IsColumnMajorTileInterleave<typename LayoutDetailsForB::Layout>::value;
+    static constexpr bool RequiresTileInterleave
+        = layout::IsColumnMajorTileInterleave<typename LayoutDetailsForB::Layout>::value;
     static_assert(!RequiresTileInterleave || (RequiresTileInterleave && (Shape::kK == LayoutDetailsForB::ThreadblockK)),
-                  "Layout K must match threadblockK");
+        "Layout K must match threadblockK");
 
 protected:
     /// Iterator to write threadblock-scoped tile of A operand to shared memory
@@ -179,18 +183,20 @@ public:
     /// Construct from tensor references
     CUTLASS_DEVICE
     DqMmaPipelined(typename Base::SharedStorage&
-                       shared_storage,  ///< Shared storage needed for internal use by threadblock-scoped GEMM
-                   int thread_idx,      ///< ID within the threadblock
-                   int warp_idx,        ///< ID of warp
-                   int lane_idx         ///< ID of each thread within a warp
-                   ):
-        Base(shared_storage, thread_idx, warp_idx, lane_idx),
-        warp_dequantizer_({shared_storage.operand_scale.data(), LayoutScale(Shape::kN)},
-                          (warp_idx % (Base::WarpCount::kM * Base::WarpCount::kN)) / Base::WarpCount::kM,
-                          lane_idx),
-        smem_iterator_A_(shared_storage.operand_A_ref(), thread_idx),
-        smem_iterator_B_(shared_storage.operand_B_ref(), thread_idx),
-        smem_iterator_scale_(LayoutScale(Shape::kN), shared_storage.operand_scale.data(), {1, Shape::kN}, thread_idx)
+                       shared_storage, ///< Shared storage needed for internal use by threadblock-scoped GEMM
+        const int group_size, ///< Will not be used, just to adapt to finegrained modifications and make the compilation
+                              ///< successful. Because DqMmaPipelined is only enabled for sm<80, so even if this
+                              ///< argument is not added, it does not affect compilation for sm>=80.
+        int thread_idx,       ///< ID within the threadblock
+        int warp_idx,         ///< ID of warp
+        int lane_idx          ///< ID of each thread within a warp
+        )
+        : Base(shared_storage, thread_idx, warp_idx, lane_idx)
+        , warp_dequantizer_({shared_storage.operand_scale.data(), LayoutScale(Shape::kN)},
+              (warp_idx % (Base::WarpCount::kM * Base::WarpCount::kN)) / Base::WarpCount::kM, lane_idx)
+        , smem_iterator_A_(shared_storage.operand_A_ref(), thread_idx)
+        , smem_iterator_B_(shared_storage.operand_B_ref(), thread_idx)
+        , smem_iterator_scale_(LayoutScale(Shape::kN), shared_storage.operand_scale.data(), {1, Shape::kN}, thread_idx)
     {
 
         // Compute warp location within threadblock tile by mapping the warp_id to
@@ -200,7 +206,7 @@ public:
         //   _k: the warp's position within the threadblock along the K dimension
 
         int warp_idx_mn = warp_idx % (Base::WarpCount::kM * Base::WarpCount::kN);
-        int warp_idx_k  = warp_idx / (Base::WarpCount::kM * Base::WarpCount::kN);
+        int warp_idx_k = warp_idx / (Base::WarpCount::kM * Base::WarpCount::kN);
 
         int warp_idx_m = warp_idx_mn % Base::WarpCount::kM;
         int warp_idx_n = warp_idx_mn / Base::WarpCount::kM;
@@ -212,13 +218,13 @@ public:
 
     /// Perform a threadblock-scoped matrix multiply-accumulate
     CUTLASS_DEVICE
-    void operator()(int              gemm_k_iterations,  ///< number of iterations of the mainloop
-                    FragmentC&       accum,              ///< destination accumulator tile
-                    IteratorA        iterator_A,         ///< iterator over A operand in global memory
-                    IteratorB        iterator_B,         ///< iterator over B operand in global memory
-                    IteratorScale    iterator_scale,     ///< iterator over scale operand in global memory
-                    FragmentC const& src_accum)
-    {  ///< source accumulator tile
+    void operator()(int gemm_k_iterations, ///< number of iterations of the mainloop
+        FragmentC& accum,                  ///< destination accumulator tile
+        IteratorA iterator_A,              ///< iterator over A operand in global memory
+        IteratorB iterator_B,              ///< iterator over B operand in global memory
+        IteratorScale iterator_scale,      ///< iterator over scale operand in global memory
+        FragmentC const& src_accum)
+    {                                      ///< source accumulator tile
 
         //
         // Prologue
@@ -226,23 +232,22 @@ public:
         TransformBAfterLDG ldg_converter;
         TransformBAfterLDS lds_converter;
 
-        using TransformA =
-            NumericArrayConverter<typename WarpFragmentA::Element, typename FragmentA::Element, FragmentA::kElements>;
+        using TransformA
+            = NumericArrayConverter<typename WarpFragmentA::Element, typename FragmentA::Element, FragmentA::kElements>;
 
         using TransformScale = NumericArrayConverter<typename SmemIteratorScale::Fragment::Element,
-                                                     typename FragmentScale::Element,
-                                                     FragmentScale::kElements>;
+            typename FragmentScale::Element, FragmentScale::kElements>;
 
         // These transforms are mainly to handle when we have bfloat activations and weights in GMEM and want
         // to issue HMMA on architectures older than Ampere. We will convert to FP16 before STS.
-        TransformA     transformA;
+        TransformA transformA;
         TransformScale transformScale;
 
         // Perform accumulation in the 'd' output operand
         accum = src_accum;
 
-        FragmentA     tb_frag_A;
-        FragmentB     tb_frag_B;
+        FragmentA tb_frag_A;
+        FragmentB tb_frag_B;
         FragmentScale tb_frag_scales;
 
         using WarpFragmentScale = typename Dequantizer::FragmentScale;
@@ -301,18 +306,21 @@ public:
 
         // Note: The main loop does not support Base::kWarpGemmIterations == 2.
         CUTLASS_GEMM_LOOP
-        for (; gemm_k_iterations > 0; --gemm_k_iterations) {
+        for (; gemm_k_iterations > 0; --gemm_k_iterations)
+        {
             //
             // Loop over GEMM K dimension
             //
 
             CUTLASS_PRAGMA_UNROLL
-            for (int warp_mma_k = 0; warp_mma_k < Base::kWarpGemmIterations; ++warp_mma_k) {
+            for (int warp_mma_k = 0; warp_mma_k < Base::kWarpGemmIterations; ++warp_mma_k)
+            {
 
                 // Load warp-level tiles from shared memory, wrapping to k offset if this is the last group
                 // as the case may be.
 
-                if (warp_mma_k == Base::kWarpGemmIterations - 1) {
+                if (warp_mma_k == Base::kWarpGemmIterations - 1)
+                {
 
                     // Write fragments to shared memory
                     this->smem_iterator_A_.store(transformA(tb_frag_A));
@@ -325,11 +333,13 @@ public:
                     ++this->smem_iterator_B_;
 
                     // Add negative offsets to return iterators to the 'start' of the circular buffer in shared memory
-                    if (smem_write_stage_idx == 1) {
+                    if (smem_write_stage_idx == 1)
+                    {
                         this->smem_iterator_A_.add_tile_offset({0, -Base::kStages});
                         this->smem_iterator_B_.add_tile_offset({-Base::kStages, 0});
                     }
-                    else {
+                    else
+                    {
                         this->warp_tile_iterator_A_.add_tile_offset(
                             {0, -Base::kStages * Policy::kPartitionsK * Base::kWarpGemmIterations});
                         this->warp_tile_iterator_B_.add_tile_offset(
@@ -344,16 +354,18 @@ public:
                 ++this->warp_tile_iterator_A_;
 
                 const int warp_tileB_k_compute_offset = warp_mma_k % Base::kNumKIterationsPerWarpBLoad;
-                const int warp_tileB_k_load_offset    = warp_mma_k / Base::kNumKIterationsPerWarpBLoad;
+                const int warp_tileB_k_load_offset = warp_mma_k / Base::kNumKIterationsPerWarpBLoad;
                 // We are just about to finish computing on a fragment of B, so initiate the load for the next fragment.
-                if (warp_tileB_k_compute_offset == Base::kNumKIterationsPerWarpBLoad - 1) {
-                    this->warp_tile_iterator_B_.set_kgroup_index((warp_tileB_k_load_offset + 1)
-                                                                 % Base::kWarpGemmIterationsForB);
+                if (warp_tileB_k_compute_offset == Base::kNumKIterationsPerWarpBLoad - 1)
+                {
+                    this->warp_tile_iterator_B_.set_kgroup_index(
+                        (warp_tileB_k_load_offset + 1) % Base::kWarpGemmIterationsForB);
                     this->warp_tile_iterator_B_.load(warp_frag_B[(warp_tileB_k_load_offset + 1) % 2]);
                     ++this->warp_tile_iterator_B_;
                 }
 
-                if (warp_mma_k == 0) {
+                if (warp_mma_k == 0)
+                {
 
                     iterator_A.load(tb_frag_A);
                     iterator_B.load(tb_frag_B);
@@ -366,8 +378,8 @@ public:
                     iterator_B.clear_mask(gemm_k_iterations <= 2);
                 }
 
-                typename TransformBAfterLDS::result_type converted_frag_B =
-                    lds_converter(warp_frag_B[warp_tileB_k_load_offset % 2]);
+                typename TransformBAfterLDS::result_type converted_frag_B
+                    = lds_converter(warp_frag_B[warp_tileB_k_load_offset % 2]);
                 warp_dequantizer_.dequantize(converted_frag_B, warp_frag_scales);
                 run_warp_mma(
                     warp_mma, accum, warp_frag_A[warp_mma_k % 2], converted_frag_B, accum, warp_tileB_k_compute_offset);
@@ -378,8 +390,8 @@ public:
 
 /////////////////////////////////////////////////////////////////////////////////////////////////
 
-}  // namespace threadblock
-}  // namespace gemm
-}  // namespace cutlass
+} // namespace threadblock
+} // namespace gemm
+} // namespace cutlass
 
 /////////////////////////////////////////////////////////////////////////////////////////////////
