@@ -60,6 +60,12 @@ template<typename ValueType>
 inline void SetEnv(const char *key,
                    ValueType value);
 
+/*!
+ * \brief Unset environment variable.
+ * \param key the name of environment variable.
+ */
+inline void UnsetEnv(const char *key);
+
 /*! \brief internal namespace for parameter manangement */
 namespace parameter {
 // forward declare ParamManager
@@ -348,7 +354,7 @@ class FieldAccessEntry {
    */
   virtual void Set(void *head, const std::string &value) const = 0;
   // check if value is OK
-  virtual void Check(void *head) const {}
+  virtual void Check(void * /*head*/) const {}
   /*!
    * \brief get the string representation of value.
    * \param head the pointer to the head of the struct
@@ -1149,5 +1155,14 @@ inline void SetEnv(const char *key,
   setenv(key, e.GetStringValue(&value).c_str(), 1);
 #endif  // _WIN32
 }
+
+// implement UnsetEnv
+inline void UnsetEnv(const char *key) {
+#ifdef _WIN32
+  _putenv_s(key, "");
+#else
+  unsetenv(key);
+#endif  // _WIN32
+}
 }  // namespace dmlc
 #endif  // DMLC_PARAMETER_H_
