@@ -532,16 +532,12 @@ class PR:
     def rerun_jenkins_ci(self) -> None:
         job_names = [
             "tvm-arm",
-            "tvm-cortexm",
             "tvm-cpu",
             "tvm-docker",
             "tvm-gpu",
             "tvm-hexagon",
             "tvm-i386",
             "tvm-lint",
-            "tvm-minimal",
-            "tvm-minimal-cross-isa",
-            "tvm-riscv",
             "tvm-wasm",
             "tvm-unity",
         ]
