@@ -19,5 +19,5 @@
 set -e
 set -u
 
-cd ${SRC_DIR}/python
-${PYTHON} setup.py install --single-version-externally-managed --record=/tmp/record.txt
+cd ${SRC_DIR}
+${PYTHON} -m pip install . --no-deps --no-build-isolation --record=/tmp/record.txt
