@@ -21,5 +21,9 @@ set -u
 set -o pipefail
 
 pip3 install \
-    keras==2.9 \
-    tensorflow==2.9.1
+    keras==3.5.0 \
+    tensorflow==2.18.0
+
+# tensorflow 2.18.0 requires ml-dtypes<0.5.0,>=0.4.0
+# ml_dtypes needs to be installed after tensorflow
+pip3 install ml_dtypes==0.5.1 --no-deps
