@@ -23,12 +23,11 @@
 #include <tvm/ffi/container/map.h>
 #include <tvm/ffi/function.h>
 #include <tvm/ffi/optional.h>
-#include <tvm/ffi/reflection/reflection.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/ffi/string.h>
 #include <tvm/ir/module.h>
-#include <tvm/node/reflection.h>
-#include <tvm/runtime/ndarray.h>
 #include <tvm/runtime/object.h>
+#include <tvm/runtime/tensor.h>
 #include <tvm/target/target.h>
 
 namespace tvm {
@@ -42,7 +41,7 @@ class BuilderInputNode : public runtime::Object {
   /*! \brief The target to be built for. */
   Target target;
   /*! \brief Parameters for Relax build module. */
-  Optional<Map<String, runtime::NDArray>> params;
+  ffi::Optional<ffi::Map<ffi::String, runtime::Tensor>> params;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -51,9 +50,8 @@ class BuilderInputNode : public runtime::Object {
         .def_ro("target", &BuilderInputNode::target)
         .def_ro("params", &BuilderInputNode::params);
   }
-
-  static constexpr const char* _type_key = "meta_schedule.BuilderInput";
-  TVM_DECLARE_FINAL_OBJECT_INFO(BuilderInputNode, runtime::Object);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("meta_schedule.BuilderInput", BuilderInputNode,
+                                    runtime::Object);
 };
 
 /*!
@@ -68,18 +66,19 @@ class BuilderInput : public runtime::ObjectRef {
    * \param target The target to be built for.
    * \param params Parameters for Relax build module.
    */
-  TVM_DLL explicit BuilderInput(IRModule mod, Target target,
-                                Optional<Map<String, runtime::NDArray>> params = std::nullopt);
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(BuilderInput, runtime::ObjectRef, BuilderInputNode);
+  TVM_DLL explicit BuilderInput(
+      IRModule mod, Target target,
+      ffi::Optional<ffi::Map<ffi::String, runtime::Tensor>> params = std::nullopt);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(BuilderInput, runtime::ObjectRef, BuilderInputNode);
 };
 
 /*! \brief The builder's output, containing the artifact path or error message if any. */
 class BuilderResultNode : public runtime::Object {
  public:
   /*! \brief The path to the built artifact. */
-  Optional<String> artifact_path;
+  ffi::Optional<ffi::String> artifact_path;
   /*! \brief The error message if any. */
-  Optional<String> error_msg;
+  ffi::Optional<ffi::String> error_msg;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -87,9 +86,8 @@ class BuilderResultNode : public runtime::Object {
         .def_ro("artifact_path", &BuilderResultNode::artifact_path)
         .def_ro("error_msg", &BuilderResultNode::error_msg);
   }
-
-  static constexpr const char* _type_key = "meta_schedule.BuilderResult";
-  TVM_DECLARE_FINAL_OBJECT_INFO(BuilderResultNode, runtime::Object);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("meta_schedule.BuilderResult", BuilderResultNode,
+                                    runtime::Object);
 };
 
 /*!
@@ -103,8 +101,10 @@ class BuilderResult : public runtime::ObjectRef {
    * \param artifact_path The path to the built artifact.
    * \param error_msg The error message if any.
    */
-  TVM_DLL explicit BuilderResult(Optional<String> artifact_path, Optional<String> error_msg);
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(BuilderResult, runtime::ObjectRef, BuilderResultNode);
+  TVM_DLL explicit BuilderResult(ffi::Optional<ffi::String> artifact_path,
+                                 ffi::Optional<ffi::String> error_msg);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(BuilderResult, runtime::ObjectRef,
+                                                BuilderResultNode);
 };
 
 /*! \brief The abstract builder interface. */
@@ -117,16 +117,16 @@ class BuilderNode : public runtime::Object {
    * \param build_inputs The inputs to be built.
    * \return The build results.
    */
-  virtual Array<BuilderResult> Build(const Array<BuilderInput>& build_inputs) = 0;
+  virtual ffi::Array<BuilderResult> Build(const ffi::Array<BuilderInput>& build_inputs) = 0;
   /*!
    * \brief The function type of `Build` method.
    * \param build_inputs The inputs to be built.
    * \return The build results.
    */
-  using FBuild = ffi::TypedFunction<Array<BuilderResult>(const Array<BuilderInput>&)>;
+  using FBuild = ffi::TypedFunction<ffi::Array<BuilderResult>(const ffi::Array<BuilderInput>&)>;
 
-  static constexpr const char* _type_key = "meta_schedule.Builder";
-  TVM_DECLARE_BASE_OBJECT_INFO(BuilderNode, runtime::Object);
+  static constexpr const bool _type_mutable = true;
+  TVM_FFI_DECLARE_OBJECT_INFO("meta_schedule.Builder", BuilderNode, runtime::Object);
 };
 
 /*!
@@ -135,13 +135,20 @@ class BuilderNode : public runtime::Object {
  */
 class Builder : public runtime::ObjectRef {
  public:
+  /*!
+   * \brief Constructor from ObjectPtr<BuilderNode>.
+   * \param data The object pointer.
+   */
+  explicit Builder(ObjectPtr<BuilderNode> data) : ObjectRef(data) {
+    TVM_FFI_ICHECK(data != nullptr);
+  }
   /*!
    * \brief Create a builder with customized build method on the python-side.
    * \param f_build The packed function to the `Build` function..
    * \return The Builder created.
    */
   static Builder PyBuilder(BuilderNode::FBuild f_build);
-  TVM_DEFINE_MUTABLE_NOTNULLABLE_OBJECT_REF_METHODS(Builder, runtime::ObjectRef, BuilderNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(Builder, runtime::ObjectRef, BuilderNode);
 };
 
 /*! \brief An abstract builder with customized build method on the python-side. */
@@ -155,13 +162,11 @@ class PyBuilderNode : public BuilderNode {
     refl::ObjectDef<PyBuilderNode>().def_ro("f_build", &PyBuilderNode::f_build);
   }
 
-  Array<BuilderResult> Build(const Array<BuilderInput>& build_inputs) final {
+  ffi::Array<BuilderResult> Build(const ffi::Array<BuilderInput>& build_inputs) final {
     ICHECK(f_build != nullptr) << "PyBuilder's Build method not implemented!";
     return f_build(build_inputs);
   }
-
-  static constexpr const char* _type_key = "meta_schedule.PyBuilder";
-  TVM_DECLARE_FINAL_OBJECT_INFO(PyBuilderNode, BuilderNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("meta_schedule.PyBuilder", PyBuilderNode, BuilderNode);
 };
 
 }  // namespace meta_schedule
