@@ -22,8 +22,7 @@
 
 #include <tvm/ffi/function.h>
 #include <tvm/ffi/optional.h>
-#include <tvm/ffi/reflection/reflection.h>
-#include <tvm/node/reflection.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/runtime/object.h>
 #include <tvm/support/random_engine.h>
 #include <tvm/tir/schedule/schedule.h>
@@ -42,7 +41,8 @@ class MutatorNode : public runtime::Object {
   virtual ~MutatorNode() = default;
 
   static void RegisterReflection() {
-    // No fields to register
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<MutatorNode>();
   }
 
   /*!
@@ -58,8 +58,8 @@ class MutatorNode : public runtime::Object {
    * \param rand_state The random state for mutation.
    * \return None if mutator failed, otherwise return the mutated trace.
    */
-  virtual Optional<tir::Trace> Apply(const tir::Trace& trace,
-                                     support::LinearCongruentialEngine::TRandState* rand_state) = 0;
+  virtual ffi::Optional<tir::Trace> Apply(
+      const tir::Trace& trace, support::LinearCongruentialEngine::TRandState* rand_state) = 0;
 
   /*!
    * \brief Clone the mutator.
@@ -67,8 +67,8 @@ class MutatorNode : public runtime::Object {
    */
   virtual Mutator Clone() const = 0;
 
-  static constexpr const char* _type_key = "meta_schedule.Mutator";
-  TVM_DECLARE_BASE_OBJECT_INFO(MutatorNode, Object);
+  static constexpr const bool _type_mutable = true;
+  TVM_FFI_DECLARE_OBJECT_INFO("meta_schedule.Mutator", MutatorNode, Object);
 };
 
 /*!
@@ -87,7 +87,7 @@ class Mutator : public runtime::ObjectRef {
    * \param trace The given trace for mutation.
    * \return None if mutator failed, otherwise return the mutated trace.
    */
-  using FApply = ffi::TypedFunction<Optional<tir::Trace>(
+  using FApply = ffi::TypedFunction<ffi::Optional<tir::Trace>(
       const tir::Trace&, support::LinearCongruentialEngine::TRandState rand_state)>;
   /*!
    * \brief Clone the mutator.
@@ -98,7 +98,7 @@ class Mutator : public runtime::ObjectRef {
    * \brief Get the mutator as string with name.
    * \return The string of the mutator.
    */
-  using FAsString = ffi::TypedFunction<String()>;
+  using FAsString = ffi::TypedFunction<ffi::String()>;
   /*! \brief Create a Mutator that mutates the decision of instruction Sample-Perfect-Tile */
   TVM_DLL static Mutator MutateTileSize();
   /*!
@@ -133,15 +133,15 @@ class Mutator : public runtime::ObjectRef {
   TVM_DLL static Mutator PyMutator(FInitializeWithTuneContext f_initialize_with_tune_context,
                                    FApply f_apply, FClone f_clone, FAsString f_as_string);
   /*! \brief Create default mutators for LLVM */
-  TVM_DLL static Map<Mutator, FloatImm, void> DefaultLLVM();
+  TVM_DLL static ffi::Map<Mutator, FloatImm, void> DefaultLLVM();
   /*! \brief Create default mutators for CUDA */
-  TVM_DLL static Map<Mutator, FloatImm, void> DefaultCUDA();
+  TVM_DLL static ffi::Map<Mutator, FloatImm, void> DefaultCUDA();
   /*! \brief Create default mutators for CUDA with TensorCore */
-  TVM_DLL static Map<Mutator, FloatImm, void> DefaultCUDATensorCore();
+  TVM_DLL static ffi::Map<Mutator, FloatImm, void> DefaultCUDATensorCore();
   /*! \brief Create default mutators for Hexagon */
-  TVM_DLL static Map<Mutator, FloatImm, void> DefaultHexagon();
+  TVM_DLL static ffi::Map<Mutator, FloatImm, void> DefaultHexagon();
 
-  TVM_DEFINE_MUTABLE_OBJECT_REF_METHODS(Mutator, ObjectRef, MutatorNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(Mutator, ObjectRef, MutatorNode);
 };
 
 /*! \brief The mutator with customized methods on the python-side. */
@@ -168,12 +168,10 @@ class PyMutatorNode : public MutatorNode {
   }
 
   void InitializeWithTuneContext(const TuneContext& context) final;
-  Optional<tir::Trace> Apply(const tir::Trace& trace,
-                             support::LinearCongruentialEngine::TRandState* rand_state) final;
+  ffi::Optional<tir::Trace> Apply(const tir::Trace& trace,
+                                  support::LinearCongruentialEngine::TRandState* rand_state) final;
   Mutator Clone() const final;
-
-  static constexpr const char* _type_key = "meta_schedule.PyMutator";
-  TVM_DECLARE_FINAL_OBJECT_INFO(PyMutatorNode, MutatorNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("meta_schedule.PyMutator", PyMutatorNode, MutatorNode);
 };
 
 }  // namespace meta_schedule
