@@ -26,11 +26,10 @@
 #include <tvm/ffi/any.h>
 #include <tvm/ffi/container/array.h>
 #include <tvm/ffi/container/map.h>
-#include <tvm/ffi/reflection/reflection.h>
+#include <tvm/ffi/reflection/access_path.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/ffi/string.h>
 #include <tvm/node/functor.h>
-#include <tvm/node/object_path.h>
-#include <tvm/node/reflection.h>
 #include <tvm/runtime/data_type.h>
 
 #include <iostream>
@@ -38,10 +37,10 @@
 
 namespace tvm {
 
-class PrinterConfigNode : public Object {
+class PrinterConfigNode : public ffi::Object {
  public:
   /*! \brief A stack that tracks the names of the binding hierarchy */
-  Array<String> binding_names = {};
+  ffi::Array<ffi::String> binding_names = {};
   /*! \brief Whether or not to show metadata. */
   bool show_meta = false;
   /*! \brief The prefix of IR nodes */
@@ -114,13 +113,13 @@ class PrinterConfigNode : public Object {
   bool show_all_struct_info = true;
 
   /* \brief Object path to be underlined */
-  Array<ObjectPath> path_to_underline = Array<ObjectPath>();
+  ffi::Array<ffi::reflection::AccessPath> path_to_underline;
   /*! \brief Object path to be annotated. */
-  Map<ObjectPath, String> path_to_annotate = Map<ObjectPath, String>();
+  ffi::Map<ffi::reflection::AccessPath, ffi::String> path_to_annotate;
   /*! \brief Object to be underlined. */
-  Array<ObjectRef> obj_to_underline = Array<ObjectRef>();
+  ffi::Array<ObjectRef> obj_to_underline = ffi::Array<ObjectRef>();
   /*! \brief Object to be annotated. */
-  Map<ObjectRef, String> obj_to_annotate = Map<ObjectRef, String>();
+  ffi::Map<ObjectRef, ffi::String> obj_to_annotate = ffi::Map<ObjectRef, ffi::String>();
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -147,33 +146,35 @@ class PrinterConfigNode : public Object {
         .def_ro("obj_to_annotate", &PrinterConfigNode::obj_to_annotate);
   }
 
-  Array<String> GetBuiltinKeywords();
+  ffi::Array<ffi::String> GetBuiltinKeywords();
 
-  static constexpr const char* _type_key = "script.PrinterConfig";
-  TVM_DECLARE_FINAL_OBJECT_INFO(PrinterConfigNode, Object);
+  static constexpr const bool _type_mutable = true;
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.PrinterConfig", PrinterConfigNode, Object);
 };
 
 class PrinterConfig : public ObjectRef {
  public:
-  explicit PrinterConfig(Map<String, ffi::Any> config_dict = Map<String, ffi::Any>());
+  explicit PrinterConfig(
+      ffi::Map<ffi::String, ffi::Any> config_dict = ffi::Map<ffi::String, ffi::Any>());
 
-  TVM_DEFINE_MUTABLE_NOTNULLABLE_OBJECT_REF_METHODS(PrinterConfig, runtime::ObjectRef,
-                                                    PrinterConfigNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(PrinterConfig, runtime::ObjectRef,
+                                                PrinterConfigNode);
 };
 
 /*! \brief Legacy behavior of ReprPrinter. */
 class TVMScriptPrinter {
  public:
   /* Convert the object to TVMScript format */
-  static std::string Script(const ObjectRef& node, const Optional<PrinterConfig>& cfg);
+  static std::string Script(const ObjectRef& node, const ffi::Optional<PrinterConfig>& cfg);
   // Allow registration to be printer.
   using FType = NodeFunctor<std::string(const ObjectRef&, const PrinterConfig&)>;
   TVM_DLL static FType& vtable();
 };
 
-#define TVM_OBJECT_ENABLE_SCRIPT_PRINTER()                                                      \
-  std::string Script(const Optional<PrinterConfig>& config = std::nullopt) const {              \
-    return TVMScriptPrinter::Script(GetRef<ObjectRef>(this), config.value_or(PrinterConfig())); \
+#define TVM_OBJECT_ENABLE_SCRIPT_PRINTER()                                              \
+  std::string Script(const ffi::Optional<PrinterConfig>& config = std::nullopt) const { \
+    return TVMScriptPrinter::Script(ffi::GetRef<ObjectRef>(this),                       \
+                                    config.value_or(PrinterConfig()));                  \
   }
 
 }  // namespace tvm
