@@ -31,13 +31,13 @@ namespace relax {
 
 /*! \brief Attributes used in Conv1d operator */
 struct Conv1DAttrs : public AttrsNodeReflAdapter<Conv1DAttrs> {
-  Array<IntImm> strides;
-  Array<IntImm> padding;
-  Array<IntImm> dilation;
+  ffi::Array<IntImm> strides;
+  ffi::Array<IntImm> padding;
+  ffi::Array<IntImm> dilation;
   int groups;
-  String data_layout;
-  String kernel_layout;
-  String out_layout;
+  ffi::String data_layout;
+  ffi::String kernel_layout;
+  ffi::String out_layout;
   DataType out_dtype;
 
   static void RegisterReflection() {
@@ -70,20 +70,18 @@ struct Conv1DAttrs : public AttrsNodeReflAdapter<Conv1DAttrs> {
         .def_ro("out_dtype", &Conv1DAttrs::out_dtype,
                 "Output data type, set to explicit type under mixed precision setting");
   }
-
-  static constexpr const char* _type_key = "relax.attrs.Conv1DAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(Conv1DAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.Conv1DAttrs", Conv1DAttrs, BaseAttrsNode);
 };  // struct Conv1dAttrs
 
 /*! \brief Attributes used in Conv2d operator */
 struct Conv2DAttrs : public AttrsNodeReflAdapter<Conv2DAttrs> {
-  Array<IntImm> strides;
-  Array<IntImm> padding;
-  Array<IntImm> dilation;
+  ffi::Array<IntImm> strides;
+  ffi::Array<IntImm> padding;
+  ffi::Array<IntImm> dilation;
   int groups;
-  String data_layout;
-  String kernel_layout;
-  String out_layout;
+  ffi::String data_layout;
+  ffi::String kernel_layout;
+  ffi::String out_layout;
   DataType out_dtype;
 
   static void RegisterReflection() {
@@ -118,20 +116,18 @@ struct Conv2DAttrs : public AttrsNodeReflAdapter<Conv2DAttrs> {
         .def_ro("out_dtype", &Conv2DAttrs::out_dtype,
                 "Output data type, set to explicit type under mixed precision setting");
   }
-
-  static constexpr const char* _type_key = "relax.attrs.Conv2DAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(Conv2DAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.Conv2DAttrs", Conv2DAttrs, BaseAttrsNode);
 };  // struct Conv2dAttrs
 
 /*! \brief Attributes used in Conv3d operator */
 struct Conv3DAttrs : public AttrsNodeReflAdapter<Conv3DAttrs> {
-  Array<IntImm> strides;
-  Array<IntImm> padding;
-  Array<IntImm> dilation;
+  ffi::Array<IntImm> strides;
+  ffi::Array<IntImm> padding;
+  ffi::Array<IntImm> dilation;
   int groups;
-  String data_layout;
-  String kernel_layout;
-  String out_layout;
+  ffi::String data_layout;
+  ffi::String kernel_layout;
+  ffi::String out_layout;
   DataType out_dtype;
 
   static void RegisterReflection() {
@@ -168,21 +164,19 @@ struct Conv3DAttrs : public AttrsNodeReflAdapter<Conv3DAttrs> {
         .def_ro("out_dtype", &Conv3DAttrs::out_dtype,
                 "Output data type, set to explicit type under mixed precision setting");
   }
-
-  static constexpr const char* _type_key = "relax.attrs.Conv3DAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(Conv3DAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.Conv3DAttrs", Conv3DAttrs, BaseAttrsNode);
 };  // struct Conv3dAttrs
 
 /*! \brief Attributes used in Conv1DTranspose operator */
 struct Conv1DTransposeAttrs : public AttrsNodeReflAdapter<Conv1DTransposeAttrs> {
-  Array<IntImm> strides;
-  Array<IntImm> padding;
-  Array<IntImm> output_padding;
-  Array<IntImm> dilation;
+  ffi::Array<IntImm> strides;
+  ffi::Array<IntImm> padding;
+  ffi::Array<IntImm> output_padding;
+  ffi::Array<IntImm> dilation;
   int groups;
-  String data_layout;
-  String kernel_layout;
-  String out_layout;
+  ffi::String data_layout;
+  ffi::String kernel_layout;
+  ffi::String out_layout;
   DataType out_dtype;
 
   static void RegisterReflection() {
@@ -218,21 +212,20 @@ struct Conv1DTransposeAttrs : public AttrsNodeReflAdapter<Conv1DTransposeAttrs>
         .def_ro("out_dtype", &Conv1DTransposeAttrs::out_dtype,
                 "Output data type, set to explicit type under mixed precision setting");
   }
-
-  static constexpr const char* _type_key = "relax.attrs.Conv1DTransposeAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(Conv1DTransposeAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.Conv1DTransposeAttrs", Conv1DTransposeAttrs,
+                                    BaseAttrsNode);
 };  // struct Conv1DTransposeAttrs
 
 /*! \brief Attributes used in Conv2d operator */
 struct Conv2DTransposeAttrs : public AttrsNodeReflAdapter<Conv2DTransposeAttrs> {
-  Array<IntImm> strides;
-  Array<IntImm> padding;
-  Array<IntImm> output_padding;
-  Array<IntImm> dilation;
+  ffi::Array<IntImm> strides;
+  ffi::Array<IntImm> padding;
+  ffi::Array<IntImm> output_padding;
+  ffi::Array<IntImm> dilation;
   int groups;
-  String data_layout;
-  String kernel_layout;
-  String out_layout;
+  ffi::String data_layout;
+  ffi::String kernel_layout;
+  ffi::String out_layout;
   DataType out_dtype;
 
   static void RegisterReflection() {
@@ -270,21 +263,20 @@ struct Conv2DTransposeAttrs : public AttrsNodeReflAdapter<Conv2DTransposeAttrs>
         .def_ro("out_dtype", &Conv2DTransposeAttrs::out_dtype,
                 "Output data type, set to explicit type under mixed precision setting");
   }
-
-  static constexpr const char* _type_key = "relax.attrs.Conv2DTransposeAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(Conv2DTransposeAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.Conv2DTransposeAttrs", Conv2DTransposeAttrs,
+                                    BaseAttrsNode);
 };  // struct Conv2DTransposeAttrs
 
 /*! \brief Attributes used in max_pool1d and avg_pool1d operator */
 struct Pool1DAttrs : public AttrsNodeReflAdapter<Pool1DAttrs> {
-  Array<IntImm> pool_size;
-  Array<IntImm> strides;
-  Array<IntImm> padding;
-  Array<IntImm> dilation;
+  ffi::Array<IntImm> pool_size;
+  ffi::Array<IntImm> strides;
+  ffi::Array<IntImm> padding;
+  ffi::Array<IntImm> dilation;
   bool ceil_mode;
   bool count_include_pad;
-  String layout;
-  String out_layout;
+  ffi::String layout;
+  ffi::String out_layout;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -313,21 +305,19 @@ struct Pool1DAttrs : public AttrsNodeReflAdapter<Pool1DAttrs> {
                 "'N', 'C', 'W' stands for batch, channel, and width"
                 "dimensions respectively. Pooling is applied on the 'W' dimensions.");
   }
-
-  static constexpr const char* _type_key = "relax.attrs.Pool1DAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(Pool1DAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.Pool1DAttrs", Pool1DAttrs, BaseAttrsNode);
 };  // struct Pool1dAttrs
 
 /*! \brief Attributes used in max_pool2d and avg_pool2d operator */
 struct Pool2DAttrs : public AttrsNodeReflAdapter<Pool2DAttrs> {
-  Array<IntImm> pool_size;
-  Array<IntImm> strides;
-  Array<IntImm> padding;
-  Array<IntImm> dilation;
+  ffi::Array<IntImm> pool_size;
+  ffi::Array<IntImm> strides;
+  ffi::Array<IntImm> padding;
+  ffi::Array<IntImm> dilation;
   bool ceil_mode;
   bool count_include_pad;
-  String layout;
-  String out_layout;
+  ffi::String layout;
+  ffi::String out_layout;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -358,21 +348,19 @@ struct Pool2DAttrs : public AttrsNodeReflAdapter<Pool2DAttrs> {
                 "dimensions respectively. Pooling is applied on the 'H' and"
                 "'W' dimensions.");
   }
-
-  static constexpr const char* _type_key = "relax.attrs.Pool2DAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(Pool2DAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.Pool2DAttrs", Pool2DAttrs, BaseAttrsNode);
 };  // struct Pool2dAttrs
 
 /*! \brief Attributes used in max_pool3d and avg_pool3d operator */
 struct Pool3DAttrs : public AttrsNodeReflAdapter<Pool3DAttrs> {
-  Array<IntImm> pool_size;
-  Array<IntImm> strides;
-  Array<IntImm> padding;
-  Array<IntImm> dilation;
+  ffi::Array<IntImm> pool_size;
+  ffi::Array<IntImm> strides;
+  ffi::Array<IntImm> padding;
+  ffi::Array<IntImm> dilation;
   bool ceil_mode;
   bool count_include_pad;
-  String layout;
-  String out_layout;
+  ffi::String layout;
+  ffi::String out_layout;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -403,16 +391,14 @@ struct Pool3DAttrs : public AttrsNodeReflAdapter<Pool3DAttrs> {
                 "dimensions respectively. Pooling is applied on the 'D', 'H' and"
                 "'W' dimensions.");
   }
-
-  static constexpr const char* _type_key = "relax.attrs.Pool3DAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(Pool3DAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.Pool3DAttrs", Pool3DAttrs, BaseAttrsNode);
 };  // struct Pool3dAttrs
 
 /*! \brief Attributes for 1d adaptive pool operator */
 struct AdaptivePool1DAttrs : public AttrsNodeReflAdapter<AdaptivePool1DAttrs> {
-  Optional<Array<IntImm>> output_size;
-  String layout;
-  String out_layout;
+  ffi::Optional<ffi::Array<IntImm>> output_size;
+  ffi::String layout;
+  ffi::String out_layout;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -429,16 +415,15 @@ struct AdaptivePool1DAttrs : public AttrsNodeReflAdapter<AdaptivePool1DAttrs> {
                 "dimensions respectively. Pooling is applied on the"
                 "'W' dimensions.");
   }
-
-  static constexpr const char* _type_key = "relax.attrs.AdaptivePool1DAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(AdaptivePool1DAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.AdaptivePool1DAttrs", AdaptivePool1DAttrs,
+                                    BaseAttrsNode);
 };  // struct AdaptivePool1DAttrs
 
 /*! \brief Attributes for 2d adaptive pool operator */
 struct AdaptivePool2DAttrs : public AttrsNodeReflAdapter<AdaptivePool2DAttrs> {
-  Optional<Array<IntImm>> output_size;
-  String layout;
-  String out_layout;
+  ffi::Optional<ffi::Array<IntImm>> output_size;
+  ffi::String layout;
+  ffi::String out_layout;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -455,16 +440,15 @@ struct AdaptivePool2DAttrs : public AttrsNodeReflAdapter<AdaptivePool2DAttrs> {
                 "dimensions respectively. Pooling is applied on the 'H' and"
                 "'W' dimensions.");
   }
-
-  static constexpr const char* _type_key = "relax.attrs.AdaptivePool2DAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(AdaptivePool2DAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.AdaptivePool2DAttrs", AdaptivePool2DAttrs,
+                                    BaseAttrsNode);
 };  // struct AdaptivePool2DAttrs
 
 /*! \brief Attributes for 3d adaptive pool operator */
 struct AdaptivePool3DAttrs : public AttrsNodeReflAdapter<AdaptivePool3DAttrs> {
-  Optional<Array<IntImm>> output_size;
-  String layout;
-  String out_layout;
+  ffi::Optional<ffi::Array<IntImm>> output_size;
+  ffi::String layout;
+  ffi::String out_layout;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -481,9 +465,8 @@ struct AdaptivePool3DAttrs : public AttrsNodeReflAdapter<AdaptivePool3DAttrs> {
                 "dimensions respectively. Pooling is applied on 'D', 'H' and"
                 "'W' dimensions.");
   }
-
-  static constexpr const char* _type_key = "relax.attrs.AdaptivePool3DAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(AdaptivePool3DAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.AdaptivePool3DAttrs", AdaptivePool3DAttrs,
+                                    BaseAttrsNode);
 };  // struct AdaptivePool3DAttrs
 
 /*! \brief Attributes used in softmax operators */
@@ -495,9 +478,7 @@ struct SoftmaxAttrs : public AttrsNodeReflAdapter<SoftmaxAttrs> {
     refl::ObjectDef<SoftmaxAttrs>().def_ro("axis", &SoftmaxAttrs::axis,
                                            "The axis to sum over when computing softmax.");
   }
-
-  static constexpr const char* _type_key = "relax.attrs.SoftmaxAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(SoftmaxAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.SoftmaxAttrs", SoftmaxAttrs, BaseAttrsNode);
 };
 
 /*! \brief Attributes used in softmax operators */
@@ -509,9 +490,7 @@ struct LeakyReluAttrs : public AttrsNodeReflAdapter<LeakyReluAttrs> {
     refl::ObjectDef<LeakyReluAttrs>().def_ro("alpha", &LeakyReluAttrs::alpha,
                                              "The slope of the negative part.");
   }
-
-  static constexpr const char* _type_key = "relax.attrs.LeakyReluAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(LeakyReluAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.LeakyReluAttrs", LeakyReluAttrs, BaseAttrsNode);
 };
 
 /*! \brief Attributes used in softplus operators */
@@ -527,9 +506,7 @@ struct SoftplusAttrs : public AttrsNodeReflAdapter<SoftplusAttrs> {
         .def_ro("threshold", &SoftplusAttrs::threshold,
                 "Value determining when to use linear approximation for numerical stability.");
   }
-
-  static constexpr const char* _type_key = "relax.attrs.SoftplusAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(SoftplusAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.SoftplusAttrs", SoftplusAttrs, BaseAttrsNode);
 };
 
 /*! \brief Attributes used in PReLU operator */
@@ -541,9 +518,7 @@ struct PReluAttrs : public AttrsNodeReflAdapter<PReluAttrs> {
     refl::ObjectDef<PReluAttrs>().def_ro("axis", &PReluAttrs::axis,
                                          "The axis along which the alpha values are applied.");
   }
-
-  static constexpr const char* _type_key = "relax.attrs.PReluAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(PReluAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.PReluAttrs", PReluAttrs, BaseAttrsNode);
 };
 
 /*! \brief Attributes used in batch_norm operator */
@@ -570,14 +545,12 @@ struct BatchNormAttrs : public AttrsNodeReflAdapter<BatchNormAttrs> {
         .def_ro("training", &BatchNormAttrs::training,
                 "Whether we are training (i.e., not in eval mode).");
   }
-
-  static constexpr const char* _type_key = "relax.attrs.BatchNormAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(BatchNormAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.BatchNormAttrs", BatchNormAttrs, BaseAttrsNode);
 };  // struct BatchNormAttrs
 
 /*! \brief Attributes used in layer_norm operator */
 struct LayerNormAttrs : public AttrsNodeReflAdapter<LayerNormAttrs> {
-  Array<Integer> axes;
+  ffi::Array<Integer> axes;
   double epsilon;
   bool center;
   bool scale;
@@ -594,16 +567,14 @@ struct LayerNormAttrs : public AttrsNodeReflAdapter<LayerNormAttrs> {
         .def_ro("scale", &LayerNormAttrs::scale,
                 "Indicating if the gamma scale will be multiplied.");
   }
-
-  static constexpr const char* _type_key = "relax.attrs.LayerNormAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(LayerNormAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.LayerNormAttrs", LayerNormAttrs, BaseAttrsNode);
 };  // struct LayerNormAttrs
 
 /*! \brief Attributes used in group_norm operator */
 struct GroupNormAttrs : public AttrsNodeReflAdapter<GroupNormAttrs> {
   int num_groups;
   int channel_axis;
-  Array<Integer> axes;
+  ffi::Array<Integer> axes;
   double epsilon;
   bool center;
   bool scale;
@@ -625,15 +596,13 @@ struct GroupNormAttrs : public AttrsNodeReflAdapter<GroupNormAttrs> {
         .def_ro("scale", &GroupNormAttrs::scale,
                 "Indicating if the gamma scale will be multiplied.");
   }
-
-  static constexpr const char* _type_key = "relax.attrs.GroupNormAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(GroupNormAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.GroupNormAttrs", GroupNormAttrs, BaseAttrsNode);
 };  // struct GroupNormAttrs
 
 /*! \brief Attributes used in instance_norm operator */
 struct InstanceNormAttrs : public AttrsNodeReflAdapter<InstanceNormAttrs> {
   int channel_axis;
-  Array<Integer> axes;
+  ffi::Array<Integer> axes;
   double epsilon;
   bool center;
   bool scale;
@@ -652,14 +621,13 @@ struct InstanceNormAttrs : public AttrsNodeReflAdapter<InstanceNormAttrs> {
         .def_ro("scale", &InstanceNormAttrs::scale,
                 "Indicating if the gamma scale will be multiplied.");
   }
-
-  static constexpr const char* _type_key = "relax.attrs.InstanceNormAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(InstanceNormAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.InstanceNormAttrs", InstanceNormAttrs,
+                                    BaseAttrsNode);
 };  // struct InstanceNormAttrs
 
 /*! \brief Attributes used in rms_norm operator */
 struct RMSNormAttrs : public AttrsNodeReflAdapter<RMSNormAttrs> {
-  Array<Integer> axes;
+  ffi::Array<Integer> axes;
   double epsilon;
 
   static void RegisterReflection() {
@@ -670,14 +638,12 @@ struct RMSNormAttrs : public AttrsNodeReflAdapter<RMSNormAttrs> {
         .def_ro("epsilon", &RMSNormAttrs::epsilon,
                 "Small float added to variance to avoid dividing by zero");
   }
-
-  static constexpr const char* _type_key = "relax.attrs.RMSNormAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(RMSNormAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.RMSNormAttrs", RMSNormAttrs, BaseAttrsNode);
 };  // struct RMSNormAttrs
 
 /*! \brief Attributes used in nll_loss operator */
 struct NLLLossAttrs : public AttrsNodeReflAdapter<NLLLossAttrs> {
-  String reduction;
+  ffi::String reduction;
   int ignore_index;
 
   static void RegisterReflection() {
@@ -689,9 +655,7 @@ struct NLLLossAttrs : public AttrsNodeReflAdapter<NLLLossAttrs> {
                 refl::DefaultValue("mean"))
         .def_ro("ignore_index", &NLLLossAttrs::ignore_index, "The target value to ignore.");
   }
-
-  static constexpr const char* _type_key = "relax.attrs.NLLLossAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(NLLLossAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.NLLLossAttrs", NLLLossAttrs, BaseAttrsNode);
 };  // struct NLLLossAttrs
 
 /*! \brief Attributes used in dropout operator */
@@ -704,16 +668,14 @@ struct DropoutAttrs : public AttrsNodeReflAdapter<DropoutAttrs> {
         "rate", &DropoutAttrs::rate,
         "Fraction of the input that gets dropped out during training time");
   }
-
-  static constexpr const char* _type_key = "relax.attrs.DropoutAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(DropoutAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.DropoutAttrs", DropoutAttrs, BaseAttrsNode);
 };  // struct DropoutAttrs
 
 /*! \brief Attributes used in Attention operator */
 struct AttentionAttrs : public AttrsNodeReflAdapter<AttentionAttrs> {
-  Optional<FloatImm> scale;
-  Optional<String> causal_mask;
-  Optional<IntImm> window_size;
+  ffi::Optional<FloatImm> scale;
+  ffi::Optional<ffi::String> causal_mask;
+  ffi::Optional<IntImm> window_size;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -726,16 +688,14 @@ struct AttentionAttrs : public AttrsNodeReflAdapter<AttentionAttrs> {
         .def_ro("window_size", &AttentionAttrs::window_size,
                 "The size of the window for sliding-window attention.");
   }
-
-  static constexpr const char* _type_key = "relax.attrs.AttentionAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(AttentionAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.AttentionAttrs", AttentionAttrs, BaseAttrsNode);
 };  // struct AttentionAttrs
 
 /*! \brief Attributes used for the padding operator */
 struct PadAttrs : public AttrsNodeReflAdapter<PadAttrs> {
-  Array<Integer> pad_width;
+  ffi::Array<Integer> pad_width;
   double pad_value = 0.0;
-  tvm::String pad_mode;
+  tvm::ffi::String pad_mode;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -751,9 +711,7 @@ struct PadAttrs : public AttrsNodeReflAdapter<PadAttrs> {
                 "\"reflect\" pads by reflecting values with respect to the edges.",
                 refl::DefaultValue("constant"));
   }
-
-  static constexpr const char* _type_key = "relax.attrs.PadAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(PadAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.PadAttrs", PadAttrs, BaseAttrsNode);
 };
 
 /*! \brief Attributes used for the pixel shuffle operator */
@@ -766,9 +724,8 @@ struct PixelShuffleAttrs : public AttrsNodeReflAdapter<PixelShuffleAttrs> {
                                                 &PixelShuffleAttrs::upscale_factor,
                                                 "Scale factor for spatial upsampling.");
   }
-
-  static constexpr const char* _type_key = "relax.attrs.PixelShuffleAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(PixelShuffleAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.PixelShuffleAttrs", PixelShuffleAttrs,
+                                    BaseAttrsNode);
 };
 
 }  // namespace relax
