@@ -43,9 +43,7 @@ struct QuantizeAttrs : public AttrsNodeReflAdapter<QuantizeAttrs> {
                 "Default value is -1, which corresponds to the last axis.",
                 refl::DefaultValue(-1));
   }
-
-  static constexpr const char* _type_key = "relax.attrs.QuantizeAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(QuantizeAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.QuantizeAttrs", QuantizeAttrs, BaseAttrsNode);
 };  // QuantizeAttrs
 
 }  // namespace relax
