@@ -47,9 +47,7 @@ struct SortAttrs : public AttrsNodeReflAdapter<SortAttrs> {
                 "If it is not specified, it defaults to the ascending order.",
                 refl::DefaultValue(false));
   }
-
-  static constexpr const char* _type_key = "relax.attrs.SortAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(SortAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.SortAttrs", SortAttrs, BaseAttrsNode);
 };  // struct SortAttrs
 
 /*! \brief Attributes used in argsort operator */
@@ -72,9 +70,7 @@ struct ArgsortAttrs : public AttrsNodeReflAdapter<ArgsortAttrs> {
         .def_ro("dtype", &ArgsortAttrs::dtype, "DType of the output indices.",
                 refl::DefaultValue(NullValue<DataType>()));
   }
-
-  static constexpr const char* _type_key = "relax.attrs.ArgsortAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(ArgsortAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.ArgsortAttrs", ArgsortAttrs, BaseAttrsNode);
 };  // struct ArgsortAttrs
 
 /*! \brief Attributes used in topk operator */
@@ -82,7 +78,7 @@ struct TopKAttrs : public AttrsNodeReflAdapter<TopKAttrs> {
   int k;
   int axis;
   bool largest;
-  String ret_type;
+  ffi::String ret_type;
   DataType dtype;
 
   static void RegisterReflection() {
@@ -104,9 +100,7 @@ struct TopKAttrs : public AttrsNodeReflAdapter<TopKAttrs> {
         .def_ro("dtype", &TopKAttrs::dtype, "Data type of the output indices.",
                 refl::DefaultValue(NullValue<DataType>()));
   }
-
-  static constexpr const char* _type_key = "relax.attrs.TopKAttrs";
-  TVM_FFI_DECLARE_FINAL_OBJECT_INFO(TopKAttrs, BaseAttrsNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.attrs.TopKAttrs", TopKAttrs, BaseAttrsNode);
 };  // struct TopKAttrs
 
 }  // namespace relax
