@@ -40,10 +40,10 @@ class DeviceMeshNode : public GlobalInfoNode {
   ffi::Shape shape;
 
   /*! \brief device ids in the mesh*/
-  Array<Integer> device_ids;
+  ffi::Array<Integer> device_ids;
 
   /*! \brief Optionally use range to represent device_ids*/
-  Optional<Range> device_range;
+  ffi::Optional<Range> device_range;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -52,29 +52,7 @@ class DeviceMeshNode : public GlobalInfoNode {
         .def_ro("device_ids", &DeviceMeshNode::device_ids)
         .def_ro("device_range", &DeviceMeshNode::device_range);
   }
-
-  static constexpr const char* _type_key = "relax.distributed.DeviceMesh";
-
-  bool SEqualReduce(const DeviceMeshNode* other, SEqualReducer equal) const {
-    if (shape.size() != other->shape.size()) {
-      return false;
-    }
-    for (int i = 0; i < static_cast<int>(shape.size()); i++) {
-      if (!equal(shape[i], other->shape[i])) {
-        return false;
-      }
-    }
-    return equal(device_ids, other->device_ids);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(device_ids);
-    for (int i = 0; i < static_cast<int>(shape.size()); i++) {
-      hash_reduce(shape[i]);
-    }
-  }
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(DeviceMeshNode, GlobalInfoNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.distributed.DeviceMesh", DeviceMeshNode, GlobalInfoNode);
 };
 
 /*!
@@ -83,9 +61,9 @@ class DeviceMeshNode : public GlobalInfoNode {
  */
 class DeviceMesh : public GlobalInfo {
  public:
-  TVM_DLL DeviceMesh(ffi::Shape shape, Array<Integer> device_ids);
+  TVM_DLL DeviceMesh(ffi::Shape shape, ffi::Array<Integer> device_ids);
   TVM_DLL DeviceMesh(ffi::Shape shape, Range device_range);
-  TVM_DEFINE_OBJECT_REF_METHODS(DeviceMesh, GlobalInfo, DeviceMeshNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(DeviceMesh, GlobalInfo, DeviceMeshNode);
 };
 
 }  // namespace distributed
