@@ -51,19 +51,8 @@ class PlacementSpecNode : public Object {
         .def_ro("kind", &PlacementSpecNode::kind);
   }
 
-  bool SEqualReduce(const PlacementSpecNode* other, SEqualReducer equal) const {
-    return equal(axis, other->axis) && equal(kind, other->kind);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(axis);
-    hash_reduce(static_cast<int>(kind));
-  }
-
-  static constexpr const char* _type_key = "relax.distributed.PlacementSpec";
-  static constexpr const bool _type_has_method_sequal_reduce = true;
-  static constexpr const bool _type_has_method_shash_reduce = true;
-  TVM_DECLARE_BASE_OBJECT_INFO(PlacementSpecNode, Object);
+  static constexpr TVMFFISEqHashKind _type_s_eq_hash_kind = kTVMFFISEqHashKindConstTreeNode;
+  TVM_FFI_DECLARE_OBJECT_INFO("relax.distributed.PlacementSpec", PlacementSpecNode, Object);
 };
 
 /*!
@@ -76,7 +65,7 @@ class PlacementSpec : public ObjectRef {
 
   TVM_DLL static PlacementSpec Replica();
 
-  TVM_DEFINE_OBJECT_REF_METHODS(PlacementSpec, ObjectRef, PlacementSpecNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(PlacementSpec, ObjectRef, PlacementSpecNode);
 };
 
 class ShardingNode : public PlacementSpecNode {
@@ -89,38 +78,24 @@ class ShardingNode : public PlacementSpecNode {
     refl::ObjectDef<ShardingNode>().def_ro("sharding_dim", &ShardingNode::sharding_dim);
   }
 
-  bool SEqualReduce(const ShardingNode* other, SEqualReducer equal) const {
-    return equal(sharding_dim, other->sharding_dim);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const { hash_reduce(sharding_dim); }
-  static constexpr const char* _type_key = "relax.distributed.Sharding";
-  TVM_DECLARE_FINAL_OBJECT_INFO(ShardingNode, PlacementSpecNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.distributed.Sharding", ShardingNode, PlacementSpecNode);
 };
 
 /*! \brief Describes how data is distributed in each dimension of the device mesh*/
 class PlacementNode : public Object {
  public:
   /*! \brief specs for each dim of device mesh.*/
-  Array<PlacementSpec> dim_specs;
+  ffi::Array<PlacementSpec> dim_specs;
 
-  String ToString() const;
+  ffi::String ToString() const;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<PlacementNode>().def_ro("dim_specs", &PlacementNode::dim_specs);
   }
 
-  bool SEqualReduce(const PlacementNode* other, SEqualReducer equal) const {
-    return equal(dim_specs, other->dim_specs);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const { hash_reduce(dim_specs); }
-
-  static constexpr const bool _type_has_method_sequal_reduce = true;
-  static constexpr const bool _type_has_method_shash_reduce = true;
-  static constexpr const char* _type_key = "relax.distributed.Placement";
-  TVM_DECLARE_FINAL_OBJECT_INFO(PlacementNode, Object);
+  static constexpr TVMFFISEqHashKind _type_s_eq_hash_kind = kTVMFFISEqHashKindConstTreeNode;
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.distributed.Placement", PlacementNode, Object);
 };
 
 /*!
@@ -129,10 +104,10 @@ class PlacementNode : public Object {
  */
 class Placement : public ObjectRef {
  public:
-  TVM_DLL explicit Placement(Array<PlacementSpec> dim_specs);
+  TVM_DLL explicit Placement(ffi::Array<PlacementSpec> dim_specs);
   /*! \brief replica dim is printed as "R" and sharding dim is printed as "S[i]".]*/
-  static Placement FromText(String text_repr);
-  TVM_DEFINE_OBJECT_REF_METHODS(Placement, ObjectRef, PlacementNode);
+  static Placement FromText(ffi::String text_repr);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(Placement, ObjectRef, PlacementNode);
 };
 
 /*!
@@ -160,20 +135,8 @@ class DTensorStructInfoNode : public StructInfoNode {
         .def_ro("placement", &DTensorStructInfoNode::placement)
         .def_ro("tensor_sinfo", &DTensorStructInfoNode::tensor_sinfo);
   }
-
-  bool SEqualReduce(const DTensorStructInfoNode* other, SEqualReducer equal) const {
-    return equal(tensor_sinfo, other->tensor_sinfo) && equal(device_mesh, other->device_mesh) &&
-           equal(placement, other->placement);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(tensor_sinfo);
-    hash_reduce(device_mesh);
-    hash_reduce(placement);
-  }
-
-  static constexpr const char* _type_key = "relax.DTensorStructInfo";
-  TVM_DECLARE_FINAL_OBJECT_INFO(DTensorStructInfoNode, StructInfoNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("relax.DTensorStructInfo", DTensorStructInfoNode,
+                                    StructInfoNode);
 };
 
 /*!
@@ -192,7 +155,7 @@ class DTensorStructInfo : public StructInfo {
   TVM_DLL DTensorStructInfo(TensorStructInfo tensor_sinfo, DeviceMesh device_mesh,
                             Placement placement, Span span = Span());
 
-  TVM_DEFINE_OBJECT_REF_METHODS(DTensorStructInfo, StructInfo, DTensorStructInfoNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(DTensorStructInfo, StructInfo, DTensorStructInfoNode);
 };
 
 }  // namespace distributed
