@@ -38,7 +38,8 @@ namespace profiling {
  * collected on that device. You can find the names of available metrics by
  * running `papi_native_avail`.
  */
-TVM_DLL MetricCollector CreatePAPIMetricCollector(Map<DeviceWrapper, Array<String>> metrics);
+TVM_DLL MetricCollector
+CreatePAPIMetricCollector(ffi::Map<DeviceWrapper, ffi::Array<ffi::String>> metrics);
 }  // namespace profiling
 }  // namespace runtime
 }  // namespace tvm
