@@ -28,7 +28,7 @@
 #include <dmlc/io.h>
 #include <dmlc/serializer.h>
 #include <tvm/runtime/base.h>
-#include <tvm/runtime/ndarray.h>
+#include <tvm/runtime/tensor.h>
 
 namespace dmlc {
 namespace serializer {
