@@ -19,7 +19,8 @@
 #ifndef TVM_SCRIPT_PRINTER_DOC_H_
 #define TVM_SCRIPT_PRINTER_DOC_H_
 
-#include <tvm/ffi/reflection/reflection.h>
+#include <tvm/ffi/reflection/access_path.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/ir/expr.h>
 #include <tvm/node/node.h>
 #include <tvm/runtime/data_type.h>
@@ -31,6 +32,8 @@ namespace tvm {
 namespace script {
 namespace printer {
 
+using AccessPath = ffi::reflection::AccessPath;
+
 // Forward declaration
 class Doc;
 
@@ -39,7 +42,7 @@ class Doc;
  * \param doc Doc to be converted
  * \param cfg The configuration of the printer
  */
-String DocToPythonScript(Doc doc, const PrinterConfig& cfg);
+ffi::String DocToPythonScript(Doc doc, const PrinterConfig& cfg);
 
 /*!
  * \brief The base class of all Doc.
@@ -61,16 +64,16 @@ class DocNode : public Object {
    * this Doc is generated, in order to position the diagnostic
    * message.
    */
-  mutable Array<ObjectPath> source_paths;
+  mutable ffi::Array<ffi::reflection::AccessPath> source_paths;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
-    refl::ObjectDef<DocNode>().def_ro("source_paths", &DocNode::source_paths);
+    refl::ObjectDef<DocNode>().def_rw("source_paths", &DocNode::source_paths);
   }
 
-  static constexpr const char* _type_key = "script.printer.Doc";
+  static constexpr bool _type_mutable = true;
 
-  TVM_DECLARE_BASE_OBJECT_INFO(DocNode, Object);
+  TVM_FFI_DECLARE_OBJECT_INFO("script.printer.Doc", DocNode, Object);
 
  public:
   virtual ~DocNode() = default;
@@ -84,9 +87,10 @@ class DocNode : public Object {
 class Doc : public ObjectRef {
  protected:
   Doc() = default;
+  explicit Doc(ObjectPtr<DocNode> data) : ObjectRef(data) {}
 
  public:
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(Doc, ObjectRef, DocNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(Doc, ObjectRef, DocNode);
 };
 
 class ExprDoc;
@@ -102,19 +106,19 @@ class ExprDocNode : public DocNode {
    * \brief Create a doc representing attribute access on the current ExprDoc
    * \param attr The attribute to access.
    */
-  ExprDoc Attr(String attr) const;
+  ExprDoc Attr(ffi::String attr) const;
 
   /*!
    * \brief Create a doc representing index access on the current ExprDoc
    * \param indices The indices to access.
    */
-  ExprDoc operator[](Array<Doc> indices) const;
+  ExprDoc operator[](ffi::Array<Doc> indices) const;
 
   /*!
    * \brief Create a doc representing calling the current ExprDoc
    * \param args The positional arguments of the function call.
    */
-  ExprDoc Call(Array<ExprDoc, void> args) const;
+  ExprDoc Call(ffi::Array<ExprDoc, void> args) const;
 
   /*!
    * \brief Create a doc representing attribute access on the current ExprDoc
@@ -122,18 +126,15 @@ class ExprDocNode : public DocNode {
    * \param kwargs_keys Keys of keywords arguments of the function call.
    * \param kwargs_values Values of keywords arguments of the function call.
    */
-  ExprDoc Call(Array<ExprDoc, void> args,  //
-               Array<String> kwargs_keys,  //
-               Array<ExprDoc, void> kwargs_values) const;
+  ExprDoc Call(ffi::Array<ExprDoc, void> args,       //
+               ffi::Array<ffi::String> kwargs_keys,  //
+               ffi::Array<ExprDoc, void> kwargs_values) const;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<ExprDocNode>();
   }
-
-  static constexpr const char* _type_key = "script.printer.ExprDoc";
-
-  TVM_DECLARE_BASE_OBJECT_INFO(ExprDocNode, DocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO("script.printer.ExprDoc", ExprDocNode, DocNode);
 };
 
 /*!
@@ -150,9 +151,11 @@ class ExprDoc : public Doc {
    * \brief Create a doc representing index access on the current ExprDoc
    * \param indices The indices to access.
    */
-  ExprDoc operator[](Array<Doc> indices) const;
+  ExprDoc operator[](ffi::Array<Doc> indices) const;
 
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(ExprDoc, Doc, ExprDocNode);
+  explicit ExprDoc(ObjectPtr<ExprDocNode> data) : Doc(data) { TVM_FFI_ICHECK(data != nullptr); }
+
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(ExprDoc, Doc, ExprDocNode);
 };
 
 /*!
@@ -170,16 +173,13 @@ class StmtDocNode : public DocNode {
    * line as the statement, or the line above, or inside the statement
    * if it spans over multiple lines.
    * */
-  mutable Optional<String> comment{std::nullopt};
+  mutable ffi::Optional<ffi::String> comment{std::nullopt};
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
-    refl::ObjectDef<StmtDocNode>().def_ro("comment", &StmtDocNode::comment);
+    refl::ObjectDef<StmtDocNode>().def_rw("comment", &StmtDocNode::comment);
   }
-
-  static constexpr const char* _type_key = "script.printer.StmtDoc";
-
-  TVM_DECLARE_BASE_OBJECT_INFO(StmtDocNode, DocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO("script.printer.StmtDoc", StmtDocNode, DocNode);
 };
 
 /*!
@@ -192,7 +192,7 @@ class StmtDoc : public Doc {
   StmtDoc() = default;
 
  public:
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(StmtDoc, Doc, StmtDocNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(StmtDoc, Doc, StmtDocNode);
 };
 
 /*!
@@ -204,16 +204,13 @@ class StmtDoc : public Doc {
 class StmtBlockDocNode : public DocNode {
  public:
   /*! \brief The list of statements. */
-  Array<StmtDoc> stmts;
+  ffi::Array<StmtDoc> stmts;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<StmtBlockDocNode>().def_ro("stmts", &StmtBlockDocNode::stmts);
   }
-
-  static constexpr const char* _type_key = "script.printer.StmtBlockDoc";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(StmtBlockDocNode, DocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.printer.StmtBlockDoc", StmtBlockDocNode, DocNode);
 };
 
 /*!
@@ -226,8 +223,8 @@ class StmtBlockDoc : public Doc {
    * \brief Constructor of StmtBlockDoc.
    * \param stmts The list of statements.
    */
-  explicit StmtBlockDoc(Array<StmtDoc> stmts);
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(StmtBlockDoc, Doc, StmtBlockDocNode);
+  explicit StmtBlockDoc(ffi::Array<StmtDoc> stmts);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(StmtBlockDoc, Doc, StmtBlockDocNode);
 };
 
 /*!
@@ -246,16 +243,13 @@ class LiteralDocNode : public ExprDocNode {
    * - String
    * - null
    */
-  ObjectRef value;
+  ffi::Any value;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<LiteralDocNode>().def_ro("value", &LiteralDocNode::value);
   }
-
-  static constexpr const char* _type_key = "script.printer.LiteralDoc";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(LiteralDocNode, ExprDocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.printer.LiteralDoc", LiteralDocNode, ExprDocNode);
 };
 
 /*!
@@ -265,22 +259,22 @@ class LiteralDocNode : public ExprDocNode {
  */
 class LiteralDoc : public ExprDoc {
  protected:
-  explicit LiteralDoc(ObjectRef value, const Optional<ObjectPath>& object_path);
+  explicit LiteralDoc(ffi::Any value, const ffi::Optional<AccessPath>& object_path);
 
  public:
   /*!
    * \brief Create a LiteralDoc to represent None/null/empty value.
    * \param p The object path
    */
-  static LiteralDoc None(const Optional<ObjectPath>& p) {
-    return LiteralDoc(ObjectRef(nullptr), p);
+  static LiteralDoc None(const ffi::Optional<AccessPath>& p) {
+    return LiteralDoc(ffi::Any(nullptr), p);
   }
   /*!
    * \brief Create a LiteralDoc to represent integer.
    * \param v The integer value.
    * \param p The object path
    */
-  static LiteralDoc Int(int64_t v, const Optional<ObjectPath>& p) {
+  static LiteralDoc Int(int64_t v, const ffi::Optional<AccessPath>& p) {
     return LiteralDoc(IntImm(DataType::Int(64), v), p);
   }
   /*!
@@ -288,7 +282,7 @@ class LiteralDoc : public ExprDoc {
    * \param v The boolean value.
    * \param p The object path
    */
-  static LiteralDoc Boolean(bool v, const Optional<ObjectPath>& p) {
+  static LiteralDoc Boolean(bool v, const ffi::Optional<AccessPath>& p) {
     return LiteralDoc(IntImm(DataType::Bool(), v), p);
   }
   /*!
@@ -296,7 +290,7 @@ class LiteralDoc : public ExprDoc {
    * \param v The float value.
    * \param p The object path
    */
-  static LiteralDoc Float(double v, const Optional<ObjectPath>& p) {
+  static LiteralDoc Float(double v, const ffi::Optional<AccessPath>& p) {
     return LiteralDoc(FloatImm(DataType::Float(64), v), p);
   }
   /*!
@@ -304,13 +298,15 @@ class LiteralDoc : public ExprDoc {
    * \param v The string value.
    * \param p The object path
    */
-  static LiteralDoc Str(const String& v, const Optional<ObjectPath>& p) { return LiteralDoc(v, p); }
+  static LiteralDoc Str(const ffi::String& v, const ffi::Optional<AccessPath>& p) {
+    return LiteralDoc(v, p);
+  }
   /*!
    * \brief Create a LiteralDoc to represent string.
    * \param v The string value.
    * \param p The object path
    */
-  static LiteralDoc DataType(const runtime::DataType& v, const Optional<ObjectPath>& p) {
+  static LiteralDoc DataType(const runtime::DataType& v, const ffi::Optional<AccessPath>& p) {
     std::string dtype = v.is_void() ? "void" : runtime::DLDataTypeToString(v);
     return LiteralDoc::Str(dtype, p);
   }
@@ -319,13 +315,13 @@ class LiteralDoc : public ExprDoc {
    * \param v The device.
    * \param p The object path
    */
-  static LiteralDoc Device(const DLDevice& v, const Optional<ObjectPath>& p) {
+  static LiteralDoc Device(const DLDevice& v, const ffi::Optional<AccessPath>& p) {
     std::ostringstream os;
     runtime::operator<<(os, v);
     return LiteralDoc::Str(os.str(), p);
   }
 
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(LiteralDoc, ExprDoc, LiteralDocNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(LiteralDoc, ExprDoc, LiteralDocNode);
 };
 
 /*!
@@ -336,16 +332,13 @@ class LiteralDoc : public ExprDoc {
 class IdDocNode : public ExprDocNode {
  public:
   /*! \brief The name of the identifier */
-  String name;
+  ffi::String name;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<IdDocNode>().def_ro("name", &IdDocNode::name);
   }
-
-  static constexpr const char* _type_key = "script.printer.IdDoc";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(IdDocNode, ExprDocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.printer.IdDoc", IdDocNode, ExprDocNode);
 };
 
 /*!
@@ -359,9 +352,9 @@ class IdDoc : public ExprDoc {
    * \brief Constructor of IdDoc.
    * \param name The name of identifier.
    */
-  explicit IdDoc(String name);
+  explicit IdDoc(ffi::String name);
   explicit IdDoc(std::nullptr_t) : ExprDoc(nullptr) {}
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(IdDoc, ExprDoc, IdDocNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(IdDoc, ExprDoc, IdDocNode);
 };
 
 /*!
@@ -372,9 +365,9 @@ class IdDoc : public ExprDoc {
 class AttrAccessDocNode : public ExprDocNode {
  public:
   /*! \brief The target expression to be accessed */
-  ExprDoc value{nullptr};
+  ExprDoc value{ffi::UnsafeInit()};
   /*! \brief The attribute to be accessed */
-  String name;
+  ffi::String name;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -382,10 +375,7 @@ class AttrAccessDocNode : public ExprDocNode {
         .def_ro("value", &AttrAccessDocNode::value)
         .def_ro("name", &AttrAccessDocNode::name);
   }
-
-  static constexpr const char* _type_key = "script.printer.AttrAccessDoc";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(AttrAccessDocNode, ExprDocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.printer.AttrAccessDoc", AttrAccessDocNode, ExprDocNode);
 };
 
 /*!
@@ -400,8 +390,8 @@ class AttrAccessDoc : public ExprDoc {
    * \param value The target expression of attribute access.
    * \param name The name of attribute to access.
    */
-  explicit AttrAccessDoc(ExprDoc value, String name);
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(AttrAccessDoc, ExprDoc, AttrAccessDocNode);
+  explicit AttrAccessDoc(ExprDoc value, ffi::String name);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(AttrAccessDoc, ExprDoc, AttrAccessDocNode);
 };
 
 /*!
@@ -412,7 +402,7 @@ class AttrAccessDoc : public ExprDoc {
 class IndexDocNode : public ExprDocNode {
  public:
   /*! \brief The container value to be accessed */
-  ExprDoc value{nullptr};
+  ExprDoc value{ffi::UnsafeInit()};
   /*!
    * \brief The indices to access
    *
@@ -420,7 +410,7 @@ class IndexDocNode : public ExprDocNode {
    * - ExprDoc (single point access like a[1, 2])
    * - SliceDoc (slice access like a[1:5, 2])
    */
-  Array<Doc> indices;  // Each element is union of: Slice / ExprDoc
+  ffi::Array<Doc> indices;  // Each element is union of: Slice / ExprDoc
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -428,10 +418,7 @@ class IndexDocNode : public ExprDocNode {
         .def_ro("value", &IndexDocNode::value)
         .def_ro("indices", &IndexDocNode::indices);
   }
-
-  static constexpr const char* _type_key = "script.printer.IndexDoc";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(IndexDocNode, ExprDocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.printer.IndexDoc", IndexDocNode, ExprDocNode);
 };
 
 /*!
@@ -446,8 +433,8 @@ class IndexDoc : public ExprDoc {
    * \param value The target expression of index access.
    * \param indices The indices to access.
    */
-  explicit IndexDoc(ExprDoc value, Array<Doc> indices);
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(IndexDoc, ExprDoc, IndexDocNode);
+  explicit IndexDoc(ExprDoc value, ffi::Array<Doc> indices);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(IndexDoc, ExprDoc, IndexDocNode);
 };
 
 /*!
@@ -458,18 +445,18 @@ class IndexDoc : public ExprDoc {
 class CallDocNode : public ExprDocNode {
  public:
   /*! \brief The callee of this function call */
-  ExprDoc callee{nullptr};
+  ExprDoc callee{ffi::UnsafeInit()};
   /*! \brief The positional arguments */
-  Array<ExprDoc> args;
+  ffi::Array<ExprDoc> args;
   /*! \brief The keys of keyword arguments */
-  Array<String> kwargs_keys;
+  ffi::Array<ffi::String> kwargs_keys;
   /*!
    * \brief The values of keyword arguments.
    *
    * The i-th element is the value of the i-th key in `kwargs_keys`.
    * It must have the same length as `kwargs_keys`.
    */
-  Array<ExprDoc> kwargs_values;
+  ffi::Array<ExprDoc> kwargs_values;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -479,10 +466,7 @@ class CallDocNode : public ExprDocNode {
         .def_ro("kwargs_keys", &CallDocNode::kwargs_keys)
         .def_ro("kwargs_values", &CallDocNode::kwargs_values);
   }
-
-  static constexpr const char* _type_key = "script.printer.CallDoc";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(CallDocNode, ExprDocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.printer.CallDoc", CallDocNode, ExprDocNode);
 };
 
 /*!
@@ -499,9 +483,9 @@ class CallDoc : public ExprDoc {
    * \param kwargs_keys Keys of keyword arguments.
    * \param kwargs_values Values of keyword arguments, must have the same length as `kwargs_keys.
    */
-  CallDoc(ExprDoc callee, Array<ExprDoc> args, Array<String> kwargs_keys,
-          Array<ExprDoc> kwargs_values);
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(CallDoc, ExprDoc, CallDocNode);
+  CallDoc(ExprDoc callee, ffi::Array<ExprDoc> args, ffi::Array<ffi::String> kwargs_keys,
+          ffi::Array<ExprDoc> kwargs_values);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(CallDoc, ExprDoc, CallDocNode);
 };
 
 /*!
@@ -555,7 +539,7 @@ class OperationDocNode : public ExprDocNode {
   /*! \brief The kind of operation (operator) */
   Kind kind;
   /*! \brief Operands of this expression */
-  Array<ExprDoc> operands;
+  ffi::Array<ExprDoc> operands;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -563,10 +547,7 @@ class OperationDocNode : public ExprDocNode {
         .def_ro("kind", &OperationDocNode::kind)
         .def_ro("operands", &OperationDocNode::operands);
   }
-
-  static constexpr const char* _type_key = "script.printer.OperationDoc";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(OperationDocNode, ExprDocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.printer.OperationDoc", OperationDocNode, ExprDocNode);
 };
 
 /*!
@@ -581,8 +562,8 @@ class OperationDoc : public ExprDoc {
    * \param kind The kind of operation.
    * \param operands Operands of this expression.
    */
-  explicit OperationDoc(OperationDocNode::Kind kind, Array<ExprDoc> operands);
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(OperationDoc, ExprDoc, OperationDocNode);
+  explicit OperationDoc(OperationDocNode::Kind kind, ffi::Array<ExprDoc> operands);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(OperationDoc, ExprDoc, OperationDocNode);
 };
 
 /*!
@@ -596,9 +577,9 @@ class OperationDoc : public ExprDoc {
 class LambdaDocNode : public ExprDocNode {
  public:
   /*! \brief The arguments of this anonymous function */
-  Array<IdDoc> args;
+  ffi::Array<IdDoc> args;
   /*! \brief The body of this anonymous function */
-  ExprDoc body{nullptr};
+  ExprDoc body{ffi::UnsafeInit()};
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -606,10 +587,7 @@ class LambdaDocNode : public ExprDocNode {
         .def_ro("args", &LambdaDocNode::args)
         .def_ro("body", &LambdaDocNode::body);
   }
-
-  static constexpr const char* _type_key = "script.printer.LambdaDoc";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(LambdaDocNode, ExprDocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.printer.LambdaDoc", LambdaDocNode, ExprDocNode);
 };
 
 /*!
@@ -624,8 +602,8 @@ class LambdaDoc : public ExprDoc {
    * \param args Arguments of this function.
    * \param body Body expression of this function.
    */
-  explicit LambdaDoc(Array<IdDoc> args, ExprDoc body);
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(LambdaDoc, ExprDoc, LambdaDocNode);
+  explicit LambdaDoc(ffi::Array<IdDoc> args, ExprDoc body);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(LambdaDoc, ExprDoc, LambdaDocNode);
 };
 
 /*!
@@ -636,16 +614,13 @@ class LambdaDoc : public ExprDoc {
 class TupleDocNode : public ExprDocNode {
  public:
   /*! \brief Elements of tuple */
-  Array<ExprDoc> elements;
+  ffi::Array<ExprDoc> elements;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<TupleDocNode>().def_ro("elements", &TupleDocNode::elements);
   }
-
-  static constexpr const char* _type_key = "script.printer.TupleDoc";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(TupleDocNode, ExprDocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.printer.TupleDoc", TupleDocNode, ExprDocNode);
 };
 
 /*!
@@ -658,13 +633,13 @@ class TupleDoc : public ExprDoc {
   /*!
    * \brief Create an empty TupleDoc
    */
-  TupleDoc() : TupleDoc(ffi::make_object<TupleDocNode>()) {}
+  TupleDoc() : ExprDoc(ffi::make_object<TupleDocNode>()) {}
   /*!
    * \brief Constructor of TupleDoc
    * \param elements Elements of tuple.
    */
-  explicit TupleDoc(Array<ExprDoc> elements);
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(TupleDoc, ExprDoc, TupleDocNode);
+  explicit TupleDoc(ffi::Array<ExprDoc> elements);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(TupleDoc, ExprDoc, TupleDocNode);
 };
 
 /*!
@@ -675,16 +650,13 @@ class TupleDoc : public ExprDoc {
 class ListDocNode : public ExprDocNode {
  public:
   /*! \brief Elements of list */
-  Array<ExprDoc> elements;
+  ffi::Array<ExprDoc> elements;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<ListDocNode>().def_ro("elements", &ListDocNode::elements);
   }
-
-  static constexpr const char* _type_key = "script.printer.ListDoc";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(ListDocNode, ExprDocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.printer.ListDoc", ListDocNode, ExprDocNode);
 };
 
 /*!
@@ -697,13 +669,13 @@ class ListDoc : public ExprDoc {
   /*!
    * \brief Create an empty ListDoc
    */
-  ListDoc() : ListDoc(ffi::make_object<ListDocNode>()) {}
+  ListDoc() : ExprDoc(ffi::make_object<ListDocNode>()) {}
   /*!
    * \brief Constructor of ListDoc
    * \param elements Elements of list.
    */
-  explicit ListDoc(Array<ExprDoc> elements);
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(ListDoc, ExprDoc, ListDocNode);
+  explicit ListDoc(ffi::Array<ExprDoc> elements);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(ListDoc, ExprDoc, ListDocNode);
 };
 
 /*!
@@ -714,14 +686,14 @@ class ListDoc : public ExprDoc {
 class DictDocNode : public ExprDocNode {
  public:
   /*! \brief keys of dictionary */
-  Array<ExprDoc> keys;
+  ffi::Array<ExprDoc> keys;
   /*!
    * \brief Values of dictionary
    *
    * The i-th element is the value of the i-th element of `keys`.
    * It must have the same length as `keys`.
    */
-  Array<ExprDoc> values;
+  ffi::Array<ExprDoc> values;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -729,10 +701,7 @@ class DictDocNode : public ExprDocNode {
         .def_ro("keys", &DictDocNode::keys)
         .def_ro("values", &DictDocNode::values);
   }
-
-  static constexpr const char* _type_key = "script.printer.DictDoc";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(DictDocNode, ExprDocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.printer.DictDoc", DictDocNode, ExprDocNode);
 };
 
 /*!
@@ -745,14 +714,14 @@ class DictDoc : public ExprDoc {
   /*!
    * \brief Create an empty dictionary
    */
-  DictDoc() : DictDoc(ffi::make_object<DictDocNode>()) {}
+  DictDoc() : ExprDoc(ffi::make_object<DictDocNode>()) {}
   /*!
    * \brief Constructor of DictDoc
    * \param keys Keys of dictionary.
    * \param values Values of dictionary, must have same length as `keys`.
    */
-  explicit DictDoc(Array<ExprDoc> keys, Array<ExprDoc> values);
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(DictDoc, ExprDoc, DictDocNode);
+  explicit DictDoc(ffi::Array<ExprDoc> keys, ffi::Array<ExprDoc> values);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(DictDoc, ExprDoc, DictDocNode);
 };
 
 /*!
@@ -765,11 +734,11 @@ class DictDoc : public ExprDoc {
 class SliceDocNode : public DocNode {
  public:
   /*! \brief The start of slice */
-  Optional<ExprDoc> start;
+  ffi::Optional<ExprDoc> start;
   /*! \brief The exclusive end of slice */
-  Optional<ExprDoc> stop;
+  ffi::Optional<ExprDoc> stop;
   /*! \brief The step of slice */
-  Optional<ExprDoc> step;
+  ffi::Optional<ExprDoc> step;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -778,10 +747,7 @@ class SliceDocNode : public DocNode {
         .def_ro("stop", &SliceDocNode::stop)
         .def_ro("step", &SliceDocNode::step);
   }
-
-  static constexpr const char* _type_key = "script.printer.SliceDoc";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(SliceDocNode, DocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.printer.SliceDoc", SliceDocNode, DocNode);
 };
 
 /*!
@@ -797,8 +763,9 @@ class SliceDoc : public Doc {
    * \param stop The exclusive end of slice.
    * \param step The step of slice.
    */
-  explicit SliceDoc(Optional<ExprDoc> start, Optional<ExprDoc> stop, Optional<ExprDoc> step);
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(SliceDoc, Doc, SliceDocNode);
+  explicit SliceDoc(ffi::Optional<ExprDoc> start, ffi::Optional<ExprDoc> stop,
+                    ffi::Optional<ExprDoc> step);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(SliceDoc, Doc, SliceDocNode);
 };
 
 /*!
@@ -809,15 +776,15 @@ class SliceDoc : public Doc {
 class AssignDocNode : public StmtDocNode {
  public:
   /*! \brief The left hand side of the assignment */
-  ExprDoc lhs{nullptr};
+  ExprDoc lhs{ffi::UnsafeInit()};
   /*!
    * \brief The right hand side of the assignment.
    *
    * If null, this doc represents declaration, e.g. `A: T.Buffer((1,2))`
    * */
-  Optional<ExprDoc> rhs;
+  ffi::Optional<ExprDoc> rhs;
   /*! \brief The type annotation of this assignment. */
-  Optional<ExprDoc> annotation;
+  ffi::Optional<ExprDoc> annotation;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -826,10 +793,7 @@ class AssignDocNode : public StmtDocNode {
         .def_ro("rhs", &AssignDocNode::rhs)
         .def_ro("annotation", &AssignDocNode::annotation);
   }
-
-  static constexpr const char* _type_key = "script.printer.AssignDoc";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(AssignDocNode, StmtDocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.printer.AssignDoc", AssignDocNode, StmtDocNode);
 };
 
 /*!
@@ -845,8 +809,8 @@ class AssignDoc : public StmtDoc {
    * \param rhs The right hand side of the assignment.
    * \param annotation The type annotation of this assignment.
    */
-  explicit AssignDoc(ExprDoc lhs, Optional<ExprDoc> rhs, Optional<ExprDoc> annotation);
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(AssignDoc, StmtDoc, AssignDocNode);
+  explicit AssignDoc(ExprDoc lhs, ffi::Optional<ExprDoc> rhs, ffi::Optional<ExprDoc> annotation);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(AssignDoc, StmtDoc, AssignDocNode);
 };
 
 /*!
@@ -857,11 +821,11 @@ class AssignDoc : public StmtDoc {
 class IfDocNode : public StmtDocNode {
  public:
   /*! \brief The predicate of the if-then-else statement. */
-  ExprDoc predicate{nullptr};
+  ExprDoc predicate{ffi::UnsafeInit()};
   /*! \brief The then branch of the if-then-else statement. */
-  Array<StmtDoc> then_branch;
+  ffi::Array<StmtDoc> then_branch;
   /*! \brief The else branch of the if-then-else statement. */
-  Array<StmtDoc> else_branch;
+  ffi::Array<StmtDoc> else_branch;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -870,10 +834,7 @@ class IfDocNode : public StmtDocNode {
         .def_ro("then_branch", &IfDocNode::then_branch)
         .def_ro("else_branch", &IfDocNode::else_branch);
   }
-
-  static constexpr const char* _type_key = "script.printer.IfDoc";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(IfDocNode, StmtDocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.printer.IfDoc", IfDocNode, StmtDocNode);
 };
 
 /*!
@@ -889,8 +850,9 @@ class IfDoc : public StmtDoc {
    * \param then_branch The then branch of the if-then-else statement.
    * \param else_branch The else branch of the if-then-else statement.
    */
-  explicit IfDoc(ExprDoc predicate, Array<StmtDoc> then_branch, Array<StmtDoc> else_branch);
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(IfDoc, StmtDoc, IfDocNode);
+  explicit IfDoc(ExprDoc predicate, ffi::Array<StmtDoc> then_branch,
+                 ffi::Array<StmtDoc> else_branch);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(IfDoc, StmtDoc, IfDocNode);
 };
 
 /*!
@@ -901,9 +863,9 @@ class IfDoc : public StmtDoc {
 class WhileDocNode : public StmtDocNode {
  public:
   /*! \brief The predicate of the while statement. */
-  ExprDoc predicate{nullptr};
+  ExprDoc predicate{ffi::UnsafeInit()};
   /*! \brief The body of the while statement. */
-  Array<StmtDoc> body;
+  ffi::Array<StmtDoc> body;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -911,10 +873,7 @@ class WhileDocNode : public StmtDocNode {
         .def_ro("predicate", &WhileDocNode::predicate)
         .def_ro("body", &WhileDocNode::body);
   }
-
-  static constexpr const char* _type_key = "script.printer.WhileDoc";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(WhileDocNode, StmtDocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.printer.WhileDoc", WhileDocNode, StmtDocNode);
 };
 
 /*!
@@ -929,8 +888,8 @@ class WhileDoc : public StmtDoc {
    * \param predicate The predicate of the while statement.
    * \param body The body of the while statement.
    */
-  explicit WhileDoc(ExprDoc predicate, Array<StmtDoc> body);
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(WhileDoc, StmtDoc, WhileDocNode);
+  explicit WhileDoc(ExprDoc predicate, ffi::Array<StmtDoc> body);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(WhileDoc, StmtDoc, WhileDocNode);
 };
 
 /*!
@@ -945,11 +904,11 @@ class WhileDoc : public StmtDoc {
 class ForDocNode : public StmtDocNode {
  public:
   /*! \brief The left hand side of the assignment of iterating variable. */
-  ExprDoc lhs{nullptr};
+  ExprDoc lhs{ffi::UnsafeInit()};
   /*! \brief The right hand side of the assignment of iterating variable. */
-  ExprDoc rhs{nullptr};
+  ExprDoc rhs{ffi::UnsafeInit()};
   /*! \brief The body of the for statement. */
-  Array<StmtDoc> body;
+  ffi::Array<StmtDoc> body;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -958,10 +917,7 @@ class ForDocNode : public StmtDocNode {
         .def_ro("rhs", &ForDocNode::rhs)
         .def_ro("body", &ForDocNode::body);
   }
-
-  static constexpr const char* _type_key = "script.printer.ForDoc";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(ForDocNode, StmtDocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.printer.ForDoc", ForDocNode, StmtDocNode);
 };
 
 /*!
@@ -977,8 +933,8 @@ class ForDoc : public StmtDoc {
    * \param rhs The right hand side of the assignment of iterating variable.
    * \param body The body of the for statement.
    */
-  explicit ForDoc(ExprDoc lhs, ExprDoc rhs, Array<StmtDoc> body);
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(ForDoc, StmtDoc, ForDocNode);
+  explicit ForDoc(ExprDoc lhs, ExprDoc rhs, ffi::Array<StmtDoc> body);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(ForDoc, StmtDoc, ForDocNode);
 };
 
 /*!
@@ -994,11 +950,11 @@ class ForDoc : public StmtDoc {
 class ScopeDocNode : public StmtDocNode {
  public:
   /*! \brief The name of the scoped variable. */
-  Optional<ExprDoc> lhs{std::nullopt};
+  ffi::Optional<ExprDoc> lhs{std::nullopt};
   /*! \brief The value of the scoped variable. */
-  ExprDoc rhs{nullptr};
+  ExprDoc rhs{ffi::UnsafeInit()};
   /*! \brief The body of the scope doc. */
-  Array<StmtDoc> body;
+  ffi::Array<StmtDoc> body;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -1007,10 +963,7 @@ class ScopeDocNode : public StmtDocNode {
         .def_ro("rhs", &ScopeDocNode::rhs)
         .def_ro("body", &ScopeDocNode::body);
   }
-
-  static constexpr const char* _type_key = "script.printer.ScopeDoc";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(ScopeDocNode, StmtDocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.printer.ScopeDoc", ScopeDocNode, StmtDocNode);
 };
 
 /*!
@@ -1026,16 +979,16 @@ class ScopeDoc : public StmtDoc {
    * \param rhs The value of the scoped variable.
    * \param body The body of the scope doc.
    */
-  explicit ScopeDoc(Optional<ExprDoc> lhs, ExprDoc rhs, Array<StmtDoc> body);
+  explicit ScopeDoc(ffi::Optional<ExprDoc> lhs, ExprDoc rhs, ffi::Array<StmtDoc> body);
 
   /*!
    * \brief Constructor of ScopeDoc.
    * \param rhs The value of the scoped variable.
    * \param body The body of the scope doc.
    */
-  explicit ScopeDoc(ExprDoc rhs, Array<StmtDoc> body);
+  explicit ScopeDoc(ExprDoc rhs, ffi::Array<StmtDoc> body);
 
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(ScopeDoc, StmtDoc, ScopeDocNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(ScopeDoc, StmtDoc, ScopeDocNode);
 };
 
 /*!
@@ -1046,16 +999,13 @@ class ScopeDoc : public StmtDoc {
 class ExprStmtDocNode : public StmtDocNode {
  public:
   /*! \brief The expression represented by this doc. */
-  ExprDoc expr{nullptr};
+  ExprDoc expr{ffi::UnsafeInit()};
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<ExprStmtDocNode>().def_ro("expr", &ExprStmtDocNode::expr);
   }
-
-  static constexpr const char* _type_key = "script.printer.ExprStmtDoc";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(ExprStmtDocNode, StmtDocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.printer.ExprStmtDoc", ExprStmtDocNode, StmtDocNode);
 };
 
 /*!
@@ -1070,7 +1020,7 @@ class ExprStmtDoc : public StmtDoc {
    * \param expr The expression represented by this doc.
    */
   explicit ExprStmtDoc(ExprDoc expr);
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(ExprStmtDoc, StmtDoc, ExprStmtDocNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(ExprStmtDoc, StmtDoc, ExprStmtDocNode);
 };
 
 /*!
@@ -1081,9 +1031,9 @@ class ExprStmtDoc : public StmtDoc {
 class AssertDocNode : public StmtDocNode {
  public:
   /*! \brief The expression to test. */
-  ExprDoc test{nullptr};
+  ExprDoc test{ffi::UnsafeInit()};
   /*! \brief The optional error message when assertion failed. */
-  Optional<ExprDoc> msg{std::nullopt};
+  ffi::Optional<ExprDoc> msg{std::nullopt};
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -1091,10 +1041,7 @@ class AssertDocNode : public StmtDocNode {
         .def_ro("test", &AssertDocNode::test)
         .def_ro("msg", &AssertDocNode::msg);
   }
-
-  static constexpr const char* _type_key = "script.printer.AssertDoc";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(AssertDocNode, StmtDocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.printer.AssertDoc", AssertDocNode, StmtDocNode);
 };
 
 /*!
@@ -1109,8 +1056,8 @@ class AssertDoc : public StmtDoc {
    * \param test The expression to test.
    * \param msg The optional error message when assertion failed.
    */
-  explicit AssertDoc(ExprDoc test, Optional<ExprDoc> msg = std::nullopt);
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(AssertDoc, StmtDoc, AssertDocNode);
+  explicit AssertDoc(ExprDoc test, ffi::Optional<ExprDoc> msg = std::nullopt);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(AssertDoc, StmtDoc, AssertDocNode);
 };
 
 /*!
@@ -1121,16 +1068,13 @@ class AssertDoc : public StmtDoc {
 class ReturnDocNode : public StmtDocNode {
  public:
   /*! \brief The value to return. */
-  ExprDoc value{nullptr};
+  ExprDoc value{ffi::UnsafeInit()};
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<ReturnDocNode>().def_ro("value", &ReturnDocNode::value);
   }
-
-  static constexpr const char* _type_key = "script.printer.ReturnDoc";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(ReturnDocNode, StmtDocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.printer.ReturnDoc", ReturnDocNode, StmtDocNode);
 };
 
 /*!
@@ -1145,7 +1089,7 @@ class ReturnDoc : public StmtDoc {
    * \param value The value to return.
    */
   explicit ReturnDoc(ExprDoc value);
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(ReturnDoc, StmtDoc, ReturnDocNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(ReturnDoc, StmtDoc, ReturnDocNode);
 };
 
 /*!
@@ -1156,7 +1100,7 @@ class ReturnDoc : public StmtDoc {
 class FunctionDocNode : public StmtDocNode {
  public:
   /*! \brief The name of function. */
-  IdDoc name{nullptr};
+  IdDoc name{ffi::UnsafeInit{}};
   /*!
    * \brief The arguments of function.
    *
@@ -1164,13 +1108,13 @@ class FunctionDocNode : public StmtDocNode {
    * `annotation` means argument type,
    * and `rhs` means default value.
    */
-  Array<AssignDoc> args;
+  ffi::Array<AssignDoc> args;
   /*! \brief Decorators of function. */
-  Array<ExprDoc> decorators;
+  ffi::Array<ExprDoc> decorators;
   /*! \brief The return type of function. */
-  Optional<ExprDoc> return_type{std::nullopt};
+  ffi::Optional<ExprDoc> return_type{std::nullopt};
   /*! \brief The body of function. */
-  Array<StmtDoc> body;
+  ffi::Array<StmtDoc> body;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -1181,10 +1125,7 @@ class FunctionDocNode : public StmtDocNode {
         .def_ro("return_type", &FunctionDocNode::return_type)
         .def_ro("body", &FunctionDocNode::body);
   }
-
-  static constexpr const char* _type_key = "script.printer.FunctionDoc";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(FunctionDocNode, StmtDocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.printer.FunctionDoc", FunctionDocNode, StmtDocNode);
 };
 
 /*!
@@ -1202,9 +1143,9 @@ class FunctionDoc : public StmtDoc {
    * \param return_type The return type of function.
    * \param body The body of function.
    */
-  explicit FunctionDoc(IdDoc name, Array<AssignDoc> args, Array<ExprDoc> decorators,
-                       Optional<ExprDoc> return_type, Array<StmtDoc> body);
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(FunctionDoc, StmtDoc, FunctionDocNode);
+  explicit FunctionDoc(IdDoc name, ffi::Array<AssignDoc> args, ffi::Array<ExprDoc> decorators,
+                       ffi::Optional<ExprDoc> return_type, ffi::Array<StmtDoc> body);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(FunctionDoc, StmtDoc, FunctionDocNode);
 };
 
 /*!
@@ -1215,11 +1156,11 @@ class FunctionDoc : public StmtDoc {
 class ClassDocNode : public StmtDocNode {
  public:
   /*! \brief The name of class. */
-  IdDoc name{nullptr};
+  IdDoc name{ffi::UnsafeInit{}};
   /*! \brief Decorators of class. */
-  Array<ExprDoc> decorators;
+  ffi::Array<ExprDoc> decorators;
   /*! \brief The body of class. */
-  Array<StmtDoc> body;
+  ffi::Array<StmtDoc> body;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -1228,10 +1169,7 @@ class ClassDocNode : public StmtDocNode {
         .def_ro("decorators", &ClassDocNode::decorators)
         .def_ro("body", &ClassDocNode::body);
   }
-
-  static constexpr const char* _type_key = "script.printer.ClassDoc";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(ClassDocNode, StmtDocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.printer.ClassDoc", ClassDocNode, StmtDocNode);
 };
 
 /*!
@@ -1247,8 +1185,8 @@ class ClassDoc : public StmtDoc {
    * \param decorators The decorator of class.
    * \param body The body of class.
    */
-  explicit ClassDoc(IdDoc name, Array<ExprDoc> decorators, Array<StmtDoc> body);
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(ClassDoc, StmtDoc, ClassDocNode);
+  explicit ClassDoc(IdDoc name, ffi::Array<ExprDoc> decorators, ffi::Array<StmtDoc> body);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(ClassDoc, StmtDoc, ClassDocNode);
 };
 
 /*!
@@ -1262,9 +1200,7 @@ class CommentDocNode : public StmtDocNode {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<CommentDocNode>();
   }
-
-  static constexpr const char* _type_key = "script.printer.CommentDoc";
-  TVM_DECLARE_FINAL_OBJECT_INFO(CommentDocNode, StmtDocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.printer.CommentDoc", CommentDocNode, StmtDocNode);
 };
 
 /*!
@@ -1274,8 +1210,8 @@ class CommentDocNode : public StmtDocNode {
  */
 class CommentDoc : public StmtDoc {
  public:
-  explicit CommentDoc(String comment);
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(CommentDoc, StmtDoc, CommentDocNode);
+  explicit CommentDoc(ffi::String comment);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(CommentDoc, StmtDoc, CommentDocNode);
 };
 
 /*!
@@ -1289,9 +1225,7 @@ class DocStringDocNode : public StmtDocNode {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<DocStringDocNode>();
   }
-
-  static constexpr const char* _type_key = "script.printer.DocStringDoc";
-  TVM_DECLARE_FINAL_OBJECT_INFO(DocStringDocNode, StmtDocNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("script.printer.DocStringDoc", DocStringDocNode, StmtDocNode);
 };
 
 /*!
@@ -1301,8 +1235,8 @@ class DocStringDocNode : public StmtDocNode {
  */
 class DocStringDoc : public StmtDoc {
  public:
-  explicit DocStringDoc(String docs);
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(DocStringDoc, StmtDoc, DocStringDocNode);
+  explicit DocStringDoc(ffi::String docs);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(DocStringDoc, StmtDoc, DocStringDocNode);
 };
 
 }  // namespace printer
