@@ -25,7 +25,7 @@
 #define TVM_TE_OPERATION_H_
 
 #include <tvm/arith/analyzer.h>
-#include <tvm/ffi/reflection/reflection.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/te/tensor.h>
 #include <tvm/tir/buffer.h>
 #include <tvm/tir/expr.h>
@@ -60,7 +60,7 @@ class TVM_DLL OperationNode : public Object {
   /*! \brief optional tag of the operation */
   std::string tag;
   /*! \brief additional attributes of the operation*/
-  Map<String, ffi::Any> attrs;
+  ffi::Map<ffi::String, ffi::Any> attrs;
   // virtual destructor.
   virtual ~OperationNode() {}
   /*! \return number of outputs */
@@ -76,12 +76,12 @@ class TVM_DLL OperationNode : public Object {
    * \param i The output index.
    * \return shape of i-th output.
    */
-  virtual Array<PrimExpr> output_shape(size_t i) const = 0;
+  virtual ffi::Array<PrimExpr> output_shape(size_t i) const = 0;
   /*!
    * \brief List all the input Tensors.
    * \return List of input tensors.
    */
-  virtual Array<Tensor> InputTensors() const = 0;
+  virtual ffi::Array<Tensor> InputTensors() const = 0;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -90,10 +90,7 @@ class TVM_DLL OperationNode : public Object {
         .def_ro("tag", &OperationNode::tag)
         .def_ro("attrs", &OperationNode::attrs);
   }
-
-  static constexpr const char* _type_key = "te.Operation";
-
-  TVM_DECLARE_BASE_OBJECT_INFO(OperationNode, Object);
+  TVM_FFI_DECLARE_OBJECT_INFO("te.Operation", OperationNode, Object);
 };
 
 /*!
@@ -102,14 +99,14 @@ class TVM_DLL OperationNode : public Object {
 class PlaceholderOpNode : public OperationNode {
  public:
   /*! \brief The shape of the input */
-  Array<PrimExpr> shape;
+  ffi::Array<PrimExpr> shape;
   /*! \brief The data type of the input. */
   DataType dtype;
   // override behavior.
   int num_outputs() const final;
   DataType output_dtype(size_t i) const final;
-  Array<PrimExpr> output_shape(size_t i) const final;
-  Array<Tensor> InputTensors() const final;
+  ffi::Array<PrimExpr> output_shape(size_t i) const final;
+  ffi::Array<Tensor> InputTensors() const final;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -117,10 +114,7 @@ class PlaceholderOpNode : public OperationNode {
         .def_ro("shape", &PlaceholderOpNode::shape)
         .def_ro("dtype", &PlaceholderOpNode::dtype);
   }
-
-  static constexpr const char* _type_key = "te.PlaceholderOp";
-
-  TVM_DECLARE_BASE_OBJECT_INFO(PlaceholderOpNode, OperationNode);
+  TVM_FFI_DECLARE_OBJECT_INFO("te.PlaceholderOp", PlaceholderOpNode, OperationNode);
 };
 
 /*!
@@ -129,9 +123,9 @@ class PlaceholderOpNode : public OperationNode {
  */
 class PlaceholderOp : public Operation {
  public:
-  TVM_DLL PlaceholderOp(std::string name, Array<PrimExpr> shape, DataType dtype);
+  TVM_DLL PlaceholderOp(std::string name, ffi::Array<PrimExpr> shape, DataType dtype);
 
-  TVM_DEFINE_OBJECT_REF_METHODS(PlaceholderOp, Operation, PlaceholderOpNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(PlaceholderOp, Operation, PlaceholderOpNode);
 };
 
 /*!
@@ -141,11 +135,11 @@ class PlaceholderOp : public Operation {
 class TVM_DLL BaseComputeOpNode : public OperationNode {
  public:
   /*! \brief IterVar on each axis */
-  Array<IterVar> axis;
+  ffi::Array<IterVar> axis;
   /*! \brief IterVar on each reduction axis, if the body is a Reduce */
-  Array<IterVar> reduce_axis;
+  ffi::Array<IterVar> reduce_axis;
   // override functions
-  Array<PrimExpr> output_shape(size_t idx) const final;
+  ffi::Array<PrimExpr> output_shape(size_t idx) const final;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -153,10 +147,7 @@ class TVM_DLL BaseComputeOpNode : public OperationNode {
         .def_ro("axis", &BaseComputeOpNode::axis)
         .def_ro("reduce_axis", &BaseComputeOpNode::reduce_axis);
   }
-
-  static constexpr const char* _type_key = "te.BaseComputeOp";
-
-  TVM_DECLARE_BASE_OBJECT_INFO(BaseComputeOpNode, OperationNode);
+  TVM_FFI_DECLARE_OBJECT_INFO("te.BaseComputeOp", BaseComputeOpNode, OperationNode);
 };
 
 /*!
@@ -165,22 +156,19 @@ class TVM_DLL BaseComputeOpNode : public OperationNode {
 class TVM_DLL ComputeOpNode : public BaseComputeOpNode {
  public:
   /*! \brief the compute expression */
-  Array<PrimExpr> body;
+  ffi::Array<PrimExpr> body;
   /*! \brief constructor */
   ComputeOpNode() {}
   // override functions
   int num_outputs() const final;
   DataType output_dtype(size_t i) const final;
-  Array<Tensor> InputTensors() const final;
+  ffi::Array<Tensor> InputTensors() const final;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<ComputeOpNode>().def_ro("body", &ComputeOpNode::body);
   }
-
-  static constexpr const char* _type_key = "te.ComputeOp";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(ComputeOpNode, BaseComputeOpNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("te.ComputeOp", ComputeOpNode, BaseComputeOpNode);
 };
 
 /*!
@@ -189,10 +177,10 @@ class TVM_DLL ComputeOpNode : public BaseComputeOpNode {
  */
 class ComputeOp : public Operation {
  public:
-  TVM_DLL ComputeOp(std::string name, std::string tag, Map<String, ffi::Any> attrs,
-                    Array<IterVar> axis, Array<PrimExpr> body);
+  TVM_DLL ComputeOp(std::string name, std::string tag, ffi::Map<ffi::String, ffi::Any> attrs,
+                    ffi::Array<IterVar> axis, ffi::Array<PrimExpr> body);
 
-  TVM_DEFINE_OBJECT_REF_METHODS(ComputeOp, Operation, ComputeOpNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(ComputeOp, Operation, ComputeOpNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(ComputeOpNode);
 };
 
@@ -204,16 +192,16 @@ class ScanOpNode : public OperationNode {
   /*! \brief IterVar to scan over */
   IterVar scan_axis;
   /*! \brief the initialization tensors */
-  Array<Tensor> init;
+  ffi::Array<Tensor> init;
   /*! \brief the update function represented by tensor */
-  Array<Tensor> update;
+  ffi::Array<Tensor> update;
   /*! \brief The placeholder to refer as states in update. */
-  Array<Tensor> state_placeholder;
+  ffi::Array<Tensor> state_placeholder;
   /*!
    * \brief the inputs to the scan, these are optionally provided
    *  But they can be helpful to provide hints to speedup get of scan body.
    */
-  Array<Tensor> inputs;
+  ffi::Array<Tensor> inputs;
   /*!
    * \brief Spatial axis to indicate spatial dimension of each output.
    *  They corresponds to flattened spatial axis of the outputs.
@@ -223,14 +211,14 @@ class ScanOpNode : public OperationNode {
    *  They do not corresponds to splittable iterations, thus the name comes
    *  with underscore.
    */
-  Array<IterVar> spatial_axis_;
+  ffi::Array<IterVar> spatial_axis_;
   /*! \brief constructor */
   ScanOpNode() {}
   // override behavior.
   int num_outputs() const final;
   DataType output_dtype(size_t i) const final;
-  Array<PrimExpr> output_shape(size_t i) const final;
-  Array<Tensor> InputTensors() const final;
+  ffi::Array<PrimExpr> output_shape(size_t i) const final;
+  ffi::Array<Tensor> InputTensors() const final;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -242,10 +230,7 @@ class ScanOpNode : public OperationNode {
         .def_ro("inputs", &ScanOpNode::inputs)
         .def_ro("spatial_axis_", &ScanOpNode::spatial_axis_);
   }
-
-  static constexpr const char* _type_key = "te.ScanOp";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(ScanOpNode, OperationNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("te.ScanOp", ScanOpNode, OperationNode);
 };
 
 /*!
@@ -254,11 +239,12 @@ class ScanOpNode : public OperationNode {
  */
 class ScanOp : public Operation {
  public:
-  TVM_DLL ScanOp(std::string name, std::string tag, Optional<Map<String, ffi::Any>> attrs,
-                 IterVar axis, Array<Tensor> init, Array<Tensor> update,
-                 Array<Tensor> state_placeholder, Array<Tensor> input);
+  TVM_DLL ScanOp(std::string name, std::string tag,
+                 ffi::Optional<ffi::Map<ffi::String, ffi::Any>> attrs, IterVar axis,
+                 ffi::Array<Tensor> init, ffi::Array<Tensor> update,
+                 ffi::Array<Tensor> state_placeholder, ffi::Array<Tensor> input);
 
-  TVM_DEFINE_OBJECT_REF_METHODS(ScanOp, Operation, ScanOpNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(ScanOp, Operation, ScanOpNode);
 };
 
 /*!
@@ -267,11 +253,11 @@ class ScanOp : public Operation {
 class ExternOpNode : public OperationNode {
  public:
   /*! \brief The input tensors */
-  Array<Tensor> inputs;
+  ffi::Array<Tensor> inputs;
   /*! \brief Symbolic placeholder representation of inputs */
-  Array<Buffer> input_placeholders;
+  ffi::Array<Buffer> input_placeholders;
   /*! \brief Symbolic placeholder representation of outputs */
-  Array<Buffer> output_placeholders;
+  ffi::Array<Buffer> output_placeholders;
   /*! \brief the statement that generates the computation. */
   Stmt body;
 
@@ -280,8 +266,8 @@ class ExternOpNode : public OperationNode {
   // override functions
   int num_outputs() const final;
   DataType output_dtype(size_t i) const final;
-  Array<PrimExpr> output_shape(size_t i) const final;
-  Array<Tensor> InputTensors() const final;
+  ffi::Array<PrimExpr> output_shape(size_t i) const final;
+  ffi::Array<Tensor> InputTensors() const final;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -291,10 +277,7 @@ class ExternOpNode : public OperationNode {
         .def_ro("output_placeholders", &ExternOpNode::output_placeholders)
         .def_ro("body", &ExternOpNode::body);
   }
-
-  static constexpr const char* _type_key = "te.ExternOp";
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(ExternOpNode, OperationNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("te.ExternOp", ExternOpNode, OperationNode);
 };
 
 /*!
@@ -303,11 +286,11 @@ class ExternOpNode : public OperationNode {
  */
 class ExternOp : public Operation {
  public:
-  TVM_DLL ExternOp(std::string name, std::string tag, Map<String, ffi::Any> attrs,
-                   Array<Tensor> inputs, Array<Buffer> input_placeholders,
-                   Array<Buffer> output_placeholders, Stmt body);
+  TVM_DLL ExternOp(std::string name, std::string tag, ffi::Map<ffi::String, ffi::Any> attrs,
+                   ffi::Array<Tensor> inputs, ffi::Array<Buffer> input_placeholders,
+                   ffi::Array<Buffer> output_placeholders, Stmt body);
 
-  TVM_DEFINE_OBJECT_REF_METHODS(ExternOp, Operation, ExternOpNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(ExternOp, Operation, ExternOpNode);
 };
 
 /*!
@@ -334,10 +317,10 @@ TVM_DLL IterVar thread_axis(Range dom, std::string tag);
 TVM_DLL IterVar reduce_axis(Range dom, std::string name = "rv");
 
 /*! \brief The compute function to specify the input source of a Tensor */
-using FCompute = std::function<PrimExpr(const Array<Var>& i)>;
+using FCompute = std::function<PrimExpr(const ffi::Array<Var>& i)>;
 
 /*! \brief The compute function to specify the inputs source of Tensors */
-using FBatchCompute = std::function<Array<PrimExpr>(const Array<Var>& i)>;
+using FBatchCompute = std::function<ffi::Array<PrimExpr>(const ffi::Array<Var>& i)>;
 
 /*!
  * \brief create a place holder tensor.
@@ -345,7 +328,7 @@ using FBatchCompute = std::function<Array<PrimExpr>(const Array<Var>& i)>;
  * \param dtype the data type of the tensor.
  * \param name The name of the Tensor.
  */
-TVM_DLL Tensor placeholder(Array<PrimExpr> shape, DataType dtype = DataType::Float(32),
+TVM_DLL Tensor placeholder(ffi::Array<PrimExpr> shape, DataType dtype = DataType::Float(32),
                            std::string name = "placeholder");
 
 /*!
@@ -357,8 +340,8 @@ TVM_DLL Tensor placeholder(Array<PrimExpr> shape, DataType dtype = DataType::Flo
  * \param tag The optional tag of the tensor.
  * \param attrs Optional additional attributes of the compute.
  */
-TVM_DLL Tensor compute(Array<PrimExpr> shape, FCompute fcompute, std::string name = "tensor",
-                       std::string tag = "", Map<String, ffi::Any> attrs = {});
+TVM_DLL Tensor compute(ffi::Array<PrimExpr> shape, FCompute fcompute, std::string name = "tensor",
+                       std::string tag = "", ffi::Map<ffi::String, ffi::Any> attrs = {});
 
 /*!
  * \brief Construct a new tensor by computing over shape,
@@ -369,9 +352,9 @@ TVM_DLL Tensor compute(Array<PrimExpr> shape, FCompute fcompute, std::string nam
  * \param tag The optional tag of the tensor.
  * \param attrs Optional additional attributes of the compute.
  */
-TVM_DLL Array<Tensor> compute(Array<PrimExpr> shape, FBatchCompute fcompute,
-                              std::string name = "tensor", std::string tag = "",
-                              Map<String, ffi::Any> attrs = {});
+TVM_DLL ffi::Array<Tensor> compute(ffi::Array<PrimExpr> shape, FBatchCompute fcompute,
+                                   std::string name = "tensor", std::string tag = "",
+                                   ffi::Map<ffi::String, ffi::Any> attrs = {});
 
 /*!
  * \brief Construct new tensors by scan.
@@ -385,34 +368,35 @@ TVM_DLL Array<Tensor> compute(Array<PrimExpr> shape, FBatchCompute fcompute,
  * \param tag The optional tag of the tensor.
  * \param attrs Optional additional attributes of the compute.
  */
-TVM_DLL Array<Tensor> scan(Array<Tensor> init, Array<Tensor> update,
-                           Array<Tensor> state_placeholder, Array<Tensor> inputs = Array<Tensor>(),
-                           std::string name = "scan", std::string tag = "",
-                           Map<String, ffi::Any> attrs = {});
+TVM_DLL ffi::Array<Tensor> scan(ffi::Array<Tensor> init, ffi::Array<Tensor> update,
+                                ffi::Array<Tensor> state_placeholder,
+                                ffi::Array<Tensor> inputs = ffi::Array<Tensor>(),
+                                std::string name = "scan", std::string tag = "",
+                                ffi::Map<ffi::String, ffi::Any> attrs = {});
 
 // same as compute, specialized for different fcompute function
-inline Tensor compute(Array<PrimExpr> shape, std::function<PrimExpr(Var)> f,
+inline Tensor compute(ffi::Array<PrimExpr> shape, std::function<PrimExpr(Var)> f,
                       std::string name = "tensor", std::string tag = "",
-                      Map<String, ffi::Any> attrs = {}) {
-  FCompute fc = [f](const Array<Var>& i) { return f(i[0]); };
+                      ffi::Map<ffi::String, ffi::Any> attrs = {}) {
+  FCompute fc = [f](const ffi::Array<Var>& i) { return f(i[0]); };
   return compute(shape, fc, name, tag, attrs);
 }
-inline Tensor compute(Array<PrimExpr> shape, std::function<PrimExpr(Var, Var)> f,
+inline Tensor compute(ffi::Array<PrimExpr> shape, std::function<PrimExpr(Var, Var)> f,
                       std::string name = "tensor", std::string tag = "",
-                      Map<String, ffi::Any> attrs = {}) {
-  FCompute fc = [f](const Array<Var>& i) { return f(i[0], i[1]); };
+                      ffi::Map<ffi::String, ffi::Any> attrs = {}) {
+  FCompute fc = [f](const ffi::Array<Var>& i) { return f(i[0], i[1]); };
   return compute(shape, fc, name, tag, attrs);
 }
-inline Tensor compute(Array<PrimExpr> shape, std::function<PrimExpr(Var, Var, Var)> f,
+inline Tensor compute(ffi::Array<PrimExpr> shape, std::function<PrimExpr(Var, Var, Var)> f,
                       std::string name = "tensor", std::string tag = "",
-                      Map<String, ffi::Any> attrs = {}) {
-  FCompute fc = [f](const Array<Var>& i) { return f(i[0], i[1], i[2]); };
+                      ffi::Map<ffi::String, ffi::Any> attrs = {}) {
+  FCompute fc = [f](const ffi::Array<Var>& i) { return f(i[0], i[1], i[2]); };
   return compute(shape, fc, name, tag, attrs);
 }
-inline Tensor compute(Array<PrimExpr> shape, std::function<PrimExpr(Var, Var, Var, Var)> f,
+inline Tensor compute(ffi::Array<PrimExpr> shape, std::function<PrimExpr(Var, Var, Var, Var)> f,
                       std::string name = "tensor", std::string tag = "",
-                      Map<String, ffi::Any> attrs = {}) {
-  FCompute fc = [f](const Array<Var>& i) { return f(i[0], i[1], i[2], i[3]); };
+                      ffi::Map<ffi::String, ffi::Any> attrs = {}) {
+  FCompute fc = [f](const ffi::Array<Var>& i) { return f(i[0], i[1], i[2], i[3]); };
   return compute(shape, fc, name, tag, attrs);
 }
 
