@@ -49,25 +49,17 @@ namespace tir {
 using IntImmNode = tvm::IntImmNode;
 using FloatImmNode = tvm::FloatImmNode;
 
-/*! \brief String constants, only used in asserts. */
+/*! \brief ffi::String constants, only used in asserts. */
 class StringImmNode : public PrimExprNode {
  public:
   /*! \brief The constant value content. */
-  String value;
+  ffi::String value;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<StringImmNode>().def_ro("value", &StringImmNode::value);
   }
-
-  bool SEqualReduce(const StringImmNode* other, SEqualReducer equal) const {
-    return equal(value, other->value);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const { hash_reduce(value); }
-
-  static constexpr const char* _type_key = "tir.StringImm";
-  TVM_DECLARE_FINAL_OBJECT_INFO(StringImmNode, PrimExprNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.StringImm", StringImmNode, PrimExprNode);
 };
 
 /*!
@@ -76,8 +68,8 @@ class StringImmNode : public PrimExprNode {
  */
 class StringImm : public PrimExpr {
  public:
-  TVM_DLL StringImm(String value, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(StringImm, PrimExpr, StringImmNode);
+  TVM_DLL StringImm(ffi::String value, Span span = Span());
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(StringImm, PrimExpr, StringImmNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(StringImmNode);
 };
 
@@ -94,18 +86,7 @@ class CastNode : public PrimExprNode {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<CastNode>().def_ro("value", &CastNode::value);
   }
-
-  bool SEqualReduce(const CastNode* other, SEqualReducer equal) const {
-    return equal(dtype, other->dtype) && equal(value, other->value);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(dtype);
-    hash_reduce(value);
-  }
-
-  static constexpr const char* _type_key = "tir.Cast";
-  TVM_DECLARE_FINAL_OBJECT_INFO(CastNode, PrimExprNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.Cast", CastNode, PrimExprNode);
 };
 
 /*!
@@ -115,7 +96,7 @@ class CastNode : public PrimExprNode {
 class Cast : public PrimExpr {
  public:
   TVM_DLL Cast(DataType dtype, PrimExpr value, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(Cast, PrimExpr, CastNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(Cast, PrimExpr, CastNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(CastNode);
 };
 
@@ -136,17 +117,9 @@ class BinaryOpNode : public PrimExprNode {
     refl::ObjectDef<T>().def_ro("a", &T::a).def_ro("b", &T::b);
   }
 
-  bool SEqualReduce(const T* other, SEqualReducer equal) const {
-    return equal(dtype, other->dtype) && equal(a, other->a) && equal(b, other->b);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(dtype);
-    hash_reduce(a);
-    hash_reduce(b);
-  }
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(T, PrimExprNode);
+  static const constexpr int _type_child_slots [[maybe_unused]] = 0;
+  static const constexpr bool _type_final [[maybe_unused]] = true;
+  TVM_FFI_DECLARE_OBJECT_INFO_PREDEFINED_TYPE_KEY(T, PrimExprNode);
 };
 
 /*! \brief a + b */
@@ -162,7 +135,7 @@ class AddNode : public BinaryOpNode<AddNode> {
 class Add : public PrimExpr {
  public:
   TVM_DLL Add(PrimExpr a, PrimExpr b, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(Add, PrimExpr, AddNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(Add, PrimExpr, AddNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(AddNode);
 };
 
@@ -180,7 +153,7 @@ class Sub : public PrimExpr {
  public:
   TVM_DLL Sub(PrimExpr a, PrimExpr b, Span span = Span());
 
-  TVM_DEFINE_OBJECT_REF_METHODS(Sub, PrimExpr, SubNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(Sub, PrimExpr, SubNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(SubNode);
 };
 
@@ -197,7 +170,7 @@ class MulNode : public BinaryOpNode<MulNode> {
 class Mul : public PrimExpr {
  public:
   TVM_DLL Mul(PrimExpr a, PrimExpr b, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(Mul, PrimExpr, MulNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(Mul, PrimExpr, MulNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(MulNode);
 };
 
@@ -217,7 +190,7 @@ class DivNode : public BinaryOpNode<DivNode> {
 class Div : public PrimExpr {
  public:
   TVM_DLL Div(PrimExpr a, PrimExpr b, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(Div, PrimExpr, DivNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(Div, PrimExpr, DivNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(DivNode);
 };
 
@@ -237,7 +210,7 @@ class ModNode : public BinaryOpNode<ModNode> {
 class Mod : public PrimExpr {
  public:
   TVM_DLL Mod(PrimExpr a, PrimExpr b, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(Mod, PrimExpr, ModNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(Mod, PrimExpr, ModNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(ModNode);
 };
 
@@ -254,7 +227,7 @@ class FloorDivNode : public BinaryOpNode<FloorDivNode> {
 class FloorDiv : public PrimExpr {
  public:
   TVM_DLL FloorDiv(PrimExpr a, PrimExpr b, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(FloorDiv, PrimExpr, FloorDivNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(FloorDiv, PrimExpr, FloorDivNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(FloorDivNode);
 };
 
@@ -271,7 +244,7 @@ class FloorModNode : public BinaryOpNode<FloorModNode> {
 class FloorMod : public PrimExpr {
  public:
   TVM_DLL FloorMod(PrimExpr a, PrimExpr b, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(FloorMod, PrimExpr, FloorModNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(FloorMod, PrimExpr, FloorModNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(FloorModNode);
 };
 
@@ -288,7 +261,7 @@ class MinNode : public BinaryOpNode<MinNode> {
 class Min : public PrimExpr {
  public:
   TVM_DLL Min(PrimExpr a, PrimExpr b, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(Min, PrimExpr, MinNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(Min, PrimExpr, MinNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(MinNode);
 };
 
@@ -305,7 +278,7 @@ class MaxNode : public BinaryOpNode<MaxNode> {
 class Max : public PrimExpr {
  public:
   TVM_DLL Max(PrimExpr a, PrimExpr b, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(Max, PrimExpr, MaxNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(Max, PrimExpr, MaxNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(MaxNode);
 };
 
@@ -326,17 +299,9 @@ class CmpOpNode : public PrimExprNode {
     refl::ObjectDef<T>().def_ro("a", &T::a).def_ro("b", &T::b);
   }
 
-  bool SEqualReduce(const T* other, SEqualReducer equal) const {
-    return equal(dtype, other->dtype) && equal(a, other->a) && equal(b, other->b);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(dtype);
-    hash_reduce(a);
-    hash_reduce(b);
-  }
-
-  TVM_DECLARE_FINAL_OBJECT_INFO(T, PrimExprNode);
+  static const constexpr int _type_child_slots [[maybe_unused]] = 0;
+  static const constexpr bool _type_final [[maybe_unused]] = true;
+  TVM_FFI_DECLARE_OBJECT_INFO_PREDEFINED_TYPE_KEY(T, PrimExprNode);
 };
 
 /*! \brief a == b */
@@ -352,7 +317,7 @@ class EQNode : public CmpOpNode<EQNode> {
 class EQ : public PrimExpr {
  public:
   TVM_DLL EQ(PrimExpr a, PrimExpr b, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(EQ, PrimExpr, EQNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(EQ, PrimExpr, EQNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(EQNode);
 };
 
@@ -369,7 +334,7 @@ class NENode : public CmpOpNode<NENode> {
 class NE : public PrimExpr {
  public:
   TVM_DLL NE(PrimExpr a, PrimExpr b, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(NE, PrimExpr, NENode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(NE, PrimExpr, NENode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(NENode);
 };
 
@@ -386,7 +351,7 @@ class LTNode : public CmpOpNode<LTNode> {
 class LT : public PrimExpr {
  public:
   TVM_DLL LT(PrimExpr a, PrimExpr b, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(LT, PrimExpr, LTNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(LT, PrimExpr, LTNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(LTNode);
 };
 
@@ -403,7 +368,7 @@ struct LENode : public CmpOpNode<LENode> {
 class LE : public PrimExpr {
  public:
   TVM_DLL LE(PrimExpr a, PrimExpr b, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(LE, PrimExpr, LENode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(LE, PrimExpr, LENode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(LENode);
 };
 
@@ -420,7 +385,7 @@ class GTNode : public CmpOpNode<GTNode> {
 class GT : public PrimExpr {
  public:
   TVM_DLL GT(PrimExpr a, PrimExpr b, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(GT, PrimExpr, GTNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(GT, PrimExpr, GTNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(GTNode);
 };
 
@@ -437,7 +402,7 @@ class GENode : public CmpOpNode<GENode> {
 class GE : public PrimExpr {
  public:
   TVM_DLL GE(PrimExpr a, PrimExpr b, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(GE, PrimExpr, GENode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(GE, PrimExpr, GENode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(GENode);
 };
 
@@ -453,19 +418,7 @@ class AndNode : public PrimExprNode {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<AndNode>().def_ro("a", &AndNode::a).def_ro("b", &AndNode::b);
   }
-
-  bool SEqualReduce(const AndNode* other, SEqualReducer equal) const {
-    return equal(dtype, other->dtype) && equal(a, other->a) && equal(b, other->b);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(dtype);
-    hash_reduce(a);
-    hash_reduce(b);
-  }
-
-  static constexpr const char* _type_key = "tir.And";
-  TVM_DECLARE_FINAL_OBJECT_INFO(AndNode, PrimExprNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.And", AndNode, PrimExprNode);
 };
 
 /*!
@@ -475,7 +428,7 @@ class AndNode : public PrimExprNode {
 class And : public PrimExpr {
  public:
   TVM_DLL And(PrimExpr a, PrimExpr b, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(And, PrimExpr, AndNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(And, PrimExpr, AndNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(AndNode);
 };
 
@@ -491,19 +444,7 @@ class OrNode : public PrimExprNode {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<OrNode>().def_ro("a", &OrNode::a).def_ro("b", &OrNode::b);
   }
-
-  bool SEqualReduce(const OrNode* other, SEqualReducer equal) const {
-    return equal(dtype, other->dtype) && equal(a, other->a) && equal(b, other->b);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(dtype);
-    hash_reduce(a);
-    hash_reduce(b);
-  }
-
-  static constexpr const char* _type_key = "tir.Or";
-  TVM_DECLARE_FINAL_OBJECT_INFO(OrNode, PrimExprNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.Or", OrNode, PrimExprNode);
 };
 
 /*!
@@ -513,7 +454,7 @@ class OrNode : public PrimExprNode {
 class Or : public PrimExpr {
  public:
   TVM_DLL Or(PrimExpr a, PrimExpr b, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(Or, PrimExpr, OrNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(Or, PrimExpr, OrNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(OrNode);
 };
 
@@ -527,18 +468,7 @@ class NotNode : public PrimExprNode {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<NotNode>().def_ro("a", &NotNode::a);
   }
-
-  bool SEqualReduce(const NotNode* other, SEqualReducer equal) const {
-    return equal(dtype, other->dtype) && equal(a, other->a);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(dtype);
-    hash_reduce(a);
-  }
-
-  static constexpr const char* _type_key = "tir.Not";
-  TVM_DECLARE_FINAL_OBJECT_INFO(NotNode, PrimExprNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.Not", NotNode, PrimExprNode);
 };
 
 /*!
@@ -548,7 +478,7 @@ class NotNode : public PrimExprNode {
 class Not : public PrimExpr {
  public:
   TVM_DLL Not(PrimExpr a, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(Not, PrimExpr, NotNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(Not, PrimExpr, NotNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(NotNode);
 };
 
@@ -575,21 +505,7 @@ class SelectNode : public PrimExprNode {
         .def_ro("true_value", &SelectNode::true_value)
         .def_ro("false_value", &SelectNode::false_value);
   }
-
-  bool SEqualReduce(const SelectNode* other, SEqualReducer equal) const {
-    return equal(dtype, other->dtype) && equal(condition, other->condition) &&
-           equal(true_value, other->true_value) && equal(false_value, other->false_value);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(dtype);
-    hash_reduce(condition);
-    hash_reduce(true_value);
-    hash_reduce(false_value);
-  }
-
-  static constexpr const char* _type_key = "tir.Select";
-  TVM_DECLARE_FINAL_OBJECT_INFO(SelectNode, PrimExprNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.Select", SelectNode, PrimExprNode);
 };
 
 /*!
@@ -600,7 +516,7 @@ class Select : public PrimExpr {
  public:
   TVM_DLL Select(PrimExpr condition, PrimExpr true_value, PrimExpr false_value, Span span = Span());
 
-  TVM_DEFINE_OBJECT_REF_METHODS(Select, PrimExpr, SelectNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(Select, PrimExpr, SelectNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(SelectNode);
 };
 
@@ -619,9 +535,9 @@ class BufferLoadNode : public PrimExprNode {
   /*! \brief The buffer variable. */
   Buffer buffer;
   /*! \brief The indices location to be loaded. */
-  Array<PrimExpr> indices;
+  ffi::Array<PrimExpr> indices;
   /*! \brief The predicate mask for loading values. */
-  Optional<PrimExpr> predicate;
+  ffi::Optional<PrimExpr> predicate;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -630,21 +546,7 @@ class BufferLoadNode : public PrimExprNode {
         .def_ro("indices", &BufferLoadNode::indices)
         .def_ro("predicate", &BufferLoadNode::predicate);
   }
-
-  bool SEqualReduce(const BufferLoadNode* other, SEqualReducer equal) const {
-    return equal(dtype, other->dtype) && equal(buffer, other->buffer) &&
-           equal(indices, other->indices);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(dtype);
-    hash_reduce(buffer);
-    hash_reduce(indices);
-    hash_reduce(predicate);
-  }
-
-  static constexpr const char* _type_key = "tir.BufferLoad";
-  TVM_DECLARE_FINAL_OBJECT_INFO(BufferLoadNode, PrimExprNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.BufferLoad", BufferLoadNode, PrimExprNode);
 
  private:
   /*! \brief Set the dtype based on the buffer/indices
@@ -669,9 +571,9 @@ class BufferLoadNode : public PrimExprNode {
  */
 class BufferLoad : public PrimExpr {
  public:
-  TVM_DLL explicit BufferLoad(Buffer buffer, Array<PrimExpr> indices,
-                              Optional<PrimExpr> predicate = std::nullopt, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(BufferLoad, PrimExpr, BufferLoadNode);
+  TVM_DLL explicit BufferLoad(Buffer buffer, ffi::Array<PrimExpr> indices,
+                              ffi::Optional<PrimExpr> predicate = std::nullopt, Span span = Span());
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(BufferLoad, PrimExpr, BufferLoadNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(BufferLoadNode);
 };
 
@@ -689,7 +591,7 @@ class ProducerLoadNode : public PrimExprNode {
   /*! \brief The buffer producer. */
   DataProducer producer;
   /*! \brief The location arguments. */
-  Array<PrimExpr> indices;
+  ffi::Array<PrimExpr> indices;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -697,20 +599,7 @@ class ProducerLoadNode : public PrimExprNode {
         .def_ro("producer", &ProducerLoadNode::producer)
         .def_ro("indices", &ProducerLoadNode::indices);
   }
-
-  bool SEqualReduce(const ProducerLoadNode* other, SEqualReducer equal) const {
-    return equal(dtype, other->dtype) && equal(producer, other->producer) &&
-           equal(indices, other->indices);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(dtype);
-    hash_reduce(producer);
-    hash_reduce(indices);
-  }
-
-  static constexpr const char* _type_key = "tir.ProducerLoad";
-  TVM_DECLARE_FINAL_OBJECT_INFO(ProducerLoadNode, PrimExprNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.ProducerLoad", ProducerLoadNode, PrimExprNode);
 };
 
 /*!
@@ -719,9 +608,10 @@ class ProducerLoadNode : public PrimExprNode {
  */
 class ProducerLoad : public PrimExpr {
  public:
-  TVM_DLL explicit ProducerLoad(DataProducer producer, Array<PrimExpr> indices, Span span = Span());
+  TVM_DLL explicit ProducerLoad(DataProducer producer, ffi::Array<PrimExpr> indices,
+                                Span span = Span());
 
-  TVM_DEFINE_OBJECT_REF_METHODS(ProducerLoad, PrimExpr, ProducerLoadNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(ProducerLoad, PrimExpr, ProducerLoadNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(ProducerLoadNode);
 };
 
@@ -750,21 +640,7 @@ class RampNode : public PrimExprNode {
         .def_ro("stride", &RampNode::stride)
         .def_ro("lanes", &RampNode::lanes);
   }
-
-  bool SEqualReduce(const RampNode* other, SEqualReducer equal) const {
-    return equal(dtype, other->dtype) && equal(base, other->base) && equal(stride, other->stride) &&
-           equal(lanes, other->lanes);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(dtype);
-    hash_reduce(base);
-    hash_reduce(stride);
-    hash_reduce(lanes);
-  }
-
-  static constexpr const char* _type_key = "tir.Ramp";
-  TVM_DECLARE_FINAL_OBJECT_INFO(RampNode, PrimExprNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.Ramp", RampNode, PrimExprNode);
 };
 
 /*!
@@ -774,7 +650,7 @@ class RampNode : public PrimExprNode {
 class Ramp : public PrimExpr {
  public:
   TVM_DLL Ramp(PrimExpr base, PrimExpr stride, PrimExpr lanes, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(Ramp, PrimExpr, RampNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(Ramp, PrimExpr, RampNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(RampNode);
 };
 
@@ -792,19 +668,7 @@ class BroadcastNode : public PrimExprNode {
         .def_ro("value", &BroadcastNode::value)
         .def_ro("lanes", &BroadcastNode::lanes);
   }
-
-  bool SEqualReduce(const BroadcastNode* other, SEqualReducer equal) const {
-    return equal(dtype, other->dtype) && equal(value, other->value) && equal(lanes, other->lanes);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(dtype);
-    hash_reduce(value);
-    hash_reduce(lanes);
-  }
-
-  static constexpr const char* _type_key = "tir.Broadcast";
-  TVM_DECLARE_FINAL_OBJECT_INFO(BroadcastNode, PrimExprNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.Broadcast", BroadcastNode, PrimExprNode);
 };
 
 /*!
@@ -814,7 +678,7 @@ class BroadcastNode : public PrimExprNode {
 class Broadcast : public PrimExpr {
  public:
   TVM_DLL Broadcast(PrimExpr value, PrimExpr lanes, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(Broadcast, PrimExpr, BroadcastNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(Broadcast, PrimExpr, BroadcastNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(BroadcastNode);
 };
 
@@ -833,25 +697,11 @@ class LetNode : public PrimExprNode {
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<LetNode>()
-        .def_ro("var", &LetNode::var)
+        .def_ro("var", &LetNode::var, refl::AttachFieldFlag::SEqHashDef())
         .def_ro("value", &LetNode::value)
         .def_ro("body", &LetNode::body);
   }
-
-  bool SEqualReduce(const LetNode* other, SEqualReducer equal) const {
-    return equal(dtype, other->dtype) && equal.DefEqual(var, other->var) &&
-           equal(value, other->value) && equal(body, other->body);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(dtype);
-    hash_reduce.DefHash(var);
-    hash_reduce(value);
-    hash_reduce(body);
-  }
-
-  static constexpr const char* _type_key = "tir.Let";
-  TVM_DECLARE_FINAL_OBJECT_INFO(LetNode, PrimExprNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.Let", LetNode, PrimExprNode);
 };
 
 /*!
@@ -861,7 +711,7 @@ class LetNode : public PrimExprNode {
 class Let : public PrimExpr {
  public:
   TVM_DLL Let(Var var, PrimExpr value, PrimExpr body, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(Let, PrimExpr, LetNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(Let, PrimExpr, LetNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(LetNode);
 };
 
@@ -879,25 +729,13 @@ class CallNode : public PrimExprNode {
   RelaxExpr op;
 
   /*! \brief The arguments. */
-  Array<PrimExpr> args;
+  ffi::Array<PrimExpr> args;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<CallNode>().def_ro("op", &CallNode::op).def_ro("args", &CallNode::args);
   }
-
-  bool SEqualReduce(const CallNode* other, SEqualReducer equal) const {
-    return equal(dtype, other->dtype) && equal(op, other->op) && equal(args, other->args);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(dtype);
-    hash_reduce(op);
-    hash_reduce(args);
-  }
-
-  static constexpr const char* _type_key = "tir.Call";
-  TVM_DECLARE_FINAL_OBJECT_INFO(CallNode, PrimExprNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.Call", CallNode, PrimExprNode);
 };
 
 /*!
@@ -906,8 +744,8 @@ class CallNode : public PrimExprNode {
  */
 class Call : public PrimExpr {
  public:
-  TVM_DLL Call(DataType dtype, RelaxExpr op, Array<PrimExpr> args, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(Call, PrimExpr, CallNode);
+  TVM_DLL Call(DataType dtype, RelaxExpr op, ffi::Array<PrimExpr> args, Span span = Span());
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(Call, PrimExpr, CallNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(CallNode);
 };
 
@@ -919,9 +757,9 @@ class Call : public PrimExpr {
 class ShuffleNode : public PrimExprNode {
  public:
   /*! \brief the input vectors. */
-  Array<PrimExpr> vectors;
+  ffi::Array<PrimExpr> vectors;
   /*! \brief The indices of each element. */
-  Array<PrimExpr> indices;
+  ffi::Array<PrimExpr> indices;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -929,20 +767,7 @@ class ShuffleNode : public PrimExprNode {
         .def_ro("vectors", &ShuffleNode::vectors)
         .def_ro("indices", &ShuffleNode::indices);
   }
-
-  bool SEqualReduce(const ShuffleNode* other, SEqualReducer equal) const {
-    return equal(dtype, other->dtype) && equal(vectors, other->vectors) &&
-           equal(indices, other->indices);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(dtype);
-    hash_reduce(vectors);
-    hash_reduce(indices);
-  }
-
-  static constexpr const char* _type_key = "tir.Shuffle";
-  TVM_DECLARE_FINAL_OBJECT_INFO(ShuffleNode, PrimExprNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.Shuffle", ShuffleNode, PrimExprNode);
 };
 
 /*!
@@ -951,11 +776,11 @@ class ShuffleNode : public PrimExprNode {
  */
 class Shuffle : public PrimExpr {
  public:
-  TVM_DLL Shuffle(Array<PrimExpr> vectors, Array<PrimExpr> indices, Span span = Span());
-  TVM_DLL static PrimExpr Concat(Array<PrimExpr> vectors, Span span = Span());
+  TVM_DLL Shuffle(ffi::Array<PrimExpr> vectors, ffi::Array<PrimExpr> indices, Span span = Span());
+  TVM_DLL static PrimExpr Concat(ffi::Array<PrimExpr> vectors, Span span = Span());
   TVM_DLL static PrimExpr ExtractElement(PrimExpr vector, int index, Span span = Span());
 
-  TVM_DEFINE_OBJECT_REF_METHODS(Shuffle, PrimExpr, ShuffleNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(Shuffle, PrimExpr, ShuffleNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(ShuffleNode);
 };
 
@@ -967,19 +792,19 @@ class Shuffle : public PrimExpr {
 class CommReducerNode : public Object {
  public:
   /*! \brief The left argument of reducer */
-  Array<Var> lhs;
+  ffi::Array<Var> lhs;
   /*! \brief The right argument of reducer */
-  Array<Var> rhs;
+  ffi::Array<Var> rhs;
   /*! \brief The result of reducer */
-  Array<PrimExpr> result;
+  ffi::Array<PrimExpr> result;
   /*!
    * \brief The identity element of reducer, which leaves other
    *  elements unchanged when combined with it, with respect to
    *  the binary operation of this reducer uses.
    */
-  Array<PrimExpr> identity_element;
+  ffi::Array<PrimExpr> identity_element;
   /*! \brief Function call operator to combine a and b */
-  Array<PrimExpr> operator()(Array<PrimExpr> a, Array<PrimExpr> b) const;
+  ffi::Array<PrimExpr> operator()(ffi::Array<PrimExpr> a, ffi::Array<PrimExpr> b) const;
   /*!
    * \brief Span that points to the original source code.
    *        Reserved debug information.
@@ -989,29 +814,15 @@ class CommReducerNode : public Object {
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<CommReducerNode>()
-        .def_ro("lhs", &CommReducerNode::lhs)
-        .def_ro("rhs", &CommReducerNode::rhs)
+        .def_ro("lhs", &CommReducerNode::lhs, refl::AttachFieldFlag::SEqHashDef())
+        .def_ro("rhs", &CommReducerNode::rhs, refl::AttachFieldFlag::SEqHashDef())
         .def_ro("result", &CommReducerNode::result)
         .def_ro("identity_element", &CommReducerNode::identity_element)
-        .def_ro("span", &CommReducerNode::span);
+        .def_ro("span", &CommReducerNode::span, refl::AttachFieldFlag::SEqHashIgnore());
   }
 
-  bool SEqualReduce(const CommReducerNode* other, SEqualReducer equal) const {
-    return equal.DefEqual(lhs, other->lhs) && equal.DefEqual(rhs, other->rhs) &&
-           equal(result, other->result) && equal(identity_element, other->identity_element);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce.DefHash(lhs);
-    hash_reduce.DefHash(rhs);
-    hash_reduce(result);
-    hash_reduce(identity_element);
-  }
-
-  static constexpr const char* _type_key = "tir.CommReducer";
-  static constexpr const bool _type_has_method_sequal_reduce = true;
-  static constexpr const bool _type_has_method_shash_reduce = true;
-  TVM_DECLARE_FINAL_OBJECT_INFO(CommReducerNode, Object);
+  static constexpr TVMFFISEqHashKind _type_s_eq_hash_kind = kTVMFFISEqHashKindTreeNode;
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.CommReducer", CommReducerNode, Object);
 };
 
 /*!
@@ -1020,10 +831,10 @@ class CommReducerNode : public Object {
  */
 class CommReducer : public ObjectRef {
  public:
-  TVM_DLL CommReducer(Array<Var> lhs, Array<Var> rhs, Array<PrimExpr> result,
-                      Array<PrimExpr> identity_element, Span span = Span());
+  TVM_DLL CommReducer(ffi::Array<Var> lhs, ffi::Array<Var> rhs, ffi::Array<PrimExpr> result,
+                      ffi::Array<PrimExpr> identity_element, Span span = Span());
 
-  TVM_DEFINE_OBJECT_REF_METHODS(CommReducer, ObjectRef, CommReducerNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(CommReducer, ObjectRef, CommReducerNode);
 };
 
 /*! \brief Reduction operator */
@@ -1032,11 +843,11 @@ class ReduceNode : public PrimExprNode {
   /*! \brief The commutative combiner */
   CommReducer combiner;
   /*! \brief The source operand */
-  Array<PrimExpr> source;
+  ffi::Array<PrimExpr> source;
   /*! \brief The init operand */
-  Array<PrimExpr> init;
+  ffi::Array<PrimExpr> init;
   /*! \brief The reduction axis */
-  Array<IterVar> axis;
+  ffi::Array<IterVar> axis;
   /*!
    * \brief Predicate on the reduction
    *  Only add the body to reduction if condition is true.
@@ -1055,27 +866,7 @@ class ReduceNode : public PrimExprNode {
         .def_ro("condition", &ReduceNode::condition)
         .def_ro("value_index", &ReduceNode::value_index);
   }
-
-  bool SEqualReduce(const ReduceNode* other, SEqualReducer equal) const {
-    // check axis first so IterVars can define the necessary variables.
-    return equal(dtype, other->dtype) && equal(axis, other->axis) &&
-           equal(combiner, other->combiner) && equal(source, other->source) &&
-           equal(init, other->init) && equal(condition, other->condition) &&
-           equal(value_index, other->value_index);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(dtype);
-    hash_reduce(axis);
-    hash_reduce(combiner);
-    hash_reduce(source);
-    hash_reduce(init);
-    hash_reduce(condition);
-    hash_reduce(value_index);
-  }
-
-  static constexpr const char* _type_key = "tir.Reduce";
-  TVM_DECLARE_FINAL_OBJECT_INFO(ReduceNode, PrimExprNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.Reduce", ReduceNode, PrimExprNode);
 };
 
 /*!
@@ -1084,10 +875,11 @@ class ReduceNode : public PrimExprNode {
  */
 class Reduce : public PrimExpr {
  public:
-  TVM_DLL Reduce(CommReducer combiner, Array<PrimExpr> src, Array<IterVar> rdom, PrimExpr condition,
-                 int value_index, Array<PrimExpr> init, Span span = Span());
+  TVM_DLL Reduce(CommReducer combiner, ffi::Array<PrimExpr> src, ffi::Array<IterVar> rdom,
+                 PrimExpr condition, int value_index, ffi::Array<PrimExpr> init,
+                 Span span = Span());
 
-  TVM_DEFINE_OBJECT_REF_METHODS(Reduce, PrimExpr, ReduceNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(Reduce, PrimExpr, ReduceNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(ReduceNode);
 };
 
@@ -1100,7 +892,7 @@ class Reduce : public PrimExpr {
  * \tparam V the value of the Map.
  */
 template <typename K, typename V>
-inline std::unordered_map<K, V> as_unordered_map(const Map<K, V>& dmap) {
+inline std::unordered_map<K, V> as_unordered_map(const ffi::Map<K, V>& dmap) {
   std::unordered_map<K, V> ret;
   for (auto kv : dmap) {
     ret[kv.first] = kv.second;
@@ -1116,8 +908,8 @@ inline constexpr bool use_default_type_traits_v<tvm::tir::StringImm> = false;
 
 template <>
 struct TypeTraits<tvm::tir::StringImm>
-    : public ObjectRefWithFallbackTraitsBase<tvm::tir::StringImm, String> {
-  TVM_FFI_INLINE static tvm::tir::StringImm ConvertFallbackValue(String value) {
+    : public ObjectRefWithFallbackTraitsBase<tvm::tir::StringImm, ffi::String> {
+  TVM_FFI_INLINE static tvm::tir::StringImm ConvertFallbackValue(ffi::String value) {
     return tvm::tir::StringImm(value);
   }
 };
