@@ -24,7 +24,7 @@
 #ifndef TVM_TIR_STMT_H_
 #define TVM_TIR_STMT_H_
 
-#include <tvm/ffi/reflection/reflection.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/tir/expr.h>
 
 #include <string>
@@ -53,17 +53,16 @@ class StmtNode : public Object {
 
   TVM_OBJECT_ENABLE_SCRIPT_PRINTER();
 
-  static constexpr const char* _type_key = "tir.Stmt";
-  static constexpr const bool _type_has_method_sequal_reduce = true;
-  static constexpr const bool _type_has_method_shash_reduce = true;
+  static constexpr TVMFFISEqHashKind _type_s_eq_hash_kind = kTVMFFISEqHashKindTreeNode;
+
   static constexpr const uint32_t _type_child_slots = 15;
-  TVM_DECLARE_BASE_OBJECT_INFO(StmtNode, Object);
+  TVM_FFI_DECLARE_OBJECT_INFO("tir.Stmt", StmtNode, Object);
 };
 
 /*! \brief Container of all statements */
 class Stmt : public ObjectRef {
  public:
-  TVM_DEFINE_OBJECT_REF_METHODS(Stmt, ObjectRef, StmtNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(Stmt, ObjectRef, StmtNode);
 };
 
 /*!
@@ -81,24 +80,11 @@ class LetStmtNode : public StmtNode {
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<LetStmtNode>()
-        .def_ro("var", &LetStmtNode::var)
+        .def_ro("var", &LetStmtNode::var, refl::AttachFieldFlag::SEqHashDef())
         .def_ro("value", &LetStmtNode::value)
         .def_ro("body", &LetStmtNode::body);
   }
-
-  bool SEqualReduce(const LetStmtNode* other, SEqualReducer equal) const {
-    return equal.DefEqual(var, other->var) && equal(value, other->value) &&
-           equal(body, other->body);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce.DefHash(var);
-    hash_reduce(value);
-    hash_reduce(body);
-  }
-
-  static constexpr const char* _type_key = "tir.LetStmt";
-  TVM_DECLARE_FINAL_OBJECT_INFO(LetStmtNode, StmtNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.LetStmt", LetStmtNode, StmtNode);
 };
 
 /*!
@@ -109,7 +95,7 @@ class LetStmt : public Stmt {
  public:
   TVM_DLL LetStmt(Var var, PrimExpr value, Stmt body, Span span = Span());
 
-  TVM_DEFINE_OBJECT_REF_METHODS(LetStmt, Stmt, LetStmtNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(LetStmt, Stmt, LetStmtNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(LetStmtNode);
 };
 
@@ -126,9 +112,9 @@ class LetStmt : public Stmt {
 class AttrStmtNode : public StmtNode {
  public:
   /*! \brief this is attribute about certain node */
-  ObjectRef node;
+  ffi::Any node;
   /*! \brief the type key of the attribute */
-  String attr_key;
+  ffi::String attr_key;
   /*! \brief The attribute value, value is well defined at current scope. */
   PrimExpr value;
   /*! \brief The body statement to be executed */
@@ -142,21 +128,7 @@ class AttrStmtNode : public StmtNode {
         .def_ro("value", &AttrStmtNode::value)
         .def_ro("body", &AttrStmtNode::body);
   }
-
-  bool SEqualReduce(const AttrStmtNode* other, SEqualReducer equal) const {
-    return equal(node, other->node) && equal(attr_key, other->attr_key) &&
-           equal(value, other->value) && equal(body, other->body);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(node);
-    hash_reduce(attr_key);
-    hash_reduce(value);
-    hash_reduce(body);
-  }
-
-  static constexpr const char* _type_key = "tir.AttrStmt";
-  TVM_DECLARE_FINAL_OBJECT_INFO(AttrStmtNode, StmtNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.AttrStmt", AttrStmtNode, StmtNode);
 };
 
 /*!
@@ -165,9 +137,10 @@ class AttrStmtNode : public StmtNode {
  */
 class AttrStmt : public Stmt {
  public:
-  TVM_DLL AttrStmt(ObjectRef node, String attr_key, PrimExpr value, Stmt body, Span span = Span());
+  TVM_DLL AttrStmt(ffi::Any node, ffi::String attr_key, PrimExpr value, Stmt body,
+                   Span span = Span());
 
-  TVM_DEFINE_OBJECT_REF_METHODS(AttrStmt, Stmt, AttrStmtNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(AttrStmt, Stmt, AttrStmtNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(AttrStmtNode);
 };
 
@@ -193,20 +166,7 @@ class AssertStmtNode : public StmtNode {
         .def_ro("message", &AssertStmtNode::message)
         .def_ro("body", &AssertStmtNode::body);
   }
-
-  bool SEqualReduce(const AssertStmtNode* other, SEqualReducer equal) const {
-    return equal(condition, other->condition) && equal(message, other->message) &&
-           equal(body, other->body);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(condition);
-    hash_reduce(message);
-    hash_reduce(body);
-  }
-
-  static constexpr const char* _type_key = "tir.AssertStmt";
-  TVM_DECLARE_FINAL_OBJECT_INFO(AssertStmtNode, StmtNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.AssertStmt", AssertStmtNode, StmtNode);
 };
 
 /*!
@@ -217,7 +177,7 @@ class AssertStmt : public Stmt {
  public:
   TVM_DLL AssertStmt(PrimExpr condition, PrimExpr message, Stmt body, Span span = Span());
 
-  TVM_DEFINE_OBJECT_REF_METHODS(AssertStmt, Stmt, AssertStmtNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(AssertStmt, Stmt, AssertStmtNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(AssertStmtNode);
 };
 
@@ -238,9 +198,9 @@ class BufferStoreNode : public StmtNode {
   /*! \brief The value to be stored. */
   PrimExpr value;
   /*! \brief The indices location to be stored. */
-  Array<PrimExpr> indices;
+  ffi::Array<PrimExpr> indices;
   /*! \brief The predicate mask for storing values. */
-  Optional<PrimExpr> predicate;
+  ffi::Optional<PrimExpr> predicate;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -250,21 +210,7 @@ class BufferStoreNode : public StmtNode {
         .def_ro("indices", &BufferStoreNode::indices)
         .def_ro("predicate", &BufferStoreNode::predicate);
   }
-
-  bool SEqualReduce(const BufferStoreNode* other, SEqualReducer equal) const {
-    return equal(buffer, other->buffer) && equal(value, other->value) &&
-           equal(indices, other->indices);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(buffer);
-    hash_reduce(value);
-    hash_reduce(indices);
-    hash_reduce(predicate);
-  }
-
-  static constexpr const char* _type_key = "tir.BufferStore";
-  TVM_DECLARE_FINAL_OBJECT_INFO(BufferStoreNode, StmtNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.BufferStore", BufferStoreNode, StmtNode);
 };
 
 /*!
@@ -273,10 +219,11 @@ class BufferStoreNode : public StmtNode {
  */
 class BufferStore : public Stmt {
  public:
-  TVM_DLL explicit BufferStore(Buffer buffer, PrimExpr value, Array<PrimExpr> indices,
-                               Optional<PrimExpr> predicate = std::nullopt, Span span = Span());
+  TVM_DLL explicit BufferStore(Buffer buffer, PrimExpr value, ffi::Array<PrimExpr> indices,
+                               ffi::Optional<PrimExpr> predicate = std::nullopt,
+                               Span span = Span());
 
-  TVM_DEFINE_OBJECT_REF_METHODS(BufferStore, Stmt, BufferStoreNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(BufferStore, Stmt, BufferStoreNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(BufferStoreNode);
 };
 
@@ -296,7 +243,7 @@ class BufferRealizeNode : public StmtNode {
   /*! \brief The buffer variable. */
   Buffer buffer;
   /*! \brief Bounds to be realized */
-  Array<Range> bounds;
+  ffi::Array<Range> bounds;
   /*! \brief Only realize if condition holds. */
   PrimExpr condition;
   /*! \brief The body of realization. */
@@ -311,25 +258,11 @@ class BufferRealizeNode : public StmtNode {
         .def_ro("body", &BufferRealizeNode::body);
   }
 
-  bool SEqualReduce(const BufferRealizeNode* other, SEqualReducer equal) const {
-    return equal(buffer, other->buffer) && equal(bounds, other->bounds) &&
-           equal(condition, other->condition) && equal(body, other->body);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(buffer);
-    hash_reduce(bounds);
-    hash_reduce(condition);
-    hash_reduce(body);
-  }
-
   BufferRealizeNode() = default;
-  BufferRealizeNode(Buffer buffer, Array<Range> bounds, PrimExpr condition, Stmt body,
+  BufferRealizeNode(Buffer buffer, ffi::Array<Range> bounds, PrimExpr condition, Stmt body,
                     Span span = Span())
       : StmtNode(span), buffer(buffer), bounds(bounds), condition(condition), body(body) {}
-
-  static constexpr const char* _type_key = "tir.BufferRealize";
-  TVM_DECLARE_FINAL_OBJECT_INFO(BufferRealizeNode, StmtNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.BufferRealize", BufferRealizeNode, StmtNode);
 };
 
 /*!
@@ -338,10 +271,10 @@ class BufferRealizeNode : public StmtNode {
  */
 class BufferRealize : public Stmt {
  public:
-  TVM_DLL explicit BufferRealize(Buffer buffer, Array<Range> bounds, PrimExpr condition, Stmt body,
-                                 Span span = Span());
+  TVM_DLL explicit BufferRealize(Buffer buffer, ffi::Array<Range> bounds, PrimExpr condition,
+                                 Stmt body, Span span = Span());
 
-  TVM_DEFINE_NOTNULLABLE_OBJECT_REF_METHODS(BufferRealize, Stmt, BufferRealizeNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(BufferRealize, Stmt, BufferRealizeNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(BufferRealizeNode);
 };
 
@@ -355,7 +288,7 @@ class AllocateNode : public StmtNode {
   /*! \brief The type of the buffer. */
   DataType dtype;
   /*! \brief The extents of the buffer. */
-  Array<PrimExpr> extents;
+  ffi::Array<PrimExpr> extents;
   /*! \brief Only allocate buffer when condition is satisfied. */
   PrimExpr condition;
   /*! \brief The body to be executed. */
@@ -366,12 +299,12 @@ class AllocateNode : public StmtNode {
    *  These annotations can be used as auxiliary hint
    *  to future transformations.
    */
-  Map<String, ffi::Any> annotations;
+  ffi::Map<ffi::String, ffi::Any> annotations;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<AllocateNode>()
-        .def_ro("buffer_var", &AllocateNode::buffer_var)
+        .def_ro("buffer_var", &AllocateNode::buffer_var, refl::AttachFieldFlag::SEqHashDef())
         .def_ro("dtype", &AllocateNode::dtype)
         .def_ro("extents", &AllocateNode::extents)
         .def_ro("condition", &AllocateNode::condition)
@@ -379,21 +312,6 @@ class AllocateNode : public StmtNode {
         .def_ro("annotations", &AllocateNode::annotations);
   }
 
-  bool SEqualReduce(const AllocateNode* other, SEqualReducer equal) const {
-    return equal.DefEqual(buffer_var, other->buffer_var) && equal(dtype, other->dtype) &&
-           equal(extents, other->extents) && equal(condition, other->condition) &&
-           equal(body, other->body) && equal(annotations, other->annotations);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce.DefHash(buffer_var);
-    hash_reduce(dtype);
-    hash_reduce(extents);
-    hash_reduce(condition);
-    hash_reduce(body);
-    hash_reduce(annotations);
-  }
-
   /*!
    * \brief If the buffer size is constant, return the size.
    *        Otherwise return 0.
@@ -406,12 +324,8 @@ class AllocateNode : public StmtNode {
    * \param extents The extents of the buffer.
    * \return The result.
    */
-  TVM_DLL static int64_t ConstantAllocationSize(const Array<PrimExpr>& extents);
-
-  static constexpr const char* _type_key = "tir.Allocate";
-  static constexpr const bool _type_has_method_sequal_reduce = true;
-  static constexpr const bool _type_has_method_shash_reduce = true;
-  TVM_DECLARE_FINAL_OBJECT_INFO(AllocateNode, StmtNode);
+  TVM_DLL static int64_t ConstantAllocationSize(const ffi::Array<PrimExpr>& extents);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.Allocate", AllocateNode, StmtNode);
 };
 
 /*!
@@ -420,11 +334,12 @@ class AllocateNode : public StmtNode {
  */
 class Allocate : public Stmt {
  public:
-  TVM_DLL Allocate(Var buffer_var, DataType dtype, Array<PrimExpr> extents, PrimExpr condition,
-                   Stmt body, Map<String, ffi::Any> annotations = Map<String, ffi::Any>(),
+  TVM_DLL Allocate(Var buffer_var, DataType dtype, ffi::Array<PrimExpr> extents, PrimExpr condition,
+                   Stmt body,
+                   ffi::Map<ffi::String, ffi::Any> annotations = ffi::Map<ffi::String, ffi::Any>(),
                    Span span = Span());
 
-  TVM_DEFINE_OBJECT_REF_METHODS(Allocate, Stmt, AllocateNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(Allocate, Stmt, AllocateNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(AllocateNode);
 };
 
@@ -437,16 +352,16 @@ class AllocateConstNode : public StmtNode {
   Var buffer_var;
   /*! \brief The optional data associated to the constant.
    */
-  Optional<runtime::NDArray> data;
+  ffi::Optional<runtime::Tensor> data;
   /*!
    * \brief If the PrimFunc containing the Stmt is added to IRModule, this is an optional index
-   * to indicate the index within "constants" attribute, that is a Array<NDArray> of IRModule.
+   * to indicate the index within "constants" attribute, that is a ffi::Array<Tensor> of IRModule.
    */
-  Optional<Integer> irmod_storage_idx;
+  ffi::Optional<Integer> irmod_storage_idx;
   /*! \brief The type of the buffer. */
   DataType dtype;
   /*! \brief The extents of the buffer. */
-  Array<PrimExpr> extents;
+  ffi::Array<PrimExpr> extents;
   /*! \brief The body to be executed. */
   Stmt body;
   /*!
@@ -455,12 +370,12 @@ class AllocateConstNode : public StmtNode {
    *  These annotations can be used as auxiliary hint
    *  to future transformations.
    */
-  Map<String, ffi::Any> annotations;
+  ffi::Map<ffi::String, ffi::Any> annotations;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<AllocateConstNode>()
-        .def_ro("buffer_var", &AllocateConstNode::buffer_var)
+        .def_ro("buffer_var", &AllocateConstNode::buffer_var, refl::AttachFieldFlag::SEqHashDef())
         .def_ro("data", &AllocateConstNode::data)
         .def_ro("irmod_storage_idx", &AllocateConstNode::irmod_storage_idx)
         .def_ro("dtype", &AllocateConstNode::dtype)
@@ -469,21 +384,6 @@ class AllocateConstNode : public StmtNode {
         .def_ro("annotations", &AllocateConstNode::annotations);
   }
 
-  bool SEqualReduce(const AllocateConstNode* other, SEqualReducer equal) const {
-    return equal.DefEqual(buffer_var, other->buffer_var) && equal(dtype, other->dtype) &&
-           equal(extents, other->extents) && equal(data, other->data) && equal(body, other->body) &&
-           equal(annotations, other->annotations);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce.DefHash(buffer_var);
-    hash_reduce(dtype);
-    hash_reduce(extents);
-    hash_reduce(body);
-    hash_reduce(annotations);
-    hash_reduce(data);
-  }
-
   /*!
    * \brief If the buffer size is constant, return the size.
    *        Otherwise return 0.
@@ -496,12 +396,8 @@ class AllocateConstNode : public StmtNode {
    * \param extents The extents of the buffer.
    * \return The result.
    */
-  TVM_DLL static int64_t ConstantAllocationSize(const Array<PrimExpr>& extents);
-
-  static constexpr const char* _type_key = "tir.AllocateConst";
-  static constexpr const bool _type_has_method_sequal_reduce = true;
-  static constexpr const bool _type_has_method_shash_reduce = true;
-  TVM_DECLARE_FINAL_OBJECT_INFO(AllocateConstNode, StmtNode);
+  TVM_DLL static int64_t ConstantAllocationSize(const ffi::Array<PrimExpr>& extents);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.AllocateConst", AllocateConstNode, StmtNode);
 };
 
 /*!
@@ -514,11 +410,11 @@ class AllocateConst : public Stmt {
    * depending on the type of ObjectRef, it will either
    * create AllocateConstNode with irmod_storage_idx or data
    */
-  TVM_DLL AllocateConst(Var buffer_var, DataType dtype, Array<PrimExpr> extents,
-                        ObjectRef data_or_idx, Stmt body,
-                        Map<String, ffi::Any> annotations = Map<String, ffi::Any>(),
-                        Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(AllocateConst, Stmt, AllocateConstNode);
+  TVM_DLL AllocateConst(
+      Var buffer_var, DataType dtype, ffi::Array<PrimExpr> extents, ObjectRef data_or_idx,
+      Stmt body, ffi::Map<ffi::String, ffi::Any> annotations = ffi::Map<ffi::String, ffi::Any>(),
+      Span span = Span());
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(AllocateConst, Stmt, AllocateConstNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(AllocateConstNode);
 };
 
@@ -536,25 +432,14 @@ class DeclBufferNode : public StmtNode {
         .def_ro("buffer", &DeclBufferNode::buffer)
         .def_ro("body", &DeclBufferNode::body);
   }
-
-  bool SEqualReduce(const DeclBufferNode* other, SEqualReducer equal) const {
-    return equal(buffer, other->buffer) && equal(body, other->body);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(buffer);
-    hash_reduce(body);
-  }
-
-  static constexpr const char* _type_key = "tir.DeclBuffer";
-  TVM_DECLARE_FINAL_OBJECT_INFO(DeclBufferNode, StmtNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.DeclBuffer", DeclBufferNode, StmtNode);
 };
 
 /*! \brief Managed reference to DeclBufferNode */
 class DeclBuffer : public Stmt {
  public:
   TVM_DLL DeclBuffer(Buffer buffer, Stmt body, Span span = Span());
-  TVM_DEFINE_OBJECT_REF_METHODS(DeclBuffer, Stmt, DeclBufferNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(DeclBuffer, Stmt, DeclBufferNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(DeclBufferNode);
 };
 
@@ -565,7 +450,7 @@ class DeclBuffer : public Stmt {
 class SeqStmtNode : public StmtNode {
  public:
   /*! \brief internal sequence content. */
-  Array<Stmt> seq;
+  ffi::Array<Stmt> seq;
 
   /*! \return get the size of the sequence */
   size_t size() const { return seq.size(); }
@@ -578,15 +463,7 @@ class SeqStmtNode : public StmtNode {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<SeqStmtNode>().def_ro("seq", &SeqStmtNode::seq);
   }
-
-  bool SEqualReduce(const SeqStmtNode* other, SEqualReducer equal) const {
-    return equal(seq, other->seq);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const { hash_reduce(seq); }
-
-  static constexpr const char* _type_key = "tir.SeqStmt";
-  TVM_DECLARE_FINAL_OBJECT_INFO(SeqStmtNode, StmtNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.SeqStmt", SeqStmtNode, StmtNode);
 };
 
 /*!
@@ -604,15 +481,7 @@ class EvaluateNode : public StmtNode {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<EvaluateNode>().def_ro("value", &EvaluateNode::value);
   }
-
-  bool SEqualReduce(const EvaluateNode* other, SEqualReducer equal) const {
-    return equal(value, other->value);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const { hash_reduce(value); }
-
-  static constexpr const char* _type_key = "tir.Evaluate";
-  TVM_DECLARE_FINAL_OBJECT_INFO(EvaluateNode, StmtNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.Evaluate", EvaluateNode, StmtNode);
 };
 
 /*!
@@ -625,7 +494,7 @@ class Evaluate : public Stmt {
 
   explicit Evaluate(int value, Span span = Span()) : Evaluate(PrimExpr(value), span) {}
 
-  TVM_DEFINE_OBJECT_REF_METHODS(Evaluate, Stmt, EvaluateNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(Evaluate, Stmt, EvaluateNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(EvaluateNode);
 };
 
@@ -637,7 +506,7 @@ class SeqStmt : public Stmt {
    * \param seq The sequence.
    * \param span The location of this object in the source code.
    */
-  TVM_DLL explicit SeqStmt(Array<Stmt> seq, Span span = Span());
+  TVM_DLL explicit SeqStmt(ffi::Array<Stmt> seq, Span span = Span());
 
   /*! \return get the size of the sequence */
   size_t size() const { return operator->()->size(); }
@@ -667,7 +536,7 @@ class SeqStmt : public Stmt {
    */
   template <typename... Args>
   static Stmt Flatten(Args&&... seq_args) {
-    Array<Stmt> seq;
+    ffi::Array<Stmt> seq;
 
     ffi::details::for_each(Flattener(&seq), std::forward<Args>(seq_args)...);
 
@@ -705,10 +574,10 @@ class SeqStmt : public Stmt {
   /*! \brief Helper class to flatten sequence of arguments into Array. */
   class Flattener {
    public:
-    explicit Flattener(Array<Stmt>* seq) : seq_(seq) {}
+    explicit Flattener(ffi::Array<Stmt>* seq) : seq_(seq) {}
 
     template <typename T>
-    static Optional<SeqStmt> AsSeqStmt(const T& t) {
+    static ffi::Optional<SeqStmt> AsSeqStmt(const T& t) {
       if constexpr (std::is_same_v<T, SeqStmt>) {
         return t;
       }
@@ -717,7 +586,7 @@ class SeqStmt : public Stmt {
       }
       if constexpr (std::is_base_of_v<Stmt, T>) {
         if (const SeqStmtNode* ptr = t.template as<SeqStmtNode>()) {
-          return GetRef<SeqStmt>(ptr);
+          return ffi::GetRef<SeqStmt>(ptr);
         } else {
           return std::nullopt;
         }
@@ -773,10 +642,10 @@ class SeqStmt : public Stmt {
     }
 
    private:
-    Array<Stmt>* seq_;
+    ffi::Array<Stmt>* seq_;
   };
 
-  TVM_DEFINE_OBJECT_REF_METHODS(SeqStmt, Stmt, SeqStmtNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(SeqStmt, Stmt, SeqStmtNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(SeqStmtNode);
 };
 
@@ -790,7 +659,7 @@ class IfThenElseNode : public StmtNode {
   /*! \brief The branch to be executed when condition is true. */
   Stmt then_case;
   /*! \brief The branch to be executed when condition is false, can be null. */
-  Optional<Stmt> else_case;
+  ffi::Optional<Stmt> else_case;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -799,20 +668,7 @@ class IfThenElseNode : public StmtNode {
         .def_ro("then_case", &IfThenElseNode::then_case)
         .def_ro("else_case", &IfThenElseNode::else_case);
   }
-
-  bool SEqualReduce(const IfThenElseNode* other, SEqualReducer equal) const {
-    return equal(condition, other->condition) && equal(then_case, other->then_case) &&
-           equal(else_case, other->else_case);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(condition);
-    hash_reduce(then_case);
-    hash_reduce(else_case);
-  }
-
-  static constexpr const char* _type_key = "tir.IfThenElse";
-  TVM_DECLARE_FINAL_OBJECT_INFO(IfThenElseNode, StmtNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.IfThenElse", IfThenElseNode, StmtNode);
 };
 
 /*!
@@ -821,10 +677,10 @@ class IfThenElseNode : public StmtNode {
  */
 class IfThenElse : public Stmt {
  public:
-  TVM_DLL IfThenElse(PrimExpr condition, Stmt then_case, Optional<Stmt> else_case = std::nullopt,
-                     Span span = Span());
+  TVM_DLL IfThenElse(PrimExpr condition, Stmt then_case,
+                     ffi::Optional<Stmt> else_case = std::nullopt, Span span = Span());
 
-  TVM_DEFINE_OBJECT_REF_METHODS(IfThenElse, Stmt, IfThenElseNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(IfThenElse, Stmt, IfThenElseNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(IfThenElseNode);
 };
 
@@ -882,7 +738,7 @@ class ForNode : public StmtNode {
    * \brief Only valid when kind == ForKind::kThreadBinding
    * The context thread that this loop variable bounds to.
    */
-  Optional<IterVar> thread_binding;
+  ffi::Optional<IterVar> thread_binding;
   /*!
    * \brief Additional annotations about the loop.
    *
@@ -891,12 +747,12 @@ class ForNode : public StmtNode {
    *  not change the control flow semantics of the loop
    *  and can be ignored in most passes.
    */
-  Map<String, ffi::Any> annotations;
+  ffi::Map<ffi::String, ffi::Any> annotations;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<ForNode>()
-        .def_ro("loop_var", &ForNode::loop_var)
+        .def_ro("loop_var", &ForNode::loop_var, refl::AttachFieldFlag::SEqHashDef())
         .def_ro("min", &ForNode::min)
         .def_ro("extent", &ForNode::extent)
         .def_ro("kind", &ForNode::kind)
@@ -904,25 +760,7 @@ class ForNode : public StmtNode {
         .def_ro("thread_binding", &ForNode::thread_binding)
         .def_ro("annotations", &ForNode::annotations);
   }
-
-  bool SEqualReduce(const ForNode* other, SEqualReducer equal) const {
-    return equal.DefEqual(loop_var, other->loop_var) && equal(min, other->min) &&
-           equal(extent, other->extent) && equal(kind, other->kind) && equal(body, other->body) &&
-           equal(thread_binding, other->thread_binding) && equal(annotations, other->annotations);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce.DefHash(loop_var);
-    hash_reduce(min);
-    hash_reduce(extent);
-    hash_reduce(kind);
-    hash_reduce(body);
-    hash_reduce(thread_binding);
-    hash_reduce(annotations);
-  }
-
-  static constexpr const char* _type_key = "tir.For";
-  TVM_DECLARE_FINAL_OBJECT_INFO(ForNode, StmtNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.For", ForNode, StmtNode);
 };
 
 /*!
@@ -932,10 +770,11 @@ class ForNode : public StmtNode {
 class For : public Stmt {
  public:
   TVM_DLL For(Var loop_var, PrimExpr min, PrimExpr extent, ForKind kind, Stmt body,
-              Optional<IterVar> thread_binding = std::nullopt,
-              Map<String, ffi::Any> annotations = Map<String, ffi::Any>(), Span span = Span());
+              ffi::Optional<IterVar> thread_binding = std::nullopt,
+              ffi::Map<ffi::String, ffi::Any> annotations = ffi::Map<ffi::String, ffi::Any>(),
+              Span span = Span());
 
-  TVM_DEFINE_OBJECT_REF_METHODS(For, Stmt, ForNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(For, Stmt, ForNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(ForNode);
 };
 
@@ -962,18 +801,7 @@ class WhileNode : public StmtNode {
         .def_ro("condition", &WhileNode::condition)
         .def_ro("body", &WhileNode::body);
   }
-
-  bool SEqualReduce(const WhileNode* other, SEqualReducer equal) const {
-    return equal(condition, other->condition) && equal(body, other->body);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(condition);
-    hash_reduce(body);
-  }
-
-  static constexpr const char* _type_key = "tir.While";
-  TVM_DECLARE_FINAL_OBJECT_INFO(WhileNode, StmtNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.While", WhileNode, StmtNode);
 };
 
 /*!
@@ -984,7 +812,7 @@ class While : public Stmt {
  public:
   TVM_DLL While(PrimExpr condition, Stmt body, Span span = Span());
 
-  TVM_DEFINE_OBJECT_REF_METHODS(While, Stmt, WhileNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(While, Stmt, WhileNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(WhileNode);
 };
 
@@ -996,7 +824,7 @@ class BufferRegionNode : public PrimExprConvertibleNode {
   /*! \brief The buffer of the buffer region. */
   Buffer buffer;
   /*! \brief The region array of the buffer region. */
-  Array<Range> region;
+  ffi::Array<Range> region;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -1005,21 +833,10 @@ class BufferRegionNode : public PrimExprConvertibleNode {
         .def_ro("region", &BufferRegionNode::region);
   }
 
-  bool SEqualReduce(const BufferRegionNode* other, SEqualReducer equal) const {
-    return equal(buffer, other->buffer) && equal(region, other->region);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(buffer);
-    hash_reduce(region);
-  }
-
   TVM_DLL PrimExpr ToPrimExpr() const final;
 
-  static constexpr const char* _type_key = "tir.BufferRegion";
-  static constexpr const bool _type_has_method_sequal_reduce = true;
-  static constexpr const bool _type_has_method_shash_reduce = true;
-  TVM_DECLARE_FINAL_OBJECT_INFO(BufferRegionNode, PrimExprConvertibleNode);
+  static constexpr TVMFFISEqHashKind _type_s_eq_hash_kind = kTVMFFISEqHashKindTreeNode;
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.BufferRegion", BufferRegionNode, PrimExprConvertibleNode);
 };
 
 /*!
@@ -1028,7 +845,7 @@ class BufferRegionNode : public PrimExprConvertibleNode {
  */
 class BufferRegion : public PrimExprConvertible {
  public:
-  TVM_DLL explicit BufferRegion(Buffer buffer, Array<Range> region);
+  TVM_DLL explicit BufferRegion(Buffer buffer, ffi::Array<Range> region);
 
   /*!
    * \brief Create a BufferRegion which is full region of the given buffer.
@@ -1043,9 +860,9 @@ class BufferRegion : public PrimExprConvertible {
    * \param indices The access point indices of the buffer
    * \return The BufferRegion which is the single point of the given buffer.
    */
-  TVM_DLL static BufferRegion FromPoint(Buffer buffer, Array<PrimExpr> indices);
+  TVM_DLL static BufferRegion FromPoint(Buffer buffer, ffi::Array<PrimExpr> indices);
 
-  TVM_DEFINE_OBJECT_REF_METHODS(BufferRegion, PrimExprConvertible, BufferRegionNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(BufferRegion, PrimExprConvertible, BufferRegionNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(BufferRegionNode);
 };
 
@@ -1072,19 +889,8 @@ class MatchBufferRegionNode : public Object {
         .def_ro("source", &MatchBufferRegionNode::source);
   }
 
-  bool SEqualReduce(const MatchBufferRegionNode* other, SEqualReducer equal) const {
-    return equal(buffer, other->buffer) && equal(source, other->source);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(buffer);
-    hash_reduce(source);
-  }
-
-  static constexpr const char* _type_key = "tir.MatchBufferRegion";
-  static constexpr const bool _type_has_method_sequal_reduce = true;
-  static constexpr const bool _type_has_method_shash_reduce = true;
-  TVM_DECLARE_FINAL_OBJECT_INFO(MatchBufferRegionNode, Object);
+  static constexpr TVMFFISEqHashKind _type_s_eq_hash_kind = kTVMFFISEqHashKindTreeNode;
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.MatchBufferRegion", MatchBufferRegionNode, Object);
 };
 
 /*!
@@ -1095,7 +901,7 @@ class MatchBufferRegion : public ObjectRef {
  public:
   TVM_DLL explicit MatchBufferRegion(Buffer buffer, BufferRegion source);
 
-  TVM_DEFINE_OBJECT_REF_METHODS(MatchBufferRegion, ObjectRef, MatchBufferRegionNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(MatchBufferRegion, ObjectRef, MatchBufferRegionNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(MatchBufferRegionNode);
 };
 
@@ -1123,15 +929,19 @@ class MatchBufferRegion : public ObjectRef {
 class BlockNode : public StmtNode {
  public:
   /*! \brief The variables of the block. */
-  Array<IterVar> iter_vars;
+  ffi::Array<IterVar> iter_vars;
   /*! \brief The read buffer regions of the block. */
-  Array<BufferRegion> reads;
+  ffi::Array<BufferRegion> reads;
   /*! \brief The write buffer regions of the block. */
-  Array<BufferRegion> writes;
+  ffi::Array<BufferRegion> writes;
   /*! \brief The name_hint of the block. */
-  String name_hint;
-  /*! \brief The body of the block. */
-  Stmt body;
+  ffi::String name_hint;
+  /*! \brief The buffer allocated in the block. */
+  ffi::Array<Buffer> alloc_buffers;
+  /*! \brief The match buffer regions. */
+  ffi::Array<MatchBufferRegion> match_buffers;
+  /*! \brief The annotation of the block. */
+  ffi::Map<ffi::String, ffi::Any> annotations;
   /*!
    * \brief The init statement is executed during the first iteration of reduction loops in a
    *  reduction block. The optional init field allows us to represent initialization and
@@ -1139,50 +949,24 @@ class BlockNode : public StmtNode {
    *  We also provide primitives to decompose the init into a separate block during scheduling.
    *  Init field is `std::nullopt` if there is no reduction iter_vars
    */
-  Optional<Stmt> init;
-  /*! \brief The buffer allocated in the block. */
-  Array<Buffer> alloc_buffers;
-  /*! \brief The match buffer regions. */
-  Array<MatchBufferRegion> match_buffers;
-  /*! \brief The annotation of the block. */
-  Map<String, ffi::Any> annotations;
+  ffi::Optional<Stmt> init;
+  /*! \brief The body of the block. */
+  Stmt body;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
     refl::ObjectDef<BlockNode>()
-        .def_ro("iter_vars", &BlockNode::iter_vars)
+        .def_ro("iter_vars", &BlockNode::iter_vars, refl::AttachFieldFlag::SEqHashDef())
         .def_ro("reads", &BlockNode::reads)
         .def_ro("writes", &BlockNode::writes)
-        .def_ro("name_hint", &BlockNode::name_hint)
-        .def_ro("body", &BlockNode::body)
-        .def_ro("init", &BlockNode::init)
+        .def_ro("name_hint", &BlockNode::name_hint, refl::AttachFieldFlag::SEqHashIgnore())
         .def_ro("alloc_buffers", &BlockNode::alloc_buffers)
         .def_ro("match_buffers", &BlockNode::match_buffers)
-        .def_ro("annotations", &BlockNode::annotations);
+        .def_ro("annotations", &BlockNode::annotations)
+        .def_ro("init", &BlockNode::init)
+        .def_ro("body", &BlockNode::body);
   }
-
-  bool SEqualReduce(const BlockNode* other, SEqualReducer equal) const {
-    // Need first reduce iter_vars, alloc_buffers and match_buffers to define new vars
-    return equal.DefEqual(iter_vars, other->iter_vars) &&
-           equal(alloc_buffers, other->alloc_buffers) &&
-           equal(match_buffers, other->match_buffers) && equal(reads, other->reads) &&
-           equal(writes, other->writes) && equal(body, other->body) && equal(init, other->init) &&
-           equal(annotations, other->annotations);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce.DefHash(iter_vars);
-    hash_reduce(alloc_buffers);
-    hash_reduce(match_buffers);
-    hash_reduce(reads);
-    hash_reduce(writes);
-    hash_reduce(body);
-    hash_reduce(init);
-    hash_reduce(annotations);
-  }
-
-  static constexpr const char* _type_key = "tir.Block";
-  TVM_DECLARE_FINAL_OBJECT_INFO(BlockNode, StmtNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.Block", BlockNode, StmtNode);
 };
 
 /*!
@@ -1191,15 +975,16 @@ class BlockNode : public StmtNode {
  */
 class Block : public Stmt {
  public:
-  TVM_DLL explicit Block(Array<IterVar> iter_vars, Array<BufferRegion> reads,
-                         Array<BufferRegion> writes, String name_hint, Stmt body,
-                         Optional<Stmt> init = std::nullopt,
-                         Array<Buffer> alloc_buffers = Array<Buffer>(),
-                         Array<MatchBufferRegion> match_buffers = Array<MatchBufferRegion>(),
-                         Map<String, ffi::Any> annotations = Map<String, ffi::Any>(),
-                         Span span = Span());
+  TVM_DLL explicit Block(
+      ffi::Array<IterVar> iter_vars, ffi::Array<BufferRegion> reads,
+      ffi::Array<BufferRegion> writes, ffi::String name_hint, Stmt body,
+      ffi::Optional<Stmt> init = std::nullopt,
+      ffi::Array<Buffer> alloc_buffers = ffi::Array<Buffer>(),
+      ffi::Array<MatchBufferRegion> match_buffers = ffi::Array<MatchBufferRegion>(),
+      ffi::Map<ffi::String, ffi::Any> annotations = ffi::Map<ffi::String, ffi::Any>(),
+      Span span = Span());
 
-  TVM_DEFINE_OBJECT_REF_METHODS(Block, Stmt, BlockNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(Block, Stmt, BlockNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(BlockNode);
 };
 
@@ -1209,7 +994,7 @@ class Block : public Stmt {
 class BlockRealizeNode : public StmtNode {
  public:
   /*! \brief The corresponding values of the iter vars. */
-  Array<PrimExpr> iter_values;
+  ffi::Array<PrimExpr> iter_values;
   /*!
    * \brief The predicate of the block realization, the block will only be executed when the
    * predicate is true.
@@ -1225,20 +1010,7 @@ class BlockRealizeNode : public StmtNode {
         .def_ro("predicate", &BlockRealizeNode::predicate)
         .def_ro("block", &BlockRealizeNode::block);
   }
-
-  bool SEqualReduce(const BlockRealizeNode* other, SEqualReducer equal) const {
-    return equal(iter_values, other->iter_values) && equal(predicate, other->predicate) &&
-           equal(block, other->block);
-  }
-
-  void SHashReduce(SHashReducer hash_reduce) const {
-    hash_reduce(iter_values);
-    hash_reduce(predicate);
-    hash_reduce(block);
-  }
-
-  static constexpr const char* _type_key = "tir.BlockRealize";
-  TVM_DECLARE_FINAL_OBJECT_INFO(BlockRealizeNode, StmtNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("tir.BlockRealize", BlockRealizeNode, StmtNode);
 };
 
 /*!
@@ -1247,10 +1019,10 @@ class BlockRealizeNode : public StmtNode {
  */
 class BlockRealize : public Stmt {
  public:
-  TVM_DLL explicit BlockRealize(Array<PrimExpr> iter_values, PrimExpr predicate, Block block,
+  TVM_DLL explicit BlockRealize(ffi::Array<PrimExpr> iter_values, PrimExpr predicate, Block block,
                                 Span span = Span());
 
-  TVM_DEFINE_OBJECT_REF_METHODS(BlockRealize, Stmt, BlockRealizeNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(BlockRealize, Stmt, BlockRealizeNode);
   TVM_DEFINE_OBJECT_REF_COW_METHOD(BlockRealizeNode);
 };
 
@@ -1305,11 +1077,6 @@ constexpr const char* pragma_import_c = "pragma_import_c";
 constexpr const char* pragma_import_llvm = "pragma_import_llvm";
 /*! \brief Try to modify the AST to support Tensor Core */
 constexpr const char* pragma_tensor_core = "pragma_tensor_core";
-/*!
- * \brief Mark of prefetch scope, value=offset,
- *  run prefetch of Tensor on the current loop scope
- */
-constexpr const char* prefetch_scope = "prefetch_scope";
 /*!
  * \brief Marks the layout transforms to be used for a tensor.
  *
@@ -1350,7 +1117,7 @@ constexpr const char* buffer_dim_align = "buffer_dim_align";
 constexpr const char* buffer_bound = "buffer_bound";
 /*!
  * \brief Bind the buffer specification to the region of the op
- *  When this scope occurs, the stmt.node is a Array<NodeRef> = [buffer, tensor]
+ *  When this scope occurs, the stmt.node is a ffi::Array<NodeRef> = [buffer, tensor]
  *  stmt.value is a tvm_tuple(min0, extent0, min1, extent1, ...).
  *  The scope represents that we need to bind the storage region of tensor to buffer.
  *  This will affect replacement of some variables inside the scope that
@@ -1543,6 +1310,9 @@ constexpr const char* explicit_read_region = "explicit_read_region";
  */
 constexpr const char* explicit_write_region = "explicit_write_region";
 
+/*! \brief ,ark a ForNode represent an irregular loop of non-structural control flow edges. */
+constexpr const char* irregular_loop_mark = "irregular_loop_mark";
+
 /*!
  * \brief Check if attr_key is a pragma key extension
  * \param attr_key The attr key to be compared
