@@ -48,8 +48,8 @@ static inline DataType CommonType(DataType type1, DataType type2) {
   return DataType(type1.code(), std::max(type1.bits(), type2.bits()), /*lanes=*/1);
 }
 
-inline BroadcastHelper BroadcastShape(const tvm::Array<tvm::PrimExpr>& shape1,
-                                      const tvm::Array<tvm::PrimExpr>& shape2) {
+inline BroadcastHelper BroadcastShape(const tvm::ffi::Array<tvm::PrimExpr>& shape1,
+                                      const tvm::ffi::Array<tvm::PrimExpr>& shape2) {
   BroadcastHelper bh;
   int s1_size = shape1.size();
   int s2_size = shape2.size();
@@ -94,8 +94,8 @@ inline BroadcastHelper BroadcastShape(const tvm::Array<tvm::PrimExpr>& shape1,
     } else {
       ICHECK(false) << "Incompatible broadcast dims: " << shape1[s1_size - i] << " and "
                     << shape2[s2_size - i]
-                    << " in: " << tvm::Array<tvm::PrimExpr>(shape1.begin(), shape1.end()) << " and "
-                    << tvm::Array<tvm::PrimExpr>(shape2.begin(), shape2.end());
+                    << " in: " << tvm::ffi::Array<tvm::PrimExpr>(shape1.begin(), shape1.end())
+                    << " and " << tvm::ffi::Array<tvm::PrimExpr>(shape2.begin(), shape2.end());
     }
   }
   // Remaining dimensions whether on shape1 or shape2 can always be completed
@@ -110,10 +110,10 @@ inline BroadcastHelper BroadcastShape(const tvm::Array<tvm::PrimExpr>& shape1,
   return bh;
 }
 
-inline tvm::Array<tvm::PrimExpr> InputIndexFromBroadcast(
-    const tvm::Array<tvm::tir::Var>& ovars, const tvm::te::Tensor& T,
+inline tvm::ffi::Array<tvm::PrimExpr> InputIndexFromBroadcast(
+    const tvm::ffi::Array<tvm::tir::Var>& ovars, const tvm::te::Tensor& T,
     const std::deque<tvm::tir::Var>& my_vars, const std::deque<tvm::tir::Var>& all_vars) {
-  tvm::Array<tvm::PrimExpr> ivars;
+  tvm::ffi::Array<tvm::PrimExpr> ivars;
   ICHECK_EQ(ovars.size(), all_vars.size());
   // N^2, could use a map but NBD.
   size_t expected_dims = T->shape.size();
@@ -141,12 +141,12 @@ inline tvm::te::Tensor WithBroadcast(FBinaryExpr op, const tvm::te::Tensor& A,
                                      const tvm::te::Tensor& B, const std::string& name = "tensor",
                                      const std::string& tag = "") {
   auto bh = BroadcastShape(A->shape, B->shape);
-  auto l = [&](tvm::Array<tvm::tir::Var> ovars) {
+  auto l = [&](tvm::ffi::Array<tvm::tir::Var> ovars) {
     return op(A(InputIndexFromBroadcast(ovars, A, bh.vars1, bh.all_vars)),
               B(InputIndexFromBroadcast(ovars, B, bh.vars2, bh.all_vars)));
   };
-  return tvm::te::compute(tvm::Array<tvm::PrimExpr>(bh.common_shape.begin(), bh.common_shape.end()),
-                          l, name, tag);
+  return tvm::te::compute(
+      tvm::ffi::Array<tvm::PrimExpr>(bh.common_shape.begin(), bh.common_shape.end()), l, name, tag);
 }
 
 }  // namespace detail
