@@ -56,7 +56,7 @@ inline tvm::te::Tensor relu(const tvm::te::Tensor& t, T threshold = static_cast<
                             std::string name = "T_relu", std::string tag = kElementWise) {
   return tvm::te::compute(
       t->shape,
-      [&](const tvm::Array<tvm::tir::Var>& i) {
+      [&](const tvm::ffi::Array<tvm::tir::Var>& i) {
         auto threshold_const = tvm::tir::make_const(t->dtype, threshold);
         return tvm::max(t(i), threshold_const);
       },
@@ -78,7 +78,7 @@ inline tvm::te::Tensor leaky_relu(const tvm::te::Tensor& t, double alpha = 0.1,
                                   std::string tag = kElementWise) {
   return tvm::te::compute(
       t->shape,
-      [&](const tvm::Array<tvm::tir::Var>& i) {
+      [&](const tvm::ffi::Array<tvm::tir::Var>& i) {
         auto value = t(i);
         auto calpha = tvm::tir::make_const(value.dtype(), alpha);
         return tvm::tir::Select(value > 0, value, value * calpha);
@@ -106,7 +106,7 @@ inline tvm::te::Tensor prelu(const tvm::te::Tensor& x, const tvm::te::Tensor& sl
 
   return tvm::te::compute(
       x->shape,
-      [&](const tvm::Array<tvm::tir::Var>& indices) {
+      [&](const tvm::ffi::Array<tvm::tir::Var>& indices) {
         auto xval = x(indices);
         return tvm::tir::Select(xval > 0, xval, xval * slope(indices[axis]));
       },
@@ -152,11 +152,11 @@ inline tvm::te::Tensor prelu(const tvm::te::Tensor& x, const tvm::te::Tensor& sl
  *
  *
  */
-inline tvm::te::Tensor pad(const tvm::te::Tensor& t, const tvm::Array<tvm::PrimExpr>& pad_before,
-                           tvm::Array<tvm::PrimExpr> pad_after = tvm::Array<tvm::PrimExpr>(),
-                           PrimExpr pad_value = PrimExpr(), std::string name = "T_pad",
-                           std::string tag = kElementWise, std::string pad_mode = "constant",
-                           const Array<PrimExpr>* dyn_output_shape = nullptr) {
+inline tvm::te::Tensor pad(
+    const tvm::te::Tensor& t, const tvm::ffi::Array<tvm::PrimExpr>& pad_before,
+    tvm::ffi::Array<tvm::PrimExpr> pad_after = tvm::ffi::Array<tvm::PrimExpr>(),
+    PrimExpr pad_value = PrimExpr(), std::string name = "T_pad", std::string tag = kElementWise,
+    std::string pad_mode = "constant", const ffi::Array<PrimExpr>* dyn_output_shape = nullptr) {
   if (pad_after.size() < pad_before.size()) {
     for (size_t i = pad_after.size(); i < pad_before.size(); ++i) {
       pad_after.push_back(pad_before[i]);
@@ -166,8 +166,8 @@ inline tvm::te::Tensor pad(const tvm::te::Tensor& t, const tvm::Array<tvm::PrimE
   arith::Analyzer analyzer;
   ICHECK_GE(pad_before.size(), 1);
   ICHECK_EQ(pad_before.size(), pad_after.size());
-  tvm::Array<tvm::PrimExpr> pad_before_int32;
-  tvm::Array<tvm::PrimExpr> pad_after_int32;
+  tvm::ffi::Array<tvm::PrimExpr> pad_before_int32;
+  tvm::ffi::Array<tvm::PrimExpr> pad_after_int32;
 
   for (const auto& ele : pad_before) {
     pad_before_int32.push_back(tvm::cast(tvm::DataType::Int(32), ele));
@@ -176,7 +176,7 @@ inline tvm::te::Tensor pad(const tvm::te::Tensor& t, const tvm::Array<tvm::PrimE
     pad_after_int32.push_back(tvm::cast(tvm::DataType::Int(32), ele));
   }
 
-  tvm::Array<tvm::PrimExpr> output_shape;
+  tvm::ffi::Array<tvm::PrimExpr> output_shape;
   if (dyn_output_shape == nullptr) {
     for (size_t i = 0; i < t->shape.size(); ++i) {
       if (i >= pad_before.size()) {
@@ -196,10 +196,10 @@ inline tvm::te::Tensor pad(const tvm::te::Tensor& t, const tvm::Array<tvm::PrimE
     pad_value = tvm::tir::make_const(t->dtype, 0);
   }
 
-  auto l = [&](tvm::Array<tvm::tir::Var> ovars) {
-    tvm::Array<tvm::PrimExpr> indices;
-    tvm::Array<tvm::PrimExpr> sel;
-    tvm::Array<tvm::PrimExpr> pad_idx;
+  auto l = [&](tvm::ffi::Array<tvm::tir::Var> ovars) {
+    tvm::ffi::Array<tvm::PrimExpr> indices;
+    tvm::ffi::Array<tvm::PrimExpr> sel;
+    tvm::ffi::Array<tvm::PrimExpr> pad_idx;
     for (size_t i = 0; i < t->shape.size(); ++i) {
       if (i >= pad_before_int32.size()) {
         indices.push_back(ovars[i]);
@@ -273,7 +273,7 @@ inline tvm::te::Tensor conv2d_nchw(const tvm::te::Tensor& I, const tvm::te::Tens
   ICHECK_EQ(4, W->shape.size());
   auto pH = I->shape[2];
   auto pW = I->shape[3];
-  tvm::Array<tvm::PrimExpr> output_shape{
+  tvm::ffi::Array<tvm::PrimExpr> output_shape{
       I->shape[0],                                                    // B
       W->shape[0],                                                    // O
       indexdiv(I->shape[2] - W->shape[2] + 2 * pad_h, stride_h) + 1,  // H
@@ -317,7 +317,7 @@ inline tvm::te::Tensor conv2d_hwcn(const tvm::te::Tensor& I, const tvm::te::Tens
   ICHECK_EQ(4, W->shape.size());
   auto pH = I->shape[2];
   auto pW = I->shape[3];
-  tvm::Array<tvm::PrimExpr> output_shape{
+  tvm::ffi::Array<tvm::PrimExpr> output_shape{
       indexdiv(I->shape[2] - W->shape[2] + 2 * pad_h, stride_h) + 1,  // H
       indexdiv(I->shape[3] - W->shape[3] + 2 * pad_w, stride_w) + 1,  // W
       I->shape[2],                                                    // B
@@ -363,7 +363,7 @@ inline tvm::te::Tensor depthwise_conv2d_nchw(const tvm::te::Tensor& I, const tvm
   auto pH = I->shape[2];
   auto pW = I->shape[3];
   auto pCM = W->shape[1];  // channel_multiplier
-  tvm::Array<tvm::PrimExpr> output_shape{
+  tvm::ffi::Array<tvm::PrimExpr> output_shape{
       I->shape[0],                                                    // B
       W->shape[1],                                                    // O
       indexdiv(I->shape[2] - W->shape[2] + 2 * pad_h, stride_h) + 1,  // H
@@ -392,7 +392,7 @@ inline tvm::te::Tensor depthwise_conv2d_nhwc(const tvm::te::Tensor& I, const tvm
   auto pH = I->shape[1];
   auto pW = I->shape[2];
   auto pCM = W->shape[1];  // channel_multiplier
-  tvm::Array<tvm::PrimExpr> output_shape{
+  tvm::ffi::Array<tvm::PrimExpr> output_shape{
       I->shape[0],                                                    // B
       indexdiv(I->shape[1] - W->shape[1] + 2 * pad_h, stride_h) + 1,  // H
       indexdiv(I->shape[2] - W->shape[2] + 2 * pad_w, stride_w) + 1,  // W
@@ -440,7 +440,7 @@ inline tvm::te::Tensor group_conv2d_ngchw(const tvm::te::Tensor& I, const tvm::t
   ICHECK_EQ(5, W->shape.size());
   auto pH = I->shape[2];
   auto pW = I->shape[3];
-  tvm::Array<tvm::PrimExpr> output_shape{
+  tvm::ffi::Array<tvm::PrimExpr> output_shape{
       I->shape[0],                                                    // B
       I->shape[1],                                                    // G
       W->shape[2],                                                    // O
@@ -454,7 +454,7 @@ inline tvm::te::Tensor group_conv2d_ngchw(const tvm::te::Tensor& I, const tvm::t
   auto T = (pad_h == 0 && pad_w == 0)
                ? I
                : pad(I, {tvm::PrimExpr(0), tvm::PrimExpr(0), tvm::PrimExpr(0), pad_h, pad_w});
-  auto l = [&](tvm::Array<tvm::tir::Var> args) {
+  auto l = [&](tvm::ffi::Array<tvm::tir::Var> args) {
     tvm::tir::Var b = args[0];
     tvm::tir::Var g = args[1];
     tvm::tir::Var o = args[2];
@@ -480,9 +480,9 @@ inline tvm::te::Tensor group_conv2d_ngchw(const tvm::te::Tensor& I, const tvm::t
  * \return A Tensor whose op member is the space_to_batch_nd operation
  */
 inline tvm::te::Tensor space_to_batch_nd(const tvm::te::Tensor& data,
-                                         const tvm::Array<Integer>& block_shape,
-                                         const tvm::Array<tvm::PrimExpr>& pad_before,
-                                         const tvm::Array<tvm::PrimExpr>& pad_after,
+                                         const tvm::ffi::Array<Integer>& block_shape,
+                                         const tvm::ffi::Array<tvm::PrimExpr>& pad_before,
+                                         const tvm::ffi::Array<tvm::PrimExpr>& pad_after,
                                          PrimExpr pad_value = PrimExpr(),
                                          std::string name = "space_to_batch_nd",
                                          std::string tag = kInjective) {
@@ -490,8 +490,8 @@ inline tvm::te::Tensor space_to_batch_nd(const tvm::te::Tensor& data,
   CHECK_EQ(pad_before.size(), pad_after.size());
   CHECK_EQ(block_shape.size(), pad_before.size())
       << "Paddings must be provided for each spatial dimension";
-  tvm::Array<tvm::PrimExpr> pad_before_int32;
-  tvm::Array<tvm::PrimExpr> pad_after_int32;
+  tvm::ffi::Array<tvm::PrimExpr> pad_before_int32;
+  tvm::ffi::Array<tvm::PrimExpr> pad_after_int32;
 
   // pad size for batch dimension is 0
   pad_before_int32.push_back(tvm::cast(tvm::DataType::Int(32), 0));
@@ -514,9 +514,9 @@ inline tvm::te::Tensor space_to_batch_nd(const tvm::te::Tensor& data,
   auto padded_shape = padded_t->shape;
 
   // infer shapes
-  tvm::Array<PrimExpr> r_shape;
-  tvm::Array<Integer> axis;
-  tvm::Array<PrimExpr> o_shape;
+  tvm::ffi::Array<PrimExpr> r_shape;
+  tvm::ffi::Array<Integer> axis;
+  tvm::ffi::Array<PrimExpr> o_shape;
 
   size_t num_block_dims = block_shape.size();
   int batch = static_cast<int>(GetConstInt(input_shape[0]));
@@ -576,15 +576,15 @@ inline tvm::te::Tensor space_to_batch_nd(const tvm::te::Tensor& data,
  * \return A Tensor whose op member is the batch_to_space_nd operation
  */
 inline tvm::te::Tensor batch_to_space_nd(const tvm::te::Tensor& data,
-                                         const tvm::Array<Integer>& block_shape,
-                                         const tvm::Array<tvm::PrimExpr>& crop_begin_list,
-                                         const tvm::Array<tvm::PrimExpr>& crop_end_list,
+                                         const tvm::ffi::Array<Integer>& block_shape,
+                                         const tvm::ffi::Array<tvm::PrimExpr>& crop_begin_list,
+                                         const tvm::ffi::Array<tvm::PrimExpr>& crop_end_list,
                                          std::string name = "batch_to_space_nd",
                                          std::string tag = kInjective) {
   // Construct shapes for reshape and transpose operation
-  Array<PrimExpr> in_shape = data->shape;
-  Array<PrimExpr> r_shape;
-  Array<Integer> axis;
+  ffi::Array<PrimExpr> in_shape = data->shape;
+  ffi::Array<PrimExpr> r_shape;
+  ffi::Array<Integer> axis;
   size_t num_block_dims = block_shape.size();
   size_t num_input_dims = in_shape.size();
   tvm::PrimExpr block_shape_prod(1);
@@ -605,7 +605,7 @@ inline tvm::te::Tensor batch_to_space_nd(const tvm::te::Tensor& data,
     r_shape.push_back(in_shape[i]);
   }
 
-  Array<PrimExpr> r_p_shape;
+  ffi::Array<PrimExpr> r_p_shape;
   r_p_shape.push_back(batch / block_shape_prod);
   for (size_t i = 1; i <= num_block_dims; i++) {
     r_p_shape.push_back(in_shape[i] * block_shape[i - 1]);
@@ -620,7 +620,7 @@ inline tvm::te::Tensor batch_to_space_nd(const tvm::te::Tensor& data,
   out = reshape(out, r_p_shape);
 
   // Crop the start and end of dimensions of out
-  Array<Integer> begin_idx, end_idx, strides;
+  ffi::Array<Integer> begin_idx, end_idx, strides;
   for (size_t i = 0; i < r_p_shape.size(); ++i) {
     strides.push_back(Integer(1));
     if (i > 0 && i <= num_block_dims) {
@@ -665,7 +665,7 @@ inline Tensor nll_loss(const Tensor& predictions, const Tensor& targets, const T
     // prediction->shape = (C,), targets->shape = (), weights->shape = (C,)
     auto T = tvm::te::compute(
         {},
-        [&](const tvm::Array<tvm::tir::Var>& target_indices) {
+        [&](const tvm::ffi::Array<tvm::tir::Var>& target_indices) {
           auto c = targets();
           return tvm::tir::Select(c != ignore_index, -predictions(c) * weights(c),
                                   tvm::tir::make_const(predictions->dtype, 0));
@@ -674,7 +674,7 @@ inline Tensor nll_loss(const Tensor& predictions, const Tensor& targets, const T
     if (reduction == "mean") {
       auto W = tvm::te::compute(
           {},
-          [&](const tvm::Array<tvm::tir::Var>& target_indices) {
+          [&](const tvm::ffi::Array<tvm::tir::Var>& target_indices) {
             auto c = targets();
             return tvm::tir::Select(c != ignore_index, weights(c),
                                     tvm::tir::make_const(predictions->dtype, 0));
@@ -687,9 +687,9 @@ inline Tensor nll_loss(const Tensor& predictions, const Tensor& targets, const T
   }
   auto T = tvm::te::compute(
       targets->shape,
-      [&](const tvm::Array<tvm::tir::Var>& target_indices) {
+      [&](const tvm::ffi::Array<tvm::tir::Var>& target_indices) {
         auto c = targets(target_indices);
-        tvm::Array<tvm::PrimExpr> pred_indices;
+        tvm::ffi::Array<tvm::PrimExpr> pred_indices;
         pred_indices.push_back(target_indices[0]);  // batch index
         pred_indices.push_back(c);                  // class index
         for (size_t i = 1; i < target_indices.size(); i++) {
@@ -703,16 +703,16 @@ inline Tensor nll_loss(const Tensor& predictions, const Tensor& targets, const T
   if (reduction == "mean") {
     auto W = tvm::te::compute(
         targets->shape,
-        [&](const tvm::Array<tvm::tir::Var>& target_indices) {
+        [&](const tvm::ffi::Array<tvm::tir::Var>& target_indices) {
           auto c = targets(target_indices);
           return tvm::tir::Select(c != ignore_index, weights(c),
                                   tvm::tir::make_const(predictions->dtype, 0));
         },
         name, tag);
-    return topi::divide(topi::sum(T, tvm::Array<Integer>(nullptr)),
-                        topi::sum(W, tvm::Array<Integer>(nullptr)));
+    return topi::divide(topi::sum(T, tvm::ffi::Array<Integer>(nullptr)),
+                        topi::sum(W, tvm::ffi::Array<Integer>(nullptr)));
   } else if (reduction == "sum") {
-    return topi::sum(T, tvm::Array<Integer>(nullptr));
+    return topi::sum(T, tvm::ffi::Array<Integer>(nullptr));
   } else {  // reduction == "none"
     return T;
   }
