@@ -15,6 +15,6 @@
 # specific language governing permissions and limitations
 # under the License.
 """FFI API for CUTLASS BYOC."""
-import tvm.ffi
+import tvm_ffi
 
-tvm.ffi._init_api("contrib.cutlass", __name__)
+tvm_ffi.init_ffi_api("contrib.cutlass", __name__)
