@@ -129,7 +129,7 @@ class CodeGen(object):
 
 def to_relax(
     graph: MSCGraph,
-    weights: Optional[Dict[str, tvm.nd.array]] = None,
+    weights: Optional[Dict[str, tvm.runtime.Tensor]] = None,
     codegen_config: Optional[Dict[str, str]] = None,
     print_config: Optional[Dict[str, str]] = None,
     build_folder: msc_utils.MSCDirectory = None,
