@@ -22,11 +22,9 @@ from functools import partial
 
 import tvm
 from tvm.relax.dpl import pattern as relax_pattern
-from tvm.relay import dataflow_pattern as relay_pattern
 
 from tvm.relax.transform import PatternCheckContext
 from tvm.relax.backend.pattern_registry import register_patterns
-from tvm.relay.op.contrib.register import register_pattern_table
 from tvm.contrib.msc.core.utils.namespace import MSCMap, MSCKey
 from tvm.contrib.msc.core import utils as msc_utils
 from tvm.contrib.msc.core import _ffi_api
@@ -330,7 +328,8 @@ def make_relax_attention_pattern() -> (
     q_trans = relax_pattern.is_op("relax.permute_dims")(weight_q)
     k_trans = relax_pattern.is_op("relax.permute_dims")(weight_k)
     v_trans = relax_pattern.is_op("relax.permute_dims")(weight_v)
-    out = relax_pattern.is_op("relax.nn.attention")(q_trans, k_trans, v_trans)
+    attention = relax_pattern.is_op("relax.nn.attention")(q_trans, k_trans, v_trans)
+    out = relax_pattern.is_op("relax.permute_dims")(attention)
     annotations = {
         "weight_q": weight_q,
         "weight_k": weight_k,
@@ -338,7 +337,8 @@ def make_relax_attention_pattern() -> (
         "q_trans": q_trans,
         "k_trans": k_trans,
         "v_trans": v_trans,
-        "attention": out,
+        "attention": attention,
+        "out": out,
     }
     return out, annotations
 
@@ -378,7 +378,8 @@ def make_relax_mask_attention_pattern() -> (
     q_trans = relax_pattern.is_op("relax.permute_dims")(weight_q)
     k_trans = relax_pattern.is_op("relax.permute_dims")(weight_k)
     v_trans = relax_pattern.is_op("relax.permute_dims")(weight_v)
-    out = relax_pattern.is_op("relax.nn.attention_bias")(q_trans, k_trans, v_trans, mask)
+    attention = relax_pattern.is_op("relax.nn.attention_bias")(q_trans, k_trans, v_trans, mask)
+    out = relax_pattern.is_op("relax.permute_dims")(attention)
     annotations = {
         "weight_q": weight_q,
         "weight_k": weight_k,
@@ -387,7 +388,8 @@ def make_relax_mask_attention_pattern() -> (
         "q_trans": q_trans,
         "k_trans": k_trans,
         "v_trans": v_trans,
-        "attention": out,
+        "attention": attention,
+        "out": out,
     }
     return out, annotations
 
@@ -617,258 +619,3 @@ register_patterns(
         ),
     ]
 )
-
-
-# TODO(tong.meng): support patterns after optimize
-@register_pattern_table("msc")
-def pattern_table():
-    """Returns list of triples describing the name, dataflow pattern and predicate for all
-    the MSC-supported operators."""
-
-    def make_relay_conv_bias_pattern(
-        op_name: str, optimized: bool = False
-    ) -> relay_pattern.DFPattern:
-        """A simple utility to create patterns for an operation fused with bias.
-
-        Parameters
-        ----------
-        op_name: str
-            The name of a Relay op, such as "relay.nn.conv2d"
-        optimized: bool
-            Whether the relay is optimized
-
-        Returns
-        -------
-        pattern: tvm.relay.dataflow_pattern.DFPattern
-            The resulting pattern describing a conv_bias operation
-        """
-
-        data = relay_pattern.wildcard()
-        weight = relay_pattern.is_constant()
-        bias = relay_pattern.is_constant()
-        conv = relay_pattern.is_op(op_name)(data, weight)
-        if optimized:
-            out = relay_pattern.is_op("add")(conv, bias)
-        else:
-            out = relay_pattern.is_op("nn.bias_add")(conv, bias)
-        return out
-
-    def _check_relay_conv_bias(call: tvm.relay.Expr) -> bool:
-        """Check if conv_bias fuse pattern is correct.
-
-        Returns
-        -------
-        pass: bool
-            Whether the pattern is correct.
-        """
-
-        if call.op.name == "nn.bias_add":
-            bias = call.args[1]
-            return len(bias.checked_type.shape) == 1
-        if call.op.name == "add":
-            return True
-        return False
-
-    def make_relay_linear_pattern(optimized: bool = False) -> relay_pattern.DFPattern:
-        """A simple utility to create patterns for linear.
-
-        Parameters
-        ----------
-        optimized: bool
-            Whether the relay is optimized
-
-        Returns
-        -------
-        pattern: tvm.relay.dataflow_pattern.DFPattern
-            The resulting pattern describing a linear operation
-        """
-
-        if optimized:
-            data = relay_pattern.wildcard()
-            weight = relay_pattern.is_constant()
-            broadcast_data = relay_pattern.is_op("broadcast_to")(data)
-            reshape_data = relay_pattern.is_op("reshape")(broadcast_data)
-            batch_matmul = relay_pattern.is_op("nn.batch_matmul")(reshape_data, weight)
-            reshape_out = relay_pattern.is_op("reshape")(batch_matmul)
-            return relay_pattern.is_op("squeeze")(reshape_out)
-        data = relay_pattern.wildcard()
-        weight = relay_pattern.is_constant()
-        trans_weight = relay_pattern.is_op("transpose")(weight)
-        broadcast_data = relay_pattern.is_op("broadcast_to")(data)
-        broadcast_weight = relay_pattern.is_op("broadcast_to")(trans_weight)
-        reshape_data = relay_pattern.is_op("reshape")(broadcast_data)
-        reshape_weight = relay_pattern.is_op("reshape")(broadcast_weight)
-        batch_matmul = relay_pattern.is_op("nn.batch_matmul")(reshape_data, reshape_weight)
-        reshape_out = relay_pattern.is_op("reshape")(batch_matmul)
-        return relay_pattern.is_op("squeeze")(reshape_out)
-
-    def _check_relay_linear(call: tvm.relay.Expr) -> bool:
-        """Check if linear pattern is correct.
-
-        Returns
-        -------
-        pass: bool
-            Whether the pattern is correct.
-        """
-
-        return True
-
-    def make_relay_linear_bias_pattern(optimized: bool = False) -> relay_pattern.DFPattern:
-        """A simple utility to create patterns for linear_bias.
-
-        Parameters
-        ----------
-        optimized: bool
-            Whether the relay is optimized
-
-        Returns
-        -------
-        pattern: DFPattern
-            The resulting pattern describing a linear_bias operation
-        """
-
-        bias = relay_pattern.is_constant()
-        linear = make_relay_linear_pattern(optimized)
-        if optimized:
-            out = relay_pattern.is_op("add")(linear, bias)
-        else:
-            out = relay_pattern.is_op("nn.bias_add")(linear, bias)
-        return out
-
-    def _check_relay_linear_bias(call: tvm.relay.Expr) -> bool:
-        """Check if linear_bias pattern is correct."""
-        return True
-
-    def make_relay_matmul_pattern(dim: int = 2, optimized: bool = False) -> relay_pattern.DFPattern:
-        """A simple utility to create patterns for matmul.
-
-        Parameters
-        ----------
-        optimized: bool
-            Whether the relay is optimized
-
-        Returns
-        -------
-        pattern: tvm.relay.dataflow_pattern.DFPattern
-            The resulting pattern describing a matmul operation
-        """
-
-        if dim == 2:
-            a = relay_pattern.wildcard()
-            b = relay_pattern.wildcard()
-            trans_b = relay_pattern.is_op("transpose")(b)
-            dense = relay_pattern.is_op("nn.dense")(a, trans_b)
-            return dense | relay_pattern.is_op("squeeze")(dense)
-        elif dim == 3:
-            a = relay_pattern.wildcard()
-            b = relay_pattern.wildcard()
-            broadcast_a = relay_pattern.is_op("broadcast_to")(a)
-            broadcast_b = relay_pattern.is_op("broadcast_to")(b)
-            reshape_a = relay_pattern.is_op("reshape")(broadcast_a)
-            reshape_b = relay_pattern.is_op("reshape")(broadcast_b)
-            batch_matmul = relay_pattern.is_op("nn.batch_matmul")(reshape_a, reshape_b)
-            reshape_out = relay_pattern.is_op("reshape")(batch_matmul)
-            return relay_pattern.is_op("squeeze")(reshape_out)
-        else:
-            raise Exception("matmul pattern only support dim 2 and 3")
-
-    def _check_relay_matmul(call: tvm.relay.Expr) -> bool:
-        """Check if matmul pattern is correct.
-
-        Returns
-        -------
-        pass: bool
-            Whether the pattern is correct.
-        """
-
-        last_call = call.args[0] if call.op.name == "squeeze" else call
-        if last_call.op.name == "nn.dense":
-            trans_b = last_call.args[1]
-            b = trans_b.args[0]
-            if len(b.checked_type.shape) != 2:
-                return False
-            return trans_b.attrs["axes"] is None or list(trans_b.attrs["axes"]) == [1, 0]
-        return True
-
-    def make_relay_embedding_pattern(optimized: bool = False) -> relay_pattern.DFPattern:
-        """A simple utility to create patterns for 1d embedding.
-
-        Returns
-        -------
-        pattern: tvm.relay.dataflow_pattern.DFPattern
-            The resulting pattern describing a embedding operation
-        """
-
-        weight = relay_pattern.is_constant()
-        data = relay_pattern.wildcard()
-        astype = relay_pattern.is_op("cast")(data)
-        return relay_pattern.is_op("take")(weight, astype)
-
-    def _check_relay_embedding(call: tvm.relay.Expr) -> bool:
-        """Check if embedding pattern is correct.
-
-        Returns
-        -------
-        pass: bool
-            Whether the pattern is correct.
-        """
-
-        weight = call.args[0]
-        cast = call.args[1]
-        return (
-            cast.attrs["dtype"] == "int32"
-            and len(weight.checked_type.shape) == 2
-            and weight.checked_type.dtype == "float32"
-        )
-
-    def make_relay_gelu_pattern(optimized: bool = False) -> relay_pattern.DFPattern:
-        """A simple utility to create patterns for gelu.
-
-        Returns
-        -------
-        pattern: tvm.relay.dataflow_pattern.DFPattern
-            The resulting pattern describing a gelu operation.
-        """
-
-        data = relay_pattern.wildcard()
-        factor_1 = relay_pattern.is_constant()
-        mul_1 = relay_pattern.is_op("multiply")(data, factor_1)
-        erf = relay_pattern.is_op("erf")(mul_1)
-        factor_2 = relay_pattern.is_constant()
-        mul_2 = relay_pattern.is_op("multiply")(erf, factor_2)
-        factor_3 = relay_pattern.is_constant()
-        add = relay_pattern.is_op("add")(factor_3, mul_2)
-        return relay_pattern.is_op("multiply")(data, add)
-
-    def _check_relay_gelu(call: tvm.relay.Expr) -> bool:
-        """Check if gelu pattern is correct.
-
-        Returns
-        -------
-        pass: bool
-            Whether the pattern is correct.
-        """
-
-        return True
-
-    return [
-        ("msc.conv1d_bias", make_relay_conv_bias_pattern("nn.conv1d"), _check_relay_conv_bias),
-        (
-            "msc.conv1d_bias",
-            make_relay_conv_bias_pattern("nn.conv1d", True),
-            _check_relay_conv_bias,
-        ),
-        ("msc.conv2d_bias", make_relay_conv_bias_pattern("nn.conv2d"), _check_relay_conv_bias),
-        (
-            "msc.conv2d_bias",
-            make_relay_conv_bias_pattern("nn.conv2d", True),
-            _check_relay_conv_bias,
-        ),
-        ("msc.linear_bias", make_relay_linear_bias_pattern(), _check_relay_linear_bias),
-        ("msc.linear", make_relay_linear_pattern(), _check_relay_linear),
-        ("msc.linear", make_relay_linear_pattern(True), _check_relay_linear),
-        ("msc.matmul", make_relay_matmul_pattern(dim=2), _check_relay_matmul),
-        ("msc.matmul", make_relay_matmul_pattern(dim=3), _check_relay_matmul),
-        ("msc.embedding", make_relay_embedding_pattern(), _check_relay_embedding),
-        ("msc.gelu", make_relay_gelu_pattern(), _check_relay_gelu),
-    ]
