@@ -16,6 +16,6 @@
 # under the License.
 """tvm.contrib.msc.framework.tensorrt._ffi_api"""
 
-import tvm.ffi
+import tvm_ffi
 
-tvm.ffi._init_api("msc.framework.tensorrt", __name__)
+tvm_ffi.init_ffi_api("msc.framework.tensorrt", __name__)
