@@ -25,7 +25,7 @@ from typing import Dict
 import tempfile
 from pathlib import Path
 
-from ..ffi import register_func
+from tvm_ffi import register_global_func
 from ..base import py_str
 from . import utils as _utils, tar as _tar, cc as _cc
 from .cc import get_target_by_dump_machine
@@ -157,7 +157,7 @@ def get_global_symbol_section_map(path, *, nm=None) -> Dict[str, str]:
     return _cc.get_global_symbol_section_map(path, nm=nm)
 
 
-@register_func("meta_schedule.builder.export_ndk")
+@register_global_func("meta_schedule.builder.export_ndk")
 def _ndk_export(mod):
     tmp_dir = tempfile.mkdtemp()
     binary_name = "tmp_binary.so"
