@@ -14,5 +14,7 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+# pylint: disable=redefined-builtin
+
 """Namespace for driver APIs"""
-from .build_module import lower, build
+from .build_module import build, compile
