@@ -18,8 +18,6 @@
 """Common data structures across all IR variants."""
 
 from . import diagnostics, instrument, transform
-from .adt import Constructor, TypeData
-from .affine_type import TensorAffineType, TupleAffineType
 from .attrs import Attrs, DictAttrs, make_node
 from .base import (
     EnvFunc,
@@ -34,33 +32,17 @@ from .base import (
     structural_hash,
 )
 from .container import Array, Map
-from .expr import BaseExpr, GlobalVar, PrimExpr, Range, RelayExpr
+from .expr import BaseExpr, GlobalVar, PrimExpr, Range, RelaxExpr
 from .function import BaseFunc, CallingConv
 from .global_info import GlobalInfo, DummyGlobalInfo, VDevice
-from .memory_pools import (
-    ConstantMemoryPools,
-    ConstantPoolInfo,
-    PoolInfo,
-    PoolInfoProperties,
-    WorkspaceMemoryPools,
-    WorkspacePoolInfo,
-)
 from .module import IRModule
 from .op import Op, register_intrin_lowering, register_op_attr
-from .tensor_type import TensorType
 from .type import (
     FuncType,
-    GlobalTypeVar,
-    IncompleteType,
     PointerType,
     PrimType,
-    RelayRefType,
     TupleType,
     Type,
-    TypeConstraint,
-    TypeKind,
-    TypeVar,
 )
-from .type_relation import TypeCall, TypeRelation
 
 from . import analysis
