@@ -15,6 +15,6 @@
 # specific language governing permissions and limitations
 # under the License.
 """Additional container data structures used across IR variants."""
-from tvm.ffi import Array, Map
+from tvm_ffi import Array, Map
 
 __all__ = ["Array", "Map"]
