@@ -15,13 +15,13 @@
 # specific language governing permissions and limitations
 # under the License.
 """Type relation and function for type checking."""
-import tvm.ffi
+import tvm_ffi
 
 from .type import Type, TypeConstraint
 from . import _ffi_api
 
 
-@tvm.ffi.register_object("TypeCall")
+@tvm_ffi.register_object("TypeCall")
 class TypeCall(Type):
     """Type function application.
 
@@ -43,7 +43,7 @@ class TypeCall(Type):
         self.__init_handle_by_constructor__(_ffi_api.TypeCall, func, args)
 
 
-@tvm.ffi.register_object("TypeRelation")
+@tvm_ffi.register_object("TypeRelation")
 class TypeRelation(TypeConstraint):
     """User defined type relation, it is an input-output relation on types.
 
