@@ -17,7 +17,7 @@
 """A database for injecting handcrafted schedule functions."""
 from typing import Callable
 
-from tvm.ffi import register_object
+from tvm_ffi import register_object
 from tvm.tir import Schedule
 
 from .. import _ffi_api
@@ -37,10 +37,10 @@ class ScheduleFnDatabase(Database):
         A string to specify the module equality testing and hashing method.
         It must be one of the followings:
           - "structural": Use StructuralEqual/Hash
-          - "ignore-ndarray": Same as "structural", but ignore ndarray raw data during
+          - "ignore-tensor": Same as "structural", but ignore tensor raw data during
                               equality testing and hashing.
           - "anchor-block": Apply equality testing and hashing on the anchor block extracted from a
-                            given module. The "ignore-ndarray" varint is used for the extracted
+                            given module. The "ignore-tensor" varint is used for the extracted
                             blocks or in case no anchor block is found.
                             For the definition of the anchor block, see tir/analysis/analysis.py.
     """
