@@ -19,7 +19,7 @@
 
 import os
 import shutil
-import tvm.ffi
+import tvm_ffi
 from tvm.contrib import coreml_runtime
 from tvm.contrib.xcode import compile_coreml
 
@@ -463,7 +463,7 @@ class CodegenCoreML(PyExprVisitor):
         compile_coreml(model, self.model_name, out_dir)
 
 
-@tvm.ffi.register_func("relax.ext.coreml")
+@tvm_ffi.register_global_func("relax.ext.coreml")
 def coreml_compiler(funcs, options, constant_names):
     """
     Create a CoreML runtime from a Relax module.
