@@ -20,6 +20,7 @@
 from typing import Any, Callable, Dict, List, Optional, Sequence, Union
 
 import tvm
+import tvm_ffi
 from tvm import relax as rx
 from tvm import tir
 from tvm.ir.module import IRModule
@@ -100,7 +101,7 @@ class TestingScope(object):
         self._bb.end_scope()
 
 
-@tvm.ffi.register_object("relax.BlockBuilder")
+@tvm_ffi.register_object("relax.BlockBuilder")
 class BlockBuilder(Object):
     """A builder to build Relax IR for testing and dev.
 
@@ -298,6 +299,10 @@ class BlockBuilder(Object):
         """
         if isinstance(expr, (list, tuple)):
             return Tuple([self._normalize_python_tuple(element) for element in expr])
+        elif expr is None:
+            from . import op
+
+            return op.null_value()
         else:
             return expr
 
