@@ -22,8 +22,9 @@
 import typing
 from typing import Dict, List, Optional, Tuple, Union
 
+import tvm_ffi
+
 import tvm
-import tvm.ffi as tvm_ffi
 from tvm.ir.container import Array
 from tvm.ir.expr import PrimExpr
 from tvm.ir.op import Op
@@ -326,7 +327,7 @@ class VarPattern(DFPattern):
 
 
 @register_df_node
-class DataflowVarPattern(DFPattern):
+class DataflowVarPattern(VarPattern):
     """A pattern for DataflowVar.
 
     Parameters
