@@ -20,6 +20,7 @@ from .nn import (
     adaptive_avg_pool2d,
     adaptive_avg_pool3d,
     attention,
+    attention_bias,
     attention_var_len,
     avg_pool1d,
     avg_pool2d,
@@ -35,6 +36,7 @@ from .nn import (
     gelu,
     gelu_tanh,
     group_norm,
+    instance_norm,
     layer_norm,
     leakyrelu,
     log_softmax,
@@ -43,7 +45,10 @@ from .nn import (
     max_pool3d,
     nll_loss,
     pad,
+    pixel_shuffle,
+    prelu,
     relu,
+    relu6,
     rms_norm,
     selu,
     silu,
