@@ -15,6 +15,6 @@
 # specific language governing permissions and limitations
 # under the License.
 """Constructor APIs"""
-import tvm.ffi
+import tvm_ffi
 
-tvm.ffi._init_api("relax.op.nn", __name__)
+tvm_ffi.init_ffi_api("relax.op.nn", __name__)
