@@ -23,12 +23,12 @@ import warnings
 from typing import Callable, Dict, List, Mapping, Optional, Sequence, Tuple, Union
 
 import numpy as np  # type: ignore
-
+import tvm_ffi
 import tvm.ir
 from tvm.ir.container import Array
 from tvm.relax import Expr, Var, StructInfo
 from tvm.relax.dpl import DFPattern
-from tvm.runtime import NDArray, Object
+from tvm.runtime import Tensor, Object
 from tvm.tir import IndexMap, PrimFunc
 
 from . import _ffi_api
@@ -36,14 +36,14 @@ from .legalize_ops.common import LegalizeFunc
 from ..expr import Var
 
 
-@tvm.ffi.register_object("relax.FunctionPass")
+@tvm_ffi.register_object("relax.FunctionPass")
 class FunctionPass(tvm.ir.transform.Pass):
     """A pass that works on each tvm.relax.Function in a module. A function
     pass class should be created through `function_pass`.
     """
 
 
-@tvm.ffi.register_object("relax.DataflowBlockPass")
+@tvm_ffi.register_object("relax.DataflowBlockPass")
 class DataflowBlockPass(tvm.ir.transform.Pass):
     """A pass that works on each tvm.relax.DataflowBlock in a module."""
 
@@ -219,7 +219,7 @@ def Gradient(
                 # return value: (orig_return_values, tuple(adjoints))
                 return ((lv1, lv2), (x_adjoint, y_adjoint))
     """
-    if require_grads is not None and not isinstance(require_grads, list):
+    if require_grads is not None and not isinstance(require_grads, (list, tvm_ffi.Array)):
         require_grads = [require_grads]
 
     return _ffi_api.Gradient(func_name, require_grads, target_index)  # type: ignore
@@ -638,7 +638,7 @@ def AttachGlobalSymbol() -> tvm.ir.transform.Pass:
 
 def BindParams(
     func_name: str,
-    params: Dict[Union[str, Var], Union[tvm.runtime.NDArray, np.ndarray]],
+    params: Dict[Union[str, Var], Union[tvm.runtime.Tensor, np.ndarray]],
 ) -> tvm.ir.transform.Pass:
     """Bind params of function of the module to constant tensors.
 
@@ -647,7 +647,7 @@ def BindParams(
     func_name: str
         The function name to be bound
 
-    params: Dict[Union[str,relax.Var], Union[tvm.runtime.NDArray, np.ndarray]]
+    params: Dict[Union[str,relax.Var], Union[tvm.runtime.Tensor, np.ndarray]]
         The map from parameter or parameter name to constant tensors.
 
     Returns
@@ -657,9 +657,9 @@ def BindParams(
     tvm_params = {}
     for k, v in params.items():
         if isinstance(v, np.ndarray):
-            v = tvm.nd.array(v)
-        assert isinstance(v, (tvm.runtime.NDArray, tvm.relax.Constant)), (
-            f"param values are expected to be TVM.NDArray,"
+            v = tvm.runtime.tensor(v)
+        assert isinstance(v, (tvm.runtime.Tensor, tvm.relax.Constant)), (
+            f"param values are expected to be TVM.Tensor,"
             f"numpy.ndarray or tvm.relax.Constant, but got {type(v)}"
         )
         tvm_params[k] = v
@@ -820,7 +820,7 @@ def FuseTIR() -> tvm.ir.transform.Pass:
     return _ffi_api.FuseTIR()  # type: ignore
 
 
-@tvm.ffi.register_object("relax.transform.PatternCheckContext")
+@tvm_ffi.register_object("relax.transform.PatternCheckContext")
 class PatternCheckContext(Object):
     """
     The input of check function `FusionPattern.check`.
@@ -854,7 +854,7 @@ class PatternCheckContext(Object):
     value_to_bound_var: Mapping[Expr, Var]
 
 
-@tvm.ffi.register_object("relax.transform.FusionPattern")
+@tvm_ffi.register_object("relax.transform.FusionPattern")
 class FusionPattern(Object):
     """
     The pattern used by `FuseOpsByPattern`. It's mainly DFPattern but with other
@@ -1223,7 +1223,7 @@ def MetaScheduleTuneTIR(
 
 
 def MetaScheduleTuneIRMod(
-    params: Dict[str, NDArray],
+    params: Dict[str, Tensor],
     work_dir: str,
     max_trials_global: int,
     max_trials_per_task: Optional[int] = None,
@@ -1233,7 +1233,7 @@ def MetaScheduleTuneIRMod(
 
     Parameters
     ----------
-    params: Dict[str, NDArray]
+    params: Dict[str, Tensor]
        model params
     work_dir: str
        work directory
