@@ -26,9 +26,9 @@ from typing import Tuple as typing_Tuple
 from typing import Any, Callable, List, Dict, Optional
 
 import tvm
+import tvm_ffi
 from .. import tir
 from ..tir import PrimExpr
-from ..runtime import String
 from . import _ffi_api
 from .expr import Tuple as rx_Tuple
 from .expr import Expr, ShapeExpr, Function, PrimValue, StringImm, te_tensor
@@ -100,7 +100,7 @@ def convert_to_expr(value: Any) -> Expr:
     if isinstance(value, float):
         return PrimValue(tir.FloatImm("float64", value))
 
-    tvm_value = tvm.ffi.convert(value)
+    tvm_value = tvm_ffi.convert(value)
     # Case 1
     if isinstance(tvm_value, Expr):  # type: ignore
         return tvm_value
@@ -114,7 +114,7 @@ def convert_to_expr(value: Any) -> Expr:
     if isinstance(tvm_value, PrimExpr):
         return PrimValue(value)
     # Case 3
-    if isinstance(tvm_value, (str, String)):
+    if isinstance(tvm_value, (str,)):
         return StringImm(value)
     # Case 4
     if isinstance(value, (tuple, list)):
