@@ -14,7 +14,7 @@
 # specific language governing permissions and limitations
 # under the License.
 """FFI APIs for tvm.script"""
-import tvm.ffi
+import tvm_ffi
 
 
-tvm.ffi._init_api("script", __name__)
+tvm_ffi.init_ffi_api("script", __name__)
