@@ -15,11 +15,11 @@
 # specific language governing permissions and limitations
 # under the License.
 """Common x86 related utilities"""
-from ..ffi import register_func
+from tvm_ffi import register_global_func
 from .codegen import target_has_features
 
 
-@register_func("tvm.topi.x86.utils.get_simd_32bit_lanes")
+@register_global_func("tvm.topi.x86.utils.get_simd_32bit_lanes")
 def get_simd_32bit_lanes():
     """X86 SIMD optimal vector length lookup.
     Parameters
