@@ -15,6 +15,6 @@
 # specific language governing permissions and limitations
 # under the License.
 """FFI for NN TOPI ops and schedules"""
-import tvm.ffi
+import tvm_ffi
 
-tvm.ffi._init_api("topi.nn", "tvm.topi.cpp.nn")
+tvm_ffi.init_ffi_api("topi.nn", "tvm.topi.cpp.nn")
