@@ -15,6 +15,6 @@
 # specific language governing permissions and limitations
 # under the License.
 """FFI for TOPI utility functions"""
-import tvm.ffi
+import tvm_ffi
 
-tvm.ffi._init_api("topi.utils", "tvm.topi.cpp.utils")
+tvm_ffi.init_ffi_api("topi.utils", "tvm.topi.cpp.utils")
