@@ -40,14 +40,14 @@ void IRMutatorWithAnalyzer::MarkBufferMapShapes(const tir::PrimFunc& func) {
   }
 }
 
-Array<PrimExpr> IRMutatorWithAnalyzer::IterMapSimplifyWithContext(const Array<PrimExpr>& indices,
-                                                                  bool non_trivial_only) {
+ffi::Array<PrimExpr> IRMutatorWithAnalyzer::IterMapSimplifyWithContext(
+    const ffi::Array<PrimExpr>& indices, bool non_trivial_only) {
   PrimExpr pred = const_true();
   for (PrimExpr val : iter_predicates_) {
     pred = pred && val;
   }
   int n = indices.size();
-  Array<PrimExpr> simplified = arith::IterMapSimplify(
+  ffi::Array<PrimExpr> simplified = arith::IterMapSimplify(
       indices, this->iter_vars_, pred, arith::IterMapLevel::Surjective, this->analyzer_);
   if (non_trivial_only) {
     for (int i = 0; i < n; ++i) {
@@ -84,7 +84,7 @@ Stmt IRMutatorWithAnalyzer::VisitStmt_(const LetStmtNode* op) {
   // as sub-class may or maynot choose to replace it.
   Stmt body = this->VisitStmt(op->body);
   if (value.same_as(op->value) && body.same_as(op->body)) {
-    return GetRef<Stmt>(op);
+    return ffi::GetRef<Stmt>(op);
   } else {
     auto n = this->CopyOnWrite(op);
     n->value = std::move(value);
@@ -105,7 +105,7 @@ Stmt IRMutatorWithAnalyzer::VisitStmt_(const IfThenElseNode* op) {
   }
 
   Stmt then_case;
-  Optional<Stmt> else_case;
+  ffi::Optional<Stmt> else_case;
   {
     With<ConstraintContext> ctx(analyzer_, real_condition);
     WithRecordIterPredicate(real_condition, [&] { then_case = this->VisitStmt(op->then_case); });
@@ -121,7 +121,7 @@ Stmt IRMutatorWithAnalyzer::VisitStmt_(const IfThenElseNode* op) {
 
   if (condition.same_as(op->condition) && then_case.same_as(op->then_case) &&
       else_case.same_as(op->else_case)) {
-    return GetRef<Stmt>(op);
+    return ffi::GetRef<Stmt>(op);
   } else {
     auto n = this->CopyOnWrite(op);
     n->condition = std::move(condition);
@@ -152,7 +152,7 @@ Stmt IRMutatorWithAnalyzer::VisitStmt_(const AssertStmtNode* op) {
   Stmt body = this->VisitStmt(op->body);
 
   if (condition.same_as(op->condition) && message.same_as(op->message) && body.same_as(op->body)) {
-    return GetRef<Stmt>(op);
+    return ffi::GetRef<Stmt>(op);
   } else {
     auto n = this->CopyOnWrite(op);
     n->condition = std::move(condition);
@@ -185,7 +185,7 @@ PrimExpr IRMutatorWithAnalyzer::VisitExpr_(const CallNode* op) {
     }
     if (cond.same_as(op->args[0]) && true_value.same_as(op->args[1]) &&
         false_value.same_as(op->args[2])) {
-      return GetRef<PrimExpr>(op);
+      return ffi::GetRef<PrimExpr>(op);
     } else {
       return Call(op->dtype, op->op, {cond, true_value, false_value});
     }
@@ -202,7 +202,7 @@ PrimExpr IRMutatorWithAnalyzer::VisitExpr_(const LetNode* op) {
   // as sub-class may or maynot choose to replace it.
   PrimExpr body = this->VisitExpr(op->body);
   if (value.same_as(op->value) && body.same_as(op->body)) {
-    return GetRef<PrimExpr>(op);
+    return ffi::GetRef<PrimExpr>(op);
   } else {
     return Let(op->var, value, body);
   }
@@ -228,7 +228,7 @@ PrimExpr IRMutatorWithAnalyzer::VisitExpr_(const SelectNode* op) {
   // normal path
   if (cond.same_as(op->condition) && true_value.same_as(op->true_value) &&
       false_value.same_as(op->false_value)) {
-    return GetRef<PrimExpr>(op);
+    return ffi::GetRef<PrimExpr>(op);
   } else {
     return Select(cond, true_value, false_value);
   }
