@@ -27,16 +27,16 @@ namespace tvm {
 namespace contrib {
 namespace msc {
 
-const Array<Doc> BaseStack::GetDocs() const {
+const ffi::Array<Doc> BaseStack::GetDocs() const {
   ICHECK(blocks_.size() == 1) << "Has incomplete blocks, please check";
   return TopBlock();
 }
 
 void BaseStack::Line(const Doc& doc) { PushDoc(doc); }
 
-void BaseStack::Line(const String& line) { Line(IdDoc(line)); }
+void BaseStack::Line(const ffi::String& line) { Line(IdDoc(line)); }
 
-void BaseStack::Comment(const String& comment, bool attach) {
+void BaseStack::Comment(const ffi::String& comment, bool attach) {
   if (attach) {
     const auto& doc = TopDoc();
     ICHECK(doc->IsInstance<StmtDocNode>()) << "Only stmt doc support attach comments";
@@ -47,38 +47,39 @@ void BaseStack::Comment(const String& comment, bool attach) {
   }
 }
 
-void BaseStack::Declare(const String& type, const String& variable, size_t len,
+void BaseStack::Declare(const ffi::String& type, const ffi::String& variable, size_t len,
                         bool use_constructor) {
   PushDoc(DocUtils::ToDeclare(type, variable, len, use_constructor));
 }
 
 void BaseStack::DeclareArgBase(const ExprDoc& value) {
   const auto& declare = PopCheckedDoc<DeclareDoc, DeclareDocNode>();
-  Array<ExprDoc> init_args = declare->init_args;
+  ffi::Array<ExprDoc> init_args = declare->init_args;
   init_args.push_back(value);
   PushDoc(DeclareDoc(declare->type, declare->variable, init_args, declare->use_constructor));
 }
 
-void BaseStack::FuncDef(const String& func_name, const String& ret_type) {
+void BaseStack::FuncDef(const ffi::String& func_name, const ffi::String& ret_type) {
   if (ret_type.size() > 0) {
-    PushDoc(FunctionDoc(IdDoc(func_name), Array<AssignDoc>(), Array<ExprDoc>(), IdDoc(ret_type),
-                        Array<StmtDoc>()));
+    PushDoc(FunctionDoc(IdDoc(func_name), ffi::Array<AssignDoc>(), ffi::Array<ExprDoc>(),
+                        IdDoc(ret_type), ffi::Array<StmtDoc>()));
   } else {
-    PushDoc(FunctionDoc(IdDoc(func_name), Array<AssignDoc>(), Array<ExprDoc>(), std::nullopt,
-                        Array<StmtDoc>()));
+    PushDoc(FunctionDoc(IdDoc(func_name), ffi::Array<AssignDoc>(), ffi::Array<ExprDoc>(),
+                        std::nullopt, ffi::Array<StmtDoc>()));
   }
 }
 
-void BaseStack::FuncArg(const String& arg, const String& annotation, const String& value) {
+void BaseStack::FuncArg(const ffi::String& arg, const ffi::String& annotation,
+                        const ffi::String& value) {
   const auto& func = PopCheckedDoc<FunctionDoc, FunctionDocNode>();
-  Array<AssignDoc> args = func->args;
+  ffi::Array<AssignDoc> args = func->args;
   args.push_back(DocUtils::ToAssign(arg, value, annotation));
   PushDoc(FunctionDoc(func->name, args, func->decorators, func->return_type, func->body));
 }
 
-void BaseStack::FuncDecorator(const String& decorator) {
+void BaseStack::FuncDecorator(const ffi::String& decorator) {
   const auto& func = PopCheckedDoc<FunctionDoc, FunctionDocNode>();
-  Array<ExprDoc> decorators = func->decorators;
+  ffi::Array<ExprDoc> decorators = func->decorators;
   decorators.push_back(IdDoc(decorator));
   PushDoc(FunctionDoc(func->name, func->args, decorators, func->return_type, func->body));
 }
@@ -95,13 +96,13 @@ void BaseStack::FuncEnd() {
   PushDoc(FunctionDoc(func->name, func->args, func->decorators, func->return_type, body));
 }
 
-void BaseStack::ClassDef(const String& class_name) {
-  PushDoc(ClassDoc(IdDoc(class_name), Array<ExprDoc>(), Array<StmtDoc>()));
+void BaseStack::ClassDef(const ffi::String& class_name) {
+  PushDoc(ClassDoc(IdDoc(class_name), ffi::Array<ExprDoc>(), ffi::Array<StmtDoc>()));
 }
 
-void BaseStack::ClassDecorator(const String& decorator) {
+void BaseStack::ClassDecorator(const ffi::String& decorator) {
   const auto& class_doc = PopCheckedDoc<ClassDoc, ClassDocNode>();
-  Array<ExprDoc> decorators = class_doc->decorators;
+  ffi::Array<ExprDoc> decorators = class_doc->decorators;
   decorators.push_back(IdDoc(decorator));
   PushDoc(ClassDoc(class_doc->name, decorators, class_doc->body));
 }
@@ -118,8 +119,8 @@ void BaseStack::ClassEnd() {
   PushDoc(ClassDoc(class_doc->name, class_doc->decorators, body));
 }
 
-void BaseStack::StructStart(const String& struct_name) {
-  PushDoc(StructDoc(IdDoc(struct_name), Array<ExprDoc>(), Array<StmtDoc>()));
+void BaseStack::StructStart(const ffi::String& struct_name) {
+  PushDoc(StructDoc(IdDoc(struct_name), ffi::Array<ExprDoc>(), ffi::Array<StmtDoc>()));
   BlockStart();
 }
 
@@ -130,13 +131,14 @@ void BaseStack::StructEnd() {
   PushDoc(StructDoc(struct_doc->name, struct_doc->decorators, body));
 }
 
-void BaseStack::ConstructorDef(const String& constructor_name) {
-  PushDoc(ConstructorDoc(IdDoc(constructor_name), Array<AssignDoc>(), Array<StmtDoc>()));
+void BaseStack::ConstructorDef(const ffi::String& constructor_name) {
+  PushDoc(ConstructorDoc(IdDoc(constructor_name), ffi::Array<AssignDoc>(), ffi::Array<StmtDoc>()));
 }
 
-void BaseStack::ConstructorArg(const String& arg, const String& annotation, const String& value) {
+void BaseStack::ConstructorArg(const ffi::String& arg, const ffi::String& annotation,
+                               const ffi::String& value) {
   const auto& func = PopCheckedDoc<ConstructorDoc, ConstructorDocNode>();
-  Array<AssignDoc> args = func->args;
+  ffi::Array<AssignDoc> args = func->args;
   args.push_back(DocUtils::ToAssign(arg, value, annotation));
   PushDoc(ConstructorDoc(func->name, args, func->body));
 }
@@ -153,20 +155,22 @@ void BaseStack::ConstructorEnd() {
   PushDoc(ConstructorDoc(func->name, func->args, body));
 }
 
-void BaseStack::LambdaDef(const String& lambda_name) {
-  PushDoc(LambdaDoc(IdDoc(lambda_name), Array<AssignDoc>(), Array<ExprDoc>(), Array<StmtDoc>()));
+void BaseStack::LambdaDef(const ffi::String& lambda_name) {
+  PushDoc(LambdaDoc(IdDoc(lambda_name), ffi::Array<AssignDoc>(), ffi::Array<ExprDoc>(),
+                    ffi::Array<StmtDoc>()));
 }
 
-void BaseStack::LambdaArg(const String& arg, const String& annotation, const String& value) {
+void BaseStack::LambdaArg(const ffi::String& arg, const ffi::String& annotation,
+                          const ffi::String& value) {
   const auto& lambda = PopCheckedDoc<LambdaDoc, LambdaDocNode>();
-  Array<AssignDoc> args = lambda->args;
+  ffi::Array<AssignDoc> args = lambda->args;
   args.push_back(DocUtils::ToAssign(arg, value, annotation));
   PushDoc(LambdaDoc(lambda->name, args, lambda->refs, lambda->body));
 }
 
-void BaseStack::LambdaRef(const String& ref) {
+void BaseStack::LambdaRef(const ffi::String& ref) {
   const auto& lambda = PopCheckedDoc<LambdaDoc, LambdaDocNode>();
-  Array<ExprDoc> refs = lambda->refs;
+  ffi::Array<ExprDoc> refs = lambda->refs;
   refs.push_back(IdDoc(ref));
   PushDoc(LambdaDoc(lambda->name, lambda->args, refs, lambda->body));
 }
@@ -176,7 +180,7 @@ void BaseStack::LambdaStart() {
   BlockStart();
 }
 
-void BaseStack::LambdaEnd(const String& ret_val) {
+void BaseStack::LambdaEnd(const ffi::String& ret_val) {
   if (ret_val.size() > 0) {
     PushDoc(ReturnDoc(IdDoc(ret_val)));
   }
@@ -191,13 +195,15 @@ void BaseStack::LambdaEnd(const ExprDoc& ret_val) {
   LambdaEnd("");
 }
 
-void BaseStack::FuncCall(const String& callee, Optional<ExprDoc> assign_to,
-                         Optional<ExprDoc> caller) {
+void BaseStack::FuncCall(const ffi::String& callee, ffi::Optional<ExprDoc> assign_to,
+                         ffi::Optional<ExprDoc> caller) {
   if (!caller.defined()) {
-    PushDoc(CallDoc(IdDoc(callee), Array<ExprDoc>(), Array<String>(), Array<ExprDoc>()));
+    PushDoc(CallDoc(IdDoc(callee), ffi::Array<ExprDoc>(), ffi::Array<ffi::String>(),
+                    ffi::Array<ExprDoc>()));
   } else {
     const auto& new_access = AttrAccessDoc(caller.value(), callee);
-    PushDoc(CallDoc(new_access, Array<ExprDoc>(), Array<String>(), Array<ExprDoc>()));
+    PushDoc(CallDoc(new_access, ffi::Array<ExprDoc>(), ffi::Array<ffi::String>(),
+                    ffi::Array<ExprDoc>()));
   }
   if (assign_to.defined()) {
     const auto& last_call = PopCheckedDoc<CallDoc, CallDocNode>();
@@ -211,14 +217,15 @@ void BaseStack::FuncCall(const String& callee, Optional<ExprDoc> assign_to,
   }
 }
 
-void BaseStack::FuncCall(const String& callee, const String& assign_to, const String& caller) {
-  Optional<ExprDoc> assign_doc;
+void BaseStack::FuncCall(const ffi::String& callee, const ffi::String& assign_to,
+                         const ffi::String& caller) {
+  ffi::Optional<ExprDoc> assign_doc;
   if (assign_to.size() == 0) {
     assign_doc = std::nullopt;
   } else {
     assign_doc = IdDoc(assign_to);
   }
-  Optional<ExprDoc> caller_doc;
+  ffi::Optional<ExprDoc> caller_doc;
   if (caller.size() == 0) {
     caller_doc = std::nullopt;
   } else {
@@ -227,26 +234,27 @@ void BaseStack::FuncCall(const String& callee, const String& assign_to, const St
   FuncCall(callee, assign_doc, caller_doc);
 }
 
-void BaseStack::MethodCall(const String& callee, bool new_line) {
+void BaseStack::MethodCall(const ffi::String& callee, bool new_line) {
   const auto& host = PopDoc();
   if (host->IsInstance<ExprDocNode>()) {
     const auto& v_callee = callee + (new_line ? DocSymbol::NextLine() : "");
     FuncCall(v_callee, std::nullopt, Downcast<ExprDoc>(host));
   } else if (const auto* a_node = host.as<AssignDocNode>()) {
     ICHECK(a_node->rhs.defined()) << "Can not find rhs for inplace host";
-    FuncCall(callee, DeclareDoc(a_node->annotation, a_node->lhs, Array<ExprDoc>(), true),
+    FuncCall(callee, DeclareDoc(a_node->annotation, a_node->lhs, ffi::Array<ExprDoc>(), true),
              a_node->rhs);
   } else {
     LOG(FATAL) << "Unexpected host type for inplace " << host->GetTypeKey();
   }
 }
 
-void BaseStack::InplaceStart(const String& callee, Optional<ExprDoc> assign_to,
-                             Optional<ExprDoc> caller) {
+void BaseStack::InplaceStart(const ffi::String& callee, ffi::Optional<ExprDoc> assign_to,
+                             ffi::Optional<ExprDoc> caller) {
   FuncCall(callee, assign_to, caller);
 }
 
-void BaseStack::InplaceStart(const String& callee, const String& assign_to, const String& caller) {
+void BaseStack::InplaceStart(const ffi::String& callee, const ffi::String& assign_to,
+                             const ffi::String& caller) {
   FuncCall(callee, assign_to, caller);
 }
 
@@ -266,7 +274,7 @@ void BaseStack::InplaceEnd() {
   }
 }
 
-void BaseStack::PopNest(const String& key) {
+void BaseStack::PopNest(const ffi::String& key) {
   const auto& last = PopDoc();
   if (last->IsInstance<CallDocNode>()) {
     CallArgBase(Downcast<CallDoc>(last), key);
@@ -275,11 +283,11 @@ void BaseStack::PopNest(const String& key) {
   }
 }
 
-void BaseStack::CallArgBase(const ExprDoc& value, const String& key) {
+void BaseStack::CallArgBase(const ExprDoc& value, const ffi::String& key) {
   const auto& last = PopDoc();
-  Array<ExprDoc> args;
-  Array<String> kwargs_keys;
-  Array<ExprDoc> kwargs_values;
+  ffi::Array<ExprDoc> args;
+  ffi::Array<ffi::String> kwargs_keys;
+  ffi::Array<ExprDoc> kwargs_values;
   // get args and kwargs
   if (const auto* call = last.as<CallDocNode>()) {
     args = call->args;
@@ -313,16 +321,16 @@ void BaseStack::CallArgBase(const ExprDoc& value, const String& key) {
   }
 }
 
-void BaseStack::ConditionIf(const String& predicate) {
-  Array<StmtDoc> else_branch{ExprStmtDoc(IdDoc("pass"))};
-  PushDoc(IfDoc(IdDoc(predicate), Array<StmtDoc>(), else_branch));
+void BaseStack::ConditionIf(const ffi::String& predicate) {
+  ffi::Array<StmtDoc> else_branch{ExprStmtDoc(IdDoc("pass"))};
+  PushDoc(IfDoc(IdDoc(predicate), ffi::Array<StmtDoc>(), else_branch));
   BlockStart();
 }
 
 void BaseStack::ConditionElse() {
   const auto& block = PopBlock();
   const auto& if_doc = PopCheckedDoc<IfDoc, IfDocNode>();
-  PushDoc(IfDoc(if_doc->predicate, DocUtils::ToStmts(block), Array<StmtDoc>()));
+  PushDoc(IfDoc(if_doc->predicate, DocUtils::ToStmts(block), ffi::Array<StmtDoc>()));
   BlockStart();
 }
 
@@ -331,7 +339,7 @@ void BaseStack::ConditionEnd() {
   const auto& if_doc = PopCheckedDoc<IfDoc, IfDocNode>();
   const auto& branch = DocUtils::ToStmts(block);
   if (if_doc->then_branch.size() == 0) {
-    PushDoc(IfDoc(if_doc->predicate, branch, Array<StmtDoc>()));
+    PushDoc(IfDoc(if_doc->predicate, branch, ffi::Array<StmtDoc>()));
   } else {
     PushDoc(IfDoc(if_doc->predicate, if_doc->then_branch, branch));
   }
@@ -344,8 +352,8 @@ void BaseStack::ForEnd() {
   PushDoc(ForDoc(for_doc->lhs, for_doc->rhs, body));
 }
 
-void BaseStack::WhileStart(const String& predicate) {
-  PushDoc(WhileDoc(IdDoc(predicate), Array<StmtDoc>()));
+void BaseStack::WhileStart(const ffi::String& predicate) {
+  PushDoc(WhileDoc(IdDoc(predicate), ffi::Array<StmtDoc>()));
   BlockStart();
 }
 
@@ -356,20 +364,20 @@ void BaseStack::WhileEnd() {
   PushDoc(WhileDoc(while_doc->predicate, body));
 }
 
-void BaseStack::SwitchStart(const String& predicate) {
-  Array<ExprDoc> predicates;
+void BaseStack::SwitchStart(const ffi::String& predicate) {
+  ffi::Array<ExprDoc> predicates;
   predicates.push_back(IdDoc(predicate));
-  PushDoc(SwitchDoc(predicates, Array<Array<StmtDoc>>(), Array<StmtDoc>()));
+  PushDoc(SwitchDoc(predicates, ffi::Array<ffi::Array<StmtDoc>>(), ffi::Array<StmtDoc>()));
   BlockStart();
 }
 
-void BaseStack::SwitchCase(const String& predicate) {
+void BaseStack::SwitchCase(const ffi::String& predicate) {
   const auto& block = PopBlock();
   const auto& switch_doc = PopCheckedDoc<SwitchDoc, SwitchDocNode>();
   auto branchs = switch_doc->branchs;
   branchs.push_back(DocUtils::ToStmts(block));
   if (predicate.size() == 0) {
-    Array<StmtDoc> default_branch{ExprStmtDoc(IdDoc("pass"))};
+    ffi::Array<StmtDoc> default_branch{ExprStmtDoc(IdDoc("pass"))};
     PushDoc(SwitchDoc(switch_doc->predicates, branchs, default_branch));
   } else {
     auto predicates = switch_doc->predicates;
@@ -392,7 +400,7 @@ void BaseStack::SwitchEnd() {
 }
 
 void BaseStack::BlockStart() {
-  Array<Doc> block;
+  ffi::Array<Doc> block;
   blocks_.push(block);
 }
 
@@ -407,11 +415,11 @@ void BaseStack::BlockEnd(bool block_docs) {
   }
 }
 
-void BaseStack::ScopeStart(const String& scope_def, const String& scope_ref) {
+void BaseStack::ScopeStart(const ffi::String& scope_def, const ffi::String& scope_ref) {
   if (scope_ref.size() > 0) {
-    PushDoc(ScopeDoc(IdDoc(scope_ref), IdDoc(scope_def), Array<StmtDoc>()));
+    PushDoc(ScopeDoc(IdDoc(scope_ref), IdDoc(scope_def), ffi::Array<StmtDoc>()));
   } else {
-    PushDoc(ScopeDoc(std::nullopt, IdDoc(scope_def), Array<StmtDoc>()));
+    PushDoc(ScopeDoc(std::nullopt, IdDoc(scope_def), ffi::Array<StmtDoc>()));
   }
   BlockStart();
 }
@@ -424,12 +432,12 @@ void BaseStack::ScopeEnd() {
 
 bool BaseStack::HasBlock() const { return blocks_.size() > 0; }
 
-const Array<Doc> BaseStack::TopBlock() const {
+const ffi::Array<Doc> BaseStack::TopBlock() const {
   ICHECK(HasBlock()) << "No block found";
   return blocks_.top();
 }
 
-const Array<Doc> BaseStack::PopBlock() {
+const ffi::Array<Doc> BaseStack::PopBlock() {
   const auto& block = TopBlock();
   blocks_.pop();
   return block;
