@@ -25,10 +25,10 @@
 #define TVM_CONTRIB_MSC_CORE_IR_GRAPH_BUILDER_H_
 
 #include <dmlc/json.h>
-#include <tvm/ffi/reflection/reflection.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/relax/expr.h>
 #include <tvm/relax/expr_functor.h>
-#include <tvm/runtime/ndarray.h>
+#include <tvm/runtime/tensor.h>
 #include <tvm/tir/data_layout.h>
 
 #include <set>
@@ -50,7 +50,7 @@ namespace msc {
 using namespace tvm::relax;
 
 using Expr = tvm::RelaxExpr;
-using tvm::runtime::NDArray;
+using tvm::runtime::Tensor;
 
 /*!
  * \brief Config for building MSCGraph.
@@ -110,10 +110,10 @@ struct MSCRBuildConfig {
 class AttrGetter {
  public:
   /*!
-   * \brief Get the attributes as Map<String, String>
+   * \brief Get the attributes as ffi::Map<ffi::String, ffi::String>
    * \param attrs the attributes.
    */
-  explicit AttrGetter(Map<String, String>* attrs) : attrs_(attrs) {}
+  explicit AttrGetter(ffi::Map<ffi::String, ffi::String>* attrs) : attrs_(attrs) {}
 
   void operator()(const Attrs& attrs) {
     if (const auto* dict_attrs = attrs.as<DictAttrsNode>()) {
@@ -122,17 +122,17 @@ class AttrGetter {
       }
     } else {
       const TVMFFITypeInfo* attrs_tinfo = TVMFFIGetTypeInfo(attrs->type_index());
-      if (attrs_tinfo->extra_info != nullptr) {
+      if (attrs_tinfo->metadata != nullptr) {
         tvm::ffi::reflection::ForEachFieldInfo(attrs_tinfo, [&](const TVMFFIFieldInfo* field_info) {
           Any field_value = tvm::ffi::reflection::FieldGetter(field_info)(attrs);
-          this->VisitAny(String(field_info->name), field_value);
+          this->VisitAny(ffi::String(field_info->name), field_value);
         });
       }
     }
   }
 
  private:
-  void VisitAny(String key, Any value) {
+  void VisitAny(ffi::String key, Any value) {
     switch (value.type_index()) {
       case kTVMFFINone: {
         attrs_->Set(key, "");
@@ -154,8 +154,9 @@ class AttrGetter {
         attrs_->Set(key, runtime::DLDataTypeToString(value.cast<DLDataType>()));
         break;
       }
+      case kTVMFFISmallStr:
       case kTVMFFIStr: {
-        attrs_->Set(key, value.cast<String>());
+        attrs_->Set(key, value.cast<ffi::String>());
         break;
       }
       default: {
@@ -170,13 +171,13 @@ class AttrGetter {
   }
 
  private:
-  Map<String, String>* attrs_;
+  ffi::Map<ffi::String, ffi::String>* attrs_;
 };
 
 class FuncAttrGetter : public ExprVisitor {
  public:
-  /*! \brief Get the attributes as Map<String, String>*/
-  Map<String, String> GetAttrs(const Expr& expr) {
+  /*! \brief Get the attributes as ffi::Map<ffi::String, ffi::String>*/
+  ffi::Map<ffi::String, ffi::String> GetAttrs(const Expr& expr) {
     VisitExpr(expr);
     return attrs_;
   }
@@ -186,13 +187,13 @@ class FuncAttrGetter : public ExprVisitor {
   void VisitExpr_(const TupleGetItemNode* op) final;
 
  private:
-  Map<String, String> attrs_;
+  ffi::Map<ffi::String, ffi::String> attrs_;
 };
 
 class FuncValueGetter : public ExprVisitor {
  public:
-  /*! \brief Get the attributes from prim value as Map<String, String>*/
-  Array<String> GetValues(const Expr& expr) {
+  /*! \brief Get the attributes from prim value as ffi::Map<ffi::String, ffi::String>*/
+  ffi::Array<ffi::String> GetValues(const Expr& expr) {
     VisitExpr(expr);
     return values_;
   }
@@ -200,7 +201,7 @@ class FuncValueGetter : public ExprVisitor {
   void VisitExpr_(const CallNode* op) final;
 
  private:
-  Array<String> values_;
+  ffi::Array<ffi::String> values_;
 };
 
 class FuncParamsFinder : public ExprVisitor {
@@ -214,7 +215,7 @@ class FuncParamsFinder : public ExprVisitor {
   }
 
   /*! \brief Find the func params and bind with arguments*/
-  Map<Expr, Expr> FindParams(const Expr& expr) {
+  ffi::Map<Expr, Expr> FindParams(const Expr& expr) {
     VisitExpr(expr);
     return params_;
   }
@@ -225,8 +226,8 @@ class FuncParamsFinder : public ExprVisitor {
 
  private:
   IRModule ref_module_;
-  Map<Expr, Expr> params_;
-  Map<Expr, Function> local_funcs_;
+  ffi::Map<Expr, Expr> params_;
+  ffi::Map<Expr, Function> local_funcs_;
 };
 
 class LayoutsFinder : public ExprVisitor {
@@ -238,7 +239,7 @@ class LayoutsFinder : public ExprVisitor {
   explicit LayoutsFinder(const IRModule& ref_module) : ExprVisitor() { ref_module_ = ref_module; }
 
   /*! \brief Find the layouts form attrs*/
-  Map<String, String> FindLayouts(const Expr& expr) {
+  ffi::Map<ffi::String, ffi::String> FindLayouts(const Expr& expr) {
     VisitExpr(expr);
     return layouts_;
   }
@@ -249,8 +250,8 @@ class LayoutsFinder : public ExprVisitor {
 
  private:
   IRModule ref_module_;
-  Map<String, String> layouts_;
-  Map<Expr, Function> local_funcs_;
+  ffi::Map<ffi::String, ffi::String> layouts_;
+  ffi::Map<Expr, Function> local_funcs_;
 };
 
 class GraphBuilder : public ExprVisitor {
@@ -261,7 +262,7 @@ class GraphBuilder : public ExprVisitor {
    * \param name the name of the graph.
    * \param options the options of build the graph.
    */
-  explicit GraphBuilder(const IRModule& ref_module, const String& name,
+  explicit GraphBuilder(const IRModule& ref_module, const ffi::String& name,
                         const std::string& options = "")
       : ExprVisitor() {
     ref_module_ = ref_module;
@@ -270,7 +271,7 @@ class GraphBuilder : public ExprVisitor {
       dmlc::JSONReader reader(&is);
       reader.Read(&config_);
     }
-    name_ = config_.graph_name.size() > 0 ? String(config_.graph_name) : name;
+    name_ = config_.graph_name.size() > 0 ? ffi::String(config_.graph_name) : name;
     if (config_.byoc_entry.size() > 0) {
       func_params_ = FuncParamsFinder(ref_module).FindParams(ref_module->Lookup(name));
     }
@@ -284,15 +285,16 @@ class GraphBuilder : public ExprVisitor {
   const MSCRBuildConfig config() { return config_; }
 
   /*! \brief Create and add MSCJoint from expr*/
-  const MSCJoint AddNode(const Expr& expr, const Optional<Expr>& binding_var = std::nullopt,
-                         const String& name = "");
+  const MSCJoint AddNode(const Expr& expr, const ffi::Optional<Expr>& binding_var = std::nullopt,
+                         const ffi::String& name = "");
 
   /*! \brief Create and add MSCPrim from prim*/
   const MSCPrim AddPrim(const PrimExpr& prim);
 
-  const MSCPrim MatchOrCreatePrim(const PrimExpr& prim, const String& op = "",
-                                  const Array<BaseJoint>& parents = Array<BaseJoint>(),
-                                  const Map<String, String>& attrs = Map<String, String>());
+  const MSCPrim MatchOrCreatePrim(
+      const PrimExpr& prim, const ffi::String& op = "",
+      const ffi::Array<BaseJoint>& parents = ffi::Array<BaseJoint>(),
+      const ffi::Map<ffi::String, ffi::String>& attrs = ffi::Map<ffi::String, ffi::String>());
 
   void VisitBindingBlock(const BindingBlock& block) final;
 
@@ -318,30 +320,30 @@ class GraphBuilder : public ExprVisitor {
 
  private:
   /*! \brief Get the node_name, optype, layout for func*/
-  const std::tuple<String, String, String> ParseFunc(const Function& func);
+  const std::tuple<ffi::String, ffi::String, ffi::String> ParseFunc(const Function& func);
 
   /*! \brief Get the plugin inputs*/
-  Array<Expr> GetPluginInputs(const Expr& expr);
+  ffi::Array<Expr> GetPluginInputs(const Expr& expr);
 
-  String name_;
+  ffi::String name_;
   IRModule ref_module_;
   MSCRBuildConfig config_;
-  Map<String, String> layouts_;
-  Array<MSCJoint> nodes_;
-  Map<String, MSCTensor> weights_;
-  Map<Expr, Array<String>> expr_tensor_map_;
-  std::unordered_map<String, std::pair<BaseJoint, size_t>> tensor_input_map_;
-  std::set<String> ignore_nodes_;
+  ffi::Map<ffi::String, ffi::String> layouts_;
+  ffi::Array<MSCJoint> nodes_;
+  ffi::Map<ffi::String, MSCTensor> weights_;
+  ffi::Map<Expr, ffi::Array<ffi::String>> expr_tensor_map_;
+  std::unordered_map<ffi::String, std::pair<BaseJoint, size_t>> tensor_input_map_;
+  std::set<ffi::String> ignore_nodes_;
   // scope name
-  String scope_name_;
-  std::set<String> setted_blocks_;
-  Array<String> block_stack_;
+  ffi::String scope_name_;
+  std::set<ffi::String> setted_blocks_;
+  ffi::Array<ffi::String> block_stack_;
   // BYOC maps
-  Map<Expr, Function> target_funcs_;
-  Map<Expr, Expr> func_params_;
+  ffi::Map<Expr, Function> target_funcs_;
+  ffi::Map<Expr, Expr> func_params_;
   // prims
-  Array<MSCPrim> prims_;
-  Map<PrimExpr, MSCPrim> prim_map_;
+  ffi::Array<MSCPrim> prims_;
+  ffi::Map<PrimExpr, MSCPrim> prim_map_;
 };
 
 class WeightsExtractor : public ExprVisitor {
@@ -357,15 +359,15 @@ class WeightsExtractor : public ExprVisitor {
   }
 
   /*! \brief Visit the constant and save weights */
-  Map<MSCTensor, NDArray> GetWeights(const Function& func);
+  ffi::Map<MSCTensor, Tensor> GetWeights(const Function& func);
 
   void VisitExpr_(const ConstantNode* op) final;
 
   void VisitExpr_(const CallNode* op) final;
 
  private:
-  Map<MSCTensor, NDArray> weights_;
-  Map<Expr, Function> local_funcs_;
+  ffi::Map<MSCTensor, Tensor> weights_;
+  ffi::Map<Expr, Function> local_funcs_;
   IRModule ref_module_;
 };
 
