@@ -17,6 +17,7 @@
  * under the License.
  */
 
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/ir/function.h>
 #include <tvm/relax/expr.h>
 #include <tvm/relax/expr_functor.h>
@@ -33,23 +34,23 @@ namespace tvm {
 namespace relax {
 using namespace tvm::contrib::msc;
 
-std::tuple<Map<Var, Expr>, Map<tir::Var, PrimExpr>> NormalizeNamedBindings(
-    const Function& func, const Map<ObjectRef, ObjectRef>& untyped_params) {
+std::tuple<ffi::Map<Var, Expr>, ffi::Map<tir::Var, PrimExpr>> NormalizeNamedBindings(
+    const Function& func, const ffi::Map<ObjectRef, ObjectRef>& untyped_params) {
   ICHECK(func.defined());
   ICHECK(untyped_params.defined());
 
   // Map from string to the variable(s) with that name.
-  std::unordered_map<std::string, Array<relax::Var>> string_lookup;
+  std::unordered_map<std::string, ffi::Array<relax::Var>> string_lookup;
   std::unordered_set<const relax::VarNode*> var_set;
   for (const auto& param : func->params) {
     string_lookup[param->name_hint()].push_back(param);
     var_set.insert(param.get());
   }
 
-  Map<relax::Var, relax::Expr> relax_var_remap;
+  ffi::Map<relax::Var, relax::Expr> relax_var_remap;
 
-  auto normalize_key = [&](ObjectRef obj) -> relax::Var {
-    if (auto opt_str = obj.as<String>()) {
+  auto normalize_key = [&](ffi::Any obj) -> relax::Var {
+    if (auto opt_str = obj.as<ffi::String>()) {
       std::string str = opt_str.value();
       auto it = string_lookup.find(str);
       CHECK(it != string_lookup.end())
@@ -76,18 +77,17 @@ std::tuple<Map<Var, Expr>, Map<tir::Var, PrimExpr>> NormalizeNamedBindings(
       LOG(FATAL)
           << "Expected bound parameter to be a relax::Var, "
           << " or a string that uniquely identifies a relax::Var param within the function.  "
-          << "However, received object " << obj << " of type " << obj->GetTypeKey();
+          << "However, received object " << obj << " of type " << obj.GetTypeKey();
     }
   };
-  auto normalize_value = [&](Var key, ObjectRef obj) -> relax::Expr {
+  auto normalize_value = [&](Var key, ffi::Any obj) -> relax::Expr {
     if (auto opt = obj.as<relax::Expr>()) {
       return opt.value();
-    } else if (auto opt = obj.as<runtime::NDArray>()) {
+    } else if (auto opt = obj.as<runtime::Tensor>()) {
       const auto& span = SpanUtils::CreateWithAttr(msc_attr::kName, key->name_hint());
       return Constant(opt.value(), StructInfo(), span);
     } else {
-      LOG(FATAL) << "Cannot coerce object of type " << obj->GetTypeKey()
-                 << " into relax expression";
+      LOG(FATAL) << "Cannot coerce object of type " << obj.GetTypeKey() << " into relax expression";
     }
   };
 
@@ -96,7 +96,7 @@ std::tuple<Map<Var, Expr>, Map<tir::Var, PrimExpr>> NormalizeNamedBindings(
   }
 
   arith::Analyzer analyzer;
-  Map<tir::Var, PrimExpr> symbolic_var_map = InferSymbolicVarMap(relax_var_remap, &analyzer);
+  ffi::Map<tir::Var, PrimExpr> symbolic_var_map = InferSymbolicVarMap(relax_var_remap, &analyzer);
 
   return {relax_var_remap, symbolic_var_map};
 }
@@ -107,7 +107,8 @@ std::tuple<Map<Var, Expr>, Map<tir::Var, PrimExpr>> NormalizeNamedBindings(
  * \param params params dict
  * \return Function
  */
-Function FunctionBindNamedParams(Function func, const Map<ObjectRef, ObjectRef>& untyped_params) {
+Function FunctionBindNamedParams(Function func,
+                                 const ffi::Map<ObjectRef, ObjectRef>& untyped_params) {
   auto [bind_dict, symbolic_var_map] = NormalizeNamedBindings(func, untyped_params);
 
   Expr bound_expr = Bind(func, bind_dict, symbolic_var_map);
@@ -121,40 +122,47 @@ Function FunctionBindNamedParams(Function func, const Map<ObjectRef, ObjectRef>&
  * \param param The param dict
  * \return The module after binding params.
  */
-IRModule BindNamedParam(IRModule m, String func_name, Map<ObjectRef, ObjectRef> bind_params) {
+IRModule BindNamedParam(IRModule m, ffi::String func_name,
+                        ffi::Map<ObjectRef, ObjectRef> bind_params) {
   IRModuleNode* new_module = m.CopyOnWrite();
-  Map<GlobalVar, BaseFunc> functions = m->functions;
+  ffi::Map<GlobalVar, BaseFunc> functions = m->functions;
   for (const auto& func_pr : functions) {
     if (const auto* relax_f = func_pr.second.as<FunctionNode>()) {
       if (relax_f->GetLinkageType() == LinkageType::kExternal) {
         // Use global_symbol if it's external linkage
-        Optional<String> gsymbol = relax_f->GetAttr<String>(tvm::attr::kGlobalSymbol);
-        if (gsymbol.defined() && gsymbol.value() == func_name) {
-          Function f_after_bind = FunctionBindNamedParams(GetRef<Function>(relax_f), bind_params);
+        ffi::Optional<ffi::String> gsymbol =
+            relax_f->GetAttr<ffi::String>(tvm::attr::kGlobalSymbol);
+        if (gsymbol.has_value() && gsymbol.value() == func_name) {
+          Function f_after_bind =
+              FunctionBindNamedParams(ffi::GetRef<Function>(relax_f), bind_params);
           new_module->Update(func_pr.first, f_after_bind);
         }
       } else {
         // Use global var's name_hint if it's internal linkage
         if (func_pr.first->name_hint == func_name) {
-          Function f_after_bind = FunctionBindNamedParams(GetRef<Function>(relax_f), bind_params);
+          Function f_after_bind =
+              FunctionBindNamedParams(ffi::GetRef<Function>(relax_f), bind_params);
           new_module->Update(func_pr.first, f_after_bind);
         }
       }
     }
   }
-  return GetRef<IRModule>(new_module);
+  return ffi::GetRef<IRModule>(new_module);
 }
 
 namespace transform {
 
-Pass BindNamedParams(String func_name, Map<ObjectRef, ObjectRef> params) {
+Pass BindNamedParams(ffi::String func_name, ffi::Map<ObjectRef, ObjectRef> params) {
   auto pass_func = [=](IRModule mod, PassContext pc) {
     return BindNamedParam(std::move(mod), func_name, params);
   };
   return CreateModulePass(pass_func, 0, "BindNamedParams", {});
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.transform.BindNamedParams").set_body_typed(BindNamedParams);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.transform.BindNamedParams", BindNamedParams);
+}
 
 }  // namespace transform
 
