@@ -22,6 +22,7 @@
  * \brief Pass for inline Exprs.
  */
 
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/relax/expr.h>
 #include <tvm/relax/expr_functor.h>
 #include <tvm/relax/transform.h>
@@ -39,7 +40,8 @@ using namespace tvm::contrib::msc;
  */
 class ParamsInliner : public ExprMutator {
  public:
-  explicit ParamsInliner(IRModule ctx_module, const String& entry_name) : ExprMutator(ctx_module) {
+  explicit ParamsInliner(IRModule ctx_module, const ffi::String& entry_name)
+      : ExprMutator(ctx_module) {
     mod_ = ctx_module;
     entry_name_ = entry_name;
   }
@@ -53,22 +55,22 @@ class ParamsInliner : public ExprMutator {
         continue;
       }
       if (func->IsInstance<FunctionNode>()) {
-        Array<Var> new_params;
-        Array<String> attrs;
+        ffi::Array<Var> new_params;
+        ffi::Array<ffi::String> attrs;
         for (const auto& p : Downcast<Function>(func)->params) {
           auto struct_info = GetStructInfo(p);
           if (struct_info->IsInstance<ShapeStructInfoNode>()) {
             continue;
           }
           if (struct_info->IsInstance<FuncStructInfoNode>()) {
-            const auto& optype_opt = func->GetAttr<String>(msc_attr::kOptype);
-            ICHECK(optype_opt.defined())
+            const auto& optype_opt = func->GetAttr<ffi::String>(msc_attr::kOptype);
+            ICHECK(optype_opt.has_value())
                 << "Can not find attr " << msc_attr::kOptype << " form extern func";
             extern_types_.Set(p, optype_opt.value());
             continue;
           }
           if (const auto* tuple_info = struct_info.as<TupleStructInfoNode>()) {
-            Array<StructInfo> new_fields;
+            ffi::Array<StructInfo> new_fields;
             for (const auto& i : tuple_info->fields) {
               if (i->IsInstance<TensorStructInfoNode>()) {
                 new_fields.push_back(i);
@@ -87,7 +89,7 @@ class ParamsInliner : public ExprMutator {
           continue;
         }
         const auto& new_func = Downcast<Function>(VisitExpr(func));
-        Map<String, ffi::Any> func_attrs = new_func->attrs->dict;
+        ffi::Map<ffi::String, ffi::Any> func_attrs = new_func->attrs->dict;
         if (attrs.size() > 0) {
           func_attrs.Set(msc_attr::kOpattrs, attrs);
         }
@@ -104,7 +106,7 @@ class ParamsInliner : public ExprMutator {
   }
 
   void VisitBinding_(const VarBindingNode* binding, const CallNode* call_node) final {
-    Array<Expr> new_args;
+    ffi::Array<Expr> new_args;
     bool has_inline = false;
     for (const auto& a : call_node->args) {
       auto struct_info = GetStructInfo(a);
@@ -123,8 +125,8 @@ class ParamsInliner : public ExprMutator {
         has_inline = true;
       } else if (call_node->op->IsInstance<GlobalVarNode>() && a->IsInstance<TupleNode>()) {
         const auto& tuple = Downcast<Tuple>(a);
-        Array<Expr> new_fields;
-        Array<StructInfo> new_infos;
+        ffi::Array<Expr> new_fields;
+        ffi::Array<StructInfo> new_infos;
 
         for (const auto& f : tuple->fields) {
           if (f->IsInstance<VarNode>()) {
@@ -151,7 +153,7 @@ class ParamsInliner : public ExprMutator {
       ReEmitBinding(binding, builder_->Normalize(new_call));
     } else if (const auto* gv_node = call_node->op.as<GlobalVarNode>()) {
       const auto& func_info = Downcast<FuncStructInfo>(gv_node->struct_info_);
-      Array<StructInfo> params_info;
+      ffi::Array<StructInfo> params_info;
       for (const auto& a : new_args) {
         ICHECK(a->struct_info_.defined())
             << "Global func argument without defined struct info " << a;
@@ -163,28 +165,31 @@ class ParamsInliner : public ExprMutator {
           Call(call_node->op, new_args, call_node->attrs, call_node->sinfo_args, call_node->span);
       ReEmitBinding(binding, builder_->Normalize(new_call));
     } else {
-      LOG_FATAL << "Unexpected shape consumer " << GetRef<Call>(call_node);
+      LOG_FATAL << "Unexpected shape consumer " << ffi::GetRef<Call>(call_node);
     }
   }
 
  private:
   IRModule mod_;
-  String entry_name_;
-  Map<Expr, String> extern_types_;
+  ffi::String entry_name_;
+  ffi::Map<Expr, ffi::String> extern_types_;
 };
 
-IRModule InlineParams(IRModule mod, const String& entry_name) {
+IRModule InlineParams(IRModule mod, const ffi::String& entry_name) {
   return ParamsInliner(mod, entry_name).Bind();
 }
 
 namespace transform {
 
-Pass InlineParams(const String& entry_name) {
+Pass InlineParams(const ffi::String& entry_name) {
   auto pass_func = [=](IRModule m, PassContext pc) { return relax::InlineParams(m, entry_name); };
   return CreateModulePass(pass_func, 0, "InlineParams", {});
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.transform.InlineParams").set_body_typed(InlineParams);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.transform.InlineParams", InlineParams);
+}
 
 }  // namespace transform
 }  // namespace relax
