@@ -21,6 +21,7 @@
  * \file src/ir/analysis.cc
  * \brief Analysis functions that must span multiple IR types
  */
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/ir/analysis.h>
 
 #include "../support/ordered_set.h"
@@ -28,22 +29,25 @@
 namespace tvm {
 namespace ir {
 
-Map<GlobalVar, Array<GlobalVar>> CollectCallMap(const IRModule& mod) {
+ffi::Map<GlobalVar, ffi::Array<GlobalVar>> CollectCallMap(const IRModule& mod) {
   struct CalleeCollectorImpl : CalleeCollector {
     void Mark(GlobalVar gvar) override { gvars.push_back(gvar); }
     support::OrderedSet<GlobalVar, ObjectPtrHash, ObjectPtrEqual> gvars;
   };
 
-  Map<GlobalVar, Array<GlobalVar>> call_map;
+  ffi::Map<GlobalVar, ffi::Array<GlobalVar>> call_map;
   for (const auto& [gvar, base_func] : mod->functions) {
     CalleeCollectorImpl collector;
     CalleeCollector::vtable()(base_func, &collector);
-    call_map.Set(gvar, Array<GlobalVar>{collector.gvars.begin(), collector.gvars.end()});
+    call_map.Set(gvar, ffi::Array<GlobalVar>{collector.gvars.begin(), collector.gvars.end()});
   }
   return call_map;
 }
 
-TVM_FFI_REGISTER_GLOBAL("ir.analysis.CollectCallMap").set_body_typed(CollectCallMap);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("ir.analysis.CollectCallMap", CollectCallMap);
+}
 
 }  // namespace ir
 }  // namespace tvm
