@@ -21,6 +21,7 @@
  * \brief The implementation of the source map data structure.
  */
 #include <tvm/ffi/function.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/ir/source_map.h>
 #include <tvm/ir/transform.h>
 
@@ -28,22 +29,31 @@
 
 namespace tvm {
 
-TVM_FFI_STATIC_INIT_BLOCK({
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
   SourceNameNode::RegisterReflection();
   SpanNode::RegisterReflection();
   SequentialSpanNode::RegisterReflection();
   SourceNode::RegisterReflection();
   SourceMapObj::RegisterReflection();
-});
+  // overrride SourceNameNode to serialization mechanism
+  refl::TypeAttrDef<SourceNameNode>()
+      .def("__data_to_json__",
+           [](const SourceNameNode* node) {
+             // simply save as the string
+             return node->name;
+           })
+      .def("__data_from_json__", SourceName::Get);
+}
 
-ObjectPtr<Object> GetSourceNameNode(const String& name) {
+ObjectPtr<SourceNameNode> GetSourceNameNode(const ffi::String& name) {
   // always return pointer as the reference can change as map re-allocate.
   // or use another level of indirection by creating a unique_ptr
-  static std::unordered_map<String, ObjectPtr<SourceNameNode>> source_map;
+  static std::unordered_map<ffi::String, ObjectPtr<SourceNameNode>> source_map;
 
   auto sn = source_map.find(name);
   if (sn == source_map.end()) {
-    ObjectPtr<SourceNameNode> n = make_object<SourceNameNode>();
+    ObjectPtr<SourceNameNode> n = ffi::make_object<SourceNameNode>();
     source_map[name] = n;
     n->name = std::move(name);
     return n;
@@ -52,13 +62,16 @@ ObjectPtr<Object> GetSourceNameNode(const String& name) {
   }
 }
 
-ObjectPtr<Object> GetSourceNameNodeByStr(const std::string& name) {
+ObjectPtr<SourceNameNode> GetSourceNameNodeByStr(const std::string& name) {
   return GetSourceNameNode(name);
 }
 
-SourceName SourceName::Get(const String& name) { return SourceName(GetSourceNameNode(name)); }
+SourceName SourceName::Get(const ffi::String& name) { return SourceName(GetSourceNameNode(name)); }
 
-TVM_FFI_REGISTER_GLOBAL("ir.SourceName").set_body_typed(SourceName::Get);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("ir.SourceName", SourceName::Get);
+}
 
 TVM_STATIC_IR_FUNCTOR(ReprPrinter, vtable)
     .set_dispatch<SourceNameNode>([](const ObjectRef& ref, ReprPrinter* p) {
@@ -66,14 +79,8 @@ TVM_STATIC_IR_FUNCTOR(ReprPrinter, vtable)
       p->stream << "SourceName(" << node->name << ", " << node << ")";
     });
 
-TVM_REGISTER_NODE_TYPE(SourceNameNode)
-    .set_creator(GetSourceNameNodeByStr)
-    .set_repr_bytes([](const Object* n) -> std::string {
-      return static_cast<const SourceNameNode*>(n)->name;
-    });
-
 Span::Span(SourceName source_name, int line, int end_line, int column, int end_column) {
-  auto n = make_object<SpanNode>();
+  auto n = ffi::make_object<SpanNode>();
   n->source_name = std::move(source_name);
   n->line = line;
   n->end_line = end_line;
@@ -92,11 +99,9 @@ Span Span::Merge(const Span& other) const {
               std::max((*this)->end_column, other->end_column));
 }
 
-TVM_REGISTER_NODE_TYPE(SpanNode);
-
-SequentialSpan::SequentialSpan(tvm::Array<Span> spans) {
-  auto n = make_object<SequentialSpanNode>();
-  tvm::Array<Span> tmp_spans;
+SequentialSpan::SequentialSpan(tvm::ffi::Array<Span> spans) {
+  auto n = ffi::make_object<SequentialSpanNode>();
+  tvm::ffi::Array<Span> tmp_spans;
   for (const Span& s : spans) {
     if (const SequentialSpanNode* seq_s = s.as<SequentialSpanNode>()) {
       tmp_spans.insert(tmp_spans.end(), seq_s->spans.begin(), seq_s->spans.end());
@@ -115,9 +120,9 @@ SequentialSpan::SequentialSpan(tvm::Array<Span> spans) {
 }
 
 SequentialSpan::SequentialSpan(std::initializer_list<Span> init) {
-  auto n = make_object<SequentialSpanNode>();
-  tvm::Array<Span> spans = tvm::Array<Span>(init);
-  tvm::Array<Span> tmp_spans;
+  auto n = ffi::make_object<SequentialSpanNode>();
+  tvm::ffi::Array<Span> spans = tvm::ffi::Array<Span>(init);
+  tvm::ffi::Array<Span> tmp_spans;
   for (const Span& s : spans) {
     if (const SequentialSpanNode* seq_s = s.as<SequentialSpanNode>()) {
       tmp_spans.insert(tmp_spans.end(), seq_s->spans.begin(), seq_s->spans.end());
@@ -135,16 +140,15 @@ SequentialSpan::SequentialSpan(std::initializer_list<Span> init) {
   data_ = std::move(n);
 }
 
-TVM_REGISTER_NODE_TYPE(SequentialSpanNode);
-
-TVM_FFI_REGISTER_GLOBAL("ir.Span").set_body_typed([](SourceName source_name, int line, int end_line,
-                                                     int column, int end_column) {
-  return Span(source_name, line, end_line, column, end_column);
-});
-
-TVM_FFI_REGISTER_GLOBAL("ir.SequentialSpan").set_body_typed([](tvm::Array<Span> spans) {
-  return SequentialSpan(spans);
-});
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def("ir.Span",
+           [](SourceName source_name, int line, int end_line, int column, int end_column) {
+             return Span(source_name, line, end_line, column, end_column);
+           })
+      .def("ir.SequentialSpan", [](tvm::ffi::Array<Span> spans) { return SequentialSpan(spans); });
+}
 
 TVM_STATIC_IR_FUNCTOR(ReprPrinter, vtable)
     .set_dispatch<SpanNode>([](const ObjectRef& ref, ReprPrinter* p) {
@@ -166,11 +170,9 @@ TVM_STATIC_IR_FUNCTOR(ReprPrinter, vtable)
       p->stream << node->spans[last] << " ])";
     });
 
-TVM_REGISTER_NODE_TYPE(SourceNode);
-
 /*! \brief Construct a source from a string. */
 Source::Source(SourceName src_name, std::string source) {
-  auto n = make_object<SourceNode>();
+  auto n = ffi::make_object<SourceNode>();
   n->source_name = std::move(src_name);
   n->source = std::move(source);
 
@@ -199,7 +201,7 @@ Source::Source(SourceName src_name, std::string source) {
   data_ = n;
 }
 
-tvm::String Source::GetLine(int line) {
+tvm::ffi::String Source::GetLine(int line) {
   VLOG(1) << "Source::GetLine: line=" << line;
   ICHECK(line - 1 < static_cast<int64_t>((*this)->line_map.size()))
       << "requested line: " << line << "at index: " << (line - 1)
@@ -210,28 +212,28 @@ tvm::String Source::GetLine(int line) {
   int line_start = range.first;
   int line_length = range.second;
   VLOG(1) << "Source::GetLine: line_start=" << line_start << " line_length=" << line_length;
-  // TODO(@jroesch): expose substring on tvm::String.
+  // TODO(@jroesch): expose substring on tvm::ffi::String.
   auto line_text = std::string((*this)->source).substr(line_start, line_length);
   VLOG(1) << "Source::GetLine: line_text=" << line_text;
   return line_text;
 }
 
-TVM_REGISTER_NODE_TYPE(SourceMapObj);
-
-SourceMap::SourceMap(Map<SourceName, Source> source_map) {
-  auto n = make_object<SourceMapObj>();
+SourceMap::SourceMap(ffi::Map<SourceName, Source> source_map) {
+  auto n = ffi::make_object<SourceMapObj>();
   n->source_map = std::move(source_map);
   data_ = std::move(n);
 }
 
 void SourceMap::Add(const Source& source) { (*this)->source_map.Set(source->source_name, source); }
 
-TVM_FFI_REGISTER_GLOBAL("SourceMapAdd")
-    .set_body_typed([](SourceMap map, String name, String content) {
-      auto src_name = SourceName::Get(name);
-      Source source(src_name, content);
-      map.Add(source);
-      return src_name;
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("SourceMapAdd", [](SourceMap map, ffi::String name, ffi::String content) {
+    auto src_name = SourceName::Get(name);
+    Source source(src_name, content);
+    map.Add(source);
+    return src_name;
+  });
+}
 
 }  // namespace tvm
