@@ -23,6 +23,7 @@
  */
 #include <dmlc/thread_local.h>
 #include <tvm/ffi/function.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/ffi/rvalue_ref.h>
 #include <tvm/ir/transform.h>
 #include <tvm/node/repr_printer.h>
@@ -53,7 +54,9 @@ struct PassContextThreadLocalEntry {
   /*! \brief The current pass context. */
   std::stack<PassContext> context_stack;
 
-  PassContextThreadLocalEntry() { default_context = PassContext(make_object<PassContextNode>()); }
+  PassContextThreadLocalEntry() {
+    default_context = PassContext(ffi::make_object<PassContextNode>());
+  }
 };
 
 /*! \brief Thread local store to hold the pass context. */
@@ -85,7 +88,7 @@ PassContext PassContext::Current() {
 }
 
 // linearly scan the pass array to match pass_name
-bool PassArrayContains(const Array<String>& pass_array, const std::string& pass_name) {
+bool PassArrayContains(const ffi::Array<ffi::String>& pass_array, const std::string& pass_name) {
   for (auto x : pass_array) {
     if (x == pass_name) return true;
   }
@@ -106,18 +109,17 @@ bool PassContext::PassEnabled(const PassInfo& info) const {
 
 class PassConfigManager {
  public:
-  void Register(std::string key, uint32_t value_type_index,
+  void Register(std::string key, ffi::String value_type_str,
                 std::function<ffi::Any(ffi::Any)> legalization) {
     ICHECK_EQ(key2vtype_.count(key), 0U);
     ValueTypeInfo info;
-    info.type_index = value_type_index;
-    info.type_key = runtime::Object::TypeIndex2Key(value_type_index);
+    info.type_str = value_type_str;
     info.legalization = legalization;
     key2vtype_[key] = info;
   }
 
   // Trying to validate and legalize a config.
-  void Legalize(Map<String, ffi::Any>* config) {
+  void Legalize(ffi::Map<ffi::String, ffi::Any>* config) {
     std::vector<std::pair<std::string, ffi::Any>> update;
     for (auto [key, value] : *config) {
       auto it = key2vtype_.find(key);
@@ -149,11 +151,11 @@ class PassConfigManager {
     }
   }
 
-  Map<String, Map<String, String>> ListConfigs() {
-    Map<String, Map<String, String>> configs;
+  ffi::Map<ffi::String, ffi::Map<ffi::String, ffi::String>> ListConfigs() {
+    ffi::Map<ffi::String, ffi::Map<ffi::String, ffi::String>> configs;
     for (const auto& kv : key2vtype_) {
-      Map<String, String> metadata;
-      metadata.Set("type", kv.second.type_key);
+      ffi::Map<ffi::String, ffi::String> metadata;
+      metadata.Set("type", kv.second.type_str);
       configs.Set(kv.first, metadata);
     }
     return configs;
@@ -166,28 +168,27 @@ class PassConfigManager {
 
  private:
   struct ValueTypeInfo {
-    std::string type_key;
-    uint32_t type_index;
+    std::string type_str;
     std::function<ffi::Any(ffi::Any)> legalization;
   };
 
   std::unordered_map<std::string, ValueTypeInfo> key2vtype_;
 };
 
-void PassContext::RegisterConfigOption(const char* key, uint32_t value_type_index,
+void PassContext::RegisterConfigOption(const char* key, ffi::String value_type_str,
                                        std::function<ffi::Any(ffi::Any)> legalization) {
-  PassConfigManager::Global()->Register(key, value_type_index, legalization);
+  PassConfigManager::Global()->Register(key, value_type_str, legalization);
 }
 
-Map<String, Map<String, String>> PassContext::ListConfigs() {
+ffi::Map<ffi::String, ffi::Map<ffi::String, ffi::String>> PassContext::ListConfigs() {
   return PassConfigManager::Global()->ListConfigs();
 }
 
-PassContext PassContext::Create() { return PassContext(make_object<PassContextNode>()); }
+PassContext PassContext::Create() { return PassContext(ffi::make_object<PassContextNode>()); }
 
 namespace {
 struct ClearOnError {
-  Array<instrument::PassInstrument>* instruments{nullptr};
+  ffi::Array<instrument::PassInstrument>* instruments{nullptr};
 
   ~ClearOnError() {
     if (instruments) {
@@ -245,7 +246,7 @@ struct ExitPassSuccesses {
 
   bool all_initialized{false};
   std::vector<instrument::PassInstrument> successes;
-  Array<instrument::PassInstrument>* instruments{nullptr};
+  ffi::Array<instrument::PassInstrument>* instruments{nullptr};
 };
 }  // namespace
 
@@ -367,20 +368,19 @@ class ModulePassNode : public PassNode {
    * \brief Get the pass information/meta data.
    */
   PassInfo Info() const override { return pass_info; }
-
-  static constexpr const char* _type_key = "transform.ModulePass";
-  TVM_DECLARE_FINAL_OBJECT_INFO(ModulePassNode, PassNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("transform.ModulePass", ModulePassNode, PassNode);
 };
 
 class ModulePass : public Pass {
  public:
   ModulePass(std::function<IRModule(IRModule, PassContext)> pass_func, PassInfo pass_info);
 
-  TVM_DEFINE_OBJECT_REF_METHODS(ModulePass, Pass, ModulePassNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(ModulePass, Pass, ModulePassNode);
 };
 
-PassInfo::PassInfo(int opt_level, String name, tvm::Array<String> required, bool traceable) {
-  auto pass_info = make_object<PassInfoNode>();
+PassInfo::PassInfo(int opt_level, ffi::String name, tvm::ffi::Array<ffi::String> required,
+                   bool traceable) {
+  auto pass_info = ffi::make_object<PassInfoNode>();
   pass_info->opt_level = opt_level;
   pass_info->name = std::move(name);
   pass_info->required = std::move(required);
@@ -390,7 +390,7 @@ PassInfo::PassInfo(int opt_level, String name, tvm::Array<String> required, bool
 
 ModulePass::ModulePass(std::function<IRModule(IRModule, PassContext)> pass_func,
                        PassInfo pass_info) {
-  auto n = make_object<ModulePassNode>();
+  auto n = ffi::make_object<ModulePassNode>();
   n->pass_func = std::move(pass_func);
   n->pass_info = std::move(pass_info);
   data_ = std::move(n);
@@ -430,15 +430,15 @@ IRModule ModulePassNode::operator()(IRModule mod, const PassContext& pass_ctx) c
   return mod;
 }
 
-Sequential::Sequential(tvm::Array<Pass> passes, PassInfo pass_info) {
-  auto n = make_object<SequentialNode>();
+Sequential::Sequential(tvm::ffi::Array<Pass> passes, PassInfo pass_info) {
+  auto n = ffi::make_object<SequentialNode>();
   n->passes = std::move(passes);
   n->pass_info = std::move(pass_info);
   data_ = std::move(n);
 }
 
-Sequential::Sequential(tvm::Array<Pass> passes, String name) {
-  auto n = make_object<SequentialNode>();
+Sequential::Sequential(tvm::ffi::Array<Pass> passes, ffi::String name) {
+  auto n = ffi::make_object<SequentialNode>();
   n->passes = std::move(passes);
   PassInfo pass_info = PassInfo(0, std::move(name), {}, /* traceable */ false);
   n->pass_info = std::move(pass_info);
@@ -458,7 +458,7 @@ void SequentialNode::ResolveDependency(const IRModule& mod) {
              << "\n";
 }
 
-Pass GetPass(const String& pass_name) {
+Pass GetPass(const ffi::String& pass_name) {
   std::optional<tvm::ffi::Function> f;
   if (pass_name.operator std::string().find("transform.") != std::string::npos) {
     f = tvm::ffi::Function::GetGlobal(pass_name);
@@ -493,22 +493,22 @@ IRModule SequentialNode::operator()(IRModule mod, const PassContext& pass_ctx) c
 }
 
 Pass CreateModulePass(std::function<IRModule(IRModule, PassContext)> pass_func, int opt_level,
-                      String name, tvm::Array<String> required, bool traceable) {
+                      ffi::String name, tvm::ffi::Array<ffi::String> required, bool traceable) {
   PassInfo pass_info = PassInfo(opt_level, name, required, traceable);
   return ModulePass(std::move(pass_func), pass_info);
 }
 
-TVM_REGISTER_NODE_TYPE(PassInfoNode);
-
-TVM_FFI_REGISTER_GLOBAL("transform.PassInfo")
-    .set_body_typed([](int opt_level, String name, tvm::Array<String> required, bool traceable) {
-      return PassInfo(opt_level, name, required, traceable);
-    });
-
-TVM_FFI_REGISTER_GLOBAL("transform.Info").set_body_packed([](ffi::PackedArgs args, ffi::Any* ret) {
-  Pass pass = args[0].cast<Pass>();
-  *ret = pass->Info();
-});
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def("transform.PassInfo",
+           [](int opt_level, ffi::String name, tvm::ffi::Array<ffi::String> required,
+              bool traceable) { return PassInfo(opt_level, name, required, traceable); })
+      .def_packed("transform.Info", [](ffi::PackedArgs args, ffi::Any* ret) {
+        Pass pass = args[0].cast<Pass>();
+        *ret = pass->Info();
+      });
+}
 
 TVM_STATIC_IR_FUNCTOR(ReprPrinter, vtable)
     .set_dispatch<PassInfoNode>([](const ObjectRef& ref, tvm::ReprPrinter* p) {
@@ -528,27 +528,27 @@ TVM_STATIC_IR_FUNCTOR(ReprPrinter, vtable)
       }
     });
 
-TVM_FFI_STATIC_INIT_BLOCK({
+TVM_FFI_STATIC_INIT_BLOCK() {
   PassContextNode::RegisterReflection();
   PassInfoNode::RegisterReflection();
   SequentialNode::RegisterReflection();
   ModulePassNode::RegisterReflection();
-});
+}
 
-TVM_REGISTER_NODE_TYPE(ModulePassNode);
-
-TVM_FFI_REGISTER_GLOBAL("transform.MakeModulePass")
-    .set_body_typed(
-        [](ffi::TypedFunction<IRModule(ffi::RValueRef<IRModule>, PassContext)> pass_func,
-           PassInfo pass_info) {
-          auto wrapped_pass_func = [pass_func](IRModule mod, PassContext ctx) {
-            return pass_func(ffi::RValueRef<IRModule>(std::move(mod)), ctx);
-          };
-          return ModulePass(wrapped_pass_func, pass_info);
-        });
-
-TVM_FFI_REGISTER_GLOBAL("transform.RunPass")
-    .set_body_typed([](Pass pass, ffi::RValueRef<IRModule> mod) { return pass(*std::move(mod)); });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def("transform.MakeModulePass",
+           [](ffi::TypedFunction<IRModule(ffi::RValueRef<IRModule>, PassContext)> pass_func,
+              PassInfo pass_info) {
+             auto wrapped_pass_func = [pass_func](IRModule mod, PassContext ctx) {
+               return pass_func(ffi::RValueRef<IRModule>(std::move(mod)), ctx);
+             };
+             return ModulePass(wrapped_pass_func, pass_info);
+           })
+      .def("transform.RunPass",
+           [](Pass pass, ffi::RValueRef<IRModule> mod) { return pass(*std::move(mod)); });
+}
 
 TVM_STATIC_IR_FUNCTOR(ReprPrinter, vtable)
     .set_dispatch<ModulePassNode>([](const ObjectRef& ref, ReprPrinter* p) {
@@ -558,18 +558,18 @@ TVM_STATIC_IR_FUNCTOR(ReprPrinter, vtable)
                 << info->opt_level;
     });
 
-TVM_REGISTER_NODE_TYPE(SequentialNode);
-
-TVM_FFI_REGISTER_GLOBAL("transform.Sequential")
-    .set_body_packed([](ffi::PackedArgs args, ffi::Any* ret) {
-      auto passes = args[0].cast<tvm::Array<Pass>>();
-      int opt_level = args[1].cast<int>();
-      std::string name = args[2].cast<std::string>();
-      auto required = args[3].cast<tvm::Array<String>>();
-      bool traceable = args[4].cast<bool>();
-      PassInfo pass_info = PassInfo(opt_level, name, required, /* traceable */ traceable);
-      *ret = Sequential(passes, pass_info);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def_packed("transform.Sequential", [](ffi::PackedArgs args, ffi::Any* ret) {
+    auto passes = args[0].cast<tvm::ffi::Array<Pass>>();
+    int opt_level = args[1].cast<int>();
+    std::string name = args[2].cast<std::string>();
+    auto required = args[3].cast<tvm::ffi::Array<ffi::String>>();
+    bool traceable = args[4].cast<bool>();
+    PassInfo pass_info = PassInfo(opt_level, name, required, /* traceable */ traceable);
+    *ret = Sequential(passes, pass_info);
+  });
+}
 
 TVM_STATIC_IR_FUNCTOR(ReprPrinter, vtable)
     .set_dispatch<SequentialNode>([](const ObjectRef& ref, ReprPrinter* p) {
@@ -585,25 +585,27 @@ TVM_STATIC_IR_FUNCTOR(ReprPrinter, vtable)
       p->stream << "]";
     });
 
-TVM_REGISTER_NODE_TYPE(PassContextNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def(
+      "transform.PassContext",
+      [](int opt_level, ffi::Array<ffi::String> required, ffi::Array<ffi::String> disabled,
+         ffi::Array<instrument::PassInstrument> instruments,
+         ffi::Optional<ffi::Map<ffi::String, ffi::Any>> config) {
+        auto pctx = PassContext::Create();
+        pctx->opt_level = opt_level;
 
-TVM_FFI_REGISTER_GLOBAL("transform.PassContext")
-    .set_body_typed([](int opt_level, Array<String> required, Array<String> disabled,
-                       Array<instrument::PassInstrument> instruments,
-                       Optional<Map<String, ffi::Any>> config) {
-      auto pctx = PassContext::Create();
-      pctx->opt_level = opt_level;
+        pctx->required_pass = std::move(required);
+        pctx->disabled_pass = std::move(disabled);
+        pctx->instruments = std::move(instruments);
 
-      pctx->required_pass = std::move(required);
-      pctx->disabled_pass = std::move(disabled);
-      pctx->instruments = std::move(instruments);
-
-      if (config.defined()) {
-        pctx->config = config.value();
-      }
-      PassConfigManager::Global()->Legalize(&(pctx->config));
-      return pctx;
-    });
+        if (config.defined()) {
+          pctx->config = config.value();
+        }
+        PassConfigManager::Global()->Legalize(&(pctx->config));
+        return pctx;
+      });
+}
 
 TVM_STATIC_IR_FUNCTOR(ReprPrinter, vtable)
     .set_dispatch<PassContextNode>([](const ObjectRef& ref, ReprPrinter* p) {
@@ -626,22 +628,21 @@ class PassContext::Internal {
   static void ExitScope(PassContext pass_ctx) { pass_ctx.ExitWithScope(); }
 };
 
-TVM_FFI_REGISTER_GLOBAL("transform.GetCurrentPassContext").set_body_typed(PassContext::Current);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def("transform.GetCurrentPassContext", PassContext::Current)
+      .def("transform.EnterPassContext", PassContext::Internal::EnterScope)
+      .def("transform.ExitPassContext", PassContext::Internal::ExitScope)
+      .def("transform.OverrideInstruments",
+           [](PassContext pass_ctx, ffi::Array<instrument::PassInstrument> instruments) {
+             pass_ctx.InstrumentExitPassContext();
+             pass_ctx->instruments = instruments;
+             pass_ctx.InstrumentEnterPassContext();
+           });
+}
 
-TVM_FFI_REGISTER_GLOBAL("transform.EnterPassContext")
-    .set_body_typed(PassContext::Internal::EnterScope);
-
-TVM_FFI_REGISTER_GLOBAL("transform.ExitPassContext")
-    .set_body_typed(PassContext::Internal::ExitScope);
-
-TVM_FFI_REGISTER_GLOBAL("transform.OverrideInstruments")
-    .set_body_typed([](PassContext pass_ctx, Array<instrument::PassInstrument> instruments) {
-      pass_ctx.InstrumentExitPassContext();
-      pass_ctx->instruments = instruments;
-      pass_ctx.InstrumentEnterPassContext();
-    });
-
-Pass PrintIR(String header, bool show_meta_data) {
+Pass PrintIR(ffi::String header, bool show_meta_data) {
   auto pass_func = [header, show_meta_data](IRModule mod, const PassContext& ctx) {
     LOG(INFO) << "PrintIR(" << header << "):\n" << mod;
     return mod;
@@ -649,9 +650,12 @@ Pass PrintIR(String header, bool show_meta_data) {
   return CreateModulePass(pass_func, 0, "PrintIR", {}, /* traceable */ false);
 }
 
-TVM_FFI_REGISTER_GLOBAL("transform.PrintIR").set_body_typed(PrintIR);
-
-TVM_FFI_REGISTER_GLOBAL("transform.ListConfigs").set_body_typed(PassContext::ListConfigs);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def("transform.PrintIR", PrintIR)
+      .def("transform.ListConfigs", PassContext::ListConfigs);
+}
 
 }  // namespace transform
 }  // namespace tvm
