@@ -22,84 +22,84 @@
  * \brief Common type system AST nodes throughout the IR.
  */
 #include <tvm/ffi/function.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/ir/type.h>
 namespace tvm {
 
-TVM_FFI_STATIC_INIT_BLOCK({
+TVM_FFI_STATIC_INIT_BLOCK() {
+  TypeNode::RegisterReflection();
   PrimTypeNode::RegisterReflection();
   PointerTypeNode::RegisterReflection();
   TupleTypeNode::RegisterReflection();
   FuncTypeNode::RegisterReflection();
   TensorMapTypeNode::RegisterReflection();
-});
+}
 
 PrimType::PrimType(runtime::DataType dtype, Span span) {
-  ObjectPtr<PrimTypeNode> n = make_object<PrimTypeNode>();
+  ObjectPtr<PrimTypeNode> n = ffi::make_object<PrimTypeNode>();
   n->dtype = dtype;
   n->span = std::move(span);
   data_ = std::move(n);
 }
 
-TVM_REGISTER_NODE_TYPE(PrimTypeNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("ir.PrimType", [](runtime::DataType dtype) { return PrimType(dtype); });
+}
 
-TVM_FFI_REGISTER_GLOBAL("ir.PrimType").set_body_typed([](runtime::DataType dtype) {
-  return PrimType(dtype);
-});
-
-PointerType::PointerType(Type element_type, String storage_scope) {
-  ObjectPtr<PointerTypeNode> n = make_object<PointerTypeNode>();
+PointerType::PointerType(Type element_type, ffi::String storage_scope) {
+  ObjectPtr<PointerTypeNode> n = ffi::make_object<PointerTypeNode>();
+  if (storage_scope.empty()) {
+    n->storage_scope = "global";
+  } else {
+    n->storage_scope = std::move(storage_scope);
+  }
   n->element_type = std::move(element_type);
-  n->storage_scope = std::move(storage_scope);
   data_ = std::move(n);
 }
 
-TVM_REGISTER_NODE_TYPE(PointerTypeNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("ir.PointerType", [](Type element_type, ffi::String storage_scope = "") {
+    return PointerType(element_type, storage_scope);
+  });
+}
 
-TVM_FFI_REGISTER_GLOBAL("ir.PointerType")
-    .set_body_typed([](Type element_type, String storage_scope = "") {
-      return PointerType(element_type, storage_scope);
-    });
-
-FuncType::FuncType(tvm::Array<Type> arg_types, Type ret_type, Span span) {
-  ObjectPtr<FuncTypeNode> n = make_object<FuncTypeNode>();
+FuncType::FuncType(tvm::ffi::Array<Type> arg_types, Type ret_type, Span span) {
+  ObjectPtr<FuncTypeNode> n = ffi::make_object<FuncTypeNode>();
   n->arg_types = std::move(arg_types);
   n->ret_type = std::move(ret_type);
   n->span = std::move(span);
   data_ = std::move(n);
 }
 
-TVM_REGISTER_NODE_TYPE(FuncTypeNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("ir.FuncType", [](tvm::ffi::Array<Type> arg_types, Type ret_type) {
+    return FuncType(arg_types, ret_type);
+  });
+}
 
-TVM_FFI_REGISTER_GLOBAL("ir.FuncType")
-    .set_body_typed([](tvm::Array<Type> arg_types, Type ret_type) {
-      return FuncType(arg_types, ret_type);
-    });
-
-TupleType::TupleType(Array<Type> fields, Span span) {
-  ObjectPtr<TupleTypeNode> n = make_object<TupleTypeNode>();
+TupleType::TupleType(ffi::Array<Type> fields, Span span) {
+  ObjectPtr<TupleTypeNode> n = ffi::make_object<TupleTypeNode>();
   n->fields = std::move(fields);
   n->span = std::move(span);
   data_ = std::move(n);
 }
 
-TupleType TupleType::Empty() { return TupleType(Array<Type>()); }
+TupleType TupleType::Empty() { return TupleType(ffi::Array<Type>()); }
 
-TVM_REGISTER_NODE_TYPE(TupleTypeNode);
-
-TVM_FFI_REGISTER_GLOBAL("ir.TupleType").set_body_typed([](Array<Type> fields) {
-  return TupleType(fields);
-});
-
-TVM_FFI_REGISTER_GLOBAL("ir.TensorMapType").set_body_typed([](Span span) {
-  return TensorMapType(span);
-});
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def("ir.TupleType", [](ffi::Array<Type> fields) { return TupleType(fields); })
+      .def("ir.TensorMapType", [](Span span) { return TensorMapType(span); });
+}
 
 TensorMapType::TensorMapType(Span span) {
-  ObjectPtr<TensorMapTypeNode> n = make_object<TensorMapTypeNode>();
+  ObjectPtr<TensorMapTypeNode> n = ffi::make_object<TensorMapTypeNode>();
   n->span = std::move(span);
   data_ = std::move(n);
 }
 
-TVM_REGISTER_NODE_TYPE(TensorMapTypeNode);
-
 }  // namespace tvm
