@@ -16,29 +16,32 @@
  * specific language governing permissions and limitations
  * under the License.
  */
+#include <tvm/ffi/reflection/registry.h>
+
 #include "../utils.h"
 
 namespace tvm {
 namespace meta_schedule {
 
-void PyCostModelNode::Load(const String& path) {
+void PyCostModelNode::Load(const ffi::String& path) {
   ICHECK(f_load != nullptr) << "PyCostModel's Load method not implemented!";
   f_load(path);
 }
 
-void PyCostModelNode::Save(const String& path) {
+void PyCostModelNode::Save(const ffi::String& path) {
   ICHECK(f_save != nullptr) << "PyCostModel's Save method not implemented!";
   f_save(path);
 }
 
-void PyCostModelNode::Update(const TuneContext& context, const Array<MeasureCandidate>& candidates,
-                             const Array<RunnerResult>& results) {
+void PyCostModelNode::Update(const TuneContext& context,
+                             const ffi::Array<MeasureCandidate>& candidates,
+                             const ffi::Array<RunnerResult>& results) {
   ICHECK(f_update != nullptr) << "PyCostModel's Update method not implemented!";
   f_update(context, candidates, results);
 }
 
 std::vector<double> PyCostModelNode::Predict(const TuneContext& context,
-                                             const Array<MeasureCandidate>& candidates) {
+                                             const ffi::Array<MeasureCandidate>& candidates) {
   ICHECK(f_predict != nullptr) << "PyCostModel's Predict method not implemented!";
   std::vector<double> result(candidates.size(), 0.0);
   f_predict(context, candidates, result.data());
@@ -50,7 +53,7 @@ CostModel CostModel::PyCostModel(PyCostModelNode::FLoad f_load,        //
                                  PyCostModelNode::FUpdate f_update,    //
                                  PyCostModelNode::FPredict f_predict,  //
                                  PyCostModelNode::FAsString f_as_string) {
-  ObjectPtr<PyCostModelNode> n = make_object<PyCostModelNode>();
+  ObjectPtr<PyCostModelNode> n = ffi::make_object<PyCostModelNode>();
   n->f_load = std::move(f_load);
   n->f_save = std::move(f_save);
   n->f_update = std::move(f_update);
@@ -68,22 +71,22 @@ TVM_STATIC_IR_FUNCTOR(ReprPrinter, vtable)
       p->stream << f_as_string();
     });
 
-TVM_REGISTER_OBJECT_TYPE(CostModelNode);
-TVM_REGISTER_NODE_TYPE(PyCostModelNode);
-
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.CostModelLoad").set_body_method(&CostModelNode::Load);
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.CostModelSave").set_body_method(&CostModelNode::Save);
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.CostModelUpdate").set_body_method(&CostModelNode::Update);
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.CostModelPredict")
-    .set_body_typed([](CostModel model,                     //
-                       const TuneContext& context,          //
-                       Array<MeasureCandidate> candidates,  //
-                       void* p_addr) -> void {
-      std::vector<double> result = model->Predict(context, candidates);
-      std::copy(result.begin(), result.end(), static_cast<double*>(p_addr));
-    });
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.CostModelPyCostModel")
-    .set_body_typed(CostModel::PyCostModel);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def_method("meta_schedule.CostModelLoad", &CostModelNode::Load)
+      .def_method("meta_schedule.CostModelSave", &CostModelNode::Save)
+      .def_method("meta_schedule.CostModelUpdate", &CostModelNode::Update)
+      .def("meta_schedule.CostModelPredict",
+           [](CostModel model,                          //
+              const TuneContext& context,               //
+              ffi::Array<MeasureCandidate> candidates,  //
+              void* p_addr) -> void {
+             std::vector<double> result = model->Predict(context, candidates);
+             std::copy(result.begin(), result.end(), static_cast<double*>(p_addr));
+           })
+      .def("meta_schedule.CostModelPyCostModel", CostModel::PyCostModel);
+}
 
 }  // namespace meta_schedule
 }  // namespace tvm
