@@ -16,6 +16,8 @@
  * specific language governing permissions and limitations
  * under the License.
  */
+#include <tvm/ffi/reflection/registry.h>
+
 #include "../module_equality.h"
 #include "../utils.h"
 
@@ -44,20 +46,20 @@ ObjectRef WorkloadNode::AsJSON() const {
   // Dump the JSON string to base64
   std::string b64_mod = Base64Encode(json_mod);
   // Output
-  return Array<ObjectRef>{SHash2Str(this->shash), String(b64_mod)};
+  return ffi::Array<ffi::Any>{SHash2Str(this->shash), ffi::String(b64_mod)};
 }
 
 Workload Workload::FromJSON(const ObjectRef& json_obj) {
-  IRModule mod{nullptr};
+  IRModule mod{ffi::UnsafeInit()};
   THashCode shash = 0;
   try {
     const ffi::ArrayObj* json_array = json_obj.as<ffi::ArrayObj>();
     CHECK(json_array && json_array->size() == 2);
     // Load json[0] => shash
-    String str_shash = json_array->at(0).cast<String>();
+    ffi::String str_shash = json_array->at(0).cast<ffi::String>();
     // Load json[1] => mod
     {
-      String b64_mod = json_array->at(1).cast<String>();
+      ffi::String b64_mod = json_array->at(1).cast<ffi::String>();
       std::string json_mod = Base64Decode(b64_mod);
       mod = LoadJSON(json_mod).cast<IRModule>();
       std::stringstream(str_shash) >> shash;
@@ -71,9 +73,11 @@ Workload Workload::FromJSON(const ObjectRef& json_obj) {
 
 /******** TuningRecord ********/
 
-TuningRecord::TuningRecord(tir::Trace trace, Workload workload, Optional<Array<FloatImm>> run_secs,
-                           Optional<Target> target, Optional<Array<ArgInfo>> args_info) {
-  ObjectPtr<TuningRecordNode> n = make_object<TuningRecordNode>();
+TuningRecord::TuningRecord(tir::Trace trace, Workload workload,
+                           ffi::Optional<ffi::Array<FloatImm>> run_secs,
+                           ffi::Optional<Target> target,
+                           ffi::Optional<ffi::Array<ArgInfo>> args_info) {
+  ObjectPtr<TuningRecordNode> n = ffi::make_object<TuningRecordNode>();
   n->trace = trace;
   n->workload = workload;
   n->run_secs = run_secs;
@@ -94,10 +98,10 @@ MeasureCandidate TuningRecordNode::AsMeasureCandidate() const {
 }
 
 ObjectRef TuningRecordNode::AsJSON() const {
-  Optional<Array<ObjectRef>> json_args_info;
-  Optional<ObjectRef> json_target;
+  ffi::Optional<ffi::Array<ObjectRef>> json_args_info;
+  ffi::Optional<ObjectRef> json_target;
   if (args_info.defined()) {
-    Array<ObjectRef> info;
+    ffi::Array<ObjectRef> info;
     info.reserve(args_info.value().size());
     for (const ArgInfo& arg_info : args_info.value()) {
       info.push_back(arg_info->AsJSON());
@@ -107,10 +111,10 @@ ObjectRef TuningRecordNode::AsJSON() const {
   if (target.defined()) {
     json_target = target.value()->Export();
   }
-  return Array<ObjectRef>{trace->AsJSON(false),  //
-                          run_secs,              //
-                          json_target,           //
-                          json_args_info};
+  return ffi::Array<ObjectRef>{trace->AsJSON(false),  //
+                               run_secs,              //
+                               json_target,           //
+                               json_args_info};
 }
 
 bool TuningRecordNode::IsValid() const {
@@ -129,10 +133,10 @@ bool TuningRecordNode::IsValid() const {
 }
 
 TuningRecord TuningRecord::FromJSON(const ObjectRef& json_obj, const Workload& workload) {
-  tir::Trace trace{nullptr};
-  Optional<Array<FloatImm>> run_secs;
-  Optional<Target> target;
-  Optional<Array<ArgInfo>> args_info;
+  tir::Trace trace{ffi::UnsafeInit()};
+  ffi::Optional<ffi::Array<FloatImm>> run_secs;
+  ffi::Optional<Target> target;
+  ffi::Optional<ffi::Array<ArgInfo>> args_info;
   try {
     const ffi::ArrayObj* json_array = json_obj.as<ffi::ArrayObj>();
     CHECK(json_array && json_array->size() == 4);
@@ -142,12 +146,12 @@ TuningRecord TuningRecord::FromJSON(const ObjectRef& json_obj, const Workload& w
     }
     // Load json[2] => target
     if (json_array->at(2) != nullptr) {
-      target = Target(json_array->at(2).cast<Map<String, ffi::Any>>());
+      target = Target(json_array->at(2).cast<ffi::Map<ffi::String, ffi::Any>>());
     }
     // Load json[3] => args_info
     if (json_array->at(3) != nullptr) {
       const ffi::ArrayObj* json_args_info = json_array->at(3).cast<const ffi::ArrayObj*>();
-      Array<ArgInfo> info;
+      ffi::Array<ArgInfo> info;
       info.reserve(json_args_info->size());
       for (Any json_arg_info : *json_args_info) {
         info.push_back(ArgInfo::FromJSON(json_arg_info.cast<ObjectRef>()));
@@ -171,15 +175,18 @@ TuningRecord TuningRecord::FromJSON(const ObjectRef& json_obj, const Workload& w
 }
 
 /******** Database ********/
-DatabaseNode::DatabaseNode(String mod_eq_name) { mod_eq_ = ModuleEquality::Create(mod_eq_name); }
+DatabaseNode::DatabaseNode(ffi::String mod_eq_name) {
+  mod_eq_ = ModuleEquality::Create(mod_eq_name);
+}
 DatabaseNode::~DatabaseNode() = default;
 
-Optional<TuningRecord> DatabaseNode::QueryTuningRecord(const IRModule& mod, const Target& target,
-                                                       const String& workload_name) {
+ffi::Optional<TuningRecord> DatabaseNode::QueryTuningRecord(const IRModule& mod,
+                                                            const Target& target,
+                                                            const ffi::String& workload_name) {
   if (!this->HasWorkload(mod)) {
     return std::nullopt;
   }
-  Array<TuningRecord> records = this->GetTopK(this->CommitWorkload(mod), 1);
+  ffi::Array<TuningRecord> records = this->GetTopK(this->CommitWorkload(mod), 1);
   if (records.empty()) {
     return std::nullopt;
   }
@@ -187,9 +194,10 @@ Optional<TuningRecord> DatabaseNode::QueryTuningRecord(const IRModule& mod, cons
   return records[0];
 }
 
-Optional<tir::Schedule> DatabaseNode::QuerySchedule(const IRModule& mod, const Target& target,
-                                                    const String& workload_name) {
-  if (Optional<TuningRecord> opt_record = this->QueryTuningRecord(mod, target, workload_name)) {
+ffi::Optional<tir::Schedule> DatabaseNode::QuerySchedule(const IRModule& mod, const Target& target,
+                                                         const ffi::String& workload_name) {
+  if (ffi::Optional<TuningRecord> opt_record =
+          this->QueryTuningRecord(mod, target, workload_name)) {
     TuningRecord record = opt_record.value();
     tir::Schedule sch =
         tir::Schedule::Traced(record->workload->mod, /*seed=*/-1, /*debug_mask=*/0,
@@ -201,9 +209,9 @@ Optional<tir::Schedule> DatabaseNode::QuerySchedule(const IRModule& mod, const T
   }
 }
 
-Optional<IRModule> DatabaseNode::QueryIRModule(const IRModule& mod, const Target& target,
-                                               const String& workload_name) {
-  if (Optional<tir::Schedule> opt_sch = this->QuerySchedule(mod, target, workload_name)) {
+ffi::Optional<IRModule> DatabaseNode::QueryIRModule(const IRModule& mod, const Target& target,
+                                                    const ffi::String& workload_name) {
+  if (ffi::Optional<tir::Schedule> opt_sch = this->QuerySchedule(mod, target, workload_name)) {
     return opt_sch.value()->mod();
   } else {
     return std::nullopt;
@@ -242,7 +250,7 @@ void Database::EnterWithScope() { ThreadLocalDatabases()->push_back(*this); }
 
 void Database::ExitWithScope() { ThreadLocalDatabases()->pop_back(); }
 
-Optional<Database> Database::Current() {
+ffi::Optional<Database> Database::Current() {
   std::vector<Database>* tls = ThreadLocalDatabases();
   if (tls->empty()) {
     return std::nullopt;
@@ -252,7 +260,7 @@ Optional<Database> Database::Current() {
 }
 
 /******** PyDatabase ********/
-PyDatabaseNode::PyDatabaseNode(String mod_eq_name) : DatabaseNode(mod_eq_name) {}
+PyDatabaseNode::PyDatabaseNode(ffi::String mod_eq_name) : DatabaseNode(mod_eq_name) {}
 
 Database Database::PyDatabase(PyDatabaseNode::FHasWorkload f_has_workload,
                               PyDatabaseNode::FCommitWorkload f_commit_workload,
@@ -262,8 +270,8 @@ Database Database::PyDatabase(PyDatabaseNode::FHasWorkload f_has_workload,
                               PyDatabaseNode::FQueryTuningRecord f_query_tuning_record,
                               PyDatabaseNode::FQuerySchedule f_query_schedule,
                               PyDatabaseNode::FQueryIRModule f_query_ir_module,
-                              PyDatabaseNode::FSize f_size, String mod_eq_name) {
-  ObjectPtr<PyDatabaseNode> n = make_object<PyDatabaseNode>(mod_eq_name);
+                              PyDatabaseNode::FSize f_size, ffi::String mod_eq_name) {
+  ObjectPtr<PyDatabaseNode> n = ffi::make_object<PyDatabaseNode>(mod_eq_name);
   n->f_has_workload = f_has_workload;
   n->f_commit_workload = f_commit_workload;
   n->f_commit_tuning_record = f_commit_tuning_record;
@@ -278,56 +286,42 @@ Database Database::PyDatabase(PyDatabaseNode::FHasWorkload f_has_workload,
 
 /******** FFI ********/
 
-TVM_FFI_STATIC_INIT_BLOCK({
+TVM_FFI_STATIC_INIT_BLOCK() {
   WorkloadNode::RegisterReflection();
   TuningRecordNode::RegisterReflection();
   PyDatabaseNode::RegisterReflection();
-});
+}
 
-TVM_REGISTER_NODE_TYPE(WorkloadNode);
-TVM_REGISTER_NODE_TYPE(TuningRecordNode);
-TVM_REGISTER_OBJECT_TYPE(DatabaseNode);
-TVM_REGISTER_NODE_TYPE(PyDatabaseNode);
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.Workload").set_body_typed([](IRModule mod) {
-  return Workload(mod);
-});
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.WorkloadAsJSON").set_body_method(&WorkloadNode::AsJSON);
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.WorkloadFromJSON").set_body_typed(&Workload::FromJSON);
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.TuningRecord")
-    .set_body_typed([](tir::Trace trace, Workload workload, Optional<Array<FloatImm>> run_secs,
-                       Optional<Target> target, Optional<Array<ArgInfo>> args_info) {
-      return TuningRecord(trace, workload, run_secs, target, args_info);
-    });
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.TuningRecordAsMeasureCandidate")
-    .set_body_method(&TuningRecordNode::AsMeasureCandidate);
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.TuningRecordAsJSON")
-    .set_body_method(&TuningRecordNode::AsJSON);
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.TuningRecordFromJSON")
-    .set_body_typed(TuningRecord::FromJSON);
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseEnterWithScope")
-    .set_body_method(&Database::EnterWithScope);
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseExitWithScope")
-    .set_body_method(&Database::ExitWithScope);
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseCurrent").set_body_typed(Database::Current);
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseHasWorkload")
-    .set_body_method(&DatabaseNode::HasWorkload);
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseCommitWorkload")
-    .set_body_method(&DatabaseNode::CommitWorkload);
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseCommitTuningRecord")
-    .set_body_method(&DatabaseNode::CommitTuningRecord);
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseGetTopK").set_body_method(&DatabaseNode::GetTopK);
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseGetAllTuningRecords")
-    .set_body_method(&DatabaseNode::GetAllTuningRecords);
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseSize").set_body_method(&DatabaseNode::Size);
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseQueryTuningRecord")
-    .set_body_method(&DatabaseNode::QueryTuningRecord);
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseQuerySchedule")
-    .set_body_method(&DatabaseNode::QuerySchedule);
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseQueryIRModule")
-    .set_body_method(&DatabaseNode::QueryIRModule);
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabaseDumpPruned")
-    .set_body_method(&DatabaseNode::DumpPruned);
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.DatabasePyDatabase").set_body_typed(Database::PyDatabase);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def("meta_schedule.Workload", [](IRModule mod) { return Workload(mod); })
+      .def_method("meta_schedule.WorkloadAsJSON", &WorkloadNode::AsJSON)
+      .def("meta_schedule.WorkloadFromJSON", &Workload::FromJSON)
+      .def("meta_schedule.TuningRecord",
+           [](tir::Trace trace, Workload workload, ffi::Optional<ffi::Array<FloatImm>> run_secs,
+              ffi::Optional<Target> target, ffi::Optional<ffi::Array<ArgInfo>> args_info) {
+             return TuningRecord(trace, workload, run_secs, target, args_info);
+           })
+      .def_method("meta_schedule.TuningRecordAsMeasureCandidate",
+                  &TuningRecordNode::AsMeasureCandidate)
+      .def_method("meta_schedule.TuningRecordAsJSON", &TuningRecordNode::AsJSON)
+      .def("meta_schedule.TuningRecordFromJSON", TuningRecord::FromJSON)
+      .def_method("meta_schedule.DatabaseEnterWithScope", &Database::EnterWithScope)
+      .def_method("meta_schedule.DatabaseExitWithScope", &Database::ExitWithScope)
+      .def("meta_schedule.DatabaseCurrent", Database::Current)
+      .def_method("meta_schedule.DatabaseHasWorkload", &DatabaseNode::HasWorkload)
+      .def_method("meta_schedule.DatabaseCommitWorkload", &DatabaseNode::CommitWorkload)
+      .def_method("meta_schedule.DatabaseCommitTuningRecord", &DatabaseNode::CommitTuningRecord)
+      .def_method("meta_schedule.DatabaseGetTopK", &DatabaseNode::GetTopK)
+      .def_method("meta_schedule.DatabaseGetAllTuningRecords", &DatabaseNode::GetAllTuningRecords)
+      .def_method("meta_schedule.DatabaseSize", &DatabaseNode::Size)
+      .def_method("meta_schedule.DatabaseQueryTuningRecord", &DatabaseNode::QueryTuningRecord)
+      .def_method("meta_schedule.DatabaseQuerySchedule", &DatabaseNode::QuerySchedule)
+      .def_method("meta_schedule.DatabaseQueryIRModule", &DatabaseNode::QueryIRModule)
+      .def_method("meta_schedule.DatabaseDumpPruned", &DatabaseNode::DumpPruned)
+      .def("meta_schedule.DatabasePyDatabase", Database::PyDatabase);
+}
 
 }  // namespace meta_schedule
 }  // namespace tvm
