@@ -43,8 +43,8 @@ void JSONDumps(Any json_obj, std::ostringstream& os) {
   } else if (auto opt_float_imm = json_obj.try_cast<FloatImm>()) {
     FloatImm float_imm = *std::move(opt_float_imm);
     os << std::setprecision(20) << float_imm->value;
-  } else if (const auto* str = json_obj.as<ffi::StringObj>()) {
-    os << '"' << support::StrEscape(str->data, str->size) << '"';
+  } else if (auto opt_str = json_obj.as<ffi::String>()) {
+    os << '"' << support::StrEscape((*opt_str).data(), (*opt_str).size()) << '"';
   } else if (const auto* array = json_obj.as<ffi::ArrayObj>()) {
     os << "[";
     int n = array->size();
@@ -57,10 +57,10 @@ void JSONDumps(Any json_obj, std::ostringstream& os) {
     os << "]";
   } else if (const auto* dict = json_obj.as<ffi::MapObj>()) {
     int n = dict->size();
-    std::vector<std::pair<String, ffi::Any>> key_values;
+    std::vector<std::pair<ffi::String, ffi::Any>> key_values;
     key_values.reserve(n);
     for (const auto& kv : *dict) {
-      if (auto key = kv.first.try_cast<String>()) {
+      if (auto key = kv.first.try_cast<ffi::String>()) {
         key_values.emplace_back(key.value(), kv.second);
       } else {
         LOG(FATAL) << "TypeError: Only string keys are supported in JSON dumps, but got: "
@@ -75,13 +75,13 @@ void JSONDumps(Any json_obj, std::ostringstream& os) {
       if (i != 0) {
         os << ",";
       }
-      os << '"' << support::StrEscape(kv.first->data, kv.first->size) << '"';
+      os << '"' << support::StrEscape(kv.first.data(), kv.first.size()) << '"';
       os << ":";
       JSONDumps(kv.second, os);
     }
     os << "}";
   } else if (json_obj.as<tir::IndexMapNode>()) {
-    JSONDumps(String(SaveJSON(json_obj)), os);
+    JSONDumps(ffi::String(SaveJSON(json_obj)), os);
   } else {
     LOG(FATAL) << "TypeError: Unsupported type in JSON object: " << json_obj.GetTypeKey();
   }
@@ -241,7 +241,7 @@ class JSONTokenizer {
       LOG(FATAL) << "ValueError: Unexpected end of string";
     }
     ++cur_;
-    *token = Token{TokenType::kString, String(str)};
+    *token = Token{TokenType::kString, ffi::String(str)};
     return true;
   }
 
@@ -315,9 +315,9 @@ class JSONParser {
     }
   }
 
-  Array<Any> ParseArray() {
+  ffi::Array<Any> ParseArray() {
     bool is_first = true;
-    Array<Any> results;
+    ffi::Array<Any> results;
     for (;;) {
       Token token;
       if (is_first) {
@@ -347,9 +347,9 @@ class JSONParser {
     return results;
   }
 
-  Map<String, ffi::Any> ParseDict() {
+  ffi::Map<ffi::String, ffi::Any> ParseDict() {
     bool is_first = true;
-    Map<String, ffi::Any> results;
+    ffi::Map<ffi::String, ffi::Any> results;
     for (;;) {
       Token token;
       if (is_first) {
@@ -371,12 +371,12 @@ class JSONParser {
         }
         // Case 3
         Any key = ParseObject(std::move(token));
-        ICHECK(key.as<ffi::StringObj>()) << "ValueError: key must be a string, but gets: " << key;
+        ICHECK(key.as<ffi::String>()) << "ValueError: key must be a string, but gets: " << key;
         token = tokenizer_.Next();
         CHECK(token.type == TokenType::kColon)
             << "ValueError: Unexpected token before: " << tokenizer_.cur_;
         Any value = ParseObject(tokenizer_.Next());
-        results.Set(Downcast<String>(key), value);
+        results.Set(Downcast<ffi::String>(key), value);
         continue;
       } else {
         LOG(FATAL) << "ValueError: Unexpected token before: " << tokenizer_.cur_;
