@@ -18,6 +18,8 @@
  */
 #include "module_equality.h"
 
+#include <tvm/ffi/extra/structural_equal.h>
+#include <tvm/ffi/extra/structural_hash.h>
 #include <tvm/ir/module.h>
 #include <tvm/node/structural_equal.h>
 #include <tvm/node/structural_hash.h>
@@ -25,8 +27,6 @@
 
 #include <memory>
 
-#include "../node/ndarray_hash_equal.h"
-
 namespace tvm {
 namespace meta_schedule {
 
@@ -34,62 +34,53 @@ class ModuleEqualityStructural : public ModuleEquality {
  public:
   size_t Hash(IRModule mod) const { return tvm::StructuralHash()(mod); }
   bool Equal(IRModule lhs, IRModule rhs) const { return tvm::StructuralEqual()(lhs, rhs); }
-  String GetName() const { return "structural"; }
+  ffi::String GetName() const { return "structural"; }
 };
 
-class SEqualHandlerIgnoreNDArray : public SEqualHandlerDefault {
+class ModuleEqualityIgnoreTensor : public ModuleEquality {
  public:
-  SEqualHandlerIgnoreNDArray() : SEqualHandlerDefault(false, nullptr, false) {}
-
- protected:
-  bool DispatchSEqualReduce(const ObjectRef& lhs, const ObjectRef& rhs, bool map_free_vars,
-                            const Optional<ObjectPathPair>& current_paths) {
-    if (auto lhs_ptr = lhs.as<runtime::NDArray::Container>(),
-        rhs_ptr = rhs.as<runtime::NDArray::Container>();
-        lhs_ptr && rhs_ptr) {
-      SEqualReducer reducer(this, nullptr, map_free_vars);
-      return NDArrayEqual(lhs_ptr, rhs_ptr, reducer, false);
-    }
-    return SEqualHandlerDefault::DispatchSEqualReduce(lhs, rhs, map_free_vars, current_paths);
+  size_t Hash(IRModule mod) const {
+    return tvm::ffi::StructuralHash::Hash(mod, /*map_free_vars=*/false,
+                                          /*skip_tensor_content=*/true);
   }
-};
-
-class ModuleEqualityIgnoreNDArray : public ModuleEquality {
- public:
-  size_t Hash(IRModule mod) const { return SHashHandlerIgnoreNDArray().Hash(mod, false); }
   bool Equal(IRModule lhs, IRModule rhs) const {
-    return SEqualHandlerIgnoreNDArray().Equal(lhs, rhs, false);
+    return tvm::ffi::StructuralEqual::Equal(lhs, rhs, /*map_free_vars=*/false,
+                                            /*skip_tensor_content=*/true);
   }
-  String GetName() const { return "ignore-ndarray"; }
+  ffi::String GetName() const { return "ignore-tensor"; }
 };
 
-// The NDArray-ignoring variant of structural equal / hash is used for the module equality
+// The Tensor-ignoring variant of structural equal / hash is used for the module equality
 // on the extracted anchor blocks.
 class ModuleEqualityAnchorBlock : public ModuleEquality {
   size_t Hash(IRModule mod) const {
     auto anchor_block = tir::FindAnchorBlock(mod);
     if (anchor_block) {
-      return SHashHandlerIgnoreNDArray().Hash(GetRef<tir::Block>(anchor_block), false);
+      return ffi::StructuralHash::Hash(ffi::GetRef<tir::Block>(anchor_block),
+                                       /*map_free_vars=*/false,
+                                       /*skip_tensor_content=*/true);
     }
-    return ModuleEqualityIgnoreNDArray().Hash(mod);
+    return ModuleEqualityIgnoreTensor().Hash(mod);
   }
   bool Equal(IRModule lhs, IRModule rhs) const {
     auto anchor_block_lhs = tir::FindAnchorBlock(lhs);
     auto anchor_block_rhs = tir::FindAnchorBlock(rhs);
     if (anchor_block_lhs && anchor_block_rhs) {
-      return SEqualHandlerIgnoreNDArray().Equal(GetRef<tir::Block>(anchor_block_lhs),
-                                                GetRef<tir::Block>(anchor_block_rhs), false);
+      return tvm::ffi::StructuralEqual::Equal(ffi::GetRef<tir::Block>(anchor_block_lhs),
+                                              ffi::GetRef<tir::Block>(anchor_block_rhs),
+                                              /*map_free_vars=*/false,
+                                              /*skip_tensor_content=*/true);
     }
-    return ModuleEqualityIgnoreNDArray().Equal(lhs, rhs);
+    return ModuleEqualityIgnoreTensor().Equal(lhs, rhs);
   }
-  String GetName() const { return "anchor-block"; }
+  ffi::String GetName() const { return "anchor-block"; }
 };
 
 std::unique_ptr<ModuleEquality> ModuleEquality::Create(const std::string& mod_eq_name) {
   if (mod_eq_name == "structural") {
     return std::make_unique<ModuleEqualityStructural>();
-  } else if (mod_eq_name == "ignore-ndarray") {
-    return std::make_unique<ModuleEqualityIgnoreNDArray>();
+  } else if (mod_eq_name == "ignore-tensor") {
+    return std::make_unique<ModuleEqualityIgnoreTensor>();
   } else if (mod_eq_name == "anchor-block") {
     return std::make_unique<ModuleEqualityAnchorBlock>();
   }
