@@ -17,6 +17,8 @@
  * under the License.
  */
 
+#include <tvm/ffi/reflection/registry.h>
+
 #include "../../tir/schedule/analysis.h"
 #include "../../tir/schedule/transform.h"
 #include "../utils.h"
@@ -29,15 +31,15 @@ namespace meta_schedule {
  * \brief Tile a subset of loops in the block according to the given tensor intrinsic, and annotate
  * the tiled block for tensorization by postproc rewrite.
  */
-Optional<tir::BlockRV> TileForIntrin(tir::Schedule sch, tir::BlockRV block,
-                                     const std::string& intrin_name) {
-  Optional<tir::LoopRV> tiled_loop_rv = TileWithTensorIntrin(sch, block, intrin_name);
+ffi::Optional<tir::BlockRV> TileForIntrin(tir::Schedule sch, tir::BlockRV block,
+                                          const std::string& intrin_name) {
+  ffi::Optional<tir::LoopRV> tiled_loop_rv = TileWithTensorIntrin(sch, block, intrin_name);
   if (!tiled_loop_rv) {
     return std::nullopt;
   }
   ICHECK(tiled_loop_rv.defined());
   tir::BlockRV outer_block = sch->Blockize(tiled_loop_rv.value());
-  sch->Annotate(outer_block, tir::attr::meta_schedule_auto_tensorize, String(intrin_name));
+  sch->Annotate(outer_block, tir::attr::meta_schedule_auto_tensorize, ffi::String(intrin_name));
   return outer_block;
 }
 
@@ -46,7 +48,7 @@ Optional<tir::BlockRV> TileForIntrin(tir::Schedule sch, tir::BlockRV block,
  */
 class MultiLevelTilingWithIntrinNode : public MultiLevelTilingNode {
  protected:
-  Array<tir::Schedule> Apply(const tir::Schedule& sch, const tir::BlockRV& block_rv) final {
+  ffi::Array<tir::Schedule> Apply(const tir::Schedule& sch, const tir::BlockRV& block_rv) final {
     auto desc_func = tir::TensorIntrin::Get(intrin_name).value()->desc;
     if (!CheckAutoTensorizeApplicable(sch, block_rv, desc_func)) {
       TVM_PY_LOG(INFO, logger) << "The workload cannot be tensorized.";
@@ -66,7 +68,7 @@ class MultiLevelTilingWithIntrinNode : public MultiLevelTilingNode {
   // Inherited from ScheduleRuleNode
   ScheduleRule Clone() const final {
     ObjectPtr<MultiLevelTilingWithIntrinNode> n =
-        make_object<MultiLevelTilingWithIntrinNode>(*this);
+        ffi::make_object<MultiLevelTilingWithIntrinNode>(*this);
     return ScheduleRule(n);
   }
 
@@ -84,19 +86,23 @@ class MultiLevelTilingWithIntrinNode : public MultiLevelTilingNode {
   }
 
  public:
-  /*! \brief The name of a tensor intrinsic. */
-  String intrin_name;
+  static void RegisterReflection() {
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<MultiLevelTilingWithIntrinNode>();
+  }
 
-  static constexpr const char* _type_key = "meta_schedule.MultiLevelTilingWithIntrin";
-  TVM_DECLARE_FINAL_OBJECT_INFO(MultiLevelTilingWithIntrinNode, MultiLevelTilingNode);
+  /*! \brief The name of a tensor intrinsic. */
+  ffi::String intrin_name;
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("meta_schedule.MultiLevelTilingWithIntrin",
+                                    MultiLevelTilingWithIntrinNode, MultiLevelTilingNode);
 };
 
-ScheduleRule ScheduleRule::MultiLevelTilingWithIntrin(String intrin_name, String structure,
-                                                      Optional<Array<String>> tile_binds,
-                                                      Optional<Integer> max_innermost_factor,
-                                                      Optional<Array<Integer>> vector_load_lens,
-                                                      Optional<Map<String, ffi::Any>> reuse_read,
-                                                      Optional<Map<String, ffi::Any>> reuse_write) {
+ScheduleRule ScheduleRule::MultiLevelTilingWithIntrin(
+    ffi::String intrin_name, ffi::String structure,
+    ffi::Optional<ffi::Array<ffi::String>> tile_binds, ffi::Optional<Integer> max_innermost_factor,
+    ffi::Optional<ffi::Array<Integer>> vector_load_lens,
+    ffi::Optional<ffi::Map<ffi::String, ffi::Any>> reuse_read,
+    ffi::Optional<ffi::Map<ffi::String, ffi::Any>> reuse_write) {
   ICHECK(tir::TensorIntrin::Get(intrin_name).defined())
       << "Provided tensor intrinsic " << intrin_name << " is not registered.";
   auto node = MultiLevelTilingInitCommon<MultiLevelTilingWithIntrinNode>(
@@ -105,9 +111,12 @@ ScheduleRule ScheduleRule::MultiLevelTilingWithIntrin(String intrin_name, String
   return ScheduleRule(node);
 }
 
-TVM_REGISTER_NODE_TYPE(MultiLevelTilingWithIntrinNode);
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.ScheduleRuleMultiLevelTilingWithIntrin")
-    .set_body_typed(ScheduleRule::MultiLevelTilingWithIntrin);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  MultiLevelTilingWithIntrinNode::RegisterReflection();
+  refl::GlobalDef().def("meta_schedule.ScheduleRuleMultiLevelTilingWithIntrin",
+                        ScheduleRule::MultiLevelTilingWithIntrin);
+}
 
 }  // namespace meta_schedule
 }  // namespace tvm
