@@ -16,7 +16,7 @@
  * specific language governing permissions and limitations
  * under the License.
  */
-#include <tvm/ffi/reflection/reflection.h>
+#include <tvm/ffi/reflection/registry.h>
 
 #include "../utils.h"
 
@@ -33,6 +33,8 @@ class ScheduleFnNode : public SpaceGeneratorNode {
 
   static void RegisterReflection() {
     // `schedule_fn_` is not registered.
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<ScheduleFnNode>();
   }
 
   void InitializeWithTuneContext(const TuneContext& context) final {
@@ -40,7 +42,7 @@ class ScheduleFnNode : public SpaceGeneratorNode {
     this->rand_state_ = ForkSeed(&context->rand_state);
   }
 
-  Array<tir::Schedule> GenerateDesignSpace(const IRModule& mod) final {
+  ffi::Array<tir::Schedule> GenerateDesignSpace(const IRModule& mod) final {
     tir::Schedule sch = tir::Schedule::Traced(
         /*mod=*/mod,
         /*rand_state=*/ForkSeed(&this->rand_state_),
@@ -56,7 +58,7 @@ class ScheduleFnNode : public SpaceGeneratorNode {
       return {sch.value()};
     }
     if (const auto* arr = obj.as<ffi::ArrayObj>()) {
-      Array<tir::Schedule> result;
+      ffi::Array<tir::Schedule> result;
       result.reserve(arr->size());
       for (Any val : *arr) {
         if (auto sch = val.as<tir::Schedule>()) {
@@ -76,20 +78,19 @@ class ScheduleFnNode : public SpaceGeneratorNode {
   }
 
   SpaceGenerator Clone() const final {
-    ObjectPtr<ScheduleFnNode> n = make_object<ScheduleFnNode>(*this);
+    ObjectPtr<ScheduleFnNode> n = ffi::make_object<ScheduleFnNode>(*this);
     CloneRules(this, n.get());
     return SpaceGenerator(n);
   }
 
-  static constexpr const char* _type_key = "meta_schedule.ScheduleFn";
-  TVM_DECLARE_FINAL_OBJECT_INFO(ScheduleFnNode, SpaceGeneratorNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("meta_schedule.ScheduleFn", ScheduleFnNode, SpaceGeneratorNode);
 };
 
-SpaceGenerator SpaceGenerator::ScheduleFn(ffi::Function schedule_fn,
-                                          Optional<Array<ScheduleRule>> sch_rules,
-                                          Optional<Array<Postproc>> postprocs,
-                                          Optional<Map<Mutator, FloatImm>> mutator_probs) {
-  ObjectPtr<ScheduleFnNode> n = make_object<ScheduleFnNode>();
+SpaceGenerator SpaceGenerator::ScheduleFn(
+    ffi::Function schedule_fn, ffi::Optional<ffi::Array<ScheduleRule>> sch_rules,
+    ffi::Optional<ffi::Array<Postproc>> postprocs,
+    ffi::Optional<ffi::Map<Mutator, FloatImm>> mutator_probs) {
+  ObjectPtr<ScheduleFnNode> n = ffi::make_object<ScheduleFnNode>();
   n->sch_rules = std::move(sch_rules);
   n->postprocs = std::move(postprocs);
   n->mutator_probs = std::move(mutator_probs);
@@ -97,11 +98,11 @@ SpaceGenerator SpaceGenerator::ScheduleFn(ffi::Function schedule_fn,
   return SpaceGenerator(n);
 }
 
-TVM_FFI_STATIC_INIT_BLOCK({ ScheduleFnNode::RegisterReflection(); });
-
-TVM_REGISTER_NODE_TYPE(ScheduleFnNode);
-TVM_FFI_REGISTER_GLOBAL("meta_schedule.SpaceGeneratorScheduleFn")
-    .set_body_typed(SpaceGenerator::ScheduleFn);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  ScheduleFnNode::RegisterReflection();
+  refl::GlobalDef().def("meta_schedule.SpaceGeneratorScheduleFn", SpaceGenerator::ScheduleFn);
+}
 
 }  // namespace meta_schedule
 }  // namespace tvm
