@@ -24,6 +24,7 @@
  */
 #include <tvm/arith/analyzer.h>
 #include <tvm/arith/iter_affine_map.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/relax/analysis.h>
 #include <tvm/tir/analysis.h>
 #include <tvm/tir/index_map.h>
@@ -39,8 +40,8 @@ using namespace tir;
 /********** Helper Functions **********/
 
 /*! \brief Checks if a transformation is bijective affine over the given ranges */
-static bool IsBijectiveAffine(const IndexMap& m, const Array<Range>& ranges) {
-  Map<tir::Var, Range> input_iters;
+static bool IsBijectiveAffine(const IndexMap& m, const ffi::Array<Range>& ranges) {
+  ffi::Map<tir::Var, Range> input_iters;
   ICHECK_EQ(m->initial_indices.size(), ranges.size());
   for (size_t i = 0; i < ranges.size(); i++) {
     input_iters.Set(m->initial_indices[i], ranges[i]);
@@ -60,7 +61,7 @@ static bool IsBijectiveAffine(const IndexMap& m, const Array<Range>& ranges) {
  */
 class IndexAnalyzer : public ExprVisitor {
  public:
-  Array<tir::Var> Analyze(const arith::IterSumExpr& expr) {
+  ffi::Array<tir::Var> Analyze(const arith::IterSumExpr& expr) {
     VisitExpr(expr);
     return iterators_;
   }
@@ -85,14 +86,14 @@ class IndexAnalyzer : public ExprVisitor {
 
   void VisitIterMark(const arith::IterMark& op) {
     if (const auto* var = op->source.as<tir::VarNode>())
-      iterators_.push_back(GetRef<tir::Var>(var));
+      iterators_.push_back(ffi::GetRef<tir::Var>(var));
     else
       VisitExpr(op->source);
     VisitExpr(op->extent);
   }
 
  private:
-  Array<tir::Var> iterators_;
+  ffi::Array<tir::Var> iterators_;
 };
 
 /*!
@@ -110,13 +111,13 @@ class IndexAnalyzer : public ExprVisitor {
  * SpatialLayout(A[s0, constant, r0, s1]) = {s0, null, null, s1}
  * SpatialLayout(A[s0 * c + s1]) = undefined
  */
-using SpatialLayout = Array<Optional<tir::Var>>;
+using SpatialLayout = ffi::Array<ffi::Optional<tir::Var>>;
 static SpatialLayout GetSpatialLayout(const arith::IterMapResult& iter_map_result) {
   ICHECK(!iter_map_result->indices.empty());
   SpatialLayout result;
   for (const arith::IterSumExpr& index : iter_map_result->indices) {
     IndexAnalyzer index_analyzer;
-    Array<tir::Var> iter_vars = index_analyzer.Analyze(index);
+    ffi::Array<tir::Var> iter_vars = index_analyzer.Analyze(index);
     if (iter_vars.size() >= 2) {
       LOG(WARNING) << "[LayoutInference] Unable to get spatial layout of access: "
                    << arith::NormalizeIterMapToExpr(index);
@@ -172,7 +173,7 @@ static bool AreIdenticalTransforms(const IndexMap& t0, const IndexMap& t1) {
   if (t0->final_indices.size() != t1->final_indices.size()) return false;
 
   // Create a new shape expression.
-  Array<PrimExpr> t1_initial_indices =
+  ffi::Array<PrimExpr> t1_initial_indices =
       t1->initial_indices.Map([](tir::Var i) -> PrimExpr { return i; });
   arith::Analyzer analyzer;
   auto t0_output = t0->MapIndices(t1_initial_indices, &analyzer);
@@ -212,9 +213,9 @@ static bool AreIdenticalTransforms(const IndexMap& t0, const IndexMap& t1) {
  * target transformation = lambda dim, C, H, W -> (dim, H, W, C // 4, C %4)
  */
 using VarSet = std::unordered_set<tir::Var>;
-static Optional<IndexMap> InferLayoutTransformation(const SpatialLayout& src_spatial_layout,
-                                                    const IndexMap& src_transformation,
-                                                    const SpatialLayout& tgt_spatial_layout) {
+static ffi::Optional<IndexMap> InferLayoutTransformation(const SpatialLayout& src_spatial_layout,
+                                                         const IndexMap& src_transformation,
+                                                         const SpatialLayout& tgt_spatial_layout) {
   // Copy over the src transformation intial and final indices
   auto initial_indices = support::AsList(src_transformation->initial_indices);
   auto final_indices = support::AsList(src_transformation->final_indices);
@@ -243,7 +244,7 @@ static Optional<IndexMap> InferLayoutTransformation(const SpatialLayout& src_spa
   auto final_indices_it = final_indices.begin();
   while (final_indices_it != final_indices.end()) {
     // Collect all the vars used in this final index.
-    Array<tir::Var> used_vars = tir::UndefinedVars(*final_indices_it);
+    ffi::Array<tir::Var> used_vars = tir::UndefinedVars(*final_indices_it);
     ICHECK(!used_vars.empty())
         << "IndexMap expression must always contain tir::Var nodes but found none in: "
         << *final_indices_it;
@@ -317,7 +318,7 @@ static Optional<IndexMap> InferLayoutTransformation(const SpatialLayout& src_spa
  */
 class BlockAnalyzer : public StmtExprVisitor {
  public:
-  explicit BlockAnalyzer(const Block& block, const Map<Buffer, IndexMap>& transformation_cache,
+  explicit BlockAnalyzer(const Block& block, const ffi::Map<Buffer, IndexMap>& transformation_cache,
                          IndexMap write_transformation)
       : can_transform_block_(true),
         write_transformation_(write_transformation),
@@ -379,7 +380,7 @@ class BlockAnalyzer : public StmtExprVisitor {
     }
     block_transformation_ = maybe_block_transformation.value();
 
-    Array<Range> block_ranges = block_->iter_vars.Map([](const IterVar& i) { return i->dom; });
+    ffi::Array<Range> block_ranges = block_->iter_vars.Map([](const IterVar& i) { return i->dom; });
     if (!IsBijectiveAffine(block_transformation_, block_ranges)) {
       can_transform_block_ = false;
       LOG(WARNING) << "[LayoutInference] Inferred block transformation is not bijective affine, "
@@ -436,7 +437,7 @@ class BlockAnalyzer : public StmtExprVisitor {
   };
 
   // Helper to break down the indices of buffer access.
-  SpatialLayout DetectBufferAccessIterMap(Array<PrimExpr> indices) {
+  SpatialLayout DetectBufferAccessIterMap(ffi::Array<PrimExpr> indices) {
     auto result = arith::DetectIterMap(
         /*indices=*/indices, /*input_iters*/ spatial_dom_,
         /*predicate*/ 1, /*check_level*/ arith::IterMapLevel::NoCheck, &arith_analyzer_);
@@ -515,19 +516,19 @@ class BlockAnalyzer : public StmtExprVisitor {
  public:
   bool CanBeTransformed() { return can_transform_block_; }
   IndexMap GetBlockTransformation() { return block_transformation_; }
-  Map<Buffer, IndexMap> GetReadBufferTransformations() { return read_buffer_transformations_; }
+  ffi::Map<Buffer, IndexMap> GetReadBufferTransformations() { return read_buffer_transformations_; }
 
  private:
   bool can_transform_block_;
   IndexMap write_transformation_;
-  Map<tir::Var, Range> spatial_dom_;
+  ffi::Map<tir::Var, Range> spatial_dom_;
   arith::Analyzer arith_analyzer_;
 
   Block block_;
   IndexMap block_transformation_;
 
-  Map<Buffer, IndexMap> read_buffer_transformations_;
-  const Map<Buffer, IndexMap>& buffer_transformation_cache_;
+  ffi::Map<Buffer, IndexMap> read_buffer_transformations_;
+  const ffi::Map<Buffer, IndexMap>& buffer_transformation_cache_;
   std::unordered_map<Buffer, BufferAccessInfo, ObjectPtrHash, ObjectPtrEqual> buffer_access_info_;
 };
 
@@ -541,14 +542,14 @@ class BlockAnalyzer : public StmtExprVisitor {
  */
 class PrimFuncAnalyzer : public StmtExprVisitor {
  public:
-  explicit PrimFuncAnalyzer(const PrimFunc& func, Array<IndexMap> write_transformations) {
+  explicit PrimFuncAnalyzer(const PrimFunc& func, ffi::Array<IndexMap> write_transformations) {
     ICHECK_LE(write_transformations.size(), func->params.size())
         << "Incompatible PrimFunc and write_transformations";
 
     size_t first_write_index = func->params.size() - write_transformations.size();
     for (size_t i = 0; i < write_transformations.size(); ++i) {
       auto param = func->params[first_write_index + i];
-      Optional<Buffer> param_buf = func->buffer_map.Get(param);
+      ffi::Optional<Buffer> param_buf = func->buffer_map.Get(param);
       ICHECK(param_buf.defined());
       ICHECK_EQ(param_buf.value()->shape.size(), write_transformations[i]->initial_indices.size())
           << "Mismatch between output buffer shape and index map";
@@ -556,10 +557,10 @@ class PrimFuncAnalyzer : public StmtExprVisitor {
     }
     VisitStmt(func->body);
   }
-  Map<Block, Map<ObjectRef, IndexMap>> GetSuggestedTransforms() {
-    Map<Block, Map<ObjectRef, IndexMap>> result;
+  ffi::Map<Block, ffi::Map<ObjectRef, IndexMap>> GetSuggestedTransforms() {
+    ffi::Map<Block, ffi::Map<ObjectRef, IndexMap>> result;
     for (const auto& [block, index_map] : block_transformations_) {
-      Map<ObjectRef, IndexMap> block_transformations;
+      ffi::Map<ObjectRef, IndexMap> block_transformations;
       block_transformations.Set(block, index_map);
       for (const auto& buffer : block_to_buffer_[block]) {
         block_transformations.Set(buffer, buffer_transformation_cache_[buffer]);
@@ -577,7 +578,7 @@ class PrimFuncAnalyzer : public StmtExprVisitor {
       return;
     }
 
-    Block block = GetRef<Block>(op);
+    Block block = ffi::GetRef<Block>(op);
     // Get block write buffer transformation.
     if (block->writes.size() != 1) return;
     auto write_buffer = block->writes[0]->buffer;
@@ -600,13 +601,13 @@ class PrimFuncAnalyzer : public StmtExprVisitor {
   }
 
  private:
-  Map<Buffer, IndexMap> buffer_transformation_cache_;
-  Map<Block, IndexMap> block_transformations_;
-  std::unordered_map<Block, Array<Buffer>, ObjectPtrHash, ObjectPtrEqual> block_to_buffer_;
+  ffi::Map<Buffer, IndexMap> buffer_transformation_cache_;
+  ffi::Map<Block, IndexMap> block_transformations_;
+  std::unordered_map<Block, ffi::Array<Buffer>, ObjectPtrHash, ObjectPtrEqual> block_to_buffer_;
 };
 
-Map<tir::Block, Map<ObjectRef, tir::IndexMap>> SuggestLayoutTransforms(
-    const PrimFunc& prim_func, Array<IndexMap> write_buffer_transformations) {
+ffi::Map<tir::Block, ffi::Map<ObjectRef, tir::IndexMap>> SuggestLayoutTransforms(
+    const PrimFunc& prim_func, ffi::Array<IndexMap> write_buffer_transformations) {
   // No changes to the PrimFunc are required if no transformations on output buffers.
   if (write_buffer_transformations.empty()) return {};
 
@@ -614,10 +615,13 @@ Map<tir::Block, Map<ObjectRef, tir::IndexMap>> SuggestLayoutTransforms(
   return analyzer.GetSuggestedTransforms();
 }
 
-TVM_FFI_REGISTER_GLOBAL(("relax.analysis.suggest_layout_transforms"))
-    .set_body_typed([](PrimFunc fn, Array<tir::IndexMap> write_buffer_transformations) {
-      return SuggestLayoutTransforms(fn, write_buffer_transformations);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.analysis.suggest_layout_transforms",
+                        [](PrimFunc fn, ffi::Array<tir::IndexMap> write_buffer_transformations) {
+                          return SuggestLayoutTransforms(fn, write_buffer_transformations);
+                        });
+}
 
 }  // namespace relax
 }  // namespace tvm
