@@ -19,6 +19,8 @@
 
 #include "./pattern_registry.h"
 
+#include <tvm/ffi/reflection/registry.h>
+
 #include "../../support/utils.h"
 
 namespace tvm {
@@ -29,15 +31,15 @@ static std::vector<FusionPattern>* GetRegistryTable() {
   return &table;
 }
 
-void RegisterPatterns(Array<FusionPattern> entries) {
+void RegisterPatterns(ffi::Array<FusionPattern> entries) {
   auto* table = GetRegistryTable();
   for (const auto& entry : entries) {
     table->push_back(entry);
   }
 }
 
-void RemovePatterns(Array<String> names) {
-  std::unordered_set<String> name_set{names.begin(), names.end()};
+void RemovePatterns(ffi::Array<ffi::String> names) {
+  std::unordered_set<ffi::String> name_set{names.begin(), names.end()};
 
   auto* table = GetRegistryTable();
   table->erase(
@@ -46,9 +48,9 @@ void RemovePatterns(Array<String> names) {
       table->end());
 }
 
-Array<FusionPattern> GetPatternsWithPrefix(const String& prefix) {
+ffi::Array<FusionPattern> GetPatternsWithPrefix(const ffi::String& prefix) {
   auto* table = GetRegistryTable();
-  Array<FusionPattern> result;
+  ffi::Array<FusionPattern> result;
   for (auto it = table->rbegin(); it != table->rend(); ++it) {
     if (support::StartsWith((*it)->name, prefix.data())) {
       result.push_back(*it);
@@ -57,7 +59,7 @@ Array<FusionPattern> GetPatternsWithPrefix(const String& prefix) {
   return result;
 }
 
-Optional<FusionPattern> GetPattern(const String& pattern_name) {
+ffi::Optional<FusionPattern> GetPattern(const ffi::String& pattern_name) {
   auto* table = GetRegistryTable();
   for (auto it = table->rbegin(); it != table->rend(); ++it) {
     if ((*it)->name == pattern_name) {
@@ -67,11 +69,14 @@ Optional<FusionPattern> GetPattern(const String& pattern_name) {
   return std::nullopt;
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.backend.RegisterPatterns").set_body_typed(RegisterPatterns);
-TVM_FFI_REGISTER_GLOBAL("relax.backend.RemovePatterns").set_body_typed(RemovePatterns);
-TVM_FFI_REGISTER_GLOBAL("relax.backend.GetPatternsWithPrefix")
-    .set_body_typed(GetPatternsWithPrefix);
-TVM_FFI_REGISTER_GLOBAL("relax.backend.GetPattern").set_body_typed(GetPattern);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def("relax.backend.RegisterPatterns", RegisterPatterns)
+      .def("relax.backend.RemovePatterns", RemovePatterns)
+      .def("relax.backend.GetPatternsWithPrefix", GetPatternsWithPrefix)
+      .def("relax.backend.GetPattern", GetPattern);
+}
 
 }  // namespace backend
 }  // namespace relax
