@@ -44,27 +44,27 @@ using transform::FusionPattern;
  * \param patterns Patterns to be registered. Patterns that appear later in the list have
  *        higher priority when partitioning DataflowBlock.
  */
-void RegisterPatterns(Array<FusionPattern> patterns);
+void RegisterPatterns(ffi::Array<FusionPattern> patterns);
 
 /*!
  * \brief Remove patterns from the registry by their name.
  * \param names The name of patterns to be removed
  */
-void RemovePatterns(Array<String> names);
+void RemovePatterns(ffi::Array<ffi::String> names);
 
 /*!
  * \brief Find patterns whose name starts with a particular prefix.
  * \param prefx The pattern name prefix.
  * \return Matched patterns, ordered by priority from high to low.
  */
-Array<FusionPattern> GetPatternsWithPrefix(const String& prefix);
+ffi::Array<FusionPattern> GetPatternsWithPrefix(const ffi::String& prefix);
 
 /*!
  * \brief Find the pattern with a particular name.
  * \param name The pattern name.
  * \return The matched pattern. std::nullopt if not found.
  */
-Optional<FusionPattern> GetPattern(const String& name);
+ffi::Optional<FusionPattern> GetPattern(const ffi::String& name);
 
 }  // namespace backend
 }  // namespace relax
