@@ -21,6 +21,7 @@
  * \file src/relax/backend/vm/codegen_tir.cc
  * \brief A codegen to generate VMTIR function(that can be compiled) from executable.
  */
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/ir/module.h>
 #include <tvm/relax/exec_builder.h>
 #include <tvm/relax/expr_functor.h>
@@ -49,11 +50,11 @@ using vm::VMFuncInfo;
  * \note Skip CallPacked with special attrs for now, as they can be
  *       further simplified with PrimValue.
  */
-class CodeGenVMTIR : public ExprFunctor<Optional<PrimExpr>(const Expr&)> {
+class CodeGenVMTIR : public ExprFunctor<ffi::Optional<PrimExpr>(const Expr&)> {
  public:
   explicit CodeGenVMTIR(relax::ExecBuilder builder, IRModule ctx_mod)
       : builder_(builder), ctx_mod_(ctx_mod) {
-    system_lib_prefix_ = ctx_mod_->GetAttr<String>(tvm::attr::kSystemLibPrefix);
+    system_lib_prefix_ = ctx_mod_->GetAttr<ffi::String>(tvm::attr::kSystemLibPrefix);
   }
 
   static IRModule Run(relax::ExecBuilder builder, IRModule mod) {
@@ -65,8 +66,8 @@ class CodeGenVMTIR : public ExprFunctor<Optional<PrimExpr>(const Expr&)> {
     // Remove relax function and turn into TIR func.
     for (auto& p : mod->functions) {
       if (auto* func = p.second.as<FunctionNode>()) {
-        auto tir_func = codegen.Codegen(GetRef<Function>(func));
-        auto gsymbol = tir_func->GetAttr<String>(tvm::attr::kGlobalSymbol);
+        auto tir_func = codegen.Codegen(ffi::GetRef<Function>(func));
+        auto gsymbol = tir_func->GetAttr<ffi::String>(tvm::attr::kGlobalSymbol);
         res_mod->Add(GlobalVar(gsymbol.value()), tir_func);
         res_mod->Remove(p.first);
       }
@@ -104,8 +105,9 @@ class CodeGenVMTIR : public ExprFunctor<Optional<PrimExpr>(const Expr&)> {
     stmt_stack_.back().emplace_back(stmt);
   }
 
-  void EmitCallPacked(String name, const Array<PrimExpr>& args, int64_t dst_anylist_slot = -1) {
-    Array<PrimExpr> all_args;
+  void EmitCallPacked(ffi::String name, const ffi::Array<PrimExpr>& args,
+                      int64_t dst_anylist_slot = -1) {
+    ffi::Array<PrimExpr> all_args;
     // negative index indicate return value can be discarded, emit call_packed
     if (dst_anylist_slot >= 0) {
       all_args = {reg_anylist_handle_, ConstInt32(dst_anylist_slot)};
@@ -123,11 +125,11 @@ class CodeGenVMTIR : public ExprFunctor<Optional<PrimExpr>(const Expr&)> {
     }
   }
 
-  void EmitCallCPacked(const tir::PrimFunc& prim_func, const Array<PrimExpr>& args,
+  void EmitCallCPacked(const tir::PrimFunc& prim_func, const ffi::Array<PrimExpr>& args,
                        int64_t dst_anylist_slot = -1) {
-    Optional<String> gsymbol = prim_func->GetAttr<String>(tvm::attr::kGlobalSymbol);
-    ICHECK(gsymbol.defined()) << "All functions must have global symbol at this phase";
-    Array<PrimExpr> all_args;
+    ffi::Optional<ffi::String> gsymbol = prim_func->GetAttr<ffi::String>(tvm::attr::kGlobalSymbol);
+    ICHECK(gsymbol.has_value()) << "All functions must have global symbol at this phase";
+    ffi::Array<PrimExpr> all_args;
     // negative index indicate return value can be discarded, emit call_packed
     if (dst_anylist_slot >= 0) {
       all_args = {reg_anylist_handle_, ConstInt32(dst_anylist_slot)};
@@ -146,9 +148,9 @@ class CodeGenVMTIR : public ExprFunctor<Optional<PrimExpr>(const Expr&)> {
   }
 
   tir::PrimFunc Codegen(const Function& func) {
-    Optional<String> gsymbol = func->GetAttr<String>(tvm::attr::kGlobalSymbol);
-    ICHECK(gsymbol.defined()) << "there should be no local functions in Relax VM codegen phase. "
-                                 "Did you forget to apply LambdaLift or AttachGlobalSymbol Pass?";
+    ffi::Optional<ffi::String> gsymbol = func->GetAttr<ffi::String>(tvm::attr::kGlobalSymbol);
+    ICHECK(gsymbol.has_value()) << "there should be no local functions in Relax VM codegen phase. "
+                                   "Did you forget to apply LambdaLift or AttachGlobalSymbol Pass?";
     // initialize the state
     stmt_stack_ = {};
     registers_num_ = 0;
@@ -158,7 +160,7 @@ class CodeGenVMTIR : public ExprFunctor<Optional<PrimExpr>(const Expr&)> {
     func_anylist_handle_ = tir::Var("f", DataType::Handle());
     const_anylist_handle_ = tir::Var("c", DataType::Handle());
 
-    Array<String> param_names;
+    ffi::Array<ffi::String> param_names;
     for (Var param : func->params) {
       param_names.push_back(param->name_hint());
     }
@@ -173,7 +175,7 @@ class CodeGenVMTIR : public ExprFunctor<Optional<PrimExpr>(const Expr&)> {
     size_t ret_reg = NewRegister();
 
     tir::Stmt body = WithNewScope([&]() {
-      Optional<PrimExpr> ret = ExprFunctor::VisitExpr(func->body);
+      ffi::Optional<PrimExpr> ret = ExprFunctor::VisitExpr(func->body);
       if (ret.defined()) {
         this->EmitCallPacked("vm.builtin.copy", {ret.value()}, ret_reg);
       }
@@ -185,9 +187,9 @@ class CodeGenVMTIR : public ExprFunctor<Optional<PrimExpr>(const Expr&)> {
     builder_->EndFunction(gsymbol.value());
 
     Type ret_type = VoidType();
-    Array<tir::Var> tir_params = {ctx_ptr_, reg_anylist_handle_, const_anylist_handle_,
-                                  func_anylist_handle_};
-    String tir_func_name = system_lib_prefix_.value_or("") + "__vmtir__" + gsymbol.value();
+    ffi::Array<tir::Var> tir_params = {ctx_ptr_, reg_anylist_handle_, const_anylist_handle_,
+                                       func_anylist_handle_};
+    ffi::String tir_func_name = system_lib_prefix_.value_or("") + "__vmtir__" + gsymbol.value();
     tir::PrimFunc tir_func(tir_params, body, ret_type, {});
     tir_func = WithAttr(tir_func, "global_symbol", tir_func_name);
     registers_num_ = 0;
@@ -196,11 +198,11 @@ class CodeGenVMTIR : public ExprFunctor<Optional<PrimExpr>(const Expr&)> {
     return tir_func;
   }
 
-  Optional<PrimExpr> VisitExpr_(const SeqExprNode* op) final {
+  ffi::Optional<PrimExpr> VisitExpr_(const SeqExprNode* op) final {
     for (auto block : op->blocks) {
       for (Binding binding : block->bindings) {
         Expr expr = GetBoundValue(binding);
-        Optional<PrimExpr> value = VisitExpr(expr);
+        ffi::Optional<PrimExpr> value = VisitExpr(expr);
 
         if (expr.as<Var>() && value.defined()) {
           // For a normalized relax module, there should be one
@@ -219,8 +221,8 @@ class CodeGenVMTIR : public ExprFunctor<Optional<PrimExpr>(const Expr&)> {
     return this->VisitExpr(op->body);
   }
 
-  Optional<PrimExpr> VisitExpr_(const CallNode* call_node) final {
-    Call call = GetRef<Call>(call_node);
+  ffi::Optional<PrimExpr> VisitExpr_(const CallNode* call_node) final {
+    Call call = ffi::GetRef<Call>(call_node);
 
     if (call_node->op == null_value_op_) {
       return tir::Call(DataType::Handle(), tir::builtin::reinterpret(),
@@ -251,7 +253,7 @@ class CodeGenVMTIR : public ExprFunctor<Optional<PrimExpr>(const Expr&)> {
     }
   }
 
-  Optional<PrimExpr> VisitExpr_(const IfNode* op) final {
+  ffi::Optional<PrimExpr> VisitExpr_(const IfNode* op) final {
     // Reserve a register for return
     size_t merge_register = NewRegister();
     PrimExpr cond_value = this->VisitExpr(op->cond).value();
@@ -271,18 +273,18 @@ class CodeGenVMTIR : public ExprFunctor<Optional<PrimExpr>(const Expr&)> {
     return RegListGet(merge_register);
   }
 
-  Optional<PrimExpr> VisitExpr_(const VarNode* op) final {
-    Var var = GetRef<Var>(op);
+  ffi::Optional<PrimExpr> VisitExpr_(const VarNode* op) final {
+    Var var = ffi::GetRef<Var>(op);
     auto it = this->var_map_.find(var);
     ICHECK(it != this->var_map_.end()) << "Var " << var << " is not defined";
     return it->second;
   }
 
-  Optional<PrimExpr> VisitExpr_(const ConstantNode* op) final {
+  ffi::Optional<PrimExpr> VisitExpr_(const ConstantNode* op) final {
     return ConstListGet(builder_->ConvertConstant(op->data).value());
   }
 
-  Optional<PrimExpr> VisitExpr_(const ShapeExprNode* op) final {
+  ffi::Optional<PrimExpr> VisitExpr_(const ShapeExprNode* op) final {
     std::vector<int64_t> shape;
     for (PrimExpr e : op->values) {
       if (auto* int_value = e.as<IntImmNode>()) {
@@ -294,19 +296,19 @@ class CodeGenVMTIR : public ExprFunctor<Optional<PrimExpr>(const Expr&)> {
     return ConstListGet(builder_->ConvertConstant(ffi::Shape(shape)).value());
   }
 
-  Optional<PrimExpr> VisitExpr_(const PrimValueNode* op) final { return op->value; }
+  ffi::Optional<PrimExpr> VisitExpr_(const PrimValueNode* op) final { return op->value; }
 
-  Optional<PrimExpr> VisitExpr_(const StringImmNode* op) final {
+  ffi::Optional<PrimExpr> VisitExpr_(const StringImmNode* op) final {
     return ConstListGet(builder_->ConvertConstant(op->value).value());
   }
 
-  Optional<PrimExpr> VisitExpr_(const DataTypeImmNode* op) final {
+  ffi::Optional<PrimExpr> VisitExpr_(const DataTypeImmNode* op) final {
     return ConstListGet(builder_->ConvertConstant(op->value).value());
   }
 
-  Optional<PrimExpr> VisitExpr_(const TupleNode* op) final {
-    Tuple tuple = GetRef<Tuple>(op);
-    Array<PrimExpr> args;
+  ffi::Optional<PrimExpr> VisitExpr_(const TupleNode* op) final {
+    Tuple tuple = ffi::GetRef<Tuple>(op);
+    ffi::Array<PrimExpr> args;
     for (auto arg : tuple->fields) {
       args.push_back(this->VisitExpr(arg).value());
     }
@@ -315,9 +317,9 @@ class CodeGenVMTIR : public ExprFunctor<Optional<PrimExpr>(const Expr&)> {
     return RegListGet(dst_register);
   }
 
-  Optional<PrimExpr> VisitExpr_(const TupleGetItemNode* op) final {
-    TupleGetItem expr = GetRef<TupleGetItem>(op);
-    Array<PrimExpr> args = {this->VisitExpr(expr->tuple).value()};
+  ffi::Optional<PrimExpr> VisitExpr_(const TupleGetItemNode* op) final {
+    TupleGetItem expr = ffi::GetRef<TupleGetItem>(op);
+    ffi::Array<PrimExpr> args = {this->VisitExpr(expr->tuple).value()};
 
     args.push_back(ConstInt64(expr->index));
 
@@ -327,12 +329,12 @@ class CodeGenVMTIR : public ExprFunctor<Optional<PrimExpr>(const Expr&)> {
   }
 
   // Lookup the function and see if it matches
-  Optional<String> LookupFunction(const Expr& expr, VMFuncInfo::FuncKind* kind) {
+  ffi::Optional<ffi::String> LookupFunction(const Expr& expr, VMFuncInfo::FuncKind* kind) {
     if (auto* ext_func = expr.as<ExternFuncNode>()) {
       *kind = VMFuncInfo::FuncKind::kPackedFunc;
       return ext_func->global_symbol;
     } else if (auto* gvar_ptr = expr.as<GlobalVarNode>()) {
-      GlobalVar gvar = GetRef<GlobalVar>(gvar_ptr);
+      GlobalVar gvar = ffi::GetRef<GlobalVar>(gvar_ptr);
       // Run a look up in the env to see if it maps to an extern func.
       auto it = ctx_mod_->functions.find(gvar);
       if (it != ctx_mod_->functions.end()) {
@@ -361,7 +363,7 @@ class CodeGenVMTIR : public ExprFunctor<Optional<PrimExpr>(const Expr&)> {
   }
   // Lookup PrimFunc in the same module
   // We can do direct PrimFunc call in such cases
-  Optional<tir::PrimFunc> LookupPrimFunc(const String& name) {
+  ffi::Optional<tir::PrimFunc> LookupPrimFunc(const ffi::String& name) {
     if (!ctx_mod_->ContainGlobalVar(name)) return std::nullopt;
 
     GlobalVar gvar = ctx_mod_->GetGlobalVar(name);
@@ -369,28 +371,28 @@ class CodeGenVMTIR : public ExprFunctor<Optional<PrimExpr>(const Expr&)> {
     if (it != ctx_mod_->functions.end()) {
       BaseFunc func = (*it).second;
       if (auto* prim_func = func.as<tir::PrimFuncNode>()) {
-        return GetRef<tir::PrimFunc>(prim_func);
+        return ffi::GetRef<tir::PrimFunc>(prim_func);
       }
     }
     return std::nullopt;
   }
 
-  Optional<PrimExpr> VisitExpr_(const GlobalVarNode* op) final {
+  ffi::Optional<PrimExpr> VisitExpr_(const GlobalVarNode* op) final {
     VMFuncInfo::FuncKind kind;
-    auto symbol = LookupFunction(GetRef<Expr>(op), &kind);
-    ICHECK(symbol.defined());
+    auto symbol = LookupFunction(ffi::GetRef<Expr>(op), &kind);
+    ICHECK(symbol.has_value());
     builder_->DeclareFunction(symbol.value(), kind);
     return FuncListGet(builder_->GetFunction(symbol.value()).value());
   }
 
-  Optional<PrimExpr> VisitExpr_(const ExternFuncNode* op) final {
+  ffi::Optional<PrimExpr> VisitExpr_(const ExternFuncNode* op) final {
     builder_->DeclareFunction(op->global_symbol, VMFuncInfo::FuncKind::kPackedFunc);
     return FuncListGet(builder_->GetFunction(op->global_symbol).value());
   }
 
   void EmitAllocStorage(const Call& call_node, int64_t dst_reg) {
     // Handle args of the call
-    Array<PrimExpr> args;
+    ffi::Array<PrimExpr> args;
     args.push_back(ctx_ptr_);
     for (Expr arg : call_node->args) {
       args.push_back(this->VisitExpr(arg).value());
@@ -400,7 +402,7 @@ class CodeGenVMTIR : public ExprFunctor<Optional<PrimExpr>(const Expr&)> {
 
   void EmitAllocTensor(const Call& call_node, int64_t dst_reg) {
     ICHECK_EQ(call_node->args.size(), 4);
-    Array<PrimExpr> args;
+    ffi::Array<PrimExpr> args;
     args.reserve(4);
     for (Expr arg : call_node->args) {
       args.push_back(this->VisitExpr(arg).value());
@@ -428,7 +430,7 @@ class CodeGenVMTIR : public ExprFunctor<Optional<PrimExpr>(const Expr&)> {
   }
 
   void EmitCallBuiltinWithCtx(const Call& call_node, int64_t dst_reg) {
-    Array<PrimExpr> args;
+    ffi::Array<PrimExpr> args;
     // if context is required, pass as first argument.
     args.push_back(ctx_ptr_);
     auto* func = call_node->args[0].as<ExternFuncNode>();
@@ -445,23 +447,23 @@ class CodeGenVMTIR : public ExprFunctor<Optional<PrimExpr>(const Expr&)> {
   }
 
   void EmitNormalCall(const Call& call_node, int64_t dst_reg) {
-    Array<PrimExpr> args = VisitArray(call_node->args);
+    ffi::Array<PrimExpr> args = VisitArray(call_node->args);
     // A function can be a closure that comes from parent
     // Do call closure to be safe.
     VMFuncInfo::FuncKind kind;
     auto symbol = LookupFunction(call_node->op, &kind);
 
-    if (symbol.defined() && kind == VMFuncInfo::FuncKind::kPackedFunc) {
+    if (symbol.has_value() && kind == VMFuncInfo::FuncKind::kPackedFunc) {
       // primfunc in the same module.
       // use cpacked to directly invoke without named based lookup
-      if (Optional<tir::PrimFunc> prim_func = LookupPrimFunc(symbol.value())) {
+      if (ffi::Optional<tir::PrimFunc> prim_func = LookupPrimFunc(symbol.value())) {
         this->EmitCallCPacked(prim_func.value(), args, dst_reg);
       } else {
         this->EmitCallPacked(symbol.value(), args, dst_reg);
       }
     } else {
       // Default path, leverage function table and invoke as closure
-      Array<PrimExpr> all_args;
+      ffi::Array<PrimExpr> all_args;
       all_args.push_back(ctx_ptr_);
       all_args.push_back(this->VisitExpr(call_node->op).value());
       for (auto arg : args) {
@@ -480,8 +482,8 @@ class CodeGenVMTIR : public ExprFunctor<Optional<PrimExpr>(const Expr&)> {
     return stmt;
   }
 
-  Array<PrimExpr> VisitArray(const Array<Expr>& arr) {
-    Array<PrimExpr> ret;
+  ffi::Array<PrimExpr> VisitArray(const ffi::Array<Expr>& arr) {
+    ffi::Array<PrimExpr> ret;
     for (size_t i = 0; i < arr.size(); ++i) {
       ret.push_back(this->VisitExpr(arr[i]).value());
     }
@@ -505,11 +507,11 @@ class CodeGenVMTIR : public ExprFunctor<Optional<PrimExpr>(const Expr&)> {
   /*! \brief Stack to build up statements */
   std::vector<std::vector<tir::Stmt>> stmt_stack_;
   /*! \brief Map from var to Expr. */
-  std::unordered_map<Var, Optional<PrimExpr>> var_map_;
+  std::unordered_map<Var, ffi::Optional<PrimExpr>> var_map_;
   /*! \brief the context module. */
   IRModule ctx_mod_;
   /*! \brief system lib prefix */
-  Optional<String> system_lib_prefix_;
+  ffi::Optional<ffi::String> system_lib_prefix_;
   /*! \brief Cache ops that need to be frequently used later to reduce lookup overhead. */
   const Op& alloc_storage_op_ = Op::Get("relax.vm.alloc_storage");
   const Op& alloc_tensor_op_ = Op::Get("relax.vm.alloc_tensor");
@@ -530,7 +532,10 @@ IRModule VMTIRCodeGen(ExecBuilder exec_builder, IRModule mod) {
   return CodeGenVMTIR::Run(exec_builder, mod);
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.VMTIRCodeGen").set_body_typed(VMTIRCodeGen);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.VMTIRCodeGen", VMTIRCodeGen);
+}
 
 }  // namespace codegen_vm
 }  // namespace relax
