@@ -17,21 +17,21 @@
  * under the License.
  */
 
-#include <tvm/ffi/reflection/reflection.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/relax/distributed/global_info.h>
 
 namespace tvm {
 namespace relax {
 namespace distributed {
 
-TVM_FFI_STATIC_INIT_BLOCK({ DeviceMeshNode::RegisterReflection(); });
+TVM_FFI_STATIC_INIT_BLOCK() { DeviceMeshNode::RegisterReflection(); }
 
-DeviceMesh::DeviceMesh(ffi::Shape shape, Array<Integer> device_ids) {
+DeviceMesh::DeviceMesh(ffi::Shape shape, ffi::Array<Integer> device_ids) {
   int prod = 1;
   for (int i = 0; i < static_cast<int>(shape.size()); i++) {
     prod *= shape[i];
   }
-  ObjectPtr<DeviceMeshNode> n = make_object<DeviceMeshNode>();
+  ObjectPtr<DeviceMeshNode> n = ffi::make_object<DeviceMeshNode>();
   CHECK_EQ(prod, static_cast<int>(device_ids.size()))
       << "The number of device ids must match the product of the shape";
   n->shape = std::move(shape);
@@ -40,8 +40,8 @@ DeviceMesh::DeviceMesh(ffi::Shape shape, Array<Integer> device_ids) {
 }
 
 DeviceMesh::DeviceMesh(ffi::Shape shape, Range device_range) {
-  ObjectPtr<DeviceMeshNode> n = make_object<DeviceMeshNode>();
-  Array<Integer> device_ids;
+  ObjectPtr<DeviceMeshNode> n = ffi::make_object<DeviceMeshNode>();
+  ffi::Array<Integer> device_ids;
   int range_start = device_range->min.as<IntImmNode>()->value;
   int range_extent = device_range->extent.as<IntImmNode>()->value;
   for (int i = range_start; i < range_start + range_extent; i++) {
@@ -59,14 +59,17 @@ DeviceMesh::DeviceMesh(ffi::Shape shape, Range device_range) {
   data_ = std::move(n);
 }
 
-TVM_REGISTER_NODE_TYPE(DeviceMeshNode);
-TVM_FFI_REGISTER_GLOBAL("relax.distributed.DeviceMesh")
-    .set_body_typed([](ffi::Shape shape, Array<Integer> device_ids, Optional<Range> device_range) {
-      if (device_range.defined())
-        return DeviceMesh(shape, device_range.value());
-      else
-        return DeviceMesh(shape, device_ids);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def(
+      "relax.distributed.DeviceMesh",
+      [](ffi::Shape shape, ffi::Array<Integer> device_ids, ffi::Optional<Range> device_range) {
+        if (device_range.defined())
+          return DeviceMesh(shape, device_range.value());
+        else
+          return DeviceMesh(shape, device_ids);
+      });
+}
 
 }  // namespace distributed
 }  // namespace relax
