@@ -21,6 +21,7 @@
  * \file tvm/relax/distributed/transform/propagate_sharding.cc
  * \brief Pass for propagating sharding information.
  */
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/relax/analysis.h>
 #include <tvm/relax/attrs/distributed.h>
 #include <tvm/relax/attrs/linear_algebra.h>
@@ -47,7 +48,7 @@ void CollectAxisGraphBinary(const VarBindingNode* binding, const CallNode* call,
   for (const auto& op_name : binary_op_names) {
     const Op& binary_op = Op::Get("relax." + op_name);
     if (call->op.same_as(binary_op)) {
-      BuildAxisGraphBinary(binding->var, GetRef<Call>(call), axis_group_graph);
+      BuildAxisGraphBinary(binding->var, ffi::GetRef<Call>(call), axis_group_graph);
       break;
     }
   }
@@ -70,7 +71,7 @@ void CollectAxisGraphUnary(const VarBindingNode* binding, const CallNode* call,
   for (const auto& op_name : unary_op_names) {
     const Op& unary_op = Op::Get("relax." + op_name);
     if (call->op.same_as(unary_op)) {
-      BuildAxisGraphUnary(binding->var, GetRef<Call>(call), axis_group_graph);
+      BuildAxisGraphUnary(binding->var, ffi::GetRef<Call>(call), axis_group_graph);
     }
   }
 }
@@ -82,7 +83,7 @@ void CollectAxisGraphReduce(const VarBindingNode* binding, const CallNode* call,
   for (const auto& op_name : reduction_op_names) {
     const Op& reduction_op = Op::Get("relax." + op_name);
     if (call->op.same_as(reduction_op)) {
-      BuildAxisGraphReduce(binding->var, GetRef<Call>(call), axis_group_graph);
+      BuildAxisGraphReduce(binding->var, ffi::GetRef<Call>(call), axis_group_graph);
       break;
     }
   }
@@ -92,7 +93,7 @@ void CollectAxisGraphMatmul(const VarBindingNode* binding, const CallNode* call,
                             AxisGroupGraph* axis_group_graph) {
   static const Op& matmul_op = Op::Get("relax.matmul");
   if (call->op.same_as(matmul_op)) {
-    BuildAxisGraphMatmul(binding->var, GetRef<Call>(call), axis_group_graph);
+    BuildAxisGraphMatmul(binding->var, ffi::GetRef<Call>(call), axis_group_graph);
   }
 }
 
@@ -100,7 +101,7 @@ void CollectAxisGraphPermuteDims(const VarBindingNode* binding, const CallNode*
                                  AxisGroupGraph* axis_group_graph) {
   static const Op& permute_dims_op = Op::Get("relax.permute_dims");
   if (call->op.same_as(permute_dims_op)) {
-    BuildAxisGraphPermuteDims(binding->var, GetRef<Call>(call), axis_group_graph);
+    BuildAxisGraphPermuteDims(binding->var, ffi::GetRef<Call>(call), axis_group_graph);
   }
 }
 
@@ -108,15 +109,15 @@ void CollectAxisGraphReshape(const VarBindingNode* binding, const CallNode* call
                              AxisGroupGraph* axis_group_graph) {
   static const Op& reshape_op = Op::Get("relax.reshape");
   if (call->op.same_as(reshape_op)) {
-    BuildAxisGraphReshape(binding->var, GetRef<Call>(call), axis_group_graph);
+    BuildAxisGraphReshape(binding->var, ffi::GetRef<Call>(call), axis_group_graph);
   }
 }
 
 void CollectAxisGraphForDeviceMesh(const VarBindingNode* binding, const CallNode* call,
                                    AxisGroupGraph* axis_group_graph) {
-  Array<Expr> tensor_list;
+  ffi::Array<Expr> tensor_list;
   static const Op& call_tir_op = Op::Get("relax.call_tir");
-  Array<Expr> args;
+  ffi::Array<Expr> args;
   if (call->op.same_as(call_tir_op)) {
     args = Downcast<Tuple>(call->args[1])->fields;
   } else {
@@ -157,8 +158,9 @@ class AxisGroupGraphBuilder : public ExprVisitor {
     CollectAxisGraphReshape(binding, val, axis_group_graph_);
     static const Op& call_tir_op = Op::Get("relax.call_tir");
     if (val->op.same_as(call_tir_op)) {
-      if (Optional<tir::PrimFunc> func = MatchPrimFunc(mod_, val->args[0])) {
-        BuildAxisGraphCallTIR(binding->var, GetRef<Call>(val), func.value(), axis_group_graph_);
+      if (ffi::Optional<tir::PrimFunc> func = MatchPrimFunc(mod_, val->args[0])) {
+        BuildAxisGraphCallTIR(binding->var, ffi::GetRef<Call>(val), func.value(),
+                              axis_group_graph_);
       }
     }
     CollectAxisGraphForDeviceMesh(binding, val, axis_group_graph_);
@@ -182,9 +184,9 @@ class AxisGroupGraphBuilder : public ExprVisitor {
   }
 
   void VisitBinding_(const VarBindingNode* binding, const VarNode* val) {
-    Array<TensorStructInfo> tensor_sinfos;
+    ffi::Array<TensorStructInfo> tensor_sinfos;
     if (const auto* tensor_sinfo = binding->var->struct_info_.as<TensorStructInfoNode>()) {
-      tensor_sinfos.push_back(GetRef<TensorStructInfo>(tensor_sinfo));
+      tensor_sinfos.push_back(ffi::GetRef<TensorStructInfo>(tensor_sinfo));
     } else if (const auto* tuple_sinfo = binding->var->struct_info_.as<TupleStructInfoNode>()) {
       ICHECK(tuple_sinfo);
       for (const auto& sinfo : tuple_sinfo->fields) {
@@ -270,7 +272,7 @@ class ShardingConflictHandler : public ExprVisitor {
     ICHECK(shape);
     int ndim = sinfo->ndim;
     std::unordered_set<int> sharded_mesh_dim;
-    Optional<DeviceMesh> device_mesh;
+    ffi::Optional<DeviceMesh> device_mesh;
     for (int i = -1; i < ndim; i++) {
       AxisShardingSpec sharding_spec;
       int has_sharding_spec;
@@ -317,7 +319,7 @@ class ShardingConflictHandler : public ExprVisitor {
   }
 
   void VisitExpr_(const CallNode* op) final {
-    Array<Expr> args = GetCallArgs(GetRef<Call>(op));
+    ffi::Array<Expr> args = GetCallArgs(ffi::GetRef<Call>(op));
     for (const auto& arg : args) {
       if (arg.as<ConstantNode>()) {
         CheckConstantNoSharding(Downcast<Constant>(arg));
@@ -347,10 +349,10 @@ class DistributedIRBuilder : public ExprMutator {
     auto mod = builder_->GetContextIRModule();
     for (const auto& [gv, base_func] : mod->functions) {
       const auto* func_ = base_func.as<FunctionNode>();
-      if (func_ == nullptr || !IsShardingAnnotatedFunc(GetRef<Function>(func_))) {
+      if (func_ == nullptr || !IsShardingAnnotatedFunc(ffi::GetRef<Function>(func_))) {
         continue;
       }
-      Function func = RewriteFunction(GetRef<Function>(func_), mod);
+      Function func = RewriteFunction(ffi::GetRef<Function>(func_), mod);
       builder_->UpdateFunction(gv, func);
     }
     return builder_->GetContextIRModule();
@@ -365,7 +367,7 @@ class DistributedIRBuilder : public ExprMutator {
     DeviceMesh device_mesh =
         std::get<0>(axis_group_graph_.GetAxisShardingSpec({expr.get(), -1, tuple_idx})).first;
     ICHECK(device_mesh.defined()) << expr << "[" << tuple_idx << "] is not assigned device mesh";
-    Array<PlacementSpec> placement_specs(
+    ffi::Array<PlacementSpec> placement_specs(
         std::vector<PlacementSpec>(device_mesh->shape.size(), PlacementSpec::Replica()));
     for (int i = 0; i < ndim; i++) {
       AxisShardingSpec sharding_spec;
@@ -386,7 +388,7 @@ class DistributedIRBuilder : public ExprMutator {
       new_sinfo =
           ConvertToDTensorStructInfo(Downcast<TensorStructInfo>(tensor->struct_info_), tensor);
     } else if (const auto* tuple = tensor->struct_info_.as<TupleStructInfoNode>()) {
-      Array<StructInfo> tuple_sinfo_fields;
+      ffi::Array<StructInfo> tuple_sinfo_fields;
       for (int i = 0; i < static_cast<int>(tuple->fields.size()); i++) {
         if (tuple->fields[i].as<TensorStructInfoNode>()) {
           tuple_sinfo_fields.push_back(
@@ -418,7 +420,7 @@ class DistributedIRBuilder : public ExprMutator {
     // Step 3. Handle Sharding Conflict
     ShardingConflictHandler::HandleShardingConflict(&axis_group_graph_, func);
     // Step 4. Rewrite Function
-    Array<Var> new_params;
+    ffi::Array<Var> new_params;
     for (const Var& var : func->params) {
       if (GetStructInfoAs<TensorStructInfoNode>(var) || GetStructInfoAs<TupleStructInfoNode>(var)) {
         Var new_param = Downcast<Var>(RewriteInputTensorAndConstant(var));
@@ -436,20 +438,20 @@ class DistributedIRBuilder : public ExprMutator {
   Expr VisitExpr_(const CallNode* call) final {
     static const Op& call_tir_op = Op::Get("relax.call_tir");
     FBuildAxisGraph f = [&](const Var& var, const Call& call, AxisGroupGraph* axis_group_graph) {
-      Optional<tir::PrimFunc> prim_func =
+      ffi::Optional<tir::PrimFunc> prim_func =
           MatchPrimFunc(this->builder_->GetContextIRModule(), call->args[0]);
       ICHECK(prim_func);
       return BuildAxisGraphCallTIR(var, call, prim_func.value(), axis_group_graph);
     };
     Call new_call = Downcast<Call>(ExprMutator::VisitExpr_(call));
-    Array<Expr> args = GetCallArgs(new_call);
+    ffi::Array<Expr> args = GetCallArgs(new_call);
     for (int i = 0; i < static_cast<int>(args.size()); i++) {
       if (args[i].as<ConstantNode>()) {
         args.Set(i, RewriteInputTensorAndConstant(args[i]));
       }
     }
 
-    ObjectPtr<CallNode> n = make_object<CallNode>(*new_call.get());
+    ObjectPtr<CallNode> n = ffi::make_object<CallNode>(*new_call.get());
     if (new_call->op.same_as(call_tir_op)) {
       // do not infer output sinfo when arg size is 0
       if (!args.empty()) {
@@ -483,13 +485,13 @@ class DistributedIRBuilder : public ExprMutator {
     return redistribute(expr, device_mesh, placement);
   }
 
-  Call RewriteOutSinfo(Call call, DeviceMesh device_mesh, Array<Placement> placements) {
+  Call RewriteOutSinfo(Call call, DeviceMesh device_mesh, ffi::Array<Placement> placements) {
     // in cases when infer fails (like arg size is 0), we use propagated sinfo for output
     Call new_call = call;
     static Op call_tir_op = Op::Get("relax.call_tir");
     if (const auto* extern_func = call->op.as<ExternFuncNode>()) {
       if (extern_func->global_symbol == "vm.builtin.distributed.attention_kv_cache_view") {
-        ObjectPtr<CallNode> new_call_node = make_object<CallNode>(*call.get());
+        ObjectPtr<CallNode> new_call_node = ffi::make_object<CallNode>(*call.get());
         StructInfo new_dtensor_sinfo = DTensorStructInfo(
             Downcast<TensorStructInfo>(call->sinfo_args[0]), device_mesh, placements[0]);
         new_call_node->sinfo_args = {new_dtensor_sinfo};
@@ -499,14 +501,14 @@ class DistributedIRBuilder : public ExprMutator {
     } else if (call->op.same_as(call_tir_op)) {
       ICHECK(call->sinfo_args.size() == 1);
       if (!SinfoCompatibleWithDistIR(call->sinfo_args)) {
-        ObjectPtr<CallNode> new_call_node = make_object<CallNode>(*call.get());
+        ObjectPtr<CallNode> new_call_node = ffi::make_object<CallNode>(*call.get());
         if (placements.size() == 1) {
           new_call_node->sinfo_args = {DTensorStructInfo(
               Downcast<TensorStructInfo>(call->sinfo_args[0]), device_mesh, placements[0])};
         } else {
           const auto* tuple_sinfo = call->sinfo_args[0].as<TupleStructInfoNode>();
           ICHECK(placements.size() == tuple_sinfo->fields.size());
-          Array<StructInfo> new_tuple_sinfo_fields;
+          ffi::Array<StructInfo> new_tuple_sinfo_fields;
           for (int i = 0; i < static_cast<int>(placements.size()); i++) {
             new_tuple_sinfo_fields.push_back(DTensorStructInfo(
                 Downcast<TensorStructInfo>(tuple_sinfo->fields[i]), device_mesh, placements[i]));
@@ -521,9 +523,9 @@ class DistributedIRBuilder : public ExprMutator {
   }
 
   void VisitBinding_(const VarBindingNode* binding, const CallNode* val) {
-    Array<TensorStructInfo> orig_output_tensor_sinfos;
+    ffi::Array<TensorStructInfo> orig_output_tensor_sinfos;
     if (const auto* tensor_sinfo = GetStructInfoAs<TensorStructInfoNode>(binding->var)) {
-      orig_output_tensor_sinfos.push_back(GetRef<TensorStructInfo>(tensor_sinfo));
+      orig_output_tensor_sinfos.push_back(ffi::GetRef<TensorStructInfo>(tensor_sinfo));
     } else if (const auto* tuple_sinfo = GetStructInfoAs<TupleStructInfoNode>(binding->var)) {
       for (const auto& sinfo : tuple_sinfo->fields) {
         orig_output_tensor_sinfos.push_back(Downcast<TensorStructInfo>(sinfo));
@@ -536,9 +538,9 @@ class DistributedIRBuilder : public ExprMutator {
     DeviceMesh device_mesh =
         std::get<0>(axis_group_graph_.GetAxisShardingSpec({binding->var.get(), -1})).first;
     ICHECK(device_mesh.defined());
-    Array<Placement> placements;  // every tuple element has a placement
+    ffi::Array<Placement> placements;  // every tuple element has a placement
     for (int idx = 0; idx < static_cast<int>(orig_output_tensor_sinfos.size()); idx++) {
-      Array<PlacementSpec> placement_specs(
+      ffi::Array<PlacementSpec> placement_specs(
           std::vector<PlacementSpec>(device_mesh->shape.size(), PlacementSpec::Replica()));
       for (int i = 0; i < orig_output_tensor_sinfos[idx]->ndim; i++) {
         AxisShardingSpec sharding_spec;
@@ -564,7 +566,7 @@ class DistributedIRBuilder : public ExprMutator {
         new_value = InsertRedistribute(new_value, device_mesh, placements[0]);
       }
       if (const auto* var = new_value.as<VarNode>()) {
-        var_remap_[binding->var->vid] = GetRef<Var>(var);
+        var_remap_[binding->var->vid] = ffi::GetRef<Var>(var);
       } else {
         ReEmitBinding(binding, builder_->Normalize(new_value));
       }
@@ -588,22 +590,22 @@ class DistributedIRBuilder : public ExprMutator {
   }
 
   void VisitBinding_(const VarBindingNode* binding, const TupleGetItemNode* val) {
-    if (tuple_getitem_remap_.count(GetRef<TupleGetItem>(val))) {
-      var_remap_[binding->var->vid] = tuple_getitem_remap_[GetRef<TupleGetItem>(val)];
+    if (tuple_getitem_remap_.count(ffi::GetRef<TupleGetItem>(val))) {
+      var_remap_[binding->var->vid] = tuple_getitem_remap_[ffi::GetRef<TupleGetItem>(val)];
     } else {
       ExprMutator::VisitBinding_(binding, val);
     }
   }
 
   Expr VisitExpr_(const VarNode* var) final {
-    auto it = input_tensor_remap_.find(GetRef<Var>(var));
+    auto it = input_tensor_remap_.find(ffi::GetRef<Var>(var));
     if (it != input_tensor_remap_.end()) {
       var_remap_[var->vid] = (*it).second;
     }
     return ExprMutator::VisitExpr_(var);
   }
 
-  Map<Var, Var> input_tensor_remap_;
+  ffi::Map<Var, Var> input_tensor_remap_;
   std::unordered_map<TupleGetItem, Var, StructuralHash, StructuralEqual> tuple_getitem_remap_;
   AxisGroupGraph axis_group_graph_;
 };
@@ -615,8 +617,10 @@ Pass PropagateSharding() {
   };
   return CreateModulePass(pass_func, 1, "PropagateSharding", {});
 }
-TVM_FFI_REGISTER_GLOBAL("relax.distributed.transform.PropagateSharding")
-    .set_body_typed(PropagateSharding);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.distributed.transform.PropagateSharding", PropagateSharding);
+}
 }  // namespace transform
 
 }  // namespace distributed
