@@ -22,7 +22,7 @@
  * \brief A transform to match a Relax Expr and rewrite
  */
 
-#include <tvm/ffi/reflection/reflection.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/ir/transform.h>
 #include <tvm/node/structural_equal.h>
 #include <tvm/relax/analysis.h>
@@ -45,11 +45,11 @@ namespace relax {
 namespace {
 class GlobalVarReplacer : public ExprMutator {
  public:
-  explicit GlobalVarReplacer(Map<GlobalVar, GlobalVar> gvar_map) : gvar_map_(gvar_map) {}
+  explicit GlobalVarReplacer(ffi::Map<GlobalVar, GlobalVar> gvar_map) : gvar_map_(gvar_map) {}
 
   using ExprMutator::VisitExpr_;
   Expr VisitExpr_(const GlobalVarNode* op) override {
-    auto gvar = GetRef<GlobalVar>(op);
+    auto gvar = ffi::GetRef<GlobalVar>(op);
     if (auto opt = gvar_map_.Get(gvar)) {
       gvar = opt.value();
     }
@@ -57,10 +57,10 @@ class GlobalVarReplacer : public ExprMutator {
   }
 
  private:
-  Map<GlobalVar, GlobalVar> gvar_map_;
+  ffi::Map<GlobalVar, GlobalVar> gvar_map_;
 };
 
-Array<Binding> TopologicalSort(const Array<Binding>& bindings) {
+ffi::Array<Binding> TopologicalSort(const ffi::Array<Binding>& bindings) {
   std::unordered_set<Var> remaining_bindings;
   for (const auto& binding : bindings) {
     remaining_bindings.insert(binding->var);
@@ -74,7 +74,7 @@ Array<Binding> TopologicalSort(const Array<Binding>& bindings) {
     bool emitted;
   };
   std::vector<DelayedBinding> delayed_bindings;
-  Array<Binding> sorted_bindings;
+  ffi::Array<Binding> sorted_bindings;
 
   // Utility function to append the
   auto push_sorted_binding = [&](Binding binding) {
@@ -159,7 +159,7 @@ void RewriteSpec::Append(RewriteSpec other) {
     gvar_name_supply->ReserveName(gvar->name_hint);
   }
 
-  Map<GlobalVar, GlobalVar> gvar_rewrites;
+  ffi::Map<GlobalVar, GlobalVar> gvar_rewrites;
   for (auto [gvar, func] : other.new_subroutines) {
     if (auto it = new_subroutines.find(gvar); it != new_subroutines.end()) {
       // The two rewrites provide the same GlobalVar.
@@ -192,34 +192,32 @@ void RewriteSpec::Append(RewriteSpec other) {
   }
 }
 
-TVM_REGISTER_NODE_TYPE(PatternMatchingRewriterNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def("relax.dpl.PatternMatchingRewriterFromPattern",
+           [](DFPattern pattern,
+              ffi::TypedFunction<ffi::Optional<Expr>(Expr, ffi::Map<DFPattern, Expr>)> func) {
+             return PatternMatchingRewriter::FromPattern(pattern, func);
+           })
+      .def("relax.dpl.PatternMatchingRewriterFromModule",
+           [](IRModule mod) { return PatternMatchingRewriter::FromModule(mod); })
+      .def("relax.dpl.PatternMatchingRewriterApply",
+           [](PatternMatchingRewriter rewriter,
+              ffi::Variant<Expr, IRModule> obj) -> ffi::Variant<Expr, IRModule> {
+             if (auto expr = obj.as<Expr>()) {
+               return rewriter(expr.value());
+             } else if (auto mod = obj.as<IRModule>()) {
+               return rewriter(mod.value());
+             } else {
+               LOG(FATAL) << "Unreachable: object does not contain either variant type";
+             }
+           });
+}
 
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.PatternMatchingRewriterFromPattern")
-    .set_body_typed([](DFPattern pattern,
-                       ffi::TypedFunction<Optional<Expr>(Expr, Map<DFPattern, Expr>)> func) {
-      return PatternMatchingRewriter::FromPattern(pattern, func);
-    });
-
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.PatternMatchingRewriterFromModule")
-    .set_body_typed([](IRModule mod) { return PatternMatchingRewriter::FromModule(mod); });
-
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.PatternMatchingRewriterApply")
-    .set_body_typed([](PatternMatchingRewriter rewriter,
-                       Variant<Expr, IRModule> obj) -> Variant<Expr, IRModule> {
-      if (auto expr = obj.as<Expr>()) {
-        return rewriter(expr.value());
-      } else if (auto mod = obj.as<IRModule>()) {
-        return rewriter(mod.value());
-      } else {
-        LOG(FATAL) << "Unreachable: object does not contain either variant type";
-      }
-    });
-
-TVM_REGISTER_NODE_TYPE(ExprPatternRewriterNode);
-
-RewriteSpec ExprPatternRewriterNode::RewriteBindings(const Array<Binding>& bindings) const {
-  Map<Var, Expr> variable_rewrites;
-  Map<Var, Expr> binding_lookup;
+RewriteSpec ExprPatternRewriterNode::RewriteBindings(const ffi::Array<Binding>& bindings) const {
+  ffi::Map<Var, Expr> variable_rewrites;
+  ffi::Map<Var, Expr> binding_lookup;
   for (const auto& binding : bindings) {
     auto bound_value = GetBoundValue(binding);
     if (auto new_expr = RewriteExpr(bound_value, binding_lookup)) {
@@ -235,8 +233,8 @@ RewriteSpec ExprPatternRewriterNode::RewriteBindings(const Array<Binding>& bindi
   }
 }
 
-Optional<Expr> ExprPatternRewriterNode::RewriteExpr(const Expr& expr,
-                                                    const Map<Var, Expr>& bindings) const {
+ffi::Optional<Expr> ExprPatternRewriterNode::RewriteExpr(
+    const Expr& expr, const ffi::Map<Var, Expr>& bindings) const {
   if (auto opt_matches = ExtractMatchedExpr(pattern, expr, bindings)) {
     auto matches = opt_matches.value();
     if (additional_bindings) {
@@ -251,7 +249,7 @@ Optional<Expr> ExprPatternRewriterNode::RewriteExpr(const Expr& expr,
       }
     }
 
-    Optional<Expr> rewritten_expr = func(expr, matches);
+    ffi::Optional<Expr> rewritten_expr = func(expr, matches);
     if (rewritten_expr.defined() && !rewritten_expr.same_as(expr)) {
       return rewritten_expr.value();
     }
@@ -259,16 +257,22 @@ Optional<Expr> ExprPatternRewriterNode::RewriteExpr(const Expr& expr,
   return std::nullopt;
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.PatternRewriter")
-    .set_body_typed([](DFPattern pattern,
-                       ffi::TypedFunction<Optional<Expr>(Expr, Map<DFPattern, Expr>)> func) {
-      return ExprPatternRewriter(pattern, func);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def(
+      "relax.dpl.PatternRewriter",
+      [](DFPattern pattern,
+         ffi::TypedFunction<ffi::Optional<Expr>(Expr, ffi::Map<DFPattern, Expr>)> func) {
+        return ExprPatternRewriter(pattern, func);
+      });
+}
 
 ExprPatternRewriter::ExprPatternRewriter(
-    DFPattern pattern, ffi::TypedFunction<Optional<Expr>(Expr, Map<DFPattern, Expr>)> func,
-    Optional<Array<DFPattern>> additional_bindings, Map<GlobalVar, BaseFunc> new_subroutines) {
-  auto node = make_object<ExprPatternRewriterNode>();
+    DFPattern pattern,
+    ffi::TypedFunction<ffi::Optional<Expr>(Expr, ffi::Map<DFPattern, Expr>)> func,
+    ffi::Optional<ffi::Array<DFPattern>> additional_bindings,
+    ffi::Map<GlobalVar, BaseFunc> new_subroutines) {
+  auto node = ffi::make_object<ExprPatternRewriterNode>();
   node->pattern = std::move(pattern);
   node->func = std::move(func);
   node->additional_bindings = std::move(additional_bindings);
@@ -276,9 +280,7 @@ ExprPatternRewriter::ExprPatternRewriter(
   data_ = std::move(node);
 }
 
-TVM_REGISTER_NODE_TYPE(OrRewriterNode);
-
-RewriteSpec OrRewriterNode::RewriteBindings(const Array<Binding>& bindings) const {
+RewriteSpec OrRewriterNode::RewriteBindings(const ffi::Array<Binding>& bindings) const {
   auto lhs_match = lhs->RewriteBindings(bindings);
   if (!lhs_match) {
     // If no rewrites found on LHS, RHS is allowed to modify any
@@ -292,7 +294,7 @@ RewriteSpec OrRewriterNode::RewriteBindings(const Array<Binding>& bindings) cons
   // the LHS.  Variable replacements from the RHS may still occur,
   // but will need to wait for the next round of
   // iterate-until-converged.
-  Array<Binding> remaining_bindings;
+  ffi::Array<Binding> remaining_bindings;
   for (const auto& binding : bindings) {
     if (!lhs_match.variable_rewrites.count(binding->var)) {
       remaining_bindings.push_back(binding);
@@ -308,25 +310,26 @@ RewriteSpec OrRewriterNode::RewriteBindings(const Array<Binding>& bindings) cons
   return lhs_match;
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.OrRewriter")
-    .set_body_typed([](PatternMatchingRewriter lhs, PatternMatchingRewriter rhs) {
-      return OrRewriter(lhs, rhs);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.dpl.OrRewriter",
+                        [](PatternMatchingRewriter lhs, PatternMatchingRewriter rhs) {
+                          return OrRewriter(lhs, rhs);
+                        });
+}
 
 OrRewriter::OrRewriter(PatternMatchingRewriter lhs, PatternMatchingRewriter rhs) {
-  auto node = make_object<OrRewriterNode>();
+  auto node = ffi::make_object<OrRewriterNode>();
   node->lhs = std::move(lhs);
   node->rhs = std::move(rhs);
   data_ = std::move(node);
 }
 
-TVM_REGISTER_NODE_TYPE(TupleRewriterNode);
-
-RewriteSpec TupleRewriterNode::RewriteBindings(const Array<Binding>& bindings) const {
+RewriteSpec TupleRewriterNode::RewriteBindings(const ffi::Array<Binding>& bindings) const {
   CHECK_LE(patterns.size(), 3) << "For performance reasons, "
                                << "matching of implicit tuple patterns is currently limited"
                                << " to tuples with 3 elements or fewer.";
-  Map<Var, Expr> variable_rewrites = GenerateVariableRewrites(bindings);
+  ffi::Map<Var, Expr> variable_rewrites = GenerateVariableRewrites(bindings);
 
   if (variable_rewrites.size()) {
     return RewriteSpec{variable_rewrites, new_subroutines};
@@ -335,10 +338,11 @@ RewriteSpec TupleRewriterNode::RewriteBindings(const Array<Binding>& bindings) c
   }
 }
 
-Map<Var, Expr> TupleRewriterNode::GenerateVariableRewrites(const Array<Binding>& bindings) const {
-  Map<Var, Expr> rewrites;
+ffi::Map<Var, Expr> TupleRewriterNode::GenerateVariableRewrites(
+    const ffi::Array<Binding>& bindings) const {
+  ffi::Map<Var, Expr> rewrites;
 
-  Map<Var, Expr> binding_lookup;
+  ffi::Map<Var, Expr> binding_lookup;
 
   std::vector<VarInfo> info_vec;
 
@@ -379,8 +383,9 @@ Map<Var, Expr> TupleRewriterNode::GenerateVariableRewrites(const Array<Binding>&
         for (size_t j_rev = 1; j_rev < info_vec.size(); j_rev++) {
           size_t j = info_vec.size() - j_rev - 1;
           if (info_vec[j].matches[i] && !info_vec[j].used &&
-              std::all_of(indices.begin() + (j + 1), indices.end(),
-                          [j](size_t prev_binding_index) { return j != prev_binding_index; })) {
+              (j + 1 >= indices.size() ||
+               std::all_of(indices.begin() + (j + 1), indices.end(),
+                           [j](size_t prev_binding_index) { return j != prev_binding_index; }))) {
             return j;
           }
         }
@@ -533,7 +538,7 @@ std::optional<std::vector<Expr>> TupleRewriterNode::TryMatchByBindingIndex(
     }
   }
 
-  Map<DFPattern, Expr> merged_matches = info_vec[indices[0]].matches[0].value();
+  ffi::Map<DFPattern, Expr> merged_matches = info_vec[indices[0]].matches[0].value();
   for (size_t i = 1; i < indices.size(); i++) {
     for (const auto& [pat, expr] : info_vec[indices[i]].matches[i].value()) {
       if (auto it = merged_matches.find(pat); it != merged_matches.end()) {
@@ -571,7 +576,7 @@ std::optional<std::vector<Expr>> TupleRewriterNode::TryMatchByBindingIndex(
   }
 
   auto full_tuple = [&]() -> relax::Expr {
-    Array<Expr> fields;
+    ffi::Array<Expr> fields;
     for (size_t index : indices) {
       fields.push_back(info_vec[index].expr);
     }
@@ -603,17 +608,22 @@ std::optional<std::vector<Expr>> TupleRewriterNode::TryMatchByBindingIndex(
   return rewrites;
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.TupleRewriter")
-    .set_body_typed([](Array<DFPattern> patterns,
-                       ffi::TypedFunction<Optional<Expr>(Expr, Map<DFPattern, Expr>)> func) {
-      return TupleRewriter(patterns, func);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def(
+      "relax.dpl.TupleRewriter",
+      [](ffi::Array<DFPattern> patterns,
+         ffi::TypedFunction<ffi::Optional<Expr>(Expr, ffi::Map<DFPattern, Expr>)> func) {
+        return TupleRewriter(patterns, func);
+      });
+}
 
-TupleRewriter::TupleRewriter(Array<DFPattern> patterns,
-                             ffi::TypedFunction<Optional<Expr>(Expr, Map<DFPattern, Expr>)> func,
-                             Optional<Array<DFPattern>> additional_bindings,
-                             Map<GlobalVar, BaseFunc> new_subroutines) {
-  auto node = make_object<TupleRewriterNode>();
+TupleRewriter::TupleRewriter(
+    ffi::Array<DFPattern> patterns,
+    ffi::TypedFunction<ffi::Optional<Expr>(Expr, ffi::Map<DFPattern, Expr>)> func,
+    ffi::Optional<ffi::Array<DFPattern>> additional_bindings,
+    ffi::Map<GlobalVar, BaseFunc> new_subroutines) {
+  auto node = ffi::make_object<TupleRewriterNode>();
   node->patterns = std::move(patterns);
   node->func = std::move(func);
   node->additional_bindings = std::move(additional_bindings);
@@ -622,8 +632,10 @@ TupleRewriter::TupleRewriter(Array<DFPattern> patterns,
 }
 
 PatternMatchingRewriter PatternMatchingRewriter::FromPattern(
-    DFPattern pattern, ffi::TypedFunction<Optional<Expr>(Expr, Map<DFPattern, Expr>)> func,
-    Optional<Array<DFPattern>> additional_bindings, Map<GlobalVar, BaseFunc> new_subroutines) {
+    DFPattern pattern,
+    ffi::TypedFunction<ffi::Optional<Expr>(Expr, ffi::Map<DFPattern, Expr>)> func,
+    ffi::Optional<ffi::Array<DFPattern>> additional_bindings,
+    ffi::Map<GlobalVar, BaseFunc> new_subroutines) {
   if (auto or_pattern = pattern.as<OrPatternNode>()) {
     auto new_additional_bindings = additional_bindings.value_or({});
     new_additional_bindings.push_back(pattern);
@@ -674,10 +686,10 @@ PatternMatchingRewriter PatternMatchingRewriter::FromModule(IRModule mod) {
     return Downcast<Function>(base_func);
   }();
 
-  Map<GlobalVar, BaseFunc> new_subroutines;
+  ffi::Map<GlobalVar, BaseFunc> new_subroutines;
   for (const auto& [gvar, func] : mod->functions) {
     if (gvar->name_hint != "pattern" && gvar->name_hint != "replacement") {
-      bool is_public = func->GetAttr<String>(tvm::attr::kGlobalSymbol).defined();
+      bool is_public = func->GetAttr<ffi::String>(tvm::attr::kGlobalSymbol).has_value();
       CHECK(!is_public) << "ValueError: "
                         << "Expected module to have no publicly-exposed functions "
                         << "other than 'pattern' and 'replacement'.  "
@@ -695,8 +707,8 @@ PatternMatchingRewriter PatternMatchingRewriter::FromModule(IRModule mod) {
       << "but the pattern has struct info " << sinfo_pattern
       << ", while the replacement has struct info " << sinfo_replacement;
 
-  Array<DFPattern> param_wildcards;
-  Map<Var, DFPattern> pattern_lookup;
+  ffi::Array<DFPattern> param_wildcards;
+  ffi::Map<Var, DFPattern> pattern_lookup;
   for (const auto& param : func_pattern->params) {
     WildcardPattern wildcard;
     param_wildcards.push_back(wildcard);
@@ -748,15 +760,15 @@ PatternMatchingRewriter PatternMatchingRewriter::FromModule(IRModule mod) {
 
   DFPattern top_pattern = make_pattern(func_pattern->body->body);
 
-  ffi::TypedFunction<Optional<Expr>(Expr, Map<DFPattern, Expr>)> rewriter_func =
+  ffi::TypedFunction<ffi::Optional<Expr>(Expr, ffi::Map<DFPattern, Expr>)> rewriter_func =
       [param_wildcards = std::move(param_wildcards),
        orig_func_replacement = std::move(func_replacement)](
-          Expr expr, Map<DFPattern, Expr> matches) -> Optional<Expr> {
+          Expr expr, ffi::Map<DFPattern, Expr> matches) -> ffi::Optional<Expr> {
     auto func_replacement = CopyWithNewVars(orig_func_replacement);
 
-    Array<BindingBlock> new_blocks;
+    ffi::Array<BindingBlock> new_blocks;
 
-    Array<Binding> wildcard_bindings;
+    ffi::Array<Binding> wildcard_bindings;
     ICHECK_EQ(param_wildcards.size(), func_replacement->params.size());
     for (size_t i = 0; i < param_wildcards.size(); i++) {
       Expr matched_expr = matches[param_wildcards[i]];
@@ -783,8 +795,8 @@ PatternMatchingRewriter PatternMatchingRewriter::FromModule(IRModule mod) {
                                               new_subroutines);
 }
 
-Optional<Map<DFPattern, Expr>> ExtractMatchedExpr(DFPattern pattern, Expr expr,
-                                                  Optional<Map<Var, Expr>> bindings_opt) {
+ffi::Optional<ffi::Map<DFPattern, Expr>> ExtractMatchedExpr(
+    DFPattern pattern, Expr expr, ffi::Optional<ffi::Map<Var, Expr>> bindings_opt) {
   auto bindings = bindings_opt.value_or({});
   DFPatternMatcher matcher(bindings);
 
@@ -795,13 +807,19 @@ Optional<Map<DFPattern, Expr>> ExtractMatchedExpr(DFPattern pattern, Expr expr,
   return matcher.GetMemo();
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.extract_matched_expr").set_body_typed(ExtractMatchedExpr);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.dpl.extract_matched_expr", ExtractMatchedExpr);
+}
 
-bool MatchExpr(DFPattern pattern, Expr expr, Optional<Map<Var, Expr>> bindings_opt) {
+bool MatchExpr(DFPattern pattern, Expr expr, ffi::Optional<ffi::Map<Var, Expr>> bindings_opt) {
   return static_cast<bool>(ExtractMatchedExpr(pattern, expr, bindings_opt));
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.match_expr").set_body_typed(MatchExpr);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.dpl.match_expr", MatchExpr);
+}
 
 /*!
  * \brief Apply pattern matching to each expression, replacing
@@ -811,9 +829,10 @@ class PatternMatchingMutator : public ExprMutator {
  public:
   using ExprMutator::VisitExpr_;
 
-  PatternMatchingMutator(const PatternMatchingRewriterNode* rewriter) : rewriter_(rewriter) {}
+  explicit PatternMatchingMutator(const PatternMatchingRewriterNode* rewriter)
+      : rewriter_(rewriter) {}
 
-  Map<GlobalVar, BaseFunc> GetNewSubroutines() const { return new_subroutines_; }
+  ffi::Map<GlobalVar, BaseFunc> GetNewSubroutines() const { return new_subroutines_; }
 
   Expr VisitExpr_(const SeqExprNode* seq) override {
     SeqExpr prev = Downcast<SeqExpr>(ExprMutator::VisitExpr_(seq));
@@ -851,13 +870,13 @@ class PatternMatchingMutator : public ExprMutator {
     return prev;
   }
 
-  Optional<SeqExpr> TryRewriteSeqExpr(const SeqExpr& seq) {
-    Array<BindingBlock> old_blocks = seq->blocks;
+  ffi::Optional<SeqExpr> TryRewriteSeqExpr(const SeqExpr& seq) {
+    ffi::Array<BindingBlock> old_blocks = seq->blocks;
 
     // If the SeqExpr's output is not a variable, treat it as if it
     // were the last variable binding of the last block.  This
     // simplifies the special handling of the SeqExpr's body.
-    Optional<Var> dummy_output_var = std::nullopt;
+    ffi::Optional<Var> dummy_output_var = std::nullopt;
     if (!seq->body->IsInstance<VarNode>()) {
       dummy_output_var = Var("dummy_output_var", GetStructInfo(seq->body));
       VarBinding dummy_binding(dummy_output_var.value(), seq->body);
@@ -868,7 +887,7 @@ class PatternMatchingMutator : public ExprMutator {
           old_blocks.pop_back();
           return last_block;
         } else {
-          return BindingBlock(Array<Binding>{});
+          return BindingBlock(ffi::Array<Binding>{});
         }
       }();
 
@@ -876,7 +895,7 @@ class PatternMatchingMutator : public ExprMutator {
       old_blocks.push_back(last_block);
     }
 
-    auto rewrite_block = [&](Array<Binding> orig_bindings) -> Array<Binding> {
+    auto rewrite_block = [&](ffi::Array<Binding> orig_bindings) -> ffi::Array<Binding> {
       auto rewrites = rewriter_->RewriteBindings(orig_bindings);
       if (!rewrites) return orig_bindings;
 
@@ -911,7 +930,7 @@ class PatternMatchingMutator : public ExprMutator {
 
     // Utility function to return the rewrites that should be applied
     // to a given block.
-    auto get_rewrites = [&](BindingBlock block) -> Array<Binding> {
+    auto get_rewrites = [&](BindingBlock block) -> ffi::Array<Binding> {
       if (block.as<DataflowBlockNode>()) {
         // Early return for DataflowBlock.  Since neither control flow
         // nor impure functions are allowed within the dataflow block,
@@ -921,8 +940,8 @@ class PatternMatchingMutator : public ExprMutator {
 
       RewriteSpec rewrites;
 
-      Array<Binding> collected_bindings;
-      Array<Binding> finalized_bindings;
+      ffi::Array<Binding> collected_bindings;
+      ffi::Array<Binding> finalized_bindings;
 
       auto handle_collected_rewrites = [&]() {
         if (collected_bindings.size()) {
@@ -1019,17 +1038,17 @@ class PatternMatchingMutator : public ExprMutator {
 
  private:
   const PatternMatchingRewriterNode* rewriter_;
-  Map<GlobalVar, BaseFunc> new_subroutines_;
+  ffi::Map<GlobalVar, BaseFunc> new_subroutines_;
 };
 
 Expr PatternMatchingRewriter::operator()(Expr expr) {
   PatternMatchingMutator mutator(get());
   auto new_expr = mutator(expr);
   auto new_subroutines = mutator.GetNewSubroutines();
-  CHECK_EQ(new_subroutines.size(), 0)
-      << "If PatternMatchingRewriter provides subroutines, "
-      << "then it must be applied to an entire IRModule.  "
-      << "However, PatternMatchingRewriter produced subroutines " << [&]() -> Array<GlobalVar> {
+  CHECK_EQ(new_subroutines.size(), 0) << "If PatternMatchingRewriter provides subroutines, "
+                                      << "then it must be applied to an entire IRModule.  "
+                                      << "However, PatternMatchingRewriter produced subroutines "
+                                      << [&]() -> ffi::Array<GlobalVar> {
     std::vector<GlobalVar> vec;
     for (const auto& [gvar, func] : new_subroutines) {
       vec.push_back(gvar);
@@ -1069,18 +1088,22 @@ tvm::transform::PassInfo PatternMatchingRewriterNode::Info() const {
 }
 
 Function RewriteCall(const DFPattern& pat,
-                     ffi::TypedFunction<Expr(Expr, Map<DFPattern, Expr>)> rewriter, Function func) {
+                     ffi::TypedFunction<Expr(Expr, ffi::Map<DFPattern, Expr>)> rewriter,
+                     Function func) {
   return Downcast<Function>(PatternMatchingRewriter::FromPattern(pat, rewriter)(func));
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.rewrite_call").set_body_typed(RewriteCall);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.dpl.rewrite_call", RewriteCall);
+}
 
-TVM_FFI_STATIC_INIT_BLOCK({
+TVM_FFI_STATIC_INIT_BLOCK() {
   PatternMatchingRewriterNode::RegisterReflection();
   ExprPatternRewriterNode::RegisterReflection();
   OrRewriterNode::RegisterReflection();
   TupleRewriterNode::RegisterReflection();
-});
+}
 
 }  // namespace relax
 }  // namespace tvm
