@@ -22,6 +22,7 @@
  * \brief The dataflow pattern language for Relax
  */
 
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/relax/dataflow_pattern.h>
 #include <tvm/relax/dataflow_pattern_functor.h>
 
@@ -31,7 +32,7 @@
 namespace tvm {
 namespace relax {
 
-TVM_FFI_STATIC_INIT_BLOCK({
+TVM_FFI_STATIC_INIT_BLOCK() {
   PatternSeqNode::RegisterReflection();
   ExprPatternNode::RegisterReflection();
   VarPatternNode::RegisterReflection();
@@ -53,7 +54,7 @@ TVM_FFI_STATIC_INIT_BLOCK({
   AttrPatternNode::RegisterReflection();
   ExternFuncPatternNode::RegisterReflection();
   ConstantPatternNode::RegisterReflection();
-});
+}
 
 #define RELAX_PATTERN_PRINTER_DEF(NODE_TYPE, REPR_LAMBDA)                 \
   TVM_STATIC_IR_FUNCTOR(ReprPrinter, vtable)                              \
@@ -62,38 +63,41 @@ TVM_FFI_STATIC_INIT_BLOCK({
         REPR_LAMBDA(p, node);                                             \
       })
 
-TVM_REGISTER_NODE_TYPE(ExternFuncPatternNode);
-ExternFuncPattern::ExternFuncPattern(String global_symbol) {
-  ObjectPtr<ExternFuncPatternNode> n = make_object<ExternFuncPatternNode>();
+ExternFuncPattern::ExternFuncPattern(ffi::String global_symbol) {
+  ObjectPtr<ExternFuncPatternNode> n = ffi::make_object<ExternFuncPatternNode>();
   n->global_symbol_ = std::move(global_symbol);
   data_ = std::move(n);
 }
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.ExternFuncPattern").set_body_typed([](String global_symbol) {
-  return ExternFuncPattern(global_symbol);
-});
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.dpl.ExternFuncPattern",
+                        [](ffi::String global_symbol) { return ExternFuncPattern(global_symbol); });
+}
 RELAX_PATTERN_PRINTER_DEF(ExternFuncPatternNode, [](auto p, auto node) {
   p->stream << "ExternFuncPattern(" << node->global_symbol() << ")";
 });
 
-TVM_REGISTER_NODE_TYPE(VarPatternNode);
-VarPattern::VarPattern(String name_hint) {
-  ObjectPtr<VarPatternNode> n = make_object<VarPatternNode>();
+VarPattern::VarPattern(ffi::String name_hint) {
+  ObjectPtr<VarPatternNode> n = ffi::make_object<VarPatternNode>();
   n->name = std::move(name_hint);
   data_ = std::move(n);
 }
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.VarPattern").set_body_typed([](String name_hint) {
-  return VarPattern(name_hint);
-});
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.dpl.VarPattern",
+                        [](ffi::String name_hint) { return VarPattern(name_hint); });
+}
 RELAX_PATTERN_PRINTER_DEF(VarPatternNode, [](auto p, auto node) {
   p->stream << "VarPattern(" << node->name_hint() << ")";
 });
 
-TVM_REGISTER_NODE_TYPE(DataflowVarPatternNode);
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.DataflowVarPattern").set_body_typed([](String name_hint) {
-  return DataflowVarPattern(name_hint);
-});
-DataflowVarPattern::DataflowVarPattern(String name_hint) {
-  ObjectPtr<DataflowVarPatternNode> n = make_object<DataflowVarPatternNode>();
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.dpl.DataflowVarPattern",
+                        [](ffi::String name_hint) { return DataflowVarPattern(name_hint); });
+}
+DataflowVarPattern::DataflowVarPattern(ffi::String name_hint) {
+  ObjectPtr<DataflowVarPatternNode> n = ffi::make_object<DataflowVarPatternNode>();
   n->name = std::move(name_hint);
   data_ = std::move(n);
 }
@@ -101,50 +105,55 @@ RELAX_PATTERN_PRINTER_DEF(DataflowVarPatternNode, [](auto p, auto node) {
   p->stream << "DataflowVarPattern(" << node->name_hint() << ")";
 });
 
-TVM_REGISTER_NODE_TYPE(GlobalVarPatternNode);
-GlobalVarPattern::GlobalVarPattern(String name_hint) {
-  ObjectPtr<GlobalVarPatternNode> n = make_object<GlobalVarPatternNode>();
+GlobalVarPattern::GlobalVarPattern(ffi::String name_hint) {
+  ObjectPtr<GlobalVarPatternNode> n = ffi::make_object<GlobalVarPatternNode>();
   n->name = std::move(name_hint);
   data_ = std::move(n);
 }
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.GlobalVarPattern").set_body_typed([](String name_hint) {
-  return GlobalVarPattern(name_hint);
-});
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.dpl.GlobalVarPattern",
+                        [](ffi::String name_hint) { return GlobalVarPattern(name_hint); });
+}
 RELAX_PATTERN_PRINTER_DEF(GlobalVarPatternNode, [](auto p, auto node) {
   p->stream << "GlobalVarPattern(" << node->name_hint() << ")";
 });
 
-TVM_REGISTER_NODE_TYPE(ExprPatternNode);
 ExprPattern::ExprPattern(Expr expr) {
-  ObjectPtr<ExprPatternNode> n = make_object<ExprPatternNode>();
+  ObjectPtr<ExprPatternNode> n = ffi::make_object<ExprPatternNode>();
   n->expr = std::move(expr);
   data_ = std::move(n);
 }
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.ExprPattern").set_body_typed([](Expr e) {
-  return ExprPattern(e);
-});
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.dpl.ExprPattern", [](Expr e) { return ExprPattern(e); });
+}
 RELAX_PATTERN_PRINTER_DEF(ExprPatternNode, [](auto p, auto node) { p->Print(node->expr); });
 
-TVM_REGISTER_NODE_TYPE(ConstantPatternNode);
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.ConstantPattern").set_body_typed([]() {
-  auto c = ConstantPattern(make_object<ConstantPatternNode>());
-  return c;
-});
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.dpl.ConstantPattern", []() {
+    auto c = ConstantPattern(ffi::make_object<ConstantPatternNode>());
+    return c;
+  });
+}
 RELAX_PATTERN_PRINTER_DEF(ConstantPatternNode,
                           [](auto p, auto node) { p->stream << "ConstantPattern()"; });
 
-TVM_REGISTER_NODE_TYPE(CallPatternNode);
-CallPattern::CallPattern(DFPattern op, Array<DFPattern> args, bool varg_default_wildcard) {
-  ObjectPtr<CallPatternNode> n = make_object<CallPatternNode>();
+CallPattern::CallPattern(DFPattern op, ffi::Array<DFPattern> args, bool varg_default_wildcard) {
+  ObjectPtr<CallPatternNode> n = ffi::make_object<CallPatternNode>();
   n->op = std::move(op);
   n->args = std::move(args);
   n->varg_default_wildcard = varg_default_wildcard;
   data_ = std::move(n);
 }
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.CallPattern")
-    .set_body_typed([](DFPattern op, Array<DFPattern> args, bool varg_default_wildcard) {
-      return CallPattern(op, args, varg_default_wildcard);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.dpl.CallPattern",
+                        [](DFPattern op, ffi::Array<DFPattern> args, bool varg_default_wildcard) {
+                          return CallPattern(op, args, varg_default_wildcard);
+                        });
+}
 RELAX_PATTERN_PRINTER_DEF(CallPatternNode, [](auto p, auto node) {
   p->stream << node->op << "(";
   for (size_t i = 0; i < node->args.size(); ++i) {
@@ -158,153 +167,167 @@ RELAX_PATTERN_PRINTER_DEF(CallPatternNode, [](auto p, auto node) {
   p->stream << ")";
 });
 
-TVM_REGISTER_NODE_TYPE(PrimArrPatternNode);
-PrimArrPattern::PrimArrPattern(Array<PrimExpr> arr) {
-  ObjectPtr<PrimArrPatternNode> n = make_object<PrimArrPatternNode>();
+PrimArrPattern::PrimArrPattern(ffi::Array<PrimExpr> arr) {
+  ObjectPtr<PrimArrPatternNode> n = ffi::make_object<PrimArrPatternNode>();
   n->fields = std::move(arr);
   data_ = std::move(n);
 }
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.PrimArrPattern").set_body_typed([](Array<PrimExpr> arr) {
-  return PrimArrPattern(std::move(arr));
-});
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.dpl.PrimArrPattern",
+                        [](ffi::Array<PrimExpr> arr) { return PrimArrPattern(std::move(arr)); });
+}
 RELAX_PATTERN_PRINTER_DEF(PrimArrPatternNode, [](auto p, auto node) {
   p->stream << "PrimArrPattern(" << node->fields << ")";
 });
 
-TVM_REGISTER_NODE_TYPE(FunctionPatternNode);
-FunctionPattern::FunctionPattern(Array<DFPattern> params, DFPattern body) {
-  ObjectPtr<FunctionPatternNode> n = make_object<FunctionPatternNode>();
+FunctionPattern::FunctionPattern(ffi::Array<DFPattern> params, DFPattern body) {
+  ObjectPtr<FunctionPatternNode> n = ffi::make_object<FunctionPatternNode>();
   n->params = std::move(params);
   n->body = std::move(body);
   data_ = std::move(n);
 }
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.FunctionPattern")
-    .set_body_typed([](Array<DFPattern> params, DFPattern body) {
-      return FunctionPattern(params, body);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def(
+      "relax.dpl.FunctionPattern",
+      [](ffi::Array<DFPattern> params, DFPattern body) { return FunctionPattern(params, body); });
+}
 RELAX_PATTERN_PRINTER_DEF(FunctionPatternNode, [](auto p, auto node) {
   p->stream << "FunctionPattern(" << node->params << ", " << node->body << ")";
 });
 
-TVM_REGISTER_NODE_TYPE(TuplePatternNode);
-TuplePattern::TuplePattern(tvm::Array<DFPattern> fields) {
-  ObjectPtr<TuplePatternNode> n = make_object<TuplePatternNode>();
+TuplePattern::TuplePattern(tvm::ffi::Array<DFPattern> fields) {
+  ObjectPtr<TuplePatternNode> n = ffi::make_object<TuplePatternNode>();
   n->fields = std::move(fields);
   data_ = std::move(n);
 }
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.TuplePattern").set_body_typed([](tvm::Array<DFPattern> fields) {
-  return TuplePattern(fields);
-});
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.dpl.TuplePattern",
+                        [](tvm::ffi::Array<DFPattern> fields) { return TuplePattern(fields); });
+}
 RELAX_PATTERN_PRINTER_DEF(TuplePatternNode, [](auto p, auto node) {
   p->stream << "TuplePattern(" << node->fields << ")";
 });
 
-TVM_REGISTER_NODE_TYPE(UnorderedTuplePatternNode);
-UnorderedTuplePattern::UnorderedTuplePattern(tvm::Array<DFPattern> fields) {
-  ObjectPtr<UnorderedTuplePatternNode> n = make_object<UnorderedTuplePatternNode>();
+UnorderedTuplePattern::UnorderedTuplePattern(tvm::ffi::Array<DFPattern> fields) {
+  ObjectPtr<UnorderedTuplePatternNode> n = ffi::make_object<UnorderedTuplePatternNode>();
   n->fields = std::move(fields);
   data_ = std::move(n);
 }
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.UnorderedTuplePattern")
-    .set_body_typed([](tvm::Array<DFPattern> fields) { return UnorderedTuplePattern(fields); });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.dpl.UnorderedTuplePattern", [](tvm::ffi::Array<DFPattern> fields) {
+    return UnorderedTuplePattern(fields);
+  });
+}
 RELAX_PATTERN_PRINTER_DEF(UnorderedTuplePatternNode, [](auto p, auto node) {
   p->stream << "UnorderedTuplePattern(" << node->fields << ")";
 });
 
-TVM_REGISTER_NODE_TYPE(TupleGetItemPatternNode);
 TupleGetItemPattern::TupleGetItemPattern(DFPattern tuple, int index) {
-  ObjectPtr<TupleGetItemPatternNode> n = make_object<TupleGetItemPatternNode>();
+  ObjectPtr<TupleGetItemPatternNode> n = ffi::make_object<TupleGetItemPatternNode>();
   n->tuple = std::move(tuple);
   n->index = index;
   data_ = std::move(n);
 }
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.TupleGetItemPattern")
-    .set_body_typed([](DFPattern tuple, int index) { return TupleGetItemPattern(tuple, index); });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.dpl.TupleGetItemPattern", [](DFPattern tuple, int index) {
+    return TupleGetItemPattern(tuple, index);
+  });
+}
 RELAX_PATTERN_PRINTER_DEF(TupleGetItemPatternNode, [](auto p, auto node) {
   p->stream << "TupleGetItemPattern(" << node->tuple << ", " << node->index << ")";
 });
 
-TVM_REGISTER_NODE_TYPE(AndPatternNode);
 AndPattern::AndPattern(DFPattern left, DFPattern right) {
-  ObjectPtr<AndPatternNode> n = make_object<AndPatternNode>();
+  ObjectPtr<AndPatternNode> n = ffi::make_object<AndPatternNode>();
   n->left = std::move(left);
   n->right = std::move(right);
   data_ = std::move(n);
 }
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.AndPattern").set_body_typed([](DFPattern left, DFPattern right) {
-  return AndPattern(left, right);
-});
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.dpl.AndPattern",
+                        [](DFPattern left, DFPattern right) { return AndPattern(left, right); });
+}
 RELAX_PATTERN_PRINTER_DEF(AndPatternNode, [](auto p, auto node) {
   p->stream << "AndPattern(" << node->left << " & " << node->right << ")";
 });
 
-TVM_REGISTER_NODE_TYPE(OrPatternNode);
 OrPattern::OrPattern(DFPattern left, DFPattern right) {
-  ObjectPtr<OrPatternNode> n = make_object<OrPatternNode>();
+  ObjectPtr<OrPatternNode> n = ffi::make_object<OrPatternNode>();
   n->left = std::move(left);
   n->right = std::move(right);
   data_ = std::move(n);
 }
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.OrPattern").set_body_typed([](DFPattern left, DFPattern right) {
-  return OrPattern(left, right);
-});
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.dpl.OrPattern",
+                        [](DFPattern left, DFPattern right) { return OrPattern(left, right); });
+}
 RELAX_PATTERN_PRINTER_DEF(OrPatternNode, [](auto p, auto node) {
   p->stream << "OrPattern(" << node->left << " | " << node->right << ")";
 });
 
-TVM_REGISTER_NODE_TYPE(NotPatternNode);
 NotPattern::NotPattern(DFPattern reject) {
-  ObjectPtr<NotPatternNode> n = make_object<NotPatternNode>();
+  ObjectPtr<NotPatternNode> n = ffi::make_object<NotPatternNode>();
   n->reject = std::move(reject);
   data_ = std::move(n);
 }
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.NotPattern").set_body_typed([](DFPattern reject) {
-  return NotPattern(reject);
-});
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.dpl.NotPattern",
+                        [](DFPattern reject) { return NotPattern(reject); });
+}
 RELAX_PATTERN_PRINTER_DEF(NotPatternNode,
                           [](auto p, auto node) { p->stream << "!(" << node->reject << ")"; });
 
-TVM_REGISTER_NODE_TYPE(WildcardPatternNode);
-WildcardPattern::WildcardPattern() { data_ = make_object<WildcardPatternNode>(); }
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.WildcardPattern").set_body_typed([]() {
-  return WildcardPattern();
-});
+WildcardPattern::WildcardPattern() { data_ = ffi::make_object<WildcardPatternNode>(); }
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.dpl.WildcardPattern", []() { return WildcardPattern(); });
+}
 RELAX_PATTERN_PRINTER_DEF(WildcardPatternNode, [](auto p, auto node) { p->stream << "*"; });
 
-TVM_REGISTER_NODE_TYPE(StructInfoPatternNode);
 StructInfoPattern::StructInfoPattern(DFPattern pattern, StructInfo struct_info) {
-  ObjectPtr<StructInfoPatternNode> n = make_object<StructInfoPatternNode>();
+  ObjectPtr<StructInfoPatternNode> n = ffi::make_object<StructInfoPatternNode>();
   n->pattern = std::move(pattern);
   n->struct_info = std::move(struct_info);
   data_ = std::move(n);
 }
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.StructInfoPattern")
-    .set_body_typed([](DFPattern pattern, StructInfo struct_info) {
-      return StructInfoPattern(pattern, struct_info);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.dpl.StructInfoPattern",
+                        [](DFPattern pattern, StructInfo struct_info) {
+                          return StructInfoPattern(pattern, struct_info);
+                        });
+}
 RELAX_PATTERN_PRINTER_DEF(StructInfoPatternNode, [](auto p, auto node) {
   p->stream << "StructInfoPattern(" << node->pattern << " has relax StructInfo "
             << node->struct_info << ")";
 });
 
-TVM_REGISTER_NODE_TYPE(ShapePatternNode);
-ShapePattern::ShapePattern(DFPattern pattern, Array<PrimExpr> shape) {
-  ObjectPtr<ShapePatternNode> n = make_object<ShapePatternNode>();
+ShapePattern::ShapePattern(DFPattern pattern, ffi::Array<PrimExpr> shape) {
+  ObjectPtr<ShapePatternNode> n = ffi::make_object<ShapePatternNode>();
   n->pattern = std::move(pattern);
   n->shape = std::move(shape);
   data_ = std::move(n);
 }
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.ShapePattern")
-    .set_body_typed([](DFPattern pattern, Array<PrimExpr> shape) {
-      return ShapePattern(pattern, shape);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def(
+      "relax.dpl.ShapePattern",
+      [](DFPattern pattern, ffi::Array<PrimExpr> shape) { return ShapePattern(pattern, shape); });
+}
 RELAX_PATTERN_PRINTER_DEF(ShapePatternNode, [](auto p, auto node) {
   p->stream << "ShapePattern(" << node->pattern << " has shape " << node->shape << ")";
 });
 
-TVM_REGISTER_NODE_TYPE(SameShapeConstraintNode);
-SameShapeConstraint::SameShapeConstraint(Array<DFPattern> args) {
-  ObjectPtr<SameShapeConstraintNode> n = make_object<SameShapeConstraintNode>();
+SameShapeConstraint::SameShapeConstraint(ffi::Array<DFPattern> args) {
+  ObjectPtr<SameShapeConstraintNode> n = ffi::make_object<SameShapeConstraintNode>();
   n->args = std::move(args);
   data_ = std::move(n);
 
@@ -312,9 +335,11 @@ SameShapeConstraint::SameShapeConstraint(Array<DFPattern> args) {
     ctx.value().add_constraint(*this);
   }
 }
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.SameShapeConstraint").set_body_typed([](Array<DFPattern> args) {
-  return SameShapeConstraint(args);
-});
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.dpl.SameShapeConstraint",
+                        [](ffi::Array<DFPattern> args) { return SameShapeConstraint(args); });
+}
 RELAX_PATTERN_PRINTER_DEF(SameShapeConstraintNode, [](auto p, auto node) {
   p->stream << "SameShapeConstraint(";
   for (size_t i = 0; i < node->args.size(); i++) {
@@ -326,30 +351,34 @@ RELAX_PATTERN_PRINTER_DEF(SameShapeConstraintNode, [](auto p, auto node) {
   p->stream << ")";
 });
 
-TVM_REGISTER_NODE_TYPE(DataTypePatternNode);
 DataTypePattern::DataTypePattern(DFPattern pattern, DataType dtype) {
-  ObjectPtr<DataTypePatternNode> n = make_object<DataTypePatternNode>();
+  ObjectPtr<DataTypePatternNode> n = ffi::make_object<DataTypePatternNode>();
   n->pattern = std::move(pattern);
   n->dtype = std::move(dtype);
   data_ = std::move(n);
 }
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.DataTypePattern")
-    .set_body_typed([](DFPattern pattern, DataType dtype) {
-      return DataTypePattern(pattern, dtype);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.dpl.DataTypePattern", [](DFPattern pattern, DataType dtype) {
+    return DataTypePattern(pattern, dtype);
+  });
+}
 RELAX_PATTERN_PRINTER_DEF(DataTypePatternNode, [](auto p, auto node) {
   p->stream << "DataTypePattern(" << node->pattern << " has dtype " << node->dtype << ")";
 });
 
-TVM_REGISTER_NODE_TYPE(AttrPatternNode);
 AttrPattern::AttrPattern(DFPattern pattern, DictAttrs attrs) {
-  ObjectPtr<AttrPatternNode> n = make_object<AttrPatternNode>();
+  ObjectPtr<AttrPatternNode> n = ffi::make_object<AttrPatternNode>();
   n->pattern = std::move(pattern);
   n->attrs = std::move(attrs);
   data_ = std::move(n);
 }
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.AttrPattern")
-    .set_body_typed([](DFPattern pattern, DictAttrs attrs) { return AttrPattern(pattern, attrs); });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.dpl.AttrPattern", [](DFPattern pattern, DictAttrs attrs) {
+    return AttrPattern(pattern, attrs);
+  });
+}
 RELAX_PATTERN_PRINTER_DEF(AttrPatternNode, [](auto p, auto node) {
   p->stream << "AttrPattern(" << node->pattern << " has attributes " << node->attrs << ")";
 });
@@ -368,10 +397,10 @@ class DFPatternDuplicator : public DFPatternFunctor<DFPattern(const DFPattern&)>
   DFPattern VisitDFPattern_(const NotPatternNode* op) override { return NotPattern(op->reject); }
   DFPattern VisitDFPattern_(const VarPatternNode* op) override { return VarPattern(op->name); }
   DFPattern VisitDFPattern_(const ConstantPatternNode* op) override {
-    return ConstantPattern(make_object<ConstantPatternNode>());
+    return ConstantPattern(ffi::make_object<ConstantPatternNode>());
   }
   DFPattern VisitDFPattern_(const WildcardPatternNode* op) override {
-    return WildcardPattern(make_object<WildcardPatternNode>());
+    return WildcardPattern(ffi::make_object<WildcardPatternNode>());
   }
   DFPattern VisitDFPattern_(const ExprPatternNode* op) override { return ExprPattern(op->expr); }
   DFPattern VisitDFPattern_(const GlobalVarPatternNode* op) override {
@@ -415,7 +444,7 @@ class DFPatternDuplicator : public DFPatternFunctor<DFPattern(const DFPattern&)>
 
 // Syntatic Sugar
 CallPattern DFPattern::operator()(const std::vector<DFPattern>& args) const {
-  return CallPattern(*this, Array<DFPattern>(args));
+  return CallPattern(*this, ffi::Array<DFPattern>(args));
 }
 OrPattern DFPattern::operator|(const DFPattern& other) const { return OrPattern(*this, other); }
 
@@ -423,7 +452,7 @@ AndPattern DFPattern::operator&(const DFPattern& other) const { return AndPatter
 
 NotPattern DFPattern::operator~() const { return NotPattern(*this); }
 
-AttrPattern DFPattern::HasAttr(const Map<String, Any>& attrs) const {
+AttrPattern DFPattern::HasAttr(const ffi::Map<ffi::String, Any>& attrs) const {
   return AttrPattern(*this, DictAttrs(attrs));
 }
 StructInfoPattern DFPattern::HasStructInfo(const StructInfo& struct_info) const {
@@ -435,7 +464,7 @@ DataTypePattern DFPattern::HasDtype(const DataType& dtype) const {
 DataTypePattern DFPattern::HasDtype(const std::string& dtype) const {
   return HasDtype(DataType(ffi::StringToDLDataType(dtype)));
 }
-ShapePattern DFPattern::HasShape(const Array<PrimExpr>& shape) const {
+ShapePattern DFPattern::HasShape(const ffi::Array<PrimExpr>& shape) const {
   return ShapePattern(*this, shape);
 }
 
@@ -446,13 +475,13 @@ std::stack<PatternContext>& pattern_ctx_stack() {
   return graph_pattern_managers;
 }
 
-Optional<PatternContext> PatternContext::Current() {
+ffi::Optional<PatternContext> PatternContext::Current() {
   if (pattern_ctx_stack().empty()) return std::nullopt;
   return pattern_ctx_stack().top();
 }
 
 PatternContext::PatternContext(bool incremental) {
-  auto n = make_object<PatternContextNode>();
+  auto n = ffi::make_object<PatternContextNode>();
   if (incremental) {
     ICHECK(!pattern_ctx_stack().empty())
         << "Incremental context needs to be built inside a existing context.";
@@ -477,18 +506,17 @@ static void sync_graph_constraints(const DFPattern& lhs, const DFPattern& rhs, P
   }
 }
 
-TVM_REGISTER_NODE_TYPE(PatternSeqNode);
 PatternSeq::PatternSeq(DFPattern init_pattern) {
-  ObjectPtr<PatternSeqNode> n = make_object<PatternSeqNode>();
+  ObjectPtr<PatternSeqNode> n = ffi::make_object<PatternSeqNode>();
   n->patterns = {init_pattern};
   n->pair_constraints = {};
   data_ = std::move(n);
 }
-PatternSeq::PatternSeq(tvm::Array<DFPattern> patterns, bool only_used_by) {
+PatternSeq::PatternSeq(tvm::ffi::Array<DFPattern> patterns, bool only_used_by) {
   ICHECK_GE(patterns.size(), 1) << "PatternSeq must have at least one pattern";
   const auto cons = PairCons(only_used_by ? PairCons::kOnlyUsedBy : PairCons::kUsedBy);
 
-  ObjectPtr<PatternSeqNode> n = make_object<PatternSeqNode>();
+  ObjectPtr<PatternSeqNode> n = ffi::make_object<PatternSeqNode>();
   n->patterns = std::move(patterns);
   n->pair_constraints = std::vector<PairCons>(n->patterns.size() - 1, cons);
   data_ = std::move(n);
@@ -505,8 +533,8 @@ PatternSeq PatternSeq::OnlyUsedBy(PatternSeq other, int index) const {
 PatternSeq PatternSeq::dup() const {
   PatternSeq ret;
 
-  ObjectPtr<PatternSeqNode> n = make_object<PatternSeqNode>();
-  n->patterns = Array<DFPattern>{};
+  ObjectPtr<PatternSeqNode> n = ffi::make_object<PatternSeqNode>();
+  n->patterns = ffi::Array<DFPattern>{};
   n->patterns.reserve(get()->patterns.size());
   n->pair_constraints = this->get()->pair_constraints;
 
@@ -520,10 +548,13 @@ PatternSeq PatternSeq::dup() const {
 
   return ret;
 }
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.PatternSeq")
-    .set_body_typed([](Array<DFPattern> patterns, bool only_used_by) {
-      return PatternSeq(std::move(patterns), only_used_by);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.dpl.PatternSeq",
+                        [](ffi::Array<DFPattern> patterns, bool only_used_by) {
+                          return PatternSeq(std::move(patterns), only_used_by);
+                        });
+}
 RELAX_PATTERN_PRINTER_DEF(PatternSeqNode, [](auto p, auto node) {
   p->stream << "[";
   for (size_t i = 0; i < node->patterns.size(); ++i) {
@@ -534,15 +565,14 @@ RELAX_PATTERN_PRINTER_DEF(PatternSeqNode, [](auto p, auto node) {
   p->stream << "]";
 });
 
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.used_by")
-    .set_body_typed([](PatternSeq lhs, PatternSeq rhs, int index) {
-      return lhs.UsedBy(rhs, index);
-    });
-
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.only_used_by")
-    .set_body_typed([](PatternSeq lhs, PatternSeq rhs, int index) {
-      return lhs.OnlyUsedBy(rhs, index);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def("relax.dpl.used_by",
+           [](PatternSeq lhs, PatternSeq rhs, int index) { return lhs.UsedBy(rhs, index); })
+      .def("relax.dpl.only_used_by",
+           [](PatternSeq lhs, PatternSeq rhs, int index) { return lhs.OnlyUsedBy(rhs, index); });
+}
 
 PatternSeq UsedBy(const PatternSeq& lhs, const PatternSeq& rhs, int index) {
   PatternSeq ret;
@@ -552,7 +582,7 @@ PatternSeq UsedBy(const PatternSeq& lhs, const PatternSeq& rhs, int index) {
   sync_graph_constraints(lhs->patterns.back(), rhs->patterns.front(),
                          PairCons{PairCons::kUsedBy, index});
 
-  Array<DFPattern> patterns;
+  ffi::Array<DFPattern> patterns;
   patterns.reserve(lhs->patterns.size() + rhs->patterns.size());
   patterns.insert(patterns.end(), lhs->patterns.begin(), lhs->patterns.end());
   patterns.insert(patterns.end(), rhs->patterns.begin(), rhs->patterns.end());
@@ -563,7 +593,7 @@ PatternSeq UsedBy(const PatternSeq& lhs, const PatternSeq& rhs, int index) {
   pair_constraints.insert(pair_constraints.end(), rhs->pair_constraints.begin(),
                           rhs->pair_constraints.end());
 
-  ObjectPtr<PatternSeqNode> n = make_object<PatternSeqNode>();
+  ObjectPtr<PatternSeqNode> n = ffi::make_object<PatternSeqNode>();
   n->patterns = std::move(patterns);
   n->pair_constraints = std::move(pair_constraints);
   ret.data_ = std::move(n);
@@ -579,7 +609,7 @@ PatternSeq OnlyUsedBy(const PatternSeq& lhs, const PatternSeq& rhs, int index) {
 
   sync_graph_constraints(lhs->patterns.back(), rhs->patterns.front(), constraint);
 
-  Array<DFPattern> patterns;
+  ffi::Array<DFPattern> patterns;
   patterns.reserve(lhs->patterns.size() + rhs->patterns.size());
   patterns.insert(patterns.end(), lhs->patterns.begin(), lhs->patterns.end());
   patterns.insert(patterns.end(), rhs->patterns.begin(), rhs->patterns.end());
@@ -590,7 +620,7 @@ PatternSeq OnlyUsedBy(const PatternSeq& lhs, const PatternSeq& rhs, int index) {
   pair_constraints.insert(pair_constraints.end(), rhs->pair_constraints.begin(),
                           rhs->pair_constraints.end());
 
-  ObjectPtr<PatternSeqNode> n = make_object<PatternSeqNode>();
+  ObjectPtr<PatternSeqNode> n = ffi::make_object<PatternSeqNode>();
   n->patterns = std::move(patterns);
   n->pair_constraints = std::move(pair_constraints);
   ret.data_ = std::move(n);
@@ -599,13 +629,13 @@ PatternSeq OnlyUsedBy(const PatternSeq& lhs, const PatternSeq& rhs, int index) {
 }
 PatternSeq operator>>(const PatternSeq& lhs, const PatternSeq& rhs) { return lhs.OnlyUsedBy(rhs); }
 
-VarPattern IsVar(const String& name) { return VarPattern(name); }
-ConstantPattern IsConst() { return ConstantPattern(make_object<ConstantPatternNode>()); }
-WildcardPattern Wildcard() { return WildcardPattern(make_object<WildcardPatternNode>()); }
+VarPattern IsVar(const ffi::String& name) { return VarPattern(name); }
+ConstantPattern IsConst() { return ConstantPattern(ffi::make_object<ConstantPatternNode>()); }
+WildcardPattern Wildcard() { return WildcardPattern(ffi::make_object<WildcardPatternNode>()); }
 ExprPattern IsExpr(const Expr& expr) { return ExprPattern(expr); }
-ExprPattern IsOp(const String& op_name) { return IsExpr(Op::Get(op_name)); }
-CallPattern IsCallTIR(const String& name, Optional<TuplePattern> var_args,
-                      Optional<DFPattern> tir_vars) {
+ExprPattern IsOp(const ffi::String& op_name) { return IsExpr(Op::Get(op_name)); }
+CallPattern IsCallTIR(const ffi::String& name, ffi::Optional<TuplePattern> var_args,
+                      ffi::Optional<DFPattern> tir_vars) {
   DFPattern arg_pattern;
   if (!var_args.defined()) {
     arg_pattern = Wildcard();
@@ -619,10 +649,10 @@ CallPattern IsCallTIR(const String& name, Optional<TuplePattern> var_args,
   return IsOp("relax.call_tir")(GlobalVarPattern(name), arg_pattern);
 }
 
-CallPattern IsCallTIR(const String& name, TuplePattern var_args) {
+CallPattern IsCallTIR(const ffi::String& name, TuplePattern var_args) {
   return IsOp("relax.call_tir")(GlobalVarPattern(name), var_args);
 }
-CallPattern IsCallDPSPacked(const String& name, Optional<TuplePattern> var_args) {
+CallPattern IsCallDPSPacked(const ffi::String& name, ffi::Optional<TuplePattern> var_args) {
   DFPattern arg_pattern;
   if (!var_args.defined()) {
     arg_pattern = Wildcard();
@@ -633,11 +663,11 @@ CallPattern IsCallDPSPacked(const String& name, Optional<TuplePattern> var_args)
   return IsOp("relax.call_dps_packed")(GlobalVarPattern(name), arg_pattern);
 }
 
-CallPattern IsCallDPSPacked(const String& name, TuplePattern var_args) {
+CallPattern IsCallDPSPacked(const ffi::String& name, TuplePattern var_args) {
   return IsOp("relax.call_dps_packed")(GlobalVarPattern(name), var_args);
 }
 
-DFPattern IsTuple(const Array<DFPattern>& fields, bool unordered) {
+DFPattern IsTuple(const ffi::Array<DFPattern>& fields, bool unordered) {
   if (unordered)
     return UnorderedTuplePattern(fields);
   else
@@ -652,29 +682,16 @@ DFPattern DFPattern::dup() const {
   return pattern;
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.dup_pattern").set_body_typed([](DFPattern pattern) {
-  return pattern.dup();
-});
-
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.dup_seq").set_body_typed([](PatternSeq seq) {
-  return seq.dup();
-});
-
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.PatternContext").set_body_typed([](bool incre) {
-  return PatternContext(incre);
-});
-
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.current_context").set_body_typed([] {
-  return PatternContext::Current();
-});
-
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.enter_context").set_body_typed([](const PatternContext& ctx) {
-  ctx.EnterWithScope();
-});
-
-TVM_FFI_REGISTER_GLOBAL("relax.dpl.exit_context").set_body_typed([](const PatternContext& ctx) {
-  ctx.ExitWithScope();
-});
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def("relax.dpl.dup_pattern", [](DFPattern pattern) { return pattern.dup(); })
+      .def("relax.dpl.dup_seq", [](PatternSeq seq) { return seq.dup(); })
+      .def("relax.dpl.PatternContext", [](bool incre) { return PatternContext(incre); })
+      .def("relax.dpl.current_context", [] { return PatternContext::Current(); })
+      .def("relax.dpl.enter_context", [](const PatternContext& ctx) { ctx.EnterWithScope(); })
+      .def("relax.dpl.exit_context", [](const PatternContext& ctx) { ctx.ExitWithScope(); });
+}
 
 }  // namespace relax
 }  // namespace tvm
