@@ -22,6 +22,7 @@
  */
 #include "./emit_te.h"
 
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/relax/struct_info.h>
 #include <tvm/tir/stmt_functor.h>
 
@@ -35,12 +36,10 @@ TVM_STATIC_IR_FUNCTOR(ReprPrinter, vtable)
       p->stream << "rxplaceholder(" << op->name << ", " << op << ")";
     });
 
-TVM_FFI_STATIC_INIT_BLOCK({ RXPlaceholderOpNode::RegisterReflection(); });
+TVM_FFI_STATIC_INIT_BLOCK() { RXPlaceholderOpNode::RegisterReflection(); }
 
-TVM_REGISTER_NODE_TYPE(RXPlaceholderOpNode);
-
-te::Tensor TETensor(Expr value, Map<tir::Var, PrimExpr> tir_var_map, std::string name) {
-  auto n = make_object<RXPlaceholderOpNode>();
+te::Tensor TETensor(Expr value, ffi::Map<tir::Var, PrimExpr> tir_var_map, std::string name) {
+  auto n = ffi::make_object<RXPlaceholderOpNode>();
   n->name = name;
   n->value = value;
 
@@ -52,7 +51,7 @@ te::Tensor TETensor(Expr value, Map<tir::Var, PrimExpr> tir_var_map, std::string
 
     int ndim = constant->data->ndim;
     ffi::Shape shape_tuple = constant->data.Shape();
-    Array<PrimExpr> shape;
+    ffi::Array<PrimExpr> shape;
     shape.reserve(ndim);
     for (int i = 0; i < ndim; ++i) {
       shape.push_back(IntImm(DataType::Int(64), shape_tuple[i]));
@@ -74,7 +73,10 @@ te::Tensor TETensor(Expr value, Map<tir::Var, PrimExpr> tir_var_map, std::string
   return te::PlaceholderOp(n).output(0);
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.TETensor").set_body_typed(TETensor);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.TETensor", TETensor);
+}
 
 }  // namespace relax
 }  // namespace tvm
