@@ -16,7 +16,7 @@
  * specific language governing permissions and limitations
  * under the License.
  */
-#include <tvm/ffi/reflection/reflection.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/relax/analysis.h>
 #include <tvm/relax/expr.h>
 #include <tvm/relax/struct_info.h>
@@ -29,7 +29,7 @@ namespace relax {
 
 using tvm::ReprPrinter;
 
-TVM_FFI_STATIC_INIT_BLOCK({
+TVM_FFI_STATIC_INIT_BLOCK() {
   IdNode::RegisterReflection();
   CallNode::RegisterReflection();
   TupleNode::RegisterReflection();
@@ -50,24 +50,23 @@ TVM_FFI_STATIC_INIT_BLOCK({
   IfNode::RegisterReflection();
   FunctionNode::RegisterReflection();
   ExternFuncNode::RegisterReflection();
-});
+}
 
-TVM_REGISTER_NODE_TYPE(IdNode);
-
-Id::Id(String name_hint) {
-  ObjectPtr<IdNode> n = make_object<IdNode>();
+Id::Id(ffi::String name_hint) {
+  ObjectPtr<IdNode> n = ffi::make_object<IdNode>();
   n->name_hint = std::move(name_hint);
   data_ = std::move(n);
 }
 
-Call::Call(Expr op, Array<Expr> args, Attrs attrs, Array<StructInfo> sinfo_args, Span span) {
+Call::Call(Expr op, ffi::Array<Expr> args, Attrs attrs, ffi::Array<StructInfo> sinfo_args,
+           Span span) {
   CHECK(!op->struct_info_.defined() || op->struct_info_->IsInstance<FuncStructInfoNode>())
       << "ValueError: "
       << "Call expects its operator to have FuncStructInfo, "
       << "but operator " << op << ", which was called with arguments " << args
       << ", has struct info " << op->struct_info_;
 
-  ObjectPtr<CallNode> n = make_object<CallNode>();
+  ObjectPtr<CallNode> n = ffi::make_object<CallNode>();
   n->op = std::move(op);
   n->args = std::move(args);
   n->attrs = std::move(attrs);
@@ -76,14 +75,15 @@ Call::Call(Expr op, Array<Expr> args, Attrs attrs, Array<StructInfo> sinfo_args,
   data_ = std::move(n);
 }
 
-Call WithFields(Call call, Optional<Expr> opt_op, Optional<Array<Expr>> opt_args,
-                Optional<Attrs> opt_attrs, Optional<Array<StructInfo>> opt_sinfo_args,
-                Optional<Span> opt_span) {
+Call WithFields(Call call, ffi::Optional<Expr> opt_op, ffi::Optional<ffi::Array<Expr>> opt_args,
+                ffi::Optional<Attrs> opt_attrs,
+                ffi::Optional<ffi::Array<StructInfo>> opt_sinfo_args,
+                ffi::Optional<Span> opt_span) {
   // Collect new values for fields.
   Expr op = opt_op.value_or(call->op);
-  Array<Expr> args = opt_args.value_or(call->args);
+  ffi::Array<Expr> args = opt_args.value_or(call->args);
   Attrs attrs = opt_attrs.value_or(call->attrs);
-  Array<StructInfo> sinfo_args = opt_sinfo_args.value_or(call->sinfo_args);
+  ffi::Array<StructInfo> sinfo_args = opt_sinfo_args.value_or(call->sinfo_args);
   Span span = opt_span.value_or(call->span);
 
   // Check if anything changed.
@@ -119,14 +119,16 @@ Call WithFields(Call call, Optional<Expr> opt_op, Optional<Array<Expr>> opt_args
   return call;
 }
 
-TVM_REGISTER_NODE_TYPE(CallNode);
-
-TVM_FFI_REGISTER_GLOBAL("relax.Call")
-    .set_body_typed([](Expr op, Array<Expr> args, Attrs attrs, Array<StructInfo> sinfo_args,
-                       Span span) { return Call(op, args, attrs, sinfo_args, span); });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.Call", [](Expr op, ffi::Array<Expr> args, Attrs attrs,
+                                         ffi::Array<StructInfo> sinfo_args, Span span) {
+    return Call(op, args, attrs, sinfo_args, span);
+  });
+}
 
 If::If(Expr cond, Expr true_branch, Expr false_branch, Span span) {
-  ObjectPtr<IfNode> n = make_object<IfNode>();
+  ObjectPtr<IfNode> n = ffi::make_object<IfNode>();
   n->cond = std::move(cond);
   n->true_branch = std::move(true_branch);
   n->false_branch = std::move(false_branch);
@@ -134,8 +136,8 @@ If::If(Expr cond, Expr true_branch, Expr false_branch, Span span) {
   data_ = std::move(n);
 }
 
-If WithFields(If if_expr, Optional<Expr> opt_cond, Optional<Expr> opt_true_branch,
-              Optional<Expr> opt_false_branch, Optional<Span> opt_span) {
+If WithFields(If if_expr, ffi::Optional<Expr> opt_cond, ffi::Optional<Expr> opt_true_branch,
+              ffi::Optional<Expr> opt_false_branch, ffi::Optional<Span> opt_span) {
   Expr cond = opt_cond.value_or(if_expr->cond);
   Expr true_branch = opt_true_branch.value_or(if_expr->true_branch);
   Expr false_branch = opt_false_branch.value_or(if_expr->false_branch);
@@ -154,16 +156,16 @@ If WithFields(If if_expr, Optional<Expr> opt_cond, Optional<Expr> opt_true_branc
   return if_expr;
 }
 
-TVM_REGISTER_NODE_TYPE(IfNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.If", [](Expr cond, Expr true_branch, Expr false_branch, Span span) {
+    return If(cond, true_branch, false_branch, span);
+  });
+}
 
-TVM_FFI_REGISTER_GLOBAL("relax.If")
-    .set_body_typed([](Expr cond, Expr true_branch, Expr false_branch, Span span) {
-      return If(cond, true_branch, false_branch, span);
-    });
-
-Tuple::Tuple(tvm::Array<Expr> fields, Span span) {
-  Optional<StructInfo> tuple_sinfo = [&]() -> Optional<StructInfo> {
-    Array<StructInfo> field_sinfo;
+Tuple::Tuple(tvm::ffi::Array<Expr> fields, Span span) {
+  ffi::Optional<StructInfo> tuple_sinfo = [&]() -> ffi::Optional<StructInfo> {
+    ffi::Array<StructInfo> field_sinfo;
     for (const auto& field : fields) {
       if (field->struct_info_.defined()) {
         field_sinfo.push_back(GetStructInfo(field));
@@ -174,21 +176,22 @@ Tuple::Tuple(tvm::Array<Expr> fields, Span span) {
     return TupleStructInfo(field_sinfo);
   }();
 
-  ObjectPtr<TupleNode> n = make_object<TupleNode>();
+  ObjectPtr<TupleNode> n = ffi::make_object<TupleNode>();
   n->fields = std::move(fields);
   n->span = std::move(span);
   n->struct_info_ = tuple_sinfo;
   data_ = std::move(n);
 }
 
-TVM_REGISTER_NODE_TYPE(TupleNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def(
+      "relax.Tuple", [](tvm::ffi::Array<Expr> fields, Span span) { return Tuple(fields, span); });
+}
 
-TVM_FFI_REGISTER_GLOBAL("relax.Tuple").set_body_typed([](tvm::Array<Expr> fields, Span span) {
-  return Tuple(fields, span);
-});
-
-Tuple WithFields(Tuple tuple, Optional<Array<Expr>> opt_fields, Optional<Span> opt_span) {
-  Array<Expr> fields = opt_fields.value_or(tuple->fields);
+Tuple WithFields(Tuple tuple, ffi::Optional<ffi::Array<Expr>> opt_fields,
+                 ffi::Optional<Span> opt_span) {
+  ffi::Array<Expr> fields = opt_fields.value_or(tuple->fields);
   Span span = opt_span.value_or(tuple->span);
 
   bool all_fields_unchanged = true;
@@ -212,7 +215,7 @@ Tuple WithFields(Tuple tuple, Optional<Array<Expr>> opt_fields, Optional<Span> o
 TupleGetItem::TupleGetItem(Expr tuple, int index, Span span) {
   CHECK_GE(index, 0) << "Index out of bounds: Tuple " << tuple
                      << " cannot be accessed with negative index " << index;
-  ObjectPtr<TupleGetItemNode> n = make_object<TupleGetItemNode>();
+  ObjectPtr<TupleGetItemNode> n = ffi::make_object<TupleGetItemNode>();
 
   if (auto* tuple_info = tuple->struct_info_.as<TupleStructInfoNode>()) {
     CHECK_LT(index, tuple_info->fields.size())
@@ -227,8 +230,8 @@ TupleGetItem::TupleGetItem(Expr tuple, int index, Span span) {
   data_ = std::move(n);
 }
 
-TupleGetItem WithFields(TupleGetItem tuple_get_item, Optional<Expr> opt_tuple,
-                        Optional<Integer> opt_index, Optional<Span> opt_span) {
+TupleGetItem WithFields(TupleGetItem tuple_get_item, ffi::Optional<Expr> opt_tuple,
+                        ffi::Optional<Integer> opt_index, ffi::Optional<Span> opt_span) {
   Expr tuple = opt_tuple.value_or(tuple_get_item->tuple);
   Integer index = opt_index.value_or(tuple_get_item->index);
   Span span = opt_span.value_or(tuple_get_item->span);
@@ -244,16 +247,15 @@ TupleGetItem WithFields(TupleGetItem tuple_get_item, Optional<Expr> opt_tuple,
   return tuple_get_item;
 }
 
-TVM_REGISTER_NODE_TYPE(TupleGetItemNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.TupleGetItem", [](Expr tuple, int index, Span span) {
+    return TupleGetItem(tuple, index, span);
+  });
+}
 
-TVM_FFI_REGISTER_GLOBAL("relax.TupleGetItem").set_body_typed([](Expr tuple, int index, Span span) {
-  return TupleGetItem(tuple, index, span);
-});
-
-TVM_REGISTER_NODE_TYPE(ShapeExprNode);
-
-ShapeExpr::ShapeExpr(Array<PrimExpr> values, Span span) {
-  ObjectPtr<ShapeExprNode> n = make_object<ShapeExprNode>();
+ShapeExpr::ShapeExpr(ffi::Array<PrimExpr> values, Span span) {
+  ObjectPtr<ShapeExprNode> n = ffi::make_object<ShapeExprNode>();
 
   n->values = values.Map([](PrimExpr value) {
     if (value->IsInstance<IntImmNode>()) {
@@ -268,14 +270,15 @@ ShapeExpr::ShapeExpr(Array<PrimExpr> values, Span span) {
   data_ = std::move(n);
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.ShapeExpr").set_body_typed([](Array<PrimExpr> values, Span span) {
-  return ShapeExpr(values, span);
-});
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.ShapeExpr", [](ffi::Array<PrimExpr> values, Span span) {
+    return ShapeExpr(values, span);
+  });
+}
 
-TVM_REGISTER_NODE_TYPE(VarNode);
-
-Var::Var(Id vid, Optional<StructInfo> struct_info_annotation, Span span) {
-  ObjectPtr<VarNode> n = make_object<VarNode>();
+Var::Var(Id vid, ffi::Optional<StructInfo> struct_info_annotation, Span span) {
+  ObjectPtr<VarNode> n = ffi::make_object<VarNode>();
   n->vid = std::move(vid);
   n->struct_info_ = std::move(struct_info_annotation);
   n->span = std::move(span);
@@ -292,29 +295,26 @@ VarNode* Var::CopyOnWrite() {
   if (!data_.unique()) {
     ObjectPtr<VarNode> node;
     if (auto dataflow_var = as<DataflowVarNode>()) {
-      node = make_object<DataflowVarNode>(*dataflow_var);
+      node = ffi::make_object<DataflowVarNode>(*dataflow_var);
     } else {
-      node = make_object<VarNode>(*(operator->()));
+      node = ffi::make_object<VarNode>(*(operator->()));
     }
     ObjectPtr<Object>(std::move(node)).swap(data_);
   }
   return static_cast<VarNode*>(data_.get());
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.Var")
-    .set_body_typed([](String name_hint, Optional<StructInfo> struct_info_annotation, Span span) {
-      return Var(name_hint, struct_info_annotation, span);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def("relax.Var", [](ffi::String name_hint, ffi::Optional<StructInfo> struct_info_annotation,
+                           Span span) { return Var(name_hint, struct_info_annotation, span); })
+      .def("relax.VarFromId", [](Id vid, ffi::Optional<StructInfo> struct_info_annotation,
+                                 Span span) { return Var(vid, struct_info_annotation, span); });
+}
 
-TVM_FFI_REGISTER_GLOBAL("relax.VarFromId")
-    .set_body_typed([](Id vid, Optional<StructInfo> struct_info_annotation, Span span) {
-      return Var(vid, struct_info_annotation, span);
-    });
-
-TVM_REGISTER_NODE_TYPE(DataflowVarNode);
-
-DataflowVar::DataflowVar(Id vid, Optional<StructInfo> struct_info_annotation, Span span) {
-  ObjectPtr<DataflowVarNode> n = make_object<DataflowVarNode>();
+DataflowVar::DataflowVar(Id vid, ffi::Optional<StructInfo> struct_info_annotation, Span span) {
+  ObjectPtr<DataflowVarNode> n = ffi::make_object<DataflowVarNode>();
   n->vid = std::move(vid);
   n->struct_info_ = std::move(struct_info_annotation);
   n->span = std::move(span);
@@ -322,23 +322,27 @@ DataflowVar::DataflowVar(Id vid, Optional<StructInfo> struct_info_annotation, Sp
   data_ = std::move(n);
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.DataflowVar")
-    .set_body_typed([](String name_hint, Optional<StructInfo> struct_info_annotation, Span span) {
-      return DataflowVar(name_hint, struct_info_annotation, span);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def("relax.DataflowVar",
+           [](ffi::String name_hint, ffi::Optional<StructInfo> struct_info_annotation, Span span) {
+             return DataflowVar(name_hint, struct_info_annotation, span);
+           })
+      .def("relax.DataflowVarFromId",
+           [](Id vid, ffi::Optional<StructInfo> struct_info_annotation, Span span) {
+             return DataflowVar(vid, struct_info_annotation, span);
+           });
+}
 
-TVM_FFI_REGISTER_GLOBAL("relax.DataflowVarFromId")
-    .set_body_typed([](Id vid, Optional<StructInfo> struct_info_annotation, Span span) {
-      return DataflowVar(vid, struct_info_annotation, span);
-    });
-
-Constant::Constant(runtime::NDArray data, Optional<StructInfo> struct_info_annotation, Span span) {
-  ObjectPtr<ConstantNode> n = make_object<ConstantNode>();
+Constant::Constant(runtime::Tensor data, ffi::Optional<StructInfo> struct_info_annotation,
+                   Span span) {
+  ObjectPtr<ConstantNode> n = ffi::make_object<ConstantNode>();
   n->data = std::move(data);
   n->span = std::move(span);
 
   // set struct info.
-  Array<PrimExpr> values;
+  ffi::Array<PrimExpr> values;
   auto shape_tuple = n->data.Shape();
   for (size_t dim = 0; dim < shape_tuple.size(); ++dim) {
     values.push_back(IntImm(DataType::Int(64), shape_tuple[dim]));
@@ -353,17 +357,16 @@ Constant::Constant(runtime::NDArray data, Optional<StructInfo> struct_info_annot
   data_ = std::move(n);
 }
 
-TVM_REGISTER_NODE_TYPE(ConstantNode);
-
-TVM_FFI_REGISTER_GLOBAL("relax.Constant")
-    .set_body_typed([](runtime::NDArray data,
-                       Optional<StructInfo> struct_info_annotation = std::nullopt,
-                       Span span = Span()) {
-      return Constant(data, struct_info_annotation, span);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def(
+      "relax.Constant",
+      [](runtime::Tensor data, ffi::Optional<StructInfo> struct_info_annotation = std::nullopt,
+         Span span = Span()) { return Constant(data, struct_info_annotation, span); });
+}
 
 PrimValue::PrimValue(PrimExpr value, Span span) {
-  ObjectPtr<PrimValueNode> n = make_object<PrimValueNode>();
+  ObjectPtr<PrimValueNode> n = ffi::make_object<PrimValueNode>();
   n->struct_info_ = PrimStructInfo(value);
   n->value = std::move(value);
   n->span = std::move(span);
@@ -374,44 +377,42 @@ PrimValue PrimValue::Int64(int64_t value, Span span) {
   return PrimValue(IntImm(DataType::Int(64), value), span);
 }
 
-TVM_REGISTER_NODE_TYPE(PrimValueNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.PrimValue",
+                        [](PrimExpr value, Span span) { return PrimValue(value, span); });
+}
 
-TVM_FFI_REGISTER_GLOBAL("relax.PrimValue").set_body_typed([](PrimExpr value, Span span) {
-  return PrimValue(value, span);
-});
-
-StringImm::StringImm(String value, Span span) {
-  ObjectPtr<StringImmNode> n = make_object<StringImmNode>();
+StringImm::StringImm(ffi::String value, Span span) {
+  ObjectPtr<StringImmNode> n = ffi::make_object<StringImmNode>();
   n->value = std::move(value);
   n->span = std::move(span);
   n->struct_info_ = ObjectStructInfo();
   data_ = std::move(n);
 }
 
-TVM_REGISTER_NODE_TYPE(StringImmNode);
-
-TVM_FFI_REGISTER_GLOBAL("relax.StringImm").set_body_typed([](String value, Span span) {
-  return StringImm(value, span);
-});
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.StringImm",
+                        [](ffi::String value, Span span) { return StringImm(value, span); });
+}
 
 DataTypeImm::DataTypeImm(DataType value, Span span) {
-  ObjectPtr<DataTypeImmNode> n = make_object<DataTypeImmNode>();
+  ObjectPtr<DataTypeImmNode> n = ffi::make_object<DataTypeImmNode>();
   n->value = std::move(value);
   n->span = std::move(span);
   n->struct_info_ = ObjectStructInfo();
   data_ = std::move(n);
 }
 
-TVM_REGISTER_NODE_TYPE(DataTypeImmNode);
-
-TVM_FFI_REGISTER_GLOBAL("relax.DataTypeImm").set_body_typed([](DataType value, Span span) {
-  return DataTypeImm(value, span);
-});
-
-TVM_REGISTER_NODE_TYPE(MatchCastNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.DataTypeImm",
+                        [](DataType value, Span span) { return DataTypeImm(value, span); });
+}
 
 MatchCast::MatchCast(Var var, Expr value, StructInfo struct_info, Span span) {
-  ObjectPtr<MatchCastNode> n = make_object<MatchCastNode>();
+  ObjectPtr<MatchCastNode> n = ffi::make_object<MatchCastNode>();
   ICHECK(var.defined()) << "MatchCast requires var to be defined";
   n->var = std::move(var);
   n->value = std::move(value);
@@ -420,79 +421,59 @@ MatchCast::MatchCast(Var var, Expr value, StructInfo struct_info, Span span) {
   data_ = std::move(n);
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.MatchCast")
-    .set_body_typed([](Var var, Expr value, StructInfo struct_info, Span span) {
-      return MatchCast(var, value, struct_info, span);
-    });
-
-bool MatchCastNode::SEqualReduce(const MatchCastNode* other, SEqualReducer equal) const {
-  if (value->IsInstance<FunctionNode>()) {
-    // Recursive function definitions may reference the bound variable
-    // within the value being bound.  In these cases, the
-    // `DefEqual(var, other->var)` must occur first, to ensure it is
-    // defined at point of use.
-    return equal.DefEqual(var, other->var) && equal.DefEqual(struct_info, other->struct_info) &&
-           equal(value, other->value);
-  } else {
-    // In all other cases, visit the bound value before the variable
-    // it is bound to, in order to provide better error messages.
-    return equal(value, other->value) && equal.DefEqual(struct_info, other->struct_info) &&
-           equal.DefEqual(var, other->var);
-  }
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.MatchCast",
+                        [](Var var, Expr value, StructInfo struct_info, Span span) {
+                          return MatchCast(var, value, struct_info, span);
+                        });
 }
-void MatchCastNode::SHashReduce(SHashReducer hash_reduce) const {
-  if (value->IsInstance<FunctionNode>()) {
-    hash_reduce.DefHash(var);
-    hash_reduce.DefHash(struct_info);
-    hash_reduce(value);
-  } else {
-    hash_reduce(value);
-    hash_reduce.DefHash(struct_info);
-    hash_reduce.DefHash(var);
-  }
-}
-
-TVM_REGISTER_NODE_TYPE(VarBindingNode);
 
 VarBinding::VarBinding(Var var, Expr value, Span span) {
-  ObjectPtr<VarBindingNode> n = make_object<VarBindingNode>();
+  ObjectPtr<VarBindingNode> n = ffi::make_object<VarBindingNode>();
   n->var = std::move(var);
   n->value = std::move(value);
   n->span = span;
   data_ = std::move(n);
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.VarBinding").set_body_typed([](Var var, Expr value, Span span) {
-  return VarBinding(var, value, span);
-});
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.VarBinding", [](Var var, Expr value, Span span) {
+    return VarBinding(var, value, span);
+  });
+}
 
-bool VarBindingNode::SEqualReduce(const VarBindingNode* other, SEqualReducer equal) const {
+bool VarBindingNode::SEqual(const VarBindingNode* other,
+                            ffi::TypedFunction<bool(AnyView, AnyView, bool, AnyView)> equal) const {
   if (value->IsInstance<FunctionNode>()) {
     // Recursive function definitions may reference the bound variable
     // within the value being bound.  In these cases, the
-    // `DefEqual(var, other->var)` must occur first, to ensure it is
+    // var comparison must occur first to define the var, to ensure it is
     // defined at point of use.
-    return equal.DefEqual(var, other->var) && equal(value, other->value);
+    return equal(var, other->var, true, "var") && equal(value, other->value, false, "value");
   } else {
     // In all other cases, visit the bound value before the variable
     // it is bound to, in order to provide better error messages.
-    return equal(value, other->value) && equal.DefEqual(var, other->var);
+    return equal(value, other->value, false, "value") && equal(var, other->var, true, "var");
   }
 }
-void VarBindingNode::SHashReduce(SHashReducer hash_reduce) const {
+
+uint64_t VarBindingNode::SHash(uint64_t init_hash,
+                               ffi::TypedFunction<uint64_t(AnyView, uint64_t, bool)> hash) const {
+  uint64_t hash_value = init_hash;
   if (value->IsInstance<FunctionNode>()) {
-    hash_reduce.DefHash(var);
-    hash_reduce(value);
+    hash_value = hash(var, hash_value, true);
+    hash_value = hash(value, hash_value, false);
   } else {
-    hash_reduce(value);
-    hash_reduce.DefHash(var);
+    hash_value = hash(value, hash_value, false);
+    hash_value = hash(var, hash_value, true);
   }
+  return hash_value;
 }
 
-TVM_REGISTER_NODE_TYPE(BindingBlockNode);
-
-BindingBlock::BindingBlock(Array<Binding> bindings, Span span) {
-  ObjectPtr<BindingBlockNode> n = make_object<BindingBlockNode>();
+BindingBlock::BindingBlock(ffi::Array<Binding> bindings, Span span) {
+  ObjectPtr<BindingBlockNode> n = ffi::make_object<BindingBlockNode>();
   n->bindings = std::move(bindings);
   n->span = span;
   data_ = std::move(n);
@@ -508,61 +489,61 @@ BindingBlockNode* BindingBlock::CopyOnWrite() {
   if (!data_.unique()) {
     ObjectPtr<BindingBlockNode> node;
     if (auto dataflow_block = as<DataflowBlockNode>()) {
-      node = make_object<DataflowBlockNode>(*dataflow_block);
+      node = ffi::make_object<DataflowBlockNode>(*dataflow_block);
     } else {
-      node = make_object<BindingBlockNode>(*(operator->()));
+      node = ffi::make_object<BindingBlockNode>(*(operator->()));
     }
     ObjectPtr<Object>(std::move(node)).swap(data_);
   }
   return static_cast<BindingBlockNode*>(data_.get());
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.BindingBlock")
-    .set_body_typed([](Array<Binding> bindings, Span span) {
-      return BindingBlock(bindings, span);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.BindingBlock", [](ffi::Array<Binding> bindings, Span span) {
+    return BindingBlock(bindings, span);
+  });
+}
 
-TVM_REGISTER_NODE_TYPE(DataflowBlockNode);
-
-DataflowBlock::DataflowBlock(Array<Binding> bindings, Span span) {
-  ObjectPtr<DataflowBlockNode> n = make_object<DataflowBlockNode>();
+DataflowBlock::DataflowBlock(ffi::Array<Binding> bindings, Span span) {
+  ObjectPtr<DataflowBlockNode> n = ffi::make_object<DataflowBlockNode>();
   n->bindings = std::move(bindings);
   n->span = span;
   data_ = std::move(n);
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.DataflowBlock")
-    .set_body_typed([](Array<Binding> bindings, Span span) {
-      return DataflowBlock(bindings, span);
-    });
-
-TVM_REGISTER_NODE_TYPE(SeqExprNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.DataflowBlock", [](ffi::Array<Binding> bindings, Span span) {
+    return DataflowBlock(bindings, span);
+  });
+}
 
 SeqExpr::SeqExpr(Expr body) {
   if (auto seq = body.as<SeqExpr>()) {
     *this = seq.value();
   } else {
-    *this = SeqExpr(Array<BindingBlock>{}, body);
+    *this = SeqExpr(ffi::Array<BindingBlock>{}, body);
   }
 }
 
-SeqExpr::SeqExpr(Array<BindingBlock> blocks, Expr body, Span span) {
-  ObjectPtr<SeqExprNode> n = make_object<SeqExprNode>();
+SeqExpr::SeqExpr(ffi::Array<BindingBlock> blocks, Expr body, Span span) {
+  ObjectPtr<SeqExprNode> n = ffi::make_object<SeqExprNode>();
   n->blocks = std::move(blocks);
   n->body = std::move(body);
   n->span = span;
   data_ = std::move(n);
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.SeqExpr")
-    .set_body_typed([](Array<BindingBlock> blocks, Expr body, Span span) {
-      return SeqExpr(blocks, body, span);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.SeqExpr", [](ffi::Array<BindingBlock> blocks, Expr body, Span span) {
+    return SeqExpr(blocks, body, span);
+  });
+}
 
-TVM_REGISTER_NODE_TYPE(FunctionNode);
-
-Function::Function(Array<Var> params, Expr body, Optional<StructInfo> ret_struct_info, bool is_pure,
-                   DictAttrs attrs, Span span) {
+Function::Function(ffi::Array<Var> params, Expr body, ffi::Optional<StructInfo> ret_struct_info,
+                   bool is_pure, DictAttrs attrs, Span span) {
   if (!attrs.defined()) {
     attrs = DictAttrs();
   }
@@ -570,7 +551,7 @@ Function::Function(Array<Var> params, Expr body, Optional<StructInfo> ret_struct
   // Set the function type.
   // For function, we take a conservative approach and require the function type
   // to be known at construction time.
-  Array<StructInfo> param_sinfo;
+  ffi::Array<StructInfo> param_sinfo;
 
   for (const Var& param : params) {
     CHECK(param->struct_info_.defined())
@@ -578,7 +559,7 @@ Function::Function(Array<Var> params, Expr body, Optional<StructInfo> ret_struct
     param_sinfo.push_back(GetStructInfo(param));
   }
 
-  Optional<StructInfo> body_sinfo;
+  ffi::Optional<StructInfo> body_sinfo;
 
   if (body->struct_info_.defined()) {
     body_sinfo = GetStructInfo(body);
@@ -604,7 +585,7 @@ Function::Function(Array<Var> params, Expr body, Optional<StructInfo> ret_struct
     auto f_shape_var_map = [&] {
       auto tir_vars = DefinableTIRVarsInStructInfo(TupleStructInfo(params.Map(GetStructInfo)));
       std::unordered_set<tir::Var> lookup(tir_vars.begin(), tir_vars.end());
-      return [lookup = std::move(lookup)](const tir::Var& var) -> Optional<PrimExpr> {
+      return [lookup = std::move(lookup)](const tir::Var& var) -> ffi::Optional<PrimExpr> {
         if (lookup.count(var)) {
           return var;
         } else {
@@ -618,10 +599,10 @@ Function::Function(Array<Var> params, Expr body, Optional<StructInfo> ret_struct
   FuncStructInfo func_sinfo(param_sinfo, ret_struct_info.value(), is_pure);
 
   // set the fields
-  ObjectPtr<FunctionNode> n = make_object<FunctionNode>();
+  ObjectPtr<FunctionNode> n = ffi::make_object<FunctionNode>();
   n->params = std::move(params);
   n->body = std::move(body);
-  n->ret_struct_info = std::move(ret_struct_info.value());
+  n->ret_struct_info = ret_struct_info.value();
   n->is_pure = is_pure;
   n->struct_info_ = std::move(func_sinfo);
   n->attrs = std::move(attrs);
@@ -629,15 +610,18 @@ Function::Function(Array<Var> params, Expr body, Optional<StructInfo> ret_struct
   data_ = std::move(n);
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.Function")
-    .set_body_typed([](Array<Var> params, Expr body, Optional<StructInfo> ret_struct_info,
-                       bool is_pure, DictAttrs attrs, Span span) {
-      return Function(params, body, ret_struct_info, is_pure, attrs, span);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.Function", [](ffi::Array<Var> params, Expr body,
+                                             ffi::Optional<StructInfo> ret_struct_info,
+                                             bool is_pure, DictAttrs attrs, Span span) {
+    return Function(params, body, ret_struct_info, is_pure, attrs, span);
+  });
+}
 
-Function Function::CreateEmpty(Array<Var> params, StructInfo ret_struct_info, bool is_pure,
+Function Function::CreateEmpty(ffi::Array<Var> params, StructInfo ret_struct_info, bool is_pure,
                                DictAttrs attrs, Span span) {
-  Array<StructInfo> param_sinfo;
+  ffi::Array<StructInfo> param_sinfo;
   for (const Var& param : params) {
     ICHECK(param->struct_info_.defined())
         << "relax.Function requires params to contain struct_info_.";
@@ -655,7 +639,7 @@ Function Function::CreateEmpty(Array<Var> params, StructInfo ret_struct_info, bo
   }();
 
   // set the fields
-  ObjectPtr<FunctionNode> n = make_object<FunctionNode>();
+  ObjectPtr<FunctionNode> n = ffi::make_object<FunctionNode>();
   n->params = std::move(params);
   n->body = std::move(body);
   n->is_pure = is_pure;
@@ -666,25 +650,32 @@ Function Function::CreateEmpty(Array<Var> params, StructInfo ret_struct_info, bo
   return Function(std::move(n));
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.FunctionCreateEmpty")
-    .set_body_typed([](Array<Var> params, StructInfo ret_struct_info, bool is_pure, DictAttrs attrs,
-                       Span span) {
-      return Function::CreateEmpty(params, ret_struct_info, is_pure, attrs, span);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def(
+      "relax.FunctionCreateEmpty", [](ffi::Array<Var> params, StructInfo ret_struct_info,
+                                      bool is_pure, DictAttrs attrs, Span span) {
+        return Function::CreateEmpty(params, ret_struct_info, is_pure, attrs, span);
+      });
+}
 
 // Special opaque derivation function for ExternFunc
 // Take look at sinfo_args to figure out the return StructInfo.
-TVM_FFI_REGISTER_GLOBAL("tvm.relax.struct_info.infer_by_sinfo_args")
-    .set_body_typed([](const Call& call, const BlockBuilder& ctx) -> StructInfo {
-      ICHECK(call->sinfo_args.defined()) << "sinfo_args field of CallNode should always be defined";
-      if (call->sinfo_args.empty()) {
-        return ObjectStructInfo();
-      } else if (call->sinfo_args.size() == 1) {
-        return call->sinfo_args[0];
-      } else {
-        return TupleStructInfo(call->sinfo_args);
-      }
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tvm.relax.struct_info.infer_by_sinfo_args",
+                        [](const Call& call, const BlockBuilder& ctx) -> StructInfo {
+                          ICHECK(call->sinfo_args.defined())
+                              << "sinfo_args field of CallNode should always be defined";
+                          if (call->sinfo_args.empty()) {
+                            return ObjectStructInfo();
+                          } else if (call->sinfo_args.size() == 1) {
+                            return call->sinfo_args[0];
+                          } else {
+                            return TupleStructInfo(call->sinfo_args);
+                          }
+                        });
+}
 
 // Get the derive function.
 FuncStructInfo GetExternFuncStructInfo() {
@@ -694,31 +685,32 @@ FuncStructInfo GetExternFuncStructInfo() {
   return FuncStructInfo::OpaqueFunc(derive);
 }
 
-TVM_REGISTER_NODE_TYPE(ExternFuncNode);
-
-ExternFunc::ExternFunc(String global_symbol, Span span)
+ExternFunc::ExternFunc(ffi::String global_symbol, Span span)
     : ExternFunc(global_symbol, GetExternFuncStructInfo(), span) {}
 
-ExternFunc::ExternFunc(String global_symbol, StructInfo struct_info, Span span) {
+ExternFunc::ExternFunc(ffi::String global_symbol, StructInfo struct_info, Span span) {
   CHECK(struct_info.as<FuncStructInfoNode>())
       << "ExternFunc must have FuncStructInfo, "
       << "but declaration of '" << global_symbol << "' received " << struct_info;
 
-  ObjectPtr<ExternFuncNode> n = make_object<ExternFuncNode>();
+  ObjectPtr<ExternFuncNode> n = ffi::make_object<ExternFuncNode>();
   n->global_symbol = std::move(global_symbol);
   n->span = span;
   n->struct_info_ = struct_info;
   data_ = std::move(n);
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.ExternFunc")
-    .set_body_typed([](String global_symbol, Optional<StructInfo> struct_info, Span span) {
-      if (struct_info.defined()) {
-        return ExternFunc(global_symbol, struct_info.value(), span);
-      } else {
-        return ExternFunc(global_symbol, span);
-      }
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.ExternFunc", [](ffi::String global_symbol,
+                                               ffi::Optional<StructInfo> struct_info, Span span) {
+    if (struct_info.defined()) {
+      return ExternFunc(global_symbol, struct_info.value(), span);
+    } else {
+      return ExternFunc(global_symbol, span);
+    }
+  });
+}
 
 Expr GetShapeOf(const Expr& expr) {
   // default case, to be normalized.
@@ -735,33 +727,31 @@ Expr GetShapeOf(const Expr& expr) {
   return call_shape_of;
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.GetShapeOf").set_body_typed([](const Expr& expr) {
-  return GetShapeOf(expr);
-});
-
-TVM_FFI_REGISTER_GLOBAL("relax.FuncWithAttr")
-    .set_body_typed([](BaseFunc func, String key, ObjectRef value) -> Optional<Function> {
-      if (func->IsInstance<relax::FunctionNode>()) {
-        return WithAttr(Downcast<relax::Function>(std::move(func)), key, value);
-      }
-      return std::nullopt;
-    });
-
-TVM_FFI_REGISTER_GLOBAL("relax.FuncWithAttrs")
-    .set_body_typed([](BaseFunc func, Map<String, ffi::Any> attr_map) -> Optional<Function> {
-      if (func->IsInstance<relax::FunctionNode>()) {
-        return WithAttrs(Downcast<relax::Function>(std::move(func)), attr_map);
-      }
-      return std::nullopt;
-    });
-
-TVM_FFI_REGISTER_GLOBAL("relax.FuncWithoutAttr")
-    .set_body_typed([](BaseFunc func, String key) -> Optional<Function> {
-      if (func->IsInstance<relax::FunctionNode>()) {
-        return WithoutAttr(Downcast<relax::Function>(std::move(func)), key);
-      }
-      return std::nullopt;
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def("relax.GetShapeOf", [](const Expr& expr) { return GetShapeOf(expr); })
+      .def("relax.FuncWithAttr",
+           [](BaseFunc func, ffi::String key, ObjectRef value) -> ffi::Optional<Function> {
+             if (func->IsInstance<relax::FunctionNode>()) {
+               return WithAttr(Downcast<relax::Function>(std::move(func)), key, value);
+             }
+             return std::nullopt;
+           })
+      .def("relax.FuncWithAttrs",
+           [](BaseFunc func, ffi::Map<ffi::String, ffi::Any> attr_map) -> ffi::Optional<Function> {
+             if (func->IsInstance<relax::FunctionNode>()) {
+               return WithAttrs(Downcast<relax::Function>(std::move(func)), attr_map);
+             }
+             return std::nullopt;
+           })
+      .def("relax.FuncWithoutAttr", [](BaseFunc func, ffi::String key) -> ffi::Optional<Function> {
+        if (func->IsInstance<relax::FunctionNode>()) {
+          return WithoutAttr(Downcast<relax::Function>(std::move(func)), key);
+        }
+        return std::nullopt;
+      });
+}
 
 }  // namespace relax
 }  // namespace tvm
