@@ -21,7 +21,7 @@
  * \file src/relax/py_expr_functor.cc
  * \brief The backbone of PyExprVisitor/PyExprMutator.
  */
-#include <tvm/ffi/reflection/reflection.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/relax/expr_functor.h>
 
 namespace tvm {
@@ -110,28 +110,29 @@ class PyExprVisitorNode : public Object, public ExprVisitor {
       PY_EXPR_VISITOR_DEFAULT(binding, f_visit_binding, ExprVisitor::VisitBinding(binding));
 
   void VisitBinding_(const VarBindingNode* binding)
-      PY_EXPR_VISITOR_DEFAULT(GetRef<VarBinding>(binding), f_visit_var_binding_,
+      PY_EXPR_VISITOR_DEFAULT(ffi::GetRef<VarBinding>(binding), f_visit_var_binding_,
                               ExprVisitor::VisitBinding_(binding));
   void VisitBinding_(const MatchCastNode* binding)
-      PY_EXPR_VISITOR_DEFAULT(GetRef<MatchCast>(binding), f_visit_match_cast_,
+      PY_EXPR_VISITOR_DEFAULT(ffi::GetRef<MatchCast>(binding), f_visit_match_cast_,
                               ExprVisitor::VisitBinding_(binding));
 
   void VisitBindingBlock(const BindingBlock& block)
       PY_EXPR_VISITOR_DEFAULT(block, f_visit_binding_block, ExprVisitor::VisitBindingBlock(block));
 
   void VisitBindingBlock_(const BindingBlockNode* block)
-      PY_EXPR_VISITOR_DEFAULT(GetRef<BindingBlock>(block), f_visit_binding_block_,
+      PY_EXPR_VISITOR_DEFAULT(ffi::GetRef<BindingBlock>(block), f_visit_binding_block_,
                               ExprVisitor::VisitBindingBlock_(block));
   void VisitBindingBlock_(const DataflowBlockNode* block)
-      PY_EXPR_VISITOR_DEFAULT(GetRef<DataflowBlock>(block), f_visit_dataflow_block_,
+      PY_EXPR_VISITOR_DEFAULT(ffi::GetRef<DataflowBlock>(block), f_visit_dataflow_block_,
                               ExprVisitor::VisitBindingBlock_(block));
 
   void VisitVarDef(const Var& var)
       PY_EXPR_VISITOR_DEFAULT(var, f_visit_var_def, ExprVisitor::VisitVarDef(var));
   void VisitVarDef_(const VarNode* var)
-      PY_EXPR_VISITOR_DEFAULT(GetRef<Var>(var), f_visit_var_def_, ExprVisitor::VisitVarDef_(var));
+      PY_EXPR_VISITOR_DEFAULT(ffi::GetRef<Var>(var), f_visit_var_def_,
+                              ExprVisitor::VisitVarDef_(var));
   void VisitVarDef_(const DataflowVarNode* var)
-      PY_EXPR_VISITOR_DEFAULT(GetRef<DataflowVar>(var), f_visit_dataflow_var_def_,
+      PY_EXPR_VISITOR_DEFAULT(ffi::GetRef<DataflowVar>(var), f_visit_dataflow_var_def_,
                               ExprVisitor::VisitVarDef_(var));
 
   void VisitSpan(const Span& span)
@@ -141,8 +142,8 @@ class PyExprVisitorNode : public Object, public ExprVisitor {
     // PyExprVisitorNode has no fields to register
   }
 
-  static constexpr const char* _type_key = "expr_functor.PyExprVisitor";
-  TVM_DECLARE_BASE_OBJECT_INFO(PyExprVisitorNode, Object);
+  static constexpr const bool _type_mutable = true;
+  TVM_FFI_DECLARE_OBJECT_INFO("expr_functor.PyExprVisitor", PyExprVisitorNode, Object);
 
  private:
   // initialize the vtable.
@@ -170,14 +171,15 @@ class PyExprVisitorNode : public Object, public ExprVisitor {
   }
 };
 
-TVM_REGISTER_NODE_TYPE(PyExprVisitorNode);
-
 /*!
  * \brief Managed reference to PyExprVisitorNode.
  * \sa PyExprVisitorNode
  */
 class PyExprVisitor : public ObjectRef {
  public:
+  explicit PyExprVisitor(ObjectPtr<PyExprVisitorNode> data) : ObjectRef(data) {
+    TVM_FFI_ICHECK(data != nullptr);
+  }
   /*!
    * \brief Create a PyExprVisitor with customized methods on the python-side.
    * \param f_visit_expr The packed function of `VisitExpr(const Expr& expr)`.
@@ -229,7 +231,7 @@ class PyExprVisitor : public ObjectRef {
       ffi::Function f_visit_dataflow_block_, ffi::Function f_visit_var_def,
       ffi::Function f_visit_var_def_, ffi::Function f_visit_dataflow_var_def_,
       ffi::Function f_visit_span) {
-    ObjectPtr<PyExprVisitorNode> n = make_object<PyExprVisitorNode>();
+    ObjectPtr<PyExprVisitorNode> n = ffi::make_object<PyExprVisitorNode>();
     n->f_visit_expr = f_visit_expr;
     n->f_visit_binding = f_visit_binding;
     n->f_visit_binding_block = f_visit_binding_block;
@@ -260,7 +262,7 @@ class PyExprVisitor : public ObjectRef {
     return PyExprVisitor(n);
   }
 
-  TVM_DEFINE_MUTABLE_NOTNULLABLE_OBJECT_REF_METHODS(PyExprVisitor, ObjectRef, PyExprVisitorNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(PyExprVisitor, ObjectRef, PyExprVisitorNode);
 };
 
 /*!
@@ -350,14 +352,14 @@ class PyExprMutatorNode : public Object, public ExprMutator {
 
   void VisitBinding_(const VarBindingNode* binding) {
     if (f_visit_var_binding_ != nullptr)
-      f_visit_var_binding_(GetRef<VarBinding>(binding));
+      f_visit_var_binding_(ffi::GetRef<VarBinding>(binding));
     else
       ExprMutator::VisitBinding_(binding);
   }
 
   void VisitBinding_(const MatchCastNode* binding) {
     if (f_visit_match_cast_ != nullptr)
-      f_visit_match_cast_(GetRef<MatchCast>(binding));
+      f_visit_match_cast_(ffi::GetRef<MatchCast>(binding));
     else
       ExprMutator::VisitBinding_(binding);
   }
@@ -367,18 +369,19 @@ class PyExprMutatorNode : public Object, public ExprMutator {
                               BindingBlock);
 
   BindingBlock VisitBindingBlock_(const BindingBlockNode* block)
-      PY_EXPR_MUTATOR_DEFAULT(GetRef<BindingBlock>(block), f_visit_binding_block_,
+      PY_EXPR_MUTATOR_DEFAULT(ffi::GetRef<BindingBlock>(block), f_visit_binding_block_,
                               ExprMutator::VisitBindingBlock_(block), BindingBlock);
   BindingBlock VisitBindingBlock_(const DataflowBlockNode* block)
-      PY_EXPR_MUTATOR_DEFAULT(GetRef<DataflowBlock>(block), f_visit_dataflow_block_,
+      PY_EXPR_MUTATOR_DEFAULT(ffi::GetRef<DataflowBlock>(block), f_visit_dataflow_block_,
                               ExprMutator::VisitBindingBlock_(block), BindingBlock);
 
   Var VisitVarDef(const Var& var)
       PY_EXPR_MUTATOR_DEFAULT(var, f_visit_var_def, ExprMutator::VisitVarDef(var), Var);
-  Var VisitVarDef_(const VarNode* var) PY_EXPR_MUTATOR_DEFAULT(GetRef<Var>(var), f_visit_var_def_,
-                                                               ExprMutator::VisitVarDef_(var), Var);
+  Var VisitVarDef_(const VarNode* var)
+      PY_EXPR_MUTATOR_DEFAULT(ffi::GetRef<Var>(var), f_visit_var_def_,
+                              ExprMutator::VisitVarDef_(var), Var);
   Var VisitVarDef_(const DataflowVarNode* var)
-      PY_EXPR_MUTATOR_DEFAULT(GetRef<DataflowVar>(var), f_visit_dataflow_var_def_,
+      PY_EXPR_MUTATOR_DEFAULT(ffi::GetRef<DataflowVar>(var), f_visit_dataflow_var_def_,
                               ExprMutator::VisitVarDef_(var), Var);
 
   /*!
@@ -402,8 +405,8 @@ class PyExprMutatorNode : public Object, public ExprMutator {
     refl::ObjectDef<PyExprMutatorNode>().def_ro("builder_", &PyExprMutatorNode::builder_);
   }
 
-  static constexpr const char* _type_key = "expr_functor.PyExprMutator";
-  TVM_DECLARE_BASE_OBJECT_INFO(PyExprMutatorNode, Object);
+  static constexpr const bool _type_mutable = true;
+  TVM_FFI_DECLARE_OBJECT_INFO("expr_functor.PyExprMutator", PyExprMutatorNode, Object);
 
  private:
   // initialize the vtable.
@@ -455,14 +458,15 @@ class PyExprMutatorNode : public Object, public ExprMutator {
   }
 };
 
-TVM_REGISTER_NODE_TYPE(PyExprMutatorNode);
-
 /*!
  * \brief Managed reference to PyExprMutatorNode.
  * \sa PyExprMutatorNode
  */
 class PyExprMutator : public ObjectRef {
  public:
+  explicit PyExprMutator(ObjectPtr<PyExprMutatorNode> data) : ObjectRef(data) {
+    TVM_FFI_ICHECK(data != nullptr);
+  }
   /*!
    * \brief Create a PyExprMutator with customized methods on the python-side.
    * \param f_visit_expr The packed function of `VisitExpr(const Expr& expr)`.
@@ -514,7 +518,7 @@ class PyExprMutator : public ObjectRef {
       ffi::Function f_visit_dataflow_block_, ffi::Function f_visit_var_def,
       ffi::Function f_visit_var_def_, ffi::Function f_visit_dataflow_var_def_,
       ffi::Function f_visit_span) {
-    ObjectPtr<PyExprMutatorNode> n = make_object<PyExprMutatorNode>();
+    ObjectPtr<PyExprMutatorNode> n = ffi::make_object<PyExprMutatorNode>();
     n->builder_ = builder_;
     n->f_visit_expr = f_visit_expr;
     n->f_visit_constant_ = f_visit_constant_;
@@ -545,162 +549,124 @@ class PyExprMutator : public ObjectRef {
     n->f_visit_span = f_visit_span;
     return PyExprMutator(n);
   }
-  TVM_DEFINE_MUTABLE_NOTNULLABLE_OBJECT_REF_METHODS(PyExprMutator, ObjectRef, PyExprMutatorNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(PyExprMutator, ObjectRef, PyExprMutatorNode);
 };
 
-TVM_FFI_REGISTER_GLOBAL("relax.MakePyExprVisitor").set_body_typed(PyExprVisitor::MakePyExprVisitor);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::ObjectDef<PyExprVisitorNode>();
+  refl::GlobalDef()
+      .def("relax.MakePyExprVisitor", PyExprVisitor::MakePyExprVisitor)
+      .def("relax.PyExprVisitorVisitExpr",
+           [](PyExprVisitor visitor, const Expr& expr) { visitor->VisitExpr(expr); })
+      .def("relax.PyExprVisitorVisitBinding",
+           [](PyExprVisitor visitor, const Binding& binding) { visitor->VisitBinding(binding); })
+      .def("relax.PyExprVisitorVisitBindingBlock",
+           [](PyExprVisitor visitor, const BindingBlock& block) {
+             visitor->VisitBindingBlock(block);
+           })
+      .def("relax.PyExprVisitorVisitVarDef",
+           [](PyExprVisitor visitor, const Var& var) { visitor->VisitVarDef(var); })
+      .def("relax.ExprVisitorVisitExpr",
+           [](PyExprVisitor visitor, const Expr& expr) { visitor->ExprVisitor::VisitExpr(expr); })
+      .def("relax.ExprVisitorVisitBinding",
+           [](PyExprVisitor visitor, const Binding& binding) {
+             if (const auto* ptr = binding.as<VarBindingNode>()) {
+               visitor->ExprVisitor::VisitBinding_(ptr);
+             } else if (const auto* ptr = binding.as<MatchCastNode>()) {
+               visitor->ExprVisitor::VisitBinding_(ptr);
+             } else {
+               LOG(FATAL) << "unreachable";
+             }
+           })
+      .def("relax.ExprVisitorVisitBindingBlock",
+           [](PyExprVisitor visitor, const BindingBlock& block) {
+             if (const auto* ptr = block.as<DataflowBlockNode>()) {
+               visitor->ExprVisitor::VisitBindingBlock_(ptr);
+             } else if (const auto* ptr = block.as<BindingBlockNode>()) {
+               visitor->ExprVisitor::VisitBindingBlock_(ptr);
+             } else {
+               LOG(FATAL) << "TypeError: Invalid type: " << block->GetTypeKey();
+             }
+           })
+      .def("relax.ExprVisitorVisitVarDef",
+           [](PyExprVisitor visitor, const Var& var) {
+             if (const auto* node = var.as<DataflowVarNode>()) {
+               visitor->ExprVisitor::VisitVarDef_(node);
+             } else if (const auto* node = var.as<VarNode>()) {
+               visitor->ExprVisitor::VisitVarDef_(node);
+             } else {
+               LOG(FATAL) << "TypeError: Invalid type: " << var->GetTypeKey();
+             }
+           })
+      .def("relax.ExprVisitorVisitSpan",
+           [](PyExprVisitor visitor, const Span& span) { visitor->ExprVisitor::VisitSpan(span); })
+      .def("relax.MakePyExprMutator", PyExprMutator::MakePyExprMutator)
+      .def("relax.PyExprMutatorVisitExpr",
+           [](PyExprMutator mutator, const Expr& expr) { return mutator->VisitExpr(expr); })
+      .def("relax.PyExprMutatorVisitBinding",
+           [](PyExprMutator mutator, const Binding& binding) { mutator->VisitBinding(binding); })
+      .def("relax.PyExprMutatorVisitBindingBlock",
+           [](PyExprMutator mutator, const BindingBlock& block) {
+             return mutator->VisitBindingBlock(block);
+           })
+      .def("relax.PyExprMutatorVisitVarDef",
+           [](PyExprMutator mutator, const Var& var) { return mutator->VisitVarDef(var); })
+      .def("relax.ExprMutatorVisitExpr",
+           [](PyExprMutator mutator, const Expr& expr) {
+             return mutator->ExprMutator::VisitExpr(expr);
+           })
+      .def("relax.ExprMutatorVisitBinding",
+           [](PyExprMutator mutator, const Binding& binding) {
+             if (const auto* ptr = binding.as<VarBindingNode>()) {
+               return mutator->ExprMutator::VisitBinding_(ptr);
+             } else if (const auto* ptr = binding.as<MatchCastNode>()) {
+               return mutator->ExprMutator::VisitBinding_(ptr);
+             } else {
+               LOG(FATAL) << "unreachable";
+             }
+           })
+      .def("relax.ExprMutatorVisitBindingBlock",
+           [](PyExprMutator mutator, const BindingBlock& block) {
+             if (const auto* node = block.as<DataflowBlockNode>()) {
+               return mutator->ExprMutator::VisitBindingBlock_(node);
+             } else if (const auto* node = block.as<BindingBlockNode>()) {
+               return mutator->ExprMutator::VisitBindingBlock_(node);
+             } else {
+               LOG(FATAL) << "TypeError: Invalid type: " << block->GetTypeKey();
+             }
+           })
+      .def("relax.ExprMutatorVisitVarDef",
+           [](PyExprMutator mutator, const Var& var) {
+             if (const auto* node = var.as<DataflowVarNode>()) {
+               return mutator->ExprMutator::VisitVarDef_(node);
+             } else if (const auto* node = var.as<VarNode>()) {
+               return mutator->ExprMutator::VisitVarDef_(node);
+             } else {
+               LOG(FATAL) << "TypeError: Invalid type: " << var->GetTypeKey();
+             }
+           })
+      .def(
+          "relax.PyExprMutatorVisitExprPostOrder",
+          [](PyExprMutator mutator, const Expr& expr) { return mutator->VisitExprPostOrder(expr); })
+      .def("relax.PyExprMutatorVisitWithNewScope",
+           [](PyExprMutator mutator, const Expr& expr) { return mutator->VisitWithNewScope(expr); })
+      .def("relax.PyExprMutatorLookupBinding",
+           [](PyExprMutator mutator, const Var& var) { return mutator->LookupBinding(var); })
+      .def("relax.PyExprMutatorWithStructInfo",
+           [](PyExprMutator mutator, Var var, StructInfo sinfo) {
+             return mutator->WithStructInfo(var, sinfo);
+           })
+      .def("relax.PyExprMutatorSetVarRemap",
+           [](PyExprMutator mutator, Id id, Var var) { return mutator->var_remap_[id] = var; })
+      .def("relax.PyExprMutatorGetVarRemap",
+           [](PyExprMutator mutator, Id id) { return mutator->var_remap_[id]; });
+}
 
-TVM_FFI_REGISTER_GLOBAL("relax.PyExprVisitorVisitExpr")
-    .set_body_typed([](PyExprVisitor visitor, const Expr& expr) { visitor->VisitExpr(expr); });
-
-TVM_FFI_REGISTER_GLOBAL("relax.PyExprVisitorVisitBinding")
-    .set_body_typed([](PyExprVisitor visitor, const Binding& binding) {
-      visitor->VisitBinding(binding);
-    });
-
-TVM_FFI_REGISTER_GLOBAL("relax.PyExprVisitorVisitBindingBlock")
-    .set_body_typed([](PyExprVisitor visitor, const BindingBlock& block) {
-      visitor->VisitBindingBlock(block);
-    });
-
-TVM_FFI_REGISTER_GLOBAL("relax.PyExprVisitorVisitVarDef")
-    .set_body_typed([](PyExprVisitor visitor, const Var& var) { visitor->VisitVarDef(var); });
-
-TVM_FFI_REGISTER_GLOBAL("relax.ExprVisitorVisitExpr")
-    .set_body_typed([](PyExprVisitor visitor, const Expr& expr) {
-      visitor->ExprVisitor::VisitExpr(expr);
-    });
-
-TVM_FFI_REGISTER_GLOBAL("relax.ExprVisitorVisitBinding")
-    .set_body_typed([](PyExprVisitor visitor, const Binding& binding) {
-      if (const auto* ptr = binding.as<VarBindingNode>()) {
-        visitor->ExprVisitor::VisitBinding_(ptr);
-      } else if (const auto* ptr = binding.as<MatchCastNode>()) {
-        visitor->ExprVisitor::VisitBinding_(ptr);
-      } else {
-        LOG(FATAL) << "unreachable";
-      }
-    });
-
-TVM_FFI_REGISTER_GLOBAL("relax.ExprVisitorVisitBindingBlock")
-    .set_body_typed([](PyExprVisitor visitor, const BindingBlock& block) {
-      if (const auto* ptr = block.as<DataflowBlockNode>()) {
-        visitor->ExprVisitor::VisitBindingBlock_(ptr);
-      } else if (const auto* ptr = block.as<BindingBlockNode>()) {
-        visitor->ExprVisitor::VisitBindingBlock_(ptr);
-      } else {
-        LOG(FATAL) << "TypeError: Invalid type: " << block->GetTypeKey();
-      }
-    });
-
-TVM_FFI_REGISTER_GLOBAL("relax.ExprVisitorVisitVarDef")
-    .set_body_typed([](PyExprVisitor visitor, const Var& var) {
-      if (const auto* node = var.as<DataflowVarNode>()) {
-        visitor->ExprVisitor::VisitVarDef_(node);
-      } else if (const auto* node = var.as<VarNode>()) {
-        visitor->ExprVisitor::VisitVarDef_(node);
-      } else {
-        LOG(FATAL) << "TypeError: Invalid type: " << var->GetTypeKey();
-      }
-    });
-
-TVM_FFI_REGISTER_GLOBAL("relax.ExprVisitorVisitSpan")
-    .set_body_typed([](PyExprVisitor visitor, const Span& span) {
-      visitor->ExprVisitor::VisitSpan(span);
-    });
-
-TVM_FFI_REGISTER_GLOBAL("relax.MakePyExprMutator").set_body_typed(PyExprMutator::MakePyExprMutator);
-
-TVM_FFI_REGISTER_GLOBAL("relax.PyExprMutatorVisitExpr")
-    .set_body_typed([](PyExprMutator mutator, const Expr& expr) {
-      return mutator->VisitExpr(expr);
-    });
-
-TVM_FFI_REGISTER_GLOBAL("relax.PyExprMutatorVisitBinding")
-    .set_body_typed([](PyExprMutator mutator, const Binding& binding) {
-      mutator->VisitBinding(binding);
-    });
-
-TVM_FFI_REGISTER_GLOBAL("relax.PyExprMutatorVisitBindingBlock")
-    .set_body_typed([](PyExprMutator mutator, const BindingBlock& block) {
-      return mutator->VisitBindingBlock(block);
-    });
-
-TVM_FFI_REGISTER_GLOBAL("relax.PyExprMutatorVisitVarDef")
-    .set_body_typed([](PyExprMutator mutator, const Var& var) {
-      return mutator->VisitVarDef(var);
-    });
-
-TVM_FFI_REGISTER_GLOBAL("relax.ExprMutatorVisitExpr")
-    .set_body_typed([](PyExprMutator mutator, const Expr& expr) {
-      return mutator->ExprMutator::VisitExpr(expr);
-    });
-
-TVM_FFI_REGISTER_GLOBAL("relax.ExprMutatorVisitBinding")
-    .set_body_typed([](PyExprMutator mutator, const Binding& binding) {
-      if (const auto* ptr = binding.as<VarBindingNode>()) {
-        return mutator->ExprMutator::VisitBinding_(ptr);
-      } else if (const auto* ptr = binding.as<MatchCastNode>()) {
-        return mutator->ExprMutator::VisitBinding_(ptr);
-      } else {
-        LOG(FATAL) << "unreachable";
-      }
-    });
-
-TVM_FFI_REGISTER_GLOBAL("relax.ExprMutatorVisitBindingBlock")
-    .set_body_typed([](PyExprMutator mutator, const BindingBlock& block) {
-      if (const auto* node = block.as<DataflowBlockNode>()) {
-        return mutator->ExprMutator::VisitBindingBlock_(node);
-      } else if (const auto* node = block.as<BindingBlockNode>()) {
-        return mutator->ExprMutator::VisitBindingBlock_(node);
-      } else {
-        LOG(FATAL) << "TypeError: Invalid type: " << block->GetTypeKey();
-      }
-    });
-
-TVM_FFI_REGISTER_GLOBAL("relax.ExprMutatorVisitVarDef")
-    .set_body_typed([](PyExprMutator mutator, const Var& var) {
-      if (const auto* node = var.as<DataflowVarNode>()) {
-        return mutator->ExprMutator::VisitVarDef_(node);
-      } else if (const auto* node = var.as<VarNode>()) {
-        return mutator->ExprMutator::VisitVarDef_(node);
-      } else {
-        LOG(FATAL) << "TypeError: Invalid type: " << var->GetTypeKey();
-      }
-    });
-
-TVM_FFI_REGISTER_GLOBAL("relax.PyExprMutatorVisitExprPostOrder")
-    .set_body_typed([](PyExprMutator mutator, const Expr& expr) {
-      return mutator->VisitExprPostOrder(expr);
-    });
-
-TVM_FFI_REGISTER_GLOBAL("relax.PyExprMutatorVisitWithNewScope")
-    .set_body_typed([](PyExprMutator mutator, const Expr& expr) {
-      return mutator->VisitWithNewScope(expr);
-    });
-
-TVM_FFI_REGISTER_GLOBAL("relax.PyExprMutatorLookupBinding")
-    .set_body_typed([](PyExprMutator mutator, const Var& var) {
-      return mutator->LookupBinding(var);
-    });
-
-TVM_FFI_REGISTER_GLOBAL("relax.PyExprMutatorWithStructInfo")
-    .set_body_typed([](PyExprMutator mutator, Var var, StructInfo sinfo) {
-      return mutator->WithStructInfo(var, sinfo);
-    });
-
-TVM_FFI_REGISTER_GLOBAL("relax.PyExprMutatorSetVarRemap")
-    .set_body_typed([](PyExprMutator mutator, Id id, Var var) {
-      return mutator->var_remap_[id] = var;
-    });
-
-TVM_FFI_REGISTER_GLOBAL("relax.PyExprMutatorGetVarRemap")
-    .set_body_typed([](PyExprMutator mutator, Id id) { return mutator->var_remap_[id]; });
-
-TVM_FFI_STATIC_INIT_BLOCK({
+TVM_FFI_STATIC_INIT_BLOCK() {
   PyExprVisitorNode::RegisterReflection();
   PyExprMutatorNode::RegisterReflection();
-});
+}
 
 }  // namespace relax
 }  // namespace tvm
