@@ -33,9 +33,10 @@ namespace tvm {
 namespace relax {
 
 /*! \brief Image resize2d operator. */
-Expr resize2d(Expr data, Expr size, Array<FloatImm> roi, String layout, String method,
-              String coordinate_transformation_mode, String rounding_method, double cubic_alpha,
-              int cubic_exclude, double extrapolation_value, Optional<DataType> out_dtype);
+Expr resize2d(Expr data, Expr size, ffi::Array<FloatImm> roi, ffi::String layout,
+              ffi::String method, ffi::String coordinate_transformation_mode,
+              ffi::String rounding_method, double cubic_alpha, int cubic_exclude,
+              double extrapolation_value, ffi::Optional<DataType> out_dtype);
 
 }  // namespace relax
 }  // namespace tvm
