@@ -33,8 +33,9 @@ namespace tvm {
 namespace relax {
 
 /*! \brief fused multi head attention */
-Expr attention(Expr query, Expr key, Expr value, Optional<Expr> bias, Optional<FloatImm> scale,
-               Optional<String> causal_mask, Optional<IntImm> window_size);
+Expr attention(Expr query, Expr key, Expr value, ffi::Optional<Expr> bias,
+               ffi::Optional<FloatImm> scale, ffi::Optional<ffi::String> causal_mask,
+               ffi::Optional<IntImm> window_size);
 
 }  // namespace relax
 }  // namespace tvm
