@@ -61,7 +61,7 @@ StructInfo InferStructInfoBroadcast(const Call& call, const BlockBuilder& ctx,
   }
 
   // VDevice
-  Optional<VDevice> vdevice = InferBinaryArithOpOutVDevice(call, ctx, lhs_sinfo, rhs_sinfo);
+  ffi::Optional<VDevice> vdevice = InferBinaryArithOpOutVDevice(call, ctx, lhs_sinfo, rhs_sinfo);
 
   auto get_ndim = [&](const StructInfo& sinfo) -> int {
     if (sinfo.as<PrimStructInfoNode>()) {
@@ -86,9 +86,9 @@ StructInfo InferStructInfoBroadcast(const Call& call, const BlockBuilder& ctx,
 
   // Shapes
 
-  auto get_shape = [](const StructInfo& sinfo) -> Optional<Array<PrimExpr>> {
+  auto get_shape = [](const StructInfo& sinfo) -> ffi::Optional<ffi::Array<PrimExpr>> {
     if (sinfo.as<PrimStructInfoNode>()) {
-      return Array<PrimExpr>{IntImm(DataType::Int(64), 1)};
+      return ffi::Array<PrimExpr>{IntImm(DataType::Int(64), 1)};
     } else if (const auto* tensor = sinfo.as<TensorStructInfoNode>()) {
       return tensor->GetShape();
     } else {
@@ -101,7 +101,7 @@ StructInfo InferStructInfoBroadcast(const Call& call, const BlockBuilder& ctx,
   auto lhs_shape = get_shape(lhs_sinfo);
   auto rhs_shape = get_shape(rhs_sinfo);
   if (lhs_shape && rhs_shape) {
-    Optional<Array<PrimExpr>> output_shape =
+    ffi::Optional<ffi::Array<PrimExpr>> output_shape =
         InferBinaryBroadcastShape(call, ctx, lhs_shape.value(), rhs_shape.value());
     if (output_shape.defined()) {
       ICHECK_EQ(static_cast<int>(output_shape.value().size()), output_ndim);
@@ -109,7 +109,7 @@ StructInfo InferStructInfoBroadcast(const Call& call, const BlockBuilder& ctx,
     }
   }
 
-  auto get_shape_expr = [](const StructInfo& sinfo) -> Optional<Expr> {
+  auto get_shape_expr = [](const StructInfo& sinfo) -> ffi::Optional<Expr> {
     if (const auto* tensor = sinfo.as<TensorStructInfoNode>()) {
       return tensor->shape;
     } else {
@@ -142,9 +142,9 @@ StructInfo InferStructInfoBroadcastCMP(const Call& call, const BlockBuilder& ctx
          const StructInfo& rhs_sinfo) { return DataType::Bool(); });
 }
 
-InferLayoutOutput InferLayoutBinaryEwise(const Call& call,
-                                         const Map<String, Array<String>>& desired_layouts,
-                                         const VarLayoutMap& var_layout_map) {
+InferLayoutOutput InferLayoutBinaryEwise(
+    const Call& call, const ffi::Map<ffi::String, ffi::Array<ffi::String>>& desired_layouts,
+    const VarLayoutMap& var_layout_map) {
   ICHECK(NoDesiredLayout(call, desired_layouts));
   LayoutDecision layout1 = GetLayoutDecision(var_layout_map, call->args[0]);
   LayoutDecision layout2 = GetLayoutDecision(var_layout_map, call->args[1]);
@@ -155,8 +155,8 @@ InferLayoutOutput InferLayoutBinaryEwise(const Call& call,
   ICHECK(!x1_sinfo->IsUnknownNdim() && !x2_sinfo->IsUnknownNdim())
       << "Unknown dim tensors should not be handled by this function";
 
-  Optional<ShapeExpr> shape1 = GetRef<ShapeExpr>(x1_sinfo->shape.as<ShapeExprNode>());
-  Optional<ShapeExpr> shape2 = GetRef<ShapeExpr>(x2_sinfo->shape.as<ShapeExprNode>());
+  ffi::Optional<ShapeExpr> shape1 = ffi::GetRef<ShapeExpr>(x1_sinfo->shape.as<ShapeExprNode>());
+  ffi::Optional<ShapeExpr> shape2 = ffi::GetRef<ShapeExpr>(x2_sinfo->shape.as<ShapeExprNode>());
   // Lets handle sub indexing as long as primal dims are matching
   if (layout1->layout.ndim_primal() == layout2->layout.ndim_primal()) {
     if ((layout1->layout.ndim() >= layout2->layout.ndim()) && shape2.defined()) {
