@@ -24,33 +24,37 @@
 
 #include "datatype.h"
 
+#include <tvm/ffi/reflection/registry.h>
+
 #include <utility>
 
 namespace tvm {
 namespace relax {
 
-TVM_FFI_STATIC_INIT_BLOCK({
+TVM_FFI_STATIC_INIT_BLOCK() {
   AstypeAttrs::RegisterReflection();
   WrapParamAttrs::RegisterReflection();
-});
+}
 
 /* relax.astype */
-TVM_REGISTER_NODE_TYPE(AstypeAttrs);
 
 Expr astype(Expr x, DataType dtype) {
-  ObjectPtr<AstypeAttrs> attrs = make_object<AstypeAttrs>();
+  ObjectPtr<AstypeAttrs> attrs = ffi::make_object<AstypeAttrs>();
   attrs->dtype = dtype;
 
   static const Op& op = Op::Get("relax.astype");
   return Call(op, {std::move(x)}, Attrs(attrs), {});
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.op.astype").set_body_typed(astype);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.op.astype", astype);
+}
 
 StructInfo InferStructInfoAstype(const Call& call, const BlockBuilder& ctx) {
   TensorStructInfo sinfo = GetUnaryInputTensorStructInfo(call, ctx);
   const auto* attrs = call->attrs.as<AstypeAttrs>();
-  ObjectPtr<TensorStructInfoNode> new_sinfo = make_object<TensorStructInfoNode>(*sinfo.get());
+  ObjectPtr<TensorStructInfoNode> new_sinfo = ffi::make_object<TensorStructInfoNode>(*sinfo.get());
   new_sinfo->dtype = attrs->dtype;
   return TensorStructInfo(new_sinfo);
 }
@@ -65,22 +69,24 @@ TVM_REGISTER_OP("relax.astype")
     .set_attr<Bool>("FPurity", Bool(true));
 
 /* relax.wrap_param */
-TVM_REGISTER_NODE_TYPE(WrapParamAttrs);
 
 Expr MakeWrapParam(Expr data, DataType dtype) {
-  ObjectPtr<WrapParamAttrs> attrs = make_object<WrapParamAttrs>();
+  ObjectPtr<WrapParamAttrs> attrs = ffi::make_object<WrapParamAttrs>();
   attrs->dtype = dtype;
 
   static const Op& op = Op::Get("relax.wrap_param");
   return Call(op, {std::move(data)}, Attrs(attrs), {});
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.op.wrap_param").set_body_typed(MakeWrapParam);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.op.wrap_param", MakeWrapParam);
+}
 
 StructInfo InferStructInfoWrapParam(const Call& call, const BlockBuilder& ctx) {
   TensorStructInfo sinfo = GetUnaryInputTensorStructInfo(call, ctx);
   const auto* attrs = call->attrs.as<WrapParamAttrs>();
-  ObjectPtr<TensorStructInfoNode> new_sinfo = make_object<TensorStructInfoNode>(*sinfo.get());
+  ObjectPtr<TensorStructInfoNode> new_sinfo = ffi::make_object<TensorStructInfoNode>(*sinfo.get());
   new_sinfo->dtype = attrs->dtype;
   return TensorStructInfo(new_sinfo);
 }
