@@ -24,6 +24,7 @@
 
 #include "linear_algebra.h"
 
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/topi/einsum.h>
 
 #include <algorithm>
@@ -33,26 +34,28 @@
 namespace tvm {
 namespace relax {
 
-TVM_FFI_STATIC_INIT_BLOCK({
+TVM_FFI_STATIC_INIT_BLOCK() {
   MatmulAttrs::RegisterReflection();
   EinsumAttrs::RegisterReflection();
-});
+}
 
 /* relax.matmul */
-TVM_REGISTER_NODE_TYPE(MatmulAttrs);
 
-Expr matmul(Expr x1, Expr x2, Optional<DataType> out_dtype) {
-  ObjectPtr<MatmulAttrs> attrs = make_object<MatmulAttrs>();
+Expr matmul(Expr x1, Expr x2, ffi::Optional<DataType> out_dtype) {
+  ObjectPtr<MatmulAttrs> attrs = ffi::make_object<MatmulAttrs>();
   attrs->out_dtype = out_dtype.value_or(DataType::Void());
 
   static const Op& op = Op::Get("relax.matmul");
   return Call(op, {std::move(x1), std::move(x2)}, Attrs(attrs), {});
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.op.matmul").set_body_typed(matmul);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.op.matmul", matmul);
+}
 
 StructInfo InferStructInfoMatmul(const Call& call, const BlockBuilder& ctx) {
-  Array<TensorStructInfo> input_sinfo = GetInputTensorStructInfo(call, ctx);
+  ffi::Array<TensorStructInfo> input_sinfo = GetInputTensorStructInfo(call, ctx);
   Expr lhs = call->args[0];
   Expr rhs = call->args[1];
   TensorStructInfo x1_sinfo = input_sinfo[0];
@@ -118,11 +121,11 @@ StructInfo InferStructInfoMatmul(const Call& call, const BlockBuilder& ctx) {
     return TensorStructInfo(out_dtype, output_ndim);
   }
 
-  Array<PrimExpr> x1_shape_prefix{x1_shape->values.begin(),
-                                  x1_shape->values.end() - 2 + x1_prepended};
-  Array<PrimExpr> x2_shape_prefix{x2_shape->values.begin(),
-                                  x2_shape->values.end() - 2 + x2_appended};
-  Optional<Array<PrimExpr>> output_shape_prefix =
+  ffi::Array<PrimExpr> x1_shape_prefix{x1_shape->values.begin(),
+                                       x1_shape->values.end() - 2 + x1_prepended};
+  ffi::Array<PrimExpr> x2_shape_prefix{x2_shape->values.begin(),
+                                       x2_shape->values.end() - 2 + x2_appended};
+  ffi::Optional<ffi::Array<PrimExpr>> output_shape_prefix =
       InferBinaryBroadcastShape(call, ctx, x1_shape_prefix, x2_shape_prefix);
   if (!output_shape_prefix.defined()) {
     if (vdev.defined()) {
@@ -143,7 +146,7 @@ StructInfo InferStructInfoMatmul(const Call& call, const BlockBuilder& ctx) {
                      << x2_reduction_length << " are not equal.");
   }
 
-  Array<PrimExpr> output_shape = output_shape_prefix.value();
+  ffi::Array<PrimExpr> output_shape = output_shape_prefix.value();
   if (!x1_prepended) {
     output_shape.push_back(x1_shape->values[x1_ndim - 2]);
   }
@@ -171,23 +174,25 @@ TVM_REGISTER_OP("relax.matmul")
     .set_attr<Bool>("FPurity", Bool(true));
 
 /* relax.einsum */
-TVM_REGISTER_NODE_TYPE(EinsumAttrs);
 
-Expr einsum(Expr operands, String subscripts) {
-  ObjectPtr<EinsumAttrs> attrs = make_object<EinsumAttrs>();
+Expr einsum(Expr operands, ffi::String subscripts) {
+  ObjectPtr<EinsumAttrs> attrs = ffi::make_object<EinsumAttrs>();
   attrs->subscripts = std::move(subscripts);
 
   static const Op& op = Op::Get("relax.einsum");
   return Call(op, {std::move(operands)}, Attrs{attrs}, {});
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.op.einsum").set_body_typed(einsum);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.op.einsum", einsum);
+}
 
 StructInfo InferStructInfoEinsum(const Call& call, const BlockBuilder& ctx) {
   if (call->args.size() != 1) {
     ctx->ReportFatal(Diagnostic::Error(call) << "Einsum op should take 1 argument");
   }
-  Array<TensorStructInfo> operands_tensor_sinfo =
+  ffi::Array<TensorStructInfo> operands_tensor_sinfo =
       GetTensorStructInfoFromTuple(call, ctx, call->args[0]);
   if (operands_tensor_sinfo.empty()) {
     ctx->ReportFatal(Diagnostic::Error(call)
@@ -214,10 +219,10 @@ StructInfo InferStructInfoEinsum(const Call& call, const BlockBuilder& ctx) {
     }
   }
 
-  String subscripts = attrs->subscripts;
+  ffi::String subscripts = attrs->subscripts;
 
   DataType operand_dtype = operands_tensor_sinfo[0]->dtype;
-  std::vector<Array<PrimExpr>> input_shapes;
+  std::vector<ffi::Array<PrimExpr>> input_shapes;
   input_shapes.reserve(operands_tensor_sinfo.size());
 
   for (TensorStructInfo tensor_sinfo : operands_tensor_sinfo) {
@@ -241,7 +246,7 @@ StructInfo InferStructInfoEinsum(const Call& call, const BlockBuilder& ctx) {
     }
   }
   // Calculate output shape using InferEinsumShape in topi
-  Array<PrimExpr> oshape = topi::InferEinsumShape(subscripts, input_shapes);
+  ffi::Array<PrimExpr> oshape = topi::InferEinsumShape(subscripts, input_shapes);
 
   if (!vdevice_unknown) {
     return TensorStructInfo(ShapeExpr(oshape), operand_dtype, vdev);
@@ -263,7 +268,10 @@ Expr outer(Expr x1, Expr x2) {
   return Call(op, {std::move(x1), std::move(x2)}, {});
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.op.outer").set_body_typed(outer);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.op.outer", outer);
+}
 
 StructInfo InferStructInfoOuter(const Call& call, const BlockBuilder& ctx) {
   auto input_sinfo = GetInputTensorStructInfo(call, ctx);
@@ -282,7 +290,7 @@ StructInfo InferStructInfoOuter(const Call& call, const BlockBuilder& ctx) {
   if (!x1_shape || !x2_shape) {
     return TensorStructInfo(x1_sinfo->dtype, 2);
   }
-  Array<PrimExpr> output_shape = {x1_shape->values[0], x2_shape->values[0]};
+  ffi::Array<PrimExpr> output_shape = {x1_shape->values[0], x2_shape->values[0]};
   return TensorStructInfo(ShapeExpr(output_shape), x1_sinfo->dtype);
 }
 
