@@ -43,14 +43,16 @@ namespace relax {
  *  2. be prepended with a prefix "relax." as the identifier string in the operator registry.
  */
 #define RELAX_REGISTER_STATISTICAL_OP_INTERFACE(OpName)                           \
-  Expr OpName(Expr x, Optional<Array<Integer>> axis, bool keepdims) {             \
-    ObjectPtr<StatisticalAttrs> attrs = make_object<StatisticalAttrs>();          \
+  Expr OpName(Expr x, ffi::Optional<ffi::Array<Integer>> axis, bool keepdims) {   \
+    ObjectPtr<StatisticalAttrs> attrs = ffi::make_object<StatisticalAttrs>();     \
     attrs->axis = std::move(axis);                                                \
     attrs->keepdims = keepdims;                                                   \
     static const Op& op = Op::Get("relax." #OpName);                              \
     return Call(op, {std::move(x)}, Attrs{attrs}, {});                            \
   }                                                                               \
-  TVM_FFI_REGISTER_GLOBAL("relax.op." #OpName).set_body_typed(OpName);            \
+  TVM_FFI_STATIC_INIT_BLOCK() {                                                   \
+    tvm::ffi::reflection::GlobalDef().def("relax.op." #OpName, OpName);           \
+  }                                                                               \
   TVM_REGISTER_OP("relax." #OpName)                                               \
       .set_num_inputs(1)                                                          \
       .add_argument("x", "Tensor", "The input data tensor")                       \
@@ -66,22 +68,22 @@ namespace relax {
  * reduced are left in the result as dimensions with size one. With this option, the result will
  * broadcast correctly against the input tensor. \return The result after reduction.
  */
-Expr max(Expr x, Optional<Array<Integer>> axis, bool keepdims);
+Expr max(Expr x, ffi::Optional<ffi::Array<Integer>> axis, bool keepdims);
 
 /*! \brief Computes the mean of tensor elements over given axes. */
-Expr mean(Expr x, Optional<Array<Integer>> axis, bool keepdims);
+Expr mean(Expr x, ffi::Optional<ffi::Array<Integer>> axis, bool keepdims);
 
 /*! \brief Computes the min of tensor elements over given axes. */
-Expr min(Expr x, Optional<Array<Integer>> axis, bool keepdims);
+Expr min(Expr x, ffi::Optional<ffi::Array<Integer>> axis, bool keepdims);
 
 /*! \brief Computes the product of tensor elements over given axes. */
-Expr prod(Expr x, Optional<Array<Integer>> axis, bool keepdims);
+Expr prod(Expr x, ffi::Optional<ffi::Array<Integer>> axis, bool keepdims);
 
 /*! \brief Computes the standard deviation of tensor elements over given axes. */
-Expr std(Expr x, Optional<Array<Integer>> axis, bool keepdims);
+Expr std(Expr x, ffi::Optional<ffi::Array<Integer>> axis, bool keepdims);
 
 /*! \brief Computes the sum of tensor elements over given axes. */
-Expr sum(Expr x, Optional<Array<Integer>> axis, bool keepdims);
+Expr sum(Expr x, ffi::Optional<ffi::Array<Integer>> axis, bool keepdims);
 
 /*!
  * \brief Numpy style cumprod op. Return the cumulative inclusive product of the elements along
@@ -96,8 +98,8 @@ Expr sum(Expr x, Optional<Array<Integer>> axis, bool keepdims);
  * \return The computed
  * result.
  */
-Expr cumprod(Expr data, Optional<int64_t> axis = std::nullopt,
-             Optional<DataType> dtype = std::nullopt, Bool exclusive = Bool(false));
+Expr cumprod(Expr data, ffi::Optional<int64_t> axis = std::nullopt,
+             ffi::Optional<DataType> dtype = std::nullopt, Bool exclusive = Bool(false));
 
 /*!
  * \brief Numpy style cumsum op. Return the cumulative inclusive sum of the elements along
@@ -111,11 +113,11 @@ Expr cumprod(Expr data, Optional<int64_t> axis = std::nullopt,
  * which the first element is not included.
  * \return The computed result.
  */
-Expr cumsum(Expr data, Optional<int64_t> axis = std::nullopt,
-            Optional<DataType> dtype = std::nullopt, Bool exclusive = Bool(false));
+Expr cumsum(Expr data, ffi::Optional<int64_t> axis = std::nullopt,
+            ffi::Optional<DataType> dtype = std::nullopt, Bool exclusive = Bool(false));
 
 /*! \brief Computes the variance of tensor elements over given axes. */
-Expr variance(Expr x, Optional<Array<Integer>> axis, bool keepdims);
+Expr variance(Expr x, ffi::Optional<ffi::Array<Integer>> axis, bool keepdims);
 
 }  // namespace relax
 }  // namespace tvm
