@@ -38,7 +38,7 @@ namespace relax {
  *  (Only for unary arith operators since all check operators don't require float dtype.)
  */
 #define RELAX_REGISTER_UNARY_OP_AND_IMPL(OpName) \
-  RELAX_UNARY_OP_INTERFACE(OpName, #OpName);     \
+  RELAX_UNARY_OP_INTERFACE(OpName, #OpName)      \
   RELAX_REGISTER_UNARY_OP(#OpName)
 
 #define RELAX_REGISTER_UNARY_ARITH_OP_AND_IMPL(OpName, RequireFloatDtype) \
