@@ -23,6 +23,7 @@
  * satisfy their temporary storage requirement.
  */
 
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/ir/name_supply.h>
 #include <tvm/relax/expr.h>
 #include <tvm/relax/expr_functor.h>
@@ -51,18 +52,18 @@ class ExternFunctionRewriter : ExprMutator {
   }
 
   Expr VisitExpr_(const FunctionNode* func_node) override {
-    if (!func_node->GetAttr<String>(attr::kCodegen) &&
-        !func_node->GetAttr<String>(attr::kComposite)) {
+    if (!func_node->GetAttr<ffi::String>(attr::kCodegen) &&
+        !func_node->GetAttr<ffi::String>(attr::kComposite)) {
       return ExprMutator::VisitExpr_(func_node);
     }
     if (auto workspace = func_node->GetAttr<Integer>(attr::kWorkspaceSize)) {
       // Append the workspace parameter to this function.
-      Array<Var> new_params = func_node->params;
+      ffi::Array<Var> new_params = func_node->params;
 
       auto sinfo = TensorStructInfo(ShapeExpr({Integer(max_workspace_size_)}), DataType::UInt(8));
       Var workspace_param(name_sup_->FreshName("workspace"), sinfo);
 
-      if (func_node->GetAttr<String>(attr::kCodegen)) {
+      if (func_node->GetAttr<ffi::String>(attr::kCodegen)) {
         workspace_var_param_ = workspace_param;
       }
 
@@ -80,7 +81,7 @@ class ExternFunctionRewriter : ExprMutator {
     if (auto var = new_op.as<Var>()) {
       if (auto callee = builder_->LookupBinding(var.value());
           callee && callee->IsInstance<FunctionNode>() &&
-          Downcast<Function>(callee.value())->GetAttr<String>(attr::kComposite)) {
+          Downcast<Function>(callee.value())->GetAttr<ffi::String>(attr::kComposite)) {
         // Append the workspace argument to this call. The callee should have been updated to accept
         // a workspace as the last parameter.
         auto new_args = call_node->args;
@@ -126,13 +127,13 @@ class WorkspaceProvider : ExprMutator {
                                WithAttr(f, tvm::attr::kGlobalSymbol, new_gvar->name_hint));
       gvar_map_[gvar] = new_gvar;
       new_gvars_.insert(new_gvar);
-      builder_->GetContextIRModule()->Remove(GetRef<GlobalVar>(gvar));
+      builder_->GetContextIRModule()->Remove(ffi::GetRef<GlobalVar>(gvar));
     }
 
     for (const auto& [gvar, f] : mod_->functions) {
       workspace_var_main_ = Var();
-      if (!f->IsInstance<relax::FunctionNode>() || f->GetAttr<String>(attr::kCodegen) ||
-          f->GetAttr<String>(attr::kComposite)) {
+      if (!f->IsInstance<relax::FunctionNode>() || f->GetAttr<ffi::String>(attr::kCodegen) ||
+          f->GetAttr<ffi::String>(attr::kComposite)) {
         continue;
       }
       auto func = Downcast<Function>(mod_->Lookup(gvar));
@@ -201,7 +202,10 @@ Pass AllocateWorkspace() {
   return CreateModulePass(pass_func, 0, "AllocateWorkspace", {});
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.transform.AllocateWorkspace").set_body_typed(AllocateWorkspace);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.transform.AllocateWorkspace", AllocateWorkspace);
+}
 
 }  // namespace transform
 }  // namespace tvm
