@@ -24,6 +24,7 @@
  *
  * Currently it removes common subexpressions within a Function.
  */
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/relax/analysis.h>
 #include <tvm/relax/expr_functor.h>
 #include <tvm/relax/transform.h>
@@ -47,7 +48,7 @@ namespace {
  */
 struct ReplacementKey {
   tvm::relax::Expr bound_value;
-  tvm::Optional<tvm::relax::StructInfo> match_cast = std::nullopt;
+  tvm::ffi::Optional<tvm::relax::StructInfo> match_cast = std::nullopt;
 
   explicit ReplacementKey(const tvm::relax::Binding& binding)
       : bound_value(GetBoundValue(binding)) {
@@ -154,7 +155,7 @@ class CommonSubexprEliminator : public ExprMutator {
     // copy of the mutator, to avoid replacing a child-scope
     // expression with a parent-scope binding, or vice versa.
     if (expr_replacements_.size() || var_remap_.size()) {
-      return VisitWithCleanScope(GetRef<Expr>(op));
+      return VisitWithCleanScope(ffi::GetRef<Expr>(op));
     } else {
       return ExprMutator::VisitExpr_(op);
     }
@@ -167,7 +168,7 @@ class CommonSubexprEliminator : public ExprMutator {
     if (op->cond.same_as(cond) && op->true_branch.same_as(true_branch) &&
         op->false_branch.same_as(false_branch) &&
         VisitAndCheckStructInfoFieldUnchanged(op->struct_info_)) {
-      return GetRef<Expr>(op);
+      return ffi::GetRef<Expr>(op);
     } else {
       return If(cond, true_branch, false_branch, op->span);
     }
@@ -192,7 +193,7 @@ class CommonSubexprEliminator : public ExprMutator {
     static const auto& allocator_attr_map = Op::GetAttrMap<Bool>("TAllocator");
     if (const auto* call = expr.as<CallNode>()) {
       if (const auto* op = call->op.as<OpNode>()) {
-        bool is_allocator = allocator_attr_map.get(GetRef<Op>(op), Bool(false))->value;
+        bool is_allocator = allocator_attr_map.get(ffi::GetRef<Op>(op), Bool(false))->value;
         if (is_allocator) {
           return true;
         }
@@ -221,8 +222,10 @@ Pass EliminateCommonSubexpr(bool call_only) {
   return CreateFunctionPass(pass_func, 1, "EliminateCommonSubexpr", {});
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.transform.EliminateCommonSubexpr")
-    .set_body_typed(EliminateCommonSubexpr);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.transform.EliminateCommonSubexpr", EliminateCommonSubexpr);
+}
 
 }  // namespace transform
 
