@@ -27,7 +27,7 @@
 #ifndef TVM_RELAX_TRANSFORM_INFER_LAYOUT_UTILS_H_
 #define TVM_RELAX_TRANSFORM_INFER_LAYOUT_UTILS_H_
 
-#include <tvm/ffi/reflection/reflection.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/relax/attrs/create.h>
 #include <tvm/relax/attrs/datatype.h>
 #include <tvm/relax/attrs/image.h>
@@ -69,15 +69,13 @@ class LayoutDecisionNode : public Object {
         .def_ro("is_unknown_dim", &LayoutDecisionNode::is_unknown_dim);
   }
 
-  TVM_DECLARE_BASE_OBJECT_INFO(LayoutDecisionNode, Object);
-
-  static constexpr const char* _type_key = "relax.transform.LayoutDecision";
+  TVM_FFI_DECLARE_OBJECT_INFO("relax.transform.LayoutDecision", LayoutDecisionNode, Object);
 };
 
 class LayoutDecision : public ObjectRef {
  public:
   LayoutDecision(Layout layout, bool is_unknown_dim = false) {  // NOLINT(*)
-    auto n = make_object<LayoutDecisionNode>();
+    auto n = ffi::make_object<LayoutDecisionNode>();
     n->layout = std::move(layout);
     n->is_unknown_dim = is_unknown_dim;
     data_ = n;
@@ -92,7 +90,7 @@ class LayoutDecision : public ObjectRef {
     return operator->()->layout.name();
   }
 
-  TVM_DEFINE_OBJECT_REF_METHODS(LayoutDecision, ObjectRef, LayoutDecisionNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(LayoutDecision, ObjectRef, LayoutDecisionNode);
 };
 
 using NLayout = NestedMsg<LayoutDecision>;
@@ -105,10 +103,10 @@ using NLayout = NestedMsg<LayoutDecision>;
  */
 class InferLayoutOutputNode : public Object {
  public:
-  Array<NLayout> input_layouts;
-  Array<NLayout> output_layouts;
+  ffi::Array<NLayout> input_layouts;
+  ffi::Array<NLayout> output_layouts;
   Attrs new_attrs;
-  Map<Integer, Expr> new_args;
+  ffi::Map<Integer, Expr> new_args;
 
   static void RegisterReflection() {
     namespace refl = tvm::ffi::reflection;
@@ -119,23 +117,21 @@ class InferLayoutOutputNode : public Object {
         .def_ro("new_args", &InferLayoutOutputNode::new_args);
   }
 
-  TVM_DECLARE_BASE_OBJECT_INFO(InferLayoutOutputNode, Object);
-
-  static constexpr const char* _type_key = "relax.transform.InferLayoutOutput";
+  TVM_FFI_DECLARE_OBJECT_INFO("relax.transform.InferLayoutOutput", InferLayoutOutputNode, Object);
 };
 
 class InferLayoutOutput : public ObjectRef {
  public:
-  explicit InferLayoutOutput(Array<NLayout> input_layouts, Array<NLayout> output_layouts,
-                             Attrs new_attrs, Map<Integer, Expr> new_args = {}) {
-    auto n = make_object<InferLayoutOutputNode>();
+  explicit InferLayoutOutput(ffi::Array<NLayout> input_layouts, ffi::Array<NLayout> output_layouts,
+                             Attrs new_attrs, ffi::Map<Integer, Expr> new_args = {}) {
+    auto n = ffi::make_object<InferLayoutOutputNode>();
     n->input_layouts = std::move(input_layouts);
     n->output_layouts = std::move(output_layouts);
     n->new_attrs = std::move(new_attrs);
     n->new_args = std::move(new_args);
     data_ = n;
   }
-  TVM_DEFINE_OBJECT_REF_METHODS(InferLayoutOutput, ObjectRef, InferLayoutOutputNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NULLABLE(InferLayoutOutput, ObjectRef, InferLayoutOutputNode);
 };
 
 struct NLayoutEqual {
@@ -150,7 +146,7 @@ struct NLayoutEqual {
   }
 };
 
-using VarLayoutMap = Map<Var, NLayout>;
+using VarLayoutMap = ffi::Map<Var, NLayout>;
 
 /*!
  * \brief Layout conversion interface.
@@ -159,7 +155,7 @@ using VarLayoutMap = Map<Var, NLayout>;
  * \param var_layout_map The layout of the variables.
  */
 using FRelaxInferLayout = ffi::TypedFunction<InferLayoutOutput(
-    const Call& call, const Map<String, Array<String>>& desired_layouts,
+    const Call& call, const ffi::Map<ffi::String, ffi::Array<ffi::String>>& desired_layouts,
     const VarLayoutMap& var_layout_map)>;
 
 /*!
@@ -225,7 +221,7 @@ Layout TransposeLike(const Layout& input, const Layout& src, const Layout& dst);
  * \param dst The destination layout.
  * \return The transposed input str.
  */
-String TransposeStrLike(const String& input, const Layout& src, const Layout& dst);
+ffi::String TransposeStrLike(const ffi::String& input, const Layout& src, const Layout& dst);
 
 /*!
  * \brief Find axis in the dst layout. 0 represents the first axis, 1 represents the second axis,
@@ -258,7 +254,8 @@ NLayout GetNLayout(const VarLayoutMap& var_layout_map, const Expr& arg);
  * \param desired_layouts The desired layouts of the operator.
  * \return True if the op is not in the desired layout.
  */
-bool NoDesiredLayout(const Call& call, const Map<String, Array<String>>& desired_layouts);
+bool NoDesiredLayout(const Call& call,
+                     const ffi::Map<ffi::String, ffi::Array<ffi::String>>& desired_layouts);
 
 /*!
  * \brief Let a tensor with ndim to follow the src layout decision.
