@@ -20,6 +20,7 @@
  * \file src/relax/transform/remove_purity_checking.cc
  * \brief Apply kForcePure in all pure functions and unwrap all calls to pure overrides
  */
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/relax/expr_functor.h>
 #include <tvm/relax/struct_info.h>
 #include <tvm/relax/transform.h>
@@ -36,7 +37,7 @@ class PurityRemover : public ExprMutator {
     bool purity = func->is_pure;
     auto ret = func;
     if (purity) {
-      ret = std::move(WithAttr<Function>(func, relax::attr::kForcePure, true));
+      ret = WithAttr<Function>(func, relax::attr::kForcePure, true);
     }
     auto new_body = VisitExpr(ret->body);
     if (!new_body.same_as(ret->body)) {
@@ -48,13 +49,13 @@ class PurityRemover : public ExprMutator {
 
   Expr VisitExpr_(const CallNode* call) override {
     if (call->op == call_pure_packed_op_) {
-      auto ret = Call(call->args[0], Array<Expr>(call->args.begin() + 1, call->args.end()),
+      auto ret = Call(call->args[0], ffi::Array<Expr>(call->args.begin() + 1, call->args.end()),
                       call->attrs, call->sinfo_args);
       return VisitExpr(ret);
     }
     if (call->op == call_inplace_packed_op_) {
       // call_inplace_packed has its own attrs so we don't pass those down
-      auto ret = Call(call->args[0], Array<Expr>(call->args.begin() + 1, call->args.end()),
+      auto ret = Call(call->args[0], ffi::Array<Expr>(call->args.begin() + 1, call->args.end()),
                       tvm::Attrs(), call->sinfo_args);
       return VisitExpr(ret);
     }
@@ -67,7 +68,7 @@ class PurityRemover : public ExprMutator {
 
   Expr VisitExpr_(const FunctionNode* func) override {
     // handling inner functions: we will remove purity annotations from them too
-    return RemovePurity(GetRef<Function>(func));
+    return RemovePurity(ffi::GetRef<Function>(func));
   }
 
  private:
@@ -88,8 +89,10 @@ Pass RemovePurityChecking() {
   return CreateFunctionPass(pass_func, 0, "RemovePurityChecking", {});
 }
 
-TVM_FFI_REGISTER_GLOBAL("relax.transform.RemovePurityChecking")
-    .set_body_typed(RemovePurityChecking);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("relax.transform.RemovePurityChecking", RemovePurityChecking);
+}
 
 }  // namespace transform
 
