@@ -84,8 +84,8 @@ class MemoizedExprTranslator : public ::tvm::relax::ExprFunctor<OutputType(const
   }
 
   virtual OutputType VisitExpr_(const VarNode* vn) {
-    ICHECK(memo_.count(GetRef<Expr>(vn)));
-    return memo_[GetRef<Expr>(vn)];
+    ICHECK(memo_.count(ffi::GetRef<Expr>(vn)));
+    return memo_[ffi::GetRef<Expr>(vn)];
   }
 
   virtual OutputType VisitBinding_(const VarBindingNode* binding) {
@@ -115,7 +115,7 @@ class MemoizedExprTranslator : public ::tvm::relax::ExprFunctor<OutputType(const
  * \param entry_functions list of entry functions
  * \return The updated module.
  */
-TVM_DLL IRModule DeadCodeElimination(const IRModule& mod, Array<String> entry_funcs);
+TVM_DLL IRModule DeadCodeElimination(const IRModule& mod, ffi::Array<ffi::String> entry_funcs);
 
 /*!
  * \brief Get the external symbol of the Relax function name.
@@ -124,8 +124,8 @@ TVM_DLL IRModule DeadCodeElimination(const IRModule& mod, Array<String> entry_fu
  * \return An external symbol.
  */
 inline std::string GetExtSymbol(const Function& func) {
-  const auto name_node = func->GetAttr<String>(tvm::attr::kGlobalSymbol);
-  ICHECK(name_node.defined()) << "Fail to retrieve external symbol.";
+  const auto name_node = func->GetAttr<ffi::String>(tvm::attr::kGlobalSymbol);
+  ICHECK(name_node.has_value()) << "Fail to retrieve external symbol.";
   return std::string(name_node.value());
 }
 
@@ -142,7 +142,7 @@ inline std::string GetExtSymbol(const Function& func) {
  */
 IRModule MakeGroupedFunctions(
     IRModule mod, const std::unordered_map<const Object*, GraphPartitioner::Group*>& partition,
-    bool lift_constants = true, const Array<String>& entry_function_names = {});
+    bool lift_constants = true, const ffi::Array<ffi::String>& entry_function_names = {});
 
 /*!
  * \brief Check if the given StructInfo is a scalar tensor. The sinfo should be an instance of
@@ -172,7 +172,7 @@ bool IsScalarTensor(const Expr& expr);
 template <typename FType>
 bool IsNestedTensorConditioned(const StructInfo& sinfo, FType f_condition) {
   if (const auto* tensor_sinfo = sinfo.as<TensorStructInfoNode>()) {
-    return f_condition(GetRef<TensorStructInfo>(tensor_sinfo));
+    return f_condition(ffi::GetRef<TensorStructInfo>(tensor_sinfo));
   } else if (const auto* tuple_sinfo = sinfo.as<TupleStructInfoNode>()) {
     return !std::any_of(
         tuple_sinfo->fields.begin(), tuple_sinfo->fields.end(),
@@ -209,7 +209,7 @@ class VarReplacer : public ExprMutator {
 
  private:
   Expr VisitExpr_(const VarNode* op) final {
-    Var var = GetRef<Var>(op);
+    Var var = ffi::GetRef<Var>(op);
     auto it = var_remap_.find(var->vid);
     return it == var_remap_.end() ? var : it->second;
   }
@@ -241,19 +241,19 @@ class SymbolicVarRenewMutator : public ExprMutator, tir::ExprMutator {
   // 1. Visit and replace all tir::Vars at the definition point
   // 2. Revisit the function again and update the use side.
   PrimExpr VisitExpr_(const tir::VarNode* op) final {
-    auto it = var_map_.find(GetRef<tir::Var>(op));
+    auto it = var_map_.find(ffi::GetRef<tir::Var>(op));
     if (it != var_map_.end()) {
       return (*it).second;
     } else {
-      auto n = make_object<tir::VarNode>(*op);
+      auto n = ffi::make_object<tir::VarNode>(*op);
       tir::Var v(n);
-      var_map_.Set(GetRef<tir::Var>(op), v);
+      var_map_.Set(ffi::GetRef<tir::Var>(op), v);
       return v;
     }
   }
 
   Expr VisitExpr_(const FunctionNode* op) {
-    tvm::Array<Var> params;
+    tvm::ffi::Array<Var> params;
     bool all_params_unchanged = true;
     for (Var param : op->params) {
       Var new_param = this->VisitVarDef(param);
@@ -267,14 +267,14 @@ class SymbolicVarRenewMutator : public ExprMutator, tir::ExprMutator {
     Expr body = this->VisitWithNewScope(op->body, params);
 
     if (all_params_unchanged && body.same_as(op->body)) {
-      return GetRef<Expr>(op);
+      return ffi::GetRef<Expr>(op);
     } else {
       auto new_ret_sinfo = this->VisitExprDepStructInfoField(op->ret_struct_info);
       return Function(params, body, new_ret_sinfo, op->is_pure, op->attrs);
     }
   }
 
-  Map<tir::Var, tir::Var> var_map_;
+  ffi::Map<tir::Var, tir::Var> var_map_;
 };
 
 /*!
@@ -286,7 +286,7 @@ class FunctionCopier : public SymbolicVarRenewMutator {
  public:
   FunctionCopier() = default;
   Function Copy(Function func) { return Downcast<Function>(VisitExpr(func)); }
-  Map<Var, Var> GetVarMap() { return relax_var_map_; }
+  ffi::Map<Var, Var> GetVarMap() { return relax_var_map_; }
 
  private:
   using relax::ExprMutator::VisitExpr;
@@ -295,7 +295,7 @@ class FunctionCopier : public SymbolicVarRenewMutator {
     Var new_var = SymbolicVarRenewMutator::VisitVarDef_(var);
     Var copied_var = DataflowVar(new_var->name_hint(), GetStructInfo(new_var), new_var->span);
     var_remap_[var->vid] = copied_var;
-    relax_var_map_.Set(GetRef<Var>(var), copied_var);
+    relax_var_map_.Set(ffi::GetRef<Var>(var), copied_var);
     return copied_var;
   }
 
@@ -303,11 +303,11 @@ class FunctionCopier : public SymbolicVarRenewMutator {
     Var new_var = SymbolicVarRenewMutator::VisitVarDef_(var);
     Var copied_var = Var(new_var->name_hint(), GetStructInfo(new_var), new_var->span);
     var_remap_[var->vid] = copied_var;
-    relax_var_map_.Set(GetRef<Var>(var), copied_var);
+    relax_var_map_.Set(ffi::GetRef<Var>(var), copied_var);
     return copied_var;
   }
 
-  Map<Var, Var> relax_var_map_;
+  ffi::Map<Var, Var> relax_var_map_;
 };
 
 /*!
@@ -319,7 +319,7 @@ class FunctionCopier : public SymbolicVarRenewMutator {
  */
 template <typename T>
 inline Constant MakeConstantScalar(T value, DataType dtype) {
-  runtime::NDArray arr = runtime::NDArray::Empty({}, dtype, {kDLCPU, 0});
+  runtime::Tensor arr = runtime::Tensor::Empty({}, dtype, {kDLCPU, 0});
   if (dtype == DataType::Float(32)) {
     *static_cast<float*>(arr->data) = static_cast<float>(value);
   } else if (dtype == DataType::Float(64)) {
@@ -360,7 +360,7 @@ inline Constant MakeConstantScalar(T value, DataType dtype) {
   return Constant(arr);
 }
 
-inline Array<Integer> GetOrderedPositiveAxes(const Array<Integer>& axes, int ndim) {
+inline ffi::Array<Integer> GetOrderedPositiveAxes(const ffi::Array<Integer>& axes, int ndim) {
   std::vector<int64_t> ret;
   ret.reserve(axes.size());
   for (const auto& axis : axes) {
@@ -376,7 +376,7 @@ inline Array<Integer> GetOrderedPositiveAxes(const Array<Integer>& axes, int ndi
   return support::AsArray<int64_t, Integer>(ret);
 }
 
-inline String GetCodegenName(const std::string& composite_name) {
+inline ffi::String GetCodegenName(const std::string& composite_name) {
   auto delim_pos = composite_name.find(".");
   ICHECK(delim_pos != std::string::npos) << "The pattern name for a composite function should "
                                             "start with a compiler name followed by period.";
@@ -384,7 +384,7 @@ inline String GetCodegenName(const std::string& composite_name) {
 }
 
 inline int GetDeviceIndex(const IRModule& mod, const VDevice& vdevice) {
-  Array<GlobalInfo> vdevices = mod->global_infos["vdevice"];
+  ffi::Array<GlobalInfo> vdevices = mod->global_infos["vdevice"];
   for (int i = 0; i < static_cast<int>(vdevices.size()); ++i) {
     if (vdevices[i] == vdevice) {
       return i;
@@ -434,7 +434,8 @@ Expr CanonicalizeBindings(Expr expr);
  *
  * \ret The updated function.
  */
-Function BundleModelParams(const Function& func, Optional<String> param_tuple_name = std::nullopt);
+Function BundleModelParams(const Function& func,
+                           ffi::Optional<ffi::String> param_tuple_name = std::nullopt);
 
 /*! \brief Compose two functions
  *
