@@ -19,7 +19,7 @@
 
 /*!
  * \file src/runtime/const_loader_module.cc
- * \brief A wrapper for initializing imported modules using constant NDArray. This
+ * \brief A wrapper for initializing imported modules using constant Tensor. This
  * module is intended to be used by various runtime in the TVM stack, i.e.
  * graph executor, relax VM, AOT runtime, and various user defined runtimes. It
  * paves the way to separate the code and metedata, which makes compilation
@@ -27,12 +27,14 @@
  * code and constants significantly reduces the efforts for handling external
  * codegen and runtimes.
  */
+#include <dmlc/memory_io.h>
 #include <tvm/ffi/container/array.h>
 #include <tvm/ffi/container/map.h>
+#include <tvm/ffi/extra/module.h>
 #include <tvm/ffi/function.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/ffi/string.h>
-#include <tvm/runtime/module.h>
-#include <tvm/runtime/ndarray.h>
+#include <tvm/runtime/tensor.h>
 
 #include <cstdint>
 
@@ -43,12 +45,12 @@ namespace runtime {
  * \brief The const-loader module is designed to manage initialization of the
  * imported submodules for the C++ runtime.
  */
-class ConstLoaderModuleNode : public ModuleNode {
+class ConstLoaderModuleObj : public ffi::ModuleObj {
  public:
-  ConstLoaderModuleNode(
-      const std::unordered_map<std::string, NDArray>& const_var_ndarray,
+  ConstLoaderModuleObj(
+      const std::unordered_map<std::string, Tensor>& const_var_tensor,
       const std::unordered_map<std::string, std::vector<std::string>>& const_vars_by_symbol)
-      : const_var_ndarray_(const_var_ndarray), const_vars_by_symbol_(const_vars_by_symbol) {
+      : const_var_tensor_(const_var_tensor), const_vars_by_symbol_(const_vars_by_symbol) {
     VLOG(1) << "Creating ConstLoaderModule";
     // Only the related submodules are cached to reduce the number of runtime
     // symbol lookup for initialization. Otherwise, symbols/primitives in the
@@ -57,7 +59,7 @@ class ConstLoaderModuleNode : public ModuleNode {
       for (const auto& var : kv.second) {
         VLOG(1) << "ConstLoaderModuleNode has constant '" << var << "' for function '" << kv.first
                 << "'";
-        ICHECK_GT(const_var_ndarray_.count(var), 0)
+        ICHECK_GT(const_var_tensor_.count(var), 0)
             << "ConstLoaderModuleNode is missing entry for constant '" << var << "' for function '"
             << kv.first << "'";
       }
@@ -65,7 +67,7 @@ class ConstLoaderModuleNode : public ModuleNode {
     }
   }
 
-  ffi::Function GetFunction(const String& name, const ObjectPtr<Object>& sptr_to_self) final {
+  ffi::Optional<ffi::Function> GetFunction(const ffi::String& name) final {
     VLOG(1) << "ConstLoaderModuleNode::GetFunction(" << name << ")";
     // Initialize and memoize the module.
     // Usually, we have some warmup runs. The module initialization should be
@@ -74,11 +76,12 @@ class ConstLoaderModuleNode : public ModuleNode {
       this->InitSubModule(name);
       initialized_[name] = true;
     }
+    ObjectRef _self = ffi::GetRef<ObjectRef>(this);
 
-    if (name == "get_const_var_ndarray") {
-      return ffi::Function([sptr_to_self, this](ffi::PackedArgs args, ffi::Any* rv) {
-        Map<String, ffi::Any> ret_map;
-        for (const auto& kv : const_var_ndarray_) {
+    if (name == "get_const_var_tensor") {
+      return ffi::Function([_self, this](ffi::PackedArgs args, ffi::Any* rv) {
+        ffi::Map<ffi::String, ffi::Any> ret_map;
+        for (const auto& kv : const_var_tensor_) {
           ret_map.Set(kv.first, kv.second);
         }
         *rv = ret_map;
@@ -88,33 +91,33 @@ class ConstLoaderModuleNode : public ModuleNode {
     // Run the module.
     // Normally we would only have a limited number of submodules. The runtime
     // symobl lookup overhead should be minimal.
-    ICHECK(!this->imports().empty());
-    for (Module it : this->imports()) {
-      ffi::Function pf = it.GetFunction(name);
-      if (pf != nullptr) return pf;
+    ICHECK(!this->imports_.empty());
+    for (const Any& it : this->imports_) {
+      ffi::Optional<ffi::Function> pf = it.cast<ffi::Module>()->GetFunction(name);
+      if (pf.has_value()) return pf.value();
     }
-    return ffi::Function(nullptr);
+    return std::nullopt;
   }
 
-  const char* type_key() const final { return "const_loader"; }
+  const char* kind() const final { return "const_loader"; }
 
   /*! \brief Get the property of the runtime module .*/
-  int GetPropertyMask() const final { return ModulePropertyMask::kBinarySerializable; };
+  int GetPropertyMask() const final { return ffi::Module::kBinarySerializable; };
 
   /*!
    * \brief Get the list of constants that is required by the given module.
    * \param symbol The symbol that is being queried.
-   * \return The list of needed NDArray.
+   * \return The list of needed Tensor.
    */
-  Array<NDArray> GetRequiredConstants(const std::string& symbol) {
-    Array<NDArray> ret;
+  ffi::Array<Tensor> GetRequiredConstants(const std::string& symbol) {
+    ffi::Array<Tensor> ret;
     ICHECK_GT(const_vars_by_symbol_.count(symbol), 0U)
         << "No constants known for function '" << symbol << "'";
     std::vector<std::string> vars = const_vars_by_symbol_[symbol];
     for (const auto& var : vars) {
-      ICHECK_GT(const_var_ndarray_.count(var), 0U)
+      ICHECK_GT(const_var_tensor_.count(var), 0U)
           << "No such constant variable '" << var << "' for function '" << symbol << "'";
-      ret.push_back(const_var_ndarray_[var]);
+      ret.push_back(const_var_tensor_[var]);
     }
     return ret;
   }
@@ -133,15 +136,14 @@ class ConstLoaderModuleNode : public ModuleNode {
    *  found module accordingly by passing the needed constants into it.
    */
   void InitSubModule(const std::string& symbol) {
-    ffi::Function init(nullptr);
-    for (Module it : this->imports()) {
+    for (const Any& it : this->imports_) {
       // Get the initialization function from the imported modules.
       std::string init_name = "__init_" + symbol;
-      init = it.GetFunction(init_name, false);
-      if (init != nullptr) {
+      ffi::Optional<ffi::Function> init = it.cast<ffi::Module>()->GetFunction(init_name, false);
+      if (init.has_value()) {
         auto md = GetRequiredConstants(symbol);
         // Initialize the module with constants.
-        int ret = init(md).cast<int>();
+        int ret = (*init)(md).cast<int>();
         // Report the error if initialization is failed.
         ICHECK_EQ(ret, 0);
         break;
@@ -149,22 +151,26 @@ class ConstLoaderModuleNode : public ModuleNode {
     }
   }
 
-  void SaveToBinary(dmlc::Stream* stream) final {
+  ffi::Bytes SaveToBytes() const final {
+    std::string bytes_buffer;
+    dmlc::MemoryStringStream ms(&bytes_buffer);
+    dmlc::Stream* stream = &ms;
+
     std::vector<std::string> variables;
-    std::vector<NDArray> const_var_ndarray;
-    for (const auto& it : const_var_ndarray_) {
-      String var_name = it.first;
+    std::vector<Tensor> const_var_tensor;
+    for (const auto& it : const_var_tensor_) {
+      ffi::String var_name = it.first;
       variables.push_back(var_name);
-      const_var_ndarray.push_back(it.second);
+      const_var_tensor.push_back(it.second);
     }
 
     // Save all variables in the function.
     stream->Write(variables);
     // Save all constant data.
-    uint64_t sz = static_cast<uint64_t>(const_var_ndarray.size());
+    uint64_t sz = static_cast<uint64_t>(const_var_tensor.size());
     stream->Write(sz);
     for (uint64_t i = 0; i < sz; i++) {
-      const_var_ndarray[i].Save(stream);
+      const_var_tensor[i].Save(stream);
     }
 
     // Save the symbol to list of required constant variables mapping
@@ -181,10 +187,12 @@ class ConstLoaderModuleNode : public ModuleNode {
     for (uint64_t i = 0; i < sz; i++) {
       stream->Write(const_vars[i]);
     }
+    return ffi::Bytes(bytes_buffer);
   }
 
-  static Module LoadFromBinary(void* strm) {
-    dmlc::Stream* stream = static_cast<dmlc::Stream*>(strm);
+  static ffi::Module LoadFromBytes(const ffi::Bytes& bytes) {
+    dmlc::MemoryFixedSizeStream ms(const_cast<char*>(bytes.data()), bytes.size());
+    dmlc::Stream* stream = &ms;
 
     // Load the variables.
     std::vector<std::string> variables;
@@ -194,17 +202,17 @@ class ConstLoaderModuleNode : public ModuleNode {
     ICHECK_EQ(static_cast<size_t>(sz), variables.size())
         << "The number of variables and ndarray counts must match";
     // Load the list of ndarray.
-    std::vector<NDArray> arrays;
+    std::vector<Tensor> arrays;
     for (uint64_t i = 0; i < sz; i++) {
-      NDArray temp;
+      Tensor temp;
       temp.Load(stream);
       arrays.push_back(temp);
     }
 
-    std::unordered_map<std::string, NDArray> const_var_ndarray;
+    std::unordered_map<std::string, Tensor> const_var_tensor;
     for (uint64_t i = 0; i < sz; i++) {
-      ICHECK_EQ(const_var_ndarray.count(variables[i]), 0U);
-      const_var_ndarray[variables[i]] = arrays[i];
+      ICHECK_EQ(const_var_tensor.count(variables[i]), 0U);
+      const_var_tensor[variables[i]] = arrays[i];
     }
 
     // Load the symbol to list of required constant variables mapping
@@ -224,8 +232,8 @@ class ConstLoaderModuleNode : public ModuleNode {
       const_vars_by_symbol[symbols[i]] = const_vars[i];
     }
 
-    auto n = make_object<ConstLoaderModuleNode>(const_var_ndarray, const_vars_by_symbol);
-    return Module(n);
+    auto n = ffi::make_object<ConstLoaderModuleObj>(const_var_tensor, const_vars_by_symbol);
+    return ffi::Module(n);
   }
 
  private:
@@ -234,21 +242,24 @@ class ConstLoaderModuleNode : public ModuleNode {
    * modules using execution engine.
    */
   std::unordered_map<std::string, bool> initialized_;
-  /*! \brief Variable name to NDArray mapping. */
-  std::unordered_map<std::string, NDArray> const_var_ndarray_;
+  /*! \brief Variable name to Tensor mapping. */
+  std::unordered_map<std::string, Tensor> const_var_tensor_;
   /*! \brief Symbol name to required constant variables mapping. */
   std::unordered_map<std::string, std::vector<std::string>> const_vars_by_symbol_;
 };
 
-Module ConstLoaderModuleCreate(
-    const std::unordered_map<std::string, NDArray>& const_var_ndarray,
+ffi::Module ConstLoaderModuleCreate(
+    const std::unordered_map<std::string, Tensor>& const_var_tensor,
     const std::unordered_map<std::string, std::vector<std::string>>& const_vars_by_symbol) {
-  auto n = make_object<ConstLoaderModuleNode>(const_var_ndarray, const_vars_by_symbol);
-  return Module(n);
+  auto n = ffi::make_object<ConstLoaderModuleObj>(const_var_tensor, const_vars_by_symbol);
+  return ffi::Module(n);
 }
 
-TVM_FFI_REGISTER_GLOBAL("runtime.module.loadbinary_const_loader")
-    .set_body_typed(ConstLoaderModuleNode::LoadFromBinary);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("ffi.Module.load_from_bytes.const_loader",
+                        ConstLoaderModuleObj::LoadFromBytes);
+}
 
 }  // namespace runtime
 }  // namespace tvm
