@@ -21,6 +21,7 @@
  * \file Use external cblas library call.
  */
 #include <tvm/ffi/function.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/runtime/data_type.h>
 #include <tvm/runtime/logging.h>
 
@@ -123,37 +124,39 @@ struct CblasDgemmBatchIterativeOp {
 };
 
 // matrix multiplication for row major
-TVM_FFI_REGISTER_GLOBAL("tvm.contrib.cblas.matmul")
-    .set_body_packed([](ffi::PackedArgs args, ffi::Any* ret) {
-      auto A = args[0].cast<DLTensor*>();
-      ICHECK(TypeMatch(A->dtype, kDLFloat, 32) || TypeMatch(A->dtype, kDLFloat, 64));
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def_packed("tvm.contrib.cblas.matmul",
+                  [](ffi::PackedArgs args, ffi::Any* ret) {
+                    auto A = args[0].cast<DLTensor*>();
+                    ICHECK(TypeMatch(A->dtype, kDLFloat, 32) || TypeMatch(A->dtype, kDLFloat, 64));
 
-      if (TypeMatch(A->dtype, kDLFloat, 32))
-        CallGemm(args, ret, CblasSgemmOp());
-      else
-        CallGemm(args, ret, CblasDgemmOp());
-    });
-
-TVM_FFI_REGISTER_GLOBAL("tvm.contrib.cblas.batch_matmul")
-    .set_body_packed([](ffi::PackedArgs args, ffi::Any* ret) {
-      auto A = args[0].cast<DLTensor*>();
-      ICHECK(TypeMatch(A->dtype, kDLFloat, 32) || TypeMatch(A->dtype, kDLFloat, 64));
-      if (TypeMatch(A->dtype, kDLFloat, 32)) {
-        CallBatchGemm(args, ret, CblasSgemmBatchOp());
-      } else {
-        CallBatchGemm(args, ret, CblasDgemmBatchOp());
-      }
-    });
-
-TVM_FFI_REGISTER_GLOBAL("tvm.contrib.cblas.batch_matmul_iterative")
-    .set_body_packed([](ffi::PackedArgs args, ffi::Any* ret) {
-      auto A = args[0].cast<DLTensor*>();
-      ICHECK(TypeMatch(A->dtype, kDLFloat, 32) || TypeMatch(A->dtype, kDLFloat, 64));
-      if (TypeMatch(A->dtype, kDLFloat, 32)) {
-        CallBatchGemm(args, ret, CblasSgemmBatchIterativeOp());
-      } else {
-        CallBatchGemm(args, ret, CblasDgemmBatchIterativeOp());
-      }
-    });
+                    if (TypeMatch(A->dtype, kDLFloat, 32))
+                      CallGemm(args, ret, CblasSgemmOp());
+                    else
+                      CallGemm(args, ret, CblasDgemmOp());
+                  })
+      .def_packed("tvm.contrib.cblas.batch_matmul",
+                  [](ffi::PackedArgs args, ffi::Any* ret) {
+                    auto A = args[0].cast<DLTensor*>();
+                    ICHECK(TypeMatch(A->dtype, kDLFloat, 32) || TypeMatch(A->dtype, kDLFloat, 64));
+                    if (TypeMatch(A->dtype, kDLFloat, 32)) {
+                      CallBatchGemm(args, ret, CblasSgemmBatchOp());
+                    } else {
+                      CallBatchGemm(args, ret, CblasDgemmBatchOp());
+                    }
+                  })
+      .def_packed("tvm.contrib.cblas.batch_matmul_iterative",
+                  [](ffi::PackedArgs args, ffi::Any* ret) {
+                    auto A = args[0].cast<DLTensor*>();
+                    ICHECK(TypeMatch(A->dtype, kDLFloat, 32) || TypeMatch(A->dtype, kDLFloat, 64));
+                    if (TypeMatch(A->dtype, kDLFloat, 32)) {
+                      CallBatchGemm(args, ret, CblasSgemmBatchIterativeOp());
+                    } else {
+                      CallBatchGemm(args, ret, CblasDgemmBatchIterativeOp());
+                    }
+                  });
+}
 }  // namespace contrib
 }  // namespace tvm
