@@ -22,6 +22,7 @@
  * \brief Use external cudnn softmax function
  */
 #include <tvm/ffi/function.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/runtime/device_api.h>
 
 #include "cudnn_utils.h"
@@ -39,8 +40,9 @@ void softmax_impl(cudnnSoftmaxAlgorithm_t alg, ffi::PackedArgs args, ffi::Any* r
   int64_t* shape = x->shape;
   if (axis < 0) axis += ndim;
   ICHECK(axis >= 0 && axis < ndim);
-
-  CuDNNThreadEntry* entry_ptr = CuDNNThreadEntry::ThreadLocal();
+  int device_id;
+  CUDA_CALL(cudaGetDevice(&device_id));
+  CuDNNThreadEntry* entry_ptr = CuDNNThreadEntry::ThreadLocal(DLDevice{kDLCUDA, device_id});
   entry_ptr->softmax_entry.data_type = CuDNNDataType::DLTypeToCuDNNType(x->dtype);
 
   // Set mode and shape descriptor
@@ -77,15 +79,17 @@ void softmax_impl(cudnnSoftmaxAlgorithm_t alg, ffi::PackedArgs args, ffi::Any* r
                                  entry_ptr->softmax_entry.shape_desc, y->data));
 }
 
-TVM_FFI_REGISTER_GLOBAL("tvm.contrib.cudnn.softmax.forward")
-    .set_body_packed([](ffi::PackedArgs args, ffi::Any* ret) {
-      softmax_impl(CUDNN_SOFTMAX_ACCURATE, args, ret);
-    });
-
-TVM_FFI_REGISTER_GLOBAL("tvm.contrib.cudnn.log_softmax.forward")
-    .set_body_packed([](ffi::PackedArgs args, ffi::Any* ret) {
-      softmax_impl(CUDNN_SOFTMAX_LOG, args, ret);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def_packed("tvm.contrib.cudnn.softmax.forward",
+                  [](ffi::PackedArgs args, ffi::Any* ret) {
+                    softmax_impl(CUDNN_SOFTMAX_ACCURATE, args, ret);
+                  })
+      .def_packed("tvm.contrib.cudnn.log_softmax.forward", [](ffi::PackedArgs args, ffi::Any* ret) {
+        softmax_impl(CUDNN_SOFTMAX_LOG, args, ret);
+      });
+}
 
 }  // namespace contrib
 }  // namespace tvm
