@@ -24,8 +24,8 @@
 #include "hexagon_module.h"
 
 #include <dmlc/memory_io.h>
+#include <tvm/ffi/extra/module.h>
 #include <tvm/ffi/function.h>
-#include <tvm/runtime/module.h>
 
 #include <string>
 #include <utility>
@@ -42,12 +42,11 @@ HexagonModuleNode::HexagonModuleNode(std::string data, std::string fmt,
                                      std::string bc_str)
     : data_(data), fmt_(fmt), fmap_(fmap), asm_(asm_str), obj_(obj_str), ir_(ir_str), bc_(bc_str) {}
 
-ffi::Function HexagonModuleNode::GetFunction(const String& name,
-                                             const ObjectPtr<Object>& sptr_to_self) {
+ffi::Optional<ffi::Function> HexagonModuleNode::GetFunction(const ffi::String& name) {
   LOG(FATAL) << "HexagonModuleNode::GetFunction is not implemented.";
 }
 
-String HexagonModuleNode::GetSource(const String& format) {
+ffi::String HexagonModuleNode::InspectSource(const ffi::String& format) const {
   if (format == "s" || format == "asm") {
     return asm_;
   }
@@ -57,7 +56,7 @@ String HexagonModuleNode::GetSource(const String& format) {
   return "";
 }
 
-void HexagonModuleNode::SaveToFile(const String& file_name, const String& format) {
+void HexagonModuleNode::WriteToFile(const ffi::String& file_name, const ffi::String& format) const {
   std::string fmt = runtime::GetFileFormat(file_name, format);
   if (fmt == "so" || fmt == "dll" || fmt == "hexagon") {
     std::string meta_file = GetMetaFilePath(file_name);
@@ -80,17 +79,22 @@ void HexagonModuleNode::SaveToFile(const String& file_name, const String& format
   }
 }
 
-void HexagonModuleNode::SaveToBinary(dmlc::Stream* stream) {
+ffi::Bytes HexagonModuleNode::SaveToBytes() const {
+  std::string buffer;
+  dmlc::MemoryStringStream ms(&buffer);
+  dmlc::Stream* stream = &ms;
   stream->Write(fmt_);
   stream->Write(fmap_);
   stream->Write(data_);
+  return ffi::Bytes(buffer);
 }
 
-Module HexagonModuleCreate(std::string data, std::string fmt,
-                           std::unordered_map<std::string, FunctionInfo> fmap, std::string asm_str,
-                           std::string obj_str, std::string ir_str, std::string bc_str) {
-  auto n = make_object<HexagonModuleNode>(data, fmt, fmap, asm_str, obj_str, ir_str, bc_str);
-  return Module(n);
+ffi::Module HexagonModuleCreate(std::string data, std::string fmt,
+                                std::unordered_map<std::string, FunctionInfo> fmap,
+                                std::string asm_str, std::string obj_str, std::string ir_str,
+                                std::string bc_str) {
+  auto n = ffi::make_object<HexagonModuleNode>(data, fmt, fmap, asm_str, obj_str, ir_str, bc_str);
+  return ffi::Module(n);
 }
 
 }  // namespace runtime
