@@ -28,6 +28,7 @@ extern "C" {
 #include <dlfcn.h>
 #include <stdlib.h>
 #include <tvm/ffi/function.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/runtime/object.h>
 
 #include <algorithm>
@@ -35,7 +36,6 @@ extern "C" {
 #include <memory>
 #include <string>
 
-#include "../../../library_module.h"
 #include "../../../minrpc/minrpc_server.h"
 #include "../../hexagon/hexagon_common.h"
 #include "../../hexagon/hexagon_device_api.h"
@@ -328,24 +328,28 @@ __attribute__((weak)) void _Get_eh_data() {}
 __attribute__((weak)) void _Parse_fde_instr() {}
 }
 
-TVM_FFI_REGISTER_GLOBAL("tvm.hexagon.load_module")
-    .set_body_packed([](tvm::ffi::PackedArgs args, tvm::ffi::Any* rv) {
-      auto soname = args[0].cast<std::string>();
-      tvm::ObjectPtr<tvm::runtime::Library> n = tvm::runtime::CreateDSOLibraryObject(soname);
-      *rv = CreateModuleFromLibrary(n);
-    });
-
-TVM_FFI_REGISTER_GLOBAL("tvm.hexagon.get_profile_output")
-    .set_body_packed([](tvm::ffi::PackedArgs args, tvm::ffi::Any* rv) {
-      auto profiling_mode = args[0].cast<std::string>();
-      auto out_file = args[1].cast<std::string>();
-      if (profiling_mode.compare("lwp") == 0) {
-        *rv = WriteLWPOutput(out_file);
-      } else {
-        HEXAGON_PRINT(ERROR, "ERROR: Unsupported profiling mode: %s", profiling_mode.c_str());
-        *rv = false;
-      }
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def_packed("tvm.hexagon.load_module",
+                  [](tvm::ffi::PackedArgs args, tvm::ffi::Any* rv) {
+                    auto soname = args[0].cast<std::string>();
+                    auto floader =
+                        tvm::ffi::Function::GetGlobalRequired("ffi.Module.load_from_file.so");
+                    *rv = floader(soname, "so");
+                  })
+      .def_packed(
+          "tvm.hexagon.get_profile_output", [](tvm::ffi::PackedArgs args, tvm::ffi::Any* rv) {
+            auto profiling_mode = args[0].cast<std::string>();
+            auto out_file = args[1].cast<std::string>();
+            if (profiling_mode.compare("lwp") == 0) {
+              *rv = WriteLWPOutput(out_file);
+            } else {
+              HEXAGON_PRINT(ERROR, "ERROR: Unsupported profiling mode: %s", profiling_mode.c_str());
+              *rv = false;
+            }
+          });
+}
 
 void SaveBinaryToFile(const std::string& file_name, const std::string& data) {
   std::ofstream fs(file_name, std::ios::out | std::ios::binary);
@@ -353,9 +357,12 @@ void SaveBinaryToFile(const std::string& file_name, const std::string& data) {
   fs.write(&data[0], data.length());
 }
 
-TVM_FFI_REGISTER_GLOBAL("tvm.rpc.server.upload")
-    .set_body_packed([](tvm::ffi::PackedArgs args, tvm::ffi::Any* rv) {
-      auto file_name = args[0].cast<std::string>();
-      auto data = args[1].cast<std::string>();
-      SaveBinaryToFile(file_name, data);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def_packed("tvm.rpc.server.upload",
+                               [](tvm::ffi::PackedArgs args, tvm::ffi::Any* rv) {
+                                 auto file_name = args[0].cast<std::string>();
+                                 auto data = args[1].cast<std::string>();
+                                 SaveBinaryToFile(file_name, data);
+                               });
+}
