@@ -19,6 +19,7 @@
 
 #include <HexagonWrapper.h>
 #include <tvm/ffi/function.h>
+#include <tvm/ffi/reflection/registry.h>
 // POSIX includes
 #include <dirent.h>
 #include <unistd.h>
@@ -1369,19 +1370,22 @@ std::optional<HEXAPI_Nullptr> SimulatorRPCChannel::to_nullptr(const detail::Mayb
       .Default(std::nullopt);
 }
 
-TVM_FFI_REGISTER_GLOBAL("tvm.contrib.hexagon.create_hexagon_session")
-    .set_body_packed([](ffi::PackedArgs args, ffi::Any* rv) {
-      ICHECK(args.size() >= 4) << args.size() << " is less than 4";
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def_packed(
+      "tvm.contrib.hexagon.create_hexagon_session", [](ffi::PackedArgs args, ffi::Any* rv) {
+        ICHECK(args.size() >= 4) << args.size() << " is less than 4";
 
-      auto session_name = args[0].cast<std::string>();
-      int stack_size = args[1].cast<int>();
-      auto sim_args = args[2].cast<std::string>();
-      auto channel = std::make_unique<SimulatorRPCChannel>(stack_size, sim_args);
-      std::shared_ptr<RPCEndpoint> endpoint =
-          RPCEndpoint::Create(std::move(channel), session_name, "", nullptr);
-      std::shared_ptr<RPCSession> session = CreateClientSession(endpoint);
-      *rv = CreateRPCSessionModule(session);
-    });
+        auto session_name = args[0].cast<std::string>();
+        int stack_size = args[1].cast<int>();
+        auto sim_args = args[2].cast<std::string>();
+        auto channel = std::make_unique<SimulatorRPCChannel>(stack_size, sim_args);
+        std::shared_ptr<RPCEndpoint> endpoint =
+            RPCEndpoint::Create(std::move(channel), session_name, "", nullptr);
+        std::shared_ptr<RPCSession> session = CreateClientSession(endpoint);
+        *rv = CreateRPCSessionModule(session);
+      });
+}
 
 }  // namespace hexagon
 }  // namespace runtime
