@@ -24,6 +24,8 @@
 #ifndef TVM_RUNTIME_MINRPC_RPC_REFERENCE_H_
 #define TVM_RUNTIME_MINRPC_RPC_REFERENCE_H_
 
+#include <tvm/ffi/container/tensor.h>
+
 namespace tvm {
 namespace ffi {
 // Forward declare TVM Object to use `Object*` in RPC protocol.
@@ -72,7 +74,7 @@ enum class RPCCode : int {
 enum class RPCServerStatus : int {
   kSuccess = 0,
   kInvalidTypeCodeObject,
-  kInvalidTypeCodeNDArray,
+  kInvalidTypeCodeTensor,
   kInvalidDLTensorFieldStride,
   kInvalidDLTensorFieldByteOffset,
   kUnknownTypeIndex,
@@ -144,8 +146,8 @@ inline const char* RPCServerStatusToString(RPCServerStatus status) {
       return "kSuccess";
     case RPCServerStatus::kInvalidTypeCodeObject:
       return "kInvalidTypeCodeObject";
-    case RPCServerStatus::kInvalidTypeCodeNDArray:
-      return "kInvalidTypeCodeNDArray";
+    case RPCServerStatus::kInvalidTypeCodeTensor:
+      return "kInvalidTypeCodeTensor";
     case RPCServerStatus::kInvalidDLTensorFieldStride:
       return "kInvalidDLTensorFieldStride";
     case RPCServerStatus::kInvalidDLTensorFieldByteOffset: {
@@ -200,7 +202,7 @@ struct RPCReference {
       num_bytes_ += sizeof(T) * num;
     }
 
-    void WriteObject(ffi::Object* obj) { num_bytes_ += channel_->GetObjectBytes(obj); }
+    void WriteFFIAny(const TVMFFIAny* obj) { num_bytes_ += channel_->GetFFIAnyProtocolBytes(obj); }
 
     void ThrowError(RPCServerStatus status) { channel_->ThrowError(status); }
 
@@ -245,7 +247,7 @@ struct RPCReference {
   static void SendDLTensor(TChannelPtr channel, DLTensor* arr) {
     DLDevice dev;
     uint64_t data;
-    // When we return NDArray, we directly return
+    // When we return Tensor, we directly return
     // the space and the context
     // The client will be further wrapping
     dev = arr->device;
@@ -255,7 +257,7 @@ struct RPCReference {
     channel->Write(arr->ndim);
     channel->Write(arr->dtype);
     channel->WriteArray(arr->shape, arr->ndim);
-    if (arr->strides != nullptr) {
+    if (!ffi::IsContiguous(*arr)) {
       channel->ThrowError(RPCServerStatus::kInvalidDLTensorFieldStride);
     }
     channel->Write(arr->byte_offset);
@@ -349,8 +351,8 @@ struct RPCReference {
           break;
         }
 
-        case ffi::TypeIndex::kTVMFFINDArray: {
-          channel->ThrowError(RPCServerStatus::kInvalidTypeCodeNDArray);
+        case ffi::TypeIndex::kTVMFFITensor: {
+          channel->ThrowError(RPCServerStatus::kInvalidTypeCodeTensor);
           break;
         }
         case ffi::TypeIndex::kTVMFFIDLTensorPtr: {
@@ -373,11 +375,7 @@ struct RPCReference {
           break;
         }
         default: {
-          if (type_index >= ffi::TypeIndex::kTVMFFIStaticObjectBegin) {
-            channel->WriteObject(reinterpret_cast<ffi::Object*>(packed_args[i].v_obj));
-          } else {
-            channel->ThrowError(RPCServerStatus::kUnknownTypeIndex);
-          }
+          channel->WriteFFIAny(&(packed_args[i]));
           break;
         }
       }
@@ -412,6 +410,9 @@ struct RPCReference {
       int32_t type_index;
       channel->Read(&type_index);
       packed_args[i].type_index = type_index;
+      packed_args[i].zero_padding = 0;
+      // clear to ensure compact for 32 bit platform
+      packed_args[i].v_int64 = 0;
       switch (type_index) {
         case ffi::TypeIndex::kTVMFFINone: {
           break;
@@ -471,8 +472,10 @@ struct RPCReference {
           break;
         }
         default: {
-          if (type_index >= ffi::TypeIndex::kTVMFFIStaticObjectBegin) {
-            channel->ReadObject(&(packed_args[i]));
+          if (type_index >= ffi::TypeIndex::kTVMFFIStaticObjectBegin ||
+              type_index == ffi::TypeIndex::kTVMFFISmallStr ||
+              type_index == ffi::TypeIndex::kTVMFFISmallBytes) {
+            channel->ReadFFIAny(&(packed_args[i]));
           } else {
             channel->ThrowError(RPCServerStatus::kUnknownTypeIndex);
           }
