@@ -29,8 +29,8 @@
 #include <tvm/runtime/device_api.h>
 #include <tvm/runtime/logging.h>
 #include <tvm/runtime/memory/memory_manager.h>
-#include <tvm/runtime/ndarray.h>
 #include <tvm/runtime/profiling.h>
+#include <tvm/runtime/tensor.h>
 
 /* There are many OpenCL platforms that do not yet support OpenCL 2.0,
  * hence we use 1.2 APIs, some of which are now deprecated.  In order
@@ -341,7 +341,7 @@ class OpenCLWorkspace : public DeviceAPI {
   }
 
   void* AllocDataSpaceView(Device dev, void* data, ffi::Shape shape, DLDataType dtype,
-                           Optional<String> mem_scope = std::nullopt);
+                           ffi::Optional<ffi::String> mem_scope = std::nullopt);
   void FreeDataSpaceView(Device dev, void* ptr);
 
   cl_device_id GetCLDeviceID(int device_id);
@@ -350,22 +350,23 @@ class OpenCLWorkspace : public DeviceAPI {
   void GetAttr(Device dev, DeviceAttrKind kind, ffi::Any* rv) final;
   void* AllocDataSpace(Device dev, size_t size, size_t alignment, DLDataType type_hint) final;
   void* AllocDataSpace(Device dev, int ndim, const int64_t* shape, DLDataType dtype,
-                       Optional<String> mem_scope = std::nullopt) final;
+                       ffi::Optional<ffi::String> mem_scope = std::nullopt) final;
   void* AllocDataSpace(Device dev, size_t width, size_t height, DLDataType type_hint,
-                       Optional<String> mem_scope = std::nullopt);
-  void* GetNativePtr(const tvm::runtime::NDArray& narr);
-  void SetNativePtr(const tvm::runtime::NDArray& narr, void* host_ptr, size_t buf_size);
+                       ffi::Optional<ffi::String> mem_scope = std::nullopt);
+  void* GetNativePtr(const tvm::runtime::Tensor& narr);
+  void SetNativePtr(const tvm::runtime::Tensor& narr, void* host_ptr, size_t buf_size);
   void SetPerfHint(Device dev, cl_uint perf_hint);
   void FreeDataSpace(Device dev, void* ptr) final;
   void StreamSync(Device dev, TVMStreamHandle stream) final;
   void* AllocWorkspace(Device dev, size_t size, DLDataType type_hint) final;
   void FreeWorkspace(Device dev, void* data) final;
-  size_t GetDataSize(const DLTensor& arr, Optional<String> mem_scope = std::nullopt) final;
+  size_t GetDataSize(const DLTensor& arr,
+                     ffi::Optional<ffi::String> mem_scope = std::nullopt) final;
 
   // cl_mem alloc utils
   void* AllocCLBuffer(Device dev, size_t size, size_t alignment, DLDataType type_hint);
   void* AllocCLImage(Device dev, void* back_buffer, size_t width, size_t height, size_t row_pitch,
-                     DLDataType type_hint, Optional<String> mem_scope);
+                     DLDataType type_hint, ffi::Optional<ffi::String> mem_scope);
 
   /*!
    * \brief Get the thread local ThreadEntry
@@ -436,9 +437,10 @@ struct BufferDescriptor {
     kImage2DNHWC,
   };
   BufferDescriptor() = default;
-  explicit BufferDescriptor(Optional<String> scope) : layout(MemoryLayoutFromScope(scope)) {}
-  static MemoryLayout MemoryLayoutFromScope(Optional<String> mem_scope);
-  static String ScopeFromMemoryLayout(MemoryLayout mem_scope);
+  explicit BufferDescriptor(ffi::Optional<ffi::String> scope)
+      : layout(MemoryLayoutFromScope(scope)) {}
+  static MemoryLayout MemoryLayoutFromScope(ffi::Optional<ffi::String> mem_scope);
+  static ffi::String ScopeFromMemoryLayout(MemoryLayout mem_scope);
 
   /* clBuffer object */
   // buffer should be the first element here
@@ -456,7 +458,7 @@ struct BufferDescriptor {
 // To make the call thread-safe, we create a thread-local kernel table
 // and lazily install new kernels into the kernel table when the kernel is called.
 // The kernels are recycled when the module get destructed.
-class OpenCLModuleNodeBase : public ModuleNode {
+class OpenCLModuleNodeBase : public ffi::ModuleObj {
  public:
   // Kernel table reference entry.
   struct KTRefEntry {
@@ -472,14 +474,14 @@ class OpenCLModuleNodeBase : public ModuleNode {
    */
   virtual cl::OpenCLWorkspace* GetGlobalWorkspace();
 
-  const char* type_key() const final { return workspace_->type_key.c_str(); }
+  const char* kind() const final { return workspace_->type_key.c_str(); }
 
   /*! \brief Get the property of the runtime module .*/
   int GetPropertyMask() const final {
-    return ModulePropertyMask::kBinarySerializable | ModulePropertyMask::kRunnable;
+    return ffi::Module::kBinarySerializable | ffi::Module::kRunnable;
   }
 
-  ffi::Function GetFunction(const String& name, const ObjectPtr<Object>& sptr_to_self) override;
+  ffi::Optional<ffi::Function> GetFunction(const ffi::String& name) override;
 
   // Initialize the programs
   virtual void Init() = 0;
@@ -509,14 +511,14 @@ class OpenCLModuleNode : public OpenCLModuleNodeBase {
                             std::unordered_map<std::string, FunctionInfo> fmap, std::string source)
       : OpenCLModuleNodeBase(fmap), data_(data), fmt_(fmt), source_(source) {}
 
-  ffi::Function GetFunction(const String& name, const ObjectPtr<Object>& sptr_to_self) final;
+  ffi::Optional<ffi::Function> GetFunction(const ffi::String& name) final;
   // Return true if OpenCL program for the requested function and device was created
   bool IsProgramCreated(const std::string& func_name, int device_id);
-  void SaveToFile(const String& file_name, const String& format) final;
-  void SaveToBinary(dmlc::Stream* stream) final;
+  void WriteToFile(const ffi::String& file_name, const ffi::String& format) const final;
+  ffi::Bytes SaveToBytes() const final;
   void SetPreCompiledPrograms(const std::string& bytes);
   std::string GetPreCompiledPrograms();
-  String GetSource(const String& format) final;
+  ffi::String InspectSource(const ffi::String& format) const final;
 
   // Initialize the programs
   void Init() override;
@@ -588,10 +590,9 @@ class OpenCLTimerNode : public TimerNode {
   OpenCLTimerNode() {}
   explicit OpenCLTimerNode(Device dev) : dev_(dev) {}
 
-  static constexpr const char* _type_key = "runtime.opencl.OpenCLTimerNode";
   static size_t count_timer_execs;
   static std::vector<size_t> event_start_idxs;
-  TVM_DECLARE_FINAL_OBJECT_INFO(OpenCLTimerNode, TimerNode);
+  TVM_FFI_DECLARE_OBJECT_INFO_FINAL("runtime.opencl.OpenCLTimerNode", OpenCLTimerNode, TimerNode);
 
  private:
   int64_t duration;
