@@ -23,6 +23,7 @@
 #include <dmlc/parameter.h>
 #include <dmlc/thread_local.h>
 #include <tvm/ffi/function.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/runtime/profiling.h>
 
 #include <sstream>
@@ -75,8 +76,8 @@ ImageInfo GetImageInfo(const cl::BufferDescriptor* desc, const DLTensor* tensor)
 }
 
 cl::BufferDescriptor::MemoryLayout cl::BufferDescriptor::MemoryLayoutFromScope(
-    Optional<String> mem_scope) {
-  if (!mem_scope.defined()) {
+    ffi::Optional<ffi::String> mem_scope) {
+  if (!mem_scope.has_value()) {
     return cl::BufferDescriptor::MemoryLayout::kBuffer1D;
   } else if (mem_scope.value() == "global.texture") {
     return cl::BufferDescriptor::MemoryLayout::kImage2DActivation;
@@ -88,7 +89,7 @@ cl::BufferDescriptor::MemoryLayout cl::BufferDescriptor::MemoryLayoutFromScope(
   LOG(FATAL) << "No memory layout defined for memory of scope: " << mem_scope.value();
 }
 
-String cl::BufferDescriptor::ScopeFromMemoryLayout(cl::BufferDescriptor::MemoryLayout layout) {
+ffi::String cl::BufferDescriptor::ScopeFromMemoryLayout(cl::BufferDescriptor::MemoryLayout layout) {
   switch (layout) {
     case cl::BufferDescriptor::MemoryLayout::kBuffer1D:
       return "global";
@@ -260,7 +261,7 @@ void* OpenCLWorkspace::AllocDataSpace(Device dev, size_t size, size_t alignment,
 }
 
 void* OpenCLWorkspace::AllocDataSpace(Device dev, size_t width, size_t height, DLDataType type_hint,
-                                      Optional<String> mem_scope) {
+                                      ffi::Optional<ffi::String> mem_scope) {
   // Texture allocation given width and height
   cl_uint row_align = GetImageAlignment(dev.device_id);
   size_t pixel_size = (type_hint.bits * type_hint.lanes + 7) / 8;
@@ -276,16 +277,16 @@ void* OpenCLWorkspace::AllocDataSpace(Device dev, size_t width, size_t height, D
     back_buffer->mbuf = buf;
   }
 
-  if (!mem_scope.defined()) {
-    mem_scope = String("global.texture");
+  if (!mem_scope.has_value()) {
+    mem_scope = ffi::String("global.texture");
   }
   return AllocCLImage(dev, back_buffer, width, height, row_pitch, type_hint, mem_scope);
 }
 
 void* OpenCLWorkspace::AllocDataSpace(Device dev, int ndim, const int64_t* shape, DLDataType dtype,
-                                      Optional<String> mem_scope) {
+                                      ffi::Optional<ffi::String> mem_scope) {
   this->Init();
-  if (!mem_scope.defined() || mem_scope.value().empty() || mem_scope.value() == "global") {
+  if (!mem_scope.has_value() || (*mem_scope).empty() || (*mem_scope) == "global") {
     size_t size = GetMemObjectSize(dev, ndim, shape, dtype);
     cl::BufferDescriptor* ret_buffer = nullptr;
     auto buf = MemoryManager::GetOrCreateAllocator(dev, AllocatorType::kPooled)
@@ -320,7 +321,7 @@ void* OpenCLWorkspace::AllocCLBuffer(Device dev, size_t size, size_t alignment,
 
 void* OpenCLWorkspace::AllocCLImage(Device dev, void* back_buffer, size_t width, size_t height,
                                     size_t row_pitch, DLDataType type_hint,
-                                    Optional<String> mem_scope) {
+                                    ffi::Optional<ffi::String> mem_scope) {
   this->Init();
   ICHECK(std::string(mem_scope.value()).find("texture") != std::string::npos)
       << "Expect texture scope while creating an Image object";
@@ -347,8 +348,8 @@ void* OpenCLWorkspace::AllocCLImage(Device dev, void* back_buffer, size_t width,
   return desc;
 }
 
-size_t OpenCLWorkspace::GetDataSize(const DLTensor& arr, Optional<String> mem_scope) {
-  if (!mem_scope.defined() || mem_scope.value().empty() || mem_scope.value() == "global") {
+size_t OpenCLWorkspace::GetDataSize(const DLTensor& arr, ffi::Optional<ffi::String> mem_scope) {
+  if (!mem_scope.has_value() || (*mem_scope).empty() || (*mem_scope) == "global") {
     return DeviceAPI::GetDataSize(arr);
   }
   cl_uint row_align = GetImageAlignment(GetThreadEntry()->device.device_id);
@@ -359,13 +360,13 @@ size_t OpenCLWorkspace::GetDataSize(const DLTensor& arr, Optional<String> mem_sc
 }
 
 void* OpenCLWorkspace::AllocDataSpaceView(Device dev, void* data, ffi::Shape shape,
-                                          DLDataType dtype, Optional<String> mem_scope) {
+                                          DLDataType dtype, ffi::Optional<ffi::String> mem_scope) {
   cl::BufferDescriptor* desc = static_cast<cl::BufferDescriptor*>(data);
 
   // Fall back for devices w/o "cl_khr_image2d_from_buffer"
   if (!IsBufferToImageSupported(dev.device_id)) {
     cl::BufferDescriptor* ret_desc = desc;  // buffer -> buffer
-    if (!mem_scope.defined() || mem_scope.value().empty() || mem_scope.value() == "global") {
+    if (!mem_scope.has_value() || (*mem_scope).empty() || (*mem_scope) == "global") {
       if (desc->layout != cl::BufferDescriptor::MemoryLayout::kBuffer1D) {
         // image -> buffer
         size_t nbytes = GetMemObjectSize(dev, shape.size(), shape.data(), dtype);
@@ -388,7 +389,7 @@ void* OpenCLWorkspace::AllocDataSpaceView(Device dev, void* data, ffi::Shape sha
     return ret_desc;
   }
 
-  if (!mem_scope.defined() || mem_scope.value().empty() || mem_scope.value() == "global") {
+  if (!mem_scope.has_value() || (*mem_scope).empty() || (*mem_scope) == "global") {
     if (desc->layout == cl::BufferDescriptor::MemoryLayout::kBuffer1D) {
       //  buffer -> buffer
       return desc;
@@ -433,12 +434,12 @@ void OpenCLWorkspace::FreeDataSpaceView(Device dev, void* ptr) {
   }
 }
 
-void* OpenCLWorkspace::GetNativePtr(const tvm::runtime::NDArray& narr) {
+void* OpenCLWorkspace::GetNativePtr(const tvm::runtime::Tensor& narr) {
   cl::BufferDescriptor* desc = static_cast<cl::BufferDescriptor*>(narr.operator->()->data);
   return desc->host_ptr;
 }
 
-void OpenCLWorkspace::SetNativePtr(const tvm::runtime::NDArray& narr, void* host_ptr,
+void OpenCLWorkspace::SetNativePtr(const tvm::runtime::Tensor& narr, void* host_ptr,
                                    size_t buf_size) {
   cl::BufferDescriptor* desc = static_cast<cl::BufferDescriptor*>(narr.operator->()->data);
 
@@ -629,7 +630,7 @@ std::string GetDeviceInfo(cl_device_id pid, cl_device_info param_name) {
 }
 
 std::string GetOpenCLVersion(cl_device_id pid) {
-  // String returned is "OpenCL $MAJOR.$MINOR $VENDOR_INFO".  To
+  // ffi::String returned is "OpenCL $MAJOR.$MINOR $VENDOR_INFO".  To
   // match other implementations, we want to return "$MAJOR.$MINOR"
   std::string ret = GetDeviceInfo(pid, CL_DEVICE_VERSION);
 
@@ -760,60 +761,61 @@ void OpenCLWorkspace::Init(const std::string& type_key, const std::string& devic
   initialized_ = true;
 }
 
-TVM_FFI_REGISTER_GLOBAL("device_api.opencl.alloc_nd")
-    .set_body_packed([](ffi::PackedArgs args, ffi::Any* rv) {
-      int32_t device_type = args[0].cast<int32_t>();
-      int32_t device_id = args[1].cast<int32_t>();
-      int32_t dtype_code_hint = args[2].cast<int32_t>();
-      int32_t dtype_bits_hint = args[3].cast<int32_t>();
-      auto scope = args[4].cast<std::string>();
-      CHECK(scope.find("texture") != std::string::npos);
-      int64_t ndim = args[5].cast<int64_t>();
-      CHECK_EQ(ndim, 2);
-      int64_t* shape = static_cast<int64_t*>(args[6].cast<void*>());
-      int64_t width = shape[0];
-      int64_t height = shape[1];
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def_packed("device_api.opencl.alloc_nd",
+                  [](ffi::PackedArgs args, ffi::Any* rv) {
+                    int32_t device_type = args[0].cast<int32_t>();
+                    int32_t device_id = args[1].cast<int32_t>();
+                    int32_t dtype_code_hint = args[2].cast<int32_t>();
+                    int32_t dtype_bits_hint = args[3].cast<int32_t>();
+                    auto scope = args[4].cast<std::string>();
+                    CHECK(scope.find("texture") != std::string::npos);
+                    int64_t ndim = args[5].cast<int64_t>();
+                    CHECK_EQ(ndim, 2);
+                    int64_t* shape = static_cast<int64_t*>(args[6].cast<void*>());
+                    int64_t width = shape[0];
+                    int64_t height = shape[1];
 
-      Device dev;
-      dev.device_type = static_cast<DLDeviceType>(device_type);
-      dev.device_id = device_id;
+                    Device dev;
+                    dev.device_type = static_cast<DLDeviceType>(device_type);
+                    dev.device_id = device_id;
 
-      DLDataType type_hint;
-      type_hint.code = static_cast<decltype(type_hint.code)>(dtype_code_hint);
-      type_hint.bits = static_cast<decltype(type_hint.bits)>(dtype_bits_hint);
-      type_hint.lanes = 1;
+                    DLDataType type_hint;
+                    type_hint.code = static_cast<decltype(type_hint.code)>(dtype_code_hint);
+                    type_hint.bits = static_cast<decltype(type_hint.bits)>(dtype_bits_hint);
+                    type_hint.lanes = 1;
 
-      *rv = OpenCLWorkspace::Global()->AllocDataSpace(dev, static_cast<size_t>(width),
-                                                      static_cast<size_t>(height), type_hint,
-                                                      String("global.texture"));
-    });
+                    *rv = OpenCLWorkspace::Global()->AllocDataSpace(
+                        dev, static_cast<size_t>(width), static_cast<size_t>(height), type_hint,
+                        ffi::String("global.texture"));
+                  })
+      .def_packed("device_api.opencl.free_nd",
+                  [](ffi::PackedArgs args, ffi::Any* rv) {
+                    int32_t device_type = args[0].cast<int32_t>();
+                    int32_t device_id = args[1].cast<int32_t>();
+                    auto scope = args[2].cast<std::string>();
+                    CHECK(scope.find("texture") != std::string::npos);
+                    void* data = args[3].cast<void*>();
+                    OpenCLWorkspace* ptr = OpenCLWorkspace::Global();
+                    Device dev;
+                    dev.device_type = static_cast<DLDeviceType>(device_type);
+                    dev.device_id = device_id;
+                    ptr->FreeDataSpace(dev, data);
+                    *rv = static_cast<int32_t>(0);
+                  })
+      .def_packed("device_api.opencl", [](ffi::PackedArgs args, ffi::Any* rv) {
+        DeviceAPI* ptr = OpenCLWorkspace::Global();
+        *rv = static_cast<void*>(ptr);
+      });
+}
 
-TVM_FFI_REGISTER_GLOBAL("device_api.opencl.free_nd")
-    .set_body_packed([](ffi::PackedArgs args, ffi::Any* rv) {
-      int32_t device_type = args[0].cast<int32_t>();
-      int32_t device_id = args[1].cast<int32_t>();
-      auto scope = args[2].cast<std::string>();
-      CHECK(scope.find("texture") != std::string::npos);
-      void* data = args[3].cast<void*>();
-      OpenCLWorkspace* ptr = OpenCLWorkspace::Global();
-      Device dev;
-      dev.device_type = static_cast<DLDeviceType>(device_type);
-      dev.device_id = device_id;
-      ptr->FreeDataSpace(dev, data);
-      *rv = static_cast<int32_t>(0);
-    });
-
-TVM_FFI_REGISTER_GLOBAL("device_api.opencl")
-    .set_body_packed([](ffi::PackedArgs args, ffi::Any* rv) {
-      DeviceAPI* ptr = OpenCLWorkspace::Global();
-      *rv = static_cast<void*>(ptr);
-    });
-
-TVM_REGISTER_OBJECT_TYPE(OpenCLTimerNode);
-
-TVM_FFI_REGISTER_GLOBAL("profiling.timer.opencl").set_body_typed([](Device dev) {
-  return Timer(make_object<OpenCLTimerNode>(dev));
-});
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("profiling.timer.opencl",
+                        [](Device dev) { return Timer(ffi::make_object<OpenCLTimerNode>(dev)); });
+}
 
 class OpenCLPooledAllocator final : public memory::PooledAllocator {
  public:
@@ -861,7 +863,7 @@ class OpenCLPooledAllocator final : public memory::PooledAllocator {
       buf.size = size;
       buf.alloc_type = AllocatorType::kPooled;
       buf.data = DeviceAPI::Get(dev)->AllocDataSpace(dev, shape.size(), shape.data(), type_hint,
-                                                     String(mem_scope));
+                                                     ffi::String(mem_scope));
       if (mem_scope.find("texture") == std::string::npos) {
         // All textures are backed by buffers - don't count in total memory
         used_memory_.fetch_add(size, std::memory_order_relaxed);
@@ -885,7 +887,8 @@ class OpenCLPooledAllocator final : public memory::PooledAllocator {
   void* CreateView(const Buffer& buffer, ffi::Shape shape, DLDataType type_hint,
                    const std::string& mem_scope) final {
     OpenCLWorkspace* ws_ = OpenCLWorkspace::Global();
-    return ws_->AllocDataSpaceView(buffer.device, buffer.data, shape, type_hint, String(mem_scope));
+    return ws_->AllocDataSpaceView(buffer.device, buffer.data, shape, type_hint,
+                                   ffi::String(mem_scope));
   }
 
   void FreeView(Device dev, void* data) final {
@@ -894,11 +897,13 @@ class OpenCLPooledAllocator final : public memory::PooledAllocator {
   }
 };
 
-TVM_FFI_REGISTER_GLOBAL("DeviceAllocator.opencl")
-    .set_body_packed([](ffi::PackedArgs args, ffi::Any* rv) {
-      Allocator* alloc = new OpenCLPooledAllocator();
-      *rv = static_cast<void*>(alloc);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def_packed("DeviceAllocator.opencl", [](ffi::PackedArgs args, ffi::Any* rv) {
+    Allocator* alloc = new OpenCLPooledAllocator();
+    *rv = static_cast<void*>(alloc);
+  });
+}
 
 }  // namespace cl
 size_t OpenCLTimerNode::count_timer_execs = 0;
