@@ -44,8 +44,9 @@ namespace runtime {
  * \param fmap The map function information map of each function.
  * \param source Generated OpenCL kernels.
  */
-Module OpenCLModuleCreate(std::string data, std::string fmt,
-                          std::unordered_map<std::string, FunctionInfo> fmap, std::string source);
+ffi::Module OpenCLModuleCreate(std::string data, std::string fmt,
+                               std::unordered_map<std::string, FunctionInfo> fmap,
+                               std::string source);
 
 /*!
  * \brief Create a opencl module from SPIRV.
@@ -54,9 +55,9 @@ Module OpenCLModuleCreate(std::string data, std::string fmt,
  * \param spirv_text The concatenated text representation of SPIRV modules.
  * \param fmap The map function information map of each function.
  */
-Module OpenCLModuleCreate(const std::unordered_map<std::string, spirv::SPIRVShader>& shaders,
-                          const std::string& spirv_text,
-                          std::unordered_map<std::string, FunctionInfo> fmap);
+ffi::Module OpenCLModuleCreate(const std::unordered_map<std::string, spirv::SPIRVShader>& shaders,
+                               const std::string& spirv_text,
+                               std::unordered_map<std::string, FunctionInfo> fmap);
 }  // namespace runtime
 }  // namespace tvm
 #endif  // TVM_RUNTIME_OPENCL_OPENCL_MODULE_H_
