@@ -22,6 +22,7 @@
  * \brief Server environment of the RPC.
  */
 #include <tvm/ffi/function.h>
+#include <tvm/ffi/reflection/registry.h>
 
 #include "../file_utils.h"
 
@@ -35,27 +36,28 @@ std::string RPCGetPath(const std::string& name) {
   return (*f)(name).cast<std::string>();
 }
 
-TVM_FFI_REGISTER_GLOBAL("tvm.rpc.server.upload")
-    .set_body_packed([](ffi::PackedArgs args, ffi::Any* rv) {
-      std::string file_name = RPCGetPath(args[0].cast<std::string>());
-      auto data = args[1].cast<std::string>();
-      SaveBinaryToFile(file_name, data);
-    });
-
-TVM_FFI_REGISTER_GLOBAL("tvm.rpc.server.download")
-    .set_body_packed([](ffi::PackedArgs args, ffi::Any* rv) {
-      std::string file_name = RPCGetPath(args[0].cast<std::string>());
-      std::string data;
-      LoadBinaryFromFile(file_name, &data);
-      LOG(INFO) << "Download " << file_name << "... nbytes=" << data.size();
-      *rv = ffi::Bytes(data);
-    });
-
-TVM_FFI_REGISTER_GLOBAL("tvm.rpc.server.remove")
-    .set_body_packed([](ffi::PackedArgs args, ffi::Any* rv) {
-      std::string file_name = RPCGetPath(args[0].cast<std::string>());
-      RemoveFile(file_name);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def_packed("tvm.rpc.server.upload",
+                  [](ffi::PackedArgs args, ffi::Any* rv) {
+                    std::string file_name = RPCGetPath(args[0].cast<std::string>());
+                    auto data = args[1].cast<std::string>();
+                    SaveBinaryToFile(file_name, data);
+                  })
+      .def_packed("tvm.rpc.server.download",
+                  [](ffi::PackedArgs args, ffi::Any* rv) {
+                    std::string file_name = RPCGetPath(args[0].cast<std::string>());
+                    std::string data;
+                    LoadBinaryFromFile(file_name, &data);
+                    LOG(INFO) << "Download " << file_name << "... nbytes=" << data.size();
+                    *rv = ffi::Bytes(data);
+                  })
+      .def_packed("tvm.rpc.server.remove", [](ffi::PackedArgs args, ffi::Any* rv) {
+        std::string file_name = RPCGetPath(args[0].cast<std::string>());
+        RemoveFile(file_name);
+      });
+}
 
 }  // namespace runtime
 }  // namespace tvm
