@@ -43,7 +43,7 @@ namespace runtime {
  * \brief Returns a static library with the contents loaded from filename which exports
  * func_names with the usual packed-func calling convention.
  */
-Module LoadStaticLibrary(const std::string& filename, Array<String> func_names);
+ffi::Module LoadStaticLibrary(const std::string& filename, ffi::Array<ffi::String> func_names);
 
 }  // namespace runtime
 }  // namespace tvm
