@@ -19,12 +19,12 @@
 
 /*!
  * \file thread_storage_scope.h
- * \brief Extract launch parameters configuration from TVMArgs.
+ * \brief Extract launch parameters configuration from ffi::PackedArgs.
  */
 #ifndef TVM_RUNTIME_THREAD_STORAGE_SCOPE_H_
 #define TVM_RUNTIME_THREAD_STORAGE_SCOPE_H_
 
-#include <tvm/runtime/packed_func.h>
+#include <tvm/ffi/function.h>
 
 #include <string>
 #include <vector>
@@ -261,18 +261,20 @@ class LaunchParamConfig {
     }
   }
   // extract workload from arguments.
-  ThreadWorkLoad Extract(TVMArgs x) const {
+  ThreadWorkLoad Extract(ffi::PackedArgs args) const {
     ThreadWorkLoad w;
     std::fill(w.work_size, w.work_size + 6, 1);
+    const TVMFFIAny* raw_args = reinterpret_cast<const TVMFFIAny*>(args.data());
+
     for (size_t i = 0; i < arg_index_map_.size(); ++i) {
       // Dynamic shapes can result in 0 dim size. Guard to ensure that the dim size is at least 1.
-      size_t size = static_cast<size_t>(x.values[base_ + i].v_int64);
+      size_t size = static_cast<size_t>(raw_args[base_ + i].v_int64);
       if (size > 0) {
         w.work_size[arg_index_map_[i]] = size;
       }
     }
     if (use_dyn_shared_memory_) {
-      w.dyn_shmem_size = static_cast<size_t>(x.values[base_ + arg_index_map_.size()].v_int64);
+      w.dyn_shmem_size = static_cast<size_t>(raw_args[base_ + arg_index_map_.size()].v_int64);
     }
     return w;
   }
