@@ -19,6 +19,8 @@
 
 #include "vulkan_device_api.h"
 
+#include <tvm/ffi/reflection/registry.h>
+
 #include <algorithm>
 #include <memory>
 #include <set>
@@ -330,12 +332,6 @@ void VulkanDeviceAPI::StreamSync(Device dev, TVMStreamHandle stream) {
   device(dev.device_id).ThreadLocalStream().Synchronize();
 }
 
-void VulkanDeviceAPI::SetStream(Device dev, TVMStreamHandle stream) {
-  ICHECK_EQ(stream, static_cast<void*>(nullptr));
-}
-
-TVMStreamHandle VulkanDeviceAPI::GetCurrentStream(Device dev) { return nullptr; }
-
 void VulkanDeviceAPI::CopyDataFromTo(const void* from, size_t from_offset, void* to,
                                      size_t to_offset, size_t size, Device dev_from, Device dev_to,
                                      DLDataType type_hint, TVMStreamHandle stream) {
@@ -455,18 +451,20 @@ VulkanDevice& VulkanDeviceAPI::device(size_t device_id) {
   return const_cast<VulkanDevice&>(const_cast<const VulkanDeviceAPI*>(this)->device(device_id));
 }
 
-TVM_FFI_REGISTER_GLOBAL("device_api.vulkan")
-    .set_body_packed([](ffi::PackedArgs args, ffi::Any* rv) {
-      DeviceAPI* ptr = VulkanDeviceAPI::Global();
-      *rv = static_cast<void*>(ptr);
-    });
-
-TVM_FFI_REGISTER_GLOBAL("device_api.vulkan.get_target_property")
-    .set_body_typed([](Device dev, const std::string& property) {
-      ffi::Any rv;
-      VulkanDeviceAPI::Global()->GetTargetProperty(dev, property, &rv);
-      return rv;
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def_packed("device_api.vulkan",
+                  [](ffi::PackedArgs args, ffi::Any* rv) {
+                    DeviceAPI* ptr = VulkanDeviceAPI::Global();
+                    *rv = static_cast<void*>(ptr);
+                  })
+      .def("device_api.vulkan.get_target_property", [](Device dev, const std::string& property) {
+        ffi::Any rv;
+        VulkanDeviceAPI::Global()->GetTargetProperty(dev, property, &rv);
+        return rv;
+      });
+}
 
 }  // namespace vulkan
 }  // namespace runtime
