@@ -205,12 +205,11 @@ VulkanModuleNode::~VulkanModuleNode() {
   }
 }
 
-ffi::Function VulkanModuleNode::GetFunction(const String& name,
-                                            const ObjectPtr<Object>& sptr_to_self) {
+ffi::Optional<ffi::Function> VulkanModuleNode::GetFunction(const ffi::String& name) {
+  ObjectPtr<Object> sptr_to_self = ffi::GetObjectPtr<Object>(this);
   ICHECK_EQ(sptr_to_self.get(), this);
-  ICHECK_NE(name, symbol::tvm_module_main) << "Device function do not have main";
   auto it = fmap_.find(name);
-  if (it == fmap_.end()) return ffi::Function();
+  if (it == fmap_.end()) return std::nullopt;
   const FunctionInfo& info = it->second;
   VulkanWrappedFunc f;
   size_t num_buffer_args = NumBufferArgs(info.arg_types);
@@ -404,7 +403,7 @@ std::shared_ptr<VulkanPipeline> VulkanModuleNode::GetPipeline(size_t device_id,
   return pe;
 }
 
-void VulkanModuleNode::SaveToFile(const String& file_name, const String& format) {
+void VulkanModuleNode::WriteToFile(const ffi::String& file_name, const ffi::String& format) const {
   std::string fmt = GetFileFormat(file_name, format);
   ICHECK_EQ(fmt, fmt_) << "Can only save to customized format vulkan";
   std::string meta_file = GetMetaFilePath(file_name);
@@ -418,13 +417,17 @@ void VulkanModuleNode::SaveToFile(const String& file_name, const String& format)
   SaveBinaryToFile(file_name, data_bin);
 }
 
-void VulkanModuleNode::SaveToBinary(dmlc::Stream* stream) {
+ffi::Bytes VulkanModuleNode::SaveToBytes() const {
+  std::string buffer;
+  dmlc::MemoryStringStream ms(&buffer);
+  dmlc::Stream* stream = &ms;
   stream->Write(fmt_);
   stream->Write(fmap_);
   stream->Write(smap_);
+  return ffi::Bytes(buffer);
 }
 
-String VulkanModuleNode::GetSource(const String& format) {
+ffi::String VulkanModuleNode::InspectSource(const ffi::String& format) const {
   // can only return disassembly code.
   return source_;
 }
