@@ -25,10 +25,10 @@ namespace script {
 namespace ir_builder {
 namespace ir {
 
-TVM_FFI_STATIC_INIT_BLOCK({ IRModuleFrameNode::RegisterReflection(); });
+TVM_FFI_STATIC_INIT_BLOCK() { IRModuleFrameNode::RegisterReflection(); }
 
 void IRModuleFrameNode::ExitWithScope() {
-  Map<GlobalVar, BaseFunc> func_map;
+  ffi::Map<GlobalVar, BaseFunc> func_map;
   CHECK_EQ(functions.size(), global_var_map.size())
       << "All functions must be defined in the IRModule. Got " << global_var_map.size()
       << "declared function(s), but only " << functions.size() << "defined function(s).";
@@ -44,8 +44,6 @@ void IRModuleFrameNode::ExitWithScope() {
   builder->result = tvm::IRModule(func_map, {}, dict_attrs, global_infos);
 }
 
-TVM_REGISTER_NODE_TYPE(IRModuleFrameNode);
-
 }  // namespace ir
 }  // namespace ir_builder
 }  // namespace script
