@@ -90,7 +90,7 @@ class CodeGenC : public ExprFunctor<void(const PrimExpr&, std::ostream&)>,
    * \param gvar The GlobalVar of the function
    * \returns The string name of the function
    */
-  String GetFunctionName(const GlobalVar& gvar);
+  ffi::String GetFunctionName(const GlobalVar& gvar);
 
   /*!
    * \brief Finalize the compilation and return the code.
@@ -131,7 +131,7 @@ class CodeGenC : public ExprFunctor<void(const PrimExpr&, std::ostream&)>,
    *
    * \param os The output stream
    */
-  virtual void PrintFunctionSignature(const String& function_name, const PrimFunc& func,
+  virtual void PrintFunctionSignature(const ffi::String& function_name, const PrimFunc& func,
                                       std::ostream& os);
 
   /*!
@@ -271,8 +271,8 @@ class CodeGenC : public ExprFunctor<void(const PrimExpr&, std::ostream&)>,
    * \param ret_type The return type of the function
    * \param os The output stream.
    */
-  virtual void GenerateForwardFunctionDeclarations(String global_symbol,
-                                                   const Array<Type>& arg_types,
+  virtual void GenerateForwardFunctionDeclarations(ffi::String global_symbol,
+                                                   const ffi::Array<Type>& arg_types,
                                                    const Type& ret_type) {}
 
   /*!
@@ -283,8 +283,9 @@ class CodeGenC : public ExprFunctor<void(const PrimExpr&, std::ostream&)>,
    * \param skip_first_arg Whether to skip the first arguments.
    * \param os The output stream.
    */
-  virtual void PrintCallExtern(Type ret_type, String global_symbol, const Array<PrimExpr>& args,
-                               bool skip_first_arg, std::ostream& os);  // NOLINT(*)
+  virtual void PrintCallExtern(Type ret_type, ffi::String global_symbol,
+                               const ffi::Array<PrimExpr>& args, bool skip_first_arg,
+                               std::ostream& os);  // NOLINT(*)
   /*!
    * \brief If buffer is allocated as type t.
    * \param buf_var The buffer variable.
@@ -319,6 +320,8 @@ class CodeGenC : public ExprFunctor<void(const PrimExpr&, std::ostream&)>,
   Integer constants_byte_alignment_ = 16;
   /*! \brief whether to print in SSA form */
   bool print_ssa_form_{false};
+  /*! \brief whether the module has a main function declared */
+  bool has_tvm_ffi_main_func_{false};
 
  private:
   /*! \brief set of volatile buf access */
@@ -337,7 +340,7 @@ class CodeGenC : public ExprFunctor<void(const PrimExpr&, std::ostream&)>,
    * functions, this is the name of the function's GlobalVar, possibly
    * altered to prevent duplicate names.
    */
-  std::unordered_map<GlobalVar, String> internal_functions_;
+  std::unordered_map<GlobalVar, ffi::String> internal_functions_;
 
   /* \brief Name supply to generate unique function names */
   NameSupply func_name_supply_;
