@@ -48,7 +48,7 @@ static PrimExpr DispatchMetalShuffle(const PrimExpr& e) {
   const CallNode* call = e.as<CallNode>();
   ICHECK(call != nullptr);
   ICHECK_EQ(call->args.size(), 5);  // mask, value, warp_id, width, warp_size
-  Array<PrimExpr> metal_args{{call->args[1], call->args[2]}};
+  ffi::Array<PrimExpr> metal_args{{call->args[1], call->args[2]}};
   return Call(call->dtype, T()(call->dtype, Downcast<Op>(call->op)), metal_args);
 }
 
