@@ -22,6 +22,7 @@
  * \brief Target kind registry
  */
 #include <tvm/ffi/function.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/ir/expr.h>
 #include <tvm/runtime/device_api.h>
 #include <tvm/target/target.h>
@@ -35,24 +36,21 @@
 
 namespace tvm {
 
-TVM_FFI_STATIC_INIT_BLOCK({ TargetKindNode::RegisterReflection(); });
-
-// helper to get internal dev function in objectref.
-struct TargetKind2ObjectPtr : public ObjectRef {
-  static ObjectPtr<Object> Get(const TargetKind& kind) {
-    return ffi::details::ObjectUnsafe::ObjectPtrFromObjectRef<Object>(kind);
-  }
-};
-
-TVM_REGISTER_NODE_TYPE(TargetKindNode)
-    .set_creator([](const std::string& name) {
-      auto kind = TargetKind::Get(name);
-      ICHECK(kind.defined()) << "Cannot find target kind \'" << name << '\'';
-      return TargetKind2ObjectPtr::Get(kind.value());
-    })
-    .set_repr_bytes([](const Object* n) -> std::string {
-      return static_cast<const TargetKindNode*>(n)->name;
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  TargetKindNode::RegisterReflection();
+  refl::TypeAttrDef<TargetKindNode>()
+      .def("__data_to_json__",
+           [](const TargetKindNode* node) {
+             // simply save as the string
+             return node->name;
+           })
+      .def("__data_from_json__", [](const ffi::String& name) {
+        auto kind = TargetKind::Get(name);
+        ICHECK(kind.has_value()) << "Cannot find target kind \'" << name << '\'';
+        return kind.value();
+      });
+}
 
 TVM_STATIC_IR_FUNCTOR(ReprPrinter, vtable)
     .set_dispatch<TargetKindNode>([](const ObjectRef& obj, ReprPrinter* p) {
@@ -64,32 +62,33 @@ TVM_STATIC_IR_FUNCTOR(ReprPrinter, vtable)
 
 using TargetKindRegistry = AttrRegistry<TargetKindRegEntry, TargetKind>;
 
-Array<String> TargetKindRegEntry::ListTargetKinds() {
+ffi::Array<ffi::String> TargetKindRegEntry::ListTargetKinds() {
   return TargetKindRegistry::Global()->ListAllNames();
 }
 
-Map<String, String> TargetKindRegEntry::ListTargetKindOptions(const TargetKind& target_kind) {
-  Map<String, String> options;
+ffi::Map<ffi::String, ffi::String> TargetKindRegEntry::ListTargetKindOptions(
+    const TargetKind& target_kind) {
+  ffi::Map<ffi::String, ffi::String> options;
   for (const auto& kv : target_kind->key2vtype_) {
     options.Set(kv.first, kv.second.type_key);
   }
   return options;
 }
 
-TargetKindRegEntry& TargetKindRegEntry::RegisterOrGet(const String& target_kind_name) {
+TargetKindRegEntry& TargetKindRegEntry::RegisterOrGet(const ffi::String& target_kind_name) {
   return TargetKindRegistry::Global()->RegisterOrGet(target_kind_name);
 }
 
-void TargetKindRegEntry::UpdateAttr(const String& key, ffi::Any value, int plevel) {
+void TargetKindRegEntry::UpdateAttr(const ffi::String& key, ffi::Any value, int plevel) {
   TargetKindRegistry::Global()->UpdateAttr(key, kind_, value, plevel);
 }
 
 const AttrRegistryMapContainerMap<TargetKind>& TargetKind::GetAttrMapContainer(
-    const String& attr_name) {
+    const ffi::String& attr_name) {
   return TargetKindRegistry::Global()->GetAttrMap(attr_name);
 }
 
-Optional<TargetKind> TargetKind::Get(const String& target_kind_name) {
+ffi::Optional<TargetKind> TargetKind::Get(const ffi::String& target_kind_name) {
   const TargetKindRegEntry* reg = TargetKindRegistry::Global()->Get(target_kind_name);
   if (reg == nullptr) {
     return std::nullopt;
@@ -142,12 +141,13 @@ static bool DetectDeviceFlag(Device device, runtime::DeviceAttrKind flag, ffi::A
   return true;
 }
 
-void CheckOrSetAttr(Map<String, ffi::Any>* attrs, const String& name, const String& value) {
+void CheckOrSetAttr(ffi::Map<ffi::String, ffi::Any>* attrs, const ffi::String& name,
+                    const ffi::String& value) {
   auto iter = attrs->find(name);
   if (iter == attrs->end()) {
     attrs->Set(name, value);
   } else {
-    auto str = (*iter).second.try_cast<String>();
+    auto str = (*iter).second.try_cast<ffi::String>();
     ICHECK(str && str.value() == value) << "ValueError: Expects \"" << name << "\" to be \""
                                         << value << "\", but gets: " << (*iter).second;
   }
@@ -164,7 +164,7 @@ TargetJSON UpdateCUDAAttrs(TargetJSON target) {
   // Update -arch=sm_xx
   if (target.count("arch")) {
     // If -arch has been specified, validate the correctness
-    String archStr = Downcast<String>(target.at("arch"));
+    ffi::String archStr = Downcast<ffi::String>(target.at("arch"));
     ICHECK(support::StartsWith(archStr, "sm_"))
         << "ValueError: CUDA target gets an invalid CUDA arch: -arch=" << archStr;
   } else {
@@ -177,7 +177,7 @@ TargetJSON UpdateCUDAAttrs(TargetJSON target) {
     } else {
       archInt = std::stod(version.cast<std::string>()) * 10 + 0.1;
     }
-    target.Set("arch", String("sm_") + std::to_string(archInt));
+    target.Set("arch", ffi::String("sm_") + std::to_string(archInt));
   }
   return target;
 }
@@ -192,7 +192,7 @@ TargetJSON UpdateNVPTXAttrs(TargetJSON target) {
   // Update -mcpu=sm_xx
   if (target.count("mcpu")) {
     // If -mcpu has been specified, validate the correctness
-    String mcpu = Downcast<String>(target.at("mcpu"));
+    ffi::String mcpu = Downcast<ffi::String>(target.at("mcpu"));
     ICHECK(support::StartsWith(mcpu, "sm_"))
         << "ValueError: NVPTX target gets an invalid CUDA arch: -mcpu=" << mcpu;
   } else {
@@ -205,7 +205,7 @@ TargetJSON UpdateNVPTXAttrs(TargetJSON target) {
     } else {
       arch = std::stod(version.cast<std::string>()) * 10 + 0.1;
     }
-    target.Set("mcpu", String("sm_") + std::to_string(arch));
+    target.Set("mcpu", ffi::String("sm_") + std::to_string(arch));
   }
   return target;
 }
@@ -220,7 +220,7 @@ TargetJSON UpdateROCmAttrs(TargetJSON target) {
   // Update -mcpu=gfx
   std::string arch = "gfx900";
   if (target.count("mcpu")) {
-    String mcpu = Downcast<String>(target.at("mcpu"));
+    ffi::String mcpu = Downcast<ffi::String>(target.at("mcpu"));
     arch = ExtractStringWithPrefix(mcpu, "gfx");
     ICHECK(!arch.empty()) << "ValueError: ROCm target gets an invalid GFX version: -mcpu=" << mcpu;
   } else {
@@ -228,7 +228,7 @@ TargetJSON UpdateROCmAttrs(TargetJSON target) {
     if (const auto f_get_rocm_arch = tvm::ffi::Function::GetGlobal("tvm_callback_rocm_get_arch")) {
       arch = (*f_get_rocm_arch)().cast<std::string>();
     }
-    target.Set("mcpu", String(arch));
+    target.Set("mcpu", ffi::String(arch));
   }
   // Update -mattr before ROCm 3.5:
   //   Before ROCm 3.5 we needed code object v2, starting
@@ -243,9 +243,9 @@ TargetJSON UpdateROCmAttrs(TargetJSON target) {
     version = val.cast<int>();
   }
   if (version < 305) {
-    Array<String> mattr;
+    ffi::Array<ffi::String> mattr;
     if (target.count("mattr")) {
-      mattr = Downcast<Array<String>>(target.at("mattr"));
+      mattr = Downcast<ffi::Array<ffi::String>>(target.at("mattr"));
     }
     mattr.push_back("-code-object-v3");
     target.Set("mattr", mattr);
@@ -259,7 +259,7 @@ TargetJSON UpdateROCmAttrs(TargetJSON target) {
  * \return The updated attributes
  */
 TargetJSON TestTargetParser(TargetJSON target) {
-  Map<String, ffi::Any> features = {{"is_test", true}};
+  ffi::Map<ffi::String, ffi::Any> features = {{"is_test", true}};
   target.Set("features", features);
   return target;
 }
@@ -267,11 +267,11 @@ TargetJSON TestTargetParser(TargetJSON target) {
 /**********  Register Target kinds and attributes  **********/
 
 TVM_REGISTER_TARGET_KIND("llvm", kDLCPU)
-    .add_attr_option<Array<String>>("mattr")
-    .add_attr_option<String>("mcpu")
-    .add_attr_option<String>("mtriple")
-    .add_attr_option<String>("mfloat-abi")
-    .add_attr_option<String>("mabi")
+    .add_attr_option<ffi::Array<ffi::String>>("mattr")
+    .add_attr_option<ffi::String>("mcpu")
+    .add_attr_option<ffi::String>("mtriple")
+    .add_attr_option<ffi::String>("mfloat-abi")
+    .add_attr_option<ffi::String>("mabi")
     .add_attr_option<int64_t>("num-cores")
     // Fast math flags, see https://llvm.org/docs/LangRef.html#fast-math-flags
     .add_attr_option<bool>("fast-math")  // implies all the below
@@ -283,9 +283,9 @@ TVM_REGISTER_TARGET_KIND("llvm", kDLCPU)
     .add_attr_option<bool>("fast-math-reassoc")
     .add_attr_option<int64_t>("opt-level")
     // LLVM command line flags, see below
-    .add_attr_option<Array<String>>("cl-opt")
+    .add_attr_option<ffi::Array<ffi::String>>("cl-opt")
     // LLVM JIT engine mcjit/orcjit
-    .add_attr_option<String>("jit")
+    .add_attr_option<ffi::String>("jit")
     // TVM & LLVM custom vector bit width
     .add_attr_option<int64_t>("vector-width")
     .set_default_keys({"cpu"})
@@ -316,16 +316,16 @@ TVM_REGISTER_TARGET_KIND("llvm", kDLCPU)
 // Hence the type is "uint".
 
 TVM_REGISTER_TARGET_KIND("c", kDLCPU)
-    .add_attr_option<String>("mcpu")
-    .add_attr_option<String>("march")
+    .add_attr_option<ffi::String>("mcpu")
+    .add_attr_option<ffi::String>("march")
     .add_attr_option<int64_t>("workspace-byte-alignment")
     .add_attr_option<int64_t>("constants-byte-alignment")
     .set_default_keys({"cpu"})
     .set_target_parser(tvm::target::parsers::cpu::ParseTarget);
 
 TVM_REGISTER_TARGET_KIND("cuda", kDLCUDA)
-    .add_attr_option<String>("mcpu")
-    .add_attr_option<String>("arch")
+    .add_attr_option<ffi::String>("mcpu")
+    .add_attr_option<ffi::String>("arch")
     .add_attr_option<int64_t>("max_shared_memory_per_block")
     .add_attr_option<int64_t>("max_threads_per_block")
     .add_attr_option<int64_t>("thread_warp_size", 32)
@@ -336,17 +336,17 @@ TVM_REGISTER_TARGET_KIND("cuda", kDLCUDA)
     .set_target_parser(UpdateCUDAAttrs);
 
 TVM_REGISTER_TARGET_KIND("nvptx", kDLCUDA)
-    .add_attr_option<String>("mcpu")
-    .add_attr_option<String>("mtriple")
+    .add_attr_option<ffi::String>("mcpu")
+    .add_attr_option<ffi::String>("mtriple")
     .add_attr_option<int64_t>("max_num_threads", 1024)
     .add_attr_option<int64_t>("thread_warp_size", 32)
     .set_default_keys({"cuda", "gpu"})
     .set_target_parser(UpdateNVPTXAttrs);
 
 TVM_REGISTER_TARGET_KIND("rocm", kDLROCM)
-    .add_attr_option<String>("mcpu")
-    .add_attr_option<String>("mtriple")
-    .add_attr_option<Array<String>>("mattr")
+    .add_attr_option<ffi::String>("mcpu")
+    .add_attr_option<ffi::String>("mtriple")
+    .add_attr_option<ffi::Array<ffi::String>>("mattr")
     // TODO(masahi): Support querying from a target device
     // On RDNA cards, thread_warp_size should be 32
     .add_attr_option<int64_t>("max_num_threads", 256)
@@ -384,7 +384,7 @@ TVM_REGISTER_TARGET_KIND("metal", kDLMetal)
     .set_default_keys({"metal", "gpu"});
 
 TVM_REGISTER_TARGET_KIND("vulkan", kDLVulkan)
-    .add_attr_option<Array<String>>("mattr")
+    .add_attr_option<ffi::Array<ffi::String>>("mattr")
     // Feature support
     .add_attr_option<bool>("supports_float16")
     .add_attr_option<bool>("supports_float32", true)
@@ -414,9 +414,9 @@ TVM_REGISTER_TARGET_KIND("vulkan", kDLVulkan)
     .add_attr_option<int64_t>("max_per_stage_descriptor_storage_buffer")
     .add_attr_option<int64_t>("max_shared_memory_per_block")
     // Other device properties
-    .add_attr_option<String>("device_type")
-    .add_attr_option<String>("device_name")
-    .add_attr_option<String>("driver_name")
+    .add_attr_option<ffi::String>("device_type")
+    .add_attr_option<ffi::String>("device_name")
+    .add_attr_option<ffi::String>("driver_name")
     .add_attr_option<int64_t>("driver_version")
     .add_attr_option<int64_t>("vulkan_api_version")
     .add_attr_option<int64_t>("max_spirv_version")
@@ -428,43 +428,42 @@ TVM_REGISTER_TARGET_KIND("webgpu", kDLWebGPU)
     .set_default_keys({"webgpu", "gpu"});
 
 TVM_REGISTER_TARGET_KIND("hexagon", kDLHexagon)
-    .add_attr_option<Array<String>>("mattr")
-    .add_attr_option<String>("mcpu")
-    .add_attr_option<String>("mtriple")
-    .add_attr_option<Array<String>>("llvm-options")
+    .add_attr_option<ffi::Array<ffi::String>>("mattr")
+    .add_attr_option<ffi::String>("mcpu")
+    .add_attr_option<ffi::String>("mtriple")
+    .add_attr_option<ffi::Array<ffi::String>>("llvm-options")
     .add_attr_option<int64_t>("num-cores")
     .add_attr_option<int64_t>("vtcm-capacity")
     .set_default_keys({"hexagon", "cpu"});
 
 TVM_REGISTER_TARGET_KIND("ext_dev", kDLExtDev);
 
-TVM_REGISTER_TARGET_KIND("hybrid", kDLCPU);
-
 TVM_REGISTER_TARGET_KIND("composite", kDLCPU)  // line break
-    .add_attr_option<Array<Target>>("devices");
+    .add_attr_option<ffi::Array<Target>>("devices");
 
 TVM_REGISTER_TARGET_KIND("test", kDLCPU)  // line break
     .set_target_parser(TestTargetParser);
 
 /**********  Registry  **********/
 
-TVM_FFI_REGISTER_GLOBAL("target.TargetKindGetAttr")
-    .set_body_typed([](TargetKind kind, String attr_name) -> ffi::Any {
-      auto target_attr_map = TargetKind::GetAttrMap<ffi::Any>(attr_name);
-      ffi::Any rv;
-      if (target_attr_map.count(kind)) {
-        rv = target_attr_map[kind];
-      }
-      return rv;
-    });
-TVM_FFI_REGISTER_GLOBAL("target.ListTargetKinds")
-    .set_body_typed(TargetKindRegEntry::ListTargetKinds);
-TVM_FFI_REGISTER_GLOBAL("target.ListTargetKindOptions")
-    .set_body_typed(TargetKindRegEntry::ListTargetKindOptions);
-TVM_FFI_REGISTER_GLOBAL("target.ListTargetKindOptionsFromName")
-    .set_body_typed([](String target_kind_name) {
-      TargetKind kind = TargetKind::Get(target_kind_name).value();
-      return TargetKindRegEntry::ListTargetKindOptions(kind);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def("target.TargetKindGetAttr",
+           [](TargetKind kind, ffi::String attr_name) -> ffi::Any {
+             auto target_attr_map = TargetKind::GetAttrMap<ffi::Any>(attr_name);
+             ffi::Any rv;
+             if (target_attr_map.count(kind)) {
+               rv = target_attr_map[kind];
+             }
+             return rv;
+           })
+      .def("target.ListTargetKinds", TargetKindRegEntry::ListTargetKinds)
+      .def("target.ListTargetKindOptions", TargetKindRegEntry::ListTargetKindOptions)
+      .def("target.ListTargetKindOptionsFromName", [](ffi::String target_kind_name) {
+        TargetKind kind = TargetKind::Get(target_kind_name).value();
+        return TargetKindRegEntry::ListTargetKindOptions(kind);
+      });
+}
 
 }  // namespace tvm
