@@ -24,6 +24,7 @@
 #include "graph.h"
 
 #include <tvm/ffi/function.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/te/operation.h>
 #include <tvm/tir/expr.h>
 #include <tvm/tir/stmt_functor.h>
@@ -36,7 +37,7 @@ namespace te {
 
 // construct a read graph that gives readers of each operation
 // that the root depend on
-ReadGraph CreateReadGraph(const Array<Operation>& roots) {
+ReadGraph CreateReadGraph(const ffi::Array<Operation>& roots) {
   ReadGraph rmap;
   std::vector<Operation> stack;
   std::unordered_set<const Object*> visited;
@@ -49,7 +50,7 @@ ReadGraph CreateReadGraph(const Array<Operation>& roots) {
   while (!stack.empty()) {
     Operation op = stack.back();
     stack.pop_back();
-    Array<Tensor> deps = op->InputTensors();
+    ffi::Array<Tensor> deps = op->InputTensors();
     rmap.Set(op, deps);
     for (Tensor t : deps) {
       if (t->op.defined() && visited.count(t->op.get()) == 0) {
@@ -62,7 +63,7 @@ ReadGraph CreateReadGraph(const Array<Operation>& roots) {
 }
 
 void PostDFSOrder(const Operation& op, const ReadGraph& g, std::unordered_set<Operation>* visited,
-                  Array<Operation>* post_order) {
+                  ffi::Array<Operation>* post_order) {
   if (visited->count(op)) return;
   visited->insert(op);
   for (const auto& t : g.at(op)) {
@@ -71,21 +72,23 @@ void PostDFSOrder(const Operation& op, const ReadGraph& g, std::unordered_set<Op
   post_order->push_back(op);
 }
 
-Array<Operation> PostDFSOrder(const Array<Operation>& roots, const ReadGraph& g) {
+ffi::Array<Operation> PostDFSOrder(const ffi::Array<Operation>& roots, const ReadGraph& g) {
   std::unordered_set<Operation> visited;
-  Array<Operation> post_order;
+  ffi::Array<Operation> post_order;
   for (Operation op : roots) {
     PostDFSOrder(op, g, &visited, &post_order);
   }
   return post_order;
 }
 
-TVM_FFI_REGISTER_GLOBAL("schedule.CreateReadGraph").set_body_typed(CreateReadGraph);
-
-TVM_FFI_REGISTER_GLOBAL("schedule.PostDFSOrder")
-    .set_body_typed([](const Array<Operation>& roots, const ReadGraph& g) {
-      return PostDFSOrder(roots, g);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def("schedule.CreateReadGraph", CreateReadGraph)
+      .def("schedule.PostDFSOrder", [](const ffi::Array<Operation>& roots, const ReadGraph& g) {
+        return PostDFSOrder(roots, g);
+      });
+}
 
 }  // namespace te
 }  // namespace tvm
