@@ -21,6 +21,7 @@
  * \file expr.cc
  */
 #include <tvm/ffi/function.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/tir/builtin.h>
 #include <tvm/tir/expr.h>
 #include <tvm/tir/op.h>
@@ -35,7 +36,7 @@
 namespace tvm {
 namespace tir {
 
-TVM_FFI_STATIC_INIT_BLOCK({
+TVM_FFI_STATIC_INIT_BLOCK() {
   VarNode::RegisterReflection();
   SizeVarNode::RegisterReflection();
   IterVarNode::RegisterReflection();
@@ -69,7 +70,7 @@ TVM_FFI_STATIC_INIT_BLOCK({
   ShuffleNode::RegisterReflection();
   CommReducerNode::RegisterReflection();
   ReduceNode::RegisterReflection();
-});
+}
 
 /* \brief Convert an object to a PrimExpr
  *
@@ -79,9 +80,11 @@ TVM_FFI_STATIC_INIT_BLOCK({
  * `expr.dtype` field), this function allows the FFI conversions to be
  * explicitly invoked.
  */
-TVM_FFI_REGISTER_GLOBAL("tir.convert").set_body_typed([](Variant<PrimExpr, Array<PrimExpr>> expr) {
-  return expr;
-});
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.convert",
+                        [](ffi::Variant<PrimExpr, ffi::Array<PrimExpr>> expr) { return expr; });
+}
 
 #define TVM_DEFINE_BINOP_CONSTRUCTOR(Name)                                                   \
   Name::Name(PrimExpr a, PrimExpr b, Span span) {                                            \
@@ -90,7 +93,7 @@ TVM_FFI_REGISTER_GLOBAL("tir.convert").set_body_typed([](Variant<PrimExpr, Array
     ICHECK(b.defined()) << "ValueError: b is undefined\n";                                   \
     CHECK(a.dtype() == b.dtype()) << "TypeError: mismatched types. " << a.dtype() << " vs. " \
                                   << b.dtype() << "\n";                                      \
-    ObjectPtr<T> node = make_object<T>();                                                    \
+    ObjectPtr<T> node = ffi::make_object<T>();                                               \
     node->dtype = a.dtype();                                                                 \
     node->a = std::move(a);                                                                  \
     node->b = std::move(b);                                                                  \
@@ -105,7 +108,7 @@ TVM_FFI_REGISTER_GLOBAL("tir.convert").set_body_typed([](Variant<PrimExpr, Array
     ICHECK(b.defined()) << "ValueError: b is undefined\n";                                   \
     CHECK(a.dtype() == b.dtype()) << "TypeError: mismatched types. " << a.dtype() << " vs. " \
                                   << b.dtype() << "\n";                                      \
-    ObjectPtr<T> node = make_object<T>();                                                    \
+    ObjectPtr<T> node = ffi::make_object<T>();                                               \
     DataType a_dtype = a.dtype();                                                            \
     node->dtype =                                                                            \
         DataType::Bool(a_dtype.get_lanes_or_vscale_factor(), a_dtype.is_scalable_vector());  \
@@ -116,8 +119,8 @@ TVM_FFI_REGISTER_GLOBAL("tir.convert").set_body_typed([](Variant<PrimExpr, Array
   }
 
 // Var
-Var::Var(String name_hint, DataType dtype, Span span) {
-  auto n = make_object<VarNode>();
+Var::Var(ffi::String name_hint, DataType dtype, Span span) {
+  auto n = ffi::make_object<VarNode>();
   n->name_hint = std::move(name_hint);
   n->type_annotation = GetTypeFromRuntimeDataType(dtype);
   n->dtype = std::move(dtype);
@@ -125,8 +128,8 @@ Var::Var(String name_hint, DataType dtype, Span span) {
   data_ = std::move(n);
 }
 
-Var::Var(String name_hint, Type type_annotation, Span span) {
-  auto n = make_object<VarNode>();
+Var::Var(ffi::String name_hint, Type type_annotation, Span span) {
+  auto n = ffi::make_object<VarNode>();
   n->name_hint = std::move(name_hint);
   n->dtype = GetRuntimeDataType(type_annotation);
   n->type_annotation = std::move(type_annotation);
@@ -134,19 +137,19 @@ Var::Var(String name_hint, Type type_annotation, Span span) {
   data_ = std::move(n);
 }
 
-Var Var::copy_with_name(const String& name) const {
+Var Var::copy_with_name(const ffi::String& name) const {
   const VarNode* node = get();
   ObjectPtr<VarNode> new_ptr;
   if (auto* ptr = this->as<SizeVarNode>()) {
-    new_ptr = make_object<SizeVarNode>(*ptr);
+    new_ptr = ffi::make_object<SizeVarNode>(*ptr);
   } else {
-    new_ptr = make_object<VarNode>(*node);
+    new_ptr = ffi::make_object<VarNode>(*node);
   }
   new_ptr->name_hint = name;
   return Var(new_ptr);
 }
 
-Var Var::copy_with_suffix(const String& suffix) const {
+Var Var::copy_with_suffix(const ffi::String& suffix) const {
   return this->copy_with_name(get()->name_hint + suffix);
 }
 
@@ -154,29 +157,29 @@ Var Var::copy_with_dtype(DataType dtype) const {
   const VarNode* node = get();
   ObjectPtr<VarNode> new_ptr;
   if (auto* ptr = this->as<SizeVarNode>()) {
-    new_ptr = make_object<SizeVarNode>(*ptr);
+    new_ptr = ffi::make_object<SizeVarNode>(*ptr);
   } else {
-    new_ptr = make_object<VarNode>(*node);
+    new_ptr = ffi::make_object<VarNode>(*node);
   }
   new_ptr->type_annotation = GetTypeFromRuntimeDataType(dtype);
   new_ptr->dtype = std::move(dtype);
   return Var(new_ptr);
 }
 
-TVM_FFI_REGISTER_GLOBAL("tir.Var").set_body_typed([](String name_hint, ffi::AnyView type,
-                                                     Span span) {
-  if (type.as<Type>()) {
-    return Var(name_hint, type.cast<Type>(), span);
-  } else {
-    return Var(name_hint, type.cast<DataType>(), span);
-  }
-});
-
-TVM_REGISTER_NODE_TYPE(VarNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.Var", [](ffi::String name_hint, ffi::AnyView type, Span span) {
+    if (type.as<Type>()) {
+      return Var(name_hint, type.cast<Type>(), span);
+    } else {
+      return Var(name_hint, type.cast<DataType>(), span);
+    }
+  });
+}
 
 // SizeVar
-SizeVar::SizeVar(String name_hint, DataType dtype, Span span) {
-  auto n = make_object<SizeVarNode>();
+SizeVar::SizeVar(ffi::String name_hint, DataType dtype, Span span) {
+  auto n = ffi::make_object<SizeVarNode>();
   n->name_hint = std::move(name_hint);
   n->type_annotation = GetTypeFromRuntimeDataType(dtype);
   n->dtype = std::move(dtype);
@@ -184,8 +187,8 @@ SizeVar::SizeVar(String name_hint, DataType dtype, Span span) {
   data_ = std::move(n);
 }
 
-SizeVar::SizeVar(String name_hint, Type type_annotation, Span span) {
-  auto n = make_object<SizeVarNode>();
+SizeVar::SizeVar(ffi::String name_hint, Type type_annotation, Span span) {
+  auto n = ffi::make_object<SizeVarNode>();
   n->name_hint = std::move(name_hint);
   n->dtype = GetRuntimeDataType(type_annotation);
   n->type_annotation = std::move(type_annotation);
@@ -193,15 +196,15 @@ SizeVar::SizeVar(String name_hint, Type type_annotation, Span span) {
   data_ = std::move(n);
 }
 
-TVM_FFI_REGISTER_GLOBAL("tir.SizeVar").set_body_typed([](String s, DataType t, Span span) {
-  return SizeVar(s, t, span);
-});
-
-TVM_REGISTER_NODE_TYPE(SizeVarNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.SizeVar",
+                        [](ffi::String s, DataType t, Span span) { return SizeVar(s, t, span); });
+}
 
 // IterVar
-IterVar::IterVar(Range dom, Var var, IterVarType t, String thread_tag, Span span) {
-  ObjectPtr<IterVarNode> n = make_object<IterVarNode>();
+IterVar::IterVar(Range dom, Var var, IterVarType t, ffi::String thread_tag, Span span) {
+  ObjectPtr<IterVarNode> n = ffi::make_object<IterVarNode>();
   if (dom.defined() && dom->extent.defined()) {
     CHECK(dom->extent.dtype().is_int())
         << "The dtype of the domain of an IterVar must be an integer type. However, the domain's "
@@ -219,180 +222,176 @@ IterVar::IterVar(Range dom, Var var, IterVarType t, String thread_tag, Span span
   data_ = std::move(n);
 }
 
-TVM_FFI_REGISTER_GLOBAL("tir.IterVar")
-    .set_body_typed([](Range dom, Var var, int iter_type, String thread_tag, Span span) {
-      return IterVar(dom, var, static_cast<IterVarType>(iter_type), thread_tag, span);
-    });
-
-TVM_REGISTER_NODE_TYPE(IterVarNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def(
+      "tir.IterVar", [](Range dom, Var var, int iter_type, ffi::String thread_tag, Span span) {
+        return IterVar(dom, var, static_cast<IterVarType>(iter_type), thread_tag, span);
+      });
+}
 
 // StringImm
-StringImm::StringImm(String value, Span span) {
-  ObjectPtr<StringImmNode> node = make_object<StringImmNode>();
+StringImm::StringImm(ffi::String value, Span span) {
+  ObjectPtr<StringImmNode> node = ffi::make_object<StringImmNode>();
   node->dtype = DataType::Handle();
   node->value = std::move(value);
   node->span = std::move(span);
   data_ = std::move(node);
 }
 
-TVM_FFI_REGISTER_GLOBAL("tir.StringImm").set_body_typed([](String value, Span span) {
-  return StringImm(value, span);
-});
-
-TVM_REGISTER_NODE_TYPE(StringImmNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.StringImm",
+                        [](ffi::String value, Span span) { return StringImm(value, span); });
+}
 
 // Cast
 Cast::Cast(DataType t, PrimExpr value, Span span) {
   ICHECK(value.defined());
   ICHECK_EQ(t.get_lanes_or_vscale_factor(), value.dtype().get_lanes_or_vscale_factor());
   ICHECK(t.is_scalable_vector() == value.dtype().is_scalable_vector());
-  ObjectPtr<CastNode> node = make_object<CastNode>();
+  ObjectPtr<CastNode> node = ffi::make_object<CastNode>();
   node->dtype = t;
   node->value = std::move(value);
   node->span = std::move(span);
   data_ = std::move(node);
 }
 
-TVM_FFI_REGISTER_GLOBAL("tir.Cast").set_body_typed([](DataType dtype, PrimExpr value, Span span) {
-  return Cast(dtype, value, span);
-});
-
-TVM_REGISTER_NODE_TYPE(CastNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.Cast", [](DataType dtype, PrimExpr value, Span span) {
+    return Cast(dtype, value, span);
+  });
+}
 
 // Add
 TVM_DEFINE_BINOP_CONSTRUCTOR(Add);
 
-TVM_FFI_REGISTER_GLOBAL("tir.Add").set_body_typed([](PrimExpr a, PrimExpr b, Span span) {
-  return Add(a, b, span);
-});
-
-TVM_REGISTER_NODE_TYPE(AddNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.Add",
+                        [](PrimExpr a, PrimExpr b, Span span) { return Add(a, b, span); });
+}
 
 // Sub
 TVM_DEFINE_BINOP_CONSTRUCTOR(Sub);
 
-TVM_FFI_REGISTER_GLOBAL("tir.Sub").set_body_typed([](PrimExpr a, PrimExpr b, Span span) {
-  return Sub(a, b, span);
-});
-
-TVM_REGISTER_NODE_TYPE(SubNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.Sub",
+                        [](PrimExpr a, PrimExpr b, Span span) { return Sub(a, b, span); });
+}
 
 // Mul
 TVM_DEFINE_BINOP_CONSTRUCTOR(Mul);
 
-TVM_FFI_REGISTER_GLOBAL("tir.Mul").set_body_typed([](PrimExpr a, PrimExpr b, Span span) {
-  return Mul(a, b, span);
-});
-
-TVM_REGISTER_NODE_TYPE(MulNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.Mul",
+                        [](PrimExpr a, PrimExpr b, Span span) { return Mul(a, b, span); });
+}
 
 // Div
 TVM_DEFINE_BINOP_CONSTRUCTOR(Div);
 
-TVM_FFI_REGISTER_GLOBAL("tir.Div").set_body_typed([](PrimExpr a, PrimExpr b, Span span) {
-  return Div(a, b, span);
-});
-
-TVM_REGISTER_NODE_TYPE(DivNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.Div",
+                        [](PrimExpr a, PrimExpr b, Span span) { return Div(a, b, span); });
+}
 
 // Mod
 TVM_DEFINE_BINOP_CONSTRUCTOR(Mod);
 
-TVM_FFI_REGISTER_GLOBAL("tir.Mod").set_body_typed([](PrimExpr a, PrimExpr b, Span span) {
-  return Mod(a, b, span);
-});
-
-TVM_REGISTER_NODE_TYPE(ModNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.Mod",
+                        [](PrimExpr a, PrimExpr b, Span span) { return Mod(a, b, span); });
+}
 
 // FloorDiv
 TVM_DEFINE_BINOP_CONSTRUCTOR(FloorDiv);
 
-TVM_FFI_REGISTER_GLOBAL("tir.FloorDiv").set_body_typed([](PrimExpr a, PrimExpr b, Span span) {
-  return FloorDiv(a, b, span);
-});
-
-TVM_REGISTER_NODE_TYPE(FloorDivNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.FloorDiv",
+                        [](PrimExpr a, PrimExpr b, Span span) { return FloorDiv(a, b, span); });
+}
 
 // FloorMod
 TVM_DEFINE_BINOP_CONSTRUCTOR(FloorMod);
 
-TVM_FFI_REGISTER_GLOBAL("tir.FloorMod").set_body_typed([](PrimExpr a, PrimExpr b, Span span) {
-  return FloorMod(a, b, span);
-});
-
-TVM_REGISTER_NODE_TYPE(FloorModNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.FloorMod",
+                        [](PrimExpr a, PrimExpr b, Span span) { return FloorMod(a, b, span); });
+}
 
 // Min
 TVM_DEFINE_BINOP_CONSTRUCTOR(Min);
 
-TVM_FFI_REGISTER_GLOBAL("tir.Min").set_body_typed([](PrimExpr a, PrimExpr b, Span span) {
-  return Min(a, b, span);
-});
-
-TVM_REGISTER_NODE_TYPE(MinNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.Min",
+                        [](PrimExpr a, PrimExpr b, Span span) { return Min(a, b, span); });
+}
 
 // Max
 TVM_DEFINE_BINOP_CONSTRUCTOR(Max);
 
-TVM_FFI_REGISTER_GLOBAL("tir.Max").set_body_typed([](PrimExpr a, PrimExpr b, Span span) {
-  return Max(a, b, span);
-});
-
-TVM_REGISTER_NODE_TYPE(MaxNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.Max",
+                        [](PrimExpr a, PrimExpr b, Span span) { return Max(a, b, span); });
+}
 
 // EQ
 TVM_DEFINE_CMPOP_CONSTRUCTOR(EQ);
 
-TVM_FFI_REGISTER_GLOBAL("tir.EQ").set_body_typed([](PrimExpr a, PrimExpr b, Span span) {
-  return EQ(a, b, span);
-});
-
-TVM_REGISTER_NODE_TYPE(EQNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.EQ", [](PrimExpr a, PrimExpr b, Span span) { return EQ(a, b, span); });
+}
 
 // NE
 TVM_DEFINE_CMPOP_CONSTRUCTOR(NE);
 
-TVM_FFI_REGISTER_GLOBAL("tir.NE").set_body_typed([](PrimExpr a, PrimExpr b, Span span) {
-  return NE(a, b, span);
-});
-
-TVM_REGISTER_NODE_TYPE(NENode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.NE", [](PrimExpr a, PrimExpr b, Span span) { return NE(a, b, span); });
+}
 
 // LT
 TVM_DEFINE_CMPOP_CONSTRUCTOR(LT);
 
-TVM_FFI_REGISTER_GLOBAL("tir.LT").set_body_typed([](PrimExpr a, PrimExpr b, Span span) {
-  return LT(a, b, span);
-});
-
-TVM_REGISTER_NODE_TYPE(LTNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.LT", [](PrimExpr a, PrimExpr b, Span span) { return LT(a, b, span); });
+}
 
 // LE
 TVM_DEFINE_CMPOP_CONSTRUCTOR(LE);
 
-TVM_FFI_REGISTER_GLOBAL("tir.LE").set_body_typed([](PrimExpr a, PrimExpr b, Span span) {
-  return LE(a, b, span);
-});
-
-TVM_REGISTER_NODE_TYPE(LENode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.LE", [](PrimExpr a, PrimExpr b, Span span) { return LE(a, b, span); });
+}
 
 // GT
 TVM_DEFINE_CMPOP_CONSTRUCTOR(GT);
 
-TVM_FFI_REGISTER_GLOBAL("tir.GT").set_body_typed([](PrimExpr a, PrimExpr b, Span span) {
-  return GT(a, b, span);
-});
-
-TVM_REGISTER_NODE_TYPE(GTNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.GT", [](PrimExpr a, PrimExpr b, Span span) { return GT(a, b, span); });
+}
 
 // GE
 TVM_DEFINE_CMPOP_CONSTRUCTOR(GE);
 
-TVM_FFI_REGISTER_GLOBAL("tir.GE").set_body_typed([](PrimExpr a, PrimExpr b, Span span) {
-  return GE(a, b, span);
-});
-
-TVM_REGISTER_NODE_TYPE(GENode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.GE", [](PrimExpr a, PrimExpr b, Span span) { return GE(a, b, span); });
+}
 
 // And
 And::And(PrimExpr a, PrimExpr b, Span span) {
@@ -402,7 +401,7 @@ And::And(PrimExpr a, PrimExpr b, Span span) {
   ICHECK(b.dtype().is_bool());
   ICHECK(a.dtype() == b.dtype()) << "TypeError: mismatched types";
 
-  ObjectPtr<AndNode> node = make_object<AndNode>();
+  ObjectPtr<AndNode> node = ffi::make_object<AndNode>();
   node->dtype =
       DataType::Bool(a.dtype().get_lanes_or_vscale_factor(), a.dtype().is_scalable_vector());
   node->a = std::move(a);
@@ -411,11 +410,11 @@ And::And(PrimExpr a, PrimExpr b, Span span) {
   data_ = std::move(node);
 }
 
-TVM_FFI_REGISTER_GLOBAL("tir.And").set_body_typed([](PrimExpr a, PrimExpr b, Span span) {
-  return And(a, b, span);
-});
-
-TVM_REGISTER_NODE_TYPE(AndNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.And",
+                        [](PrimExpr a, PrimExpr b, Span span) { return And(a, b, span); });
+}
 
 // Or
 Or::Or(PrimExpr a, PrimExpr b, Span span) {
@@ -425,7 +424,7 @@ Or::Or(PrimExpr a, PrimExpr b, Span span) {
   ICHECK(b.dtype().is_bool());
   ICHECK(a.dtype() == b.dtype()) << "TypeError: mismatched types";
 
-  ObjectPtr<OrNode> node = make_object<OrNode>();
+  ObjectPtr<OrNode> node = ffi::make_object<OrNode>();
   node->dtype =
       DataType::Bool(a.dtype().get_lanes_or_vscale_factor(), a.dtype().is_scalable_vector());
   node->a = std::move(a);
@@ -434,18 +433,17 @@ Or::Or(PrimExpr a, PrimExpr b, Span span) {
   data_ = std::move(node);
 }
 
-TVM_FFI_REGISTER_GLOBAL("tir.Or").set_body_typed([](PrimExpr a, PrimExpr b, Span span) {
-  return Or(a, b, span);
-});
-
-TVM_REGISTER_NODE_TYPE(OrNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.Or", [](PrimExpr a, PrimExpr b, Span span) { return Or(a, b, span); });
+}
 
 // Not
 Not::Not(PrimExpr a, Span span) {
   ICHECK(a.defined()) << "ValueError: a is undefined";
   ICHECK(a.dtype().is_bool());
 
-  ObjectPtr<NotNode> node = make_object<NotNode>();
+  ObjectPtr<NotNode> node = ffi::make_object<NotNode>();
   DataType a_dtype = a.dtype();
   node->dtype = DataType::Bool(a_dtype.get_lanes_or_vscale_factor(), a_dtype.is_scalable_vector());
   node->a = std::move(a);
@@ -453,11 +451,10 @@ Not::Not(PrimExpr a, Span span) {
   data_ = std::move(node);
 }
 
-TVM_FFI_REGISTER_GLOBAL("tir.Not").set_body_typed([](PrimExpr a, Span span) {
-  return Not(a, span);
-});
-
-TVM_REGISTER_NODE_TYPE(NotNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.Not", [](PrimExpr a, Span span) { return Not(a, span); });
+}
 
 // Select
 Select::Select(PrimExpr condition, PrimExpr true_value, PrimExpr false_value, Span span) {
@@ -472,7 +469,7 @@ Select::Select(PrimExpr condition, PrimExpr true_value, PrimExpr false_value, Sp
       << "TypeError: mismatched types. "
       << "False type: " << false_value.dtype() << "; True type: " << true_value.dtype();
 
-  ObjectPtr<SelectNode> node = make_object<SelectNode>();
+  ObjectPtr<SelectNode> node = ffi::make_object<SelectNode>();
   node->dtype = true_value.dtype();
   node->condition = std::move(condition);
   node->true_value = std::move(true_value);
@@ -481,12 +478,13 @@ Select::Select(PrimExpr condition, PrimExpr true_value, PrimExpr false_value, Sp
   data_ = std::move(node);
 }
 
-TVM_FFI_REGISTER_GLOBAL("tir.Select")
-    .set_body_typed([](PrimExpr condition, PrimExpr true_value, PrimExpr false_value, Span span) {
-      return Select(condition, true_value, false_value, span);
-    });
-
-TVM_REGISTER_NODE_TYPE(SelectNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def(
+      "tir.Select", [](PrimExpr condition, PrimExpr true_value, PrimExpr false_value, Span span) {
+        return Select(condition, true_value, false_value, span);
+      });
+}
 
 // Ramp
 Ramp::Ramp(PrimExpr base, PrimExpr stride, PrimExpr lanes, Span span) {
@@ -498,7 +496,7 @@ Ramp::Ramp(PrimExpr base, PrimExpr stride, PrimExpr lanes, Span span) {
     stride = cast(base.dtype(), stride);
   }
 
-  ObjectPtr<RampNode> node = make_object<RampNode>();
+  ObjectPtr<RampNode> node = ffi::make_object<RampNode>();
   auto* lanes_as_int = lanes.as<IntImmNode>();
   if (lanes_as_int) {
     int lanes = static_cast<int>(lanes_as_int->value);
@@ -520,19 +518,19 @@ Ramp::Ramp(PrimExpr base, PrimExpr stride, PrimExpr lanes, Span span) {
   data_ = std::move(node);
 }
 
-TVM_FFI_REGISTER_GLOBAL("tir.Ramp")
-    .set_body_typed([](PrimExpr base, PrimExpr stride, PrimExpr lanes, Span span) {
-      return Ramp(base, stride, lanes, span);
-    });
-
-TVM_REGISTER_NODE_TYPE(RampNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.Ramp", [](PrimExpr base, PrimExpr stride, PrimExpr lanes, Span span) {
+    return Ramp(base, stride, lanes, span);
+  });
+}
 
 // Broadcast
 Broadcast::Broadcast(PrimExpr value, PrimExpr lanes, Span span) {
   ICHECK(value.defined());
   ICHECK(value.dtype().is_scalar());
 
-  ObjectPtr<BroadcastNode> node = make_object<BroadcastNode>();
+  ObjectPtr<BroadcastNode> node = ffi::make_object<BroadcastNode>();
   auto* lanes_int = lanes.as<IntImmNode>();
   if (lanes_int) {
     int lanes = static_cast<int>(lanes_int->value);
@@ -553,12 +551,12 @@ Broadcast::Broadcast(PrimExpr value, PrimExpr lanes, Span span) {
   data_ = node;
 }
 
-TVM_FFI_REGISTER_GLOBAL("tir.Broadcast")
-    .set_body_typed([](PrimExpr value, PrimExpr lanes, Span span) {
-      return Broadcast(value, lanes, span);
-    });
-
-TVM_REGISTER_NODE_TYPE(BroadcastNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.Broadcast", [](PrimExpr value, PrimExpr lanes, Span span) {
+    return Broadcast(value, lanes, span);
+  });
+}
 
 // Let
 Let::Let(Var var, PrimExpr value, PrimExpr body, Span span) {
@@ -566,7 +564,7 @@ Let::Let(Var var, PrimExpr value, PrimExpr body, Span span) {
   ICHECK(body.defined());
   ICHECK_EQ(value.dtype(), var.dtype());
 
-  ObjectPtr<LetNode> node = make_object<LetNode>();
+  ObjectPtr<LetNode> node = ffi::make_object<LetNode>();
   node->dtype = body.dtype();
   node->var = std::move(var);
   node->value = std::move(value);
@@ -575,20 +573,20 @@ Let::Let(Var var, PrimExpr value, PrimExpr body, Span span) {
   data_ = std::move(node);
 }
 
-TVM_FFI_REGISTER_GLOBAL("tir.Let").set_body_typed([](Var var, PrimExpr value, PrimExpr body,
-                                                     Span span) {
-  return Let(var, value, body, span);
-});
-
-TVM_REGISTER_NODE_TYPE(LetNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.Let", [](Var var, PrimExpr value, PrimExpr body, Span span) {
+    return Let(var, value, body, span);
+  });
+}
 
 // Call
-Call::Call(DataType dtype, RelaxExpr op, Array<PrimExpr> args, Span span) {
+Call::Call(DataType dtype, RelaxExpr op, ffi::Array<PrimExpr> args, Span span) {
   for (size_t i = 0; i < args.size(); ++i) {
     ICHECK(args[i].defined()) << "arg " << i << " is not defined()";
   }
 
-  ObjectPtr<CallNode> node = make_object<CallNode>();
+  ObjectPtr<CallNode> node = ffi::make_object<CallNode>();
   node->dtype = dtype;
   node->op = std::move(op);
   node->args = std::move(args);
@@ -596,42 +594,44 @@ Call::Call(DataType dtype, RelaxExpr op, Array<PrimExpr> args, Span span) {
   data_ = std::move(node);
 }
 
-TVM_FFI_REGISTER_GLOBAL("tir.Call")
-    .set_body_typed([](Optional<DataType> dtype, RelaxExpr op,
-                       Array<Variant<String, DLDataType, IterVar, BufferRegion, PrimExpr>> args,
-                       Span span) {
-      Array<PrimExpr> prim_expr_args;
-      for (const auto& it : args) {
-        if (auto opt_str = it.as<String>()) {
-          prim_expr_args.push_back(StringImm(opt_str.value()));
-        } else if (auto opt_dtype = it.as<DLDataType>()) {
-          prim_expr_args.push_back(StringImm(ffi::DLDataTypeToString(opt_dtype.value())));
-        } else if (const auto* iter_var = it.as<IterVarNode>()) {
-          prim_expr_args.push_back(iter_var->var);
-        } else if (const auto* br = it.as<BufferRegionNode>()) {
-          Array<PrimExpr> indices;
-          for (Range r : br->region) {
-            if (is_one(r->extent)) {
-              indices.push_back(r->min);
-            } else if (r->extent.as<IntImmNode>()) {
-              indices.push_back(tir::Ramp(r->min, make_const(r->min->dtype, 1), r->extent));
-            } else {
-              LOG(FATAL) << "ValueError: Cannot convert to BufferLoad: "
-                         << GetRef<BufferRegion>(br);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def(
+      "tir.Call",
+      [](ffi::Optional<DataType> dtype, RelaxExpr op,
+         ffi::Array<ffi::Variant<ffi::String, DLDataType, IterVar, BufferRegion, PrimExpr>> args,
+         Span span) {
+        ffi::Array<PrimExpr> prim_expr_args;
+        for (const auto& it : args) {
+          if (auto opt_str = it.as<ffi::String>()) {
+            prim_expr_args.push_back(StringImm(opt_str.value()));
+          } else if (auto opt_dtype = it.as<DLDataType>()) {
+            prim_expr_args.push_back(StringImm(ffi::DLDataTypeToString(opt_dtype.value())));
+          } else if (const auto* iter_var = it.as<IterVarNode>()) {
+            prim_expr_args.push_back(iter_var->var);
+          } else if (const auto* br = it.as<BufferRegionNode>()) {
+            ffi::Array<PrimExpr> indices;
+            for (Range r : br->region) {
+              if (is_one(r->extent)) {
+                indices.push_back(r->min);
+              } else if (r->extent.as<IntImmNode>()) {
+                indices.push_back(tir::Ramp(r->min, make_const(r->min->dtype, 1), r->extent));
+              } else {
+                LOG(FATAL) << "ValueError: Cannot convert to BufferLoad: "
+                           << ffi::GetRef<BufferRegion>(br);
+              }
             }
+            prim_expr_args.push_back(BufferLoad(br->buffer, indices));
+          } else {
+            prim_expr_args.push_back(Downcast<PrimExpr>(it));
           }
-          prim_expr_args.push_back(BufferLoad(br->buffer, indices));
-        } else {
-          prim_expr_args.push_back(Downcast<PrimExpr>(it));
         }
-      }
-      return Call(dtype.value_or(DataType::Void()), op, prim_expr_args, span);
-    });
-
-TVM_REGISTER_NODE_TYPE(CallNode);
+        return Call(dtype.value_or(DataType::Void()), op, prim_expr_args, span);
+      });
+}
 
 // Shuffle
-Shuffle::Shuffle(Array<PrimExpr> vectors, Array<PrimExpr> indices, Span span) {
+Shuffle::Shuffle(ffi::Array<PrimExpr> vectors, ffi::Array<PrimExpr> indices, Span span) {
   ICHECK_NE(vectors.size(), 0U);
   ICHECK_NE(indices.size(), 0U);
 
@@ -644,7 +644,7 @@ Shuffle::Shuffle(Array<PrimExpr> vectors, Array<PrimExpr> indices, Span span) {
   }
   ICHECK_LE(indices.size(), static_cast<size_t>(total_lanes));
 
-  ObjectPtr<ShuffleNode> node = make_object<ShuffleNode>();
+  ObjectPtr<ShuffleNode> node = ffi::make_object<ShuffleNode>();
   node->dtype = base_type.with_lanes(static_cast<int>(indices.size()));
   node->vectors = std::move(vectors);
   node->indices = std::move(indices);
@@ -652,12 +652,12 @@ Shuffle::Shuffle(Array<PrimExpr> vectors, Array<PrimExpr> indices, Span span) {
   data_ = node;
 }
 
-PrimExpr Shuffle::Concat(Array<PrimExpr> vectors, Span span) {
+PrimExpr Shuffle::Concat(ffi::Array<PrimExpr> vectors, Span span) {
   ICHECK_NE(vectors.size(), 0);
   if (vectors.size() == 1) {
     return vectors[0];
   }
-  Array<PrimExpr> indices;
+  ffi::Array<PrimExpr> indices;
   int index = 0;
   for (const PrimExpr& e : vectors) {
     for (int i = 0; i < e.dtype().lanes(); ++i) {
@@ -671,16 +671,17 @@ PrimExpr Shuffle::ExtractElement(PrimExpr vector, int index, Span span) {
   return Shuffle({vector}, {Integer(index)}, span);
 }
 
-TVM_FFI_REGISTER_GLOBAL("tir.Shuffle")
-    .set_body_typed([](Array<PrimExpr> vectors, Array<PrimExpr> indices, Span span) {
-      return Shuffle(vectors, indices, span);
-    });
-
-TVM_REGISTER_NODE_TYPE(ShuffleNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.Shuffle",
+                        [](ffi::Array<PrimExpr> vectors, ffi::Array<PrimExpr> indices, Span span) {
+                          return Shuffle(vectors, indices, span);
+                        });
+}
 
 // CommReducer
-CommReducer::CommReducer(Array<Var> lhs, Array<Var> rhs, Array<PrimExpr> result,
-                         Array<PrimExpr> identity_element, Span span) {
+CommReducer::CommReducer(ffi::Array<Var> lhs, ffi::Array<Var> rhs, ffi::Array<PrimExpr> result,
+                         ffi::Array<PrimExpr> identity_element, Span span) {
   size_t n_group = result.size();
   CHECK_EQ(lhs.size(), n_group) << "ValueError: The number of vars in `lhs` must equal to the "
                                    "number of elements in `results`";
@@ -710,7 +711,7 @@ CommReducer::CommReducer(Array<Var> lhs, Array<Var> rhs, Array<PrimExpr> result,
     p_result->SetItem(i, Substitute(result[i], var_map));
   }
 
-  auto node = make_object<CommReducerNode>();
+  auto node = ffi::make_object<CommReducerNode>();
   node->lhs = lhs;
   node->rhs = rhs;
   node->result = result;
@@ -719,11 +720,12 @@ CommReducer::CommReducer(Array<Var> lhs, Array<Var> rhs, Array<PrimExpr> result,
   data_ = std::move(node);
 }
 
-Array<PrimExpr> CommReducerNode::operator()(Array<PrimExpr> a, Array<PrimExpr> b) const {
+ffi::Array<PrimExpr> CommReducerNode::operator()(ffi::Array<PrimExpr> a,
+                                                 ffi::Array<PrimExpr> b) const {
   ICHECK_EQ(a.size(), b.size());
   ICHECK_EQ(lhs.size(), a.size());
   ICHECK_EQ(rhs.size(), b.size());
-  Map<Var, PrimExpr> value_map;
+  ffi::Map<Var, PrimExpr> value_map;
   for (size_t i = 0; i < a.size(); ++i) {
     value_map.Set(lhs[i], a[i]);
     value_map.Set(rhs[i], b[i]);
@@ -731,27 +733,26 @@ Array<PrimExpr> CommReducerNode::operator()(Array<PrimExpr> a, Array<PrimExpr> b
   return Substitute(this->result, value_map);
 }
 
-TVM_FFI_REGISTER_GLOBAL("tir.CommReducer")
-    .set_body_typed([](Array<Var> lhs, Array<Var> rhs, Array<PrimExpr> result,
-                       Array<PrimExpr> identity_element, Span span) {
-      return CommReducer(lhs, rhs, result, identity_element, span);
-    });
-
-TVM_FFI_REGISTER_GLOBAL("tir.CommReducerCombine")
-    .set_body_method(&tir::CommReducerNode::operator());
-
-TVM_REGISTER_NODE_TYPE(CommReducerNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def("tir.CommReducer",
+           [](ffi::Array<Var> lhs, ffi::Array<Var> rhs, ffi::Array<PrimExpr> result,
+              ffi::Array<PrimExpr> identity_element,
+              Span span) { return CommReducer(lhs, rhs, result, identity_element, span); })
+      .def_method("tir.CommReducerCombine", &tir::CommReducerNode::operator());
+}
 
 // Reduce
-Reduce::Reduce(CommReducer combiner, Array<PrimExpr> source, Array<IterVar> axis,
-               PrimExpr condition, int value_index, Array<PrimExpr> init, Span span) {
+Reduce::Reduce(CommReducer combiner, ffi::Array<PrimExpr> source, ffi::Array<IterVar> axis,
+               PrimExpr condition, int value_index, ffi::Array<PrimExpr> init, Span span) {
   for (size_t i = 0; i < axis.size(); ++i) {
     ICHECK_EQ(axis[i]->iter_type, kCommReduce) << "Can only take axis created by reduce_axis";
   }
   if (!condition.defined()) {
     condition = const_true();
   }
-  auto n = make_object<ReduceNode>();
+  auto n = ffi::make_object<ReduceNode>();
   ICHECK(source.defined());
   for (size_t i = 0; i < axis.size(); ++i) {
     ICHECK(axis[i].defined());
@@ -777,13 +778,14 @@ Reduce::Reduce(CommReducer combiner, Array<PrimExpr> source, Array<IterVar> axis
   data_ = std::move(n);
 }
 
-TVM_FFI_REGISTER_GLOBAL("tir.Reduce")
-    .set_body_typed([](CommReducer combiner, Array<PrimExpr> source, Array<IterVar> axis,
-                       PrimExpr condition, int value_index, Array<PrimExpr> init, Span span) {
-      return Reduce(combiner, source, axis, condition, value_index, init, span);
-    });
-
-TVM_REGISTER_NODE_TYPE(ReduceNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def(
+      "tir.Reduce", [](CommReducer combiner, ffi::Array<PrimExpr> source, ffi::Array<IterVar> axis,
+                       PrimExpr condition, int value_index, ffi::Array<PrimExpr> init, Span span) {
+        return Reduce(combiner, source, axis, condition, value_index, init, span);
+      });
+}
 
 // BufferLoad
 void BufferLoadNode::LegalizeDType() {
@@ -814,8 +816,8 @@ void BufferLoadNode::LegalizeDType() {
   }
 }
 
-BufferLoad::BufferLoad(Buffer buffer, Array<PrimExpr> indices, Optional<PrimExpr> predicate,
-                       Span span) {
+BufferLoad::BufferLoad(Buffer buffer, ffi::Array<PrimExpr> indices,
+                       ffi::Optional<PrimExpr> predicate, Span span) {
   ICHECK_EQ(buffer->shape.size(), indices.size())
       << "Buffer " << buffer->name << " is " << buffer->shape.size()
       << "-dimensional, cannot be indexed with the " << indices.size()
@@ -843,7 +845,7 @@ BufferLoad::BufferLoad(Buffer buffer, Array<PrimExpr> indices, Optional<PrimExpr
         << ".";
   }
 
-  ObjectPtr<BufferLoadNode> node = make_object<BufferLoadNode>();
+  ObjectPtr<BufferLoadNode> node = ffi::make_object<BufferLoadNode>();
   node->buffer = std::move(buffer);
   node->indices = std::move(indices);
   node->predicate = std::move(predicate);
@@ -852,15 +854,17 @@ BufferLoad::BufferLoad(Buffer buffer, Array<PrimExpr> indices, Optional<PrimExpr
   data_ = std::move(node);
 }
 
-TVM_FFI_REGISTER_GLOBAL("tir.BufferLoad")
-    .set_body_typed([](Buffer buffer, Array<PrimExpr> indices, Optional<PrimExpr> predicate,
-                       Span span) { return BufferLoad(buffer, indices, predicate, span); });
-
-TVM_REGISTER_NODE_TYPE(BufferLoadNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.BufferLoad", [](Buffer buffer, ffi::Array<PrimExpr> indices,
+                                             ffi::Optional<PrimExpr> predicate, Span span) {
+    return BufferLoad(buffer, indices, predicate, span);
+  });
+}
 
 // ProducerLoad
-ProducerLoad::ProducerLoad(DataProducer producer, Array<PrimExpr> indices, Span span) {
-  ObjectPtr<ProducerLoadNode> node = make_object<ProducerLoadNode>();
+ProducerLoad::ProducerLoad(DataProducer producer, ffi::Array<PrimExpr> indices, Span span) {
+  ObjectPtr<ProducerLoadNode> node = ffi::make_object<ProducerLoadNode>();
   node->dtype = producer->GetDataType();
   node->producer = std::move(producer);
   node->indices = std::move(indices);
@@ -868,12 +872,13 @@ ProducerLoad::ProducerLoad(DataProducer producer, Array<PrimExpr> indices, Span
   data_ = std::move(node);
 }
 
-TVM_FFI_REGISTER_GLOBAL("tir.ProducerLoad")
-    .set_body_typed([](DataProducer producer, Array<PrimExpr> indices, Span span) {
-      return ProducerLoad(producer, indices, span);
-    });
-
-TVM_REGISTER_NODE_TYPE(ProducerLoadNode);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.ProducerLoad",
+                        [](DataProducer producer, ffi::Array<PrimExpr> indices, Span span) {
+                          return ProducerLoad(producer, indices, span);
+                        });
+}
 
 }  // namespace tir
 }  // namespace tvm
