@@ -23,7 +23,7 @@
  *        StmtExprVisitor/StmtExprMutator.
  */
 
-#include <tvm/ffi/reflection/reflection.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/tir/expr_functor.h>
 #include <tvm/tir/stmt_functor.h>
 
@@ -215,11 +215,12 @@ class PyStmtExprVisitorNode : public Object, public StmtExprVisitor {
   }
 
   static void RegisterReflection() {
-    // No fields to register as they are not visited
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<PyStmtExprVisitorNode>();
   }
 
-  static constexpr const char* _type_key = "tir.PyStmtExprVisitor";
-  TVM_DECLARE_BASE_OBJECT_INFO(PyStmtExprVisitorNode, Object);
+  static constexpr const bool _type_mutable = true;
+  TVM_FFI_DECLARE_OBJECT_INFO("tir.PyStmtExprVisitor", PyStmtExprVisitorNode, Object);
 
  private:
   // Statement functions
@@ -342,6 +343,9 @@ class PyStmtExprVisitorNode : public Object, public StmtExprVisitor {
  */
 class PyStmtExprVisitor : public ObjectRef {
  public:
+  explicit PyStmtExprVisitor(ObjectPtr<PyStmtExprVisitorNode> data) : ObjectRef(data) {
+    TVM_FFI_ICHECK(data != nullptr);
+  }
   TVM_DLL static PyStmtExprVisitor MakePyStmtExprVisitor(ffi::Function f_visit_stmt,            //
                                                          ffi::Function f_visit_expr,            //
                                                          ffi::Function f_visit_let_stmt,        //
@@ -392,7 +396,7 @@ class PyStmtExprVisitor : public ObjectRef {
                                                          ffi::Function f_visit_int_imm,         //
                                                          ffi::Function f_visit_float_imm,       //
                                                          ffi::Function f_visit_string_imm) {
-    ObjectPtr<PyStmtExprVisitorNode> n = make_object<PyStmtExprVisitorNode>();
+    ObjectPtr<PyStmtExprVisitorNode> n = ffi::make_object<PyStmtExprVisitorNode>();
     n->f_visit_stmt = std::move(f_visit_stmt);
     n->f_visit_expr = std::move(f_visit_expr);
     // Set statement functions
@@ -448,8 +452,8 @@ class PyStmtExprVisitor : public ObjectRef {
     return PyStmtExprVisitor(n);
   }
 
-  TVM_DEFINE_MUTABLE_NOTNULLABLE_OBJECT_REF_METHODS(PyStmtExprVisitor, ObjectRef,
-                                                    PyStmtExprVisitorNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(PyStmtExprVisitor, ObjectRef,
+                                                PyStmtExprVisitorNode);
 };
 
 /*! \brief The python interface of StmtExprMutator. */
@@ -578,11 +582,12 @@ class PyStmtExprMutatorNode : public Object, public StmtExprMutator {
   }
 
   static void RegisterReflection() {
-    // No fields to register as they are not visited
+    namespace refl = tvm::ffi::reflection;
+    refl::ObjectDef<PyStmtExprMutatorNode>();
   }
 
-  static constexpr const char* _type_key = "tir.PyStmtExprMutator";
-  TVM_DECLARE_BASE_OBJECT_INFO(PyStmtExprMutatorNode, Object);
+  static constexpr const bool _type_mutable = true;
+  TVM_FFI_DECLARE_OBJECT_INFO("tir.PyStmtExprMutator", PyStmtExprMutatorNode, Object);
 
  private:
   // Statement functions
@@ -702,6 +707,9 @@ class PyStmtExprMutatorNode : public Object, public StmtExprMutator {
 /*! \brief Managed reference to PyStmtExprMutatorNode. */
 class PyStmtExprMutator : public ObjectRef {
  public:
+  explicit PyStmtExprMutator(ObjectPtr<PyStmtExprMutatorNode> data) : ObjectRef(data) {
+    TVM_FFI_ICHECK(data != nullptr);
+  }
   /*!
    * \brief Create a PyStmtExprMutator with customized methods on the python-side.
    * \return The PyStmtExprMutator created.
@@ -756,7 +764,7 @@ class PyStmtExprMutator : public ObjectRef {
                                                          ffi::Function f_visit_int_imm,         //
                                                          ffi::Function f_visit_float_imm,       //
                                                          ffi::Function f_visit_string_imm) {
-    ObjectPtr<PyStmtExprMutatorNode> n = make_object<PyStmtExprMutatorNode>();
+    ObjectPtr<PyStmtExprMutatorNode> n = ffi::make_object<PyStmtExprMutatorNode>();
     n->f_visit_stmt = std::move(f_visit_stmt);
     n->f_visit_expr = std::move(f_visit_expr);
     // Statement functions
@@ -812,60 +820,57 @@ class PyStmtExprMutator : public ObjectRef {
     return PyStmtExprMutator(n);
   }
 
-  TVM_DEFINE_MUTABLE_NOTNULLABLE_OBJECT_REF_METHODS(PyStmtExprMutator, ObjectRef,
-                                                    PyStmtExprMutatorNode);
+  TVM_FFI_DEFINE_OBJECT_REF_METHODS_NOTNULLABLE(PyStmtExprMutator, ObjectRef,
+                                                PyStmtExprMutatorNode);
 };
 
 // ================================================
 // TVM Register
 // ================================================
 
-TVM_FFI_STATIC_INIT_BLOCK({
+TVM_FFI_STATIC_INIT_BLOCK() {
   PyStmtExprVisitorNode::RegisterReflection();
   PyStmtExprMutatorNode::RegisterReflection();
-});
+}
 
-TVM_REGISTER_NODE_TYPE(PyStmtExprVisitorNode);
-TVM_REGISTER_NODE_TYPE(PyStmtExprMutatorNode);
-
-TVM_FFI_REGISTER_GLOBAL("tir.MakePyStmtExprVisitor")
-    .set_body_typed(PyStmtExprVisitor::MakePyStmtExprVisitor);
-TVM_FFI_REGISTER_GLOBAL("tir.MakePyStmtExprMutator")
-    .set_body_typed(PyStmtExprMutator::MakePyStmtExprMutator);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def("tir.MakePyStmtExprVisitor", PyStmtExprVisitor::MakePyStmtExprVisitor)
+      .def("tir.MakePyStmtExprMutator", PyStmtExprMutator::MakePyStmtExprMutator);
+}
 
 // StmtExprVisitor
-TVM_FFI_REGISTER_GLOBAL("tir.PyStmtExprVisitorDefaultVisitExpr")
-    .set_body_typed([](PyStmtExprVisitor visitor, const PrimExpr& expr) {
-      visitor->DefaultVisitExpr(expr);
-    });
-TVM_FFI_REGISTER_GLOBAL("tir.PyStmtExprVisitorDefaultVisitStmt")
-    .set_body_typed([](PyStmtExprVisitor visitor, const Stmt& stmt) {
-      visitor->DefaultVisitStmt(stmt);
-    });
-TVM_FFI_REGISTER_GLOBAL("tir.PyStmtExprVisitorVisitStmt")
-    .set_body_typed([](PyStmtExprVisitor visitor, const Stmt& stmt) { visitor->VisitStmt(stmt); });
-TVM_FFI_REGISTER_GLOBAL("tir.PyStmtExprVisitorVisitExpr")
-    .set_body_typed([](PyStmtExprVisitor visitor, const PrimExpr& expr) {
-      visitor->VisitExpr(expr);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def("tir.PyStmtExprVisitorDefaultVisitExpr",
+           [](PyStmtExprVisitor visitor, const PrimExpr& expr) { visitor->DefaultVisitExpr(expr); })
+      .def("tir.PyStmtExprVisitorDefaultVisitStmt",
+           [](PyStmtExprVisitor visitor, const Stmt& stmt) { visitor->DefaultVisitStmt(stmt); })
+      .def("tir.PyStmtExprVisitorVisitStmt",
+           [](PyStmtExprVisitor visitor, const Stmt& stmt) { visitor->VisitStmt(stmt); })
+      .def("tir.PyStmtExprVisitorVisitExpr",
+           [](PyStmtExprVisitor visitor, const PrimExpr& expr) { visitor->VisitExpr(expr); });
+}
 
 // StmtExprMutator
-TVM_FFI_REGISTER_GLOBAL("tir.PyStmtExprMutatorDefaultVisitExpr")
-    .set_body_typed([](PyStmtExprMutator mutator, const PrimExpr& expr) {
-      return mutator->DefaultVisitExpr(expr);
-    });
-TVM_FFI_REGISTER_GLOBAL("tir.PyStmtExprMutatorDefaultVisitStmt")
-    .set_body_typed([](PyStmtExprMutator mutator, const Stmt& stmt) {
-      return mutator->DefaultVisitStmt(stmt);
-    });
-TVM_FFI_REGISTER_GLOBAL("tir.PyStmtExprMutatorVisitExpr")
-    .set_body_typed([](PyStmtExprMutator mutator, const PrimExpr& expr) {
-      return mutator->VisitExpr(expr);
-    });
-TVM_FFI_REGISTER_GLOBAL("tir.PyStmtExprMutatorVisitStmt")
-    .set_body_typed([](PyStmtExprMutator mutator, const Stmt& stmt) {
-      return mutator->VisitStmt(stmt);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def("tir.PyStmtExprMutatorDefaultVisitExpr",
+           [](PyStmtExprMutator mutator, const PrimExpr& expr) {
+             return mutator->DefaultVisitExpr(expr);
+           })
+      .def("tir.PyStmtExprMutatorDefaultVisitStmt",
+           [](PyStmtExprMutator mutator, const Stmt& stmt) {
+             return mutator->DefaultVisitStmt(stmt);
+           })
+      .def("tir.PyStmtExprMutatorVisitExpr",
+           [](PyStmtExprMutator mutator, const PrimExpr& expr) { return mutator->VisitExpr(expr); })
+      .def("tir.PyStmtExprMutatorVisitStmt",
+           [](PyStmtExprMutator mutator, const Stmt& stmt) { return mutator->VisitStmt(stmt); });
+}
 
 }  // namespace tir
 }  // namespace tvm
