@@ -16,24 +16,26 @@
  * specific language governing permissions and limitations
  * under the License.
  */
+#include <tvm/ffi/reflection/registry.h>
+
 #include "./utils.h"
 
 namespace tvm {
 namespace tir {
 
-TVM_FFI_STATIC_INIT_BLOCK({
+TVM_FFI_STATIC_INIT_BLOCK() {
   InstructionKindNode::RegisterReflection();
   InstructionNode::RegisterReflection();
-});
+}
 
 bool InstructionKindNode::IsPostproc() const {
   static InstructionKind inst_enter_postproc = InstructionKind::Get("EnterPostproc");
   return this == inst_enter_postproc.get();
 }
 
-Instruction::Instruction(InstructionKind kind, Array<Any> inputs, Array<Any> attrs,
-                         Array<Any> outputs) {
-  ObjectPtr<InstructionNode> n = make_object<InstructionNode>();
+Instruction::Instruction(InstructionKind kind, ffi::Array<Any> inputs, ffi::Array<Any> attrs,
+                         ffi::Array<Any> outputs) {
+  ObjectPtr<InstructionNode> n = ffi::make_object<InstructionNode>();
   n->kind = std::move(kind);
   n->inputs = std::move(inputs);
   n->attrs = std::move(attrs);
@@ -43,17 +45,17 @@ Instruction::Instruction(InstructionKind kind, Array<Any> inputs, Array<Any> att
 
 using InstructionKindRegistry = AttrRegistry<InstructionKindRegEntry, InstructionKind>;
 
-InstructionKind InstructionKind::Get(const String& name) {
+InstructionKind InstructionKind::Get(const ffi::String& name) {
   const InstructionKindRegEntry* reg = InstructionKindRegistry::Global()->Get(name);
   ICHECK(reg != nullptr) << "AttributeError: Instruction kind " << name << " is not registered";
   return reg->inst_kind_;
 }
 
 InstructionKindRegEntry::InstructionKindRegEntry(uint32_t reg_index) {
-  this->inst_kind_ = InstructionKind(make_object<InstructionKindNode>());
+  this->inst_kind_ = InstructionKind(ffi::make_object<InstructionKindNode>());
 }
 
-InstructionKindRegEntry& InstructionKindRegEntry::RegisterOrGet(const String& name) {
+InstructionKindRegEntry& InstructionKindRegEntry::RegisterOrGet(const ffi::String& name) {
   return InstructionKindRegistry::Global()->RegisterOrGet(name);
 }
 
@@ -63,29 +65,29 @@ TVM_STATIC_IR_FUNCTOR(ReprPrinter, vtable)
     .set_dispatch<InstructionNode>([](const ObjectRef& obj, ReprPrinter* p) {
       const auto* self = obj.as<InstructionNode>();
       ICHECK_NOTNULL(self);
-      Array<Any> inputs;
+      ffi::Array<Any> inputs;
       inputs.reserve(self->inputs.size());
       for (const Any& obj : self->inputs) {
         if (obj == nullptr) {
-          inputs.push_back(String("None"));
+          inputs.push_back(ffi::String("None"));
+        } else if (auto opt_str = obj.as<ffi::String>()) {
+          inputs.push_back(ffi::String('"' + (*opt_str).operator std::string() + '"'));
         } else if (obj.as<BlockRVNode>() || obj.as<LoopRVNode>()) {
-          inputs.push_back(String("_"));
-        } else if (const auto* str_obj = obj.as<ffi::StringObj>()) {
-          inputs.push_back(String('"' + std::string(str_obj->data) + '"'));
-        } else if (obj.type_index() < ffi::TypeIndex::kTVMFFIStaticObjectBegin) {
+          inputs.push_back(ffi::String("_"));
+        } else if (obj.type_index() < ffi::TypeIndex::kTVMFFISmallStr) {
           inputs.push_back(obj);
         } else if (obj.as<IntImmNode>() || obj.as<FloatImmNode>()) {
           inputs.push_back(obj);
         } else if (const auto* expr = obj.as<PrimExprNode>()) {
-          PrimExpr new_expr =
-              Substitute(GetRef<PrimExpr>(expr), [](const Var& var) -> Optional<PrimExpr> {
-                ObjectPtr<VarNode> new_var = make_object<VarNode>(*var.get());
+          PrimExpr new_expr = Substitute(
+              ffi::GetRef<PrimExpr>(expr), [](const Var& var) -> ffi::Optional<PrimExpr> {
+                ObjectPtr<VarNode> new_var = ffi::make_object<VarNode>(*var.get());
                 new_var->name_hint = "_";
                 return Var(new_var);
               });
           std::ostringstream os;
           os << new_expr;
-          inputs.push_back(String(os.str()));
+          inputs.push_back(ffi::String(os.str()));
         } else if (obj.as<IndexMapNode>()) {
           inputs.push_back(obj);
         } else {
@@ -97,20 +99,21 @@ TVM_STATIC_IR_FUNCTOR(ReprPrinter, vtable)
           /*inputs=*/inputs,
           /*attrs=*/self->attrs,
           /*decision=*/Any(nullptr),
-          /*outputs=*/Array<String>(self->outputs.size(), String("_")));
+          /*outputs=*/ffi::Array<ffi::String>(self->outputs.size(), ffi::String("_")));
     });
 
 /**************** FFI ****************/
 
-TVM_REGISTER_NODE_TYPE(InstructionNode);
-TVM_REGISTER_NODE_TYPE(InstructionKindNode);
-
-TVM_FFI_REGISTER_GLOBAL("tir.schedule.InstructionKindGet").set_body_typed(InstructionKind::Get);
-TVM_FFI_REGISTER_GLOBAL("tir.schedule.Instruction")
-    .set_body_typed([](InstructionKind kind, Array<Any> inputs, Array<Any> attrs,
-                       Array<Any> outputs) -> Instruction {
-      return Instruction(kind, inputs, attrs, outputs);
-    });
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef()
+      .def("tir.schedule.InstructionKindGet", InstructionKind::Get)
+      .def("tir.schedule.Instruction",
+           [](InstructionKind kind, ffi::Array<Any> inputs, ffi::Array<Any> attrs,
+              ffi::Array<Any> outputs) -> Instruction {
+             return Instruction(kind, inputs, attrs, outputs);
+           });
+}
 
 }  // namespace tir
 }  // namespace tvm
