@@ -21,9 +21,10 @@
 namespace tvm {
 namespace tir {
 
-void Annotate(ScheduleState self, const StmtSRef& sref, const String& ann_key, const Any& ann_val) {
+void Annotate(ScheduleState self, const StmtSRef& sref, const ffi::String& ann_key,
+              const Any& ann_val) {
   // Extract annotation
-  const Map<String, ffi::Any>* annotations = nullptr;
+  const ffi::Map<ffi::String, ffi::Any>* annotations = nullptr;
   if (const auto* loop = sref->StmtAs<ForNode>()) {
     annotations = &loop->annotations;
   } else if (const auto* block = sref->StmtAs<BlockNode>()) {
@@ -36,27 +37,27 @@ void Annotate(ScheduleState self, const StmtSRef& sref, const String& ann_key, c
     return;
   }
   // Add the new annotation
-  Map<String, ffi::Any> new_ann(*annotations);
+  ffi::Map<ffi::String, ffi::Any> new_ann(*annotations);
   new_ann.Set(ann_key, ann_val);
   // Create the new stmt
   if (const auto* loop = sref->StmtAs<ForNode>()) {
-    ObjectPtr<ForNode> n = make_object<ForNode>(*loop);
+    ObjectPtr<ForNode> n = ffi::make_object<ForNode>(*loop);
     n->annotations = std::move(new_ann);
     self->Replace(sref, For(n), {});
   } else if (const auto* block = sref->StmtAs<BlockNode>()) {
-    ObjectPtr<BlockNode> n = make_object<BlockNode>(*block);
+    ObjectPtr<BlockNode> n = ffi::make_object<BlockNode>(*block);
     n->annotations = std::move(new_ann);
     Block p(n);
-    self->Replace(sref, p, {{GetRef<Block>(block), p}});
+    self->Replace(sref, p, {{ffi::GetRef<Block>(block), p}});
   } else {
     LOG(FATAL) << "TypeError: Unknown type of sref: " << sref->stmt->GetTypeKey();
     throw;
   }
 }
 
-void Unannotate(ScheduleState self, const StmtSRef& sref, const String& ann_key) {
+void Unannotate(ScheduleState self, const StmtSRef& sref, const ffi::String& ann_key) {
   // Extract annotation
-  const Map<String, ffi::Any>* annotations = nullptr;
+  const ffi::Map<ffi::String, ffi::Any>* annotations = nullptr;
   if (const auto* loop = sref->StmtAs<ForNode>()) {
     annotations = &loop->annotations;
   } else if (const auto* block = sref->StmtAs<BlockNode>()) {
@@ -67,18 +68,18 @@ void Unannotate(ScheduleState self, const StmtSRef& sref, const String& ann_key)
   // Remove the annotation
   ICHECK(annotations->find(ann_key) != annotations->end())
       << "IndexError: Cannot find annotation key: " << ann_key;
-  Map<String, ffi::Any> new_ann(*annotations);
+  ffi::Map<ffi::String, ffi::Any> new_ann(*annotations);
   new_ann.erase(ann_key);
   // Create the new stmt
   if (const auto* loop = sref->StmtAs<ForNode>()) {
-    ObjectPtr<ForNode> n = make_object<ForNode>(*loop);
+    ObjectPtr<ForNode> n = ffi::make_object<ForNode>(*loop);
     n->annotations = std::move(new_ann);
     self->Replace(sref, For(n), {});
   } else if (const auto* block = sref->StmtAs<BlockNode>()) {
-    ObjectPtr<BlockNode> n = make_object<BlockNode>(*block);
+    ObjectPtr<BlockNode> n = ffi::make_object<BlockNode>(*block);
     n->annotations = std::move(new_ann);
     Block p(n);
-    self->Replace(sref, p, {{GetRef<Block>(block), p}});
+    self->Replace(sref, p, {{ffi::GetRef<Block>(block), p}});
   } else {
     LOG(FATAL) << "TypeError: Unknown type of sref: " << sref->stmt->GetTypeKey();
     throw;
@@ -95,7 +96,7 @@ struct AnnotateTraits : public UnpackedInstTraits<AnnotateTraits> {
   static constexpr size_t kNumDecisions = 0;
 
   static void UnpackedApplyToSchedule(Schedule sch, ObjectRef block_or_loop_rv, Any ann_val,
-                                      String ann_key) {
+                                      ffi::String ann_key) {
     if (auto block = block_or_loop_rv.as<BlockRV>()) {
       return sch->Annotate(block.value(), ann_key, ann_val);
     }
@@ -106,8 +107,8 @@ struct AnnotateTraits : public UnpackedInstTraits<AnnotateTraits> {
     throw;
   }
 
-  static String UnpackedAsPython(Array<String> outputs, ObjectRef block_or_loop_rv, Any ann_val,
-                                 String ann_key) {
+  static ffi::String UnpackedAsPython(ffi::Array<ffi::String> outputs, ObjectRef block_or_loop_rv,
+                                      Any ann_val, ffi::String ann_key) {
     PythonAPICall py("annotate");
     py.Input("block_or_loop", block_or_loop_rv);
     py.Input("ann_key", ann_key);
@@ -128,7 +129,8 @@ struct UnannotateTraits : public UnpackedInstTraits<UnannotateTraits> {
   static constexpr size_t kNumAttrs = 1;
   static constexpr size_t kNumDecisions = 0;
 
-  static void UnpackedApplyToSchedule(Schedule sch, ObjectRef block_or_loop_rv, String ann_key) {
+  static void UnpackedApplyToSchedule(Schedule sch, ObjectRef block_or_loop_rv,
+                                      ffi::String ann_key) {
     if (auto block = block_or_loop_rv.as<BlockRV>()) {
       return sch->Unannotate(block.value(), ann_key);
     }
@@ -139,8 +141,8 @@ struct UnannotateTraits : public UnpackedInstTraits<UnannotateTraits> {
     throw;
   }
 
-  static String UnpackedAsPython(Array<String> outputs, ObjectRef block_or_loop_rv,
-                                 String ann_key) {
+  static ffi::String UnpackedAsPython(ffi::Array<ffi::String> outputs, ObjectRef block_or_loop_rv,
+                                      ffi::String ann_key) {
     PythonAPICall py("unannotate");
     py.Input("block_or_loop", block_or_loop_rv);
     py.Input("ann_key", ann_key);
