@@ -30,13 +30,13 @@ class StorageAlignAxisOutOfRangeError : public ScheduleError {
   explicit StorageAlignAxisOutOfRangeError(IRModule mod, Buffer buffer, int axis)
       : mod_(std::move(mod)), buffer_(std::move(buffer)), axis_(axis) {}
 
-  String FastErrorString() const final {
+  ffi::String FastErrorString() const final {
     return "ScheduleError: The input `axis` is out of range. It is required to be in range "
            "[-ndim, ndim) where `ndim` is the number of dimensions of the buffer to set "
            "storage alignment.";
   }
 
-  String DetailRenderTemplate() const final {
+  ffi::String DetailRenderTemplate() const final {
     std::ostringstream os;
     int ndim = static_cast<int>(buffer_->shape.size());
     os << "The buffer to set storage alignment of, " << buffer_->name << ", has " << ndim
@@ -47,7 +47,7 @@ class StorageAlignAxisOutOfRangeError : public ScheduleError {
   }
 
   IRModule mod() const final { return mod_; }
-  Array<ObjectRef> LocationsOfInterest() const final { return {}; }
+  ffi::Array<ObjectRef> LocationsOfInterest() const final { return {}; }
 
   static int CheckAndUpdate(const IRModule& mod, const Buffer& buffer, int axis) {
     int ndim = static_cast<int>(buffer->shape.size());
@@ -71,12 +71,12 @@ class NonAllocatedBufferError : public ScheduleError {
  public:
   explicit NonAllocatedBufferError(IRModule mod, Buffer buffer) : mod_(mod), buffer_(buffer) {}
 
-  String FastErrorString() const final {
+  ffi::String FastErrorString() const final {
     return "ScheduleError: The input buffer is not allocated by a block. This means the buffer is "
            " either a function parameter or defined in `match_buffer` of a block.";
   }
 
-  String DetailRenderTemplate() const final {
+  ffi::String DetailRenderTemplate() const final {
     std::ostringstream os;
     os << "The input buffer " << buffer_->name
        << " is not allocated by a block. This means the buffer is either a function parameter or "
@@ -94,7 +94,7 @@ class NonAllocatedBufferError : public ScheduleError {
     return defining_site_sref.value();
   }
 
-  Array<ObjectRef> LocationsOfInterest() const final { return {}; }
+  ffi::Array<ObjectRef> LocationsOfInterest() const final { return {}; }
   IRModule mod() const final { return mod_; }
 
  private:
@@ -107,12 +107,12 @@ class StorageAlignInvalidFactorError : public ScheduleError {
   explicit StorageAlignInvalidFactorError(IRModule mod, int factor)
       : mod_(std::move(mod)), factor_(factor) {}
 
-  String FastErrorString() const final {
+  ffi::String FastErrorString() const final {
     return "ScheduleError: The input `factor` of storage_align is expected to be a positive "
            "number.";
   }
 
-  String DetailRenderTemplate() const final {
+  ffi::String DetailRenderTemplate() const final {
     std::ostringstream os;
     os << "The input `factor` of storage_align is expected to be a positive number. However, the "
           "input `factor` is "
@@ -126,7 +126,7 @@ class StorageAlignInvalidFactorError : public ScheduleError {
     }
   }
 
-  Array<ObjectRef> LocationsOfInterest() const final { return {}; }
+  ffi::Array<ObjectRef> LocationsOfInterest() const final { return {}; }
   IRModule mod() const final { return mod_; }
 
  private:
@@ -139,12 +139,12 @@ class StorageAlignInvalidAnnotationError : public ScheduleError {
   explicit StorageAlignInvalidAnnotationError(IRModule mod, Block block)
       : mod_(std::move(mod)), block_(std::move(block)) {}
 
-  String FastErrorString() const final {
+  ffi::String FastErrorString() const final {
     return "ScheduleError: The block annotation for storage align is expected to be an array of "
            "4-integer-tuples (buffer_index, axis, factor, offset).";
   }
 
-  String DetailRenderTemplate() const final {
+  ffi::String DetailRenderTemplate() const final {
     std::ostringstream os;
     os << "The block annotation for storage align is expected to be an array of 4-integer-tuples "
           "(buffer_index, axis, factor, offset). However, the block annotation with key "
@@ -168,7 +168,7 @@ class StorageAlignInvalidAnnotationError : public ScheduleError {
     return storage_align_annotation;
   }
 
-  Array<ObjectRef> LocationsOfInterest() const final { return {block_}; }
+  ffi::Array<ObjectRef> LocationsOfInterest() const final { return {block_}; }
   IRModule mod() const final { return mod_; }
 
  private:
@@ -194,7 +194,7 @@ class StorageScopeMutator : private ReplaceBufferMutator {
    * \return The new block after the mutation
    */
   static Block Mutate(const Block& allocate_site, const Buffer& old_buffer,
-                      const String& storage_scope, Map<Block, Block>* block_sref_reuse) {
+                      const ffi::String& storage_scope, ffi::Map<Block, Block>* block_sref_reuse) {
     Buffer new_buffer = WithScope(old_buffer, storage_scope);
     StorageScopeMutator mutator(old_buffer, new_buffer, storage_scope, block_sref_reuse);
     Stmt new_block = mutator.VisitStmt(allocate_site);
@@ -202,8 +202,8 @@ class StorageScopeMutator : private ReplaceBufferMutator {
   }
 
  private:
-  StorageScopeMutator(const Buffer& old_buffer, Buffer new_buffer, String storage_scope,
-                      Map<Block, Block>* block_sref_reuse)
+  StorageScopeMutator(const Buffer& old_buffer, Buffer new_buffer, ffi::String storage_scope,
+                      ffi::Map<Block, Block>* block_sref_reuse)
       : ReplaceBufferMutator(old_buffer, std::move(new_buffer), block_sref_reuse) {}
 
   MatchBufferRegion VisitMatchBufferRegion(const MatchBufferRegion& match_buffer) final {
@@ -222,8 +222,8 @@ class StorageScopeMutator : private ReplaceBufferMutator {
 void StorageAlign(ScheduleState self, const StmtSRef& block_sref, int buffer_index, int axis,
                   int factor, int offset) {
   const BlockNode* block_ptr = TVM_SREF_TO_BLOCK(block_sref);
-  Buffer buffer =
-      GetNthAccessBuffer(self, GetRef<Block>(block_ptr), buffer_index, BufferIndexType::kWrite);
+  Buffer buffer = GetNthAccessBuffer(self, ffi::GetRef<Block>(block_ptr), buffer_index,
+                                     BufferIndexType::kWrite);
   StorageAlignInvalidFactorError::Check(self->mod, factor);
   axis = StorageAlignAxisOutOfRangeError::CheckAndUpdate(self->mod, buffer, axis);
   NonAllocatedBufferError::CheckAndGetBufferAllocationSite(self->mod, block_sref, buffer);
@@ -231,7 +231,7 @@ void StorageAlign(ScheduleState self, const StmtSRef& block_sref, int buffer_ind
   // Step 1: Get existing or create new annotation value.
   StorageAlignAnnotation storage_align_annotation =
       StorageAlignInvalidAnnotationError::CheckAndGetAnnotation(self->mod,
-                                                                GetRef<Block>(block_ptr));
+                                                                ffi::GetRef<Block>(block_ptr));
 
   // Step 2: Update the annotation value
   bool found = false;
@@ -250,14 +250,14 @@ void StorageAlign(ScheduleState self, const StmtSRef& block_sref, int buffer_ind
 
   // Step 3: Replace the block with the new annotation
   Block new_block = WithAnnotation(block_ptr, attr::buffer_dim_align, storage_align_annotation);
-  self->Replace(block_sref, new_block, {{GetRef<Block>(block_ptr), new_block}});
+  self->Replace(block_sref, new_block, {{ffi::GetRef<Block>(block_ptr), new_block}});
 }
 
 void SetScope(ScheduleState self, const StmtSRef& block_sref, int buffer_index,
-              const String& storage_scope) {
+              const ffi::String& storage_scope) {
   const BlockNode* block = TVM_SREF_TO_BLOCK(block_sref);
   Buffer buffer =
-      GetNthAccessBuffer(self, GetRef<Block>(block), buffer_index, BufferIndexType::kWrite);
+      GetNthAccessBuffer(self, ffi::GetRef<Block>(block), buffer_index, BufferIndexType::kWrite);
 
   // Step 1. If `storage_scope` equals the original storage scope of the buffer, just return.
   if (buffer.scope() == storage_scope) {
@@ -274,9 +274,9 @@ void SetScope(ScheduleState self, const StmtSRef& block_sref, int buffer_index,
 
   // Step 4. Recursively replace the old buffer to a new buffer, where the new buffer has the given
   // storage scope. In the meanwhile, collect the block sref reuse information.
-  Map<Block, Block> block_reuse_map;
-  Block new_block = StorageScopeMutator::Mutate(GetRef<Block>(alloc_site), buffer, storage_scope,
-                                                &block_reuse_map);
+  ffi::Map<Block, Block> block_reuse_map;
+  Block new_block = StorageScopeMutator::Mutate(ffi::GetRef<Block>(alloc_site), buffer,
+                                                storage_scope, &block_reuse_map);
   self->Replace(alloc_site_sref, new_block, block_reuse_map);
 }
 
@@ -294,7 +294,7 @@ class DTypeMutator : private ReplaceBufferMutator {
    * \return The new block after the mutation
    */
   static Block Mutate(const Block& allocate_site, const Buffer& old_buffer, const DataType& dtype,
-                      Map<Block, Block>* block_sref_reuse) {
+                      ffi::Map<Block, Block>* block_sref_reuse) {
     Buffer new_buffer = WithDType(old_buffer, dtype);
     DTypeMutator mutator(old_buffer, new_buffer, dtype, block_sref_reuse);
     Stmt new_block = mutator.VisitStmt(allocate_site);
@@ -303,7 +303,7 @@ class DTypeMutator : private ReplaceBufferMutator {
 
  private:
   DTypeMutator(const Buffer& old_buffer, Buffer new_buffer, const DataType& dtype,
-               Map<Block, Block>* block_sref_reuse)
+               ffi::Map<Block, Block>* block_sref_reuse)
       : ReplaceBufferMutator(old_buffer, std::move(new_buffer), block_sref_reuse),
         src_dtype_(old_buffer->dtype),
         tgt_dtype_(dtype) {}
@@ -343,11 +343,11 @@ class DTypeMutator : private ReplaceBufferMutator {
 };
 
 void UnsafeSetDType(ScheduleState self, const StmtSRef& block_sref, int buffer_index,
-                    const String& dtype) {
+                    const ffi::String& dtype) {
   const BlockNode* block = TVM_SREF_TO_BLOCK(block_sref);
   Buffer buffer =
-      GetNthAccessBuffer(self, GetRef<Block>(block), buffer_index, BufferIndexType::kWrite);
-  DataType target_dtype(StringToDLDataType(dtype));
+      GetNthAccessBuffer(self, ffi::GetRef<Block>(block), buffer_index, BufferIndexType::kWrite);
+  DataType target_dtype(ffi::StringToDLDataType(dtype));
 
   // Step 1. If `dtype` equals the original data type, just return.
   if (buffer->dtype == target_dtype) {
@@ -361,9 +361,9 @@ void UnsafeSetDType(ScheduleState self, const StmtSRef& block_sref, int buffer_i
 
   // Step 3. Recursively replace old buffer to a new buffer, where the new buffer has the given
   // dtype, and insert data type conversions.
-  Map<Block, Block> block_reuse_map;
+  ffi::Map<Block, Block> block_reuse_map;
   Block new_block =
-      DTypeMutator::Mutate(GetRef<Block>(alloc_site), buffer, target_dtype, &block_reuse_map);
+      DTypeMutator::Mutate(ffi::GetRef<Block>(alloc_site), buffer, target_dtype, &block_reuse_map);
   self->Replace(alloc_site_sref, new_block, block_reuse_map);
 }
 
@@ -384,8 +384,9 @@ struct StorageAlignTraits : public UnpackedInstTraits<StorageAlignTraits> {
                              offset->value);
   }
 
-  static String UnpackedAsPython(Array<String> outputs, String block_rv, Integer buffer_index,
-                                 Integer axis, Integer factor, Integer offset) {
+  static ffi::String UnpackedAsPython(ffi::Array<ffi::String> outputs, ffi::String block_rv,
+                                      Integer buffer_index, Integer axis, Integer factor,
+                                      Integer offset) {
     PythonAPICall py("storage_align");
     py.Input("block", block_rv);
     py.Input("buffer_index", buffer_index);
@@ -409,12 +410,12 @@ struct SetScopeTraits : public UnpackedInstTraits<SetScopeTraits> {
   static constexpr size_t kNumDecisions = 0;
 
   static void UnpackedApplyToSchedule(Schedule sch, BlockRV block_rv, Integer buffer_index,
-                                      String storage_scope) {
+                                      ffi::String storage_scope) {
     return sch->SetScope(block_rv, buffer_index->value, storage_scope);
   }
 
-  static String UnpackedAsPython(Array<String> outputs, String block_rv, Integer buffer_index,
-                                 String storage_scope) {
+  static ffi::String UnpackedAsPython(ffi::Array<ffi::String> outputs, ffi::String block_rv,
+                                      Integer buffer_index, ffi::String storage_scope) {
     PythonAPICall py("set_scope");
     py.Input("block", block_rv);
     py.Input("buffer_index", buffer_index);
@@ -436,12 +437,12 @@ struct UnsafeSetDTypeTraits : public UnpackedInstTraits<UnsafeSetDTypeTraits> {
   static constexpr size_t kNumDecisions = 0;
 
   static void UnpackedApplyToSchedule(Schedule sch, BlockRV block_rv, Integer buffer_index,
-                                      String dtype) {
+                                      ffi::String dtype) {
     return sch->UnsafeSetDType(block_rv, buffer_index->value, dtype);
   }
 
-  static String UnpackedAsPython(Array<String> outputs, String block_rv, Integer buffer_index,
-                                 String dtype) {
+  static ffi::String UnpackedAsPython(ffi::Array<ffi::String> outputs, ffi::String block_rv,
+                                      Integer buffer_index, ffi::String dtype) {
     PythonAPICall py("unsafe_set_dtype");
     py.Input("block", block_rv);
     py.Input("buffer_index", buffer_index);
