@@ -29,13 +29,13 @@ class WrongBlockIterTypeError : public ScheduleError {
                   ? "parallel"
                   : (for_kind == ForKind::kVectorized ? "vectorize" : "bind");
   }
-  String FastErrorString() const final {
+  ffi::String FastErrorString() const final {
     std::ostringstream os;
     os << "ScheduleError: The \"" << op_str_
        << "\" cannot be fulfilled with regard to some of its underlying block";
     return os.str();
   }
-  String DetailRenderTemplate() const final {
+  ffi::String DetailRenderTemplate() const final {
     std::ostringstream os;
     if (op_str_ != "bind") {
       os << "The \"" << op_str_
@@ -52,7 +52,7 @@ class WrongBlockIterTypeError : public ScheduleError {
     return os.str();
   }
   IRModule mod() const final { return mod_; }
-  Array<ObjectRef> LocationsOfInterest() const final { return {block_}; }
+  ffi::Array<ObjectRef> LocationsOfInterest() const final { return {block_}; }
   IRModule mod_;
   std::string op_str_;
   Var loop_var_;
@@ -127,8 +127,8 @@ void CheckParallelizability(const ScheduleState& self, const For& loop, ForKind
       if (!self->stmt2ref.count(realize->block.get())) {
         return false;
       }
-      CheckLoopParallelizableInBlock(self, for_kind, loop->loop_var, GetRef<BlockRealize>(realize),
-                                     thread_scope);
+      CheckLoopParallelizableInBlock(self, for_kind, loop->loop_var,
+                                     ffi::GetRef<BlockRealize>(realize), thread_scope);
     }
     return true;
   });
@@ -144,7 +144,7 @@ void CheckParallelizability(const ScheduleState& self, const For& loop, ForKind
  * `for_kind` is `kThreadBinding`
  */
 void ParallelizeComputation(const ScheduleState& self, const StmtSRef& loop_sref, ForKind for_kind,
-                            Optional<String> thread_axis) {
+                            ffi::Optional<ffi::String> thread_axis) {
   const ForNode* loop = TVM_SREF_TO_FOR(loop_sref);
 
   /*
@@ -163,14 +163,14 @@ void ParallelizeComputation(const ScheduleState& self, const StmtSRef& loop_sref
 
   // Step 2. Check whether the loop can be parallelized/vectorized/bound with regard to each
   // underlying block.
-  CheckParallelizability(self, GetRef<For>(loop), for_kind,
-                         thread_axis.defined() ? runtime::ThreadScope::Create(thread_axis.value())
-                                               : runtime::ThreadScope{-1, -1});
+  CheckParallelizability(self, ffi::GetRef<For>(loop), for_kind,
+                         thread_axis.has_value() ? runtime::ThreadScope::Create(thread_axis.value())
+                                                 : runtime::ThreadScope{-1, -1});
 
   // Step 3. Loop update and IR replacement
-  ObjectPtr<ForNode> new_loop = make_object<ForNode>(*loop);
+  ObjectPtr<ForNode> new_loop = ffi::make_object<ForNode>(*loop);
   new_loop->kind = for_kind;
-  if (thread_axis.defined()) {
+  if (thread_axis.has_value()) {
     new_loop->thread_binding = IterVar(/*dom=*/Range(nullptr),                                    //
                                        /*var=*/Var(thread_axis.value(), loop->loop_var.dtype()),  //
                                        /*iter_type=*/kThreadIndex,                                //
@@ -189,13 +189,13 @@ void Vectorize(ScheduleState self, const StmtSRef& loop_sref) {
   ParallelizeComputation(self, loop_sref, ForKind::kVectorized, std::nullopt);
 }
 
-void Bind(ScheduleState self, const StmtSRef& loop_sref, const String& thread_axis) {
+void Bind(ScheduleState self, const StmtSRef& loop_sref, const ffi::String& thread_axis) {
   ParallelizeComputation(self, loop_sref, ForKind::kThreadBinding, thread_axis);
 }
 
 void Unroll(ScheduleState self, const StmtSRef& loop_sref) {
   const ForNode* loop = TVM_SREF_TO_FOR(loop_sref);
-  ObjectPtr<ForNode> new_loop = make_object<ForNode>(*loop);
+  ObjectPtr<ForNode> new_loop = ffi::make_object<ForNode>(*loop);
   new_loop->kind = ForKind::kUnrolled;
   new_loop->thread_binding = std::nullopt;
   self->Replace(loop_sref, For(new_loop), {});
@@ -216,7 +216,7 @@ struct ParallelTraits : public UnpackedInstTraits<ParallelTraits> {
     return sch->Parallel(loop_rv);
   }
 
-  static String UnpackedAsPython(Array<String> outputs, String loop_rv) {
+  static ffi::String UnpackedAsPython(ffi::Array<ffi::String> outputs, ffi::String loop_rv) {
     PythonAPICall py("parallel");
     py.Input("loop", loop_rv);
     return py.Str();
@@ -239,7 +239,7 @@ struct VectorizeTraits : public UnpackedInstTraits<VectorizeTraits> {
     return sch->Vectorize(loop_rv);
   }
 
-  static String UnpackedAsPython(Array<String> outputs, String loop_rv) {
+  static ffi::String UnpackedAsPython(ffi::Array<ffi::String> outputs, ffi::String loop_rv) {
     PythonAPICall py("vectorize");
     py.Input("loop", loop_rv);
     return py.Str();
@@ -258,11 +258,12 @@ struct BindTraits : public UnpackedInstTraits<BindTraits> {
   static constexpr size_t kNumAttrs = 1;
   static constexpr size_t kNumDecisions = 0;
 
-  static void UnpackedApplyToSchedule(Schedule sch, LoopRV loop_rv, String thread) {
+  static void UnpackedApplyToSchedule(Schedule sch, LoopRV loop_rv, ffi::String thread) {
     return sch->Bind(loop_rv, thread);
   }
 
-  static String UnpackedAsPython(Array<String> outputs, String loop_rv, String thread) {
+  static ffi::String UnpackedAsPython(ffi::Array<ffi::String> outputs, ffi::String loop_rv,
+                                      ffi::String thread) {
     PythonAPICall py("bind");
     py.Input("loop", loop_rv);
     py.Input("thread_axis", thread);
@@ -284,7 +285,7 @@ struct UnrollTraits : public UnpackedInstTraits<UnrollTraits> {
 
   static void UnpackedApplyToSchedule(Schedule sch, LoopRV loop_rv) { return sch->Unroll(loop_rv); }
 
-  static String UnpackedAsPython(Array<String> outputs, String loop_rv) {
+  static ffi::String UnpackedAsPython(ffi::Array<ffi::String> outputs, ffi::String loop_rv) {
     PythonAPICall py("unroll");
     py.Input("loop", loop_rv);
     return py.Str();
