@@ -22,9 +22,11 @@
 namespace tvm {
 namespace tir {
 
-Array<StmtSRef> GetBlocks(const ScheduleState& self, const String& name, const GlobalVar& gv) {
+ffi::Array<StmtSRef> GetBlocks(const ScheduleState& self, const ffi::String& name,
+                               const GlobalVar& gv) {
   struct Finder : public StmtVisitor {
-    explicit Finder(const ScheduleState& self, const String& name) : self_(self), name_(name) {}
+    explicit Finder(const ScheduleState& self, const ffi::String& name)
+        : self_(self), name_(name) {}
 
     void VisitStmt_(const BlockNode* block) override {
       if (block->name_hint == name_) {
@@ -36,8 +38,8 @@ Array<StmtSRef> GetBlocks(const ScheduleState& self, const String& name, const G
     }
 
     const ScheduleState& self_;
-    const String& name_;
-    Array<StmtSRef> results_;
+    const ffi::String& name_;
+    ffi::Array<StmtSRef> results_;
   };
 
   BaseFunc func = self->mod->Lookup(gv);
@@ -47,16 +49,16 @@ Array<StmtSRef> GetBlocks(const ScheduleState& self, const String& name, const G
   return std::move(finder.results_);
 }
 
-Array<StmtSRef> GetLoops(const StmtSRef& block_sref) {
+ffi::Array<StmtSRef> GetLoops(const StmtSRef& block_sref) {
   std::vector<StmtSRef> result;
   for (StmtSRefNode* parent = block_sref->parent; parent && parent->stmt->IsInstance<ForNode>();
        parent = parent->parent) {
-    result.push_back(GetRef<StmtSRef>(parent));
+    result.push_back(ffi::GetRef<StmtSRef>(parent));
   }
   return {result.rbegin(), result.rend()};
 }
 
-Array<StmtSRef> GetChildBlocks(const ScheduleState& self, const StmtSRef& parent_sref) {
+ffi::Array<StmtSRef> GetChildBlocks(const ScheduleState& self, const StmtSRef& parent_sref) {
   struct Collector : public StmtVisitor {
    private:
     void VisitStmt_(const BlockNode* block) final { result.push_back(self->stmt2ref.at(block)); }
@@ -65,7 +67,7 @@ Array<StmtSRef> GetChildBlocks(const ScheduleState& self, const StmtSRef& parent
     explicit Collector(const ScheduleState& self) : self(self) {}
 
     const ScheduleState& self;
-    Array<StmtSRef> result;
+    ffi::Array<StmtSRef> result;
   };
   Collector collector(self);
   if (parent_sref->stmt->IsInstance<ForNode>()) {
@@ -78,17 +80,17 @@ Array<StmtSRef> GetChildBlocks(const ScheduleState& self, const StmtSRef& parent
   return std::move(collector.result);
 }
 
-Array<StmtSRef> GetProducers(const ScheduleState& self, const StmtSRef& block_sref) {
+ffi::Array<StmtSRef> GetProducers(const ScheduleState& self, const StmtSRef& block_sref) {
   StmtSRef scope_root = GetScopeRoot(self, block_sref, /*require_stage_pipeline=*/false);
   return tir::GetProducers(block_sref, self->GetBlockScope(scope_root));
 }
 
-Array<StmtSRef> GetConsumers(const ScheduleState& self, const StmtSRef& block_sref) {
+ffi::Array<StmtSRef> GetConsumers(const ScheduleState& self, const StmtSRef& block_sref) {
   StmtSRef scope_root = GetScopeRoot(self, block_sref, /*require_stage_pipeline=*/false);
   return tir::GetConsumers(block_sref, self->GetBlockScope(scope_root));
 }
 
-Array<StmtSRef> GetOutputBlocks(const ScheduleState& self, const StmtSRef& scope_sref) {
+ffi::Array<StmtSRef> GetOutputBlocks(const ScheduleState& self, const StmtSRef& scope_sref) {
   const auto* scope_block = TVM_SREF_TO_BLOCK(scope_sref);
   return tir::GetOutputBlocks(self, scope_block);
 }
@@ -104,11 +106,12 @@ struct GetBlockTraits : public UnpackedInstTraits<GetBlockTraits> {
   static constexpr size_t kNumAttrs = 2;
   static constexpr size_t kNumDecisions = 0;
 
-  static BlockRV UnpackedApplyToSchedule(Schedule sch, String name, String func_name) {
+  static BlockRV UnpackedApplyToSchedule(Schedule sch, ffi::String name, ffi::String func_name) {
     return sch->GetBlock(name, func_name);
   }
 
-  static String UnpackedAsPython(Array<String> outputs, String name, String func_name) {
+  static ffi::String UnpackedAsPython(ffi::Array<ffi::String> outputs, ffi::String name,
+                                      ffi::String func_name) {
     PythonAPICall py("get_block");
     py.Input("name", name);
     py.Input("func_name", func_name);
@@ -129,11 +132,11 @@ struct GetLoopsTraits : public UnpackedInstTraits<GetLoopsTraits> {
   static constexpr size_t kNumAttrs = 0;
   static constexpr size_t kNumDecisions = 0;
 
-  static Array<LoopRV> UnpackedApplyToSchedule(Schedule sch, BlockRV block_rv) {
+  static ffi::Array<LoopRV> UnpackedApplyToSchedule(Schedule sch, BlockRV block_rv) {
     return sch->GetLoops(block_rv);
   }
 
-  static String UnpackedAsPython(Array<String> outputs, String block_rv) {
+  static ffi::String UnpackedAsPython(ffi::Array<ffi::String> outputs, ffi::String block_rv) {
     PythonAPICall py("get_loops");
     py.Input("block", block_rv);
     py.OutputList(outputs);
@@ -153,7 +156,7 @@ struct GetChildBlocksTraits : public UnpackedInstTraits<GetChildBlocksTraits> {
   static constexpr size_t kNumAttrs = 0;
   static constexpr size_t kNumDecisions = 0;
 
-  static Array<BlockRV> UnpackedApplyToSchedule(Schedule sch, ObjectRef block_or_loop_rv) {
+  static ffi::Array<BlockRV> UnpackedApplyToSchedule(Schedule sch, ObjectRef block_or_loop_rv) {
     if (auto block = block_or_loop_rv.as<BlockRV>()) {
       return sch->GetChildBlocks(block.value());
     }
@@ -164,7 +167,8 @@ struct GetChildBlocksTraits : public UnpackedInstTraits<GetChildBlocksTraits> {
     throw;
   }
 
-  static String UnpackedAsPython(Array<String> outputs, String block_or_loop_rv) {
+  static ffi::String UnpackedAsPython(ffi::Array<ffi::String> outputs,
+                                      ffi::String block_or_loop_rv) {
     PythonAPICall py("get_child_blocks");
     py.Input("", block_or_loop_rv);
     py.OutputList(outputs);
@@ -184,11 +188,11 @@ struct GetProducersTraits : public UnpackedInstTraits<GetProducersTraits> {
   static constexpr size_t kNumAttrs = 0;
   static constexpr size_t kNumDecisions = 0;
 
-  static Array<BlockRV> UnpackedApplyToSchedule(Schedule sch, BlockRV block_rv) {
+  static ffi::Array<BlockRV> UnpackedApplyToSchedule(Schedule sch, BlockRV block_rv) {
     return sch->GetProducers(block_rv);
   }
 
-  static String UnpackedAsPython(Array<String> outputs, String block_rv) {
+  static ffi::String UnpackedAsPython(ffi::Array<ffi::String> outputs, ffi::String block_rv) {
     PythonAPICall py("get_producers");
     py.Input("block", block_rv);
     py.OutputList(outputs);
@@ -208,11 +212,11 @@ struct GetConsumersTraits : public UnpackedInstTraits<GetConsumersTraits> {
   static constexpr size_t kNumAttrs = 0;
   static constexpr size_t kNumDecisions = 0;
 
-  static Array<BlockRV> UnpackedApplyToSchedule(Schedule sch, BlockRV block_rv) {
+  static ffi::Array<BlockRV> UnpackedApplyToSchedule(Schedule sch, BlockRV block_rv) {
     return sch->GetConsumers(block_rv);
   }
 
-  static String UnpackedAsPython(Array<String> outputs, String block_rv) {
+  static ffi::String UnpackedAsPython(ffi::Array<ffi::String> outputs, ffi::String block_rv) {
     PythonAPICall py("get_consumers");
     py.Input("block", block_rv);
     py.OutputList(outputs);
@@ -232,11 +236,11 @@ struct GetOutputBlocksTraits : public UnpackedInstTraits<GetOutputBlocksTraits>
   static constexpr size_t kNumAttrs = 0;
   static constexpr size_t kNumDecisions = 0;
 
-  static Array<BlockRV> UnpackedApplyToSchedule(Schedule sch, BlockRV block_rv) {
+  static ffi::Array<BlockRV> UnpackedApplyToSchedule(Schedule sch, BlockRV block_rv) {
     return sch->GetOutputBlocks(block_rv);
   }
 
-  static String UnpackedAsPython(Array<String> outputs, String block_rv) {
+  static ffi::String UnpackedAsPython(ffi::Array<ffi::String> outputs, ffi::String block_rv) {
     PythonAPICall py("get_output_blocks");
     py.Input("block", block_rv);
     py.OutputList(outputs);
