@@ -99,7 +99,8 @@ class FloatConfig {
    * \return The FloatConfig class containing internal floating point representation.
    */
   static FloatConfig FromDataType(DataType dtype) {
-    CHECK(dtype.is_float() || dtype.is_bfloat16() || dtype.is_float8() || dtype.is_float4())
+    CHECK(dtype.is_float() || dtype.is_bfloat16() || dtype.is_float8() || dtype.is_float6() ||
+          dtype.is_float4())
         << "FloatConfig is only applicable to floating point data types, got " << dtype
         << " instead.";
     if (dtype.is_float()) {
@@ -121,12 +122,43 @@ class FloatConfig {
       // NVIDIA/Arm/Intel's FP8 formats for Deep Learning
       // Reference: https://arxiv.org/abs/2209.05433
       switch (dtype.code()) {
+        case DataType::kFloat8_e3m4:
+          // E3M4 format, not consistent with IEEE-754
+          return FloatConfig(3, 4, 3, InftyStyle::kNone, NaNStyle::kAllOnes);
+        case DataType::kFloat8_e4m3:
+          // E4M3 format, not consistent with IEEE-754
+          return FloatConfig(4, 3, 7, InftyStyle::kNone, NaNStyle::kAllOnes);
+        case DataType::kFloat8_e4m3b11fnuz:
+          // E4M3 variant with b11 encoding, not consistent with IEEE-754
+          return FloatConfig(4, 3, 7, InftyStyle::kNone, NaNStyle::kAllOnes);
         case DataType::kFloat8_e4m3fn:
           // E4M3 format, not consistent with IEEE-754
           return FloatConfig(4, 3, 7, InftyStyle::kNone, NaNStyle::kAllOnes);
-        default:
-          // E5M2 format, consistent with IEEE-754
+        case DataType::kFloat8_e4m3fnuz:
+          // UE4M3 format, not consistent with IEEE-754
+          return FloatConfig(4, 3, 7, InftyStyle::kNone, NaNStyle::kAllOnes);
+        case DataType::kFloat8_e5m2:
+          // UE5M2 format, consistent with IEEE-754
           return FloatConfig(5, 2, 15, InftyStyle::kIEEE, NaNStyle::kIEEE);
+        case DataType::kFloat8_e5m2fnuz:
+          // UE5M2 format, not consistent with IEEE-754
+          return FloatConfig(5, 2, 15, InftyStyle::kNone, NaNStyle::kAllOnes);
+        case DataType::kFloat8_e8m0fnu:
+          // UE8M0 format, not consistent with IEEE-754
+          return FloatConfig(8, 0, 127, InftyStyle::kNone, NaNStyle::kAllOnes);
+        default:
+          LOG(FATAL) << "Unknown float8 variant: " << dtype;
+      }
+    } else if (dtype.is_float6()) {  // float6
+      switch (dtype.code()) {
+        case DataType::kFloat6_e2m3fn:
+          // E2M3 format, not consistent with IEEE-754
+          return FloatConfig(2, 3, 1, InftyStyle::kNone, NaNStyle::kNone);
+        case DataType::kFloat6_e3m2fn:
+          // E3M2 format, not consistent with IEEE-754
+          return FloatConfig(3, 2, 3, InftyStyle::kNone, NaNStyle::kNone);
+        default:
+          LOG(FATAL) << "Unknown float6 variant: " << dtype;
       }
     } else {
       // float4
