@@ -22,6 +22,7 @@
  */
 
 #include <tvm/arith/iter_affine_map.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/tir/analysis.h>
 #include <tvm/tir/stmt_functor.h>
 #include <tvm/tir/transform.h>
@@ -64,21 +65,21 @@ class BufferFlattener : public arith::IRMutatorWithAnalyzer {
         << "Unexpected MatchBufferRegion found during tir.transform.FlattenBuffer.  "
         << "All MatchBufferRegion should be removed in tir.transform.LowerMatchBuffer.";
 
-    Block block = GetRef<Block>(op);
+    Block block = ffi::GetRef<Block>(op);
 
-    Array<Buffer> alloc_buffers = op->alloc_buffers;
+    ffi::Array<Buffer> alloc_buffers = op->alloc_buffers;
     alloc_buffers.MutateByApply([this](Buffer buf) { return GetFlattenedBuffer(buf); });
     if (!alloc_buffers.same_as(op->alloc_buffers)) {
       block.CopyOnWrite()->alloc_buffers = alloc_buffers;
     }
 
-    Array<BufferRegion> reads = op->reads;
+    ffi::Array<BufferRegion> reads = op->reads;
     reads.MutateByApply([this](BufferRegion region) { return MutateBufferRegion(region); });
     if (!reads.same_as(op->reads)) {
       block.CopyOnWrite()->reads = reads;
     }
 
-    Array<BufferRegion> writes = op->writes;
+    ffi::Array<BufferRegion> writes = op->writes;
     writes.MutateByApply([this](BufferRegion region) { return MutateBufferRegion(region); });
     if (!writes.same_as(op->writes)) {
       block.CopyOnWrite()->writes = writes;
@@ -90,7 +91,7 @@ class BufferFlattener : public arith::IRMutatorWithAnalyzer {
   Stmt VisitStmt_(const AllocateNode* op) final {
     // Determine the flattened extents first, before stripping of
     // DeclBuffer.
-    auto new_extents = [&]() -> Array<PrimExpr> {
+    auto new_extents = [&]() -> ffi::Array<PrimExpr> {
       if (op->extents.size() == 1) {
         // No flattening required for buffers that are already flat
         return op->extents;
@@ -218,7 +219,8 @@ class BufferFlattener : public arith::IRMutatorWithAnalyzer {
     }
   }
 
-  Array<PrimExpr> GetSimplifiedElemOffset(const Buffer& buffer, const Array<PrimExpr>& indices) {
+  ffi::Array<PrimExpr> GetSimplifiedElemOffset(const Buffer& buffer,
+                                               const ffi::Array<PrimExpr>& indices) {
     auto flattened_indices = buffer->ElemOffset(indices);
     return this->IterMapSimplifyWithContext(flattened_indices, false);
   }
@@ -242,17 +244,17 @@ class BufferFlattener : public arith::IRMutatorWithAnalyzer {
       return region;
     }
 
-    Array<PrimExpr> min_values;
-    Array<PrimExpr> max_values;
+    ffi::Array<PrimExpr> min_values;
+    ffi::Array<PrimExpr> max_values;
     for (const auto& range : region->region) {
       min_values.push_back(range->min);
       max_values.push_back(range->min + range->extent - 1);
     }
 
-    Array<PrimExpr> flattened_min = GetSimplifiedElemOffset(orig_buf, min_values);
-    Array<PrimExpr> flattened_max = GetSimplifiedElemOffset(orig_buf, max_values);
+    ffi::Array<PrimExpr> flattened_min = GetSimplifiedElemOffset(orig_buf, min_values);
+    ffi::Array<PrimExpr> flattened_max = GetSimplifiedElemOffset(orig_buf, max_values);
 
-    Array<Range> flattened_ranges;
+    ffi::Array<Range> flattened_ranges;
     ICHECK_EQ(flattened_min.size(), flattened_max.size());
     for (size_t i = 0; i < flattened_min.size(); i++) {
       flattened_ranges.push_back(Range(flattened_min[i], flattened_max[i] + 1));
@@ -265,7 +267,7 @@ class BufferFlattener : public arith::IRMutatorWithAnalyzer {
   std::unordered_map<Buffer, Buffer, ObjectPtrHash, ObjectPtrEqual> buffer_remap_;
 
   /*! \brief The updated external buffer map. */
-  Map<Var, Buffer> updated_extern_buffer_map_;
+  ffi::Map<Var, Buffer> updated_extern_buffer_map_;
 };
 
 PrimFunc FlattenBuffer(PrimFunc f) { return BufferFlattener::Flatten(f); }
@@ -279,7 +281,10 @@ Pass FlattenBuffer() {
   return CreatePrimFuncPass(pass_func, 0, "tir.FlattenBuffer", {});
 }
 
-TVM_FFI_REGISTER_GLOBAL("tir.transform.FlattenBuffer").set_body_typed(FlattenBuffer);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.transform.FlattenBuffer", FlattenBuffer);
+}
 }  // namespace transform
 
 }  // namespace tir
