@@ -22,6 +22,7 @@
  * \file tir/transforms/lower_tvm_buildin.cc
  */
 #include <tvm/ffi/function.h>
+#include <tvm/ffi/reflection/registry.h>
 #include <tvm/tir/builtin.h>
 #include <tvm/tir/expr.h>
 #include <tvm/tir/stmt_functor.h>
@@ -39,7 +40,7 @@ namespace tir {
 class BuiltinLower : public StmtExprMutator {
  public:
   static PrimFunc Build(PrimFunc func) {
-    Optional<PrimExpr> device_type = std::nullopt;
+    ffi::Optional<PrimExpr> device_type = std::nullopt;
     if (auto target = func->GetAttr<Target>(tvm::attr::kTarget)) {
       device_type = Integer(target.value()->kind->default_device_type);
     }
@@ -49,7 +50,7 @@ class BuiltinLower : public StmtExprMutator {
     return func;
   }
 
-  explicit BuiltinLower(Optional<PrimExpr> device_type = std::nullopt)
+  explicit BuiltinLower(ffi::Optional<PrimExpr> device_type = std::nullopt)
       : device_type_(device_type) {}
 
   // NOTE: Right now, we make the following scoping requirement
@@ -316,7 +317,7 @@ class BuiltinLower : public StmtExprMutator {
     }
 
     if (min.same_as(op->min) && extent.same_as(op->extent) && body.same_as(op->body)) {
-      return GetRef<Stmt>(op);
+      return ffi::GetRef<Stmt>(op);
     } else {
       auto n = CopyOnWrite(op);
       n->min = std::move(min);
@@ -369,7 +370,7 @@ class BuiltinLower : public StmtExprMutator {
                   << "but was instead the expression " << device_type_ << " with type "
                   << device_type_.value()->GetTypeKey();
 
-    String device_name = runtime::DLDeviceType2Str(as_int->value);
+    ffi::String device_name = runtime::DLDeviceType2Str(as_int->value);
     return StringImm("device_api." + device_name + "." + method_name);
   }
 
@@ -520,6 +521,9 @@ class BuiltinLower : public StmtExprMutator {
         prep_seq->emplace_back(TVMStructSet(args_stack, stack_offset, builtin::kTVMFFIAnyTypeIndex,
                                             ConstInt32(arg_type_index)));
       }
+      // set zero padding to ensure compatibility with FFI convention
+      prep_seq->emplace_back(
+          TVMStructSet(args_stack, stack_offset, builtin::kTVMFFIAnyZeroPadding, ConstInt32(0)));
       // handle arg value
       // NOTE: the intrinsic codegen will handle padding value clear for 32bit
       // types or types that are smaller than 64 bits.
@@ -577,6 +581,8 @@ class BuiltinLower : public StmtExprMutator {
     // explicitly set return value to None to avoid bad state interpretation
     prep_seq.emplace_back(TVMStructSet(scope.stack_ffi_any, num_args, builtin::kTVMFFIAnyTypeIndex,
                                        ConstInt32(ffi::TypeIndex::kTVMFFINone)));
+    prep_seq.emplace_back(
+        TVMStructSet(scope.stack_ffi_any, num_args, builtin::kTVMFFIAnyZeroPadding, ConstInt32(0)));
     prep_seq.emplace_back(TVMStructSet(scope.stack_ffi_any, num_args, builtin::kTVMFFIAnyUnionValue,
                                        make_zero(DataType::Int(64))));
     // Verify stack size matches earlier value.
@@ -588,9 +594,9 @@ class BuiltinLower : public StmtExprMutator {
     scope.run_sizes.shape_stack = restore_shape_stack;
     scope.run_sizes.array_stack = restore_array_stack;
     scope.run_sizes.arg_stack = arg_stack_begin;
-    Array<PrimExpr> packed_args = {op->args[name_offset], scope.stack_ffi_any,
-                                   ConstInt32(arg_stack_begin),
-                                   ConstInt32(arg_stack_begin + num_args)};
+    ffi::Array<PrimExpr> packed_args = {op->args[name_offset], scope.stack_ffi_any,
+                                        ConstInt32(arg_stack_begin),
+                                        ConstInt32(arg_stack_begin + num_args)};
     if (pass_last_arg_as_traced_value) {
       // pass in last element as traced value
       // used by call_packed_traced
@@ -620,7 +626,7 @@ class BuiltinLower : public StmtExprMutator {
     std::string fdevapi_prefix = "device_api.";
     fdevapi_prefix += runtime::DLDeviceType2Str(device_type_.as<IntImmNode>()->value);
 
-    Array<PrimExpr> args = {
+    ffi::Array<PrimExpr> args = {
         GetDeviceMethodName("alloc_nd"),
         device_type_.value(),
         device_id_.value(),
@@ -651,8 +657,8 @@ class BuiltinLower : public StmtExprMutator {
 
   // The prepration sequence to be emitted before the current statement.
   std::vector<std::vector<Stmt>> prep_seq_stack_;
-  Optional<PrimExpr> device_type_{std::nullopt};
-  Optional<PrimExpr> device_id_{std::nullopt};
+  ffi::Optional<PrimExpr> device_type_{std::nullopt};
+  ffi::Optional<PrimExpr> device_id_{std::nullopt};
 
   bool is_precheck_{false};
 
@@ -673,7 +679,10 @@ Pass LowerTVMBuiltin() {
   return CreatePrimFuncPass(pass_func, 0, "tir.LowerTVMBuiltin", {});
 }
 
-TVM_FFI_REGISTER_GLOBAL("tir.transform.LowerTVMBuiltin").set_body_typed(LowerTVMBuiltin);
+TVM_FFI_STATIC_INIT_BLOCK() {
+  namespace refl = tvm::ffi::reflection;
+  refl::GlobalDef().def("tir.transform.LowerTVMBuiltin", LowerTVMBuiltin);
+}
 
 }  // namespace transform
 }  // namespace tir
