@@ -19,12 +19,12 @@
 
 #include <gtest/gtest.h>
 #include <tvm/runtime/logging.h>
-#include <tvm/runtime/ndarray.h>
+#include <tvm/runtime/tensor.h>
 
 using namespace tvm;
 
-TEST(NDArrayTest, IsContiguous_ContiguousStride) {
-  auto array = runtime::NDArray::Empty({5, 10}, DataType::Float(32), {kDLCPU});
+TEST(TensorTest, IsContiguous_ContiguousStride) {
+  auto array = runtime::Tensor::Empty({5, 10}, DataType::Float(32), {kDLCPU});
   DLManagedTensor* managed_tensor = array.ToDLPack();
 
   int64_t strides[] = {10, 1};
@@ -35,8 +35,8 @@ TEST(NDArrayTest, IsContiguous_ContiguousStride) {
   managed_tensor->deleter(managed_tensor);
 }
 
-TEST(NDArrayTest, IsContiguous_NullStride) {
-  auto array = runtime::NDArray::Empty({5, 10}, DataType::Float(32), {kDLCPU});
+TEST(TensorTest, IsContiguous_NullStride) {
+  auto array = runtime::Tensor::Empty({5, 10}, DataType::Float(32), {kDLCPU});
   DLManagedTensor* managed_tensor = array.ToDLPack();
 
   managed_tensor->dl_tensor.strides = nullptr;
@@ -46,8 +46,8 @@ TEST(NDArrayTest, IsContiguous_NullStride) {
   managed_tensor->deleter(managed_tensor);
 }
 
-TEST(NDArrayTest, IsContiguous_AnyStrideForSingular) {
-  auto array = runtime::NDArray::Empty({5, 1, 10}, DataType::Float(32), {kDLCPU});
+TEST(TensorTest, IsContiguous_AnyStrideForSingular) {
+  auto array = runtime::Tensor::Empty({5, 1, 10}, DataType::Float(32), {kDLCPU});
   DLManagedTensor* managed_tensor = array.ToDLPack();
 
   int64_t strides[] = {10, 1, 1};  // strides[1] is normalized to 1 because shape[1] == 1.
@@ -59,8 +59,8 @@ TEST(NDArrayTest, IsContiguous_AnyStrideForSingular) {
   managed_tensor->deleter(managed_tensor);
 }
 
-TEST(NDArrayTest, IsContiguous_UncontiguousStride) {
-  auto array = runtime::NDArray::Empty({5, 1, 10}, DataType::Float(32), {kDLCPU});
+TEST(TensorTest, IsContiguous_UncontiguousStride) {
+  auto array = runtime::Tensor::Empty({5, 1, 10}, DataType::Float(32), {kDLCPU});
   DLManagedTensor* managed_tensor = array.ToDLPack();
 
   int64_t strides[] = {1, 1, 1};
