@@ -25,7 +25,7 @@ trap cleanup EXIT
 
 make cppdoc 2>/tmp/$$.log.txt
 
-grep -v -E "ENABLE_PREPROCESSING|unsupported tag|Inheritance graph" < /tmp/$$.log.txt > /tmp/$$.logclean.txt || true
+grep -v -E "ENABLE_PREPROCESSING|unsupported tag|Inheritance graph|use_default_type_traits_v" < /tmp/$$.log.txt > /tmp/$$.logclean.txt || true
 echo "---------Error Log----------"
 cat /tmp/$$.logclean.txt
 echo "----------------------------"
