@@ -41,8 +41,8 @@ def test_int_intrin(target, dev, dtype):
         (x,) = sch.get_loops(sch.get_block("B"))
         sch.bind(x, "threadIdx.x")
         f = tvm.compile(sch.mod, target=target)
-        a = tvm.nd.array(np.random.randint(0, 100000, size=n).astype(A.dtype), dev)
-        b = tvm.nd.array(np.zeros(shape=(n,)).astype(B.dtype), dev)
+        a = tvm.runtime.tensor(np.random.randint(0, 100000, size=n).astype(A.dtype), dev)
+        b = tvm.runtime.tensor(np.zeros(shape=(n,)).astype(B.dtype), dev)
         f(a, b)
         ref = np.vectorize(partial(np_func, dtype=dtype))(a.numpy())
         tvm.testing.assert_allclose(b.numpy(), ref)
