@@ -37,8 +37,8 @@ def test_metal_inf_nan():
         (x,) = sch.get_loops(sch.get_block("C"))
         sch.bind(x, "threadIdx.x")
         fun = tvm.compile(sch.mod, target=target)
-        a = tvm.nd.empty((n,), A.dtype, dev)
-        c = tvm.nd.empty((n,), A.dtype, dev)
+        a = tvm.runtime.empty((n,), A.dtype, dev)
+        c = tvm.runtime.empty((n,), A.dtype, dev)
         # Only need to test compiling here
         fun(a, c)
 
@@ -70,8 +70,8 @@ def test_unaligned_vectorize():
     dev = tvm.metal()
 
     a = (np.arange(6).reshape(2, 3)).astype("float32")
-    a_nd = tvm.nd.array(a, dev)
-    b_nd = tvm.nd.empty((6,), "float32", dev)
+    a_nd = tvm.runtime.tensor(a, dev)
+    b_nd = tvm.runtime.empty((6,), "float32", dev)
     f = tvm.compile(IRModule, target=target)
     f(a_nd, b_nd)
     np.testing.assert_allclose(b_nd.numpy(), a.reshape(6), atol=1e-5, rtol=1e-5)
@@ -90,8 +90,8 @@ def test_metal_erf():
         (x,) = sch.get_loops(sch.get_block("C"))
         sch.bind(x, "threadIdx.x")
         fun = tvm.compile(sch.mod, target=target)
-        a = tvm.nd.empty((n,), A.dtype, dev)
-        c = tvm.nd.empty((n,), A.dtype, dev)
+        a = tvm.runtime.empty((n,), A.dtype, dev)
+        c = tvm.runtime.empty((n,), A.dtype, dev)
         # Only need to test compiling here
         fun(a, c)
 
@@ -119,7 +119,7 @@ def test_ramp():
 
     f = tvm.compile(IRModule, target=target)
     dev = tvm.metal()
-    a_nd = tvm.nd.empty((1, 2), "int32", dev)
+    a_nd = tvm.runtime.empty((1, 2), "int32", dev)
     f(a_nd)
     assert tuple(a_nd.numpy()[0, :]) == (0, 3)
 
@@ -141,8 +141,8 @@ def test_select_vectorize():
     target = "metal"
     dev = tvm.metal()
     a = np.arange(6).astype("float32")
-    a_nd = tvm.nd.array(a, dev)
-    b_nd = tvm.nd.empty((6,), "float32", dev)
+    a_nd = tvm.runtime.tensor(a, dev)
+    b_nd = tvm.runtime.empty((6,), "float32", dev)
     f = tvm.compile(IRModule, target=target)
     f(a_nd, b_nd)
     a.reshape(3, 2)[:, 1] = 0
@@ -162,8 +162,8 @@ def test_vectorized_uint8():
 
     dev = tvm.metal()
     a = np.arange(16).astype("uint8")
-    a_nd = tvm.nd.array(a, dev)
-    b_nd = tvm.nd.empty((16,), "float32", dev)
+    a_nd = tvm.runtime.tensor(a, dev)
+    b_nd = tvm.runtime.empty((16,), "float32", dev)
     f = tvm.compile(func, target="metal")
     f(a_nd, b_nd)
     np.testing.assert_allclose(b_nd.numpy(), a.astype("float32"), atol=1e-5, rtol=1e-5)
@@ -180,14 +180,14 @@ def test_func_with_trailing_pod_params():
                 vi = T.axis.spatial(16, i)
                 B[vi] = A[vi] + x
 
-    @tvm.register_func("tvm_callback_metal_compile")
+    @tvm.register_global_func("tvm_callback_metal_compile")
     def compile_metal(src, target):
         return xcode.compile_metal(src)
 
     mod = tvm.IRModule({"main": func})
 
     f = tvm.compile(mod, target="metal")
-    src: str = f.imported_modules[0].get_source()
+    src: str = f.imports[0].inspect_source()
     occurrences = src.count("struct func_kernel_args_t")
     assert occurrences == 1, occurrences
 
