@@ -86,7 +86,7 @@ def test_alloc_storage_with_scope_global(hexagon_launcher):
         vm_mod = session.get_executor_from_factory(lib)
         # This is the important line which tests nd allocator
         vm_rt = relax.VirtualMachine(vm_mod, dev, memory_cfg="naive")
-        x = tvm.nd.array(arg0, dev)
+        x = tvm.runtime.tensor(arg0, dev)
         vm_rt.set_input("main", x)
         vm_rt.invoke_stateful("main")
         hexagon_output = vm_rt.get_outputs("main").numpy()
