@@ -188,12 +188,12 @@ class TestAsyncSoftwarePipeline:
         with hexagon_launcher.create_session() as hexagon_session:
             dev = hexagon_session.device
             mod = hexagon_session.load_module(func)
-            out = tvm.nd.array(out_np, device=dev)
-            a = tvm.nd.array(a_np, device=dev)
+            out = tvm.runtime.tensor(out_np, device=dev)
+            a = tvm.runtime.tensor(a_np, device=dev)
             if comp_type == "single_input":
                 mod(a, out)
             else:
-                b = tvm.nd.array(b_np, device=dev)
+                b = tvm.runtime.tensor(b_np, device=dev)
                 mod(a, b, out)
 
             verify(out, ref)
