@@ -60,9 +60,9 @@ class ElemwiseSumIRModule:
 
 
 def generate_add_test_data(hexagon_session: Session, n=128 * 1024):
-    a = tvm.nd.array(np.random.uniform(size=n).astype("float32"), hexagon_session.device)
-    b = tvm.nd.array(np.random.uniform(size=n).astype("float32"), hexagon_session.device)
-    c = tvm.nd.array(np.zeros(n, dtype="float32"), hexagon_session.device)
+    a = tvm.runtime.tensor(np.random.uniform(size=n).astype("float32"), hexagon_session.device)
+    b = tvm.runtime.tensor(np.random.uniform(size=n).astype("float32"), hexagon_session.device)
+    c = tvm.runtime.tensor(np.zeros(n, dtype="float32"), hexagon_session.device)
     return (a, b, c, n)
 
 
