@@ -26,6 +26,8 @@ import sys
 from multiprocessing import Process
 from typing import Any, Callable, List
 
+from tvm.script import tir as T
+
 
 import tvm
 import tvm.testing
@@ -134,6 +136,55 @@ def test_nvshmem_empty(session_kind: di.Session, num_workers: int):
     sess.sync_worker_0()
 
 
+def test_nvshmem_compile():
+    if tvm.get_global_func("runtime.disco.nvshmem.init_nvshmem_uid", True) is None:
+        return
+
+    num_workers = 4
+    sess = di.ProcessSession(num_workers=num_workers)
+
+    f_init_nvshmem_uid = tvm.get_global_func("runtime.disco.nvshmem.init_nvshmem_uid")
+    uid = f_init_nvshmem_uid()
+    init_dfunc = sess.get_global_func("runtime.disco.nvshmem.init_nvshmem")
+    init_dfunc(uid, num_workers, 0)
+    sess.sync_worker_0()
+
+    @T.prim_func
+    def main(A: T.Buffer((8, 16), "float32"), B: T.Buffer((16, 8), "float32")):
+        for i in T.thread_binding(T.int64(8), thread="threadIdx.y"):
+            for j in T.thread_binding(T.int64(16), thread="threadIdx.x"):
+                with T.block("T_transpose"):
+                    v0 = T.axis.spatial(T.int64(8), i)
+                    v1 = T.axis.spatial(T.int64(16), j)
+                    T.reads(A[v0, v1])
+                    T.writes(B[v1, v0])
+                    B[v1, v0] = A[v0, v1]
+
+    with tempfile.TemporaryDirectory() as tmpdir:
+        path = tmpdir + "/test.so"
+        A_np = np.arange(8 * 16).astype("float32").reshape([8, 16])
+        B_np = np.zeros((16, 8), dtype="float32")
+        A_array = sess.empty(A_np.shape, "float32")
+        B_array = sess.empty(B_np.shape, "float32")
+        A_array.debug_copy_from(0, A_np)
+
+        target = tvm.target.Target("cuda")
+        tvm.compile(main, target=target).export_library(path)
+        mod = sess.load_vm_module(path)
+        mod["main"](A_array, B_array)
+
+        B_res = B_array.debug_get_from_remote(0).numpy()
+        np.testing.assert_equal(B_res, A_np.T)
+
+        # sync all workers to make sure the temporary files are cleaned up after all workers
+        # finish the execution
+        sess._sync_all()
+
+    finalize_dfunc = sess.get_global_func("runtime.disco.nvshmem.finalize_nvshmem")
+    finalize_dfunc()
+    sess.sync_worker_0()
+
+
 if __name__ == "__main__":
     # After the first call to `nvshmem_init`, a subsequent call to `nvshmem_init`
     # or `nvshmem_init_thread` in the same program results in undefined behavior.
@@ -145,3 +196,8 @@ if __name__ == "__main__":
                 p = Process(target=test_func, args=[session_kind, num_workers])
                 p.start()
                 p.join()
+
+    # testing compilation flow
+    p = Process(target=test_nvshmem_compile)
+    p.start()
+    p.join()
