@@ -31,7 +31,7 @@ def test_decode_gemv_1():
     class Before:
         @T.prim_func
         def func(W: T.Buffer((4096, 512), "uint32"), S: T.Buffer((4096, 128), "float16"), V: T.Buffer((1, 1, 4096), "float16"), C: T.Buffer((1, 1, 4096), "float16")):
-            T.func_attr({"global_symbol": "main", "tir.noalias": T.bool(True)})
+            T.func_attr({"global_symbol": "main", "tir.noalias": True})
             # with T.block("root"):
             B = T.alloc_buffer((4096, 4096), "float16")
             for i, j in T.grid(4096, 4096):
@@ -54,7 +54,7 @@ def test_decode_gemv_1():
     class After:
         @T.prim_func
         def func(W_handle: T.handle, S_handle: T.handle, V_handle: T.handle, C_handle: T.handle):
-            T.func_attr({"global_symbol": "main", "tir.is_scheduled": 1, "tir.noalias": T.bool(True)})
+            T.func_attr({"global_symbol": "main", "tir.is_scheduled": True, "tir.noalias": True})
             W = T.match_buffer(W_handle, (4096, 512), "uint32")
             S = T.match_buffer(S_handle, (4096, 128), "float16")
             V = T.match_buffer(V_handle, (1, 1, 4096), "float16")
@@ -106,7 +106,7 @@ def test_decode_gemv_2():
     class Before:
         @T.prim_func
         def func(W: T.Buffer((512, 4096), "uint32"), S: T.Buffer((128, 4096), "float16"), V: T.Buffer((1, 1, 4096), "float16"), C: T.Buffer((1, 1, 4096), "float16")):
-            T.func_attr({"global_symbol": "main", "tir.noalias": T.bool(True)})
+            T.func_attr({"global_symbol": "main", "tir.noalias": True})
             # with T.block("root"):
             B = T.alloc_buffer((4096, 4096), "float16")
             for i, j in T.grid(4096, 4096):
@@ -129,7 +129,7 @@ def test_decode_gemv_2():
     class After:
         @T.prim_func
         def func(W: T.Buffer((512, 4096), "uint32"), S: T.Buffer((128, 4096), "float16"), V: T.Buffer((1, 1, 4096), "float16"), C: T.Buffer((1, 1, 4096), "float16")):
-            T.func_attr({"global_symbol": "main", "tir.is_scheduled": 1, "tir.noalias": T.bool(True)})
+            T.func_attr({"global_symbol": "main", "tir.is_scheduled": True, "tir.noalias": True})
             # with T.block("root"):
             C_rf_local = T.alloc_buffer((16, 1, 1, 4096), "float16", scope="local")
             for i2_i0_i1_fused_0 in T.thread_binding(256, thread="blockIdx.x"):
@@ -169,7 +169,7 @@ def test_decode_gemv_3():
     class Before:
         @T.prim_func
         def func(W: T.Buffer((512, 4096), "uint32"), S: T.Buffer((128, 4096), "float16"), V: T.Buffer((1, 1, 4096), "float16"), C: T.Buffer((1, 1, 4096), "float16")):
-            T.func_attr({"global_symbol": "main", "tir.noalias": T.bool(True)})
+            T.func_attr({"global_symbol": "main", "tir.noalias": True})
             # with T.block("root"):
             B = T.alloc_buffer((4096, 4096), "float16")
             for i, j in T.grid(4096, 4096):
@@ -191,7 +191,7 @@ def test_decode_gemv_3():
     class After:
         @T.prim_func
         def func(W_handle: T.handle, S_handle: T.handle, V_handle: T.handle, C_handle: T.handle):
-            T.func_attr({"global_symbol": "main", "tir.is_scheduled": 1, "tir.noalias": T.bool(True)})
+            T.func_attr({"global_symbol": "main", "tir.is_scheduled": True, "tir.noalias": True})
             W = T.match_buffer(W_handle, (512, 4096), "uint32")
             S = T.match_buffer(S_handle, (128, 4096), "float16")
             V = T.match_buffer(V_handle, (1, 1, 4096), "float16")
@@ -245,7 +245,7 @@ def test_decode_gemv_4():
     class Before:
         @T.prim_func
         def func(W: T.Buffer((4096, 512), "uint32"), S: T.Buffer((4096, 128), "float16"), V: T.Buffer((1, 1, 4096), "float16"), C: T.Buffer((1, 1, 4096), "float16")):
-            T.func_attr({"global_symbol": "main", "tir.noalias": T.bool(True)})
+            T.func_attr({"global_symbol": "main", "tir.noalias": True})
             # with T.block("root"):
             B = T.alloc_buffer((4096, 4096), "float16")
             for i, j in T.grid(4096, 4096):
@@ -268,7 +268,7 @@ def test_decode_gemv_4():
     class After:
         @T.prim_func
         def func(W: T.Buffer((4096, 512), "uint32"), S: T.Buffer((4096, 128), "float16"), V: T.Buffer((1, 1, 4096), "float16"), C: T.Buffer((1, 1, 4096), "float16")):
-            T.func_attr({"global_symbol": "main", "tir.is_scheduled": 1, "tir.noalias": T.bool(True)})
+            T.func_attr({"global_symbol": "main", "tir.is_scheduled": True, "tir.noalias": True})
             # with T.block("root"):
             C_rf_local = T.alloc_buffer((16, 1, 1, 4096), "float16", scope="local")
             for i2_0_i0_i1_fused_0 in T.thread_binding(32, thread="blockIdx.x"):
@@ -310,7 +310,7 @@ def test_decode_gemv_sigmoid():
     class Before:
         @T.prim_func
         def func(W: T.Buffer((4096, 512), "uint32"), S: T.Buffer((4096, 128), "float16"), V: T.Buffer((1, 1, 4096), "float16"), D: T.Buffer((1, 1, 4096), "float16")):
-            T.func_attr({"global_symbol": "main", "tir.noalias": T.bool(True)})
+            T.func_attr({"global_symbol": "main", "tir.noalias": True})
             # with T.block("root"):
             B = T.alloc_buffer((4096, 4096), "float16")
             C = T.alloc_buffer((1, 1, 4096), "float16")
@@ -339,7 +339,7 @@ def test_decode_gemv_sigmoid():
     class After:
         @T.prim_func
         def func(W_handle: T.handle, S_handle: T.handle, V_handle: T.handle, D_handle: T.handle):
-            T.func_attr({"global_symbol": "main", "tir.is_scheduled": 1, "tir.noalias": T.bool(True)})
+            T.func_attr({"global_symbol": "main", "tir.is_scheduled": True, "tir.noalias": True})
             W = T.match_buffer(W_handle, (4096, 512), "uint32")
             S = T.match_buffer(S_handle, (4096, 128), "float16")
             V = T.match_buffer(V_handle, (1, 1, 4096), "float16")
@@ -399,7 +399,7 @@ def test_decode_gemv_1_fp32():
     class Before:
         @T.prim_func
         def func(W: T.Buffer((4096, 512), "uint32"), S: T.Buffer((4096, 128), "float16"), V: T.Buffer((1, 1, 4096), "float16"), C: T.Buffer((1, 1, 4096), "float16")):
-            T.func_attr({"global_symbol": "main", "tir.noalias": T.bool(True)})
+            T.func_attr({"global_symbol": "main", "tir.noalias": True})
             # with T.block("root"):
             B = T.alloc_buffer((4096, 4096), "float16")
             C_fp32 = T.alloc_buffer((1, 1, 4096), "float32")
@@ -428,7 +428,7 @@ def test_decode_gemv_1_fp32():
     class After:
         @T.prim_func
         def func(W_handle: T.handle, S_handle: T.handle, V_handle: T.handle, C_handle: T.handle):
-            T.func_attr({"global_symbol": "main", "tir.is_scheduled": 1, "tir.noalias": T.bool(True)})
+            T.func_attr({"global_symbol": "main", "tir.is_scheduled": True, "tir.noalias": True})
             W = T.match_buffer(W_handle, (4096, 512), "uint32")
             S = T.match_buffer(S_handle, (4096, 128), "float16")
             V = T.match_buffer(V_handle, (1, 1, 4096), "float16")
@@ -504,7 +504,7 @@ def test_reduction_no_spatial():
     class After:
         @T.prim_func
         def main(A_handle: T.handle, B_handle: T.handle, rms_norm_handle: T.handle):
-            T.func_attr({"tir.is_scheduled": 1, "tir.noalias": T.bool(True)})
+            T.func_attr({"tir.is_scheduled": True, "tir.noalias": True})
             A = T.match_buffer(A_handle, (1, 1, 4096), "float16")
             B = T.match_buffer(B_handle, (4096,), "float16")
             rms_norm = T.match_buffer(rms_norm_handle, (1, 4096), "float16")
@@ -560,7 +560,7 @@ def test_spatial_inner_no_broadcasting():
     class Module:
         @T.prim_func
         def main(lv575: T.Buffer((1376, 4096), "uint32"), lv576: T.Buffer((344, 4096), "float16"), lv574: T.Buffer((1, 1, 11008), "float16"), lv570: T.Buffer((1, 1, 4096), "float16"), p_output0_intermediate: T.Buffer((1, 1, 4096), "float16")):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             p_output0_intermediate_1 = T.alloc_buffer((11008, 4096), "float16")
             var_matmul_intermediate = T.alloc_buffer((1, 1, 4096), "float16")
             for i, j in T.grid(11008, 4096):
@@ -588,7 +588,7 @@ def test_spatial_inner_no_broadcasting():
     class Expected:
         @T.prim_func
         def main(lv575: T.Buffer((1376, 4096), "uint32"), lv576: T.Buffer((344, 4096), "float16"), lv574: T.Buffer((1, 1, 11008), "float16"), lv570: T.Buffer((1, 1, 4096), "float16"), p_output0_intermediate: T.Buffer((1, 1, 4096), "float16")):
-            T.func_attr({"tir.is_scheduled": 1, "tir.noalias": T.bool(True)})
+            T.func_attr({"tir.is_scheduled": True, "tir.noalias": True})
             var_matmul_intermediate_local = T.alloc_buffer((1, 1, 4096), "float16", scope="local")
             var_matmul_intermediate_rf_local = T.alloc_buffer((16, 1, 1, 4096), "float16", scope="local")
             for ax0_fused_0 in T.thread_binding(256, thread="blockIdx.x"):
@@ -639,7 +639,7 @@ def test_spatial_inner_broadcasting():
     class Module:
         @T.prim_func
         def main(A: T.Buffer((256, 256), "float32"), B: T.Buffer((256, 256), "float32")):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             temp_local = T.alloc_buffer((256,))
             for j in T.serial(256):
                 for k in T.serial(256):
@@ -661,7 +661,7 @@ def test_spatial_inner_broadcasting():
     class Expected:
         @T.prim_func
         def main(A: T.Buffer((256, 256), "float32"), B: T.Buffer((256, 256), "float32")):
-            T.func_attr({"tir.is_scheduled": 1, "tir.noalias": T.bool(True)})
+            T.func_attr({"tir.is_scheduled": True, "tir.noalias": True})
             temp_local_shared = T.alloc_buffer((256,), scope="shared")
             temp_local_rf_local = T.alloc_buffer((16, 256), scope="local")
             for ax0_fused_0 in T.thread_binding(16, thread="blockIdx.x"):
@@ -714,7 +714,7 @@ def test_reduction_inner_no_broadcasting():
     class Module:
         @T.prim_func
         def main(A: T.Buffer((256, 256), "float32"), B: T.Buffer((256,), "float32")):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             temp_local = T.alloc_buffer((256,))
             for i in T.serial(256):
                 for k in T.serial(256):
@@ -736,7 +736,7 @@ def test_reduction_inner_no_broadcasting():
     class Expected:
         @T.prim_func
         def main(A: T.Buffer((256, 256), "float32"), B: T.Buffer((256,), "float32")):
-            T.func_attr({"tir.is_scheduled": 1, "tir.noalias": T.bool(True)})
+            T.func_attr({"tir.is_scheduled": True, "tir.noalias": True})
             # with T.block("root"):
             temp_local_local = T.alloc_buffer((256,), scope="local")
             temp_local_rf_local = T.alloc_buffer((256, 256), scope="local")
@@ -782,7 +782,7 @@ def test_reduction_inner_no_broadcasting2():
     class Module:
         @T.prim_func
         def main(lv9: T.Buffer((2560, 320), "uint32"), lv10: T.Buffer((2560, 80), "float16"), lv1: T.Buffer((1, 2560), "float16"), p_output0_intermediate: T.Buffer((1, 2560), "float32")):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             # with T.block("root"):
             p_output0_intermediate_1 = T.alloc_buffer((2560, 2560), "float16")
             var_matmul_intermediate = T.alloc_buffer((1, 2560), "float16")
@@ -811,7 +811,7 @@ def test_reduction_inner_no_broadcasting2():
     class Expected:
         @T.prim_func
         def main(lv9: T.Buffer((2560, 320), "uint32"), lv10: T.Buffer((2560, 80), "float16"), lv1: T.Buffer((1, 2560), "float16"), p_output0_intermediate: T.Buffer((1, 2560), "float32")):
-            T.func_attr({"tir.is_scheduled": 1, "tir.noalias": T.bool(True)})
+            T.func_attr({"tir.is_scheduled": True, "tir.noalias": True})
             # with T.block("root"):
             var_matmul_intermediate_local = T.alloc_buffer((1, 2560), "float16", scope="local")
             var_matmul_intermediate_rf_local = T.alloc_buffer((16, 1, 2560), "float16", scope="local")
@@ -864,7 +864,7 @@ def test_reduction_inner_spatial_choose_perfect_factor():
     class Module:
         @T.prim_func
         def main(var_A: T.handle, var_B: T.handle, matmul: T.Buffer((T.int64(1), T.int64(32), T.int64(1), T.int64(100)), "float16")):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             n = T.int64()
             A = T.match_buffer(var_A, (T.int64(1), T.int64(32), T.int64(1), n), "float16")
             B = T.match_buffer(var_B, (T.int64(1), T.int64(32), n, T.int64(100)), "float16")
@@ -881,7 +881,7 @@ def test_reduction_inner_spatial_choose_perfect_factor():
     class Expected:
         @T.prim_func
         def main(var_A: T.handle, var_B: T.handle, matmul: T.Buffer((T.int64(1), T.int64(32), T.int64(1), T.int64(100)), "float16")):
-            T.func_attr({"tir.is_scheduled": 1, "tir.noalias": T.bool(True)})
+            T.func_attr({"tir.is_scheduled": True, "tir.noalias": True})
             n = T.int64()
             A = T.match_buffer(var_A, (T.int64(1), T.int64(32), T.int64(1), n), "float16")
             B = T.match_buffer(var_B, (T.int64(1), T.int64(32), n, T.int64(100)), "float16")
@@ -932,7 +932,7 @@ def test_repeat_transpose_gemv():
     class Before:
         @T.prim_func(private=True)
         def fused_relax_repeat_relax_permute_dims_relax_matmul1(p_lv716: T.handle, p_astype66: T.handle, var_matmul_intermediate: T.Buffer((T.int64(1), T.int64(32), T.int64(1), T.int64(128)), "float16")):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             kv_seq_len = T.int64()
             lv716 = T.match_buffer(p_lv716, (T.int64(1), kv_seq_len, T.int64(8), T.int64(128)), "float16")
             astype66 = T.match_buffer(p_astype66, (T.int64(1), T.int64(32), T.int64(1), kv_seq_len), "float16")
@@ -963,7 +963,7 @@ def test_repeat_transpose_gemv():
     class Expected:
         @T.prim_func(private=True)
         def fused_relax_repeat_relax_permute_dims_relax_matmul1(p_lv716: T.handle, p_astype66: T.handle, var_matmul_intermediate: T.Buffer((T.int64(1), T.int64(32), T.int64(1), T.int64(128)), "float16")):
-            T.func_attr({"tir.is_scheduled": 1, "tir.noalias": T.bool(True)})
+            T.func_attr({"tir.is_scheduled": True, "tir.noalias": True})
             kv_seq_len = T.int64()
             lv716 = T.match_buffer(p_lv716, (T.int64(1), kv_seq_len, T.int64(8), T.int64(128)), "float16")
             astype66 = T.match_buffer(p_astype66, (T.int64(1), T.int64(32), T.int64(1), kv_seq_len), "float16")
@@ -1018,7 +1018,7 @@ def test_gemv_dyn_shape_epilogue():
             B: T.Buffer((T.int64(1), T.int64(1), T.int64(4096)), "float16"),
             var_C: T.handle,
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             vocab_size = T.int64()
             A = T.match_buffer(var_A, (T.int64(4096), vocab_size), "float16")
             C = T.match_buffer(var_C, (T.int64(1), T.int64(1), vocab_size))
@@ -1045,7 +1045,7 @@ def test_gemv_dyn_shape_epilogue():
     class Expected:
         @T.prim_func(private=True)
         def main(var_A: T.handle, B: T.Buffer((T.int64(1), T.int64(1), T.int64(4096)), "float16"), var_C: T.handle):
-            T.func_attr({"tir.is_scheduled": 1, "tir.noalias": T.bool(True)})
+            T.func_attr({"tir.is_scheduled": True, "tir.noalias": True})
             vocab_size = T.int64()
             A = T.match_buffer(var_A, (T.int64(4096), vocab_size), "float16")
             C = T.match_buffer(var_C, (T.int64(1), T.int64(1), vocab_size))
@@ -1098,7 +1098,7 @@ def test_gemv_output_one_element():
     class Before:
         @T.prim_func(private=True)
         def main(A: T.Buffer((T.int64(1), T.int64(2048)), "float16"), weight: T.Buffer((T.int64(1), T.int64(2048)), "float16"), out: T.Buffer((T.int64(1), T.int64(1)), "float16")):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             NT_matmul_intermediate = T.alloc_buffer((T.int64(1), T.int64(1)), "float16")
             for i0, i1, k in T.grid(T.int64(1), T.int64(1), T.int64(2048)):
                 with T.block("NT_matmul"):
@@ -1116,7 +1116,7 @@ def test_gemv_output_one_element():
     class Expected:
         @T.prim_func(private=True)
         def main(A: T.Buffer((T.int64(1), T.int64(2048)), "float16"), weight: T.Buffer((T.int64(1), T.int64(2048)), "float16"), out: T.Buffer((T.int64(1), T.int64(1)), "float16")):
-            T.func_attr({"tir.is_scheduled": 1, "tir.noalias": T.bool(True)})
+            T.func_attr({"tir.is_scheduled": True, "tir.noalias": True})
             NT_matmul_intermediate_shared = T.alloc_buffer((T.int64(1), T.int64(1)), "float16", scope="shared")
             NT_matmul_intermediate_rf_local = T.alloc_buffer((T.int64(1024), T.int64(1), T.int64(1)), "float16", scope="local")
             for ax0_fused in T.thread_binding(T.int64(1), thread="blockIdx.x"):
@@ -1160,7 +1160,7 @@ def test_no_reduction_loop_check():
     class Before:
         @T.prim_func(private=True)
         def matmul(lv43: T.Buffer((T.int64(1), T.int64(32), T.int64(1)), "float16"), lv44: T.Buffer((T.int64(1), T.int64(1), T.int64(1)), "float16"), matmul: T.Buffer((T.int64(1), T.int64(32), T.int64(1)), "float16")):
-            T.func_attr({"op_pattern": 4, "tir.noalias": T.bool(True)})
+            T.func_attr({"op_pattern": 4, "tir.noalias": True})
             # with T.block("root"):
             for i0, i1, i2, k in T.grid(T.int64(1), T.int64(32), T.int64(1), T.int64(1)):
                 with T.block("matmul"):
