@@ -38,7 +38,7 @@ def test_mlp():
             A: T.Buffer((T.int64(128), T.int64(64)), "float32"),
             T_multiply: T.Buffer((T.int64(128), T.int64(64)), "float32"),
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             # with T.block("root"):
             T_multiply_1 = T.alloc_buffer((T.int64(128), T.int64(64)))
             compute = T.alloc_buffer((T.int64(128), T.int64(64)))
@@ -81,7 +81,7 @@ def test_mlp():
             B: T.Buffer((T.int64(128), T.int64(64)), "float32"),
             matmul_1: T.Buffer((T.int64(128), T.int64(64)), "float32"),
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             # with T.block("root"):
             for i0, i1, k in T.grid(T.int64(128), T.int64(64), T.int64(128)):
                 with T.block("matmul"):
@@ -98,7 +98,7 @@ def test_mlp():
             B: T.Buffer((T.int64(64), T.int64(128)), "float32"),
             matmul_1: T.Buffer((T.int64(128), T.int64(128)), "float32"),
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             # with T.block("root"):
             for i0, i1, k in T.grid(T.int64(128), T.int64(128), T.int64(64)):
                 with T.block("matmul"):
@@ -197,7 +197,7 @@ def test_mlp_with_tuple():
             A: T.Buffer((T.int64(128), T.int64(64)), "float32"),
             T_multiply: T.Buffer((T.int64(128), T.int64(64)), "float32"),
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             # with T.block("root"):
             T_multiply_1 = T.alloc_buffer((T.int64(128), T.int64(64)))
             compute = T.alloc_buffer((T.int64(128), T.int64(64)))
@@ -240,7 +240,7 @@ def test_mlp_with_tuple():
             B: T.Buffer((T.int64(64), T.int64(128)), "float32"),
             matmul: T.Buffer((T.int64(64), T.int64(128)), "float32"),
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             # with T.block("root"):
             for i0, i1, k in T.grid(T.int64(64), T.int64(128), T.int64(64)):
                 with T.block("matmul"):
@@ -257,7 +257,7 @@ def test_mlp_with_tuple():
             B: T.Buffer((T.int64(128), T.int64(64)), "float32"),
             matmul: T.Buffer((T.int64(128), T.int64(64)), "float32"),
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             # with T.block("root"):
             for i0, i1, k in T.grid(T.int64(128), T.int64(64), T.int64(128)):
                 with T.block("matmul"):
@@ -274,7 +274,7 @@ def test_mlp_with_tuple():
             T_split: T.Buffer((64, 64), "float32"),
             T_split_1: T.Buffer((64, 64), "float32"),
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             # with T.block("root"):
             for ax1, ax2 in T.grid(64, 64):
                 with T.block("T_split"):
