@@ -68,17 +68,17 @@ def test_qdq_op_infer_struct_info_symbolic():
     )
 
 
-def test_qdq_e4m3_float8_op_infer_struct_info_symbolic():
+def test_qdq_float8_e4m3fn_op_infer_struct_info_symbolic():
     bb = relax.BlockBuilder()
     n = tir.Var("n", "int64")
     x = relax.Var("x", R.Tensor((n, 3), "float32"))
-    dx = relax.Var("dx", R.Tensor((n, 3), "e4m3_float8"))
+    dx = relax.Var("dx", R.Tensor((n, 3), "float8_e4m3fn"))
     s = relax.Var("s", R.Tensor([3], "float32"))
     zp = relax.Var("zp", R.Tensor([3], "float16"))
     _check_inference(
         bb,
-        relax.op.quantize(x, s, zp, 1, "e4m3_float8"),
-        relax.TensorStructInfo((n, 3), "e4m3_float8"),
+        relax.op.quantize(x, s, zp, 1, "float8_e4m3fn"),
+        relax.TensorStructInfo((n, 3), "float8_e4m3fn"),
     )
     _check_inference(
         bb,
@@ -87,8 +87,8 @@ def test_qdq_e4m3_float8_op_infer_struct_info_symbolic():
     )
 
 
-def test_qdq_e5m2_float8_op_infer_struct_info_symbolic():
-    dtype = "e5m2_float8"
+def test_qdq_float8_e5m2_op_infer_struct_info_symbolic():
+    dtype = "float8_e5m2"
     bb = relax.BlockBuilder()
     n = tir.Var("n", "int64")
     x = relax.Var("x", R.Tensor((n, 3), "float32"))
