@@ -56,7 +56,7 @@ def run_cpu(mod, func_name, *args, exec_mode):
 def test_unique(exec_mode):
     # TODO(prakalp): also add test for compiling and running on cuda device.
     data_numpy = np.random.randint(0, 16, (16, 16))
-    data = tvm.nd.array(data_numpy)
+    data = tvm.runtime.tensor(data_numpy)
     result, result_sorted = run_cpu(InputModule, "foo", data, exec_mode=exec_mode)
 
     expected_output_sorted, indices = np.unique(data_numpy, return_index=True)
@@ -91,7 +91,7 @@ def test_print(exec_mode):
             run_cpu(
                 PrintTest,
                 "foo",
-                tvm.nd.array(np.array(1).astype("int32")),
+                tvm.runtime.tensor(np.array(1).astype("int32")),
                 exec_mode=exec_mode,
             )
             test_out.seek(0)
@@ -108,7 +108,7 @@ def test_assert_passes(exec_mode):
         _ = R.assert_op(relax.const(True))
         return x
 
-    run_cpu(func, tvm.nd.array(np.array(1).astype("int32")), exec_mode=exec_mode)
+    run_cpu(func, tvm.runtime.tensor(np.array(1).astype("int32")), exec_mode=exec_mode)
 
 
 def test_assert_passes_with_format_args(exec_mode):
@@ -117,7 +117,7 @@ def test_assert_passes_with_format_args(exec_mode):
         _ = R.assert_op(relax.const(True), x, format="You won't see me")
         return x
 
-    run_cpu(func, tvm.nd.array(np.array(1).astype("int32")), exec_mode=exec_mode)
+    run_cpu(func, tvm.runtime.tensor(np.array(1).astype("int32")), exec_mode=exec_mode)
 
 
 def test_assert_fails(exec_mode):
@@ -127,7 +127,7 @@ def test_assert_fails(exec_mode):
         return x
 
     with pytest.raises(AssertionError, match="Assertion Failed"):
-        run_cpu(func, tvm.nd.array(np.array(1).astype("int32")), exec_mode=exec_mode)
+        run_cpu(func, tvm.runtime.tensor(np.array(1).astype("int32")), exec_mode=exec_mode)
 
 
 def test_assert_fails_with_message(exec_mode):
@@ -137,7 +137,7 @@ def test_assert_fails_with_message(exec_mode):
         return x
 
     with pytest.raises(AssertionError, match="I failed..."):
-        run_cpu(func, tvm.nd.array(np.array(1).astype("int32")), exec_mode=exec_mode)
+        run_cpu(func, tvm.runtime.tensor(np.array(1).astype("int32")), exec_mode=exec_mode)
 
 
 def test_assert_fails_with_args(exec_mode):
@@ -147,7 +147,7 @@ def test_assert_fails_with_args(exec_mode):
         return x
 
     with pytest.raises(AssertionError, match="5, 5"):
-        run_cpu(func, tvm.nd.array(np.array(5).astype("int32")), exec_mode=exec_mode)
+        run_cpu(func, tvm.runtime.tensor(np.array(5).astype("int32")), exec_mode=exec_mode)
 
 
 def test_assert_fails_with_formatted_args(exec_mode):
@@ -157,7 +157,7 @@ def test_assert_fails_with_formatted_args(exec_mode):
         return x
 
     with pytest.raises(AssertionError, match="Number: 6"):
-        run_cpu(func, tvm.nd.array(np.array(6).astype("int32")), exec_mode=exec_mode)
+        run_cpu(func, tvm.runtime.tensor(np.array(6).astype("int32")), exec_mode=exec_mode)
 
 
 def test_assert_on_argument_passes(exec_mode):
@@ -166,8 +166,8 @@ def test_assert_on_argument_passes(exec_mode):
         _ = R.assert_op(condition)
         return x
 
-    condition = tvm.nd.array(np.array(True))
-    x = tvm.nd.array(np.array(5).astype("int32"))
+    condition = tvm.runtime.tensor(np.array(True))
+    x = tvm.runtime.tensor(np.array(5).astype("int32"))
     run_cpu(func, condition, x, exec_mode=exec_mode)
 
 
@@ -177,8 +177,8 @@ def test_assert_on_argument_fails(exec_mode):
         _ = R.assert_op(condition)
         return x
 
-    condition = tvm.nd.array(np.array(False))
-    x = tvm.nd.array(np.array(5).astype("int32"))
+    condition = tvm.runtime.tensor(np.array(False))
+    x = tvm.runtime.tensor(np.array(5).astype("int32"))
     with pytest.raises(AssertionError):
         run_cpu(func, condition, x, exec_mode=exec_mode)
 
@@ -190,7 +190,7 @@ def test_assert_on_symbolic_var_passes(exec_mode):
         _ = R.assert_op(R.prim_value(N % 8 == 0))
         return x
 
-    x = tvm.nd.array(np.arange(8, dtype="int32"))
+    x = tvm.runtime.tensor(np.arange(8, dtype="int32"))
     run_cpu(func, x, exec_mode=exec_mode)
 
 
@@ -201,7 +201,7 @@ def test_assert_on_symbolic_var_fails(exec_mode):
         _ = R.assert_op(R.prim_value(N % 8 == 0))
         return x
 
-    x = tvm.nd.array(np.arange(10, dtype="int32"))
+    x = tvm.runtime.tensor(np.arange(10, dtype="int32"))
     with pytest.raises(AssertionError):
         run_cpu(func, x, exec_mode=exec_mode)
 
@@ -238,14 +238,17 @@ def test_op_shape_of(exec_mode):
     assert const_shape == tvm.runtime.ShapeTuple([2, 2])
 
     scalar_shape = run_cpu(
-        ShapeOfTest, "get_shape", tvm.nd.array(np.array(1, dtype="int32")), exec_mode=exec_mode
+        ShapeOfTest,
+        "get_shape",
+        tvm.runtime.tensor(np.array(1, dtype="int32")),
+        exec_mode=exec_mode,
     )
     assert scalar_shape == tvm.runtime.ShapeTuple([])
 
     tensor_shape = run_cpu(
         ShapeOfTest,
         "get_shape",
-        tvm.nd.array(np.zeros((1, 2, 3)).astype("int32")),
+        tvm.runtime.tensor(np.zeros((1, 2, 3)).astype("int32")),
         exec_mode=exec_mode,
     )
     assert tensor_shape == tvm.runtime.ShapeTuple([1, 2, 3])
@@ -253,7 +256,7 @@ def test_op_shape_of(exec_mode):
     constrained_shape = run_cpu(
         ShapeOfTest,
         "get_constrained_shape",
-        tvm.nd.array(np.zeros((1,)).astype("int32")),
+        tvm.runtime.tensor(np.zeros((1,)).astype("int32")),
         exec_mode=exec_mode,
     )
     assert constrained_shape == tvm.runtime.ShapeTuple([1])
@@ -283,25 +286,25 @@ def test_op_shape_to_tensor(exec_mode):
     out2d = run_cpu(
         ShapeToTensorTest, "const_shape", tvm.runtime.ShapeTuple([3, 2]), exec_mode=exec_mode
     )
-    assert isinstance(out2d, tvm.runtime.ndarray.NDArray)
+    assert isinstance(out2d, tvm.runtime.Tensor)
     assert np.array_equal(out2d.numpy(), np.array([3, 2]))
 
     out3d = run_cpu(
         ShapeToTensorTest, "const_shape", tvm.runtime.ShapeTuple([3, 3, 2]), exec_mode=exec_mode
     )
-    assert isinstance(out3d, tvm.runtime.ndarray.NDArray)
+    assert isinstance(out3d, tvm.runtime.Tensor)
     assert np.array_equal(out3d.numpy(), np.array([3, 3, 2]))
 
     out4d = run_cpu(
         ShapeToTensorTest, "const_shape", tvm.runtime.ShapeTuple([3, 3, 2, 2]), exec_mode=exec_mode
     )
-    assert isinstance(out4d, tvm.runtime.ndarray.NDArray)
+    assert isinstance(out4d, tvm.runtime.Tensor)
     assert np.array_equal(out4d.numpy(), np.array([3, 3, 2, 2]))
 
     outs = run_cpu(
         ShapeToTensorTest, "symbolic_shape", tvm.runtime.ShapeTuple([3, 2]), exec_mode=exec_mode
     )
-    assert isinstance(outs, tvm.runtime.ndarray.NDArray)
+    assert isinstance(outs, tvm.runtime.Tensor)
     assert np.array_equal(outs.numpy(), np.array([3, 2]))
 
 
@@ -317,7 +320,7 @@ def test_op_call_pure_packed(exec_mode):
 
     np.random.seed(0)  # to avoid flakiness
     arr = np.random.rand(3, 4).astype("float32")
-    copy_found = run_cpu(CallPureTest, "pure_copy", tvm.nd.array(arr), exec_mode=exec_mode)
+    copy_found = run_cpu(CallPureTest, "pure_copy", tvm.runtime.tensor(arr), exec_mode=exec_mode)
     assert (copy_found.numpy() == arr).all()
 
 
@@ -335,7 +338,7 @@ def test_op_call_inplace_packed(exec_mode):
             )
             return z
 
-    @tvm.register_func("test.inplace.add", override=True)
+    @tvm.register_global_func("test.inplace.add", override=True)
     def inplace_add(a, b):
         arr_a = a.numpy()
         arr_b = b.numpy()
@@ -362,18 +365,18 @@ def test_op_call_inplace_packed(exec_mode):
     arr_a = np.random.rand(3, 4).astype("float32")
     arr_b = np.random.rand(3, 4).astype("float32")
     sum = arr_a + arr_b
-    tvm_arr_a = tvm.nd.array(arr_a)
+    tvm_arr_a = tvm.runtime.tensor(arr_a)
     result = run_cpu(
-        CallInplaceAddTest, "inplace_add", tvm_arr_a, tvm.nd.array(arr_b), exec_mode=exec_mode
+        CallInplaceAddTest, "inplace_add", tvm_arr_a, tvm.runtime.tensor(arr_b), exec_mode=exec_mode
     )
     assert result == tvm_arr_a
     assert (result.numpy() == sum).all()
 
-    @tvm.register_func("test.inplace.tuple_add", override=True)
+    @tvm.register_global_func("test.inplace.tuple_add", override=True)
     def inplace_tuple_add(a, b):
         arr_a = a.numpy()
         arr_b = b.numpy()
-        c = tvm.nd.array(arr_a + arr_b)
+        c = tvm.runtime.tensor(arr_a + arr_b)
         for i in range(len(arr_a)):
             for j in range(len(arr_a[i])):
                 arr_a[i][j] = arr_a[i][j] + arr_b[i][j]
@@ -397,8 +400,8 @@ def test_op_call_inplace_packed(exec_mode):
     arr_a = np.random.rand(3, 4).astype("float32")
     arr_b = np.random.rand(3, 4).astype("float32")
     sum = arr_a + arr_b
-    tvm_arr_a = tvm.nd.array(arr_a)
-    tvm_arr_b = tvm.nd.array(arr_b)
+    tvm_arr_a = tvm.runtime.tensor(arr_a)
+    tvm_arr_b = tvm.runtime.tensor(arr_b)
     result = run_cpu(CallInplaceTuple, "inplace_tuple", tvm_arr_a, tvm_arr_b, exec_mode=exec_mode)
     assert result[0] == tvm_arr_a
     assert (result[0].numpy() == sum).all()
@@ -406,6 +409,82 @@ def test_op_call_inplace_packed(exec_mode):
     assert (result[1].numpy() == sum).all()
 
 
+def test_op_call_py_func(exec_mode):
+    """Test R.call_py_func operator functionality."""
+    import torch
+
+    def torch_relu(x):
+        if isinstance(x, tvm.runtime.Tensor):
+            x_torch = torch.from_numpy(x.numpy())
+        elif hasattr(x, "asnumpy"):
+            x_torch = torch.from_numpy(x.asnumpy())
+        else:
+            x_np = np.array(x)
+            if isinstance(x_np, tvm.runtime.Tensor):
+                x_torch = torch.from_numpy(x_np.numpy())
+            elif len(x_np) > 0 and isinstance(x_np[0], tvm.runtime.Tensor):
+                x_torch = torch.from_numpy(np.array([t.numpy() for t in x_np]))
+                if x_torch.ndim > 1:
+                    x_torch = x_torch.flatten()
+            else:
+                x_torch = torch.from_numpy(x_np)
+        result = torch.relu(x_torch)
+        return tvm.runtime.tensor(result.numpy())
+
+    def torch_sigmoid(x):
+        if isinstance(x, tvm.runtime.Tensor):
+            x_torch = torch.from_numpy(x.numpy())
+        elif hasattr(x, "asnumpy"):
+            x_torch = torch.from_numpy(x.asnumpy())
+        else:
+            x_np = np.array(x)
+            if isinstance(x_np, tvm.runtime.Tensor):
+                x_torch = torch.from_numpy(x_np.numpy())
+            elif len(x_np) > 0 and isinstance(x_np[0], tvm.runtime.Tensor):
+                x_torch = torch.from_numpy(np.array([t.numpy() for t in x_np]))
+                if x_torch.ndim > 1:
+                    x_torch = x_torch.flatten()
+            else:
+                x_torch = torch.from_numpy(x_np)
+        result = torch.sigmoid(x_torch)
+        return tvm.runtime.tensor(result.numpy())
+
+    register_func = tvm.get_global_func("vm.builtin.register_py_func")
+    register_func("torch_relu", torch_relu)
+    register_func("torch_sigmoid", torch_sigmoid)
+
+    @tvm.script.ir_module
+    class CallPyFuncTest:
+        @R.function
+        def simple_call(x: R.Tensor((3,), "float32")):
+            result = R.call_py_func(R.str("torch_relu"), (x,), out_sinfo=R.Tensor((3,), "float32"))
+            return result
+
+        @R.function
+        def multiple_calls(x: R.Tensor((2,), "float32")):
+            y = R.call_py_func(R.str("torch_relu"), (x,), out_sinfo=R.Tensor((2,), "float32"))
+            z = R.call_py_func(R.str("torch_sigmoid"), (y,), out_sinfo=R.Tensor((2,), "float32"))
+            return z
+
+    np.random.seed(0)
+    x_data = np.array([-1.0, 0.0, 1.0], dtype=np.float32)
+    x_tvm = tvm.runtime.tensor(x_data)
+
+    result = run_cpu(CallPyFuncTest, "simple_call", x_tvm, exec_mode=exec_mode)
+    expected = np.maximum(x_data, 0.0)
+    assert (result.numpy() == expected).all()
+
+    y_data = np.array([-0.5, 0.5], dtype=np.float32)
+    y_tvm = tvm.runtime.tensor(y_data)
+
+    result2 = run_cpu(CallPyFuncTest, "multiple_calls", y_tvm, exec_mode=exec_mode)
+    expected2 = 1.0 / (1.0 + np.exp(-np.maximum(y_data, 0.0)))
+    assert (result2.numpy() == expected2).all()
+
+    clear_func = tvm.get_global_func("vm.builtin.clear_py_func_registry")
+    clear_func()
+
+
 def test_op_to_device(exec_mode):
     @tvm.script.ir_module
     class CallToDevice:
@@ -422,7 +501,7 @@ def test_op_to_device(exec_mode):
 
     np.random.seed(0)  # to avoid flakiness
     arr = np.random.rand(3, 4).astype("float32")
-    copy_found = run_cpu(CallToDevice, "to_dev", tvm.nd.array(arr), exec_mode=exec_mode)
+    copy_found = run_cpu(CallToDevice, "to_dev", tvm.runtime.tensor(arr), exec_mode=exec_mode)
     assert (copy_found.numpy() == arr).all()
 
 
@@ -439,7 +518,7 @@ def test_op_to_vdevice(exec_mode):
 
     np.random.seed(0)
     arr = np.random.rand(3, 4).astype("float32")
-    copy_found = run_cpu(ToVDevice, "to_vdev", tvm.nd.array(arr), exec_mode=exec_mode)
+    copy_found = run_cpu(ToVDevice, "to_vdev", tvm.runtime.tensor(arr), exec_mode=exec_mode)
     assert (copy_found.numpy() == arr).all()
 
 
@@ -454,10 +533,10 @@ def test_scalar_tensor_as_branch_condition(exec_mode):
             out = R.prim_value(10)
         return out
 
-    res = run_cpu(func, tvm.nd.array(np.array(True)), exec_mode=exec_mode)
+    res = run_cpu(func, tvm.runtime.tensor(np.array(True)), exec_mode=exec_mode)
     assert res == 5
 
-    res = run_cpu(func, tvm.nd.array(np.array(False)), exec_mode=exec_mode)
+    res = run_cpu(func, tvm.runtime.tensor(np.array(False)), exec_mode=exec_mode)
     assert res == 10
 
 
@@ -491,10 +570,10 @@ def test_computed_prim_value_as_branch_condition(exec_mode):
             out = R.prim_value(10)
         return out
 
-    res = run_cpu(func, tvm.nd.array(np.arange(16)), exec_mode=exec_mode)
+    res = run_cpu(func, tvm.runtime.tensor(np.arange(16)), exec_mode=exec_mode)
     assert res == 5
 
-    res = run_cpu(func, tvm.nd.array(np.arange(20)), exec_mode=exec_mode)
+    res = run_cpu(func, tvm.runtime.tensor(np.arange(20)), exec_mode=exec_mode)
     assert res == 10
 
 
