@@ -79,7 +79,7 @@ w_uv = None
 
 
 # Register a dumb function for testing purpose.
-@tvm.register_func("test.dumb_function", override=True)
+@tvm.register_global_func("test.dumb_function", override=True)
 def _dumb_function():
     raise RuntimeError("Dumb function isn't supposed to be accessed.")
 
@@ -134,7 +134,7 @@ def set_global_func(dtype):
         with target:
             mod = dl.ApplyDefaultSchedule(dl.gpu.Fallback())(mod)
         f = tvm.tir.build(mod["main"], target=target)
-        builts.append(f.entry_func)
+        builts.append(f.main)
 
     (
         ftranspose_append,
@@ -185,7 +185,7 @@ def create_kv_cache(dtype):
         1,
         10000,
         None,  # rope_ext_factors
-        tvm.nd.empty((), dtype, device=device),
+        tvm.runtime.empty((), dtype, device=device),
         None,  # f_transpose_append_mha
         ftranspose_append,
         ["tir", fmla_prefill_ragged],  # fattn_prefill_ragged
@@ -218,7 +218,7 @@ def verify_cached_kv(kv_cache, seq_ids, expected_kv):
     for seq_id in seq_ids:
         kv_expected = expected_kv[seq_id]
         seq_length = expected_kv[seq_id].shape[1]
-        kv_actual = tvm.nd.empty(kv_expected.shape, dtype=dtype, device=device)
+        kv_actual = tvm.runtime.empty(kv_expected.shape, dtype=dtype, device=device)
         fdebug_get_kv(kv_cache, seq_id, 0, seq_length, kv_actual)
         torch.testing.assert_close(
             torch.from_numpy(kv_actual.numpy()).to(device_torch), kv_expected, rtol=1e-3, atol=1e-3
@@ -301,17 +301,17 @@ def apply_attention(
             is_decode_request = False
 
     for layer_id in range(num_layers):
-        queries = tvm.nd.array(global_new_q[layer_id].cpu().numpy(), device)
-        key_value = tvm.nd.array(global_new_kv[layer_id].cpu().numpy(), device)
+        queries = tvm.runtime.tensor(global_new_q[layer_id].cpu().numpy(), device)
+        key_value = tvm.runtime.tensor(global_new_kv[layer_id].cpu().numpy(), device)
         total_seq_length = global_new_q[layer_id].shape[0]
-        outputs1 = tvm.nd.empty(
+        outputs1 = tvm.runtime.empty(
             (total_seq_length, num_attention_heads, v_head_dim), dtype, device=device
         )
-        lse1 = tvm.nd.empty((total_seq_length, num_attention_heads), "float32", device=device)
-        outputs2 = tvm.nd.empty(
+        lse1 = tvm.runtime.empty((total_seq_length, num_attention_heads), "float32", device=device)
+        outputs2 = tvm.runtime.empty(
             (total_seq_length, num_attention_heads, kv_lora_rank), dtype, device=device
         )
-        lse2 = tvm.nd.empty((total_seq_length, num_attention_heads), "float32", device=device)
+        lse2 = tvm.runtime.empty((total_seq_length, num_attention_heads), "float32", device=device)
 
         fappend_mla_kv(kv_cache, layer_id, key_value)
         if not is_decode_request:
@@ -328,8 +328,8 @@ def apply_attention(
                 total_seq_length, num_attention_heads, qk_rope_head_dim
             )
             keys = torch.cat([keys, k_pe_expanded], dim=2)
-            keys_tvm = tvm.nd.array(keys.cpu().numpy(), device)
-            values_tvm = tvm.nd.array(values.cpu().numpy(), device)
+            keys_tvm = tvm.runtime.tensor(keys.cpu().numpy(), device)
+            values_tvm = tvm.runtime.tensor(values.cpu().numpy(), device)
             fself_attn(kv_cache, layer_id, sm_scale, queries, keys_tvm, values_tvm, outputs1, lse1)
 
         if not all_new_sequences or is_decode_request:
@@ -340,9 +340,9 @@ def apply_attention(
             queries_lora_np = torch.cat(
                 [torch.bmm(queries_lora_np.permute(1, 0, 2), w_uk).permute(1, 0, 2), q_pe], dim=2
             )
-            queries_lora = tvm.nd.array(queries_lora_np.cpu().numpy(), device)
+            queries_lora = tvm.runtime.tensor(queries_lora_np.cpu().numpy(), device)
             fcross_attn(kv_cache, layer_id, sm_scale, queries_lora, outputs2, lse2)
-            cross_attn_output = tvm.nd.array(
+            cross_attn_output = tvm.runtime.tensor(
                 torch.bmm(
                     torch.from_numpy(outputs2.numpy()).to(device_torch).permute(1, 0, 2), w_uv
                 )
