@@ -277,18 +277,26 @@ def test_tracking_through_externally_exposed_func(provide_entry_func_name):
 
 def test_unused_relax_func_symbolic_shape():
     # Test with relax function w/ symbolic shape.
-    @tvm.script.ir_module
+    @tvm.script.ir_module(check_well_formed=False)
     class InputModule:
         @T.prim_func
-        def tir_add(
-            x: T.Buffer((16, 16), "float32"),
-            y: T.Buffer((16, 16), "float32"),
-            z: T.Buffer((16, 16), "float32"),
+        def tir_matmul(
+            x_handle: T.handle,
+            y_handle: T.handle,
+            z_handle: T.handle,
         ) -> None:
-            for i, j in T.grid(16, 16):
-                with T.block("add"):
-                    vi, vj = T.axis.remap("SS", [i, j])
-                    z[vi, vj] = x[vi, vj] + y[vi, vj]
+            m = T.int64()
+            n = T.int64()
+            k = T.int64()
+            x = T.match_buffer(x_handle, (m, n), "float32")
+            y = T.match_buffer(y_handle, (n, k), "float32")
+            z = T.match_buffer(z_handle, (m, k), "float32")
+            for i, j, k in T.grid(m, k, n):
+                with T.block("matmul"):
+                    vi, vj, vk = T.axis.remap("SSR", [i, j, k])
+                    with T.init():
+                        z[vi, vj] = 0.0
+                    z[vi, vj] = z[vi, vj] + x[vi, vk] * y[vk, vj]
 
         @R.function(private=True)
         def unused_func(x: R.Tensor(("m", "n"), "float32"), w: R.Tensor(("n", "k"), "float32")):
@@ -298,7 +306,7 @@ def test_unused_relax_func_symbolic_shape():
         @R.function
         def main(x: R.Tensor(("m", "n"), "float32"), w: R.Tensor(("n", "k"), "float32")):
             m, k = T.int64(), T.int64()
-            gv0 = R.call_tir(InputModule.tir_add, (x, w), R.Tensor((m + 1, k), dtype="float32"))
+            gv0 = R.call_tir(InputModule.tir_matmul, (x, w), R.Tensor((m, k), dtype="float32"))
             return gv0
 
     mod = InputModule
@@ -306,7 +314,7 @@ def test_unused_relax_func_symbolic_shape():
 
     new_mod = DeadCodeElimination()(mod)
     assert check_if_func_exists(new_mod, "main")
-    assert check_if_func_exists(new_mod, "tir_add")
+    assert check_if_func_exists(new_mod, "tir_matmul")
     assert not check_if_func_exists(new_mod, "unused_func")
 
 
@@ -346,6 +354,42 @@ def test_unused_prim_func():
     assert check_if_func_exists(new_mod, "unused_func")
 
 
+def test_preserve_indirectly_used_prim_func():
+    @tvm.script.ir_module
+    class InputModule:
+        @R.function
+        def main(
+            x: R.Tensor((16, 16), "float32"), w: R.Tensor((16, 16), "float32")
+        ) -> R.Tensor((16, 16), "float32"):
+            gv0 = R.call_tir(
+                InputModule.tir_add_tensors,
+                [x, w],
+                out_sinfo=R.Tensor((16, 16), "float32"),
+            )
+            return gv0
+
+        @T.prim_func(private=True)
+        def tir_add_tensors(
+            x: T.Buffer((16, 16), "float32"),
+            y: T.Buffer((16, 16), "float32"),
+            z: T.Buffer((16, 16), "float32"),
+        ):
+            for i, j in T.grid(16, 16):
+                with T.block("add"):
+                    vi, vj = T.axis.remap("SS", [i, j])
+                    z[vi, vj] = InputModule.tir_add_float32(x[vi, vj], y[vi, vj])
+
+        @T.prim_func(private=True)
+        def tir_add_float32(x: T.float32, y: T.float32) -> T.float32:
+            return x + y
+
+    mod = InputModule
+    assert mod
+    new_mod = DeadCodeElimination()(mod)
+
+    tvm.ir.assert_structural_equal(mod, new_mod)
+
+
 def test_multiple_unused_funcs():
     @tvm.script.ir_module
     class InputModule:
@@ -399,7 +443,11 @@ def test_unused_dfb():
                 )
                 lv1: R.Tensor((4, 3, 3, 3), dtype="float32") = R.permute_dims(w, axes=[0, 2, 3, 1])
                 lv2: R.Tensor((2, 26, 26, 4), dtype="float32") = R.nn.conv2d(
-                    lv0, lv1, data_layout="NHWC", kernel_layout="OHWI", out_layout="NHWC"
+                    lv0,
+                    lv1,
+                    data_layout="NHWC",
+                    kernel_layout="OHWI",
+                    out_layout="NHWC",
                 )
                 lv3: R.Tensor((2, 4, 26, 26), dtype="float32") = R.permute_dims(
                     lv2, axes=[0, 3, 1, 2]
@@ -428,7 +476,11 @@ def test_unused_dfb():
                 )
                 lv1: R.Tensor((4, 3, 3, 3), dtype="float32") = R.permute_dims(w, axes=[0, 2, 3, 1])
                 lv2: R.Tensor((2, 26, 26, 4), dtype="float32") = R.nn.conv2d(
-                    lv0, lv1, data_layout="NHWC", kernel_layout="OHWI", out_layout="NHWC"
+                    lv0,
+                    lv1,
+                    data_layout="NHWC",
+                    kernel_layout="OHWI",
+                    out_layout="NHWC",
                 )
                 R.output(lv2)
             gv3 = R.astype(lv2, dtype="float16")
@@ -454,7 +506,7 @@ def test_unused_dfb2():
                 R.output(lv0)
 
             gv_x = R.astype(x, dtype="float16")
-            gv_w = R.astype(x, dtype="float16")
+            gv_w = R.astype(w, dtype="float16")
 
             with R.dataflow():
                 lv1: R.Tensor((2, 28, 28, 3), dtype="float16") = R.permute_dims(
@@ -464,7 +516,11 @@ def test_unused_dfb2():
                     gv_w, axes=[0, 2, 3, 1]
                 )
                 lv3: R.Tensor((2, 26, 26, 4), dtype="float16") = R.nn.conv2d(
-                    lv1, lv2, data_layout="NHWC", kernel_layout="OHWI", out_layout="NHWC"
+                    lv1,
+                    lv2,
+                    data_layout="NHWC",
+                    kernel_layout="OHWI",
+                    out_layout="NHWC",
                 )
                 # dead instruction -> usee lv1 also dead.
                 lv4: R.Tensor((2, 3, 28, 28), dtype="float32") = R.permute_dims(
@@ -481,7 +537,7 @@ def test_unused_dfb2():
             w: R.Tensor((4, 3, 3, 3), dtype="float32"),
         ):
             gv_x = R.astype(x, dtype="float16")
-            gv_w = R.astype(x, dtype="float16")
+            gv_w = R.astype(w, dtype="float16")
 
             with R.dataflow():
                 lv1: R.Tensor((2, 28, 28, 3), dtype="float16") = R.permute_dims(
@@ -491,7 +547,11 @@ def test_unused_dfb2():
                     gv_w, axes=[0, 2, 3, 1]
                 )
                 lv3: R.Tensor((2, 26, 26, 4), dtype="float16") = R.nn.conv2d(
-                    lv1, lv2, data_layout="NHWC", kernel_layout="OHWI", out_layout="NHWC"
+                    lv1,
+                    lv2,
+                    data_layout="NHWC",
+                    kernel_layout="OHWI",
+                    out_layout="NHWC",
                 )
                 R.output(lv3)
             return lv3
@@ -658,5 +718,86 @@ def test_well_formed_output_with_restricted_scope():
     tvm.ir.assert_structural_equal(Expected, After)
 
 
+def test_recursively_defined_lambda():
+    """DCE may be applied to recursively-defined functions
+
+    While most expressions may only contain references to
+    previously-defined variables, local Relax function definitions may
+    contain references to themselves.
+
+    This is a regression test.  In previous implementations, the
+    recursive use of `while_loop` resulted in an error, as
+    `while_loop` was not considered in-scope by the `CollectVarUsage`
+    utility until after the body of `while_loop` had been visited.
+
+    """
+
+    @I.ir_module
+    class Before:
+        @R.function
+        def main(x: R.Tensor((2, 3), "float32")) -> R.Tensor:
+            @R.function
+            def while_loop(
+                i: R.Tensor((), "int32"), s: R.Tensor((2, 3), "float32")
+            ) -> R.Tensor((2, 3), "float32"):
+                cond = R.call_pure_packed(
+                    "test.vm.less", i, R.const(10), sinfo_args=R.Tensor((), dtype="bool")
+                )
+                c = R.const(1, dtype="int32")
+                if cond:
+                    new_i = R.add(i, c)
+                    new_s = R.add(s, x)
+                    r = while_loop(new_i, new_s)
+                else:
+                    r = s
+                return r
+
+            gv = while_loop(R.const(0), x)
+            return gv
+
+    Expected = Before
+
+    verify(Before, Expected)
+
+
+def test_recursively_defined_closure():
+    """DCE may be applied to recursively-defined closures
+
+    This test is identical to `test_recursively_defined_lambda`,
+    except that the threshold for recursion is defined in an enclosed
+    variable outside of the recursive function.
+
+    """
+
+    @I.ir_module
+    class Before:
+        @R.function
+        def main(x: R.Tensor((2, 3), "float32")) -> R.Tensor:
+            threshold = R.const(10)
+
+            @R.function
+            def while_loop(
+                i: R.Tensor((), "int32"), s: R.Tensor((2, 3), "float32")
+            ) -> R.Tensor((2, 3), "float32"):
+                cond = R.call_pure_packed(
+                    "test.vm.less", i, threshold, sinfo_args=R.Tensor((), dtype="bool")
+                )
+                c = R.const(1, dtype="int32")
+                if cond:
+                    new_i = R.add(i, c)
+                    new_s = R.add(s, x)
+                    r = while_loop(new_i, new_s)
+                else:
+                    r = s
+                return r
+
+            gv = while_loop(R.const(0), x)
+            return gv
+
+    Expected = Before
+
+    verify(Before, Expected)
+
+
 if __name__ == "__main__":
     tvm.testing.main()
