@@ -48,7 +48,7 @@ def test_fuse_simple():
         x = relax.Var("x", R.Tensor([10, 20], "float32"))
         p0 = relax.Var("p0", R.Tensor((), "float32"))
 
-        with bb.function("fused_add_exp_squeeze", [x, p0], attrs={"Primitive": 1}, private=True):
+        with bb.function("fused_add_exp_squeeze", [x, p0], attrs={"Primitive": True}, private=True):
             with bb.dataflow():
                 lv0 = bb.emit_te(topi.add, x, p0)
                 lv1 = bb.emit_te(topi.exp, lv0)
@@ -101,7 +101,7 @@ def test_conv2d_fuse():
         w = relax.Var("w", R.Tensor((16, 16, 3, 3), dtype))
         p0 = relax.Var("p0", R.Tensor((), dtype))
         with bb.function(
-            "fused_conv2d_add1_add2", [x, w, p0], attrs={"Primitive": 1}, private=True
+            "fused_conv2d_add1_add2", [x, w, p0], attrs={"Primitive": True}, private=True
         ):
             with bb.dataflow():
                 lv0 = bb.emit_te(
@@ -121,7 +121,7 @@ def test_conv2d_fuse():
         x = relax.Var("x", R.Tensor((1, 16, 64, 64), dtype))
         w = relax.Var("w", R.Tensor((16, 16, 1, 1), dtype))
         y = relax.Var("y", R.Tensor((1, 16, 64, 64), dtype))
-        with bb.function("fused_conv2d1_add2", [x, w, y], attrs={"Primitive": 1}, private=True):
+        with bb.function("fused_conv2d1_add2", [x, w, y], attrs={"Primitive": True}, private=True):
             with bb.dataflow():
                 lv0 = bb.emit_te(
                     topi.nn.conv2d,
@@ -199,7 +199,7 @@ def test_concatenate():
         w = relax.Var("w", R.Tensor((1, 16, 32, 32), "float32"))
         p0 = relax.Var("p0", R.Tensor((), "float32"))
         with bb.function(
-            "fused_upsampling_concatenate_add", [w, x, p0], attrs={"Primitive": 1}, private=True
+            "fused_upsampling_concatenate_add", [w, x, p0], attrs={"Primitive": True}, private=True
         ):
             with bb.dataflow():
                 lv0 = bb.emit_te(topi.nn.upsampling, w, scale_h=2.0, scale_w=2.0)
@@ -293,7 +293,7 @@ def test_fuse_tuple_get_elemwise():
         with bb.function(
             "fused_split_sigmoid_tanh_exp_multiply_add",
             [dense],
-            attrs={"Primitive": 1},
+            attrs={"Primitive": True},
             private=True,
         ):
             with bb.dataflow():
@@ -347,7 +347,7 @@ def test_tuple_get_root():
 
         # Grouped function
         x = relax.Var("x", R.Tensor((1, 3 * dim), "float32"))
-        with bb.function("fused_split", [x], attrs={"Primitive": 1}, private=True):
+        with bb.function("fused_split", [x], attrs={"Primitive": True}, private=True):
             with bb.dataflow():
                 lv0 = bb.emit_te(topi.split, x, indices_or_sections=3, axis=1)
                 gv = bb.emit_output(relax.TupleGetItem(lv0, 0))
@@ -404,7 +404,7 @@ def test_tuple_intermediate():
         with bb.function(
             "fused_squeeze_add_squeeze1_add_add_add_concatenate_squeeze2_add1",
             [x, p0, p1, p2, p3, p4],
-            attrs={"Primitive": 1},
+            attrs={"Primitive": True},
             private=True,
         ):
             with bb.dataflow():
@@ -507,7 +507,7 @@ def test_tuple_consecutive():
         with bb.function(
             "fused_add_add_add_concatenate_add1_add_add_add_concatenate_add1_add_add_add_concatenate_add1_concatenate1",
             [x, p0, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11],
-            attrs={"Primitive": 1},
+            attrs={"Primitive": True},
             private=True,
         ):
             with bb.dataflow():
@@ -532,7 +532,9 @@ def test_tuple_consecutive():
         # Grouped function 2
         concat = relax.Var("concat", R.Tensor((1, 144, 64, 64), "float32"))
         p0 = relax.Var("p0", R.Tensor((), "float32"))
-        with bb.function("fused_pool2d_add2", [concat, p0], attrs={"Primitive": 1}, private=True):
+        with bb.function(
+            "fused_pool2d_add2", [concat, p0], attrs={"Primitive": True}, private=True
+        ):
             with bb.dataflow():
                 lv0 = bb.emit_te(
                     topi.nn.pool2d,
@@ -618,7 +620,7 @@ def test_inception_like():
         # Grouped function 1
         x = relax.Var("x", R.Tensor((1, 16, 64, 64), "float32"))
         w = relax.Var("w", R.Tensor((16, 16, 3, 3), "float32"))
-        with bb.function("fused_conv2d_relu", [x, w], attrs={"Primitive": 1}, private=True):
+        with bb.function("fused_conv2d_relu", [x, w], attrs={"Primitive": True}, private=True):
             with bb.dataflow():
                 lv0 = bb.emit_te(
                     topi.nn.conv2d,
@@ -635,7 +637,7 @@ def test_inception_like():
         # Grouped function 2
         x = relax.Var("x", R.Tensor((1, 32, 64, 64), "float32"))
         w = relax.Var("w", R.Tensor((16, 32, 3, 3), "float32"))
-        with bb.function("fused_conv2d1_relu", [x, w], attrs={"Primitive": 1}, private=True):
+        with bb.function("fused_conv2d1_relu", [x, w], attrs={"Primitive": True}, private=True):
             with bb.dataflow():
                 lv0 = bb.emit_te(
                     topi.nn.conv2d,
@@ -700,7 +702,7 @@ def test_fuse_parallel_injective():
         with bb.function(
             "fused_add_squeeze_transpose_transpose1_left_shift",
             [x, p0],
-            attrs={"Primitive": 1},
+            attrs={"Primitive": True},
             private=True,
         ):
             with bb.dataflow():
@@ -746,7 +748,7 @@ def test_softmax():
 
         # Grouped function
         x = relax.Var("x", R.Tensor((16, 16), "float32"))
-        with bb.function("fused_softmax_cast", [x], attrs={"Primitive": 1}, private=True):
+        with bb.function("fused_softmax_cast", [x], attrs={"Primitive": True}, private=True):
             with bb.dataflow():
                 lv0 = bb.emit_te(topi.nn.softmax, x)
                 gv = bb.emit_output(bb.call_te(topi.cast, lv0, dtype="float16"))
@@ -793,7 +795,7 @@ def test_multiple_relax_functions():
 
         x = relax.Var("x", R.Tensor([10, 20], "float32"))
         p0 = relax.Var("p0", R.Tensor((), "float32"))
-        with bb.function("fused_add_exp_squeeze", [x, p0], attrs={"Primitive": 1}, private=True):
+        with bb.function("fused_add_exp_squeeze", [x, p0], attrs={"Primitive": True}, private=True):
             with bb.dataflow():
                 lv0 = bb.emit_te(topi.add, x, p0)
                 lv1 = bb.emit_te(topi.exp, lv0)
@@ -803,7 +805,9 @@ def test_multiple_relax_functions():
 
         x = relax.Var("x", R.Tensor([20, 10], "float32"))
         p0 = relax.Var("p0", R.Tensor((), "float32"))
-        with bb.function("fused_add1_exp1_squeeze1", [x, p0], attrs={"Primitive": 1}, private=True):
+        with bb.function(
+            "fused_add1_exp1_squeeze1", [x, p0], attrs={"Primitive": True}, private=True
+        ):
             with bb.dataflow():
                 lv0 = bb.emit_te(topi.add, x, p0)
                 lv1 = bb.emit_te(topi.exp, lv0)
@@ -952,7 +956,7 @@ def test_layer_norm_silu():
 
         @R.function(private=True)
         def fused_layer_norm_relu(x: R.Tensor((1, 512, 64, 64), dtype="float32"), mean: R.Tensor((64, 64), dtype="float32"), var: R.Tensor((64, 64), dtype="float32")) -> R.Tensor((1, 512, 64, 64), dtype="float32"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             cls = Expected
             with R.dataflow():
                 gv0 = R.call_tir(cls.layer_norm, (x, mean, var), out_sinfo=R.Tensor((1, 512, 64, 64), 'float32'))
@@ -1094,7 +1098,7 @@ def test_multiple_paths():
 
         @R.function(private=True)
         def fused_conv2d_add_add2(inp_0: R.Tensor((2, 320, 64, 64), dtype="float32"), w1: R.Tensor((320, 320, 3, 3), dtype="float32"), lv28: R.Tensor((1, 320, 1, 1), dtype="float32"), lv35: R.Tensor((2, 320, 1, 1), dtype="float32")) -> R.Tensor((2, 320, 64, 64), dtype="float32"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             cls = Expected
             with R.dataflow():
                 lv27 = R.call_tir(cls.conv2d, (inp_0, w1), out_sinfo=R.Tensor((2, 320, 64, 64), dtype="float32"))
@@ -1106,7 +1110,7 @@ def test_multiple_paths():
         @R.function(private=True)
         def fused_matmul_add1(inp_1: R.Tensor((2, 1280), dtype="float32"), lv31: R.Tensor((1280, 320), dtype="float32"), b2: R.Tensor((320,), dtype="float32")) -> R.Tensor((2, 320), dtype="float32"):
             cls = Expected
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             with R.dataflow():
                 lv32 = R.call_tir(cls.matmul, (inp_1, lv31), out_sinfo=R.Tensor((2, 320), dtype="float32"))
                 gv = R.call_tir(cls.add1, (lv32, b2), out_sinfo=R.Tensor((2, 320), dtype="float32"))
@@ -1239,7 +1243,7 @@ def test_dead_group():
 
         @R.function(private=True)
         def fused_matmul1_add1(inp_1: R.Tensor((1, 128), dtype="float32"), lv4: R.Tensor((128, 10), dtype="float32"), linear2_bias: R.Tensor((10,), dtype="float32")) -> R.Tensor((1, 10), dtype="float32"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             cls = Expected
             with R.dataflow():
                 lv5 = R.call_tir(cls.matmul1, (inp_1, lv4), out_sinfo=R.Tensor((1, 10), dtype="float32"))
@@ -1283,7 +1287,7 @@ def test_symbolic_shape_aware_fuse():
         def fused_add_exp_squeeze(
             x: R.Tensor(["n", "m"], "float32"), p0: R.Tensor([], "float32")
         ) -> R.Tensor(["n", "m"], dtype="float32"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             with R.dataflow():
                 lv0 = R.emit_te(topi.add, x, p0)
                 lv1 = R.emit_te(topi.exp, lv0)
@@ -1321,7 +1325,7 @@ def test_symbolic_shape_aware_fuse_2():
         def fused_full_trilu_broadcast_to(
             s: R.Shape(["n"]),
         ) -> R.Tensor([1, 1, "n", "n"], "float32"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             n = T.int64()
             with R.dataflow():
                 lv0 = R.emit_te(topi.full, [n, n], "float32", 0)
@@ -1369,7 +1373,7 @@ def test_shape_expr_arg():
         def fused_full_trilu_broadcast_to(
             s: R.Shape(["n"]),
         ) -> R.Tensor([1, 1, "n", "n"], "float32"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             n = T.int64()
             with R.dataflow():
                 lv0 = R.emit_te(topi.full, [n, n], "float32", 0)
@@ -1470,7 +1474,7 @@ def test_partially_used_tuple_param():
             param_0: R.Tensor((), dtype="float32"),
             param_1: R.Tensor((), dtype="float32"),
         ) -> R.Tensor((2,), dtype="float32"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             with R.dataflow():
                 y0 = R.emit_te(topi.add, x_0, param_0)
                 gv = R.emit_te(topi.divide, y0, param_1)
@@ -1510,7 +1514,7 @@ def test_call_tir_inplace():
             B: T.Buffer((), "float32"),
             Out: T.Buffer((T.int64(10), T.int64(20)), "float32"),
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             for ax0, ax1 in T.grid(T.int64(10), T.int64(20)):
                 with T.block("T_add"):
                     v_ax0, v_ax1 = T.axis.remap("SS", [ax0, ax1])
@@ -1520,7 +1524,7 @@ def test_call_tir_inplace():
 
         @T.prim_func(private=True)
         def exp_inplace(A: T.Buffer((T.int64(10), T.int64(20)), "float32")):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             for i0, i1 in T.grid(T.int64(10), T.int64(20)):
                 with T.block("compute"):
                     v_i0, v_i1 = T.axis.remap("SS", [i0, i1])
@@ -1530,7 +1534,7 @@ def test_call_tir_inplace():
 
         @T.prim_func(private=True)
         def squeeze_inplace(A: T.Buffer((T.int64(10), T.int64(20)), "float32")):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             for ax0, ax1 in T.grid(T.int64(10), T.int64(20)):
                 with T.block("T_squeeze"):
                     v_ax0, v_ax1 = T.axis.remap("SS", [ax0, ax1])
@@ -1572,7 +1576,7 @@ def test_call_tir_inplace():
             B: T.Buffer((), "float32"),
             Out: T.Buffer((T.int64(10), T.int64(20)), "float32"),
         ):
-            T.func_attr({"tir.noalias": T.bool(True), "op_pattern": 0})
+            T.func_attr({"tir.noalias": True, "op_pattern": 0})
             for ax0, ax1 in T.grid(T.int64(10), T.int64(20)):
                 with T.block("T_add"):
                     v_ax0, v_ax1 = T.axis.remap("SS", [ax0, ax1])
@@ -1582,7 +1586,7 @@ def test_call_tir_inplace():
 
         @T.prim_func(private=True)
         def exp_inplace(A: T.Buffer((T.int64(10), T.int64(20)), "float32")):
-            T.func_attr({"tir.noalias": T.bool(True), "op_pattern": 0})
+            T.func_attr({"tir.noalias": True, "op_pattern": 0})
             for i0, i1 in T.grid(T.int64(10), T.int64(20)):
                 with T.block("compute"):
                     v_i0, v_i1 = T.axis.remap("SS", [i0, i1])
@@ -1592,7 +1596,7 @@ def test_call_tir_inplace():
 
         @T.prim_func(private=True)
         def squeeze_inplace(A: T.Buffer((T.int64(10), T.int64(20)), "float32")):
-            T.func_attr({"tir.noalias": T.bool(True), "op_pattern": 0})
+            T.func_attr({"tir.noalias": True, "op_pattern": 0})
             for ax0, ax1 in T.grid(T.int64(10), T.int64(20)):
                 with T.block("T_squeeze"):
                     v_ax0, v_ax1 = T.axis.remap("SS", [ax0, ax1])
@@ -1604,7 +1608,7 @@ def test_call_tir_inplace():
         def fused_add_exp_inplace_squeeze_inplace(
             x: R.Tensor((10, 20), dtype="float32"), p0: R.Tensor((), dtype="float32")
         ) -> R.Tensor((10, 20), dtype="float32"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             cls = Expected
             with R.dataflow():
                 lv = R.call_tir(
@@ -1648,7 +1652,7 @@ def test_packed_params():
     class Before:
         @T.prim_func(private=True)
         def cast(lv: T.Buffer((T.int64(16), T.int64(16)), "float16"), compute: T.Buffer((T.int64(16), T.int64(16)), "float32")):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             # with T.block("root"):
             for i0, i1 in T.grid(T.int64(16), T.int64(16)):
                 with T.block("compute"):
@@ -1659,7 +1663,7 @@ def test_packed_params():
 
         @T.prim_func(private=True)
         def matmul(x: T.Buffer((T.int64(16), T.int64(16)), "float32"), lv2: T.Buffer((T.int64(16), T.int64(16)), "float32"), T_matmul: T.Buffer((T.int64(16), T.int64(16)), "float32")):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             # with T.block("root"):
             for ax0, ax1, k in T.grid(T.int64(16), T.int64(16), T.int64(16)):
                 with T.block("T_matmul"):
