@@ -567,7 +567,7 @@ def test_multiple_relax_functions():
 
         x = relax.Var("x", R.Tensor([10, 20], "float32"))
         p0 = relax.Var("p0", R.Tensor((), "float32"))
-        with bb.function("fused_add_exp_squeeze", [x, p0], attrs={"Primitive": 1}, private=True):
+        with bb.function("fused_add_exp_squeeze", [x, p0], attrs={"Primitive": True}, private=True):
             with bb.dataflow():
                 lv0 = bb.emit_te(topi.add, x, p0)
                 lv1 = bb.emit_te(topi.exp, lv0)
@@ -577,7 +577,9 @@ def test_multiple_relax_functions():
 
         x = relax.Var("x", R.Tensor([20, 10], "float32"))
         p0 = relax.Var("p0", R.Tensor((), "float32"))
-        with bb.function("fused_add1_exp1_squeeze1", [x, p0], attrs={"Primitive": 1}, private=True):
+        with bb.function(
+            "fused_add1_exp1_squeeze1", [x, p0], attrs={"Primitive": True}, private=True
+        ):
             with bb.dataflow():
                 lv0 = bb.emit_te(topi.add, x, p0)
                 lv1 = bb.emit_te(topi.exp, lv0)
@@ -707,7 +709,7 @@ def test_symbolic_shape_aware_fuse():
         def fused_add_exp_squeeze(
             x: R.Tensor(["n", "m"], "float32"), p0: R.Tensor([], "float32")
         ) -> R.Tensor(["n", "m"], dtype="float32"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             with R.dataflow():
                 lv0 = R.emit_te(topi.add, x, p0)
                 lv1 = R.emit_te(topi.exp, lv0)
@@ -755,7 +757,7 @@ def test_fuse_of_dynamic_kernel_with_var_params_and_static_args():
 
         @R.function(private=True)
         def fused_function(x: R.Tensor([16, 32], "float32")) -> R.Tensor([16, 32], dtype="float32"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             cls = Before
             with R.dataflow():
                 y = R.call_tir(cls.dynamic_tir_kernel, [x], out_sinfo=R.Tensor([16, 32], "float32"))
@@ -829,7 +831,7 @@ def test_fuse_of_dynamic_kernel_with_expression_params_and_static_args():
             B: R.Tensor([16], "float32"),
             C: R.Tensor([32], "float32"),
         ):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             cls = Before
             with R.dataflow():
                 y = R.call_tir(
@@ -862,7 +864,7 @@ def test_fuse_of_dynamic_kernel_with_expression_params_and_static_args():
             C: T.Buffer(T.int64(32), "float32"),
             Z: T.Buffer(T.int64(512), "float32"),
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             Y = T.alloc_buffer((T.int64(512),))
             for i, j in T.grid(T.int64(16), T.int64(32)):
                 with T.block("compute"):
@@ -903,7 +905,7 @@ def test_symbolic_shape_aware_fuse_with_allocation():
             y: R.Tensor((1, "n", 4096), dtype="float32"),
             rms_norm_weight: R.Tensor((4096,), dtype="float32"),
         ) -> R.Tensor((1, "n", 4096), dtype="float32"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             with R.dataflow():
                 lv0 = R.emit_te(te_mean, x, axis=2)
                 lv1 = R.emit_te(topi.add, lv0, lv0)
@@ -969,7 +971,7 @@ def test_symbolic_var_in_call_tir_args():
             y: R.Tensor((2048, 128), dtype="float32"),
             len: R.Shape(["m"]),
         ) -> R.Tensor((1, 1, 32, 128), dtype="float32"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             m = T.int64()
             cls = Before
             with R.dataflow():
@@ -1004,7 +1006,7 @@ def test_symbolic_var_in_call_tir_args():
             rotary: T.Buffer((T.int64(1), T.int64(1), T.int64(32), T.int64(128)), "float32"),
             m: T.int64,
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             T_add = T.alloc_buffer((T.int64(1), T.int64(1), T.int64(32), T.int64(128)))
             for ax0, ax1, ax2, ax3 in T.grid(T.int64(1), T.int64(1), T.int64(32), T.int64(128)):
                 with T.block("T_add"):
@@ -1049,7 +1051,7 @@ def test_same_buffer_multiple_read():
             ),
             T_concat: T.Buffer((T.int64(2), T.int64(4), T.int64(64), T.int64(64)), "float32"),
         ):
-            T.func_attr({"op_pattern": 2, "tir.noalias": T.bool(True)})
+            T.func_attr({"op_pattern": 2, "tir.noalias": True})
             for ax0, ax1, ax2, ax3 in T.grid(T.int64(2), T.int64(4), T.int64(64), T.int64(64)):
                 with T.block("T_concat"):
                     v_ax0, v_ax1, v_ax2, v_ax3 = T.axis.remap("SSSS", [ax0, ax1, ax2, ax3])
@@ -1069,7 +1071,7 @@ def test_same_buffer_multiple_read():
             rxplaceholder: T.Buffer((T.int64(2), T.int64(4), T.int64(64), T.int64(64)), "float32"),
             T_transpose: T.Buffer((T.int64(2), T.int64(64), T.int64(64), T.int64(4)), "float32"),
         ):
-            T.func_attr({"op_pattern": 2, "tir.noalias": T.bool(True)})
+            T.func_attr({"op_pattern": 2, "tir.noalias": True})
             for ax0, ax1, ax2, ax3 in T.grid(T.int64(2), T.int64(64), T.int64(64), T.int64(4)):
                 with T.block("T_transpose"):
                     v_ax0, v_ax1, v_ax2, v_ax3 = T.axis.remap("SSSS", [ax0, ax1, ax2, ax3])
@@ -1083,7 +1085,7 @@ def test_same_buffer_multiple_read():
         def fused_concatenate_transpose2(
             inp_0: R.Tensor((1, 4, 64, 64), dtype="float32")
         ) -> R.Tensor((2, 64, 64, 4), dtype="float32"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             cls = Module
             with R.dataflow():
                 lv = R.call_tir(
@@ -1117,7 +1119,7 @@ def test_same_buffer_multiple_read():
                 (T.int64(2), T.int64(64), T.int64(64), T.int64(4)), "float32"
             ),
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             T_concat_handle_intermediate = T.alloc_buffer(
                 (T.int64(2), T.int64(4), T.int64(64), T.int64(64))
             )
@@ -1167,7 +1169,7 @@ def test_tir_expression_in_shape():
             y: R.Tensor(("n - 1", 4), dtype="float32"),
             tir_vars: R.Shape(["n"]),
         ) -> R.Tensor(("n - 1", 3), dtype="float32"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             with R.dataflow():
                 lv = R.emit_te(topi.transpose, x)
                 gv = R.emit_te(topi.matmul, y, lv)
@@ -1195,7 +1197,7 @@ def test_tir_expression_in_shape():
             p_output0: T.handle,
             n: T.int64,
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             y = T.match_buffer(p_y, (n - T.int64(1), T.int64(4)))
             var_T_matmul_intermediate = T.match_buffer(p_output0, (n - T.int64(1), T.int64(3)))
             var_T_transpose_intermediate = T.alloc_buffer((T.int64(4), T.int64(3)))
@@ -1242,7 +1244,7 @@ def test_tuple_input_unused_field():
             A: T.Buffer((T.int64(4), T.int64(8), T.int64(2048)), "float32"),
             T_reshape: T.Buffer((T.int64(4), T.int64(8), T.int64(32), T.int64(64)), "float32"),
         ):
-            T.func_attr({"op_pattern": 2, "tir.noalias": T.bool(True)})
+            T.func_attr({"op_pattern": 2, "tir.noalias": True})
             # with T.block("root"):
             for ax0, ax1, ax2, ax3 in T.grid(T.int64(4), T.int64(8), T.int64(32), T.int64(64)):
                 with T.block("T_reshape"):
@@ -1276,7 +1278,7 @@ def test_tuple_input_unused_field():
                 R.Tensor((4, 8, 2048), dtype="float32"), R.Tensor((4, 8, 2048), dtype="float32")
             )
         ) -> R.Tensor((4, 8, 32, 64), dtype="float32"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             cls = Module
             with R.dataflow():
                 lv1: R.Tensor((4, 8, 2048), dtype="float32") = lv[0]
@@ -1307,7 +1309,7 @@ def test_tuple_input_unused_field():
                 (T.int64(4), T.int64(8), T.int64(32), T.int64(64)), "float32"
             ),
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             # with T.block("root"):
             for ax0, ax1, ax2, ax3 in T.grid(T.int64(4), T.int64(8), T.int64(32), T.int64(64)):
                 with T.block("T_reshape"):
@@ -1390,7 +1392,7 @@ def test_unique_duplicated_buffer_allocation():
         def fused_func(
             input_embeds: R.Tensor((4096, 4096), dtype="float16"),
         ) -> R.Tensor((4096, 4096), dtype="float16"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             cls = Module
             with R.dataflow():
                 lv = R.call_tir(
@@ -1407,7 +1409,7 @@ def test_unique_duplicated_buffer_allocation():
             input_embeds: T.Buffer((T.int64(4096), T.int64(4096)), "float16"),
             Out_intermediate_1: T.Buffer((T.int64(4096), T.int64(4096)), "float16"),
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             Out_intermediate = T.alloc_buffer((T.int64(4096), T.int64(4096)), "float16")
             for i, j in T.grid(T.int64(4096), T.int64(4096)):
                 with T.block("add"):
@@ -1485,7 +1487,7 @@ def test_symbolic_var_in_buffer_shape():
             y: R.Tensor((2048, 128), dtype="float32"),
             len: R.Shape(["m"]),
         ) -> R.Tensor((1, "sequence_length", 32, 128), dtype="float32"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             sequence_length = T.int64()
             m = T.int64()
             cls = Before
@@ -1521,7 +1523,7 @@ def test_symbolic_var_in_buffer_shape():
             rotary_handle: T.handle,
             m: T.int64,
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
 
             sequence_length = T.int64()
 
@@ -1593,7 +1595,7 @@ def test_symbolic_var_called_with_static_shape():
         def fused(
             x: R.Tensor([64], dtype="float32"),
         ) -> R.Tensor([1], dtype="float32"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             cls = Before
             with R.dataflow():
                 gv = R.call_tir(
@@ -1680,7 +1682,7 @@ def test_symbolic_var_called_with_multiple_static_shapes():
             x: R.Tensor([64], dtype="float32"),
             y: R.Tensor([16], dtype="float32"),
         ) -> R.Tensor([1], dtype="float32"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             cls = Before
             with R.dataflow():
                 x_sum = R.call_tir(
@@ -1792,7 +1794,7 @@ def test_symbolic_var_called_with_static_argument():
         def fused(
             x: R.Tensor([64], dtype="float32"),
         ) -> R.Tensor([1], dtype="float32"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             cls = Before
             with R.dataflow():
                 gv = R.call_tir(
@@ -1883,7 +1885,7 @@ def test_gather():
             input_ids: R.Tensor((1,), dtype="int32"),
             input_embeds: R.Tensor((4096, 4096), dtype="float16"),
         ) -> R.Tensor((1, 4096), dtype="float16"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             cls = Before
             with R.dataflow():
                 lv = R.call_tir(
@@ -1903,7 +1905,7 @@ def test_gather():
             input_embeds: T.Buffer((T.int64(4096), T.int64(4096)), "float16"),
             T_take: T.Buffer((T.int64(1), T.int64(4096)), "float16"),
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             Out_handle_intermediate = T.alloc_buffer((T.int64(4096), T.int64(4096)), "float16")
             for i, j in T.grid(T.int64(4096), T.int64(4096)):
                 with T.block("add"):
@@ -1941,7 +1943,7 @@ def test_inplace_simple():
         def add_inplace(
             A: T.Buffer((T.int64(10), T.int64(20)), "float32"), B: T.Buffer((), "float32")
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             for ax0, ax1 in T.grid(T.int64(10), T.int64(20)):
                 with T.block("T_add"):
                     v_ax0, v_ax1 = T.axis.remap("SS", [ax0, ax1])
@@ -1951,7 +1953,7 @@ def test_inplace_simple():
 
         @T.prim_func(private=True)
         def exp_inplace(A: T.Buffer((T.int64(10), T.int64(20)), "float32")):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             for i0, i1 in T.grid(T.int64(10), T.int64(20)):
                 with T.block("compute"):
                     v_i0, v_i1 = T.axis.remap("SS", [i0, i1])
@@ -1961,7 +1963,7 @@ def test_inplace_simple():
 
         @T.prim_func(private=True)
         def squeeze_inplace(A: T.Buffer((T.int64(10), T.int64(20)), "float32")):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             for ax0, ax1 in T.grid(T.int64(10), T.int64(20)):
                 with T.block("T_squeeze"):
                     v_ax0, v_ax1 = T.axis.remap("SS", [ax0, ax1])
@@ -1973,7 +1975,7 @@ def test_inplace_simple():
         def fused_add_exp_squeeze(
             x: R.Tensor((10, 20), dtype="float32"), p0: R.Tensor((), dtype="float32")
         ) -> R.Tensor((10, 20), dtype="float32"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             cls = Module
             with R.dataflow():
                 # This overwrites x and is actually evil because the function is marked as pure
@@ -2020,7 +2022,7 @@ def test_inplace_simple():
         def fused_add_exp_squeeze(
             x: T.Buffer((T.int64(10), T.int64(20)), "float32"), p0: T.Buffer((), "float32")
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             for ax0, ax1 in T.grid(T.int64(10), T.int64(20)):
                 with T.block("T_add"):
                     v_ax0, v_ax1 = T.axis.remap("SS", [ax0, ax1])
@@ -2064,7 +2066,7 @@ def test_fuse_inplace_and_non_inplace():
             B: T.Buffer((), "float32"),
             Out: T.Buffer((T.int64(10), T.int64(20)), "float32"),
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             for ax0, ax1 in T.grid(T.int64(10), T.int64(20)):
                 with T.block("T_add"):
                     v_ax0, v_ax1 = T.axis.remap("SS", [ax0, ax1])
@@ -2072,7 +2074,7 @@ def test_fuse_inplace_and_non_inplace():
 
         @T.prim_func(private=True)
         def exp_inplace(A: T.Buffer((T.int64(10), T.int64(20)), "float32")):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             for i0, i1 in T.grid(T.int64(10), T.int64(20)):
                 with T.block("compute"):
                     v_i0, v_i1 = T.axis.remap("SS", [i0, i1])
@@ -2080,7 +2082,7 @@ def test_fuse_inplace_and_non_inplace():
 
         @T.prim_func(private=True)
         def squeeze_inplace(A: T.Buffer((T.int64(10), T.int64(20)), "float32")):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             for ax0, ax1 in T.grid(T.int64(10), T.int64(20)):
                 with T.block("T_squeeze"):
                     v_ax0, v_ax1 = T.axis.remap("SS", [ax0, ax1])
@@ -2090,7 +2092,7 @@ def test_fuse_inplace_and_non_inplace():
         def fused_add_exp_squeeze(
             x: R.Tensor((10, 20), dtype="float32"), p0: R.Tensor((), dtype="float32")
         ) -> R.Tensor((10, 20), dtype="float32"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             cls = Module
             with R.dataflow():
                 lv = R.call_tir(
@@ -2133,7 +2135,7 @@ def test_fuse_inplace_and_non_inplace():
             p0: T.Buffer((), "float32"),
             p_output0: T.Buffer((T.int64(10), T.int64(20)), "float32"),
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             for ax0, ax1 in T.grid(T.int64(10), T.int64(20)):
                 with T.block("T_add"):
                     v_ax0, v_ax1 = T.axis.remap("SS", [ax0, ax1])
@@ -2174,7 +2176,7 @@ def test_use_as_inplace_and_dps():
             B: T.Buffer((), "float32"),
             Out: T.Buffer((T.int64(10), T.int64(20)), "float32"),
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             for ax0, ax1 in T.grid(T.int64(10), T.int64(20)):
                 with T.block("T_add"):
                     v_ax0, v_ax1 = T.axis.remap("SS", [ax0, ax1])
@@ -2184,7 +2186,7 @@ def test_use_as_inplace_and_dps():
         def fused_sums(
             x: R.Tensor((10, 20), dtype="float32"), p0: R.Tensor((), dtype="float32")
         ) -> R.Tensor((10, 20), dtype="float32"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             cls = Module
             with R.dataflow():
                 lv = R.call_tir(
@@ -2225,7 +2227,7 @@ def test_use_as_inplace_and_dps():
             p0: T.Buffer((), "float32"),
             p_output0: T.Buffer((T.int64(10), T.int64(20)), "float32"),
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             for ax0, ax1 in T.grid(T.int64(10), T.int64(20)):
                 with T.block("T_add"):
                     v_ax0, v_ax1 = T.axis.remap("SS", [ax0, ax1])
@@ -2336,7 +2338,7 @@ def test_fuse_with_axis_separators():
             y: R.Tensor([T.int64(16), T.int64(32)], "float32"),
             z: R.Tensor([T.int64(16), T.int64(32)], "float32"),
         ) -> R.Tensor([T.int64(16), T.int64(32)], dtype="float32"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             cls = Before
             with R.dataflow():
                 w = R.call_tir(
@@ -2417,7 +2419,7 @@ def test_fuse_with_axis_separators_inconsistent_buffer_mapping():
         def fused_function(
             x: R.Tensor([T.int64(16), T.int64(32)], "float32"),
         ) -> R.Tensor([T.int64(16), T.int64(32)], dtype="float32"):
-            R.func_attr({"Primitive": 1})
+            R.func_attr({"Primitive": True})
             cls = Before
             with R.dataflow():
                 out = R.call_tir(
