@@ -47,7 +47,7 @@ class Conv2dReLUx2:
         data1: R.Tensor((1, 64, 56, 56), dtype="float32"),
         weight11: R.Tensor((64, 64, 3, 3), dtype="float32"),
     ) -> R.Tensor((1, 64, 56, 56), dtype="float32"):
-        R.func_attr({"Primitive": 1, "Composite": "dnnl.conv2d_relu"})
+        R.func_attr({"Primitive": True, "Composite": "dnnl.conv2d_relu"})
         with R.dataflow():
             lv1: R.Tensor((1, 64, 56, 56), dtype="float32") = R.nn.conv2d(
                 data1,
@@ -63,7 +63,7 @@ class Conv2dReLUx2:
         conv1: R.Tensor((1, 64, 56, 56), dtype="float32"),
         weight21: R.Tensor((64, 64, 3, 3), dtype="float32"),
     ) -> R.Tensor((1, 64, 54, 54), dtype="float32"):
-        R.func_attr({"Primitive": 1, "Composite": "dnnl.conv2d_relu"})
+        R.func_attr({"Primitive": True, "Composite": "dnnl.conv2d_relu"})
         with R.dataflow():
             lv2: R.Tensor((1, 64, 54, 54), dtype="float32") = R.nn.conv2d(
                 conv1,
@@ -161,7 +161,7 @@ class Diamond:
     def fused_relax_nn_gelu(
         lv: R.Tensor((1, 64, 54, 54), dtype="float32"),
     ) -> R.Tensor((1, 64, 54, 54), dtype="float32"):
-        R.func_attr({"Primitive": 1, "Composite": "compiler_A.gelu"})
+        R.func_attr({"Primitive": True, "Composite": "compiler_A.gelu"})
         with R.dataflow():
             gv: R.Tensor((1, 64, 54, 54), dtype="float32") = R.nn.gelu(lv)
             R.output(gv)
@@ -171,7 +171,7 @@ class Diamond:
     def fused_relax_nn_relu(
         lv1: R.Tensor((1, 64, 54, 54), dtype="float32"),
     ) -> R.Tensor((1, 64, 54, 54), dtype="float32"):
-        R.func_attr({"Primitive": 1, "Composite": "compiler_A.relu"})
+        R.func_attr({"Primitive": True, "Composite": "compiler_A.relu"})
         with R.dataflow():
             gv1: R.Tensor((1, 64, 54, 54), dtype="float32") = R.nn.relu(lv1)
             R.output(gv1)
@@ -182,7 +182,7 @@ class Diamond:
         lv5: R.Tensor((1, 64, 54, 54), dtype="float32"),
         gelu1: R.Tensor((1, 64, 54, 54), dtype="float32"),
     ) -> R.Tensor((1, 64, 54, 54), dtype="float32"):
-        R.func_attr({"Primitive": 1, "Composite": "compiler_A.add"})
+        R.func_attr({"Primitive": True, "Composite": "compiler_A.add"})
         with R.dataflow():
             gv3: R.Tensor((1, 64, 54, 54), dtype="float32") = R.add(lv5, gelu1)
             R.output(gv3)
@@ -193,7 +193,7 @@ class Diamond:
         data1: R.Tensor((1, 64, 56, 56), dtype="float32"),
         weight1: R.Tensor((64, 64, 3, 3), dtype="float32"),
     ) -> R.Tensor((1, 64, 54, 54), dtype="float32"):
-        R.func_attr({"Primitive": 1, "Composite": "compiler_A.conv2d"})
+        R.func_attr({"Primitive": True, "Composite": "compiler_A.conv2d"})
         with R.dataflow():
             gv4: R.Tensor((1, 64, 54, 54), dtype="float32") = R.nn.conv2d(
                 data1,
@@ -323,7 +323,7 @@ class Diamond_cyclic_dep:
     def fused_relax_nn_gelu(
         lv: R.Tensor((1, 64, 54, 54), dtype="float32"),
     ) -> R.Tensor((1, 64, 54, 54), dtype="float32"):
-        R.func_attr({"Primitive": 1, "Composite": "compiler_B.gelu"})
+        R.func_attr({"Primitive": True, "Composite": "compiler_B.gelu"})
         with R.dataflow():
             gv: R.Tensor((1, 64, 54, 54), dtype="float32") = R.nn.gelu(lv)
             R.output(gv)
@@ -333,7 +333,7 @@ class Diamond_cyclic_dep:
     def fused_relax_nn_relu(
         lv1: R.Tensor((1, 64, 54, 54), dtype="float32"),
     ) -> R.Tensor((1, 64, 54, 54), dtype="float32"):
-        R.func_attr({"Primitive": 1, "Composite": "compiler_A.relu"})
+        R.func_attr({"Primitive": True, "Composite": "compiler_A.relu"})
         with R.dataflow():
             gv1: R.Tensor((1, 64, 54, 54), dtype="float32") = R.nn.relu(lv1)
             R.output(gv1)
@@ -344,7 +344,7 @@ class Diamond_cyclic_dep:
         lv5: R.Tensor((1, 64, 54, 54), dtype="float32"),
         gelu1: R.Tensor((1, 64, 54, 54), dtype="float32"),
     ) -> R.Tensor((1, 64, 54, 54), dtype="float32"):
-        R.func_attr({"Primitive": 1, "Composite": "compiler_A.add"})
+        R.func_attr({"Primitive": True, "Composite": "compiler_A.add"})
         with R.dataflow():
             gv3: R.Tensor((1, 64, 54, 54), dtype="float32") = R.add(lv5, gelu1)
             R.output(gv3)
@@ -355,7 +355,7 @@ class Diamond_cyclic_dep:
         data1: R.Tensor((1, 64, 56, 56), dtype="float32"),
         weight1: R.Tensor((64, 64, 3, 3), dtype="float32"),
     ) -> R.Tensor((1, 64, 54, 54), dtype="float32"):
-        R.func_attr({"Primitive": 1, "Composite": "compiler_A.conv2d"})
+        R.func_attr({"Primitive": True, "Composite": "compiler_A.conv2d"})
         with R.dataflow():
             gv4: R.Tensor((1, 64, 54, 54), dtype="float32") = R.nn.conv2d(
                 data1,
@@ -491,7 +491,7 @@ class MultipleProducers:
     def fused_relax_nn_relu(
         x11: R.Tensor((10,), dtype="float32"),
     ) -> R.Tensor((10,), dtype="float32"):
-        R.func_attr({"Primitive": 1, "Composite": "compiler_A.relu"})
+        R.func_attr({"Primitive": True, "Composite": "compiler_A.relu"})
         with R.dataflow():
             gv2: R.Tensor((10,), dtype="float32") = R.nn.relu(x11)
             R.output(gv2)
@@ -501,7 +501,7 @@ class MultipleProducers:
     def fused_relax_nn_gelu(
         x21: R.Tensor((10,), dtype="float32"),
     ) -> R.Tensor((10,), dtype="float32"):
-        R.func_attr({"Primitive": 1, "Composite": "compiler_A.gelu"})
+        R.func_attr({"Primitive": True, "Composite": "compiler_A.gelu"})
         with R.dataflow():
             gv3: R.Tensor((10,), dtype="float32") = R.nn.gelu(x21)
             R.output(gv3)
@@ -511,7 +511,7 @@ class MultipleProducers:
     def fused_relax_add(
         lv: R.Tensor((10,), dtype="float32"), gelu1: R.Tensor((10,), dtype="float32")
     ) -> R.Tensor((10,), dtype="float32"):
-        R.func_attr({"Primitive": 1, "Composite": "compiler_A.add"})
+        R.func_attr({"Primitive": True, "Composite": "compiler_A.add"})
         with R.dataflow():
             gv: R.Tensor((10,), dtype="float32") = R.add(lv, gelu1)
             R.output(gv)
@@ -601,7 +601,7 @@ class MultipleProducersCyclic:
     def fused_relax_nn_relu(
         x11: R.Tensor((10,), dtype="float32"),
     ) -> R.Tensor((10,), dtype="float32"):
-        R.func_attr({"Primitive": 1, "Composite": "compiler_A.relu"})
+        R.func_attr({"Primitive": True, "Composite": "compiler_A.relu"})
         with R.dataflow():
             gv2: R.Tensor((10,), dtype="float32") = R.nn.relu(x11)
             R.output(gv2)
@@ -611,7 +611,7 @@ class MultipleProducersCyclic:
     def fused_relax_nn_gelu(
         x21: R.Tensor((10,), dtype="float32"),
     ) -> R.Tensor((10,), dtype="float32"):
-        R.func_attr({"Primitive": 1, "Composite": "compiler_A.gelu"})
+        R.func_attr({"Primitive": True, "Composite": "compiler_A.gelu"})
         with R.dataflow():
             gv3: R.Tensor((10,), dtype="float32") = R.nn.gelu(x21)
             R.output(gv3)
@@ -621,7 +621,7 @@ class MultipleProducersCyclic:
     def fused_relax_add(
         lv: R.Tensor((10,), dtype="float32"), gelu1: R.Tensor((10,), dtype="float32")
     ) -> R.Tensor((10,), dtype="float32"):
-        R.func_attr({"Primitive": 1, "Composite": "compiler_A.add"})
+        R.func_attr({"Primitive": True, "Composite": "compiler_A.add"})
         with R.dataflow():
             gv: R.Tensor((10,), dtype="float32") = R.add(lv, gelu1)
             R.output(gv)
@@ -724,7 +724,7 @@ class MergeCompilerRegionsExample:
     def fused_relax_nn_relu(
         add2: R.Tensor((10,), dtype="float32"),
     ) -> R.Tensor((10,), dtype="float32"):
-        R.func_attr({"Primitive": 1, "Composite": "compiler_A.relu"})
+        R.func_attr({"Primitive": True, "Composite": "compiler_A.relu"})
         with R.dataflow():
             gv: R.Tensor((10,), dtype="float32") = R.nn.relu(add2)
             R.output(gv)
@@ -734,7 +734,7 @@ class MergeCompilerRegionsExample:
     def fused_relax_add(
         x11: R.Tensor((10,), dtype="float32"), x21: R.Tensor((10,), dtype="float32")
     ) -> R.Tensor((10,), dtype="float32"):
-        R.func_attr({"Primitive": 1, "Composite": "compiler_A.add"})
+        R.func_attr({"Primitive": True, "Composite": "compiler_A.add"})
         with R.dataflow():
             gv2: R.Tensor((10,), dtype="float32") = R.add(x11, x21)
             R.output(gv2)
@@ -744,7 +744,7 @@ class MergeCompilerRegionsExample:
     def fused_relax_nn_gelu(
         x31: R.Tensor((10,), dtype="float32"),
     ) -> R.Tensor((10,), dtype="float32"):
-        R.func_attr({"Primitive": 1, "Composite": "compiler_B.gelu"})
+        R.func_attr({"Primitive": True, "Composite": "compiler_B.gelu"})
         with R.dataflow():
             gv3: R.Tensor((10,), dtype="float32") = R.nn.gelu(x31)
             R.output(gv3)
@@ -873,7 +873,7 @@ class ModuleWithNonComposite:
         data1: R.Tensor((1, 64, 56, 56), dtype="float32"),
         weight1: R.Tensor((64, 64, 3, 3), dtype="float32"),
     ) -> R.Tensor((1, 64, 56, 56), dtype="float32"):
-        R.func_attr({"Composite": "tensorrt.conv2d", "Primitive": 1})
+        R.func_attr({"Composite": "tensorrt.conv2d", "Primitive": True})
         with R.dataflow():
             gv: R.Tensor((1, 64, 56, 56), dtype="float32") = R.nn.conv2d(
                 data1,
@@ -1011,7 +1011,7 @@ def test_reshape():
         def fused_relax_matmul(
             lv: R.Tensor((1, 784), dtype="float32"), lv1: R.Tensor((784, 512), dtype="float32")
         ) -> R.Tensor((1, 512), dtype="float32"):
-            R.func_attr({"Composite": "tensorrt.matmul", "Primitive": 1})
+            R.func_attr({"Composite": "tensorrt.matmul", "Primitive": True})
             with R.dataflow():
                 gv: R.Tensor((1, 512), dtype="float32") = R.matmul(lv, lv1, out_dtype="float32")
                 R.output(gv)
@@ -1021,7 +1021,7 @@ def test_reshape():
         def fused_relax_reshape(
             inp_0: R.Tensor((1, 1, 28, 28), dtype="float32"), param_0: R.Shape([1, 784])
         ) -> R.Tensor((1, 784), dtype="float32"):
-            R.func_attr({"Composite": "tensorrt.reshape", "Primitive": 1})
+            R.func_attr({"Composite": "tensorrt.reshape", "Primitive": True})
             with R.dataflow():
                 gv: R.Tensor((1, 784), dtype="float32") = R.reshape(inp_0, param_0)
                 R.output(gv)
@@ -1129,7 +1129,7 @@ def test_handle_existence_of_call_tir():
         def fused_relax_nn_relu(
             Input: R.Tensor([10], dtype="float32"),
         ) -> R.Tensor([10], dtype="float32"):
-            R.func_attr({"Composite": "compiler_A.relu", "Primitive": 1})
+            R.func_attr({"Composite": "compiler_A.relu", "Primitive": True})
             with R.dataflow():
                 Output = R.nn.relu(Input)
                 R.output(Output)
@@ -1140,7 +1140,7 @@ def test_handle_existence_of_call_tir():
             Input: T.Buffer(T.int64(10), "float32"),
             Output: T.Buffer(T.int64(10), "float32"),
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             for i in range(T.int64(10)):
                 with T.block("compute"):
                     vi = T.axis.remap("S", [i])
@@ -1150,7 +1150,7 @@ def test_handle_existence_of_call_tir():
         def fused_relax_nn_gelu(
             Input: R.Tensor([10], dtype="float32"),
         ) -> R.Tensor([10], dtype="float32"):
-            R.func_attr({"Composite": "compiler_A.gelu", "Primitive": 1})
+            R.func_attr({"Composite": "compiler_A.gelu", "Primitive": True})
             with R.dataflow():
                 Output = R.nn.gelu(Input)
                 R.output(Output)
@@ -1192,7 +1192,7 @@ def test_handle_existence_of_call_tir():
             Input: T.Buffer(T.int64(10), "float32"),
             Output: T.Buffer(T.int64(10), "float32"),
         ):
-            T.func_attr({"tir.noalias": T.bool(True)})
+            T.func_attr({"tir.noalias": True})
             for i in range(T.int64(10)):
                 with T.block("compute"):
                     vi = T.axis.remap("S", [i])
