@@ -67,7 +67,7 @@ def test_alloc_storage_with_scope_global():
     dev = tvm.cpu()
     # This is the important line which tests nd allocator
     vm_rt = relax.VirtualMachine(lib, dev, memory_cfg="naive")
-    x = tvm.nd.array(arg0, dev)
+    x = tvm.runtime.tensor(arg0, dev)
     vm_rt.set_input("main", x)
     vm_rt.invoke_stateful("main")
     output = vm_rt.get_outputs("main").numpy()
