@@ -74,9 +74,9 @@ def test_gemm_mma_m8n8k4_row_col_fp64pf64fp64():
     C_np = np.zeros([8, 8]).astype("float64")
 
     ctx = tvm.cuda()
-    A_tvm = tvm.nd.array(A_np, ctx)
-    B_tvm = tvm.nd.array(B_np, ctx)
-    C_tvm = tvm.nd.array(C_np, ctx)
+    A_tvm = tvm.runtime.tensor(A_np, ctx)
+    B_tvm = tvm.runtime.tensor(B_np, ctx)
+    C_tvm = tvm.runtime.tensor(C_np, ctx)
 
     cuda_mod(A_tvm, B_tvm, C_tvm)
 
@@ -150,9 +150,9 @@ def test_gemm_mma_m8n8k4_row_row_fp16fp16fp16():
     C_np = np.zeros([16, 16]).astype("float16")
 
     ctx = tvm.cuda()
-    A_tvm = tvm.nd.array(A_np, ctx)
-    B_tvm = tvm.nd.array(B_np, ctx)
-    C_tvm = tvm.nd.array(C_np, ctx)
+    A_tvm = tvm.runtime.tensor(A_np, ctx)
+    B_tvm = tvm.runtime.tensor(B_np, ctx)
+    C_tvm = tvm.runtime.tensor(C_np, ctx)
 
     cuda_mod(A_tvm, B_tvm, C_tvm)
 
@@ -233,9 +233,9 @@ def test_gemm_mma_m8n8k4_row_row_fp16fp16fp32():
     C_np = np.zeros([16, 16]).astype("float32")
 
     ctx = tvm.cuda()
-    A_tvm = tvm.nd.array(A_np, ctx)
-    B_tvm = tvm.nd.array(B_np, ctx)
-    C_tvm = tvm.nd.array(C_np, ctx)
+    A_tvm = tvm.runtime.tensor(A_np, ctx)
+    B_tvm = tvm.runtime.tensor(B_np, ctx)
+    C_tvm = tvm.runtime.tensor(C_np, ctx)
 
     cuda_mod(A_tvm, B_tvm, C_tvm)
 
@@ -304,9 +304,9 @@ def test_gemm_mma_m8n8k16_row_col_s8s8s32():
     C_np = np.zeros([8, 8]).astype("int32")
 
     ctx = tvm.cuda()
-    A_tvm = tvm.nd.array(A_np, ctx)
-    B_tvm = tvm.nd.array(B_np, ctx)
-    C_tvm = tvm.nd.array(C_np, ctx)
+    A_tvm = tvm.runtime.tensor(A_np, ctx)
+    B_tvm = tvm.runtime.tensor(B_np, ctx)
+    C_tvm = tvm.runtime.tensor(C_np, ctx)
 
     cuda_mod(A_tvm, B_tvm, C_tvm)
 
@@ -375,9 +375,9 @@ def test_gemm_mma_m8n8k16_row_col_s8u8s32():
     C_np = np.zeros([8, 8]).astype("int32")
 
     ctx = tvm.cuda()
-    A_tvm = tvm.nd.array(A_np, ctx)
-    B_tvm = tvm.nd.array(B_np, ctx)
-    C_tvm = tvm.nd.array(C_np, ctx)
+    A_tvm = tvm.runtime.tensor(A_np, ctx)
+    B_tvm = tvm.runtime.tensor(B_np, ctx)
+    C_tvm = tvm.runtime.tensor(C_np, ctx)
 
     cuda_mod(A_tvm, B_tvm, C_tvm)
 
@@ -442,9 +442,9 @@ def test_gemm_mma_m8n8k32_row_col_s4s4s32():
     cuda_mod = tvm.compile(sch.mod, target="cuda")
 
     ctx = tvm.cuda()
-    A_tvm = tvm.nd.empty([8, 32], "int4", ctx)
-    B_tvm = tvm.nd.empty([8, 32], "int4", ctx)
-    C_tvm = tvm.nd.empty([8, 8], "int32", ctx)
+    A_tvm = tvm.runtime.empty([8, 32], "int4", ctx)
+    B_tvm = tvm.runtime.empty([8, 32], "int4", ctx)
+    C_tvm = tvm.runtime.empty([8, 8], "int32", ctx)
 
     cuda_mod(A_tvm, B_tvm, C_tvm)
     # Currently the correctness is not checked.
@@ -505,9 +505,9 @@ def test_gemm_mma_m8n8k32_row_col_s4u4s32():
     cuda_mod = tvm.compile(sch.mod, target="cuda")
 
     ctx = tvm.cuda()
-    A_tvm = tvm.nd.empty([8, 32], "int4", ctx)
-    B_tvm = tvm.nd.empty([8, 32], "uint4", ctx)
-    C_tvm = tvm.nd.empty([8, 8], "int32", ctx)
+    A_tvm = tvm.runtime.empty([8, 32], "int4", ctx)
+    B_tvm = tvm.runtime.empty([8, 32], "uint4", ctx)
+    C_tvm = tvm.runtime.empty([8, 8], "int32", ctx)
 
     cuda_mod(A_tvm, B_tvm, C_tvm)
     # Currently the correctness is not checked.
@@ -574,9 +574,9 @@ def test_gemm_mma_m16n8k8_row_col_fp16fp16fp32():
     C_np = np.zeros([16, 8]).astype("float32")
 
     ctx = tvm.cuda()
-    A_tvm = tvm.nd.array(A_np, ctx)
-    B_tvm = tvm.nd.array(B_np, ctx)
-    C_tvm = tvm.nd.array(C_np, ctx)
+    A_tvm = tvm.runtime.tensor(A_np, ctx)
+    B_tvm = tvm.runtime.tensor(B_np, ctx)
+    C_tvm = tvm.runtime.tensor(C_np, ctx)
 
     cuda_mod(A_tvm, B_tvm, C_tvm)
 
@@ -650,9 +650,9 @@ def test_gemm_mma_m16n8k16_row_col_fp16fp16fp16():
     C_np = np.zeros([16, 8]).astype("float16")
 
     ctx = tvm.cuda()
-    A_tvm = tvm.nd.array(A_np, ctx)
-    B_tvm = tvm.nd.array(B_np, ctx)
-    C_tvm = tvm.nd.array(C_np, ctx)
+    A_tvm = tvm.runtime.tensor(A_np, ctx)
+    B_tvm = tvm.runtime.tensor(B_np, ctx)
+    C_tvm = tvm.runtime.tensor(C_np, ctx)
 
     cuda_mod(A_tvm, B_tvm, C_tvm)
 
@@ -726,9 +726,9 @@ def test_gemm_mma_m16n8k16_row_col_fp16fp16fp32():
     C_np = np.zeros([16, 8]).astype("float32")
 
     ctx = tvm.cuda()
-    A_tvm = tvm.nd.array(A_np, ctx)
-    B_tvm = tvm.nd.array(B_np, ctx)
-    C_tvm = tvm.nd.array(C_np, ctx)
+    A_tvm = tvm.runtime.tensor(A_np, ctx)
+    B_tvm = tvm.runtime.tensor(B_np, ctx)
+    C_tvm = tvm.runtime.tensor(C_np, ctx)
 
     cuda_mod(A_tvm, B_tvm, C_tvm)
 
@@ -802,9 +802,9 @@ def test_gemm_mma_m16n8k16_row_col_s8s8s32():
     C_np = np.zeros([16, 8]).astype("int32")
 
     ctx = tvm.cuda()
-    A_tvm = tvm.nd.array(A_np, ctx)
-    B_tvm = tvm.nd.array(B_np, ctx)
-    C_tvm = tvm.nd.array(C_np, ctx)
+    A_tvm = tvm.runtime.tensor(A_np, ctx)
+    B_tvm = tvm.runtime.tensor(B_np, ctx)
+    C_tvm = tvm.runtime.tensor(C_np, ctx)
 
     cuda_mod(A_tvm, B_tvm, C_tvm)
 
@@ -878,9 +878,9 @@ def test_gemm_mma_m16n8k16_row_col_s8u8s32():
     C_np = np.zeros([16, 8]).astype("int32")
 
     ctx = tvm.cuda()
-    A_tvm = tvm.nd.array(A_np, ctx)
-    B_tvm = tvm.nd.array(B_np, ctx)
-    C_tvm = tvm.nd.array(C_np, ctx)
+    A_tvm = tvm.runtime.tensor(A_np, ctx)
+    B_tvm = tvm.runtime.tensor(B_np, ctx)
+    C_tvm = tvm.runtime.tensor(C_np, ctx)
 
     cuda_mod(A_tvm, B_tvm, C_tvm)
 
@@ -954,9 +954,9 @@ def test_gemm_mma_m16n8k32_row_col_s8s8s32():
     C_np = np.zeros([16, 8]).astype("int32")
 
     ctx = tvm.cuda()
-    A_tvm = tvm.nd.array(A_np, ctx)
-    B_tvm = tvm.nd.array(B_np, ctx)
-    C_tvm = tvm.nd.array(C_np, ctx)
+    A_tvm = tvm.runtime.tensor(A_np, ctx)
+    B_tvm = tvm.runtime.tensor(B_np, ctx)
+    C_tvm = tvm.runtime.tensor(C_np, ctx)
 
     cuda_mod(A_tvm, B_tvm, C_tvm)
 
@@ -1030,9 +1030,9 @@ def test_gemm_mma_m16n8k32_row_col_s8u8s32():
     C_np = np.zeros([16, 8]).astype("int32")
 
     ctx = tvm.cuda()
-    A_tvm = tvm.nd.array(A_np, ctx)
-    B_tvm = tvm.nd.array(B_np, ctx)
-    C_tvm = tvm.nd.array(C_np, ctx)
+    A_tvm = tvm.runtime.tensor(A_np, ctx)
+    B_tvm = tvm.runtime.tensor(B_np, ctx)
+    C_tvm = tvm.runtime.tensor(C_np, ctx)
 
     cuda_mod(A_tvm, B_tvm, C_tvm)
 
@@ -1102,9 +1102,9 @@ def test_gemm_mma_m16n8k64_row_col_s4s4s32():
     cuda_mod = tvm.compile(sch.mod, target="cuda")
 
     ctx = tvm.cuda()
-    A_tvm = tvm.nd.empty([16, 64], "int4", ctx)
-    B_tvm = tvm.nd.empty([8, 64], "int4", ctx)
-    C_tvm = tvm.nd.empty([16, 8], "int32", ctx)
+    A_tvm = tvm.runtime.empty([16, 64], "int4", ctx)
+    B_tvm = tvm.runtime.empty([8, 64], "int4", ctx)
+    C_tvm = tvm.runtime.empty([16, 8], "int32", ctx)
 
     cuda_mod(A_tvm, B_tvm, C_tvm)
     # Currently the correctness is not checked.
@@ -1170,9 +1170,9 @@ def test_gemm_mma_m16n8k64_row_col_s4u4s32():
     cuda_mod = tvm.compile(sch.mod, target="cuda")
 
     ctx = tvm.cuda()
-    A_tvm = tvm.nd.empty([16, 64], "int4", ctx)
-    B_tvm = tvm.nd.empty([8, 64], "uint4", ctx)
-    C_tvm = tvm.nd.empty([16, 8], "int32", ctx)
+    A_tvm = tvm.runtime.empty([16, 64], "int4", ctx)
+    B_tvm = tvm.runtime.empty([8, 64], "uint4", ctx)
+    C_tvm = tvm.runtime.empty([16, 8], "int32", ctx)
 
     cuda_mod(A_tvm, B_tvm, C_tvm)
     # Currently the correctness is not checked.
@@ -1239,9 +1239,9 @@ def test_gemm_mma_m16n8k256_row_col_b1b1s32():
     cuda_mod = tvm.compile(sch.mod, target="cuda")
 
     ctx = tvm.cuda()
-    A_tvm = tvm.nd.empty([16, 256], "int1", ctx)
-    B_tvm = tvm.nd.empty([8, 256], "int1", ctx)
-    C_tvm = tvm.nd.empty([16, 8], "int32", ctx)
+    A_tvm = tvm.runtime.empty([16, 256], "int1", ctx)
+    B_tvm = tvm.runtime.empty([8, 256], "int1", ctx)
+    C_tvm = tvm.runtime.empty([16, 8], "int32", ctx)
 
     cuda_mod(A_tvm, B_tvm, C_tvm)
     # Currently the correctness is not checked.
