@@ -283,10 +283,10 @@ def test_mma_sp_m16n8k16_f16():
         meta = get_meta_m16n8k16_half(mask)
 
         ctx = tvm.cuda()
-        A_tvm = tvm.nd.array(A_np, ctx)
-        B_tvm = tvm.nd.array(B_np, ctx)
-        C_tvm = tvm.nd.array(np.zeros_like(C_np), ctx)
-        meta_tvm = tvm.nd.array(meta, ctx)
+        A_tvm = tvm.runtime.tensor(A_np, ctx)
+        B_tvm = tvm.runtime.tensor(B_np, ctx)
+        C_tvm = tvm.runtime.tensor(np.zeros_like(C_np), ctx)
+        meta_tvm = tvm.runtime.tensor(meta, ctx)
         cuda_mod(A_tvm, B_tvm, C_tvm, meta_tvm)
 
         tvm.testing.assert_allclose(C_tvm.numpy(), C_np, atol=1e-3, rtol=1e-3)
@@ -322,10 +322,10 @@ def test_mma_sp_m16n8k32_f16():
         meta = get_meta_m16n8k32_half(mask)
 
         ctx = tvm.cuda()
-        A_tvm = tvm.nd.array(A_np, ctx)
-        B_tvm = tvm.nd.array(B_np, ctx)
-        C_tvm = tvm.nd.array(np.zeros_like(C_np), ctx)
-        meta_tvm = tvm.nd.array(meta, ctx)
+        A_tvm = tvm.runtime.tensor(A_np, ctx)
+        B_tvm = tvm.runtime.tensor(B_np, ctx)
+        C_tvm = tvm.runtime.tensor(np.zeros_like(C_np), ctx)
+        meta_tvm = tvm.runtime.tensor(meta, ctx)
         cuda_mod(A_tvm, B_tvm, C_tvm, meta_tvm)
 
     tvm.testing.assert_allclose(C_tvm.numpy(), C_np, atol=1e-3, rtol=1e-3)
