@@ -38,7 +38,6 @@ TAG_DICT = {
     "hexagon": "Hexagon",
     "metal": "Metal",
     "vulkan": "Vulkan",
-    "cmsis-nn": "CMSIS-NN",
     "clml": "OpenCL & CLML",
     "opencl": "OpenCL & CLML",
     "openclml": "OpenCL & CLML",
@@ -46,8 +45,6 @@ TAG_DICT = {
     "acl": "ArmComputeLibrary",
     "rocm": "ROCm",
     "crt": "CRT",
-    "micronpu": "micoNPU",
-    "microtvm": "microTVM",
     "web": "web",
     "wasm": "web",
     "runtime": "Runtime",
@@ -59,8 +56,6 @@ TAG_DICT = {
     "tir": "TIR",
     "tensorflow": "Frontend",
     "tflite": "Frontend",
-    "paddle": "Frontend",
-    "oneflow": "Frontend",
     "pytorch": "Frontend",
     "torch": "Frontend",
     "keras": "Frontend",
@@ -86,7 +81,6 @@ TAG_DICT = {
     "fix": "BugFix",
     "bug": "BugFix",
     "hotfix": "BugFix",
-    "relay": "Relay",
     "qnn": "Relay",
     "quantization": "Relay",
     "relax": "Relax",
