@@ -27,8 +27,8 @@ fi
 
 source tests/scripts/setup-pytest-env.sh
 
-# setup cython
-cd python; python3 setup.py build_ext --inplace; cd ..
+# setup tvm-ffi into python folder
+python3 -m pip install  -v --target=python ./3rdparty/tvm-ffi/
 
 # disable hexagon tests for now
 exit 0
