@@ -20,8 +20,8 @@ set -euxo pipefail
 
 source tests/scripts/setup-pytest-env.sh
 
-# setup cython
-cd python; python3 setup.py build_ext --inplace; cd ..
+# setup tvm-ffi into python folder
+python3 -m pip install  -v --target=python ./3rdparty/tvm-ffi/
 
 # cleanup pycache
 find . -type f -path "*.pyc" | xargs rm -f
