@@ -46,7 +46,9 @@ test("GetGlobal", () => {
   // check function argument with different types.
   assert(fecho(1123) == 1123);
   assert(fecho("xyz") == "xyz");
-
+  // test long string as the abi can be different from small str
+  const long_str = "1234567890123456789abcdefghijklmnopqrstuvwxyz";
+  assert(fecho(long_str) == long_str);
   let bytes = new Uint8Array([1, 2, 3]);
   let rbytes = fecho(bytes);
   assert(rbytes.length == bytes.length);
@@ -55,6 +57,16 @@ test("GetGlobal", () => {
     assert(rbytes[i] == bytes[i]);
   }
 
+  const long_bytes = new Uint8Array(1024);
+  for (let i = 0; i < long_bytes.length; ++i) {
+    long_bytes[i] = i;
+  }
+  let rlong_bytes = fecho(long_bytes);
+  assert(rlong_bytes.length == long_bytes.length);
+  for (let i = 0; i < long_bytes.length; ++i) {
+    assert(rlong_bytes[i] == long_bytes[i]);
+  }
+
   assert(fecho(undefined) == undefined);
 
   tvm.beginScope();
@@ -146,7 +158,7 @@ test("ExceptionPassing", () => {
   tvm.endScope();
 });
 
-test("NDArrayCbArg", () => {
+test("TensorCbArg", () => {
   tvm.beginScope();
   let use_count = tvm.getGlobalFunc("testing.object_use_count");
   let record = [];
